class
	PD_ADDRESS_LIST

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_ASSERT_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end
	S_GCC_BUILTIN_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_IP_ADDRESS]} addresses.make
			Precursor
		end

feature {ANY} -- fields

	count: INTEGER_32 assign set_count

	addresses: CE_POINTER [PD_IP_ADDRESS] assign set_addresses

	faulty: INTEGER_32 assign set_faulty

	connected: BOOLEAN assign set_connected

	refcount: INTEGER_32 assign set_refcount

feature {ANY} -- field setters

	set_count (a_count: INTEGER_32)
		do
			count := a_count
			update_memory_field ("count")
		end

	set_addresses (a_addresses: CE_POINTER [PD_IP_ADDRESS])
		do
			addresses := a_addresses
			update_memory_field ("addresses")
		end

	set_faulty (a_faulty: INTEGER_32)
		do
			faulty := a_faulty
			update_memory_field ("faulty")
		end

	set_connected (a_connected: BOOLEAN)
		do
			connected := a_connected
			update_memory_field ("connected")
		end

	set_refcount (a_refcount: INTEGER_32)
		do
			refcount := a_refcount
			update_memory_field ("refcount")
		end

feature {ANY} -- public routines

	address_list_address_at (a_pos: INTEGER_32): CE_POINTER [PD_IP_ADDRESS]
		do
			if a_pos >= faulty then
				if not (a_pos < count) then
					assert_fail (ce_string ("pos >= al->faulty && pos < al->count"), ce_string ("host.c"), {NATURAL_32} 101, ce_string ("address_list_address_at"))
					update_class_fields -- Current might have been changed; synchronize
				end
			else
				assert_fail (ce_string ("pos >= al->faulty && pos < al->count"), ce_string ("host.c"), {NATURAL_32} 101, ce_string ("address_list_address_at"))
				update_class_fields -- Current might have been changed; synchronize
			end
			Result := (addresses + a_pos)
		end

	address_list_delete
		do
			free (create {CE_VOID_POINTER}.make_cast (addresses))
			update_class_fields -- Current might have been changed; synchronize
			free (create {CE_VOID_POINTER}.make_shared (c_pointer))
			update_class_fields -- Current might have been changed; synchronize
		end

	address_list_release
		local
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_break: BOOLEAN
		do
			Current.refcount := refcount - 1
			update_class_fields -- Current might have been changed; synchronize
			from
			until
				(l_break)
			loop
				l_break := False
				l_tmp := builtin_expect (((create {P_OPTIONS_STATIC}).opt.item.item.debug2).to_integer, 0)
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp.to_boolean then
					(create {P_LOG_STATIC}).debug_logprintf ([ce_string ("Releasing 0x%%0*lx (new refcount %%d).%N"), ({NATURAL_32} 2 * {PLATFORM}.Pointer_bytes.to_natural_32).to_integer_32, (c_pointer.to_integer_32).to_natural_32, refcount])
					update_class_fields -- Current might have been changed; synchronize
				end
				l_break := True
			end
			l_break := False
			if refcount <= 0 then
				from
				until
					(l_break)
				loop
					l_break := False
					l_tmp___0 := builtin_expect (((create {P_OPTIONS_STATIC}).opt.item.item.debug2).to_integer, 0)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___0.to_boolean then
						(create {P_LOG_STATIC}).debug_logprintf ([ce_string ("Deleting unused 0x%%0*lx.%N"), ({NATURAL_32} 2 * {PLATFORM}.Pointer_bytes.to_natural_32).to_integer_32, (c_pointer.to_integer_32).to_natural_32])
						update_class_fields -- Current might have been changed; synchronize
					end
					l_break := True
				end
				l_break := False;
				((create {CE_CLASS_POINTER [PD_ADDRESS_LIST]}.make_shared (c_pointer))).item.address_list_delete
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	cache_store (a_host: CE_POINTER [INTEGER_8])
		local
			l_tmp: CE_POINTER [INTEGER_8]
			l_i: INTEGER_32
			l_tmp___0: CE_POINTER [INTEGER_8]
			l_tmp___1: INTEGER_32
		do
			if ((create {P_HOST_STATIC}).host_name_addresses_map.item).is_void then
				(create {P_HOST_STATIC}).host_name_addresses_map.item := (create {P_HASH_STATIC}).make_nocase_string_hash_table (0)
				update_class_fields -- Current might have been changed; synchronize
			end
			Current.refcount := refcount + 1
			update_class_fields -- Current might have been changed; synchronize
			l_tmp := xstrdup_lower (a_host)
			update_class_fields; -- Current might have been changed; synchronize
			((create {P_HOST_STATIC}).host_name_addresses_map.item).item.hash_table_put (create {CE_VOID_POINTER}.make_cast (l_tmp), create {CE_VOID_POINTER}.make_shared (c_pointer))
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___1 := builtin_expect (((create {P_OPTIONS_STATIC}).opt.item.item.debug2).to_integer, 0)
			update_class_fields -- Current might have been changed; synchronize
			if l_tmp___1.to_boolean then
				(create {P_LOG_STATIC}).debug_logprintf ([ce_string ("Caching %%s =>"), a_host])
				update_class_fields -- Current might have been changed; synchronize
				l_i := 0
				from
				until
					not (l_i < count)
				loop
					l_tmp___0 := (addresses + l_i).item.print_address
					update_class_fields; -- Current might have been changed; synchronize
					(create {P_LOG_STATIC}).debug_logprintf ([ce_string (" %%s"), l_tmp___0])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				(create {P_LOG_STATIC}).debug_logprintf ([ce_string ("%N")])
				update_class_fields -- Current might have been changed; synchronize
			end
		end

feature {PD_ADDRESS_LIST} -- private routines

	xstrdup_lower (a_s: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_copy: CE_POINTER [INTEGER_8]
			l_p___0: CE_POINTER [INTEGER_8]
		do
			l_copy := ((create {P_XMALLOC_STATIC}).xstrdup (a_s))
			l_p___0 := l_copy
			from
			until
				not ((l_p___0[0]).to_boolean)
			loop
				l_p___0[0] := ((create {P_C_CTYPE_STATIC}).c2_tolower ((l_p___0[0]).to_integer_32)).to_integer_8
				l_p___0 := l_p___0 + 1
			end
			Result := l_copy
		end

feature {ANY}

	copy (a_other: like Current)
		do
			count := a_other.count
			addresses := a_other.addresses
			faulty := a_other.faulty
			connected := a_other.connected
			refcount := a_other.refcount
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				count = a_other.count and
				addresses.is_equal (a_other.addresses) and
				faulty = a_other.faulty and
				connected = a_other.connected and
				refcount = a_other.refcount
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("count"),
					create {CE_STRUCTURE_FIELD}.make ("addresses"),
					create {CE_STRUCTURE_FIELD}.make ("faulty"),
					create {CE_STRUCTURE_FIELD}.make ("connected"),
					create {CE_STRUCTURE_FIELD}.make ("refcount")
				>>)
			init_structure_info
		end

end
