class
	PD_HASH_TABLE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create hash_function.make
			create test_function.make
			create {CE_CLASS_POINTER [PD_CELL]} cells.make
			Precursor
		end

feature {ANY} -- fields

	hash_function: CE_ROUTINE assign set_hash_function

	test_function: CE_ROUTINE assign set_test_function

	cells: CE_POINTER [PD_CELL] assign set_cells

	size: INTEGER_32 assign set_size

	count: INTEGER_32 assign set_count

	resize_threshold: INTEGER_32 assign set_resize_threshold

	prime_offset: INTEGER_32 assign set_prime_offset

feature {ANY} -- field setters

	set_hash_function (a_hash_function: CE_ROUTINE)
		do
			hash_function := a_hash_function
			update_memory_field ("hash_function")
		end

	set_test_function (a_test_function: CE_ROUTINE)
		do
			test_function := a_test_function
			update_memory_field ("test_function")
		end

	set_cells (a_cells: CE_POINTER [PD_CELL])
		do
			cells := a_cells
			update_memory_field ("cells")
		end

	set_size (a_size: INTEGER_32)
		do
			size := a_size
			update_memory_field ("size")
		end

	set_count (a_count: INTEGER_32)
		do
			count := a_count
			update_memory_field ("count")
		end

	set_resize_threshold (a_resize_threshold: INTEGER_32)
		do
			resize_threshold := a_resize_threshold
			update_memory_field ("resize_threshold")
		end

	set_prime_offset (a_prime_offset: INTEGER_32)
		do
			prime_offset := a_prime_offset
			update_memory_field ("prime_offset")
		end

feature {ANY} -- public routines

	hash_table_destroy
		do
			free (create {CE_VOID_POINTER}.make_cast (cells))
			update_class_fields -- Current might have been changed; synchronize
			free (create {CE_VOID_POINTER}.make_shared (c_pointer))
			update_class_fields -- Current might have been changed; synchronize
		end

	find_cell (a_key: CE_POINTER [ANY]): CE_POINTER [PD_CELL]
		local
			l_cells: CE_POINTER [PD_CELL]
			l_size: INTEGER_32
			l_c: CE_POINTER [PD_CELL]
			l_tmp: NATURAL_32
			l_equals: CE_ROUTINE
			l_tmp___0: INTEGER_32
			l_break: BOOLEAN
		do
			l_cells := cells
			l_size := size
			l_tmp := hash_function.natural_32_item ([a_key])
			update_class_fields -- Current might have been changed; synchronize
			l_c := l_cells + (l_tmp \\ l_size.to_natural_32).to_integer_32
			l_equals := test_function
			from
			until
				not (not (l_break) and then (((l_c.item.key).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32))
			loop
				l_break := False
				l_tmp___0 := l_equals.integer_32_item ([a_key, l_c.item.key])
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp___0.to_boolean then
					l_break := True
				end
				if not (l_break) then
					if not l_c.is_equal (l_cells + (l_size - 1)) then
						l_c := l_c + 1
					else
						l_c := l_cells
					end
				end
			end
			l_break := False
			Result := l_c
		end

	hash_table_get (a_key: CE_POINTER [ANY]): CE_POINTER [ANY]
		local
			l_c: CE_POINTER [PD_CELL]
			l_tmp: CE_POINTER [PD_CELL]
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.find_cell (a_key)
			update_class_fields -- Current might have been changed; synchronize
			l_c := l_tmp
			if ((l_c.item.key).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32 then
				Result := (l_c.item.value)
			else
				Result := (create {CE_VOID_POINTER}.make_shared (default_pointer))
			end
		end

	hash_table_get_pair (a_lookup_key, a_orig_key, a_value: CE_POINTER [ANY]): INTEGER_32
		local
			l_c: CE_POINTER [PD_CELL]
			l_tmp: CE_POINTER [PD_CELL]
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.find_cell (a_lookup_key)
			update_class_fields -- Current might have been changed; synchronize
			l_c := l_tmp
			if ((l_c.item.key).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32 then
				if (not a_orig_key.is_void) then
					(create {CE_POINTER_POINTER [ANY]}.make_cast (a_orig_key)).item := l_c.item.key
					update_class_fields -- Current might have been changed; synchronize
				end
				if (not a_value.is_void) then
					(create {CE_POINTER_POINTER [ANY]}.make_cast (a_value)).item := l_c.item.value
					update_class_fields -- Current might have been changed; synchronize
				end
				Result := 1
			else
				Result := 0
			end
		end

	hash_table_contains (a_key: CE_POINTER [ANY]): INTEGER_32
		local
			l_c: CE_POINTER [PD_CELL]
			l_tmp: CE_POINTER [PD_CELL]
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.find_cell (a_key)
			update_class_fields -- Current might have been changed; synchronize
			l_c := l_tmp
			Result := (((l_c.item.key).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32).to_integer
		end

	grow_hash_table
		local
			l_hasher: CE_ROUTINE
			l_old_cells: CE_POINTER [PD_CELL]
			l_old_end: CE_POINTER [PD_CELL]
			l_c: CE_POINTER [PD_CELL]
			l_cells: CE_POINTER [PD_CELL]
			l_newsize: INTEGER_32
			l_tmp: CE_POINTER [ANY]
			l_new_c: CE_POINTER [PD_CELL]
			l_tmp___0: NATURAL_32
			l_return: BOOLEAN
		do
			l_hasher := hash_function
			l_old_cells := cells
			l_old_end := cells + size
			l_newsize := (create {P_HASH_STATIC}).prime_size (size * 2, create {CE_INTEGER_32_POINTER}.make_shared (c_pointer + (create {PD_HASH_TABLE}.make).offset_of ("prime_offset")))
			update_class_fields -- Current might have been changed; synchronize
			Current.size := l_newsize
			update_class_fields -- Current might have been changed; synchronize
			Current.resize_threshold := (l_newsize.to_double * 0.75).truncated_to_integer
			update_class_fields -- Current might have been changed; synchronize
			l_tmp := (create {P_XMALLOC_STATIC}).xmalloc (l_newsize.to_natural_32 * (create {PD_CELL}.make).structure_size.to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_cells := create {CE_CLASS_POINTER [PD_CELL]}.make_cast (l_tmp)
			memset (create {CE_VOID_POINTER}.make_cast (l_cells), 255, l_newsize.to_natural_32 * (create {PD_CELL}.make).structure_size.to_natural_32).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			Current.cells := l_cells
			update_class_fields -- Current might have been changed; synchronize
			l_c := l_old_cells
			from
			until
				not (not (l_return) and then ((l_c.c_pointer.to_integer_32).to_natural_32 < (l_old_end.c_pointer.to_integer_32).to_natural_32))
			loop
				if ((l_c.item.key).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32 then
					l_tmp___0 := l_hasher.natural_32_item ([l_c.item.key])
					update_class_fields -- Current might have been changed; synchronize
					l_new_c := l_cells + (l_tmp___0 \\ l_newsize.to_natural_32).to_integer_32
					from
					until
						((l_new_c.item.key).c_pointer.to_integer_32).to_natural_32 = ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32
					loop
						if not l_new_c.is_equal (l_cells + (l_newsize - 1)) then
							l_new_c := l_new_c + 1
						else
							l_new_c := l_cells
						end
					end
					l_new_c.item := (l_c.item).twin
					update_class_fields -- Current might have been changed; synchronize
				end
				l_c := l_c + 1
			end
			free (create {CE_VOID_POINTER}.make_cast (l_old_cells))
			update_class_fields -- Current might have been changed; synchronize
		end

	hash_table_put (a_key, a_value: CE_POINTER [ANY])
		local
			l_c: CE_POINTER [PD_CELL]
			l_tmp: CE_POINTER [PD_CELL]
			l_return: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.find_cell (a_key)
			update_class_fields -- Current might have been changed; synchronize
			l_c := l_tmp
			if ((l_c.item.key).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32 then
				l_c.item.key := a_key
				update_class_fields -- Current might have been changed; synchronize
				l_c.item.value := a_value
				update_class_fields -- Current might have been changed; synchronize
				l_return := True
			end
			if not l_return then
				if count >= resize_threshold then
					((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.grow_hash_table
					update_class_fields -- Current might have been changed; synchronize
					l_c := ((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.find_cell (a_key)
					update_class_fields -- Current might have been changed; synchronize
				end
				Current.count := count + 1
				update_class_fields -- Current might have been changed; synchronize
				l_c.item.key := a_key
				update_class_fields -- Current might have been changed; synchronize
				l_c.item.value := a_value
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	hash_table_remove (a_key: CE_POINTER [ANY]): INTEGER_32
		local
			l_c: CE_POINTER [PD_CELL]
			l_tmp: CE_POINTER [PD_CELL]
			l_size: INTEGER_32
			l_cells: CE_POINTER [PD_CELL]
			l_hasher: CE_ROUTINE
			l_key2: CE_POINTER [ANY]
			l_c_new: CE_POINTER [PD_CELL]
			l_tmp___0: NATURAL_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_tmp := ((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.find_cell (a_key)
					update_class_fields -- Current might have been changed; synchronize
					l_c := l_tmp
					if (((l_c.item.key).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						Result := 0
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 3 then -- else:
						l_size := size
						l_cells := cells
						l_hasher := hash_function
						l_c.item.key := create {CE_VOID_POINTER}.make_shared (default_pointer + (((0).to_natural_32).bit_not).to_integer_32)
						update_class_fields -- Current might have been changed; synchronize
						Current.count := count - 1
						update_class_fields -- Current might have been changed; synchronize
						if not l_c.is_equal (l_cells + (l_size - 1)) then
							l_c := l_c + 1
						else
							l_c := l_cells
						end
				when 4 then -- loop condition:
						if not (not (l_return) and then (((l_c.item.key).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32)) then
							l_pos := -1 + 11 -- skip loop
						end
				when 5 then -- loop body:
						l_key2 := l_c.item.key
						l_tmp___0 := l_hasher.natural_32_item ([l_key2])
						update_class_fields -- Current might have been changed; synchronize
						l_c_new := l_cells + (l_tmp___0 \\ l_size.to_natural_32).to_integer_32
				when 6 then -- loop condition:
						if ((l_c_new.item.key).c_pointer.to_integer_32).to_natural_32 = ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32 then
							l_pos := -1 + 9 -- skip loop
						end
				when 7 then -- loop body:
						if l_key2.is_equal (l_c_new.item.key) then
							l_pos := -1 + 10 -- goto next_rehash
						end
				when 8 then
						if not l_c_new.is_equal (l_cells + (l_size - 1)) then
							l_c_new := l_c_new + 1
						else
							l_c_new := l_cells
						end
						l_pos := -1 + 6 -- back to loop condition
				when 9 then -- loop done
						l_c_new.item := (l_c.item).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c.item.key := create {CE_VOID_POINTER}.make_shared (default_pointer + (((0).to_natural_32).bit_not).to_integer_32)
						update_class_fields -- Current might have been changed; synchronize
				when 10 then -- next_rehash:
						
						if not l_c.is_equal (l_cells + (l_size - 1)) then
							l_c := l_c + 1
						else
							l_c := l_cells
						end
						l_pos := -1 + 4 -- back to loop condition
				when 11 then -- loop done
						Result := 1
				when 12 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	hash_table_clear
		do
			memset (create {CE_VOID_POINTER}.make_cast (cells), 255, size.to_natural_32 * (create {PD_CELL}.make).structure_size.to_natural_32).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			Current.count := 0
			update_class_fields -- Current might have been changed; synchronize
		end

	hash_table_for_each (a_fn: CE_ROUTINE; a_arg: CE_POINTER [ANY])
		local
			l_c: CE_POINTER [PD_CELL]
			l_end: CE_POINTER [PD_CELL]
			l_key: CE_POINTER [ANY]
			l_tmp: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_c := cells
					l_end := cells + size
				when 1 then -- loop condition:
					if not (not (l_return) and then ((l_c.c_pointer.to_integer_32).to_natural_32 < (l_end.c_pointer.to_integer_32).to_natural_32)) then
						l_pos := -1 + 5 -- skip loop
					end
				when 2 then -- loop body:
						if ((l_c.item.key).c_pointer.to_integer_32).to_natural_32 = ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32 then
							l_pos := -1 + 4 -- skip if block
						end
				when 3 then -- if: | repeat:
						l_key := l_c.item.key
						l_tmp := a_fn.integer_32_item ([l_key, l_c.item.value, a_arg])
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp.to_boolean then
							l_return := True
						end
						if not l_return and then (not (l_c.item.key).is_equal (l_key)) then
							if ((l_c.item.key).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_VOID_POINTER}.make_shared (default_pointer + ({NATURAL_32} 4_294_967_295).to_integer_32)).c_pointer.to_integer_32).to_natural_32 then
								l_pos := -1 + 3 -- goto repeat
							end
						end
				when 4 then
						if not l_return then
							l_c := l_c + 1
						end
						l_pos := -1 + 1 -- back to loop condition
				when 5 then -- loop done
					if not l_return then
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	hash_table_iterate (a_iter: CE_POINTER [PD_HASH_TABLE_ITERATOR])
		require
			not a_iter.is_void
		do
			a_iter.item.pos := create {CE_VOID_POINTER}.make_cast (cells)
			update_class_fields -- Current might have been changed; synchronize
			a_iter.item.end2 := create {CE_VOID_POINTER}.make_cast ((cells + size))
			update_class_fields -- Current might have been changed; synchronize
		end

	hash_table_count: INTEGER_32
		do
			Result := count
		end

	string_set_add (a_s: CE_POINTER [INTEGER_8])
		local
			l_tmp: INTEGER_32
			l_tmp___0: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.hash_table_contains (create {CE_VOID_POINTER}.make_cast (a_s))
			update_class_fields -- Current might have been changed; synchronize
			if l_tmp.to_boolean then
				l_return := True
			end
			if not l_return then
				l_tmp___0 := (create {P_XMALLOC_STATIC}).xstrdup (a_s)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.hash_table_put (create {CE_VOID_POINTER}.make_cast (l_tmp___0), create {CE_VOID_POINTER}.make_cast (ce_string ("1")))
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	string_set_contains (a_s: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.hash_table_contains (create {CE_VOID_POINTER}.make_cast (a_s))
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	string_set_to_array (a_array: CE_POINTER [CE_POINTER [INTEGER_8]])
		local
			l_array: CE_POINTER [CE_POINTER [INTEGER_8]]
			l_iter: CE_ARRAY [PD_HASH_TABLE_ITERATOR]
			l_tmp: CE_POINTER [CE_POINTER [INTEGER_8]]
			l_tmp___0: INTEGER_32
			l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_HASH_TABLE_ITERATOR]} l_iter.make (<<1>>)

			l_array := a_array;
			((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.hash_table_iterate (l_iter)
			update_class_fields -- Current might have been changed; synchronize
			from
			until
				(l_break)
			loop
				l_break := False
				l_tmp___0 := l_iter.item.hash_table_iter_next
				update_class_fields -- Current might have been changed; synchronize
				if not l_tmp___0.to_boolean then
					l_break := True
				end
				if not (l_break) then
					l_tmp := l_array
					l_array := l_array + 1
					l_tmp.item := create {CE_INTEGER_8_POINTER}.make_cast (l_iter.item.key)
					update_class_fields -- Current might have been changed; synchronize
				end
			end
			l_break := False
		end

	string_set_free
		local
			l_iter: CE_ARRAY [PD_HASH_TABLE_ITERATOR]
			l_tmp: INTEGER_32
			l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_HASH_TABLE_ITERATOR]} l_iter.make (<<1>>);

			((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.hash_table_iterate (l_iter)
			update_class_fields -- Current might have been changed; synchronize
			from
			until
				(l_break)
			loop
				l_break := False
				l_tmp := l_iter.item.hash_table_iter_next
				update_class_fields -- Current might have been changed; synchronize
				if not l_tmp.to_boolean then
					l_break := True
				end
				if not (l_break) then
					free (l_iter.item.key)
					update_class_fields -- Current might have been changed; synchronize
				end
			end
			l_break := False;
			((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.hash_table_destroy
			update_class_fields -- Current might have been changed; synchronize
		end

	free_keys_and_values
		local
			l_iter: CE_ARRAY [PD_HASH_TABLE_ITERATOR]
			l_tmp: INTEGER_32
			l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_HASH_TABLE_ITERATOR]} l_iter.make (<<1>>);

			((create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make_shared (c_pointer))).item.hash_table_iterate (l_iter)
			update_class_fields -- Current might have been changed; synchronize
			from
			until
				(l_break)
			loop
				l_break := False
				l_tmp := l_iter.item.hash_table_iter_next
				update_class_fields -- Current might have been changed; synchronize
				if not l_tmp.to_boolean then
					l_break := True
				end
				if not (l_break) then
					free (l_iter.item.key)
					update_class_fields -- Current might have been changed; synchronize
					free (l_iter.item.value)
					update_class_fields -- Current might have been changed; synchronize
				end
			end
			l_break := False
		end

feature {ANY}

	copy (a_other: like Current)
		do
			hash_function := a_other.hash_function
			test_function := a_other.test_function
			cells := a_other.cells
			size := a_other.size
			count := a_other.count
			resize_threshold := a_other.resize_threshold
			prime_offset := a_other.prime_offset
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				hash_function.is_equal (a_other.hash_function) and
				test_function.is_equal (a_other.test_function) and
				cells.is_equal (a_other.cells) and
				size = a_other.size and
				count = a_other.count and
				resize_threshold = a_other.resize_threshold and
				prime_offset = a_other.prime_offset
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("hash_function"),
					create {CE_STRUCTURE_FIELD}.make ("test_function"),
					create {CE_STRUCTURE_FIELD}.make ("cells"),
					create {CE_STRUCTURE_FIELD}.make ("size"),
					create {CE_STRUCTURE_FIELD}.make ("count"),
					create {CE_STRUCTURE_FIELD}.make ("resize_threshold"),
					create {CE_STRUCTURE_FIELD}.make ("prime_offset")
				>>)
			init_structure_info
		end

end
