class
	P_ERROR_STATIC

inherit
	CE_UTILITIES
	S_SIGNAL_STATIC
	S_STDIO_STATIC
	S_UNISTD_STATIC
	S_STRING_STATIC
	S_STDLIB_STATIC

feature {ANY} -- public attributes

	dummy_val: CE_STATIC [CE_ARRAY [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_ARRAY}.make (<<1>>)
		end

	dummy_ptr: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := dummy_val.item
		end

	mem_amount: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	last_mem_amount: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := -1
		end

	memory_list: CE_STATIC [CE_ARRAY [PD_LIST_HEAD]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_LIST_HEAD]}.make (<<1>>)
			Result.item[0] := create {PD_LIST_HEAD}.make_initialized ([create {CE_VOID_POINTER}.make_cast (memory_list.item), create {CE_VOID_POINTER}.make_cast (memory_list.item)])
		end

	errbuf: CE_STATIC [CE_ARRAY [NATURAL_8]]
		once ("PROCESS")
			Result.item := create {CE_NATURAL_8_ARRAY}.make (<<4_096>>)
		end

feature {ANY} -- public routines

	do_not_optimize_here (a_p: CE_POINTER [ANY])
		do
			dummy_ptr.item[0] := (0).to_integer_8
		end

	force_dump
		do
			Io.error.put_string ("%N%/27/[1mForcing core dump%/27/[0m%N")
			Io.standard_default.flush
			Io.error.flush;
			(create {P_KBD_STATIC}).fatal_tty_exit
			raise (11).do_nothing
		end

	check_memory_leaks
		local
			l_r: INTEGER_32
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			l_tmp: CE_POINTER [INTEGER_8]
		do
			if mem_amount.item.to_boolean then
				fprintf ([(create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), ce_string ("%N%/27/[1mMemory leak by %%ld bytes%/27/[0m%N"), mem_amount.item]).do_nothing
				Io.error.put_string ("%NList of blocks: ")
				l_r := 0
				l_ah := create {CE_CLASS_POINTER [PD_ALLOC_HEADER]}.make_cast (memory_list.item.item.next)
				from
				until
					l_ah.is_equal (memory_list.item)
				loop
					if l_r.to_boolean then
						l_tmp := ce_string (", ")
					else
						l_tmp := ce_string ("")
					end
					fprintf ([(create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), ce_string ("%%s%%p:%%d @ %%s:%%d"), l_tmp, create {CE_INTEGER_8_POINTER}.make_cast (l_ah) + ((((create {PD_ALLOC_HEADER}.make).structure_size.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4_294_967_288)).to_integer_32, l_ah.item.size, l_ah.item.file, l_ah.item.line]).do_nothing
					l_r := 1
					if (not (l_ah.item.comment).is_void) then
						Io.error.put_string (":%"" + eif_string (create {CE_INTEGER_8_POINTER}.make_shared ((l_ah.item.comment).c_pointer)) + "%"")
					end
					l_ah := l_ah.item.next
				end
				Io.error.put_string ("%N")
				force_dump
			end
		end

	er (a_b: INTEGER_32; a_m: CE_POINTER [NATURAL_8]; a_l: CE_VA_LIST)
		local
			l_l: CE_VA_LIST
		do
			l_l := a_l.twin
			if a_b.to_boolean then
				Io.error.put_string ("")
			end
			vfprintf ((create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), create {CE_INTEGER_8_POINTER}.make_cast (a_m), l_l.twin).do_nothing
			Io.error.put_string ("%N")
			sleep ({NATURAL_32} 1).do_nothing
		end

	error (a_args: TUPLE [a_m: CE_POINTER [NATURAL_8]])
		local
			l_l: CE_VA_LIST
		do
			create l_l.make_shared (default_pointer)

			l_l := create {CE_VA_LIST}.make (a_args, agent error)
			er (1, a_args.a_m, l_l.twin)
		end

	int_error (a_args: TUPLE [a_m: CE_POINTER [NATURAL_8]])
		local
			l_l: CE_VA_LIST
		do
			create l_l.make_shared (default_pointer)

			l_l := create {CE_VA_LIST}.make (a_args, agent int_error)
			sprintf ([create {CE_INTEGER_8_POINTER}.make_cast (errbuf.item), ce_string ("%/27/[1mINTERNAL ERROR%/27/[0m at %%s:%%d: "), (create {P_LINKS_STATIC}).errfile.item, (create {P_LINKS_STATIC}).errline.item]).do_nothing
			strcat (create {CE_INTEGER_8_POINTER}.make_cast (errbuf.item), create {CE_INTEGER_8_POINTER}.make_cast (a_args.a_m)).do_nothing
			er (1, errbuf.item, l_l.twin)
			force_dump
		end

	debug_mem_alloc (a_file: CE_POINTER [NATURAL_8]; a_line: INTEGER_32; a_size: NATURAL_32): CE_POINTER [ANY]
		local
			l_size: NATURAL_32
			l_p: CE_POINTER [ANY]
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			l_return, l_break: BOOLEAN
		do
			l_size := a_size
			if not l_size.to_boolean then
				Result := (create {CE_VOID_POINTER}.make_shared (default_pointer + -1))
				l_return := True
			end
			if not l_return then
				if l_size > {NATURAL_32} 2_147_483_647 then
					from
					until
						False
					loop
						error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("error.c"), 106]);
						(create {P_KBD_STATIC}).fatal_tty_exit
						exit (4)
					end
				end
				mem_amount.item := (mem_amount.item.to_natural_32 + l_size.to_natural_32).to_integer_32
				l_size := l_size + ((((create {PD_ALLOC_HEADER}.make).structure_size.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4_294_967_288))
				l_p := malloc (l_size)
				if l_p.is_void then
					error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("ERROR: out of memory (malloc returned NULL)"))]);
					(create {P_KBD_STATIC}).fatal_tty_exit
					exit (4)
					Result := (create {CE_VOID_POINTER}.make_shared (default_pointer))
					l_return := True
				end
			end
			if not l_return then
				l_ah := create {CE_CLASS_POINTER [PD_ALLOC_HEADER]}.make_cast (l_p)
				l_p := create {CE_VOID_POINTER}.make_cast ((create {CE_INTEGER_8_POINTER}.make_cast (l_p) + ((((create {PD_ALLOC_HEADER}.make).structure_size.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4_294_967_288)).to_integer_32))
				l_ah.item.size := (l_size - ((((create {PD_ALLOC_HEADER}.make).structure_size.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4_294_967_288))).to_integer_32
				l_ah.item.file := a_file
				l_ah.item.line := a_line
				l_ah.item.comment := create {CE_NATURAL_8_POINTER}.make_shared (default_pointer)
				from
				until
					(l_break)
				loop
					l_break := False
					do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (memory_list.item))
					l_ah.item.next := (create {CE_CLASS_POINTER [PD_ALLOC_HEADER]}.make_cast (memory_list.item)).item.next
					l_ah.item.prev := create {CE_CLASS_POINTER [PD_ALLOC_HEADER]}.make_cast (memory_list.item);
					(create {CE_CLASS_POINTER [PD_ALLOC_HEADER]}.make_cast (memory_list.item)).item.next := l_ah;
					(l_ah.item.next).item.prev := l_ah
					do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (memory_list.item))
					l_break := True
				end
				l_break := False
				Result := l_p
			end
		end

	debug_mem_free (a_file: CE_POINTER [NATURAL_8]; a_line: INTEGER_32; a_p: CE_POINTER [ANY])
		local
			l_p: CE_POINTER [ANY]
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			l_return: BOOLEAN
		do
			l_p := a_p
			if (l_p.c_pointer.to_integer_32).to_natural_32 = ((create {CE_VOID_POINTER}.make_shared (default_pointer + -1)).c_pointer.to_integer_32).to_natural_32 then
				l_return := True
			end
			if not l_return and then (l_p.is_void) then
				(create {P_LINKS_STATIC}).errfile.item := a_file;
				(create {P_LINKS_STATIC}).errline.item := a_line
				int_error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("mem_free(NULL)"))])
				l_return := True
			end
			if not l_return then
				l_p := create {CE_VOID_POINTER}.make_cast ((create {CE_INTEGER_8_POINTER}.make_cast (l_p) - ((((create {PD_ALLOC_HEADER}.make).structure_size.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4_294_967_288)).to_integer_32))
				l_ah := create {CE_CLASS_POINTER [PD_ALLOC_HEADER]}.make_cast (l_p)
				do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_ah));
				(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_cast (l_ah.item.next)).item.prev := create {CE_VOID_POINTER}.make_cast (l_ah.item.prev);
				(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_cast (l_ah.item.prev)).item.next := create {CE_VOID_POINTER}.make_cast (l_ah.item.next)
				do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_ah))
				if (not (l_ah.item.comment).is_void) then
					free (create {CE_VOID_POINTER}.make_cast (l_ah.item.comment))
				end
				mem_amount.item := mem_amount.item - (l_ah.item.size).to_integer_32
				free (l_p)
			end
		end

	debug_mem_realloc (a_file: CE_POINTER [NATURAL_8]; a_line: INTEGER_32; a_p: CE_POINTER [ANY]; a_size: NATURAL_32): CE_POINTER [ANY]
		local
			l_p: CE_POINTER [ANY]
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			l_return: BOOLEAN
		do
			l_p := a_p
			if (l_p.c_pointer.to_integer_32).to_natural_32 = ((create {CE_VOID_POINTER}.make_shared (default_pointer + -1)).c_pointer.to_integer_32).to_natural_32 then
				Result := (debug_mem_alloc (a_file, a_line, a_size))
				l_return := True
			end
			if not l_return and then (l_p.is_void) then
				(create {P_LINKS_STATIC}).errfile.item := a_file;
				(create {P_LINKS_STATIC}).errline.item := a_line
				int_error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("mem_realloc(NULL, %%d)")), a_size])
				Result := (create {CE_VOID_POINTER}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return and then (not a_size.to_boolean) then
				debug_mem_free (a_file, a_line, l_p)
				Result := (create {CE_VOID_POINTER}.make_shared (default_pointer + -1))
				l_return := True
			end
			if not l_return then
				if a_size > {NATURAL_32} 2_147_483_647 then
					from
					until
						False
					loop
						error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("error.c"), 167]);
						(create {P_KBD_STATIC}).fatal_tty_exit
						exit (4)
					end
				end
				l_p := realloc (create {CE_VOID_POINTER}.make_cast ((create {CE_INTEGER_8_POINTER}.make_cast (l_p) - ((((create {PD_ALLOC_HEADER}.make).structure_size.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4_294_967_288)).to_integer_32)), a_size + ((((create {PD_ALLOC_HEADER}.make).structure_size.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4_294_967_288)))
				if l_p.is_void then
					error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("ERROR: out of memory (realloc returned NULL)"))]);
					(create {P_KBD_STATIC}).fatal_tty_exit
					exit (4)
					Result := (create {CE_VOID_POINTER}.make_shared (default_pointer))
					l_return := True
				end
			end
			if not l_return then
				l_ah := create {CE_CLASS_POINTER [PD_ALLOC_HEADER]}.make_cast (l_p)
				mem_amount.item := (mem_amount.item.to_natural_32 + (a_size - (l_ah.item.size).to_natural_32).to_natural_32).to_integer_32
				l_ah.item.size := a_size.to_integer_32;
				(l_ah.item.prev).item.next := l_ah;
				(l_ah.item.next).item.prev := l_ah
				Result := (create {CE_VOID_POINTER}.make_cast ((create {CE_INTEGER_8_POINTER}.make_cast (l_p) + ((((create {PD_ALLOC_HEADER}.make).structure_size.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4_294_967_288)).to_integer_32)))
			end
		end

	set_mem_comment (a_p: CE_POINTER [ANY]; a_c: CE_POINTER [NATURAL_8]; a_l: INTEGER_32)
		local
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			l_tmp: CE_POINTER [NATURAL_8]
		do
			l_ah := create {CE_CLASS_POINTER [PD_ALLOC_HEADER]}.make_cast ((create {CE_INTEGER_8_POINTER}.make_cast (a_p) - ((((create {PD_ALLOC_HEADER}.make).structure_size.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4_294_967_288)).to_integer_32))
			if (not (l_ah.item.comment).is_void) then
				free (create {CE_VOID_POINTER}.make_cast (l_ah.item.comment))
			end
			l_tmp := create {CE_NATURAL_8_POINTER}.make_cast ((malloc ((a_l + 1).to_natural_32)))
			l_ah.item.comment := l_tmp
			if (not l_tmp.is_void) then
				memcpy (create {CE_VOID_POINTER}.make_cast (l_ah.item.comment), create {CE_VOID_POINTER}.make_cast (a_c), a_l.to_natural_32).do_nothing
				l_ah.item.comment[a_l] := (0).to_natural_8
			end
		end

end
