class
	PD_DIALOG

inherit
	PD_DIALOG_STATIC
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_NATURAL_8_POINTER} title.make
			create fn.make
			create handle_event.make
			create abort.make
			create {CE_VOID_POINTER} udata.make
			create {CE_VOID_POINTER} udata2.make
			create refresh.make
			create {CE_VOID_POINTER} refresh_data.make
			create {CE_CLASS_ARRAY [PD_DIALOG_ITEM]} items.make (<<1>>)
		end

feature {ANY} -- fields

	title: CE_POINTER [NATURAL_8] assign set_title

	fn: CE_ROUTINE assign set_fn

	handle_event: CE_ROUTINE assign set_handle_event

	abort: CE_ROUTINE assign set_abort

	udata: CE_POINTER [ANY] assign set_udata

	udata2: CE_POINTER [ANY] assign set_udata2

	align: INTEGER_32 assign set_align

	refresh: CE_ROUTINE assign set_refresh

	refresh_data: CE_POINTER [ANY] assign set_refresh_data

	items: CE_ARRAY [PD_DIALOG_ITEM]

feature {ANY} -- field setters

	set_title (a_title: CE_POINTER [NATURAL_8])
		do
			title := a_title
			update_memory_field ("title")
		end

	set_fn (a_fn: CE_ROUTINE)
		do
			fn := a_fn
			update_memory_field ("fn")
		end

	set_handle_event (a_handle_event: CE_ROUTINE)
		do
			handle_event := a_handle_event
			update_memory_field ("handle_event")
		end

	set_abort (a_abort: CE_ROUTINE)
		do
			abort := a_abort
			update_memory_field ("abort")
		end

	set_udata (a_udata: CE_POINTER [ANY])
		do
			udata := a_udata
			update_memory_field ("udata")
		end

	set_udata2 (a_udata2: CE_POINTER [ANY])
		do
			udata2 := a_udata2
			update_memory_field ("udata2")
		end

	set_align (a_align: INTEGER_32)
		do
			align := a_align
			update_memory_field ("align")
		end

	set_refresh (a_refresh: CE_ROUTINE)
		do
			refresh := a_refresh
			update_memory_field ("refresh")
		end

	set_refresh_data (a_refresh_data: CE_POINTER [ANY])
		do
			refresh_data := a_refresh_data
			update_memory_field ("refresh_data")
		end

feature {ANY} -- public routines

	bookmark_edit_done
		local
			l_id: INTEGER_32
			l_parent: CE_POINTER [PD_DIALOG_DATA]
		do
			l_id := (udata2.c_pointer.to_integer_32).to_integer_32
			bookmark_update (l_id, items[0].data, items[1].data).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			l_parent := create {CE_CLASS_POINTER [PD_DIALOG_DATA]}.make_cast (udata)
			if (not l_parent.is_void) then
				(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_shared ((create {CE_CLASS_POINTER [PD_DLG_DATA_ITEM_DATA_BOX]}.make_cast ((l_parent.item.dlg).item.items[5].data)).c_pointer + (create {PD_DLG_DATA_ITEM_DATA_BOX}.make).offset_of ("items"))).item.bookmark_dlg_list_update.do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	bookmark_add_add
		local
			l_parent: CE_POINTER [PD_DIALOG_DATA]
		do
			(create {P_BOOKMARK_STATIC}).add_bookmark (items[0].data, items[1].data)
			update_class_fields -- Current might have been changed; synchronize
			l_parent := create {CE_CLASS_POINTER [PD_DIALOG_DATA]}.make_cast (udata)
			if (not l_parent.is_void) then
				(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_shared ((create {CE_CLASS_POINTER [PD_DLG_DATA_ITEM_DATA_BOX]}.make_cast ((l_parent.item.dlg).item.items[5].data)).c_pointer + (create {PD_DLG_DATA_ITEM_DATA_BOX}.make).offset_of ("items"))).item.bookmark_dlg_list_update.do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
		end

feature {PD_DIALOG} -- private routines

	bookmark_update (a_id: INTEGER_32; a_title, a_url: CE_POINTER [NATURAL_8]): INTEGER_32
		local
			l_bm: CE_POINTER [PD_BOOKMARK]
			l_return: BOOLEAN
		do
			l_bm := ((create {P_BOOKMARK_STATIC}).get_bookmark_by_id (a_id))
			if l_bm.is_void then
				Result := 0
				l_return := True
			end
			if not l_return then
				if (not a_title.is_void) then
					(create {P_ERROR_STATIC}).debug_mem_free (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("bookmark.c")), 173, create {CE_VOID_POINTER}.make_cast (l_bm.item.title))
					l_bm.item.title := (create {P_LINKS_STATIC}).debug_stracpy (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("bookmark.c")), 174, a_title)
				end
				if (not a_url.is_void) then
					(create {P_ERROR_STATIC}).debug_mem_free (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("bookmark.c")), 178, create {CE_VOID_POINTER}.make_cast (l_bm.item.url))
					l_bm.item.url := (create {P_LINKS_STATIC}).debug_stracpy (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("bookmark.c")), 179, a_url)
				end
				Result := 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			title := a_other.title
			fn := a_other.fn
			handle_event := a_other.handle_event
			abort := a_other.abort
			udata := a_other.udata
			udata2 := a_other.udata2
			align := a_other.align
			refresh := a_other.refresh
			refresh_data := a_other.refresh_data
			items := a_other.items.twin
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				title.is_equal (a_other.title) and
				fn.is_equal (a_other.fn) and
				handle_event.is_equal (a_other.handle_event) and
				abort.is_equal (a_other.abort) and
				udata.is_equal (a_other.udata) and
				udata2.is_equal (a_other.udata2) and
				align = a_other.align and
				refresh.is_equal (a_other.refresh) and
				refresh_data.is_equal (a_other.refresh_data) and
				items.is_equal (a_other.items)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("title"),
					create {CE_STRUCTURE_FIELD}.make ("fn"),
					create {CE_STRUCTURE_FIELD}.make ("handle_event"),
					create {CE_STRUCTURE_FIELD}.make ("abort"),
					create {CE_STRUCTURE_FIELD}.make ("udata"),
					create {CE_STRUCTURE_FIELD}.make ("udata2"),
					create {CE_STRUCTURE_FIELD}.make ("align"),
					create {CE_STRUCTURE_FIELD}.make ("refresh"),
					create {CE_STRUCTURE_FIELD}.make ("refresh_data"),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("items", <<1>>)
				>>)
			init_structure_info
		end

end
