class
	PD_CONTEXT_SHA256_T

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_NATURAL_32_ARRAY} total.make (<<2>>)
			create {CE_NATURAL_32_ARRAY} state.make (<<8>>)
			create {CE_NATURAL_8_ARRAY} buffer.make (<<64>>)
		end

feature {ANY} -- fields

	total: CE_ARRAY [NATURAL_32]

	state: CE_ARRAY [NATURAL_32]

	buffer: CE_ARRAY [NATURAL_8]

feature {PD_CONTEXT_SHA256_T} -- private attributes

	sha256_padding: CE_STATIC [CE_ARRAY [NATURAL_8]]
		once ("PROCESS")
			Result.item := create {CE_NATURAL_8_ARRAY}.make_initialized (<<
					128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
				>>)
		end

feature {ANY} -- public routines

	sha256_start
		do
			total[0] := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			total[1] := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			state[0] := (1_779_033_703).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			state[1] := {NATURAL_32} 3_144_134_277
			update_class_fields -- Current might have been changed; synchronize
			state[2] := (1_013_904_242).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			state[3] := {NATURAL_32} 2_773_480_762
			update_class_fields -- Current might have been changed; synchronize
			state[4] := (1_359_893_119).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			state[5] := {NATURAL_32} 2_600_822_924
			update_class_fields -- Current might have been changed; synchronize
			state[6] := (528_734_635).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			state[7] := (1_541_459_225).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
		end

	sha256_process (a_data: CE_POINTER [NATURAL_8])
		require
			not a_data.is_void
		local
			l_temp1: NATURAL_32
			l_temp2: NATURAL_32
			l_W: CE_ARRAY [NATURAL_32]
			l_A: NATURAL_32
			l_B: NATURAL_32
			l_C: NATURAL_32
			l_D: NATURAL_32
			l_E: NATURAL_32
			l_F: NATURAL_32
			l_G: NATURAL_32
			l_H: NATURAL_32
		do
			create {CE_NATURAL_32_ARRAY} l_W.make (<<64>>)

			l_W[0] := ((((((((a_data[0]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[1]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[2]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[3]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[1] := ((((((((a_data[4]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[5]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[6]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[7]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[2] := ((((((((a_data[8]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[9]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[10]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[11]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[3] := ((((((((a_data[12]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[13]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[14]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[15]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[4] := ((((((((a_data[16]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[17]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[18]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[19]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[5] := ((((((((a_data[20]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[21]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[22]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[23]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[6] := ((((((((a_data[24]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[25]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[26]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[27]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[7] := ((((((((a_data[28]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[29]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[30]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[31]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[8] := ((((((((a_data[32]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[33]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[34]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[35]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[9] := ((((((((a_data[36]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[37]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[38]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[39]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[10] := ((((((((a_data[40]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[41]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[42]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[43]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[11] := ((((((((a_data[44]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[45]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[46]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[47]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[12] := ((((((((a_data[48]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[49]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[50]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[51]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[13] := ((((((((a_data[52]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[53]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[54]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[55]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[14] := ((((((((a_data[56]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[57]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[58]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[59]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_W[15] := ((((((((a_data[60]).to_natural_32).bit_shift_left (24))).bit_or ((((a_data[61]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_data[62]).to_natural_32).bit_shift_left (8))))).bit_or ((a_data[63]).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_A := state[0]
			l_B := state[1]
			l_C := state[2]
			l_D := state[3]
			l_E := state[4]
			l_F := state[5]
			l_G := state[6]
			l_H := state[7]
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 1_116_352_408) + l_W[0]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 1_899_447_441) + l_W[1]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 3_049_323_471) + l_W[2]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 3_921_009_573) + l_W[3]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 961_987_163) + l_W[4]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 1_508_970_993) + l_W[5]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 2_453_635_748) + l_W[6]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 2_870_763_221) + l_W[7]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 3_624_381_080) + l_W[8]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 310_598_401) + l_W[9]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 607_225_278) + l_W[10]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 1_426_881_987) + l_W[11]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 1_925_078_388) + l_W[12]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 2_162_078_206) + l_W[13]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 2_614_888_103) + l_W[14]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 3_248_222_580) + l_W[15]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[16] := ((((((((((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[14]).bit_shift_left (15))))).bit_xor (((((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[14]).bit_shift_left (13))))))).bit_xor (((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[9]) + ((((((((((l_W[1]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[1]).bit_shift_left (25))))).bit_xor (((((((l_W[1]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[1]).bit_shift_left (14))))))).bit_xor (((((l_W[1]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[0]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 3_835_390_401) + l_W[16]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[17] := ((((((((((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[15]).bit_shift_left (15))))).bit_xor (((((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[15]).bit_shift_left (13))))))).bit_xor (((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[10]) + ((((((((((l_W[2]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[2]).bit_shift_left (25))))).bit_xor (((((((l_W[2]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[2]).bit_shift_left (14))))))).bit_xor (((((l_W[2]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[1]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 4_022_224_774) + l_W[17]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[18] := ((((((((((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[16]).bit_shift_left (15))))).bit_xor (((((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[16]).bit_shift_left (13))))))).bit_xor (((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[11]) + ((((((((((l_W[3]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[3]).bit_shift_left (25))))).bit_xor (((((((l_W[3]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[3]).bit_shift_left (14))))))).bit_xor (((((l_W[3]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[2]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 264_347_078) + l_W[18]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[19] := ((((((((((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[17]).bit_shift_left (15))))).bit_xor (((((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[17]).bit_shift_left (13))))))).bit_xor (((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[12]) + ((((((((((l_W[4]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[4]).bit_shift_left (25))))).bit_xor (((((((l_W[4]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[4]).bit_shift_left (14))))))).bit_xor (((((l_W[4]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[3]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 604_807_628) + l_W[19]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[20] := ((((((((((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[18]).bit_shift_left (15))))).bit_xor (((((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[18]).bit_shift_left (13))))))).bit_xor (((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[13]) + ((((((((((l_W[5]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[5]).bit_shift_left (25))))).bit_xor (((((((l_W[5]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[5]).bit_shift_left (14))))))).bit_xor (((((l_W[5]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[4]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 770_255_983) + l_W[20]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[21] := ((((((((((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[19]).bit_shift_left (15))))).bit_xor (((((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[19]).bit_shift_left (13))))))).bit_xor (((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[14]) + ((((((((((l_W[6]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[6]).bit_shift_left (25))))).bit_xor (((((((l_W[6]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[6]).bit_shift_left (14))))))).bit_xor (((((l_W[6]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[5]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 1_249_150_122) + l_W[21]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[22] := ((((((((((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[20]).bit_shift_left (15))))).bit_xor (((((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[20]).bit_shift_left (13))))))).bit_xor (((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[15]) + ((((((((((l_W[7]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[7]).bit_shift_left (25))))).bit_xor (((((((l_W[7]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[7]).bit_shift_left (14))))))).bit_xor (((((l_W[7]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[6]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 1_555_081_692) + l_W[22]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[23] := ((((((((((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[21]).bit_shift_left (15))))).bit_xor (((((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[21]).bit_shift_left (13))))))).bit_xor (((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[16]) + ((((((((((l_W[8]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[8]).bit_shift_left (25))))).bit_xor (((((((l_W[8]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[8]).bit_shift_left (14))))))).bit_xor (((((l_W[8]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[7]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 1_996_064_986) + l_W[23]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[24] := ((((((((((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[22]).bit_shift_left (15))))).bit_xor (((((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[22]).bit_shift_left (13))))))).bit_xor (((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[17]) + ((((((((((l_W[9]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[9]).bit_shift_left (25))))).bit_xor (((((((l_W[9]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[9]).bit_shift_left (14))))))).bit_xor (((((l_W[9]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[8]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 2_554_220_882) + l_W[24]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[25] := ((((((((((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[23]).bit_shift_left (15))))).bit_xor (((((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[23]).bit_shift_left (13))))))).bit_xor (((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[18]) + ((((((((((l_W[10]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[10]).bit_shift_left (25))))).bit_xor (((((((l_W[10]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[10]).bit_shift_left (14))))))).bit_xor (((((l_W[10]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[9]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 2_821_834_349) + l_W[25]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[26] := ((((((((((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[24]).bit_shift_left (15))))).bit_xor (((((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[24]).bit_shift_left (13))))))).bit_xor (((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[19]) + ((((((((((l_W[11]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[11]).bit_shift_left (25))))).bit_xor (((((((l_W[11]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[11]).bit_shift_left (14))))))).bit_xor (((((l_W[11]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[10]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 2_952_996_808) + l_W[26]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[27] := ((((((((((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[25]).bit_shift_left (15))))).bit_xor (((((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[25]).bit_shift_left (13))))))).bit_xor (((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[20]) + ((((((((((l_W[12]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[12]).bit_shift_left (25))))).bit_xor (((((((l_W[12]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[12]).bit_shift_left (14))))))).bit_xor (((((l_W[12]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[11]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 3_210_313_671) + l_W[27]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[28] := ((((((((((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[26]).bit_shift_left (15))))).bit_xor (((((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[26]).bit_shift_left (13))))))).bit_xor (((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[21]) + ((((((((((l_W[13]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[13]).bit_shift_left (25))))).bit_xor (((((((l_W[13]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[13]).bit_shift_left (14))))))).bit_xor (((((l_W[13]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[12]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 3_336_571_891) + l_W[28]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[29] := ((((((((((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[27]).bit_shift_left (15))))).bit_xor (((((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[27]).bit_shift_left (13))))))).bit_xor (((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[22]) + ((((((((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[14]).bit_shift_left (25))))).bit_xor (((((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[14]).bit_shift_left (14))))))).bit_xor (((((l_W[14]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[13]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 3_584_528_711) + l_W[29]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[30] := ((((((((((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[28]).bit_shift_left (15))))).bit_xor (((((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[28]).bit_shift_left (13))))))).bit_xor (((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[23]) + ((((((((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[15]).bit_shift_left (25))))).bit_xor (((((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[15]).bit_shift_left (14))))))).bit_xor (((((l_W[15]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[14]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 113_926_993) + l_W[30]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[31] := ((((((((((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[29]).bit_shift_left (15))))).bit_xor (((((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[29]).bit_shift_left (13))))))).bit_xor (((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[24]) + ((((((((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[16]).bit_shift_left (25))))).bit_xor (((((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[16]).bit_shift_left (14))))))).bit_xor (((((l_W[16]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[15]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 338_241_895) + l_W[31]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[32] := ((((((((((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[30]).bit_shift_left (15))))).bit_xor (((((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[30]).bit_shift_left (13))))))).bit_xor (((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[25]) + ((((((((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[17]).bit_shift_left (25))))).bit_xor (((((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[17]).bit_shift_left (14))))))).bit_xor (((((l_W[17]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[16]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 666_307_205) + l_W[32]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[33] := ((((((((((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[31]).bit_shift_left (15))))).bit_xor (((((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[31]).bit_shift_left (13))))))).bit_xor (((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[26]) + ((((((((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[18]).bit_shift_left (25))))).bit_xor (((((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[18]).bit_shift_left (14))))))).bit_xor (((((l_W[18]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[17]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 773_529_912) + l_W[33]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[34] := ((((((((((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[32]).bit_shift_left (15))))).bit_xor (((((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[32]).bit_shift_left (13))))))).bit_xor (((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[27]) + ((((((((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[19]).bit_shift_left (25))))).bit_xor (((((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[19]).bit_shift_left (14))))))).bit_xor (((((l_W[19]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[18]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 1_294_757_372) + l_W[34]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[35] := ((((((((((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[33]).bit_shift_left (15))))).bit_xor (((((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[33]).bit_shift_left (13))))))).bit_xor (((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[28]) + ((((((((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[20]).bit_shift_left (25))))).bit_xor (((((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[20]).bit_shift_left (14))))))).bit_xor (((((l_W[20]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[19]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 1_396_182_291) + l_W[35]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[36] := ((((((((((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[34]).bit_shift_left (15))))).bit_xor (((((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[34]).bit_shift_left (13))))))).bit_xor (((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[29]) + ((((((((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[21]).bit_shift_left (25))))).bit_xor (((((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[21]).bit_shift_left (14))))))).bit_xor (((((l_W[21]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[20]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 1_695_183_700) + l_W[36]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[37] := ((((((((((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[35]).bit_shift_left (15))))).bit_xor (((((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[35]).bit_shift_left (13))))))).bit_xor (((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[30]) + ((((((((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[22]).bit_shift_left (25))))).bit_xor (((((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[22]).bit_shift_left (14))))))).bit_xor (((((l_W[22]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[21]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 1_986_661_051) + l_W[37]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[38] := ((((((((((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[36]).bit_shift_left (15))))).bit_xor (((((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[36]).bit_shift_left (13))))))).bit_xor (((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[31]) + ((((((((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[23]).bit_shift_left (25))))).bit_xor (((((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[23]).bit_shift_left (14))))))).bit_xor (((((l_W[23]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[22]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 2_177_026_350) + l_W[38]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[39] := ((((((((((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[37]).bit_shift_left (15))))).bit_xor (((((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[37]).bit_shift_left (13))))))).bit_xor (((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[32]) + ((((((((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[24]).bit_shift_left (25))))).bit_xor (((((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[24]).bit_shift_left (14))))))).bit_xor (((((l_W[24]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[23]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 2_456_956_037) + l_W[39]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[40] := ((((((((((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[38]).bit_shift_left (15))))).bit_xor (((((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[38]).bit_shift_left (13))))))).bit_xor (((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[33]) + ((((((((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[25]).bit_shift_left (25))))).bit_xor (((((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[25]).bit_shift_left (14))))))).bit_xor (((((l_W[25]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[24]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 2_730_485_921) + l_W[40]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[41] := ((((((((((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[39]).bit_shift_left (15))))).bit_xor (((((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[39]).bit_shift_left (13))))))).bit_xor (((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[34]) + ((((((((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[26]).bit_shift_left (25))))).bit_xor (((((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[26]).bit_shift_left (14))))))).bit_xor (((((l_W[26]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[25]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 2_820_302_411) + l_W[41]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[42] := ((((((((((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[40]).bit_shift_left (15))))).bit_xor (((((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[40]).bit_shift_left (13))))))).bit_xor (((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[35]) + ((((((((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[27]).bit_shift_left (25))))).bit_xor (((((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[27]).bit_shift_left (14))))))).bit_xor (((((l_W[27]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[26]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 3_259_730_800) + l_W[42]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[43] := ((((((((((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[41]).bit_shift_left (15))))).bit_xor (((((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[41]).bit_shift_left (13))))))).bit_xor (((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[36]) + ((((((((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[28]).bit_shift_left (25))))).bit_xor (((((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[28]).bit_shift_left (14))))))).bit_xor (((((l_W[28]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[27]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 3_345_764_771) + l_W[43]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[44] := ((((((((((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[42]).bit_shift_left (15))))).bit_xor (((((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[42]).bit_shift_left (13))))))).bit_xor (((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[37]) + ((((((((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[29]).bit_shift_left (25))))).bit_xor (((((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[29]).bit_shift_left (14))))))).bit_xor (((((l_W[29]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[28]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 3_516_065_817) + l_W[44]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[45] := ((((((((((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[43]).bit_shift_left (15))))).bit_xor (((((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[43]).bit_shift_left (13))))))).bit_xor (((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[38]) + ((((((((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[30]).bit_shift_left (25))))).bit_xor (((((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[30]).bit_shift_left (14))))))).bit_xor (((((l_W[30]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[29]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 3_600_352_804) + l_W[45]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[46] := ((((((((((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[44]).bit_shift_left (15))))).bit_xor (((((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[44]).bit_shift_left (13))))))).bit_xor (((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[39]) + ((((((((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[31]).bit_shift_left (25))))).bit_xor (((((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[31]).bit_shift_left (14))))))).bit_xor (((((l_W[31]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[30]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 4_094_571_909) + l_W[46]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[47] := ((((((((((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[45]).bit_shift_left (15))))).bit_xor (((((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[45]).bit_shift_left (13))))))).bit_xor (((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[40]) + ((((((((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[32]).bit_shift_left (25))))).bit_xor (((((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[32]).bit_shift_left (14))))))).bit_xor (((((l_W[32]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[31]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 275_423_344) + l_W[47]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[48] := ((((((((((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[46]).bit_shift_left (15))))).bit_xor (((((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[46]).bit_shift_left (13))))))).bit_xor (((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[41]) + ((((((((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[33]).bit_shift_left (25))))).bit_xor (((((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[33]).bit_shift_left (14))))))).bit_xor (((((l_W[33]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[32]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 430_227_734) + l_W[48]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[49] := ((((((((((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[47]).bit_shift_left (15))))).bit_xor (((((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[47]).bit_shift_left (13))))))).bit_xor (((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[42]) + ((((((((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[34]).bit_shift_left (25))))).bit_xor (((((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[34]).bit_shift_left (14))))))).bit_xor (((((l_W[34]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[33]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 506_948_616) + l_W[49]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[50] := ((((((((((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[48]).bit_shift_left (15))))).bit_xor (((((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[48]).bit_shift_left (13))))))).bit_xor (((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[43]) + ((((((((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[35]).bit_shift_left (25))))).bit_xor (((((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[35]).bit_shift_left (14))))))).bit_xor (((((l_W[35]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[34]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 659_060_556) + l_W[50]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[51] := ((((((((((((l_W[49]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[49]).bit_shift_left (15))))).bit_xor (((((((l_W[49]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[49]).bit_shift_left (13))))))).bit_xor (((((l_W[49]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[44]) + ((((((((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[36]).bit_shift_left (25))))).bit_xor (((((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[36]).bit_shift_left (14))))))).bit_xor (((((l_W[36]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[35]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 883_997_877) + l_W[51]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[52] := ((((((((((((l_W[50]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[50]).bit_shift_left (15))))).bit_xor (((((((l_W[50]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[50]).bit_shift_left (13))))))).bit_xor (((((l_W[50]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[45]) + ((((((((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[37]).bit_shift_left (25))))).bit_xor (((((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[37]).bit_shift_left (14))))))).bit_xor (((((l_W[37]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[36]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 958_139_571) + l_W[52]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[53] := ((((((((((((l_W[51]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[51]).bit_shift_left (15))))).bit_xor (((((((l_W[51]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[51]).bit_shift_left (13))))))).bit_xor (((((l_W[51]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[46]) + ((((((((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[38]).bit_shift_left (25))))).bit_xor (((((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[38]).bit_shift_left (14))))))).bit_xor (((((l_W[38]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[37]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 1_322_822_218) + l_W[53]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[54] := ((((((((((((l_W[52]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[52]).bit_shift_left (15))))).bit_xor (((((((l_W[52]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[52]).bit_shift_left (13))))))).bit_xor (((((l_W[52]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[47]) + ((((((((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[39]).bit_shift_left (25))))).bit_xor (((((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[39]).bit_shift_left (14))))))).bit_xor (((((l_W[39]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[38]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 1_537_002_063) + l_W[54]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[55] := ((((((((((((l_W[53]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[53]).bit_shift_left (15))))).bit_xor (((((((l_W[53]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[53]).bit_shift_left (13))))))).bit_xor (((((l_W[53]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[48]) + ((((((((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[40]).bit_shift_left (25))))).bit_xor (((((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[40]).bit_shift_left (14))))))).bit_xor (((((l_W[40]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[39]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 1_747_873_779) + l_W[55]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			l_W[56] := ((((((((((((l_W[54]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[54]).bit_shift_left (15))))).bit_xor (((((((l_W[54]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[54]).bit_shift_left (13))))))).bit_xor (((((l_W[54]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[49]) + ((((((((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[41]).bit_shift_left (25))))).bit_xor (((((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[41]).bit_shift_left (14))))))).bit_xor (((((l_W[41]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[40]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_H + ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_E).bit_shift_left (26))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_E).bit_shift_left (21))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_E).bit_shift_left (7))))))) + ((l_G).bit_xor (((l_E).bit_and (((l_F).bit_xor (l_G))))))) + {NATURAL_32} 1_955_562_222) + l_W[56]
			l_temp2 := ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_A).bit_shift_left (30))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_A).bit_shift_left (19))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_A).bit_shift_left (10)))))) + ((((l_A).bit_and (l_B))).bit_or (((l_C).bit_and (((l_A).bit_or (l_B))))))
			l_D := l_D + l_temp1
			l_H := l_temp1 + l_temp2
			l_W[57] := ((((((((((((l_W[55]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[55]).bit_shift_left (15))))).bit_xor (((((((l_W[55]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[55]).bit_shift_left (13))))))).bit_xor (((((l_W[55]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[50]) + ((((((((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[42]).bit_shift_left (25))))).bit_xor (((((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[42]).bit_shift_left (14))))))).bit_xor (((((l_W[42]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[41]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_G + ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_D).bit_shift_left (26))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_D).bit_shift_left (21))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_D).bit_shift_left (7))))))) + ((l_F).bit_xor (((l_D).bit_and (((l_E).bit_xor (l_F))))))) + {NATURAL_32} 2_024_104_815) + l_W[57]
			l_temp2 := ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_H).bit_shift_left (30))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_H).bit_shift_left (19))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_H).bit_shift_left (10)))))) + ((((l_H).bit_and (l_A))).bit_or (((l_B).bit_and (((l_H).bit_or (l_A))))))
			l_C := l_C + l_temp1
			l_G := l_temp1 + l_temp2
			l_W[58] := ((((((((((((l_W[56]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[56]).bit_shift_left (15))))).bit_xor (((((((l_W[56]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[56]).bit_shift_left (13))))))).bit_xor (((((l_W[56]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[51]) + ((((((((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[43]).bit_shift_left (25))))).bit_xor (((((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[43]).bit_shift_left (14))))))).bit_xor (((((l_W[43]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[42]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_F + ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_C).bit_shift_left (26))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_C).bit_shift_left (21))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_C).bit_shift_left (7))))))) + ((l_E).bit_xor (((l_C).bit_and (((l_D).bit_xor (l_E))))))) + {NATURAL_32} 2_227_730_452) + l_W[58]
			l_temp2 := ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_G).bit_shift_left (30))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_G).bit_shift_left (19))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_G).bit_shift_left (10)))))) + ((((l_G).bit_and (l_H))).bit_or (((l_A).bit_and (((l_G).bit_or (l_H))))))
			l_B := l_B + l_temp1
			l_F := l_temp1 + l_temp2
			l_W[59] := ((((((((((((l_W[57]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[57]).bit_shift_left (15))))).bit_xor (((((((l_W[57]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[57]).bit_shift_left (13))))))).bit_xor (((((l_W[57]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[52]) + ((((((((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[44]).bit_shift_left (25))))).bit_xor (((((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[44]).bit_shift_left (14))))))).bit_xor (((((l_W[44]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[43]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_E + ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_B).bit_shift_left (26))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_B).bit_shift_left (21))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_B).bit_shift_left (7))))))) + ((l_D).bit_xor (((l_B).bit_and (((l_C).bit_xor (l_D))))))) + {NATURAL_32} 2_361_852_424) + l_W[59]
			l_temp2 := ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_F).bit_shift_left (30))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_F).bit_shift_left (19))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_F).bit_shift_left (10)))))) + ((((l_F).bit_and (l_G))).bit_or (((l_H).bit_and (((l_F).bit_or (l_G))))))
			l_A := l_A + l_temp1
			l_E := l_temp1 + l_temp2
			l_W[60] := ((((((((((((l_W[58]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[58]).bit_shift_left (15))))).bit_xor (((((((l_W[58]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[58]).bit_shift_left (13))))))).bit_xor (((((l_W[58]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[53]) + ((((((((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[45]).bit_shift_left (25))))).bit_xor (((((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[45]).bit_shift_left (14))))))).bit_xor (((((l_W[45]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[44]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_D + ((((((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_A).bit_shift_left (26))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_A).bit_shift_left (21))))))).bit_xor (((((((l_A).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_A).bit_shift_left (7))))))) + ((l_C).bit_xor (((l_A).bit_and (((l_B).bit_xor (l_C))))))) + {NATURAL_32} 2_428_436_474) + l_W[60]
			l_temp2 := ((((((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_E).bit_shift_left (30))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_E).bit_shift_left (19))))))).bit_xor (((((((l_E).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_E).bit_shift_left (10)))))) + ((((l_E).bit_and (l_F))).bit_or (((l_G).bit_and (((l_E).bit_or (l_F))))))
			l_H := l_H + l_temp1
			l_D := l_temp1 + l_temp2
			l_W[61] := ((((((((((((l_W[59]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[59]).bit_shift_left (15))))).bit_xor (((((((l_W[59]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[59]).bit_shift_left (13))))))).bit_xor (((((l_W[59]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[54]) + ((((((((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[46]).bit_shift_left (25))))).bit_xor (((((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[46]).bit_shift_left (14))))))).bit_xor (((((l_W[46]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[45]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_C + ((((((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_H).bit_shift_left (26))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_H).bit_shift_left (21))))))).bit_xor (((((((l_H).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_H).bit_shift_left (7))))))) + ((l_B).bit_xor (((l_H).bit_and (((l_A).bit_xor (l_B))))))) + {NATURAL_32} 2_756_734_187) + l_W[61]
			l_temp2 := ((((((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_D).bit_shift_left (30))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_D).bit_shift_left (19))))))).bit_xor (((((((l_D).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_D).bit_shift_left (10)))))) + ((((l_D).bit_and (l_E))).bit_or (((l_F).bit_and (((l_D).bit_or (l_E))))))
			l_G := l_G + l_temp1
			l_C := l_temp1 + l_temp2
			l_W[62] := ((((((((((((l_W[60]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[60]).bit_shift_left (15))))).bit_xor (((((((l_W[60]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[60]).bit_shift_left (13))))))).bit_xor (((((l_W[60]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[55]) + ((((((((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[47]).bit_shift_left (25))))).bit_xor (((((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[47]).bit_shift_left (14))))))).bit_xor (((((l_W[47]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[46]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_B + ((((((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_G).bit_shift_left (26))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_G).bit_shift_left (21))))))).bit_xor (((((((l_G).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_G).bit_shift_left (7))))))) + ((l_A).bit_xor (((l_G).bit_and (((l_H).bit_xor (l_A))))))) + {NATURAL_32} 3_204_031_479) + l_W[62]
			l_temp2 := ((((((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_C).bit_shift_left (30))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_C).bit_shift_left (19))))))).bit_xor (((((((l_C).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_C).bit_shift_left (10)))))) + ((((l_C).bit_and (l_D))).bit_or (((l_E).bit_and (((l_C).bit_or (l_D))))))
			l_F := l_F + l_temp1
			l_B := l_temp1 + l_temp2
			l_W[63] := ((((((((((((l_W[61]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (17))).bit_or (((l_W[61]).bit_shift_left (15))))).bit_xor (((((((l_W[61]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (19))).bit_or (((l_W[61]).bit_shift_left (13))))))).bit_xor (((((l_W[61]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (10)))) + l_W[56]) + ((((((((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (7))).bit_or (((l_W[48]).bit_shift_left (25))))).bit_xor (((((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (18))).bit_or (((l_W[48]).bit_shift_left (14))))))).bit_xor (((((l_W[48]).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (3))))) + l_W[47]
			update_class_fields -- Current might have been changed; synchronize
			l_temp1 := (((l_A + ((((((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (6))).bit_or (((l_F).bit_shift_left (26))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (11))).bit_or (((l_F).bit_shift_left (21))))))).bit_xor (((((((l_F).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (25))).bit_or (((l_F).bit_shift_left (7))))))) + ((l_H).bit_xor (((l_F).bit_and (((l_G).bit_xor (l_H))))))) + {NATURAL_32} 3_329_325_298) + l_W[63]
			l_temp2 := ((((((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (2))).bit_or (((l_B).bit_shift_left (30))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (13))).bit_or (((l_B).bit_shift_left (19))))))).bit_xor (((((((l_B).bit_and ({NATURAL_32} 4_294_967_295))).bit_shift_right (22))).bit_or (((l_B).bit_shift_left (10)))))) + ((((l_B).bit_and (l_C))).bit_or (((l_D).bit_and (((l_B).bit_or (l_C))))))
			l_E := l_E + l_temp1
			l_A := l_temp1 + l_temp2
			state[0] := state[0] + l_A
			update_class_fields -- Current might have been changed; synchronize
			state[1] := state[1] + l_B
			update_class_fields -- Current might have been changed; synchronize
			state[2] := state[2] + l_C
			update_class_fields -- Current might have been changed; synchronize
			state[3] := state[3] + l_D
			update_class_fields -- Current might have been changed; synchronize
			state[4] := state[4] + l_E
			update_class_fields -- Current might have been changed; synchronize
			state[5] := state[5] + l_F
			update_class_fields -- Current might have been changed; synchronize
			state[6] := state[6] + l_G
			update_class_fields -- Current might have been changed; synchronize
			state[7] := state[7] + l_H
			update_class_fields -- Current might have been changed; synchronize
		end

	sha256_finish (a_digest: CE_POINTER [NATURAL_8])
		require
			not a_digest.is_void
		local
			l_last: NATURAL_32
			l_padn: NATURAL_32
			l_high: NATURAL_32
			l_low: NATURAL_32
			l_msglen: CE_ARRAY [NATURAL_8]
		do
			create {CE_NATURAL_8_ARRAY} l_msglen.make (<<8>>)

			l_high := (((total[0]).bit_shift_right (29))).bit_or (((total[1]).bit_shift_left (3)))
			l_low := (total[0]).bit_shift_left (3)
			l_msglen[0] := ((l_high).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			l_msglen[1] := ((l_high).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			l_msglen[2] := ((l_high).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			l_msglen[3] := l_high.to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			l_msglen[4] := ((l_low).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			l_msglen[5] := ((l_low).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			l_msglen[6] := ((l_low).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			l_msglen[7] := l_low.to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			l_last := (total[0]).bit_and ({NATURAL_32} 63)
			if l_last < {NATURAL_32} 56 then
				l_padn := {NATURAL_32} 56 - l_last
			else
				l_padn := {NATURAL_32} 120 - l_last
			end
			(create {P_SHA256_STATIC}).sha256_update ((create {CE_CLASS_POINTER [PD_CONTEXT_SHA256_T]}.make_shared (c_pointer)), sha256_padding.item, l_padn)
			update_class_fields; -- Current might have been changed; synchronize
			(create {P_SHA256_STATIC}).sha256_update ((create {CE_CLASS_POINTER [PD_CONTEXT_SHA256_T]}.make_shared (c_pointer)), l_msglen, (8).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			a_digest[0] := ((state[0]).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[1] := ((state[0]).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[2] := ((state[0]).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[3] := (state[0]).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[4] := ((state[1]).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[5] := ((state[1]).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[6] := ((state[1]).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[7] := (state[1]).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[8] := ((state[2]).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[9] := ((state[2]).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[10] := ((state[2]).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[11] := (state[2]).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[12] := ((state[3]).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[13] := ((state[3]).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[14] := ((state[3]).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[15] := (state[3]).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[16] := ((state[4]).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[17] := ((state[4]).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[18] := ((state[4]).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[19] := (state[4]).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[20] := ((state[5]).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[21] := ((state[5]).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[22] := ((state[5]).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[23] := (state[5]).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[24] := ((state[6]).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[25] := ((state[6]).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[26] := ((state[6]).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[27] := (state[6]).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[28] := ((state[7]).bit_shift_right (24)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[29] := ((state[7]).bit_shift_right (16)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[30] := ((state[7]).bit_shift_right (8)).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
			a_digest[31] := (state[7]).to_natural_8
			update_class_fields -- Current might have been changed; synchronize
		end

feature {ANY}

	copy (a_other: like Current)
		do
			total := a_other.total.twin
			state := a_other.state.twin
			buffer := a_other.buffer.twin
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				total.is_equal (a_other.total) and
				state.is_equal (a_other.state) and
				buffer.is_equal (a_other.buffer)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("total", <<2>>),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("state", <<8>>),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("buffer", <<64>>)
				>>)
			init_structure_info
		end

end
