class
	PD_COOKIE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_COOKIE]} next.make
			create {CE_INTEGER_8_POINTER} name.make
			create {CE_INTEGER_8_POINTER} value.make
			create {CE_INTEGER_8_POINTER} path.make
			create {CE_INTEGER_8_POINTER} domain.make
			create {CE_INTEGER_8_POINTER} expirestr.make
			create {CE_INTEGER_8_POINTER} version.make
			create {CE_INTEGER_8_POINTER} maxage.make
		end

feature {ANY} -- fields

	next: CE_POINTER [PD_COOKIE] assign set_next

	name: CE_POINTER [INTEGER_8] assign set_name

	value: CE_POINTER [INTEGER_8] assign set_value

	path: CE_POINTER [INTEGER_8] assign set_path

	domain: CE_POINTER [INTEGER_8] assign set_domain

	expires: INTEGER_64 assign set_expires

	expirestr: CE_POINTER [INTEGER_8] assign set_expirestr

	tailmatch: BOOLEAN assign set_tailmatch

	version: CE_POINTER [INTEGER_8] assign set_version

	maxage: CE_POINTER [INTEGER_8] assign set_maxage

	secure: BOOLEAN assign set_secure

	livecookie: BOOLEAN assign set_livecookie

	httponly: BOOLEAN assign set_httponly

feature {ANY} -- field setters

	set_next (a_next: CE_POINTER [PD_COOKIE])
		do
			next := a_next
			update_memory_field ("next")
		end

	set_name (a_name: CE_POINTER [INTEGER_8])
		do
			name := a_name
			update_memory_field ("name")
		end

	set_value (a_value: CE_POINTER [INTEGER_8])
		do
			value := a_value
			update_memory_field ("value")
		end

	set_path (a_path: CE_POINTER [INTEGER_8])
		do
			path := a_path
			update_memory_field ("path")
		end

	set_domain (a_domain: CE_POINTER [INTEGER_8])
		do
			domain := a_domain
			update_memory_field ("domain")
		end

	set_expires (a_expires: INTEGER_64)
		do
			expires := a_expires
			update_memory_field ("expires")
		end

	set_expirestr (a_expirestr: CE_POINTER [INTEGER_8])
		do
			expirestr := a_expirestr
			update_memory_field ("expirestr")
		end

	set_tailmatch (a_tailmatch: BOOLEAN)
		do
			tailmatch := a_tailmatch
			update_memory_field ("tailmatch")
		end

	set_version (a_version: CE_POINTER [INTEGER_8])
		do
			version := a_version
			update_memory_field ("version")
		end

	set_maxage (a_maxage: CE_POINTER [INTEGER_8])
		do
			maxage := a_maxage
			update_memory_field ("maxage")
		end

	set_secure (a_secure: BOOLEAN)
		do
			secure := a_secure
			update_memory_field ("secure")
		end

	set_livecookie (a_livecookie: BOOLEAN)
		do
			livecookie := a_livecookie
			update_memory_field ("livecookie")
		end

	set_httponly (a_httponly: BOOLEAN)
		do
			httponly := a_httponly
			update_memory_field ("httponly")
		end

feature {ANY} -- public routines

	freecookie
		do
			if (not expirestr.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (expirestr)])
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not domain.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (domain)])
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not path.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (path)])
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not name.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (name)])
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not value.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (value)])
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not maxage.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (maxage)])
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not version.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (version)])
				update_class_fields -- Current might have been changed; synchronize
			end
			(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_shared (c_pointer)])
			update_class_fields -- Current might have been changed; synchronize
		end

feature {ANY}

	copy (a_other: like Current)
		do
			next := a_other.next
			name := a_other.name
			value := a_other.value
			path := a_other.path
			domain := a_other.domain
			expires := a_other.expires
			expirestr := a_other.expirestr
			tailmatch := a_other.tailmatch
			version := a_other.version
			maxage := a_other.maxage
			secure := a_other.secure
			livecookie := a_other.livecookie
			httponly := a_other.httponly
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				next.is_equal (a_other.next) and
				name.is_equal (a_other.name) and
				value.is_equal (a_other.value) and
				path.is_equal (a_other.path) and
				domain.is_equal (a_other.domain) and
				expires = a_other.expires and
				expirestr.is_equal (a_other.expirestr) and
				tailmatch = a_other.tailmatch and
				version.is_equal (a_other.version) and
				maxage.is_equal (a_other.maxage) and
				secure = a_other.secure and
				livecookie = a_other.livecookie and
				httponly = a_other.httponly
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("next"),
					create {CE_STRUCTURE_FIELD}.make ("name"),
					create {CE_STRUCTURE_FIELD}.make ("value"),
					create {CE_STRUCTURE_FIELD}.make ("path"),
					create {CE_STRUCTURE_FIELD}.make ("domain"),
					create {CE_STRUCTURE_FIELD}.make ("expires"),
					create {CE_STRUCTURE_FIELD}.make ("expirestr"),
					create {CE_STRUCTURE_FIELD}.make ("tailmatch"),
					create {CE_STRUCTURE_FIELD}.make ("version"),
					create {CE_STRUCTURE_FIELD}.make ("maxage"),
					create {CE_STRUCTURE_FIELD}.make ("secure"),
					create {CE_STRUCTURE_FIELD}.make ("livecookie"),
					create {CE_STRUCTURE_FIELD}.make ("httponly")
				>>)
			init_structure_info
		end

end
