class
	PD_CURL_MULTI

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create easy.make
			create {CE_CLASS_POINTER [PD_CURL_LLIST]} msglist.make
			create socket_cb.make
			create {CE_VOID_POINTER} socket_userp.make
			create {CE_CLASS_POINTER [PD_CURL_HASH]} hostcache.make
			create {CE_CLASS_POINTER [PD_CURL_TREE]} timetree.make
			create {CE_CLASS_POINTER [PD_CURL_HASH]} sockhash.make
			create {CE_CLASS_POINTER [PD_CONNCACHE]} connc.make
			create {CE_CLASS_POINTER [PD_CLOSURE]} closure.make
			create timer_cb.make
			create {CE_VOID_POINTER} timer_userp.make
			create timer_lastcall.make
			Precursor
		end

feature {ANY} -- fields

	type: INTEGER_32 assign set_type

	easy: PD_CURL_ONE_EASY assign set_easy

	num_easy: INTEGER_32 assign set_num_easy

	num_alive: INTEGER_32 assign set_num_alive

	msglist: CE_POINTER [PD_CURL_LLIST] assign set_msglist

	socket_cb: CE_ROUTINE assign set_socket_cb

	socket_userp: CE_POINTER [ANY] assign set_socket_userp

	hostcache: CE_POINTER [PD_CURL_HASH] assign set_hostcache

	timetree: CE_POINTER [PD_CURL_TREE] assign set_timetree

	sockhash: CE_POINTER [PD_CURL_HASH] assign set_sockhash

	pipelining_enabled: BOOLEAN assign set_pipelining_enabled

	connc: CE_POINTER [PD_CONNCACHE] assign set_connc

	maxconnects: INTEGER_32 assign set_maxconnects

	closure: CE_POINTER [PD_CLOSURE] assign set_closure

	timer_cb: CE_ROUTINE assign set_timer_cb

	timer_userp: CE_POINTER [ANY] assign set_timer_userp

	timer_lastcall: SD_TIMEVAL assign set_timer_lastcall

feature {ANY} -- field setters

	set_type (a_type: INTEGER_32)
		do
			type := a_type
			update_memory_field ("type")
		end

	set_easy (a_easy: PD_CURL_ONE_EASY)
		do
			easy := a_easy
			update_memory_field ("easy")
		end

	set_num_easy (a_num_easy: INTEGER_32)
		do
			num_easy := a_num_easy
			update_memory_field ("num_easy")
		end

	set_num_alive (a_num_alive: INTEGER_32)
		do
			num_alive := a_num_alive
			update_memory_field ("num_alive")
		end

	set_msglist (a_msglist: CE_POINTER [PD_CURL_LLIST])
		do
			msglist := a_msglist
			update_memory_field ("msglist")
		end

	set_socket_cb (a_socket_cb: CE_ROUTINE)
		do
			socket_cb := a_socket_cb
			update_memory_field ("socket_cb")
		end

	set_socket_userp (a_socket_userp: CE_POINTER [ANY])
		do
			socket_userp := a_socket_userp
			update_memory_field ("socket_userp")
		end

	set_hostcache (a_hostcache: CE_POINTER [PD_CURL_HASH])
		do
			hostcache := a_hostcache
			update_memory_field ("hostcache")
		end

	set_timetree (a_timetree: CE_POINTER [PD_CURL_TREE])
		do
			timetree := a_timetree
			update_memory_field ("timetree")
		end

	set_sockhash (a_sockhash: CE_POINTER [PD_CURL_HASH])
		do
			sockhash := a_sockhash
			update_memory_field ("sockhash")
		end

	set_pipelining_enabled (a_pipelining_enabled: BOOLEAN)
		do
			pipelining_enabled := a_pipelining_enabled
			update_memory_field ("pipelining_enabled")
		end

	set_connc (a_connc: CE_POINTER [PD_CONNCACHE])
		do
			connc := a_connc
			update_memory_field ("connc")
		end

	set_maxconnects (a_maxconnects: INTEGER_32)
		do
			maxconnects := a_maxconnects
			update_memory_field ("maxconnects")
		end

	set_closure (a_closure: CE_POINTER [PD_CLOSURE])
		do
			closure := a_closure
			update_memory_field ("closure")
		end

	set_timer_cb (a_timer_cb: CE_ROUTINE)
		do
			timer_cb := a_timer_cb
			update_memory_field ("timer_cb")
		end

	set_timer_userp (a_timer_userp: CE_POINTER [ANY])
		do
			timer_userp := a_timer_userp
			update_memory_field ("timer_userp")
		end

	set_timer_lastcall (a_timer_lastcall: SD_TIMEVAL)
		do
			timer_lastcall := a_timer_lastcall
			update_memory_field ("timer_lastcall")
		end

feature {PD_CURL_MULTI} -- private attributes

	tv_zero: CE_STATIC [SD_TIMEVAL]
		once ("PROCESS")
			Result.item := create {SD_TIMEVAL}.make_initialized ([(0).to_integer_32, (0).to_integer_32])
		end

	none: CE_STATIC [SD_TIMEVAL]
		once ("PROCESS")
			Result.item := create {SD_TIMEVAL}.make_initialized ([(0).to_integer_32, (0).to_integer_32])
		end

feature {ANY} -- public routines

	multi_addmsg (a_msg: CE_POINTER [PD_CURL_MESSAGE]): INTEGER_32
		local
			l_tmp: INTEGER_32
			l_return: BOOLEAN
		do
			l_tmp := (create {P_LLIST_STATIC}).Curl_llist_insert_next (msglist, msglist.item.tail, create {CE_VOID_POINTER}.make_cast (a_msg))
			update_class_fields -- Current might have been changed; synchronize
			if not l_tmp.to_boolean then
				Result := ({PC_CURLMCODE}.CURLM_OUT_OF_MEMORY)
				l_return := True
			end
			if not l_return then
				Result := ({PC_CURLMCODE}.CURLM_OK)
			end
		end

	Curl_multi_canPipeline: BOOLEAN
		do
			Result := pipelining_enabled
		end

	multi_socket (a_checkall: BOOLEAN; a_s, a_ev_bitmask: INTEGER_32; a_running_handles: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			l_s: CE_ARRAY [INTEGER_32]
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_t: CE_ARRAY [CE_POINTER [PD_CURL_TREE]]
			l_now: SD_TIMEVAL
			l_tmp: SD_TIMEVAL
			l_easyp: CE_POINTER [PD_CURL_ONE_EASY]
			l_entry: CE_POINTER [PD_CURL_SH_ENTRY]
			l_tmp___0: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_32_ARRAY} l_s.make (<<1>>)
			create {CE_POINTER_ARRAY [PD_CURL_TREE]} l_t.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_s.item := a_s
					update_class_fields -- Current might have been changed; synchronize
					l_result := {PC_CURLMCODE}.CURLM_OK
					l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_shared (default_pointer)
					l_tmp := ((create {P_TIMEVAL_STATIC}).curlx_tvnow).twin
					update_class_fields -- Current might have been changed; synchronize
					l_now := l_tmp.twin
					if not a_checkall then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_result := curl_multi_perform (create {CE_VOID_POINTER}.make_shared (c_pointer), a_running_handles)
						update_class_fields -- Current might have been changed; synchronize
						l_easyp := easy.next
						from
						until
							l_easyp.is_equal (create {CE_CLASS_POINTER [PD_CURL_ONE_EASY]}.make_shared (c_pointer + (create {PD_CURL_MULTI}.make).offset_of ("easy")))
						loop
							(create {P_MULTI_STATIC}).singlesocket ((create {CE_CLASS_POINTER [PD_CURL_MULTI]}.make_shared (c_pointer)), l_easyp)
							update_class_fields -- Current might have been changed; synchronize
							l_easyp := l_easyp.item.next
						end
						Result := l_result
						l_return := True
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 3 then -- else:
						if l_s.item = -1 then
							l_pos := -1 + 12 -- skip if block
						end
				when 4 then -- if:
						l_tmp___0 := sockhash.item.Curl_hash_pick (create {CE_VOID_POINTER}.make_cast (l_s), {PLATFORM}.Integer_32_bytes.to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_entry := create {CE_CLASS_POINTER [PD_CURL_SH_ENTRY]}.make_cast (l_tmp___0)
						if (l_entry.is_void) then
							l_pos := -1 + 12 -- skip if block
						end
				when 5 then -- if:
						l_data := l_entry.item.easy
						if l_data.item.magic /= {NATURAL_32} 3_235_830_701 then
							Result := ({PC_CURLMCODE}.CURLM_INTERNAL_ERROR)
							l_return := True
						end
						if l_return then
							l_pos := -1 + 12 -- skip if block
						end
				when 6 then -- if:
						if ((l_data.item.set.one_easy).item.easy_conn).is_void then
							l_pos := -1 + 11 -- skip if block
						end
				when 7 then -- if:
						if not (((a_ev_bitmask).bit_and (2)).to_boolean) then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if (not (((l_data.item.set.one_easy).item.easy_conn).item.send_pipe).is_void) then
							if (not ((((l_data.item.set.one_easy).item.easy_conn).item.send_pipe).item.head).is_void) then
								l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (((((l_data.item.set.one_easy).item.easy_conn).item.send_pipe).item.head).item.ptr)
							else
								l_pos := -1 + 10 -- goto _L___0
							end
						else
							l_pos := -1 + 10 -- goto _L___0
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L___0:
						if ((a_ev_bitmask).bit_and (1)).to_boolean then
							if (not (((l_data.item.set.one_easy).item.easy_conn).item.recv_pipe).is_void) then
								if (not ((((l_data.item.set.one_easy).item.easy_conn).item.recv_pipe).item.head).is_void) then
									l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (((((l_data.item.set.one_easy).item.easy_conn).item.recv_pipe).item.head).item.ptr)
								end
							end
						end
				when 11 then -- if done
						if (not ((l_data.item.set.one_easy).item.easy_conn).is_void) then
							((l_data.item.set.one_easy).item.easy_conn).item.cselect_bits := a_ev_bitmask
							update_class_fields -- Current might have been changed; synchronize
						end
						from
						until
							(l_break)
						loop
							l_break := False
							l_result := multi_runsingle ((create {CE_CLASS_POINTER [PD_CURL_MULTI]}.make_shared (c_pointer)), l_now.twin, l_data.item.set.one_easy)
							update_class_fields -- Current might have been changed; synchronize
							if -1 /= l_result then
								l_break := True
							end
						end
						l_break := False
						if (not ((l_data.item.set.one_easy).item.easy_conn).is_void) then
							((l_data.item.set.one_easy).item.easy_conn).item.cselect_bits := 0
							update_class_fields -- Current might have been changed; synchronize
						end
						if 0 >= l_result then
							(create {P_MULTI_STATIC}).singlesocket ((create {CE_CLASS_POINTER [PD_CURL_MULTI]}.make_shared (c_pointer)), l_data.item.set.one_easy)
							update_class_fields -- Current might have been changed; synchronize
						end
						l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_shared (default_pointer)
				when 12 then -- if done | if done | if done | if done
					if not l_return then
						l_now.tv_usec := l_now.tv_usec + (40_000)
						update_class_fields -- Current might have been changed; synchronize
						if l_now.tv_usec > 1_000_000 then
							l_now.tv_sec := (l_now.tv_sec) + 1
							update_class_fields -- Current might have been changed; synchronize
							l_now.tv_usec := l_now.tv_usec - (1_000_000)
							update_class_fields -- Current might have been changed; synchronize
						end
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if (not l_data.is_void) then
								from
								until
									(l_break)
								loop
									l_break := False
									l_result := multi_runsingle ((create {CE_CLASS_POINTER [PD_CURL_MULTI]}.make_shared (c_pointer)), l_now.twin, l_data.item.set.one_easy)
									update_class_fields -- Current might have been changed; synchronize
									if -1 /= l_result then
										l_break := True
									end
								end
								l_break := False
								if 0 >= l_result then
									(create {P_MULTI_STATIC}).singlesocket ((create {CE_CLASS_POINTER [PD_CURL_MULTI]}.make_shared (c_pointer)), l_data.item.set.one_easy)
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							Current.timetree := Curl_splaygetbest (l_now.twin, timetree, l_t)
							update_class_fields -- Current might have been changed; synchronize
							if (not (l_t.item).is_void) then
								l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (l_t.item.item.payload)
								add_next_timeout (l_now.twin, (create {CE_CLASS_POINTER [PD_CURL_MULTI]}.make_shared (c_pointer)), create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (l_t.item.item.payload)).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							end
							if (l_t.item).is_void then
								l_break := True
							end
						end
						l_break := False
						a_running_handles.item := num_alive
						update_class_fields -- Current might have been changed; synchronize
						Result := l_result
					end
				when 13 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	multi_timeout (a_timeout_ms: CE_POINTER [INTEGER_32]): INTEGER_32
		require
			not a_timeout_ms.is_void
		local
			l_now: SD_TIMEVAL
			l_tmp: SD_TIMEVAL
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
		do
			if (not timetree.is_void) then
				l_tmp := ((create {P_TIMEVAL_STATIC}).curlx_tvnow).twin
				update_class_fields -- Current might have been changed; synchronize
				l_now := l_tmp.twin
				Current.timetree := (create {P_SPLAY_STATIC}).Curl_splay (tv_zero.item.twin, timetree)
				update_class_fields -- Current might have been changed; synchronize
				if timetree.item.key.tv_sec < l_now.tv_sec then
					l_tmp___3 := -1
				else
					if timetree.item.key.tv_sec > l_now.tv_sec then
						l_tmp___2 := 1
					else
						if timetree.item.key.tv_usec < l_now.tv_usec then
							l_tmp___1 := -1
						else
							if timetree.item.key.tv_usec > l_now.tv_usec then
								l_tmp___0 := 1
							else
								l_tmp___0 := 0
							end
							l_tmp___1 := l_tmp___0
						end
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3 > 0 then
					a_timeout_ms.item := (create {P_TIMEVAL_STATIC}).curlx_tvdiff ((timetree.item.key).twin, l_now.twin)
					update_class_fields -- Current might have been changed; synchronize
					if not (a_timeout_ms.item).to_boolean then
						a_timeout_ms.item := 1
						update_class_fields -- Current might have been changed; synchronize
					end
				else
					a_timeout_ms.item := 0
					update_class_fields -- Current might have been changed; synchronize
				end
			else
				a_timeout_ms.item := -1
				update_class_fields -- Current might have been changed; synchronize
			end
			Result := ({PC_CURLMCODE}.CURLM_OK)
		end

	update_timer: INTEGER_32
		local
			l_timeout_ms: CE_ARRAY [INTEGER_32]
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_tmp___5: INTEGER_32
			l_tmp___6: INTEGER_32
			l_tmp___7: INTEGER_32
			l_tmp___8: INTEGER_32
			l_tmp___9: INTEGER_32
			l_return: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_timeout_ms.make (<<1>>)

			if timer_cb.is_void then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_tmp := ((create {CE_CLASS_POINTER [PD_CURL_MULTI]}.make_shared (c_pointer))).item.multi_timeout (l_timeout_ms)
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp.to_boolean then
					Result := (-1)
					l_return := True
				end
			end
			if not l_return and then (l_timeout_ms.item < 0) then
				if none.item.tv_sec < timer_lastcall.tv_sec then
					l_tmp___4 := -1
				else
					if none.item.tv_sec > timer_lastcall.tv_sec then
						l_tmp___3 := 1
					else
						if none.item.tv_usec < timer_lastcall.tv_usec then
							l_tmp___2 := -1
						else
							if none.item.tv_usec > timer_lastcall.tv_usec then
								l_tmp___1 := 1
							else
								l_tmp___1 := 0
							end
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					l_tmp___4 := l_tmp___3
				end
				if l_tmp___4.to_boolean then
					Current.timer_lastcall := none.item.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := timer_cb.integer_32_item ([create {CE_VOID_POINTER}.make_shared (c_pointer), -1, timer_userp])
					update_class_fields -- Current might have been changed; synchronize
					Result := l_tmp___0
					l_return := True
				end
				if not l_return then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				if timetree.item.key.tv_sec < timer_lastcall.tv_sec then
					l_tmp___8 := -1
				else
					if timetree.item.key.tv_sec > timer_lastcall.tv_sec then
						l_tmp___7 := 1
					else
						if timetree.item.key.tv_usec < timer_lastcall.tv_usec then
							l_tmp___6 := -1
						else
							if timetree.item.key.tv_usec > timer_lastcall.tv_usec then
								l_tmp___5 := 1
							else
								l_tmp___5 := 0
							end
							l_tmp___6 := l_tmp___5
						end
						l_tmp___7 := l_tmp___6
					end
					l_tmp___8 := l_tmp___7
				end
				if l_tmp___8 = 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Current.timer_lastcall := (timetree.item.key).twin
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___9 := timer_cb.integer_32_item ([create {CE_VOID_POINTER}.make_shared (c_pointer), l_timeout_ms.item, timer_userp])
				update_class_fields -- Current might have been changed; synchronize
				Result := l_tmp___9
			end
		end

	multi_connc_remove_handle (a_data: CE_POINTER [PD_SESSIONHANDLE])
		local
			l_i: INTEGER_32
			l_conn: CE_POINTER [PD_CONNECTDATA]
			l_tmp: INTEGER_32
		do
			l_i := 0
			from
			until
				not (l_i.to_integer_32 < connc.item.num)
			loop
				l_conn := connc.item.connects[l_i]
				if (not l_conn.is_void) then
					if (l_conn.item.data).is_equal (a_data) then
						if ((l_conn.item.protocol).bit_and (((((((1).bit_shift_left (2))).bit_or (((1).bit_shift_left (12))))).bit_or (((1).bit_shift_left (14)))).to_integer_32)).to_boolean then
							l_tmp := ((create {CE_CLASS_POINTER [PD_CURL_MULTI]}.make_shared (c_pointer))).item.add_closure (a_data)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp = 0 then
								a_data.item.state.shared_conn := create {CE_VOID_POINTER}.make_shared (c_pointer)
								update_class_fields -- Current might have been changed; synchronize
							else
								(create {P_URL_STATIC}).Curl_disconnect (l_conn).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								connc.item.connects[l_i] := create {CE_CLASS_POINTER [PD_CONNECTDATA]}.make_shared (default_pointer)
								update_class_fields -- Current might have been changed; synchronize
							end
						else
							l_conn.item.data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_shared (default_pointer)
							update_class_fields -- Current might have been changed; synchronize
						end
					end
				end
				l_i := l_i + 1
			end
		end

	add_closure (a_data: CE_POINTER [PD_SESSIONHANDLE]): INTEGER_32
		local
			l_cl: CE_POINTER [PD_CLOSURE]
			l_p: CE_POINTER [PD_CLOSURE]
			l_add: BOOLEAN
			l_n: CE_POINTER [PD_CLOSURE]
			l_inuse: BOOLEAN
			l_i: INTEGER_32
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			l_cl := closure
			l_p := create {CE_CLASS_POINTER [PD_CLOSURE]}.make_shared (default_pointer)
			l_add := (1).to_boolean
			from
			until
				not (not (l_return) and then (not l_cl.is_void))
			loop
				l_inuse := (0).to_boolean
				l_i := 0
				from
				until
					not (not (l_break) and then (l_i.to_integer_32 < connc.item.num))
				loop
					l_break := False
					if (not (connc.item.connects[l_i]).is_void) then
						if ((connc.item.connects[l_i]).item.data).is_equal (l_cl.item.easy_handle) then
							l_inuse := (1).to_boolean
							l_break := True
						end
					end
					if not (l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
				l_n := l_cl.item.next
				if not l_inuse then
					(l_cl.item.easy_handle).item.state.shared_conn := create {CE_VOID_POINTER}.make_shared (default_pointer)
					update_class_fields -- Current might have been changed; synchronize
					if (l_cl.item.easy_handle).item.state.closed then
						(create {P_SENDF_STATIC}).Curl_infof ([a_data, ce_string ("Delayed kill of easy handle %%p%N"), l_cl.item.easy_handle])
						update_class_fields; -- Current might have been changed; synchronize
						(l_cl.item.easy_handle).item.Curl_close.do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					if (not l_p.is_void) then
						l_p.item.next := l_n
						update_class_fields -- Current might have been changed; synchronize
					else
						Current.closure := l_n
						update_class_fields -- Current might have been changed; synchronize
					end
					(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_cl)])
					update_class_fields -- Current might have been changed; synchronize
				else
					if (l_cl.item.easy_handle).is_equal (a_data) then
						l_add := (0).to_boolean
					end
					l_p := l_cl
				end
				l_cl := l_n
			end
			if l_add then
				l_tmp := create {CE_VOID_POINTER}.make_cast ((create {P_EASY_STATIC}).Curl_ccalloc.item.pointer_item ([(1).to_natural_32, (create {PD_CLOSURE}.make).structure_size.to_natural_32]))
				update_class_fields -- Current might have been changed; synchronize
				l_cl := create {CE_CLASS_POINTER [PD_CLOSURE]}.make_cast (l_tmp)
				if l_cl.is_void then
					Result := ({PC_CURLMCODE}.CURLM_OUT_OF_MEMORY)
					l_return := True
				end
				if not l_return then
					l_cl.item.easy_handle := a_data
					update_class_fields -- Current might have been changed; synchronize
					l_cl.item.next := closure
					update_class_fields -- Current might have been changed; synchronize
					Current.closure := l_cl
					update_class_fields -- Current might have been changed; synchronize
				end
			end
			if not l_return then
				Result := ({PC_CURLMCODE}.CURLM_OK)
			end
		end

feature {PD_CURL_MULTI} -- private routines

	curl_multi_perform (a_multi_handle: CE_POINTER [ANY]; a_running_handles: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			l_multi: CE_POINTER [PD_CURL_MULTI]
			l_easy: CE_POINTER [PD_CURL_ONE_EASY]
			l_returncode: INTEGER_32
			l_t: CE_ARRAY [CE_POINTER [PD_CURL_TREE]]
			l_now: SD_TIMEVAL
			l_result: INTEGER_32
			l_wc: CE_POINTER [PD_WILDCARDDATA]
			l_ret: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_POINTER_ARRAY [PD_CURL_TREE]} l_t.make (<<1>>)

			l_multi := create {CE_CLASS_POINTER [PD_CURL_MULTI]}.make_cast (a_multi_handle)
			l_returncode := {PC_CURLMCODE}.CURLM_OK
			l_now := (((create {P_TIMEVAL_STATIC}).curlx_tvnow).twin).twin
			if (not l_multi.is_void) then
				if l_multi.item.type /= 764_702 then
					Result := ({PC_CURLMCODE}.CURLM_BAD_HANDLE)
					l_return := True
				end
			else
				Result := ({PC_CURLMCODE}.CURLM_BAD_HANDLE)
				l_return := True
			end
			if not l_return then
				l_easy := l_multi.item.easy.next
				from
				until
					not (not (l_return) and then (not l_easy.is_equal (create {CE_CLASS_POINTER [PD_CURL_ONE_EASY]}.make_shared (l_multi.c_pointer + (create {PD_CURL_MULTI}.make).offset_of ("easy")))))
				loop
					l_wc := create {CE_CLASS_POINTER [PD_WILDCARDDATA]}.make_shared ((l_easy.item.easy_handle).c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("wildcard"))
					if (l_easy.item.easy_handle).item.set.wildcardmatch then
						if (l_wc.item.filelist).is_void then
							l_ret := (l_wc.item.Curl_wildcard_init)
							if l_ret.to_boolean then
								Result := ({PC_CURLMCODE}.CURLM_OUT_OF_MEMORY)
								l_return := True
							end
						end
					end
					if not l_return then
						from
						until
							(l_break)
						loop
							l_break := False
							l_result := multi_runsingle (l_multi, l_now.twin, l_easy)
							if -1 /= l_result then
								l_break := True
							end
						end
						l_break := False
						if (l_easy.item.easy_handle).item.set.wildcardmatch then
							if l_wc.item.state = {NATURAL_32} 6 then
								(create {P_WILDCARD_STATIC}).Curl_wildcard_dtor (l_wc)
							elseif l_result.to_boolean then
								(create {P_WILDCARD_STATIC}).Curl_wildcard_dtor (l_wc)
							end
						end
						if l_result.to_boolean then
							l_returncode := l_result
						end
						l_easy := l_easy.item.next
					end
				end
			end
			if not l_return then
				from
				until
					(l_break)
				loop
					l_break := False
					l_multi.item.timetree := Curl_splaygetbest (l_now.twin, l_multi.item.timetree, l_t)
					if (not (l_t.item).is_void) then
						add_next_timeout (l_now.twin, l_multi, create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (l_t.item.item.payload)).do_nothing
					end
					if (l_t.item).is_void then
						l_break := True
					end
				end
				l_break := False
				a_running_handles.item := l_multi.item.num_alive
				if 0 >= l_returncode then
					l_multi.item.update_timer.do_nothing
				end
				Result := l_returncode
			end
		end

	multi_runsingle (a_multi: CE_POINTER [PD_CURL_MULTI]; a_now: SD_TIMEVAL; a_easy: CE_POINTER [PD_CURL_ONE_EASY]): INTEGER_32
		require
			not a_easy.is_void
		local
			l_now: CE_ARRAY [SD_TIMEVAL]
			l_msg: CE_POINTER [PD_CURL_MESSAGE]
			l_connected: CE_ARRAY [BOOLEAN]
			l_async: CE_ARRAY [BOOLEAN]
			l_protocol_connect: CE_ARRAY [BOOLEAN]
			l_dophase_done: CE_ARRAY [BOOLEAN]
			l_done: CE_ARRAY [BOOLEAN]
			l_result: INTEGER_32
			l_k: CE_POINTER [PD_SINGLEREQUEST]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_timeout_ms: INTEGER_32
			l_disconnect_conn: BOOLEAN
			l_tmp___2: INTEGER_32
			l_dns: CE_ARRAY [CE_POINTER [PD_CURL_DNS_ENTRY]]
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_tmp___5: INTEGER_32
			l_wc: CE_POINTER [PD_WILDCARDDATA]
			l_newurl: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_follow: NATURAL_32
			l_drc: NATURAL_32
			l_retry: BOOLEAN
			l_tmp___7: INTEGER_32
			l_buffersize: INTEGER_32
			l_tmp___9: INTEGER_32
			l_buffersize___0: INTEGER_32
			l_tmp___10: INTEGER_32
			l_newurl___0: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_retry___0: BOOLEAN
			l_follow___0: NATURAL_32
			l_tmp___11: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [SD_TIMEVAL]} l_now.make (<<1>>)
			create {CE_BOOLEAN_ARRAY} l_connected.make (<<1>>)
			create {CE_BOOLEAN_ARRAY} l_async.make (<<1>>)
			create {CE_BOOLEAN_ARRAY} l_protocol_connect.make (<<1>>)
			create {CE_BOOLEAN_ARRAY} l_dophase_done.make (<<1>>)
			create {CE_BOOLEAN_ARRAY} l_done.make (<<1>>)
			create {CE_CLASS_POINTER [PD_SESSIONHANDLE]} l_data.make
			create {CE_POINTER_ARRAY [PD_CURL_DNS_ENTRY]} l_dns.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_newurl.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_newurl___0.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_now.item := a_now.twin
					l_msg := create {CE_CLASS_POINTER [PD_CURL_MESSAGE]}.make_shared (default_pointer)
					l_protocol_connect.item := (0).to_boolean
					l_done.item := (0).to_boolean
					l_result := {PC_CURLMCODE}.CURLM_OK
					if (a_easy.item.easy_handle).item.magic /= {NATURAL_32} 3_235_830_701 then
						Result := ({PC_CURLMCODE}.CURLM_BAD_EASY_HANDLE)
						l_return := True
					end
					if l_return then
						l_pos := -1 + 76 -- skip if block
					end
				when 1 then -- if:
						l_data := a_easy.item.easy_handle
				when 2 then -- loop condition:
						if (l_return or l_break) then
							l_pos := -1 + 76 -- skip loop
						end
				when 3 then -- loop body:
						l_break := False
						l_disconnect_conn := (0).to_boolean
						if l_data.item.state.pipe_broke then
							(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("Pipe broke: handle 0x%%p, url = %%s%N"), a_easy, l_data.item.state.path])
							if a_easy.item.state < {NATURAL_32} 15 then
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_CONNECT)
								l_result := -1
								a_easy.item.result2 := {PC_CURLCODE}.CURLE_OK
							end
							l_data.item.state.pipe_broke := (0).to_boolean
							a_easy.item.easy_conn := create {CE_CLASS_POINTER [PD_CONNECTDATA]}.make_shared (default_pointer)
							l_break := True
						end
						if not (l_return or l_break) then
							if (not (a_easy.item.easy_conn).is_void) then
								if a_easy.item.state > {NATURAL_32} 1 then
									if a_easy.item.state < {NATURAL_32} 15 then
										(a_easy.item.easy_conn).item.data := l_data
									end
								end
							end
							if (not (a_easy.item.easy_conn).is_void) then
								if a_easy.item.state >= {NATURAL_32} 1 then
									l_timeout_ms := (a_easy.item.easy_conn).item.Curl_timeleft (l_now, (a_easy.item.state <= {NATURAL_32} 6))
									if l_timeout_ms < 0 then
										if a_easy.item.state = {NATURAL_32} 2 then
											l_data.item.Curl_failf ([ce_string ("Resolving timed out after %%ld milliseconds"), ((create {P_TIMEVAL_STATIC}).curlx_tvdiff ((l_now.item).twin, (l_data.item.progress.t_startsingle).twin))])
										elseif a_easy.item.state = {NATURAL_32} 3 then
											l_data.item.Curl_failf ([ce_string ("Connection timed out after %%ld milliseconds"), ((create {P_TIMEVAL_STATIC}).curlx_tvdiff ((l_now.item).twin, (l_data.item.progress.t_startsingle).twin))])
										else
											l_k := create {CE_CLASS_POINTER [PD_SINGLEREQUEST]}.make_shared (l_data.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("req"))
											l_data.item.Curl_failf ([ce_string ("Operation timed out after %%ld milliseconds with %%lld out of %%lld bytes received"), ((create {P_TIMEVAL_STATIC}).curlx_tvdiff ((l_now.item).twin, (l_data.item.progress.t_startsingle).twin)), l_k.item.bytecount, l_k.item.size])
										end
										a_easy.item.result2 := {PC_CURLCODE}.CURLE_OPERATION_TIMEDOUT
										a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_COMPLETED)
										l_break := True
									end
								end
							end
						end
						if (l_return or l_break) then
							l_pos := -1 + 75 -- skip if block
						end
				when 4 then -- if:
						inspect (a_easy.item.state).to_integer_32 -- switch 1 head
						when 0 then l_pos := -1 + 5
						when 1 then l_pos := -1 + 6
						when 2 then l_pos := -1 + 7
						when 4 then l_pos := -1 + 9
						when 3 then l_pos := -1 + 10
						when 5 then l_pos := -1 + 12
						when 6 then l_pos := -1 + 17
						when 7 then l_pos := -1 + 18
						when 8 then l_pos := -1 + 45
						when 9 then l_pos := -1 + 46
						when 10 then l_pos := -1 + 47
						when 11 then l_pos := -1 + 48
						when 13 then l_pos := -1 + 49
						when 12 then l_pos := -1 + 55
						when 14 then l_pos := -1 + 69
						when 15 then l_pos := -1 + 71
						when 16 then l_pos := -1 + 72
						else l_pos := -1 + 73
						end
				when 5 then -- switch 1, case 0
						a_easy.item.result2 := l_data.item.Curl_pretransfer
						if {NATURAL_32} 0 = a_easy.item.result2 then
							a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_CONNECT)
							l_result := -1
							l_data.item.state.used_interface := {PC_USED_INTERFACE}.Curl_if_multi
						end
						l_pos := -1 + 74 -- switch 1; break
				when 6 then -- switch 1, case 1;
						(create {P_PROGRESS_STATIC}).Curl_pgrsTime (l_data, {PC_TIMERID}.TIMER_STARTSINGLE)
						a_easy.item.result2 := l_data.item.Curl_connect (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), l_async, l_protocol_connect)
						if {NATURAL_32} 0 = a_easy.item.result2 then
							a_easy.item.result2 := l_data.item.addHandleToSendOrPendPipeline (a_easy.item.easy_conn)
							if {NATURAL_32} 0 = a_easy.item.result2 then
								if l_async.item then
									a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_WAITRESOLVE)
								else
									l_result := -1
									if l_protocol_connect.item then
										if a_multi.item.pipelining_enabled then
											l_tmp___2 := 6
										else
											l_tmp___2 := 7
										end
										a_easy.item.multistate (l_tmp___2.to_natural_32)
									elseif (a_easy.item.easy_conn).item.bits.tunnel_connecting then
										a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_WAITPROXYCONNECT)
									else
										a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_WAITCONNECT)
									end
								end
							end
						end
						l_pos := -1 + 74 -- switch 1; break
				when 7 then -- switch 1, case 2
						l_dns.item := create {CE_CLASS_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer)
						a_easy.item.result2 := (create {P_HOSTSYN_STATIC}).Curl_is_resolved (a_easy.item.easy_conn, l_dns)
						if (not (l_dns.item).is_void) then
							(create {P_MULTI_STATIC}).singlesocket (a_multi, a_easy)
							a_easy.item.result2 := (create {P_URL_STATIC}).Curl_async_resolved (a_easy.item.easy_conn, l_protocol_connect)
							if {NATURAL_32} 0 /= a_easy.item.result2 then
								a_easy.item.easy_conn := create {CE_CLASS_POINTER [PD_CONNECTDATA]}.make_shared (default_pointer)
							else
								l_result := -1
								if l_protocol_connect.item then
									if a_multi.item.pipelining_enabled then
										l_tmp___3 := 6
									else
										l_tmp___3 := 7
									end
									a_easy.item.multistate (l_tmp___3.to_natural_32)
								elseif (a_easy.item.easy_conn).item.bits.tunnel_connecting then
									a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_WAITPROXYCONNECT)
								else
									a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_WAITCONNECT)
								end
							end
						end
						if {NATURAL_32} 0 /= a_easy.item.result2 then
							l_disconnect_conn := (1).to_boolean
							l_pos := -1 + 74 -- switch 1; break
						end
				when 8 then
						l_pos := -1 + 74 -- switch 1; break
				when 9 then -- switch 1, case 4
						a_easy.item.result2 := (a_easy.item.easy_conn).item.Curl_http_connect (l_protocol_connect)
						if (a_easy.item.easy_conn).item.bits.proxy_connect_closed then
							if (not (l_data.item.set.errorbuffer).is_void) then
								l_data.item.set.errorbuffer[0] := (('%/0/').code).to_integer_8
							end
							l_data.item.state.errorbuf := (0).to_boolean
							a_easy.item.result2 := {PC_CURLCODE}.CURLE_OK
							l_result := -1
							a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_CONNECT)
						elseif {NATURAL_32} 0 = a_easy.item.result2 then
							if not (a_easy.item.easy_conn).item.bits.tunnel_connecting then
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_WAITCONNECT)
							end
						end
						l_pos := -1 + 74 -- switch 1; break
				when 10 then -- switch 1, case 3
						a_easy.item.result2 := (a_easy.item.easy_conn).item.Curl_is_connected (0, l_connected)
						if l_connected.item then
							a_easy.item.result2 := (a_easy.item.easy_conn).item.Curl_connected_proxy
							if not (a_easy.item.result2).to_boolean then
								a_easy.item.result2 := (a_easy.item.easy_conn).item.Curl_protocol_connect (l_protocol_connect)
							end
						end
						if {NATURAL_32} 0 /= a_easy.item.result2 then
							l_disconnect_conn := (1).to_boolean
							l_pos := -1 + 74 -- switch 1; break
						end
				when 11 then
						if l_connected.item then
							if not l_protocol_connect.item then
								if (a_easy.item.easy_conn).item.bits.tunnel_connecting then
									a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_WAITPROXYCONNECT)
								else
									a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_PROTOCONNECT)
								end
							else
								if a_multi.item.pipelining_enabled then
									l_tmp___4 := 6
								else
									l_tmp___4 := 7
								end
								a_easy.item.multistate (l_tmp___4.to_natural_32)
							end
							l_result := -1
						end
						l_pos := -1 + 74 -- switch 1; break
				when 12 then -- switch 1, case 5
						a_easy.item.result2 := Curl_protocol_connecting (a_easy.item.easy_conn, l_protocol_connect)
						if a_easy.item.result2 /= {NATURAL_32} 0 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if l_protocol_connect.item then
							if a_multi.item.pipelining_enabled then
								l_tmp___5 := 6
							else
								l_tmp___5 := 7
							end
							a_easy.item.multistate (l_tmp___5.to_natural_32)
							l_result := -1
						else
							l_pos := -1 + 15 -- goto _L
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 15 then -- else: | _L:
						if (a_easy.item.result2).to_boolean then
							l_data.item.Curl_posttransfer.do_nothing;
							(create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), a_easy.item.result2, (0).to_boolean).do_nothing
							l_disconnect_conn := (1).to_boolean
						end
				when 16 then
						l_pos := -1 + 74 -- switch 1; break
				when 17 then -- switch 1, case 6
						if not (a_easy.item.easy_conn).item.writechannel_inuse then
							if ((create {P_MULTI_STATIC}).isHandleAtHead (l_data, (a_easy.item.easy_conn).item.send_pipe)) then
								(a_easy.item.easy_conn).item.writechannel_inuse := (1).to_boolean
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DO)
								l_result := -1
							end
						end
						l_pos := -1 + 74 -- switch 1; break
				when 18 then -- switch 1, case 7
						if not (l_data.item.set.connect_only) then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if:;
						(a_easy.item.easy_conn).item.bits.close := (0).to_boolean
						a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DONE)
						a_easy.item.result2 := {PC_CURLCODE}.CURLE_OK
						l_result := {PC_CURLMCODE}.CURLM_OK
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 44
				when 21 then -- else:
						a_easy.item.result2 := (create {P_URL_STATIC}).Curl_do (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), l_dophase_done)
						if {NATURAL_32} 0 /= a_easy.item.result2 then
							l_pos := -1 + 28 -- skip if block
						end
				when 22 then -- if:
						if l_dophase_done.item then
							l_pos := -1 + 26 -- skip if block
						end
				when 23 then -- if:
						if l_data.item.set.wildcardmatch then
							l_wc := create {CE_CLASS_POINTER [PD_WILDCARDDATA]}.make_shared (l_data.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("wildcard"))
							if l_wc.item.state = {NATURAL_32} 6 then
								(create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), {PC_CURLCODE}.CURLE_OK, (0).to_boolean).do_nothing
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DONE)
								l_result := -1
								l_pos := -1 + 74 -- switch 1; break
							elseif l_wc.item.state = {NATURAL_32} 4 then
								(create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), {PC_CURLCODE}.CURLE_OK, (0).to_boolean).do_nothing
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DONE)
								l_result := -1
								l_pos := -1 + 74 -- switch 1; break
							end
						end
				when 24 then
						a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DOING)
						l_result := {PC_CURLMCODE}.CURLM_OK
				when 25 then -- if block done, skip else block:
						l_pos := -1 + 27
				when 26 then -- else:
						if (a_easy.item.easy_conn).item.bits.do_more then
							a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DO_MORE)
							l_result := {PC_CURLMCODE}.CURLM_OK
						else
							a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DO_DONE)
							l_result := -1
						end
				when 27 then -- if done | if block done, skip else block:
						l_pos := -1 + 44
				when 28 then -- else:
						if {NATURAL_32} 55 /= a_easy.item.result2 then
							l_pos := -1 + 43 -- skip if block
						end
				when 29 then -- if:
						if not ((a_easy.item.easy_conn).item.bits.reuse) then
							l_pos := -1 + 41 -- skip if block
						end
				when 30 then -- if:
						l_newurl.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
						l_follow := {PC_FOLLOWTYPE}.FOLLOW_NONE
						l_retry := (0).to_boolean
						l_drc := (a_easy.item.easy_conn).item.Curl_retry_request (l_newurl)
						if l_drc.to_boolean then
							a_easy.item.result2 := l_drc
							l_disconnect_conn := (1).to_boolean
						else
							if (not (l_newurl.item).is_void) then
								l_tmp___7 := 1
							else
								l_tmp___7 := 0
							end
							l_retry := l_tmp___7.to_boolean
						end
						l_data.item.Curl_posttransfer.do_nothing
						l_drc := (create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), a_easy.item.result2, (0).to_boolean)
						if not l_retry then
							l_pos := -1 + 39 -- skip if block
						end
				when 31 then -- if:
						if l_drc /= {NATURAL_32} 0 then
							l_pos := -1 + 34 -- skip if block
						end
				when 32 then -- if:
						l_pos := -1 + 35 -- goto _L___0
				when 33 then -- if block done, skip else block:
						l_pos := -1 + 38
				when 34 then -- else:
						if l_drc /= {NATURAL_32} 55 then
							l_pos := -1 + 37 -- skip if block
						end
				when 35 then -- if: | _L___0:
						l_follow := {PC_FOLLOWTYPE}.FOLLOW_RETRY
						l_drc := (create {P_TRANSFER_STATIC}).Curl_follow (l_data, l_newurl.item, l_follow)
						if l_drc = {NATURAL_32} 0 then
							a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_CONNECT)
							l_result := -1
							a_easy.item.result2 := {PC_CURLCODE}.CURLE_OK
						else
							a_easy.item.result2 := l_drc;
							(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_newurl.item)])
						end
				when 36 then -- if block done, skip else block:
						l_pos := -1 + 38
				when 37 then -- else:
						a_easy.item.result2 := l_drc;
						(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_newurl.item)])
				when 38 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 40
				when 39 then -- else:
						l_disconnect_conn := (1).to_boolean
				when 40 then -- if done | if block done, skip else block:
						l_pos := -1 + 42
				when 41 then -- else:
						l_data.item.Curl_posttransfer.do_nothing;
						(create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), a_easy.item.result2, (0).to_boolean).do_nothing
						l_disconnect_conn := (1).to_boolean
				when 42 then -- if done | if block done, skip else block:
						l_pos := -1 + 44
				when 43 then -- else:
						l_data.item.Curl_posttransfer.do_nothing;
						(create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), a_easy.item.result2, (0).to_boolean).do_nothing
						l_disconnect_conn := (1).to_boolean
				when 44 then -- if done | if done
						l_pos := -1 + 74 -- switch 1; break
				when 45 then -- switch 1, case 8
						a_easy.item.result2 := Curl_protocol_doing (a_easy.item.easy_conn, l_dophase_done)
						if {NATURAL_32} 0 = a_easy.item.result2 then
							if l_dophase_done.item then
								if (a_easy.item.easy_conn).item.bits.do_more then
									a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DO_MORE)
									l_result := {PC_CURLMCODE}.CURLM_OK
								else
									a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DO_DONE)
									l_result := -1
								end
							end
						else
							l_data.item.Curl_posttransfer.do_nothing;
							(create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), a_easy.item.result2, (0).to_boolean).do_nothing
							l_disconnect_conn := (1).to_boolean
						end
						l_pos := -1 + 74 -- switch 1; break
				when 46 then -- switch 1, case 9
						a_easy.item.result2 := (a_easy.item.easy_conn).item.Curl_is_connected (1, l_connected)
						if l_connected.item then
							a_easy.item.result2 := Curl_do_more (a_easy.item.easy_conn)
							if {NATURAL_32} 0 = a_easy.item.result2 then
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DO_DONE)
								l_result := -1
							else
								l_data.item.Curl_posttransfer.do_nothing;
								(create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), a_easy.item.result2, (0).to_boolean).do_nothing
								l_disconnect_conn := (1).to_boolean
							end
						end
						l_pos := -1 + 74 -- switch 1; break
				when 47 then -- switch 1, case 10
						moveHandleFromSendToRecvPipeline (l_data, a_easy.item.easy_conn)
						checkPendPipeline (a_easy.item.easy_conn).do_nothing
						a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_WAITPERFORM)
						l_result := -1
						l_pos := -1 + 74 -- switch 1; break
				when 48 then -- switch 1, case 11
						if not (a_easy.item.easy_conn).item.readchannel_inuse then
							if ((create {P_MULTI_STATIC}).isHandleAtHead (l_data, (a_easy.item.easy_conn).item.recv_pipe)) then
								(a_easy.item.easy_conn).item.readchannel_inuse := (1).to_boolean
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_PERFORM)
								l_result := -1
							end
						end
						l_pos := -1 + 74 -- switch 1; break
				when 49 then -- switch 1, case 13
						if l_data.item.set.max_send_speed /= {INTEGER_64} 0 then
							l_pos := -1 + 52 -- skip if block
						end
				when 50 then -- if:
						l_pos := -1 + 53 -- goto _L___1
				when 51 then -- if block done, skip else block:
						l_pos := -1 + 54
				when 52 then -- else:
						if not (l_data.item.progress.ulspeed < l_data.item.set.max_send_speed) then
							l_pos := -1 + 54 -- skip if block
						end
				when 53 then -- if: | _L___1:
						if l_data.item.set.max_recv_speed = {INTEGER_64} 0 then
							a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_PERFORM)
						elseif l_data.item.progress.dlspeed < l_data.item.set.max_recv_speed then
							a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_PERFORM)
						end
				when 54 then -- if done
						l_pos := -1 + 74 -- switch 1; break
				when 55 then -- switch 1, case 12
						if l_data.item.set.max_send_speed > {INTEGER_64} 0 then
							if l_data.item.progress.ulspeed > l_data.item.set.max_send_speed then
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_TOOFAST)
								if (l_data.item.set.buffer_size).to_boolean then
									l_tmp___9 := l_data.item.set.buffer_size
								else
									l_tmp___9 := 16_384
								end
								l_buffersize := l_tmp___9.to_integer_32
								l_timeout_ms := (create {P_TRANSFER_STATIC}).Curl_sleep_time (l_data.item.set.max_send_speed, l_data.item.progress.ulspeed, l_buffersize)
								l_data.item.Curl_expire (l_timeout_ms)
								l_pos := -1 + 74 -- switch 1; break
							end
						end
				when 56 then
						if l_data.item.set.max_recv_speed > {INTEGER_64} 0 then
							if l_data.item.progress.dlspeed > l_data.item.set.max_recv_speed then
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_TOOFAST)
								if (l_data.item.set.buffer_size).to_boolean then
									l_tmp___10 := l_data.item.set.buffer_size
								else
									l_tmp___10 := 16_384
								end
								l_buffersize___0 := l_tmp___10.to_integer_32
								l_timeout_ms := (create {P_TRANSFER_STATIC}).Curl_sleep_time (l_data.item.set.max_recv_speed, l_data.item.progress.dlspeed, l_buffersize___0)
								l_data.item.Curl_expire (l_timeout_ms)
								l_pos := -1 + 74 -- switch 1; break
							end
						end
				when 57 then
						a_easy.item.result2 := (a_easy.item.easy_conn).item.Curl_readwrite (l_done)
						l_k := create {CE_CLASS_POINTER [PD_SINGLEREQUEST]}.make_shared (l_data.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("req"))
						if not ((l_k.item.keepon).bit_and (1)).to_boolean then
							(a_easy.item.easy_conn).item.readchannel_inuse := (0).to_boolean
						end
						if not ((l_k.item.keepon).bit_and (((1).bit_shift_left (1)))).to_boolean then
							(a_easy.item.easy_conn).item.writechannel_inuse := (0).to_boolean
						end
						if not ((a_easy.item.result2).to_boolean) then
							l_pos := -1 + 60 -- skip if block
						end
				when 58 then -- if:
						if not (((a_easy.item.easy_conn).item.protocol).bit_and (((1).bit_shift_left (2)).to_integer_32)).to_boolean then
							(a_easy.item.easy_conn).item.bits.close := (1).to_boolean
						end
						l_data.item.Curl_posttransfer.do_nothing;
						(create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), a_easy.item.result2, (0).to_boolean).do_nothing
				when 59 then -- if block done, skip else block:
						l_pos := -1 + 68
				when 60 then -- else:
						if 1 /= (l_done.item).to_integer then
							l_pos := -1 + 68 -- skip if block
						end
				when 61 then -- if:
						l_newurl___0.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
						l_retry___0 := (0).to_boolean
						l_follow___0 := {PC_FOLLOWTYPE}.FOLLOW_NONE
						a_easy.item.result2 := (a_easy.item.easy_conn).item.Curl_retry_request (l_newurl___0)
						if not (a_easy.item.result2).to_boolean then
							if (not (l_newurl___0.item).is_void) then
								l_tmp___11 := 1
							else
								l_tmp___11 := 0
							end
							l_retry___0 := l_tmp___11.to_boolean
						end
						l_data.item.Curl_posttransfer.do_nothing
						moveHandleFromRecvToDonePipeline (l_data, a_easy.item.easy_conn)
						if (not (((a_easy.item.easy_conn).item.recv_pipe).item.head).is_void) then
							(create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast ((((a_easy.item.easy_conn).item.recv_pipe).item.head).item.ptr)).item.Curl_expire (1)
						end
						checkPendPipeline (a_easy.item.easy_conn).do_nothing
						if (l_data.item.req.newurl).is_void then
							l_pos := -1 + 64 -- skip if block
						end
				when 62 then -- if:
						l_pos := -1 + 65 -- goto _L___2
				when 63 then -- if block done, skip else block:
						l_pos := -1 + 68
				when 64 then -- else:
						if not l_retry___0 then
							l_pos := -1 + 67 -- skip if block
						end
				when 65 then -- if: | _L___2:
						if not l_retry___0 then
							l_newurl___0.item := l_data.item.req.newurl
							l_data.item.req.newurl := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
							l_follow___0 := {PC_FOLLOWTYPE}.FOLLOW_REDIR
						else
							l_follow___0 := {PC_FOLLOWTYPE}.FOLLOW_RETRY
						end
						a_easy.item.result2 := (create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), {PC_CURLCODE}.CURLE_OK, (0).to_boolean)
						if a_easy.item.result2 = {NATURAL_32} 0 then
							a_easy.item.result2 := (create {P_TRANSFER_STATIC}).Curl_follow (l_data, l_newurl___0.item, l_follow___0)
						end
						if {NATURAL_32} 0 = a_easy.item.result2 then
							a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_CONNECT)
							l_result := -1
						elseif (not (l_newurl___0.item).is_void) then
							(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_newurl___0.item)])
						end
				when 66 then -- if block done, skip else block:
						l_pos := -1 + 68
				when 67 then -- else:
						if (not (l_data.item.req.location).is_void) then
							l_newurl___0.item := l_data.item.req.location
							l_data.item.req.location := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
							a_easy.item.result2 := (create {P_TRANSFER_STATIC}).Curl_follow (l_data, l_newurl___0.item, {PC_FOLLOWTYPE}.FOLLOW_FAKE)
							if (a_easy.item.result2).to_boolean then
								(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_newurl___0.item)])
							end
						end
						a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_DONE)
						l_result := -1
				when 68 then -- if done | if done | if done
						l_pos := -1 + 74 -- switch 1; break
				when 69 then -- switch 1, case 14
						if (not (a_easy.item.easy_conn).is_void) then
							(create {P_URL_STATIC}).Curl_removeHandleFromPipeline (l_data, (a_easy.item.easy_conn).item.recv_pipe).do_nothing;
							(create {P_URL_STATIC}).Curl_removeHandleFromPipeline (l_data, (a_easy.item.easy_conn).item.done_pipe).do_nothing
							checkPendPipeline (a_easy.item.easy_conn).do_nothing
							if (a_easy.item.easy_conn).item.bits.stream_was_rewound then
								l_result := -1
							end
							a_easy.item.result2 := (create {P_URL_STATIC}).Curl_done (create {CE_POINTER_POINTER [PD_CONNECTDATA]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("easy_conn")), {PC_CURLCODE}.CURLE_OK, (0).to_boolean)
							if (not (a_easy.item.easy_conn).is_void) then
								a_easy.item.easy_conn := create {CE_CLASS_POINTER [PD_CONNECTDATA]}.make_shared (default_pointer)
							end
						end
						if l_data.item.set.wildcardmatch then
							if l_data.item.wildcard.state /= {NATURAL_32} 6 then
								l_result := -1
								a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_INIT)
								l_pos := -1 + 74 -- switch 1; break
							end
						end
				when 70 then
						a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_COMPLETED)
						l_pos := -1 + 74 -- switch 1; break
				when 71 then -- switch 1, case 15
						a_easy.item.easy_conn := create {CE_CLASS_POINTER [PD_CONNECTDATA]}.make_shared (default_pointer)
						l_data.item.Curl_expire (0)
						l_pos := -1 + 74 -- switch 1; break
				when 72 then -- switch 1, case 16
						Result := ({PC_CURLMCODE}.CURLM_OK)
						l_return := True
				when 73 then -- switch 1, default:
						if not l_return then
							Result := ({PC_CURLMCODE}.CURLM_INTERNAL_ERROR)
							l_return := True
						end
				when 74 then -- switch 1, done:
						if not l_return then
							if {NATURAL_32} 15 > a_easy.item.state then
								if {NATURAL_32} 0 /= a_easy.item.result2 then
									l_data.item.state.pipe_broke := (0).to_boolean
									if (not (a_easy.item.easy_conn).is_void) then
										(a_easy.item.easy_conn).item.writechannel_inuse := (0).to_boolean;
										(a_easy.item.easy_conn).item.readchannel_inuse := (0).to_boolean;
										(create {P_URL_STATIC}).Curl_removeHandleFromPipeline (l_data, (a_easy.item.easy_conn).item.send_pipe).do_nothing;
										(create {P_URL_STATIC}).Curl_removeHandleFromPipeline (l_data, (a_easy.item.easy_conn).item.recv_pipe).do_nothing;
										(create {P_URL_STATIC}).Curl_removeHandleFromPipeline (l_data, (a_easy.item.easy_conn).item.done_pipe).do_nothing
										checkPendPipeline (a_easy.item.easy_conn).do_nothing
									end
									if l_disconnect_conn then
										(create {P_URL_STATIC}).Curl_disconnect (a_easy.item.easy_conn).do_nothing
										a_easy.item.easy_conn := create {CE_CLASS_POINTER [PD_CONNECTDATA]}.make_shared (default_pointer)
									end
									a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_COMPLETED)
								elseif (not (a_easy.item.easy_conn).is_void) then
									if ((a_easy.item.easy_conn).item.Curl_pgrsUpdate).to_boolean then
										a_easy.item.result2 := {PC_CURLCODE}.CURLE_ABORTED_BY_CALLBACK
									end
								end
							end
							l_break := True
						end
				when 75 then -- if done
						l_pos := -1 + 2 -- back to loop condition
				when 76 then -- loop done | if done
					if not l_return then
						l_break := False
						if {NATURAL_32} 15 = a_easy.item.state then
							if l_data.item.dns.hostcachetype = {NATURAL_32} 3 then
								l_data.item.dns.hostcache := create {CE_CLASS_POINTER [PD_CURL_HASH]}.make_shared (default_pointer)
								l_data.item.dns.hostcachetype := {PC_HOSTCACHETYPE}.HCACHE_NONE
							end
							l_msg := create {CE_CLASS_POINTER [PD_CURL_MESSAGE]}.make_shared (a_easy.c_pointer + (create {PD_CURL_ONE_EASY}.make).offset_of ("msg"))
							l_msg.item.extmsg.msg := {PC_CURLMSG}.CURLMSG_DONE
							l_msg.item.extmsg.easy_handle := create {CE_VOID_POINTER}.make_cast (l_data)
							l_msg.item.extmsg.data.result2 := a_easy.item.result2
							l_result := a_multi.item.multi_addmsg (l_msg)
							a_easy.item.multistate ({PC_CURLMSTATE}.CURLM_STATE_MSGSENT)
						end
						Result := l_result
					end
				when 77 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	add_next_timeout (a_now: SD_TIMEVAL; a_multi: CE_POINTER [PD_CURL_MULTI]; a_d: CE_POINTER [PD_SESSIONHANDLE]): INTEGER_32
		require
			not a_d.is_void
		local
			l_tv: CE_POINTER [SD_TIMEVAL]
			l_list: CE_POINTER [PD_CURL_LLIST]
			l_e: CE_POINTER [PD_CURL_LLIST_ELEMENT]
			l_n: CE_POINTER [PD_CURL_LLIST_ELEMENT]
			l_diff: INTEGER_32
			l_break: BOOLEAN
		do
			l_tv := create {CE_CLASS_POINTER [SD_TIMEVAL]}.make_shared (a_d.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("state") + (create {PD_URLSTATE}.make).offset_of ("expiretime"))
			l_list := a_d.item.state.timeoutlist
			l_e := l_list.item.head
			from
			until
				not (not (l_break) and then (not l_e.is_void))
			loop
				l_break := False
				l_n := l_e.item.next
				l_diff := ((create {P_TIMEVAL_STATIC}).curlx_tvdiff (((create {CE_CLASS_POINTER [SD_TIMEVAL]}.make_cast (l_e.item.ptr)).item).twin, a_now.twin))
				if l_diff <= 0 then
					l_list.item.Curl_llist_remove (l_e, create {CE_VOID_POINTER}.make_shared (default_pointer)).do_nothing
				else
					l_break := True
				end
				if not (l_break) then
					l_e := l_n
				end
			end
			l_break := False
			if not (l_list.item.size).to_boolean then
				l_tv.item.tv_sec := (0).to_integer_32
				l_tv.item.tv_usec := (0).to_integer_32
			else
				l_e := l_list.item.head
				memcpy (create {CE_VOID_POINTER}.make_cast (l_tv), l_e.item.ptr, (create {SD_TIMEVAL}.make).structure_size.to_natural_32).do_nothing
				l_list.item.Curl_llist_remove (l_e, create {CE_VOID_POINTER}.make_shared (default_pointer)).do_nothing
				a_multi.item.timetree := (create {P_SPLAY_STATIC}).Curl_splayinsert ((l_tv.item).twin, a_multi.item.timetree, create {CE_CLASS_POINTER [PD_CURL_TREE]}.make_shared (a_d.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("state") + (create {PD_URLSTATE}.make).offset_of ("timenode")))
			end
			Result := ({PC_CURLMCODE}.CURLM_OK)
		end

	Curl_splaygetbest (a_i: SD_TIMEVAL; a_t: CE_POINTER [PD_CURL_TREE]; a_removed: CE_POINTER [CE_POINTER [PD_CURL_TREE]]): CE_POINTER [PD_CURL_TREE]
		require
			not a_removed.is_void
		local
			l_t: CE_POINTER [PD_CURL_TREE]
			l_x: CE_POINTER [PD_CURL_TREE]
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_tmp___5: INTEGER_32
			l_tmp___6: INTEGER_32
			l_return: BOOLEAN
		do
			l_t := a_t
			if l_t.is_void then
				a_removed.item := create {CE_CLASS_POINTER [PD_CURL_TREE]}.make_shared (default_pointer)
				Result := (create {CE_CLASS_POINTER [PD_CURL_TREE]}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				l_t := (create {P_SPLAY_STATIC}).Curl_splay (a_i.twin, l_t)
				if a_i.tv_sec < l_t.item.key.tv_sec then
					l_tmp___2 := -1
				else
					if a_i.tv_sec > l_t.item.key.tv_sec then
						l_tmp___1 := 1
					else
						if a_i.tv_usec < l_t.item.key.tv_usec then
							l_tmp___0 := -1
						else
							if a_i.tv_usec > l_t.item.key.tv_usec then
								l_tmp := 1
							else
								l_tmp := 0
							end
							l_tmp___0 := l_tmp
						end
						l_tmp___1 := l_tmp___0
					end
					l_tmp___2 := l_tmp___1
				end
				if l_tmp___2 < 0 then
					if (not (l_t.item.smaller).is_void) then
						l_t := (create {P_SPLAY_STATIC}).Curl_splay (((l_t.item.smaller).item.key).twin, l_t)
					else
						a_removed.item := create {CE_CLASS_POINTER [PD_CURL_TREE]}.make_shared (default_pointer)
						Result := l_t
						l_return := True
					end
				end
			end
			if not l_return then
				if a_i.tv_sec < l_t.item.key.tv_sec then
					l_tmp___6 := -1
				else
					if a_i.tv_sec > l_t.item.key.tv_sec then
						l_tmp___5 := 1
					else
						if a_i.tv_usec < l_t.item.key.tv_usec then
							l_tmp___4 := -1
						else
							if a_i.tv_usec > l_t.item.key.tv_usec then
								l_tmp___3 := 1
							else
								l_tmp___3 := 0
							end
							l_tmp___4 := l_tmp___3
						end
						l_tmp___5 := l_tmp___4
					end
					l_tmp___6 := l_tmp___5
				end
				if l_tmp___6 >= 0 then
					l_x := l_t.item.same
					if (not l_x.is_void) then
						l_x.item.key := (l_t.item.key).twin
						l_x.item.larger := l_t.item.larger
						l_x.item.smaller := l_t.item.smaller
						a_removed.item := l_t
						Result := l_x
						l_return := True
					end
					if not l_return then
						if (l_t.item.smaller).is_void then
							l_x := l_t.item.larger
						else
							l_x := (create {P_SPLAY_STATIC}).Curl_splay (a_i.twin, l_t.item.smaller)
							l_x.item.larger := l_t.item.larger
						end
						a_removed.item := l_t
						Result := l_x
					end
				else
					a_removed.item := create {CE_CLASS_POINTER [PD_CURL_TREE]}.make_shared (default_pointer)
					Result := l_t
				end
			end
		end

	Curl_protocol_connecting (a_conn: CE_POINTER [PD_CONNECTDATA]; a_done: CE_POINTER [BOOLEAN]): NATURAL_32
		require
			not a_done.is_void
		local
			l_result: NATURAL_32
		do
			l_result := {PC_CURLCODE}.CURLE_OK
			if (not a_conn.is_void) then
				if (not ((a_conn.item.handler).item.connecting).is_void) then
					a_done.item := (0).to_boolean
					l_result := ((a_conn.item.handler).item.connecting).natural_32_item ([a_conn, a_done])
				else
					a_done.item := (1).to_boolean
				end
			else
				a_done.item := (1).to_boolean
			end
			Result := l_result
		end

	Curl_protocol_doing (a_conn: CE_POINTER [PD_CONNECTDATA]; a_done: CE_POINTER [BOOLEAN]): NATURAL_32
		require
			not a_done.is_void
		local
			l_result: NATURAL_32
		do
			l_result := {PC_CURLCODE}.CURLE_OK
			if (not a_conn.is_void) then
				if (not ((a_conn.item.handler).item.doing).is_void) then
					a_done.item := (0).to_boolean
					l_result := ((a_conn.item.handler).item.doing).natural_32_item ([a_conn, a_done])
				else
					a_done.item := (1).to_boolean
				end
			else
				a_done.item := (1).to_boolean
			end
			Result := l_result
		end

	Curl_do_more (a_conn: CE_POINTER [PD_CONNECTDATA]): NATURAL_32
		require
			not a_conn.is_void
		local
			l_result: NATURAL_32
		do
			l_result := {PC_CURLCODE}.CURLE_OK
			if (not ((a_conn.item.handler).item.do_more).is_void) then
				l_result := ((a_conn.item.handler).item.do_more).natural_32_item ([a_conn])
			end
			if l_result = {NATURAL_32} 0 then
				a_conn.item.do_complete
			end
			Result := l_result
		end

	checkPendPipeline (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		require
			not a_conn.is_void
		local
			l_result: INTEGER_32
			l_sendhead: CE_POINTER [PD_CURL_LLIST_ELEMENT]
			l_pipeLen: NATURAL_32
			l_curr: CE_POINTER [PD_CURL_LLIST_ELEMENT]
			l_maxPipeLen: NATURAL_32
			l_tmp: INTEGER_32
			l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_result := 0
					l_sendhead := (a_conn.item.send_pipe).item.head
					l_pipeLen := (a_conn.item.send_pipe).item.size + (a_conn.item.recv_pipe).item.size
					if not (a_conn.item.server_supports_pipelining) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if l_pipeLen /= {NATURAL_32} 0 then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						l_curr := (a_conn.item.pend_pipe).item.head
						if a_conn.item.server_supports_pipelining then
							l_tmp := 5
						else
							l_tmp := 1
						end
						l_maxPipeLen := l_tmp.to_natural_32
						from
						until
							(l_break)
						loop
							l_break := False
							if l_pipeLen < l_maxPipeLen then
								if l_curr.is_void then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_break) then
								Curl_llist_move (a_conn.item.pend_pipe, l_curr, a_conn.item.send_pipe, (a_conn.item.send_pipe).item.tail).do_nothing;
								(create {P_PROGRESS_STATIC}).Curl_pgrsTime (create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (l_curr.item.ptr), {PC_TIMERID}.TIMER_PRETRANSFER)
								l_result := l_result + 1
								l_curr := (a_conn.item.pend_pipe).item.head
								l_pipeLen := l_pipeLen + 1
							end
						end
						l_break := False
				when 5 then -- if done
					if l_result.to_boolean then
						a_conn.item.now := ((create {P_TIMEVAL_STATIC}).curlx_tvnow).twin
						if not l_sendhead.is_equal ((a_conn.item.send_pipe).item.head) then
							a_conn.item.writechannel_inuse := (0).to_boolean;
							(create {P_SENDF_STATIC}).Curl_infof ([a_conn.item.data, ce_string ("%%p is at send pipe head!%N"), ((a_conn.item.send_pipe).item.head).item.ptr]);
							(create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (((a_conn.item.send_pipe).item.head).item.ptr)).item.Curl_expire (1)
						end
					end
					Result := l_result
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	moveHandleFromSendToRecvPipeline (a_handle: CE_POINTER [PD_SESSIONHANDLE]; a_conn: CE_POINTER [PD_CONNECTDATA])
		require
			not a_conn.is_void
		local
			l_curr: CE_POINTER [PD_CURL_LLIST_ELEMENT]
			l_break: BOOLEAN
		do
			l_curr := (a_conn.item.send_pipe).item.head
			from
			until
				not (not (l_break) and then (not l_curr.is_void))
			loop
				l_break := False
				if (l_curr.item.ptr).is_equal (a_handle) then
					Curl_llist_move (a_conn.item.send_pipe, l_curr, a_conn.item.recv_pipe, (a_conn.item.recv_pipe).item.tail).do_nothing
					if (not ((a_conn.item.send_pipe).item.head).is_void) then
						a_conn.item.writechannel_inuse := (0).to_boolean;
						(create {P_SENDF_STATIC}).Curl_infof ([a_conn.item.data, ce_string ("%%p is at send pipe head B!%N"), ((a_conn.item.send_pipe).item.head).item.ptr]);
						(create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (((a_conn.item.send_pipe).item.head).item.ptr)).item.Curl_expire (1)
					end
					l_break := True
				end
				if not (l_break) then
					l_curr := l_curr.item.next
				end
			end
			l_break := False
		end

	moveHandleFromRecvToDonePipeline (a_handle: CE_POINTER [PD_SESSIONHANDLE]; a_conn: CE_POINTER [PD_CONNECTDATA])
		require
			not a_conn.is_void
		local
			l_curr: CE_POINTER [PD_CURL_LLIST_ELEMENT]
			l_break: BOOLEAN
		do
			l_curr := (a_conn.item.recv_pipe).item.head
			from
			until
				not (not (l_break) and then (not l_curr.is_void))
			loop
				l_break := False
				if (l_curr.item.ptr).is_equal (a_handle) then
					Curl_llist_move (a_conn.item.recv_pipe, l_curr, a_conn.item.done_pipe, (a_conn.item.done_pipe).item.tail).do_nothing
					l_break := True
				end
				if not (l_break) then
					l_curr := l_curr.item.next
				end
			end
			l_break := False
		end

	Curl_llist_move (a_list: CE_POINTER [PD_CURL_LLIST]; a_e: CE_POINTER [PD_CURL_LLIST_ELEMENT]; a_to_list: CE_POINTER [PD_CURL_LLIST]; a_to_e: CE_POINTER [PD_CURL_LLIST_ELEMENT]): INTEGER_32
		local
			l_return: BOOLEAN
		do
			if a_e.is_void then
				Result := 0
				l_return := True
			elseif a_list.item.size = {NATURAL_32} 0 then
				Result := 0
				l_return := True
			end
			if not l_return then
				if a_e.is_equal (a_list.item.head) then
					a_list.item.head := a_e.item.next
					if (a_list.item.head).is_void then
						a_list.item.tail := create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]}.make_shared (default_pointer)
					else
						(a_e.item.next).item.prev := create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]}.make_shared (default_pointer)
					end
				else
					(a_e.item.prev).item.next := a_e.item.next
					if (a_e.item.next).is_void then
						a_list.item.tail := a_e.item.prev
					else
						(a_e.item.next).item.prev := a_e.item.prev
					end
				end
				a_list.item.size := (a_list.item.size) - 1
				if a_to_list.item.size = {NATURAL_32} 0 then
					a_to_list.item.head := a_e;
					(a_to_list.item.head).item.prev := create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]}.make_shared (default_pointer);
					(a_to_list.item.head).item.next := create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]}.make_shared (default_pointer)
					a_to_list.item.tail := a_e
				else
					a_e.item.next := a_to_e.item.next
					a_e.item.prev := a_to_e
					if (not (a_to_e.item.next).is_void) then
						(a_to_e.item.next).item.prev := a_e
					else
						a_to_list.item.tail := a_e
					end
					a_to_e.item.next := a_e
				end
				a_to_list.item.size := (a_to_list.item.size) + 1
				Result := 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			type := a_other.type
			easy := a_other.easy.twin
			num_easy := a_other.num_easy
			num_alive := a_other.num_alive
			msglist := a_other.msglist
			socket_cb := a_other.socket_cb
			socket_userp := a_other.socket_userp
			hostcache := a_other.hostcache
			timetree := a_other.timetree
			sockhash := a_other.sockhash
			pipelining_enabled := a_other.pipelining_enabled
			connc := a_other.connc
			maxconnects := a_other.maxconnects
			closure := a_other.closure
			timer_cb := a_other.timer_cb
			timer_userp := a_other.timer_userp
			timer_lastcall := a_other.timer_lastcall.twin
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				type = a_other.type and
				easy.is_equal (a_other.easy) and
				num_easy = a_other.num_easy and
				num_alive = a_other.num_alive and
				msglist.is_equal (a_other.msglist) and
				socket_cb.is_equal (a_other.socket_cb) and
				socket_userp.is_equal (a_other.socket_userp) and
				hostcache.is_equal (a_other.hostcache) and
				timetree.is_equal (a_other.timetree) and
				sockhash.is_equal (a_other.sockhash) and
				pipelining_enabled = a_other.pipelining_enabled and
				connc.is_equal (a_other.connc) and
				maxconnects = a_other.maxconnects and
				closure.is_equal (a_other.closure) and
				timer_cb.is_equal (a_other.timer_cb) and
				timer_userp.is_equal (a_other.timer_userp) and
				timer_lastcall.is_equal (a_other.timer_lastcall)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("type"),
					create {CE_STRUCTURE_FIELD}.make ("easy"),
					create {CE_STRUCTURE_FIELD}.make ("num_easy"),
					create {CE_STRUCTURE_FIELD}.make ("num_alive"),
					create {CE_STRUCTURE_FIELD}.make ("msglist"),
					create {CE_STRUCTURE_FIELD}.make ("socket_cb"),
					create {CE_STRUCTURE_FIELD}.make ("socket_userp"),
					create {CE_STRUCTURE_FIELD}.make ("hostcache"),
					create {CE_STRUCTURE_FIELD}.make ("timetree"),
					create {CE_STRUCTURE_FIELD}.make ("sockhash"),
					create {CE_STRUCTURE_FIELD}.make ("pipelining_enabled"),
					create {CE_STRUCTURE_FIELD}.make ("connc"),
					create {CE_STRUCTURE_FIELD}.make ("maxconnects"),
					create {CE_STRUCTURE_FIELD}.make ("closure"),
					create {CE_STRUCTURE_FIELD}.make ("timer_cb"),
					create {CE_STRUCTURE_FIELD}.make ("timer_userp"),
					create {CE_STRUCTURE_FIELD}.make ("timer_lastcall")
				>>)
			init_structure_info
		end

end
