class
	PD_DIGESTDATA

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_INTEGER_8_POINTER} nonce.make
			create {CE_INTEGER_8_POINTER} cnonce.make
			create {CE_INTEGER_8_POINTER} realm.make
			create {CE_INTEGER_8_POINTER} opaque.make
			create {CE_INTEGER_8_POINTER} qop.make
			create {CE_INTEGER_8_POINTER} algorithm.make
		end

feature {ANY} -- fields

	nonce: CE_POINTER [INTEGER_8] assign set_nonce

	cnonce: CE_POINTER [INTEGER_8] assign set_cnonce

	realm: CE_POINTER [INTEGER_8] assign set_realm

	algo: INTEGER_32 assign set_algo

	stale: BOOLEAN assign set_stale

	opaque: CE_POINTER [INTEGER_8] assign set_opaque

	qop: CE_POINTER [INTEGER_8] assign set_qop

	algorithm: CE_POINTER [INTEGER_8] assign set_algorithm

	nc: INTEGER_32 assign set_nc

feature {ANY} -- field setters

	set_nonce (a_nonce: CE_POINTER [INTEGER_8])
		do
			nonce := a_nonce
			update_memory_field ("nonce")
		end

	set_cnonce (a_cnonce: CE_POINTER [INTEGER_8])
		do
			cnonce := a_cnonce
			update_memory_field ("cnonce")
		end

	set_realm (a_realm: CE_POINTER [INTEGER_8])
		do
			realm := a_realm
			update_memory_field ("realm")
		end

	set_algo (a_algo: INTEGER_32)
		do
			algo := a_algo
			update_memory_field ("algo")
		end

	set_stale (a_stale: BOOLEAN)
		do
			stale := a_stale
			update_memory_field ("stale")
		end

	set_opaque (a_opaque: CE_POINTER [INTEGER_8])
		do
			opaque := a_opaque
			update_memory_field ("opaque")
		end

	set_qop (a_qop: CE_POINTER [INTEGER_8])
		do
			qop := a_qop
			update_memory_field ("qop")
		end

	set_algorithm (a_algorithm: CE_POINTER [INTEGER_8])
		do
			algorithm := a_algorithm
			update_memory_field ("algorithm")
		end

	set_nc (a_nc: INTEGER_32)
		do
			nc := a_nc
			update_memory_field ("nc")
		end

feature {ANY} -- public routines

	Curl_digest_cleanup_one
		do
			if (not nonce.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (nonce)])
				update_class_fields -- Current might have been changed; synchronize
			end
			Current.nonce := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			if (not cnonce.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (cnonce)])
				update_class_fields -- Current might have been changed; synchronize
			end
			Current.cnonce := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			if (not realm.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (realm)])
				update_class_fields -- Current might have been changed; synchronize
			end
			Current.realm := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			if (not opaque.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (opaque)])
				update_class_fields -- Current might have been changed; synchronize
			end
			Current.opaque := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			if (not qop.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (qop)])
				update_class_fields -- Current might have been changed; synchronize
			end
			Current.qop := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			if (not algorithm.is_void) then
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (algorithm)])
				update_class_fields -- Current might have been changed; synchronize
			end
			Current.algorithm := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			Current.nc := 0
			update_class_fields -- Current might have been changed; synchronize
			Current.algo := 0
			update_class_fields -- Current might have been changed; synchronize
			Current.stale := (0).to_boolean
			update_class_fields -- Current might have been changed; synchronize
		end

feature {ANY}

	copy (a_other: like Current)
		do
			nonce := a_other.nonce
			cnonce := a_other.cnonce
			realm := a_other.realm
			algo := a_other.algo
			stale := a_other.stale
			opaque := a_other.opaque
			qop := a_other.qop
			algorithm := a_other.algorithm
			nc := a_other.nc
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				nonce.is_equal (a_other.nonce) and
				cnonce.is_equal (a_other.cnonce) and
				realm.is_equal (a_other.realm) and
				algo = a_other.algo and
				stale = a_other.stale and
				opaque.is_equal (a_other.opaque) and
				qop.is_equal (a_other.qop) and
				algorithm.is_equal (a_other.algorithm) and
				nc = a_other.nc
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("nonce"),
					create {CE_STRUCTURE_FIELD}.make ("cnonce"),
					create {CE_STRUCTURE_FIELD}.make ("realm"),
					create {CE_STRUCTURE_FIELD}.make ("algo"),
					create {CE_STRUCTURE_FIELD}.make ("stale"),
					create {CE_STRUCTURE_FIELD}.make ("opaque"),
					create {CE_STRUCTURE_FIELD}.make ("qop"),
					create {CE_STRUCTURE_FIELD}.make ("algorithm"),
					create {CE_STRUCTURE_FIELD}.make ("nc")
				>>)
			init_structure_info
		end

end
