class
	P_C_RADIX2_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC

feature {ANY} -- public routines

	gsl_fft_complex_radix2_forward (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := -1
			l_status := (gsl_fft_complex_radix2_transform (a_data, a_stride, a_n, l_sign))
			Result := l_status
		end

	gsl_fft_complex_radix2_backward (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_fft_complex_radix2_transform (a_data, a_stride, a_n, l_sign))
			Result := l_status
		end

	gsl_fft_complex_radix2_inverse (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_norm___0: REAL_64
			l_i: NATURAL_32
			l_return: BOOLEAN
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_fft_complex_radix2_transform (a_data, a_stride, a_n, l_sign))
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_norm___0 := 1.0 / a_n.to_real_64
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] * l_norm___0
					a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] * l_norm___0
					l_i := l_i + 1
				end
				Result := l_status
			end
		end

	gsl_fft_complex_radix2_transform (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_sign: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_dual: NATURAL_32
			l_bit: NATURAL_32
			l_logn: NATURAL_32
			l_status: INTEGER_32
			l_w_real: REAL_64
			l_w_imag: REAL_64
			l_theta: REAL_64
			l_s: REAL_64
			l_t: REAL_64
			l_s2: REAL_64
			l_a___5: NATURAL_32
			l_b___0: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_z1_real: REAL_64
			l_z1_imag: REAL_64
			l_wd_real: REAL_64
			l_wd_imag: REAL_64
			l_tmp_real: REAL_64
			l_tmp_imag: REAL_64
			l_i___0: NATURAL_32
			l_j___0: NATURAL_32
			l_z1_real___0: REAL_64
			l_z1_imag___0: REAL_64
			l_wd_real___0: REAL_64
			l_wd_imag___0: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_logn := (0).to_natural_32
			if a_n = {NATURAL_32} 1 then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_result := (create {P_FACTORIZE_STATIC}).fft_binary_logn (a_n)
				if l_result = -1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("n is not a power of 2"), ce_string ("c_radix2.c"), 90, 4)
						Result := 4
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				else
					l_logn := l_result.to_natural_32
				end
			end
			if not l_return then
				l_status := (create {P_BITREVERSE_STATIC}).fft_complex_bitreverse_order (a_data, a_stride, a_n, l_logn)
				l_dual := (1).to_natural_32
				l_bit := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_bit < l_logn))
				loop
					l_w_real := 1.0
					l_w_imag := 0.0
					l_theta := ((2.0 * a_sign.to_double) * 3.14159265358979323846) / (2.0 * l_dual.to_real_64)
					l_s := (sin (l_theta))
					l_t := (sin (l_theta / 2.0))
					l_s2 := (2.0 * l_t) * l_t
					l_b___0 := (0).to_natural_32
					from
					until
						not (l_b___0 < a_n)
					loop
						l_i := l_b___0
						l_j := l_b___0 + l_dual
						l_z1_real := a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32]
						l_z1_imag := a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32]
						l_wd_real := l_z1_real
						l_wd_imag := l_z1_imag
						a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] - l_wd_real
						a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] - l_wd_imag
						a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] + l_wd_real
						a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] + l_wd_imag
						l_b___0 := l_b___0 + ({NATURAL_32} 2 * l_dual)
					end
					l_a___5 := (1).to_natural_32
					from
					until
						not (not (l_return) and then (l_a___5 < l_dual))
					loop
						l_tmp_real := (l_w_real - l_s * l_w_imag) - l_s2 * l_w_real
						l_tmp_imag := (l_w_imag + l_s * l_w_real) - l_s2 * l_w_imag
						l_w_real := l_tmp_real
						l_w_imag := l_tmp_imag
						l_b___0 := (0).to_natural_32
						from
						until
							not (l_b___0 < a_n)
						loop
							l_i___0 := l_b___0 + l_a___5
							l_j___0 := (l_b___0 + l_a___5) + l_dual
							l_z1_real___0 := a_data[(({NATURAL_32} 2 * a_stride) * l_j___0).to_integer_32]
							l_z1_imag___0 := a_data[(({NATURAL_32} 2 * a_stride) * l_j___0 + {NATURAL_32} 1).to_integer_32]
							l_wd_real___0 := l_w_real * l_z1_real___0 - l_w_imag * l_z1_imag___0
							l_wd_imag___0 := l_w_real * l_z1_imag___0 + l_w_imag * l_z1_real___0
							a_data[(({NATURAL_32} 2 * a_stride) * l_j___0).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i___0).to_integer_32] - l_wd_real___0
							a_data[(({NATURAL_32} 2 * a_stride) * l_j___0 + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i___0 + {NATURAL_32} 1).to_integer_32] - l_wd_imag___0
							a_data[(({NATURAL_32} 2 * a_stride) * l_i___0).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i___0).to_integer_32] + l_wd_real___0
							a_data[(({NATURAL_32} 2 * a_stride) * l_i___0 + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i___0 + {NATURAL_32} 1).to_integer_32] + l_wd_imag___0
							l_b___0 := l_b___0 + ({NATURAL_32} 2 * l_dual)
						end
						l_a___5 := l_a___5 + 1
					end
					l_dual := l_dual * ({NATURAL_32} 2)
					l_bit := l_bit + 1
				end
				Result := 0
			end
		end

	gsl_fft_complex_radix2_dif_forward (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := -1
			l_status := (gsl_fft_complex_radix2_dif_transform (a_data, a_stride, a_n, l_sign))
			Result := l_status
		end

	gsl_fft_complex_radix2_dif_backward (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_fft_complex_radix2_dif_transform (a_data, a_stride, a_n, l_sign))
			Result := l_status
		end

	gsl_fft_complex_radix2_dif_inverse (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_norm___0: REAL_64
			l_i: NATURAL_32
			l_return: BOOLEAN
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_fft_complex_radix2_dif_transform (a_data, a_stride, a_n, l_sign))
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_norm___0 := 1.0 / a_n.to_real_64
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] * l_norm___0
					a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] * l_norm___0
					l_i := l_i + 1
				end
				Result := l_status
			end
		end

	gsl_fft_complex_radix2_dif_transform (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_sign: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_dual: NATURAL_32
			l_bit: NATURAL_32
			l_logn: NATURAL_32
			l_status: INTEGER_32
			l_w_real: REAL_64
			l_w_imag: REAL_64
			l_theta: REAL_64
			l_s: REAL_64
			l_t: REAL_64
			l_s2: REAL_64
			l_a___5: NATURAL_32
			l_b___0: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_t1_real: REAL_64
			l_t1_imag: REAL_64
			l_t2_real: REAL_64
			l_t2_imag: REAL_64
			l_tmp_real: REAL_64
			l_tmp_imag: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_logn := (0).to_natural_32
			if a_n = {NATURAL_32} 1 then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_result := (create {P_FACTORIZE_STATIC}).fft_binary_logn (a_n)
				if l_result = -1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("n is not a power of 2"), ce_string ("c_radix2.c"), 247, 4)
						Result := 4
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				else
					l_logn := l_result.to_natural_32
				end
			end
			if not l_return then
				l_dual := a_n // {NATURAL_32} 2
				l_bit := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_bit < l_logn))
				loop
					l_w_real := 1.0
					l_w_imag := 0.0
					l_theta := ((2.0 * a_sign.to_double) * 3.14159265358979323846) / ({NATURAL_32} 2 * l_dual).to_real_64
					l_s := (sin (l_theta))
					l_t := (sin (l_theta / 2.0))
					l_s2 := (2.0 * l_t) * l_t
					l_b___0 := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_b___0 < l_dual))
					loop
						l_a___5 := (0).to_natural_32
						from
						until
							not (l_a___5 < a_n)
						loop
							l_i := l_b___0 + l_a___5
							l_j := (l_b___0 + l_a___5) + l_dual
							l_t1_real := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] + a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32]
							l_t1_imag := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] + a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32]
							l_t2_real := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] - a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32]
							l_t2_imag := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] - a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32]
							a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := l_t1_real
							a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := l_t1_imag
							a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32] := l_w_real * l_t2_real - l_w_imag * l_t2_imag
							a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32] := l_w_real * l_t2_imag + l_w_imag * l_t2_real
							l_a___5 := l_a___5 + ({NATURAL_32} 2 * l_dual)
						end
						l_tmp_real := (l_w_real - l_s * l_w_imag) - l_s2 * l_w_real
						l_tmp_imag := (l_w_imag + l_s * l_w_real) - l_s2 * l_w_imag
						l_w_real := l_tmp_real
						l_w_imag := l_tmp_imag
						l_b___0 := l_b___0 + 1
					end
					l_dual := l_dual // ({NATURAL_32} 2)
					l_bit := l_bit + 1
				end
				l_status := (create {P_BITREVERSE_STATIC}).fft_complex_bitreverse_order (a_data, a_stride, a_n, l_logn)
				Result := 0
			end
		end

	gsl_fft_complex_float_radix2_forward (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := -1
			l_status := (gsl_fft_complex_float_radix2_transform (a_data, a_stride, a_n, l_sign))
			Result := l_status
		end

	gsl_fft_complex_float_radix2_backward (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_fft_complex_float_radix2_transform (a_data, a_stride, a_n, l_sign))
			Result := l_status
		end

	gsl_fft_complex_float_radix2_inverse (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_norm___0: REAL_32
			l_i: NATURAL_32
			l_return: BOOLEAN
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_fft_complex_float_radix2_transform (a_data, a_stride, a_n, l_sign))
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_norm___0 := (1.0 / a_n.to_real_64).truncated_to_real
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] * l_norm___0
					a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] * l_norm___0
					l_i := l_i + 1
				end
				Result := l_status
			end
		end

	gsl_fft_complex_float_radix2_transform (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_sign: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_dual: NATURAL_32
			l_bit: NATURAL_32
			l_logn: NATURAL_32
			l_status: INTEGER_32
			l_w_real: REAL_32
			l_w_imag: REAL_32
			l_theta: REAL_64
			l_s: REAL_32
			l_t: REAL_32
			l_s2: REAL_32
			l_a___5: NATURAL_32
			l_b___0: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_wd_real: REAL_32
			l_wd_imag: REAL_32
			l_tmp_real: REAL_32
			l_tmp_imag: REAL_32
			l_i___0: NATURAL_32
			l_j___0: NATURAL_32
			l_z1_real___0: REAL_32
			l_z1_imag___0: REAL_32
			l_wd_real___0: REAL_32
			l_wd_imag___0: REAL_32
			l_return, l_break: BOOLEAN
		do
			l_logn := (0).to_natural_32
			if a_n = {NATURAL_32} 1 then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_result := (create {P_FACTORIZE_STATIC}).fft_binary_logn (a_n)
				if l_result = -1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("n is not a power of 2"), ce_string ("c_radix2.c"), 90, 4)
						Result := 4
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				else
					l_logn := l_result.to_natural_32
				end
			end
			if not l_return then
				l_status := (create {P_BITREVERSE_STATIC}).fft_complex_float_bitreverse_order (a_data, a_stride, a_n, l_logn)
				l_dual := (1).to_natural_32
				l_bit := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_bit < l_logn))
				loop
					l_w_real := (1.0).truncated_to_real
					l_w_imag := (0.0).truncated_to_real
					l_theta := ((2.0 * a_sign.to_double) * 3.14159265358979323846) / (2.0 * l_dual.to_real_64)
					l_s := (sin (l_theta)).truncated_to_real
					l_t := (sin (l_theta / 2.0)).truncated_to_real
					l_s2 := ((2.0 * l_t.to_double) * l_t.to_double).truncated_to_real
					l_b___0 := (0).to_natural_32
					from
					until
						not (l_b___0 < a_n)
					loop
						l_i := l_b___0
						l_j := l_b___0 + l_dual
						l_z1_real := a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32]
						l_z1_imag := a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32]
						l_wd_real := l_z1_real
						l_wd_imag := l_z1_imag
						a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] - l_wd_real
						a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] - l_wd_imag
						a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] + l_wd_real
						a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] + l_wd_imag
						l_b___0 := l_b___0 + ({NATURAL_32} 2 * l_dual)
					end
					l_a___5 := (1).to_natural_32
					from
					until
						not (not (l_return) and then (l_a___5 < l_dual))
					loop
						l_tmp_real := (l_w_real - l_s * l_w_imag) - l_s2 * l_w_real
						l_tmp_imag := (l_w_imag + l_s * l_w_real) - l_s2 * l_w_imag
						l_w_real := l_tmp_real
						l_w_imag := l_tmp_imag
						l_b___0 := (0).to_natural_32
						from
						until
							not (l_b___0 < a_n)
						loop
							l_i___0 := l_b___0 + l_a___5
							l_j___0 := (l_b___0 + l_a___5) + l_dual
							l_z1_real___0 := a_data[(({NATURAL_32} 2 * a_stride) * l_j___0).to_integer_32]
							l_z1_imag___0 := a_data[(({NATURAL_32} 2 * a_stride) * l_j___0 + {NATURAL_32} 1).to_integer_32]
							l_wd_real___0 := l_w_real * l_z1_real___0 - l_w_imag * l_z1_imag___0
							l_wd_imag___0 := l_w_real * l_z1_imag___0 + l_w_imag * l_z1_real___0
							a_data[(({NATURAL_32} 2 * a_stride) * l_j___0).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i___0).to_integer_32] - l_wd_real___0
							a_data[(({NATURAL_32} 2 * a_stride) * l_j___0 + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i___0 + {NATURAL_32} 1).to_integer_32] - l_wd_imag___0
							a_data[(({NATURAL_32} 2 * a_stride) * l_i___0).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i___0).to_integer_32] + l_wd_real___0
							a_data[(({NATURAL_32} 2 * a_stride) * l_i___0 + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i___0 + {NATURAL_32} 1).to_integer_32] + l_wd_imag___0
							l_b___0 := l_b___0 + ({NATURAL_32} 2 * l_dual)
						end
						l_a___5 := l_a___5 + 1
					end
					l_dual := l_dual * ({NATURAL_32} 2)
					l_bit := l_bit + 1
				end
				Result := 0
			end
		end

	gsl_fft_complex_float_radix2_dif_forward (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := -1
			l_status := (gsl_fft_complex_float_radix2_dif_transform (a_data, a_stride, a_n, l_sign))
			Result := l_status
		end

	gsl_fft_complex_float_radix2_dif_backward (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_fft_complex_float_radix2_dif_transform (a_data, a_stride, a_n, l_sign))
			Result := l_status
		end

	gsl_fft_complex_float_radix2_dif_inverse (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_norm___0: REAL_32
			l_i: NATURAL_32
			l_return: BOOLEAN
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_fft_complex_float_radix2_dif_transform (a_data, a_stride, a_n, l_sign))
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_norm___0 := (1.0 / a_n.to_real_64).truncated_to_real
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] * l_norm___0
					a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] * l_norm___0
					l_i := l_i + 1
				end
				Result := l_status
			end
		end

	gsl_fft_complex_float_radix2_dif_transform (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_sign: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_dual: NATURAL_32
			l_bit: NATURAL_32
			l_logn: NATURAL_32
			l_status: INTEGER_32
			l_w_real: REAL_32
			l_w_imag: REAL_32
			l_theta: REAL_64
			l_s: REAL_32
			l_t: REAL_32
			l_s2: REAL_32
			l_a___5: NATURAL_32
			l_b___0: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_t1_real: REAL_32
			l_t1_imag: REAL_32
			l_t2_real: REAL_32
			l_t2_imag: REAL_32
			l_tmp_real: REAL_32
			l_tmp_imag: REAL_32
			l_return, l_break: BOOLEAN
		do
			l_logn := (0).to_natural_32
			if a_n = {NATURAL_32} 1 then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_result := (create {P_FACTORIZE_STATIC}).fft_binary_logn (a_n)
				if l_result = -1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("n is not a power of 2"), ce_string ("c_radix2.c"), 247, 4)
						Result := 4
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				else
					l_logn := l_result.to_natural_32
				end
			end
			if not l_return then
				l_dual := a_n // {NATURAL_32} 2
				l_bit := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_bit < l_logn))
				loop
					l_w_real := (1.0).truncated_to_real
					l_w_imag := (0.0).truncated_to_real
					l_theta := ((2.0 * a_sign.to_double) * 3.14159265358979323846) / ({NATURAL_32} 2 * l_dual).to_real_64
					l_s := (sin (l_theta)).truncated_to_real
					l_t := (sin (l_theta / 2.0)).truncated_to_real
					l_s2 := ((2.0 * l_t.to_double) * l_t.to_double).truncated_to_real
					l_b___0 := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_b___0 < l_dual))
					loop
						l_a___5 := (0).to_natural_32
						from
						until
							not (l_a___5 < a_n)
						loop
							l_i := l_b___0 + l_a___5
							l_j := (l_b___0 + l_a___5) + l_dual
							l_t1_real := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] + a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32]
							l_t1_imag := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] + a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32]
							l_t2_real := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] - a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32]
							l_t2_imag := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] - a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32]
							a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := l_t1_real
							a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := l_t1_imag
							a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32] := l_w_real * l_t2_real - l_w_imag * l_t2_imag
							a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32] := l_w_real * l_t2_imag + l_w_imag * l_t2_real
							l_a___5 := l_a___5 + ({NATURAL_32} 2 * l_dual)
						end
						l_tmp_real := (l_w_real - l_s * l_w_imag) - l_s2 * l_w_real
						l_tmp_imag := (l_w_imag + l_s * l_w_real) - l_s2 * l_w_imag
						l_w_real := l_tmp_real
						l_w_imag := l_tmp_imag
						l_b___0 := l_b___0 + 1
					end
					l_dual := l_dual // ({NATURAL_32} 2)
					l_bit := l_bit + 1
				end
				l_status := (create {P_BITREVERSE_STATIC}).fft_complex_float_bitreverse_order (a_data, a_stride, a_n, l_logn)
				Result := 0
			end
		end

end
