class
	P_DWT_STATIC

inherit
	CE_UTILITIES

feature {ANY} -- public routines

	binary_logn (a_n: NATURAL_32): INTEGER_32
		local
			l_ntest: NATURAL_32
			l_logn: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_logn := (0).to_natural_32
			l_k := (1).to_natural_32
			from
			until
				not (l_k < a_n)
			loop
				l_k := l_k * ({NATURAL_32} 2)
				l_logn := l_logn + 1
			end
			l_ntest := ((1).bit_shift_left (l_logn.to_integer_32)).to_natural_32
			if a_n /= l_ntest then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				Result := l_logn.to_integer_32
			end
		end

	gsl_wavelet_transform (a_w: CE_POINTER [PD_GSL_WAVELET]; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_dir: INTEGER_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		require
			not a_work.is_void
		local
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_work.item.n < a_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("not enough workspace provided"), ce_string ("dwt.c"), 128, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if (binary_logn (a_n)) = -1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("n is not a power of 2"), ce_string ("dwt.c"), 133, 4)
						Result := 4
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return and then (a_n < {NATURAL_32} 2) then
				Result := 0
				l_return := True
			end
			if not l_return then
				if a_dir = 1 then
					l_i := a_n
					from
					until
						not (l_i >= {NATURAL_32} 2)
					loop
						a_w.item.dwt_step (a_data, a_stride, l_i, a_dir, a_work)
						l_i := (l_i).bit_shift_right ((1))
					end
				else
					l_i := (2).to_natural_32
					from
					until
						not (l_i <= a_n)
					loop
						a_w.item.dwt_step (a_data, a_stride, l_i, a_dir, a_work)
						l_i := (l_i).bit_shift_left ((1))
					end
				end
				Result := 0
			end
		end

	gsl_wavelet_transform_forward (a_w: CE_POINTER [PD_GSL_WAVELET]; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		require
			not a_work.is_void
		do
			Result := (gsl_wavelet_transform (a_w, a_data, a_stride, a_n, {PC_GSL_WAVELET_DIRECTION}.gsl_wavelet_forward, a_work))
		end

	gsl_wavelet_transform_inverse (a_w: CE_POINTER [PD_GSL_WAVELET]; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		require
			not a_work.is_void
		do
			Result := (gsl_wavelet_transform (a_w, a_data, a_stride, a_n, -1, a_work))
		end

	gsl_wavelet2d_transform (a_w: CE_POINTER [PD_GSL_WAVELET]; a_data: CE_POINTER [REAL_64]; a_tda, a_size1, a_size2: NATURAL_32; a_dir: INTEGER_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		local
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_size1 /= a_size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("2d dwt works only with square matrix"), ce_string ("dwt.c"), 196, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_work.item.n < a_size1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("not enough workspace provided"), ce_string ("dwt.c"), 201, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if (binary_logn (a_size1)) = -1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("n is not a power of 2"), ce_string ("dwt.c"), 206, 4)
						Result := 4
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return and then (a_size1 < {NATURAL_32} 2) then
				Result := 0
				l_return := True
			end
			if not l_return then
				if a_dir = 1 then
					l_i := (0).to_natural_32
					from
					until
						not (l_i < a_size1)
					loop
						gsl_wavelet_transform (a_w, a_data + (a_tda * l_i).to_integer_32, (1).to_natural_32, a_size1, a_dir, a_work).do_nothing
						l_i := l_i + 1
					end
					l_i := (0).to_natural_32
					from
					until
						not (l_i < a_size2)
					loop
						gsl_wavelet_transform (a_w, a_data + l_i.to_integer_32, a_tda, a_size2, a_dir, a_work).do_nothing
						l_i := l_i + 1
					end
				else
					l_i := (0).to_natural_32
					from
					until
						not (l_i < a_size2)
					loop
						gsl_wavelet_transform (a_w, a_data + l_i.to_integer_32, a_tda, a_size2, a_dir, a_work).do_nothing
						l_i := l_i + 1
					end
					l_i := (0).to_natural_32
					from
					until
						not (l_i < a_size1)
					loop
						gsl_wavelet_transform (a_w, a_data + (a_tda * l_i).to_integer_32, (1).to_natural_32, a_size1, a_dir, a_work).do_nothing
						l_i := l_i + 1
					end
				end
				Result := 0
			end
		end

	gsl_wavelet2d_nstransform (a_w: CE_POINTER [PD_GSL_WAVELET]; a_data: CE_POINTER [REAL_64]; a_tda, a_size1, a_size2: NATURAL_32; a_dir: INTEGER_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_size1 /= a_size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("2d dwt works only with square matrix"), ce_string ("dwt.c"), 250, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_work.item.n < a_size1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("not enough workspace provided"), ce_string ("dwt.c"), 255, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if (binary_logn (a_size1)) = -1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("n is not a power of 2"), ce_string ("dwt.c"), 260, 4)
						Result := 4
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return and then (a_size1 < {NATURAL_32} 2) then
				Result := 0
				l_return := True
			end
			if not l_return then
				if a_dir = 1 then
					l_i := a_size1
					from
					until
						not (not (l_return) and then (l_i >= {NATURAL_32} 2))
					loop
						l_j := (0).to_natural_32
						from
						until
							not (l_j < l_i)
						loop
							a_w.item.dwt_step (a_data + (a_tda * l_j).to_integer_32, (1).to_natural_32, l_i, a_dir, a_work)
							l_j := l_j + 1
						end
						l_j := (0).to_natural_32
						from
						until
							not (l_j < l_i)
						loop
							a_w.item.dwt_step (a_data + l_j.to_integer_32, a_tda, l_i, a_dir, a_work)
							l_j := l_j + 1
						end
						l_i := (l_i).bit_shift_right ((1))
					end
				else
					l_i := (2).to_natural_32
					from
					until
						not (not (l_return) and then (l_i <= a_size1))
					loop
						l_j := (0).to_natural_32
						from
						until
							not (l_j < l_i)
						loop
							a_w.item.dwt_step (a_data + l_j.to_integer_32, a_tda, l_i, a_dir, a_work)
							l_j := l_j + 1
						end
						l_j := (0).to_natural_32
						from
						until
							not (l_j < l_i)
						loop
							a_w.item.dwt_step (a_data + (a_tda * l_j).to_integer_32, (1).to_natural_32, l_i, a_dir, a_work)
							l_j := l_j + 1
						end
						l_i := (l_i).bit_shift_left ((1))
					end
				end
				Result := 0
			end
		end

	gsl_wavelet2d_transform_forward (a_w: CE_POINTER [PD_GSL_WAVELET]; a_data: CE_POINTER [REAL_64]; a_tda, a_size1, a_size2: NATURAL_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		do
			Result := (gsl_wavelet2d_transform (a_w, a_data, a_tda, a_size1, a_size2, {PC_GSL_WAVELET_DIRECTION}.gsl_wavelet_forward, a_work))
		end

	gsl_wavelet2d_transform_inverse (a_w: CE_POINTER [PD_GSL_WAVELET]; a_data: CE_POINTER [REAL_64]; a_tda, a_size1, a_size2: NATURAL_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		do
			Result := (gsl_wavelet2d_transform (a_w, a_data, a_tda, a_size1, a_size2, -1, a_work))
		end

	gsl_wavelet2d_nstransform_forward (a_w: CE_POINTER [PD_GSL_WAVELET]; a_data: CE_POINTER [REAL_64]; a_tda, a_size1, a_size2: NATURAL_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		do
			Result := (gsl_wavelet2d_nstransform (a_w, a_data, a_tda, a_size1, a_size2, {PC_GSL_WAVELET_DIRECTION}.gsl_wavelet_forward, a_work))
		end

	gsl_wavelet2d_nstransform_inverse (a_w: CE_POINTER [PD_GSL_WAVELET]; a_data: CE_POINTER [REAL_64]; a_tda, a_size1, a_size2: NATURAL_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		do
			Result := (gsl_wavelet2d_nstransform (a_w, a_data, a_tda, a_size1, a_size2, -1, a_work))
		end

	gsl_wavelet2d_transform_matrix (a_w: CE_POINTER [PD_GSL_WAVELET]; a_a___5: CE_POINTER [PD_GSL_MATRIX]; a_dir: INTEGER_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		require
			not a_a___5.is_void
		do
			Result := (gsl_wavelet2d_transform (a_w, a_a___5.item.data, a_a___5.item.tda, a_a___5.item.size1, a_a___5.item.size2, a_dir, a_work))
		end

	gsl_wavelet2d_transform_matrix_forward (a_w: CE_POINTER [PD_GSL_WAVELET]; a_a___5: CE_POINTER [PD_GSL_MATRIX]; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		require
			not a_a___5.is_void
		do
			Result := (gsl_wavelet2d_transform (a_w, a_a___5.item.data, a_a___5.item.tda, a_a___5.item.size1, a_a___5.item.size2, {PC_GSL_WAVELET_DIRECTION}.gsl_wavelet_forward, a_work))
		end

	gsl_wavelet2d_transform_matrix_inverse (a_w: CE_POINTER [PD_GSL_WAVELET]; a_a___5: CE_POINTER [PD_GSL_MATRIX]; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		require
			not a_a___5.is_void
		do
			Result := (gsl_wavelet2d_transform (a_w, a_a___5.item.data, a_a___5.item.tda, a_a___5.item.size1, a_a___5.item.size2, -1, a_work))
		end

	gsl_wavelet2d_nstransform_matrix (a_w: CE_POINTER [PD_GSL_WAVELET]; a_a___5: CE_POINTER [PD_GSL_MATRIX]; a_dir: INTEGER_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		require
			not a_a___5.is_void
		do
			Result := (gsl_wavelet2d_nstransform (a_w, a_a___5.item.data, a_a___5.item.tda, a_a___5.item.size1, a_a___5.item.size2, a_dir, a_work))
		end

	gsl_wavelet2d_nstransform_matrix_forward (a_w: CE_POINTER [PD_GSL_WAVELET]; a_a___5: CE_POINTER [PD_GSL_MATRIX]; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		require
			not a_a___5.is_void
		do
			Result := (gsl_wavelet2d_nstransform (a_w, a_a___5.item.data, a_a___5.item.tda, a_a___5.item.size1, a_a___5.item.size2, {PC_GSL_WAVELET_DIRECTION}.gsl_wavelet_forward, a_work))
		end

	gsl_wavelet2d_nstransform_matrix_inverse (a_w: CE_POINTER [PD_GSL_WAVELET]; a_a___5: CE_POINTER [PD_GSL_MATRIX]; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE]): INTEGER_32
		require
			not a_a___5.is_void
		do
			Result := (gsl_wavelet2d_nstransform (a_w, a_a___5.item.data, a_a___5.item.tda, a_a___5.item.size1, a_a___5.item.size2, -1, a_work))
		end

end
