class
	P_HERM_STATIC

inherit
	CE_UTILITIES
	S_STDLIB_STATIC

feature {ANY} -- public routines

	gsl_eigen_herm_alloc (a_n: NATURAL_32): CE_POINTER [PD_GSL_EIGEN_HERM_WORKSPACE]
		local
			l_w: CE_POINTER [PD_GSL_EIGEN_HERM_WORKSPACE]
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension must be positive integer"), ce_string ("herm.c"), 43, 4)
					Result := (create {CE_CLASS_POINTER [PD_GSL_EIGEN_HERM_WORKSPACE]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_w := create {CE_CLASS_POINTER [PD_GSL_EIGEN_HERM_WORKSPACE]}.make_cast ((malloc ((create {PD_GSL_EIGEN_HERM_WORKSPACE}.make).structure_size.to_natural_32)))
				if l_w.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for workspace"), ce_string ("herm.c"), 50, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_EIGEN_HERM_WORKSPACE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_w.item.d := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_n * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_w.item.d).is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for diagonal"), ce_string ("herm.c"), 57, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_EIGEN_HERM_WORKSPACE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_w.item.sd := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_n * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_w.item.sd).is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for subdiagonal"), ce_string ("herm.c"), 64, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_EIGEN_HERM_WORKSPACE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_w.item.tau := create {CE_REAL_64_POINTER}.make_cast ((malloc (({NATURAL_32} 2 * a_n) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_w.item.tau).is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for tau"), ce_string ("herm.c"), 71, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_EIGEN_HERM_WORKSPACE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_w.item.size := a_n
				Result := l_w
			end
		end

	gsl_eigen_herm_free (a_w: CE_POINTER [PD_GSL_EIGEN_HERM_WORKSPACE])
		local
			l_return: BOOLEAN
		do
			if a_w.is_void then
				l_return := True
			end
			if not l_return then
				free (create {CE_VOID_POINTER}.make_cast (a_w.item.tau))
				free (create {CE_VOID_POINTER}.make_cast (a_w.item.sd))
				free (create {CE_VOID_POINTER}.make_cast (a_w.item.d))
				free (create {CE_VOID_POINTER}.make_cast (a_w))
			end
		end

end
