class
	P_KURTOSIS_SOURCE_STATIC

inherit
	CE_UTILITIES

feature {ANY} -- public routines

	gsl_stats_long_double_kurtosis (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_double_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_long_double_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_long_double_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_long_double_kurtosis_m_sd (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := ((a_data[(l_i * a_stride).to_integer_32] - real_96_from_real_64 (a_mean)) / real_96_from_real_64 (a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_kurtosis (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_kurtosis_m_sd (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32] - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_float_kurtosis (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_float_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_float_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_float_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_float_kurtosis_m_sd (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_ulong_kurtosis (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ulong_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_ulong_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_ulong_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_ulong_kurtosis_m_sd (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_long_kurtosis (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_long_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_long_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_long_kurtosis_m_sd (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_uint_kurtosis (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uint_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_uint_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_uint_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_uint_kurtosis_m_sd (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_int_kurtosis (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_int_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_int_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_int_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_int_kurtosis_m_sd (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_ushort_kurtosis (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ushort_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_ushort_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_ushort_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_ushort_kurtosis_m_sd (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_short_kurtosis (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_short_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_short_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_short_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_short_kurtosis_m_sd (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_uchar_kurtosis (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uchar_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_uchar_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_uchar_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_uchar_kurtosis_m_sd (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

	gsl_stats_char_kurtosis (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_est_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_char_mean (a_data, a_stride, a_n))
			l_est_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_char_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_char_kurtosis_m_sd (a_data, a_stride, a_n, l_mean, l_est_sd))
		end

	gsl_stats_char_kurtosis_m_sd (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_avg: REAL_96
			l_kurtosis: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_avg := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_avg := l_avg + ((((l_x * l_x) * l_x) * l_x - l_avg) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			l_kurtosis := l_avg - real_96_from_real_64 (3.0)
			Result := l_kurtosis.truncated_to_real_64
		end

end
