class
	P_MATHIEU_CHARV_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC

feature {ANY} -- public routines

	ceer (a_order: INTEGER_32; a_qq, a_aa: REAL_64; a_nterms: INTEGER_32): REAL_64
		local
			l_term: REAL_64
			l_term1: REAL_64
			l_ii: INTEGER_32
			l_n1: INTEGER_32
		do
			if a_order = 0 then
				l_term := 0.0
			else
				l_term := ((2.0 * a_qq) * a_qq) / a_aa
				if a_order /= 2 then
					l_n1 := a_order // 2 - 1
					l_ii := 0
					from
					until
						not (l_ii < l_n1)
					loop
						l_term := (a_qq * a_qq) / ((a_aa - (4.0 * (l_ii + 1).to_double) * (l_ii + 1).to_double) - l_term)
						l_ii := l_ii + 1
					end
				end
			end
			l_term := l_term + (a_order * a_order).to_double
			l_term1 := 0.0
			l_ii := 0
			from
			until
				not (l_ii < a_nterms)
			loop
				l_term1 := (a_qq * a_qq) / ((a_aa - (a_order.to_double + 2.0 * (a_nterms - l_ii).to_double) * (a_order.to_double + 2.0 * (a_nterms - l_ii).to_double)) - l_term1)
				l_ii := l_ii + 1
			end
			if a_order = 0 then
				l_term1 := l_term1 * (2.0)
			end
			Result := ((l_term + l_term1) - a_aa)
		end

	ceor (a_order: INTEGER_32; a_qq, a_aa: REAL_64; a_nterms: INTEGER_32): REAL_64
		local
			l_term: REAL_64
			l_term1: REAL_64
			l_ii: INTEGER_32
			l_n1: INTEGER_32
		do
			l_term := a_qq
			l_n1 := ((a_order.to_real).to_double / 2.0 - 0.5).truncated_to_integer
			l_ii := 0
			from
			until
				not (l_ii < l_n1)
			loop
				l_term := (a_qq * a_qq) / ((a_aa - (2.0 * l_ii.to_double + 1.0) * (2.0 * l_ii.to_double + 1.0)) - l_term)
				l_ii := l_ii + 1
			end
			l_term := l_term + (a_order * a_order).to_double
			l_term1 := 0.0
			l_ii := 0
			from
			until
				not (l_ii < a_nterms)
			loop
				l_term1 := (a_qq * a_qq) / ((a_aa - (a_order.to_double + 2.0 * (a_nterms - l_ii).to_double) * (a_order.to_double + 2.0 * (a_nterms - l_ii).to_double)) - l_term1)
				l_ii := l_ii + 1
			end
			Result := ((l_term + l_term1) - a_aa)
		end

	seer (a_order: INTEGER_32; a_qq, a_aa: REAL_64; a_nterms: INTEGER_32): REAL_64
		local
			l_term: REAL_64
			l_term1: REAL_64
			l_ii: INTEGER_32
			l_n1: INTEGER_32
		do
			l_term := 0.0
			l_n1 := a_order // 2 - 1
			l_ii := 0
			from
			until
				not (l_ii < l_n1)
			loop
				l_term := (a_qq * a_qq) / ((a_aa - (4.0 * (l_ii + 1).to_double) * (l_ii + 1).to_double) - l_term)
				l_ii := l_ii + 1
			end
			l_term := l_term + (a_order * a_order).to_double
			l_term1 := 0.0
			l_ii := 0
			from
			until
				not (l_ii < a_nterms)
			loop
				l_term1 := (a_qq * a_qq) / ((a_aa - (a_order.to_double + 2.0 * (a_nterms - l_ii).to_double) * (a_order.to_double + 2.0 * (a_nterms - l_ii).to_double)) - l_term1)
				l_ii := l_ii + 1
			end
			Result := ((l_term + l_term1) - a_aa)
		end

	seor (a_order: INTEGER_32; a_qq, a_aa: REAL_64; a_nterms: INTEGER_32): REAL_64
		local
			l_term: REAL_64
			l_term1: REAL_64
			l_ii: INTEGER_32
			l_n1: INTEGER_32
		do
			l_term := -1.0 * a_qq
			l_n1 := ((a_order.to_real).to_double / 2.0 - 0.5).truncated_to_integer
			l_ii := 0
			from
			until
				not (l_ii < l_n1)
			loop
				l_term := (a_qq * a_qq) / ((a_aa - (2.0 * l_ii.to_double + 1.0) * (2.0 * l_ii.to_double + 1.0)) - l_term)
				l_ii := l_ii + 1
			end
			l_term := l_term + (a_order * a_order).to_double
			l_term1 := 0.0
			l_ii := 0
			from
			until
				not (l_ii < a_nterms)
			loop
				l_term1 := (a_qq * a_qq) / ((a_aa - (a_order.to_double + 2.0 * (a_nterms - l_ii).to_double) * (a_order.to_double + 2.0 * (a_nterms - l_ii).to_double)) - l_term1)
				l_ii := l_ii + 1
			end
			Result := ((l_term + l_term1) - a_aa)
		end

	asymptotic (a_order: INTEGER_32; a_qq: REAL_64): REAL_64
		local
			l_asymp: REAL_64
			l_nn: REAL_64
			l_n2: REAL_64
			l_n4: REAL_64
			l_n6: REAL_64
			l_hh: REAL_64
			l_ah___2: REAL_64
			l_ah2: REAL_64
			l_ah3: REAL_64
			l_ah4: REAL_64
			l_ah5: REAL_64
		do
			l_nn := (2 * a_order + 1).to_double
			l_n2 := l_nn * l_nn
			l_n4 := l_n2 * l_n2
			l_n6 := l_n4 * l_n2
			l_hh := (2).to_double * (sqrt (a_qq))
			l_ah___2 := (16).to_double * l_hh
			l_ah2 := l_ah___2 * l_ah___2
			l_ah3 := l_ah2 * l_ah___2
			l_ah4 := l_ah3 * l_ah___2
			l_ah5 := l_ah4 * l_ah___2
			l_asymp := ((-2).to_double * a_qq + l_nn * l_hh) - 0.125 * (l_n2 + (1).to_double)
			l_asymp := l_asymp - (((0.25 * l_nn) * (l_n2 + (3).to_double)) / l_ah___2)
			l_asymp := l_asymp - ((0.25 * (((5).to_double * l_n4 + (34).to_double * l_n2) + (9).to_double)) / l_ah2)
			l_asymp := l_asymp - (((0.25 * l_nn) * (((33).to_double * l_n4 + (410).to_double * l_n2) + (405).to_double)) / l_ah3)
			l_asymp := l_asymp - (((((63).to_double * l_n6 + (1_260).to_double * l_n4) + (2_943).to_double * l_n2) + (486).to_double) / l_ah4)
			l_asymp := l_asymp - ((l_nn * ((((527).to_double * l_n6 + (15_617).to_double * l_n4) + (69_001).to_double * l_n2) + (41_607).to_double)) / l_ah5)
			Result := l_asymp
		end

	solve_cubic (a_c2___0, a_c1___2, a_c0___0: REAL_64): REAL_64
		local
			l_qq: REAL_64
			l_rr: REAL_64
			l_ww: REAL_64
			l_ss: REAL_64
			l_tt: REAL_64
			l_t1: REAL_64
			l_tmp___0: REAL_64
			l_tmp___4: REAL_64
			l_theta: REAL_64
			l_tmp___9: REAL_64
		do
			l_qq := ((3).to_double * a_c1___2 - a_c2___0 * a_c2___0) / (9).to_double
			l_rr := ((((9).to_double * a_c2___0) * a_c1___2 - (27).to_double * a_c0___0) - (((2).to_double * a_c2___0) * a_c2___0) * a_c2___0) / (54).to_double
			l_ww := (l_qq * l_qq) * l_qq + l_rr * l_rr
			if l_ww >= (0).to_double then
				l_t1 := l_rr + (sqrt (l_ww))
				l_tmp___0 := fabs (l_t1)
				l_ss := (l_tmp___0 / l_t1) * (pow ((fabs (l_t1)), (1).to_double / 3.))
				l_t1 := l_rr - (sqrt (l_ww))
				l_tmp___4 := fabs (l_t1)
				l_tt := (l_tmp___4 / l_t1) * (pow ((fabs (l_t1)), (1).to_double / 3.))
			else
				l_theta := (acos (l_rr / (sqrt ((-l_qq * l_qq) * l_qq))))
				l_tmp___9 := sqrt (-l_qq)
				l_ss := ((2).to_double * l_tmp___9) * (cos ((l_theta + (4).to_double * 3.14159265358979323846) / 3.))
				l_tt := 0.0
			end
			Result := ((l_ss + l_tt) - a_c2___0 / (3).to_double)
		end

	approx_c (a_order: INTEGER_32; a_qq: REAL_64): REAL_64
		local
			l_approx: REAL_64
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_n2: REAL_64
			l_n22: REAL_64
			l_q2___0: REAL_64
			l_q4: REAL_64
			l_tmp___9: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_order < 0 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Undefined order for Mathieu function"), ce_string ("mathieu_charv.c"), 213, 4)
							Result := (0.0)
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 7 -- skip if block
					end
				when 1 then -- if:
						inspect a_order -- switch 1 head
						when 0 then l_pos := -1 + 2
						when 1 then l_pos := -1 + 3
						when 2 then l_pos := -1 + 4
						when 3 then l_pos := -1 + 5
						else l_pos := -1 + 6
						end
				when 2 then -- switch 1, case 0
						if a_qq <= (4).to_double then
							Result := ((2).to_double - (sqrt ((4).to_double + ((2).to_double * a_qq) * a_qq)))
							l_return := True
						else
							Result := (asymptotic (a_order, a_qq))
							l_return := True
						end
						l_pos := -1 + 7 -- switch 1; break
				when 3 then -- switch 1, case 1
						if a_qq <= (4).to_double then
							Result := ((5).to_double + 0.5 * (a_qq - (sqrt ((((5).to_double * a_qq) * a_qq - (16).to_double * a_qq) + (64).to_double))))
							l_return := True
						else
							Result := (asymptotic (a_order, a_qq))
							l_return := True
						end
						l_pos := -1 + 7 -- switch 1; break
				when 4 then -- switch 1, case 2
						if a_qq <= (3).to_double then
							l_c2___0 := -8.0
							l_c1___2 := (-48).to_double - ((3).to_double * a_qq) * a_qq
							l_c0___0 := ((20).to_double * a_qq) * a_qq
						else
							Result := (asymptotic (a_order, a_qq))
							l_return := True
						end
						l_pos := -1 + 7 -- switch 1; break
				when 5 then -- switch 1, case 3
						if a_qq <= 6.25 then
							l_c2___0 := -a_qq - (8).to_double
							l_c1___2 := ((16).to_double * a_qq - (128).to_double) - ((2).to_double * a_qq) * a_qq
							l_c0___0 := (a_qq * a_qq) * (a_qq + (8).to_double)
						else
							Result := (asymptotic (a_order, a_qq))
							l_return := True
						end
						l_pos := -1 + 7 -- switch 1; break
				when 6 then -- switch 1, default:
						if a_order < 70 then
							if 1.7 * a_order.to_double > (2).to_double * (sqrt (a_qq)) then
								l_n2 := (a_order * a_order).to_double
								l_n22 := (l_n2 - (1).to_double) * (l_n2 - (1).to_double)
								l_q2___0 := a_qq * a_qq
								l_q4 := l_q2___0 * l_q2___0
								l_approx := l_n2 + (0.5 * l_q2___0) / (l_n2 - (1).to_double)
								l_approx := l_approx + ((((5).to_double * l_n2 + (7).to_double) * l_q4) / ((((32).to_double * l_n22) * (l_n2 - (1).to_double)) * (l_n2 - (4).to_double)))
								l_approx := l_approx + (((((((9).to_double * l_n2) * l_n2 + (58).to_double * l_n2) + (29).to_double) * l_q4) * l_q2___0) / ((((((64).to_double * l_n22) * l_n22) * (l_n2 - (1).to_double)) * (l_n2 - (4).to_double)) * (l_n2 - (9).to_double)))
								if 1.4 * a_order.to_double < (2).to_double * (sqrt (a_qq)) then
									l_approx := l_approx + (asymptotic (a_order, a_qq))
									l_approx := l_approx * (0.5)
								end
							else
								l_approx := asymptotic (a_order, a_qq)
							end
							Result := l_approx
							l_return := True
						else
							Result := (a_order * a_order).to_double
							l_return := True
						end
				when 7 then -- switch 1, done: | if done
					if not l_return then
						l_approx := solve_cubic (l_c2___0, l_c1___2, l_c0___0)
						if l_approx < (0).to_double then
							if (sqrt (a_qq)) > 0.1 * a_order.to_double then
								Result := (asymptotic (a_order - 1, a_qq))
							else
								l_tmp___9 := fabs (l_approx)
								Result := ((a_order * a_order).to_double + l_tmp___9)
							end
						else
							l_tmp___9 := fabs (l_approx)
							Result := ((a_order * a_order).to_double + l_tmp___9)
						end
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	approx_s (a_order: INTEGER_32; a_qq: REAL_64): REAL_64
		local
			l_approx: REAL_64
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_n2: REAL_64
			l_n22: REAL_64
			l_q2___0: REAL_64
			l_q4: REAL_64
			l_tmp___8: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_order < 1 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Undefined order for Mathieu function"), ce_string ("mathieu_charv.c"), 301, 4)
							Result := (0.0)
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						inspect a_order -- switch 1 head
						when 1 then l_pos := -1 + 2
						when 2 then l_pos := -1 + 3
						when 3 then l_pos := -1 + 4
						else l_pos := -1 + 5
						end
				when 2 then -- switch 1, case 1
						if a_qq <= (4).to_double then
							Result := ((5).to_double - 0.5 * (a_qq + (sqrt ((((5).to_double * a_qq) * a_qq + (16).to_double * a_qq) + (64).to_double))))
							l_return := True
						else
							Result := (asymptotic (a_order - 1, a_qq))
							l_return := True
						end
						l_pos := -1 + 6 -- switch 1; break
				when 3 then -- switch 1, case 2
						if a_qq <= (5).to_double then
							Result := ((10).to_double - (sqrt ((36).to_double + a_qq * a_qq)))
							l_return := True
						else
							Result := (asymptotic (a_order - 1, a_qq))
							l_return := True
						end
						l_pos := -1 + 6 -- switch 1; break
				when 4 then -- switch 1, case 3
						if a_qq <= 6.25 then
							l_c2___0 := a_qq - (8).to_double
							l_c1___2 := ((-128).to_double - (16).to_double * a_qq) - ((2).to_double * a_qq) * a_qq
							l_c0___0 := (a_qq * a_qq) * ((8).to_double - a_qq)
						else
							Result := (asymptotic (a_order - 1, a_qq))
							l_return := True
						end
						l_pos := -1 + 6 -- switch 1; break
				when 5 then -- switch 1, default:
						if a_order < 70 then
							if 1.7 * a_order.to_double > (2).to_double * (sqrt (a_qq)) then
								l_n2 := (a_order * a_order).to_double
								l_n22 := (l_n2 - (1).to_double) * (l_n2 - (1).to_double)
								l_q2___0 := a_qq * a_qq
								l_q4 := l_q2___0 * l_q2___0
								l_approx := l_n2 + (0.5 * l_q2___0) / (l_n2 - (1).to_double)
								l_approx := l_approx + ((((5).to_double * l_n2 + (7).to_double) * l_q4) / ((((32).to_double * l_n22) * (l_n2 - (1).to_double)) * (l_n2 - (4).to_double)))
								l_approx := l_approx + (((((((9).to_double * l_n2) * l_n2 + (58).to_double * l_n2) + (29).to_double) * l_q4) * l_q2___0) / ((((((64).to_double * l_n22) * l_n22) * (l_n2 - (1).to_double)) * (l_n2 - (4).to_double)) * (l_n2 - (9).to_double)))
								if 1.4 * a_order.to_double < (2).to_double * (sqrt (a_qq)) then
									l_approx := l_approx + (asymptotic (a_order - 1, a_qq))
									l_approx := l_approx * (0.5)
								end
							else
								l_approx := asymptotic (a_order - 1, a_qq)
							end
							Result := l_approx
							l_return := True
						else
							Result := (a_order * a_order).to_double
							l_return := True
						end
				when 6 then -- switch 1, done: | if done
					if not l_return then
						l_approx := solve_cubic (l_c2___0, l_c1___2, l_c0___0)
						if l_approx < (0).to_double then
							if (sqrt (a_qq)) > 0.1 * a_order.to_double then
								Result := (asymptotic (a_order - 1, a_qq))
							else
								l_tmp___8 := fabs (l_approx)
								Result := ((a_order * a_order).to_double + l_tmp___8)
							end
						else
							l_tmp___8 := fabs (l_approx)
							Result := ((a_order * a_order).to_double + l_tmp___8)
						end
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_mathieu_a (a_order: INTEGER_32; a_qq: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_order: INTEGER_32
			l_even_odd: INTEGER_32
			l_nterms: INTEGER_32
			l_ii: INTEGER_32
			l_counter: INTEGER_32
			l_maxcount: INTEGER_32
			l_dir: INTEGER_32
			l_a1___4: REAL_64
			l_a2___3: REAL_64
			l_fa: REAL_64
			l_fa1: REAL_64
			l_dela: REAL_64
			l_aa_orig: REAL_64
			l_da: REAL_64
			l_aa: REAL_64
			l_aa_approx: REAL_64
			l_tmp___1: REAL_64
			l_return, l_break, l_continue: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_order := a_order
					l_nterms := 50
					l_counter := 0
					l_maxcount := 1_000
					l_dir := 0
					l_da := 0.025
					l_even_odd := 0
					if l_order \\ 2 /= 0 then
						l_even_odd := 1
					end
					if a_qq = (0).to_double then
						a_result.item.val := (l_order * l_order).to_double
						a_result.item.err := 0.0
						Result := 0
						l_return := True
					end
					if not l_return then
						if l_order < 0 then
							l_order := l_order * (-1)
						end
						if a_qq < 0.0 then
							if l_even_odd = 0 then
								Result := (gsl_sf_mathieu_a (l_order, -a_qq, a_result))
								l_return := True
							else
								Result := (gsl_sf_mathieu_b (l_order, -a_qq, a_result))
								l_return := True
							end
						end
					end
					if l_return then
						l_pos := -1 + 15 -- skip if block
					end
				when 1 then -- if:
						l_aa_approx := approx_c (l_order, a_qq)
						l_aa := l_aa_approx
						l_aa_orig := l_aa
				when 2 then -- loop condition:
						if not (not (l_return or l_break) and then (l_counter < l_maxcount)) then
							l_pos := -1 + 14 -- skip loop
						end
				when 3 then -- loop body:
						l_break := False
						l_continue := False
						l_a1___4 := l_aa + 0.001
						l_ii := 0
						if l_even_odd = 0 then
							l_fa1 := ceer (l_order, a_qq, l_a1___4, l_nterms)
						else
							l_fa1 := ceor (l_order, a_qq, l_a1___4, l_nterms)
						end
						from
						until
							(l_break)
						loop
							l_break := False
							if l_even_odd = 0 then
								l_fa := ceer (l_order, a_qq, l_aa, l_nterms)
							else
								l_fa := ceor (l_order, a_qq, l_aa, l_nterms)
							end
							l_a2___3 := l_a1___4
							l_a1___4 := l_aa
							if l_fa = l_fa1 then
								a_result.item.err := 2.2204460492503131e-16
								l_break := True
							end
							if not (l_break) then
								l_aa := l_aa - (((l_aa - l_a2___3) / (l_fa - l_fa1)) * l_fa)
								l_dela := fabs (l_aa - l_a2___3)
								if l_dela < 2.2204460492503131e-16 then
									a_result.item.err := 2.2204460492503131e-16
									l_break := True
								end
							end
							if not (l_break) and then (l_ii > 40) then
								a_result.item.err := l_dela
								l_break := True
							end
							if not (l_break) then
								l_fa1 := l_fa
								l_ii := l_ii + 1
							end
						end
						l_break := False
						l_tmp___1 := fabs (l_aa - l_aa_orig)
						if not (l_tmp___1 > (3).to_double + (0.01 * l_order.to_double) * (fabs (l_aa_orig))) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 8 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if not (l_order > 10) then
							l_pos := -1 + 12 -- skip if block
						end
				when 7 then -- if:
						if not ((fabs (l_aa - l_aa_orig)) > 1.5 * l_order.to_double) then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if: | _L:
						l_counter := l_counter + 1
						if l_counter = l_maxcount then
							a_result.item.err := fabs (l_aa - l_aa_orig)
							l_break := True
						end
						if not (l_return or l_break or l_continue) then
							if l_aa > l_aa_orig then
								if l_dir = 1 then
									l_da := l_da / (2).to_double
								end
								l_dir := -1
							else
								if l_dir = -1 then
									l_da := l_da / (2).to_double
								end
								l_dir := 1
							end
							l_aa_approx := l_aa_approx + ((l_dir.to_double * l_da) * l_counter.to_double)
							l_aa := l_aa_approx
							l_continue := True
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else:
						l_break := True
				when 11 then -- if done | if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						l_break := True
				when 13 then -- if done
						l_pos := -1 + 2 -- back to loop condition
				when 14 then -- loop done
						l_break := False
						l_continue := False
						a_result.item.val := l_aa
						if l_counter = l_maxcount then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("Wrong characteristic Mathieu value"), ce_string ("mathieu_charv.c"), 489, 5)
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
				when 15 then -- if done
					if not l_return then
						Result := 0
					end
				when 16 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_mathieu_b (a_order: INTEGER_32; a_qq: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_order: INTEGER_32
			l_even_odd: INTEGER_32
			l_nterms: INTEGER_32
			l_ii: INTEGER_32
			l_counter: INTEGER_32
			l_maxcount: INTEGER_32
			l_dir: INTEGER_32
			l_a1___4: REAL_64
			l_a2___3: REAL_64
			l_fa: REAL_64
			l_fa1: REAL_64
			l_dela: REAL_64
			l_aa_orig: REAL_64
			l_da: REAL_64
			l_aa: REAL_64
			l_aa_approx: REAL_64
			l_tmp___1: REAL_64
			l_return, l_break, l_continue: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_order := a_order
					l_nterms := 50
					l_counter := 0
					l_maxcount := 1_000
					l_dir := 0
					l_da := 0.025
					l_even_odd := 0
					if l_order \\ 2 /= 0 then
						l_even_odd := 1
					end
					if l_order = 0 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Characteristic value undefined for order 0"), ce_string ("mathieu_charv.c"), 511, 5)
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return and then (a_qq = (0).to_double) then
						a_result.item.val := (l_order * l_order).to_double
						a_result.item.err := 0.0
						Result := 0
						l_return := True
					end
					if not l_return then
						if l_order < 0 then
							l_order := l_order * (-1)
						end
						if a_qq < 0.0 then
							if l_even_odd = 0 then
								Result := (gsl_sf_mathieu_b (l_order, -a_qq, a_result))
								l_return := True
							else
								Result := (gsl_sf_mathieu_a (l_order, -a_qq, a_result))
								l_return := True
							end
						end
					end
					if l_return then
						l_pos := -1 + 15 -- skip if block
					end
				when 1 then -- if:
						l_aa_approx := approx_s (l_order, a_qq)
						l_aa := l_aa_approx
						l_aa_orig := l_aa
				when 2 then -- loop condition:
						if not (not (l_return or l_break) and then (l_counter < l_maxcount)) then
							l_pos := -1 + 14 -- skip loop
						end
				when 3 then -- loop body:
						l_break := False
						l_continue := False
						l_a1___4 := l_aa + 0.001
						l_ii := 0
						if l_even_odd = 0 then
							l_fa1 := seer (l_order, a_qq, l_a1___4, l_nterms)
						else
							l_fa1 := seor (l_order, a_qq, l_a1___4, l_nterms)
						end
						from
						until
							(l_break)
						loop
							l_break := False
							if l_even_odd = 0 then
								l_fa := seer (l_order, a_qq, l_aa, l_nterms)
							else
								l_fa := seor (l_order, a_qq, l_aa, l_nterms)
							end
							l_a2___3 := l_a1___4
							l_a1___4 := l_aa
							if l_fa = l_fa1 then
								a_result.item.err := 2.2204460492503131e-16
								l_break := True
							end
							if not (l_return or l_break) then
								l_aa := l_aa - (((l_aa - l_a2___3) / (l_fa - l_fa1)) * l_fa)
								l_dela := fabs (l_aa - l_a2___3)
								if l_dela < 1.e-18 then
									a_result.item.err := 2.2204460492503131e-16
									l_break := True
								end
							end
							if not (l_return or l_break) and then (l_ii > 40) then
								a_result.item.err := l_dela
								l_break := True
							end
							if not (l_return or l_break) then
								l_fa1 := l_fa
								l_ii := l_ii + 1
							end
						end
						l_break := False
						l_tmp___1 := fabs (l_aa - l_aa_orig)
						if not (l_tmp___1 > (3).to_double + (0.01 * l_order.to_double) * (fabs (l_aa_orig))) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 8 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if not (l_order > 10) then
							l_pos := -1 + 12 -- skip if block
						end
				when 7 then -- if:
						if not ((fabs (l_aa - l_aa_orig)) > 1.5 * l_order.to_double) then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if: | _L:
						l_counter := l_counter + 1
						if l_counter = l_maxcount then
							a_result.item.err := fabs (l_aa - l_aa_orig)
							l_break := True
						end
						if not (l_return or l_break or l_continue) then
							if l_aa > l_aa_orig then
								if l_dir = 1 then
									l_da := l_da / (2).to_double
								end
								l_dir := -1
							else
								if l_dir = -1 then
									l_da := l_da / (2).to_double
								end
								l_dir := 1
							end
							l_aa_approx := l_aa_approx + ((l_dir.to_double * l_da) * l_counter.to_double)
							l_aa := l_aa_approx
							l_continue := True
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else:
						l_break := True
				when 11 then -- if done | if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						l_break := True
				when 13 then -- if done
						l_pos := -1 + 2 -- back to loop condition
				when 14 then -- loop done
						l_break := False
						l_continue := False
						a_result.item.val := l_aa
						if l_counter = l_maxcount then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("Wrong characteristic Mathieu value"), ce_string ("mathieu_charv.c"), 621, 5)
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
				when 15 then -- if done
					if not l_return then
						Result := 0
					end
				when 16 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

end
