class
	P_SKEW_SOURCE_STATIC

inherit
	CE_UTILITIES

feature {ANY} -- public routines

	gsl_stats_long_double_skew (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_double_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_long_double_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_long_double_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_long_double_skew_m_sd (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := ((a_data[(l_i * a_stride).to_integer_32] - real_96_from_real_64 (a_mean)) / real_96_from_real_64 (a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_skew (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_skew_m_sd (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32] - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_float_skew (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_float_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_float_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_float_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_float_skew_m_sd (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_ulong_skew (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ulong_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_ulong_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_ulong_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_ulong_skew_m_sd (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_long_skew (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_long_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_long_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_long_skew_m_sd (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_uint_skew (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uint_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_uint_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_uint_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_uint_skew_m_sd (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_int_skew (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_int_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_int_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_int_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_int_skew_m_sd (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_ushort_skew (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ushort_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_ushort_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_ushort_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_ushort_skew_m_sd (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_short_skew (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_short_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_short_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_short_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_short_skew_m_sd (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_uchar_skew (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uchar_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_uchar_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_uchar_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_uchar_skew_m_sd (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

	gsl_stats_char_skew (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
			l_sd: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_char_mean (a_data, a_stride, a_n))
			l_sd := ((create {P_VARIANCE_SOURCE_STATIC}).gsl_stats_char_sd_m (a_data, a_stride, a_n, l_mean))
			Result := (gsl_stats_char_skew_m_sd (a_data, a_stride, a_n, l_mean, l_sd))
		end

	gsl_stats_char_skew_m_sd (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32; a_mean, a_sd: REAL_64): REAL_64
		local
			l_skew: REAL_96
			l_i: NATURAL_32
			l_x: REAL_96
		do
			l_skew := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_x := real_96_from_real_64 ((((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean) / a_sd))
				l_skew := l_skew + (((l_x * l_x) * l_x - l_skew) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_skew.truncated_to_real_64
		end

end
