class
	P_STEFFENSON_STATIC

inherit
	CE_UTILITIES

feature {ANY} -- public attributes

	steffenson_type: CE_STATIC [CE_ARRAY [PD_GSL_ROOT_FDFSOLVER_TYPE]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_GSL_ROOT_FDFSOLVER_TYPE]}.make_initialized (<<
					create {PD_GSL_ROOT_FDFSOLVER_TYPE}.make_initialized ([ce_string ("steffenson"), (create {PD_STEFFENSON_STATE_T}.make).structure_size.to_natural_32, (create {PD_GSL_FUNCTION_FDF_STRUCT_STATIC}).steffenson_init_address, steffenson_iterate_address])
				>>)
		end

	gsl_root_fdfsolver_steffenson: CE_STATIC [CE_POINTER [PD_GSL_ROOT_FDFSOLVER_TYPE]]
		once ("PROCESS")
			Result.item := steffenson_type.item
		end

feature {ANY} -- public routines

	steffenson_iterate (a_vstate: CE_POINTER [ANY]; a_fdf: CE_POINTER [PD_GSL_FUNCTION_FDF_STRUCT]; a_root: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_STEFFENSON_STATE_T]
			l_x_new: REAL_64
			l_f_new: CE_ARRAY [REAL_64]
			l_df_new: CE_ARRAY [REAL_64]
			l_x_1: REAL_64
			l_x: REAL_64
			l_u: REAL_64
			l_v: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_f_new.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_df_new.make (<<1>>)

			l_state := create {CE_CLASS_POINTER [PD_STEFFENSON_STATE_T]}.make_cast (a_vstate)
			l_x_1 := l_state.item.x_1
			l_x := l_state.item.x
			if l_state.item.df = 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("derivative is zero"), ce_string ("steffenson.c"), 94, 12)
					Result := 12
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_x_new := l_x - l_state.item.f / l_state.item.df;
				(a_fdf.item.fdf).call ([l_x_new, a_fdf.item.params, l_f_new, l_df_new])
				l_state.item.x_2 := l_x_1
				l_state.item.x_1 := l_x
				l_state.item.x := l_x_new
				l_state.item.f := l_f_new.item
				l_state.item.df := l_df_new.item
				if not ((create {P_INFNAN_STATIC}).gsl_finite (l_f_new.item)).to_boolean then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("function value is not finite"), ce_string ("steffenson.c"), 110, 9)
						Result := 9
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				if l_state.item.count < 3 then
					a_root.item := l_x_new
					l_state.item.count := (l_state.item.count) + 1
				else
					l_u := l_x - l_x_1
					l_v := (l_x_new - (2).to_double * l_x) + l_x_1
					if l_v = (0).to_double then
						a_root.item := l_x_new
					else
						a_root.item := l_x_1 - (l_u * l_u) / l_v
					end
				end
				if not ((create {P_INFNAN_STATIC}).gsl_finite (l_df_new.item)).to_boolean then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("derivative value is not finite"), ce_string ("steffenson.c"), 131, 9)
						Result := 9
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				Result := 0
			end
		end

feature {ANY} -- routine addresses

	steffenson_iterate_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent steffenson_iterate)
		end

end
