class
	PD_GEAR1_STATE_T

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} k.make
			create {CE_REAL_64_POINTER} y0.make
			create {CE_REAL_64_POINTER} y0_orig.make
			create {CE_REAL_64_POINTER} y_onestep.make
		end

feature {ANY} -- fields

	k: CE_POINTER [REAL_64] assign set_k

	y0: CE_POINTER [REAL_64] assign set_y0

	y0_orig: CE_POINTER [REAL_64] assign set_y0_orig

	y_onestep: CE_POINTER [REAL_64] assign set_y_onestep

feature {ANY} -- field setters

	set_k (a_k: CE_POINTER [REAL_64])
		do
			k := a_k
			update_memory_field ("k")
		end

	set_y0 (a_y0: CE_POINTER [REAL_64])
		do
			y0 := a_y0
			update_memory_field ("y0")
		end

	set_y0_orig (a_y0_orig: CE_POINTER [REAL_64])
		do
			y0_orig := a_y0_orig
			update_memory_field ("y0_orig")
		end

	set_y_onestep (a_y_onestep: CE_POINTER [REAL_64])
		do
			y_onestep := a_y_onestep
			update_memory_field ("y_onestep")
		end

feature {ANY} -- public routines

	gear1_step (a_y: CE_POINTER [REAL_64]; a_h, a_t: REAL_64; a_dim: NATURAL_32; a_sys: CE_POINTER [PD_GSL_ODEIV_SYSTEM]): INTEGER_32
		local
			l_iter_steps: INTEGER_32
			l_nu: INTEGER_32
			l_i: NATURAL_32
			l_y0___0: CE_POINTER [REAL_64]
			l_k: CE_POINTER [REAL_64]
			l_s: INTEGER_32
			l_tmp: INTEGER_32
			l_return: BOOLEAN
		do
			l_iter_steps := 3
			l_y0___0 := y0
			l_k := k
			l_nu := 0
			from
			until
				not (not (l_return) and then (l_nu < l_iter_steps))
			loop
				l_tmp := (a_sys.item.function).integer_32_item ([a_t + a_h, a_y, l_k, a_sys.item.params])
				update_class_fields -- Current might have been changed; synchronize
				l_s := l_tmp
				if l_s /= 0 then
					Result := l_s
					l_return := True
				end
				if not l_return then
					l_i := (0).to_natural_32
					from
					until
						not (l_i < a_dim)
					loop
						a_y[l_i.to_integer_32] := l_y0___0[l_i.to_integer_32] + a_h * l_k[l_i.to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
					l_nu := l_nu + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			k := a_other.k
			y0 := a_other.y0
			y0_orig := a_other.y0_orig
			y_onestep := a_other.y_onestep
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				k.is_equal (a_other.k) and
				y0.is_equal (a_other.y0) and
				y0_orig.is_equal (a_other.y0_orig) and
				y_onestep.is_equal (a_other.y_onestep)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("k"),
					create {CE_STRUCTURE_FIELD}.make ("y0"),
					create {CE_STRUCTURE_FIELD}.make ("y0_orig"),
					create {CE_STRUCTURE_FIELD}.make ("y_onestep")
				>>)
			init_structure_info
		end

end
