class
	PD_GSL_CHEB_SERIES_STRUCT

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} c.make
			create {CE_REAL_64_POINTER} f.make
			Precursor
		end

feature {ANY} -- fields

	c: CE_POINTER [REAL_64] assign set_c

	order: NATURAL_32 assign set_order

	a: REAL_64 assign set_a

	b: REAL_64 assign set_b

	order_sp: NATURAL_32 assign set_order_sp

	f: CE_POINTER [REAL_64] assign set_f

feature {ANY} -- field setters

	set_c (a_c: CE_POINTER [REAL_64])
		do
			c := a_c
			update_memory_field ("c")
		end

	set_order (a_order: NATURAL_32)
		do
			order := a_order
			update_memory_field ("order")
		end

	set_a (a_a: REAL_64)
		do
			a := a_a
			update_memory_field ("a")
		end

	set_b (a_b: REAL_64)
		do
			b := a_b
			update_memory_field ("b")
		end

	set_order_sp (a_order_sp: NATURAL_32)
		do
			order_sp := a_order_sp
			update_memory_field ("order_sp")
		end

	set_f (a_f: CE_POINTER [REAL_64])
		do
			f := a_f
			update_memory_field ("f")
		end

feature {ANY} -- public routines

	gsl_cheb_eval (a_x: REAL_64): REAL_64
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_y: REAL_64
			l_y2: REAL_64
			l_temp: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			l_y := ((2.0 * a_x - a) - b) / (b - a)
			l_y2 := 2.0 * l_y
			l_i := order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			Result := ((l_y * l_d1 - l_d2) + 0.5 * c[0])
		end

	gsl_cheb_eval_n (a_n: NATURAL_32; a_x: REAL_64): REAL_64
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_eval_order: NATURAL_32
			l_tmp: NATURAL_32
			l_y: REAL_64
			l_y2: REAL_64
			l_temp: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			if a_n < order then
				l_tmp := a_n
			else
				l_tmp := order
			end
			l_eval_order := l_tmp
			l_y := ((2.0 * a_x - a) - b) / (b - a)
			l_y2 := 2.0 * l_y
			l_i := l_eval_order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			Result := ((l_y * l_d1 - l_d2) + 0.5 * c[0])
		end

	gsl_cheb_eval_err (a_x: REAL_64; a_result, a_abserr: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_result.is_void
			not a_abserr.is_void
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_y: REAL_64
			l_y2: REAL_64
			l_absc: REAL_64
			l_temp: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			l_y := ((2. * a_x - a) - b) / (b - a)
			l_y2 := 2.0 * l_y
			l_absc := 0.0
			l_i := order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			a_result.item := (l_y * l_d1 - l_d2) + 0.5 * c[0]
			update_class_fields -- Current might have been changed; synchronize
			l_i := (0).to_natural_32
			from
			until
				not (l_i <= order)
			loop
				l_tmp := fabs (c[l_i.to_integer_32])
				update_class_fields -- Current might have been changed; synchronize
				l_absc := l_absc + l_tmp
				l_i := l_i + 1
			end
			l_tmp___0 := fabs (c[order.to_integer_32])
			update_class_fields -- Current might have been changed; synchronize
			a_abserr.item := l_tmp___0 + l_absc * 2.2204460492503131e-16
			update_class_fields -- Current might have been changed; synchronize
			Result := 0
		end

	gsl_cheb_eval_n_err (a_n: NATURAL_32; a_x: REAL_64; a_result, a_abserr: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_result.is_void
			not a_abserr.is_void
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_y: REAL_64
			l_y2: REAL_64
			l_absc: REAL_64
			l_eval_order: NATURAL_32
			l_tmp: NATURAL_32
			l_temp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			l_y := ((2. * a_x - a) - b) / (b - a)
			l_y2 := 2.0 * l_y
			l_absc := 0.0
			if a_n < order then
				l_tmp := a_n
			else
				l_tmp := order
			end
			l_eval_order := l_tmp
			l_i := l_eval_order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			a_result.item := (l_y * l_d1 - l_d2) + 0.5 * c[0]
			update_class_fields -- Current might have been changed; synchronize
			l_i := (0).to_natural_32
			from
			until
				not (l_i <= l_eval_order)
			loop
				l_tmp___0 := fabs (c[l_i.to_integer_32])
				update_class_fields -- Current might have been changed; synchronize
				l_absc := l_absc + l_tmp___0
				l_i := l_i + 1
			end
			l_tmp___1 := fabs (c[l_eval_order.to_integer_32])
			update_class_fields -- Current might have been changed; synchronize
			a_abserr.item := l_tmp___1 + l_absc * 2.2204460492503131e-16
			update_class_fields -- Current might have been changed; synchronize
			Result := 0
		end

	gsl_cheb_eval_mode_e (a_x: REAL_64; a_mode: NATURAL_32; a_result, a_abserr: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_result.is_void
			not a_abserr.is_void
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_y: REAL_64
			l_y2: REAL_64
			l_absc: REAL_64
			l_eval_order: NATURAL_32
			l_tmp: NATURAL_32
			l_temp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			l_y := ((2. * a_x - a) - b) / (b - a)
			l_y2 := 2.0 * l_y
			l_absc := 0.0
			l_tmp := (create {P_GSL_GSL_MODE_STATIC}).GSL_MODE_PREC (a_mode)
			update_class_fields -- Current might have been changed; synchronize
			if l_tmp = {NATURAL_32} 0 then
				l_eval_order := order
			else
				l_eval_order := order_sp
			end
			l_i := l_eval_order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			a_result.item := (l_y * l_d1 - l_d2) + 0.5 * c[0]
			update_class_fields -- Current might have been changed; synchronize
			l_i := (0).to_natural_32
			from
			until
				not (l_i <= l_eval_order)
			loop
				l_tmp___0 := fabs (c[l_i.to_integer_32])
				update_class_fields -- Current might have been changed; synchronize
				l_absc := l_absc + l_tmp___0
				l_i := l_i + 1
			end
			l_tmp___1 := fabs (c[l_eval_order.to_integer_32])
			update_class_fields -- Current might have been changed; synchronize
			a_abserr.item := l_tmp___1 + l_absc * 2.2204460492503131e-16
			update_class_fields -- Current might have been changed; synchronize
			Result := 0
		end

	gsl_cheb_eval_mode (a_x: REAL_64; a_mode: NATURAL_32): REAL_64
		local
			l_result: CE_ARRAY [REAL_64]
			l_abserr: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_result.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_abserr.make (<<1>>)

			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_CHEB_SERIES_STRUCT]}.make_shared (c_pointer))).item.gsl_cheb_eval_mode_e (a_x, a_mode, l_result, l_abserr)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_cheb_eval_mode"), ce_string ("eval.c"), 200, l_status)
					update_class_fields -- Current might have been changed; synchronize
					Result := (l_result.item)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item)
			end
		end

	gsl_cheb_calc_integ (a_f: CE_POINTER [PD_GSL_CHEB_SERIES_STRUCT]): INTEGER_32
		require
			not a_f.is_void
		local
			l_n: NATURAL_32
			l_con: REAL_64
			l_sum: REAL_64
			l_fac: REAL_64
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := (a_f.item.order + {NATURAL_32} 1)
			l_con := 0.25 * (a_f.item.b - a_f.item.a)
			if order /= a_f.item.order then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("order of chebyshev series must be equal"), ce_string ("integ.c"), 33, 8)
					update_class_fields -- Current might have been changed; synchronize
					Result := 8
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Current.a := a_f.item.a
				update_class_fields -- Current might have been changed; synchronize
				Current.b := a_f.item.b
				update_class_fields -- Current might have been changed; synchronize
				if l_n = {NATURAL_32} 1 then
					c[0] := 0.
					update_class_fields -- Current might have been changed; synchronize
				elseif l_n = {NATURAL_32} 2 then
					c[1] := l_con * a_f.item.c[0]
					update_class_fields -- Current might have been changed; synchronize
					c[0] := 2.0 * c[1]
					update_class_fields -- Current might have been changed; synchronize
				else
					l_sum := 0.0
					l_fac := 1.0
					l_i := (1).to_natural_32
					from
					until
						not (l_i <= l_n - {NATURAL_32} 2)
					loop
						c[l_i.to_integer_32] := (l_con * (a_f.item.c[(l_i - {NATURAL_32} 1).to_integer_32] - a_f.item.c[(l_i + {NATURAL_32} 1).to_integer_32])) / l_i.to_real_64
						update_class_fields -- Current might have been changed; synchronize
						l_sum := l_sum + (l_fac * c[l_i.to_integer_32])
						l_fac := -l_fac
						l_i := l_i + 1
					end
					c[(l_n - {NATURAL_32} 1).to_integer_32] := (l_con * a_f.item.c[(l_n - {NATURAL_32} 2).to_integer_32]) / (l_n.to_real_64 - 1.0)
					update_class_fields -- Current might have been changed; synchronize
					l_sum := l_sum + (l_fac * c[(l_n - {NATURAL_32} 1).to_integer_32])
					c[0] := 2.0 * l_sum
					update_class_fields -- Current might have been changed; synchronize
				end
				Result := 0
			end
		end

	gsl_cheb_init (a_func: CE_POINTER [PD_GSL_FUNCTION_STRUCT]; a_a___5, a_b___0: REAL_64): INTEGER_32
		local
			l_k: NATURAL_32
			l_j: NATURAL_32
			l_bma: REAL_64
			l_bpa: REAL_64
			l_fac: REAL_64
			l_y: REAL_64
			l_tmp: REAL_64
			l_sum: REAL_64
			l_tmp___0: REAL_64
			l_return, l_break: BOOLEAN
		do
			if a_a___5 >= a_b___0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("null function interval [a,b]"), ce_string ("init.c"), 73, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Current.a := a_a___5
				update_class_fields -- Current might have been changed; synchronize
				Current.b := a_b___0
				update_class_fields -- Current might have been changed; synchronize
				l_bma := 0.5 * (b - a)
				l_bpa := 0.5 * (b + a)
				l_fac := 2.0 / (order.to_real_64 + 1.0)
				l_k := (0).to_natural_32
				from
				until
					not (l_k <= order)
				loop
					l_tmp := cos ((3.14159265358979323846 * (l_k.to_real_64 + 0.5)) / (order + {NATURAL_32} 1).to_real_64)
					update_class_fields -- Current might have been changed; synchronize
					l_y := l_tmp
					f[l_k.to_integer_32] := (a_func.item.function).real_64_item ([l_y * l_bma + l_bpa, a_func.item.params])
					update_class_fields -- Current might have been changed; synchronize
					l_k := l_k + 1
				end
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j <= order))
				loop
					l_sum := 0.0
					l_k := (0).to_natural_32
					from
					until
						not (l_k <= order)
					loop
						l_tmp___0 := cos (((3.14159265358979323846 * l_j.to_real_64) * (l_k.to_real_64 + 0.5)) / (order + {NATURAL_32} 1).to_real_64)
						update_class_fields -- Current might have been changed; synchronize
						l_sum := l_sum + (f[l_k.to_integer_32] * l_tmp___0)
						l_k := l_k + 1
					end
					c[l_j.to_integer_32] := l_fac * l_sum
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_cheb_order: NATURAL_32
		do
			Result := order
		end

	gsl_cheb_size: NATURAL_32
		do
			Result := (order + {NATURAL_32} 1)
		end

	gsl_cheb_coeffs: CE_POINTER [REAL_64]
		do
			Result := c
		end

	gsl_cheb_calc_deriv (a_f: CE_POINTER [PD_GSL_CHEB_SERIES_STRUCT]): INTEGER_32
		require
			not a_f.is_void
		local
			l_n: NATURAL_32
			l_con: REAL_64
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := (a_f.item.order + {NATURAL_32} 1)
			l_con := 2.0 / (a_f.item.b - a_f.item.a)
			if order /= a_f.item.order then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("order of chebyshev series must be equal"), ce_string ("deriv.c"), 34, 8)
					update_class_fields -- Current might have been changed; synchronize
					Result := 8
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Current.a := a_f.item.a
				update_class_fields -- Current might have been changed; synchronize
				Current.b := a_f.item.b
				update_class_fields -- Current might have been changed; synchronize
				c[(l_n - {NATURAL_32} 1).to_integer_32] := 0.0
				update_class_fields -- Current might have been changed; synchronize
				if l_n > {NATURAL_32} 1 then
					c[(l_n - {NATURAL_32} 2).to_integer_32] := (2.0 * (l_n.to_real_64 - 1.0)) * a_f.item.c[(l_n - {NATURAL_32} 1).to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_n - {NATURAL_32} 3
					from
					until
						not (l_i > {NATURAL_32} 0)
					loop
						c[l_i.to_integer_32] := c[(l_i + {NATURAL_32} 2).to_integer_32] + (2.0 * (l_i.to_real_64 + 1.0)) * a_f.item.c[(l_i + {NATURAL_32} 1).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i - 1
					end
					c[0] := c[2] + 2.0 * a_f.item.c[1]
					update_class_fields -- Current might have been changed; synchronize
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_n)
					loop
						c[l_i.to_integer_32] := c[l_i.to_integer_32] * l_con
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				end
				Result := 0
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			c := a_other.c
			order := a_other.order
			a := a_other.a
			b := a_other.b
			order_sp := a_other.order_sp
			f := a_other.f
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				c.is_equal (a_other.c) and
				order = a_other.order and
				a = a_other.a and
				b = a_other.b and
				order_sp = a_other.order_sp and
				f.is_equal (a_other.f)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("c"),
					create {CE_STRUCTURE_FIELD}.make ("order"),
					create {CE_STRUCTURE_FIELD}.make ("a"),
					create {CE_STRUCTURE_FIELD}.make ("b"),
					create {CE_STRUCTURE_FIELD}.make ("order_sp"),
					create {CE_STRUCTURE_FIELD}.make ("f")
				>>)
			init_structure_info
		end

end
