class
	PD_GSL_EIGEN_GEN_WORKSPACE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} work.make
			create {CE_CLASS_POINTER [PD_GSL_MATRIX]} H.make
			create {CE_CLASS_POINTER [PD_GSL_MATRIX]} R.make
			create {CE_CLASS_POINTER [PD_GSL_MATRIX]} Q.make
			create {CE_CLASS_POINTER [PD_GSL_MATRIX]} Z.make
		end

feature {ANY} -- fields

	size: NATURAL_32 assign set_size

	work: CE_POINTER [PD_GSL_VECTOR] assign set_work

	n_evals: NATURAL_32 assign set_n_evals

	max_iterations: NATURAL_32 assign set_max_iterations

	n_iter: NATURAL_32 assign set_n_iter

	eshift: REAL_64 assign set_eshift

	needtop: INTEGER_32 assign set_needtop

	atol: REAL_64 assign set_atol

	btol: REAL_64 assign set_btol

	ascale: REAL_64 assign set_ascale

	bscale: REAL_64 assign set_bscale

	H: CE_POINTER [PD_GSL_MATRIX] assign set_H

	R: CE_POINTER [PD_GSL_MATRIX] assign set_R

	compute_s: INTEGER_32 assign set_compute_s

	compute_t: INTEGER_32 assign set_compute_t

	Q: CE_POINTER [PD_GSL_MATRIX] assign set_Q

	Z: CE_POINTER [PD_GSL_MATRIX] assign set_Z

feature {ANY} -- field setters

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

	set_work (a_work: CE_POINTER [PD_GSL_VECTOR])
		do
			work := a_work
			update_memory_field ("work")
		end

	set_n_evals (a_n_evals: NATURAL_32)
		do
			n_evals := a_n_evals
			update_memory_field ("n_evals")
		end

	set_max_iterations (a_max_iterations: NATURAL_32)
		do
			max_iterations := a_max_iterations
			update_memory_field ("max_iterations")
		end

	set_n_iter (a_n_iter: NATURAL_32)
		do
			n_iter := a_n_iter
			update_memory_field ("n_iter")
		end

	set_eshift (a_eshift: REAL_64)
		do
			eshift := a_eshift
			update_memory_field ("eshift")
		end

	set_needtop (a_needtop: INTEGER_32)
		do
			needtop := a_needtop
			update_memory_field ("needtop")
		end

	set_atol (a_atol: REAL_64)
		do
			atol := a_atol
			update_memory_field ("atol")
		end

	set_btol (a_btol: REAL_64)
		do
			btol := a_btol
			update_memory_field ("btol")
		end

	set_ascale (a_ascale: REAL_64)
		do
			ascale := a_ascale
			update_memory_field ("ascale")
		end

	set_bscale (a_bscale: REAL_64)
		do
			bscale := a_bscale
			update_memory_field ("bscale")
		end

	set_H (a_H: CE_POINTER [PD_GSL_MATRIX])
		do
			H := a_H
			update_memory_field ("H")
		end

	set_R (a_R: CE_POINTER [PD_GSL_MATRIX])
		do
			R := a_R
			update_memory_field ("R")
		end

	set_compute_s (a_compute_s: INTEGER_32)
		do
			compute_s := a_compute_s
			update_memory_field ("compute_s")
		end

	set_compute_t (a_compute_t: INTEGER_32)
		do
			compute_t := a_compute_t
			update_memory_field ("compute_t")
		end

	set_Q (a_Q: CE_POINTER [PD_GSL_MATRIX])
		do
			Q := a_Q
			update_memory_field ("Q")
		end

	set_Z (a_Z: CE_POINTER [PD_GSL_MATRIX])
		do
			Z := a_Z
			update_memory_field ("Z")
		end

feature {ANY} -- public routines

	gsl_eigen_gen_params (a_compute_s, a_compute_t, a_balance: INTEGER_32)
		do
			Current.compute_s := a_compute_s
			update_class_fields -- Current might have been changed; synchronize
			Current.compute_t := a_compute_t
			update_class_fields -- Current might have been changed; synchronize
		end

feature {ANY}

	copy (a_other: like Current)
		do
			size := a_other.size
			work := a_other.work
			n_evals := a_other.n_evals
			max_iterations := a_other.max_iterations
			n_iter := a_other.n_iter
			eshift := a_other.eshift
			needtop := a_other.needtop
			atol := a_other.atol
			btol := a_other.btol
			ascale := a_other.ascale
			bscale := a_other.bscale
			H := a_other.H
			R := a_other.R
			compute_s := a_other.compute_s
			compute_t := a_other.compute_t
			Q := a_other.Q
			Z := a_other.Z
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				size = a_other.size and
				work.is_equal (a_other.work) and
				n_evals = a_other.n_evals and
				max_iterations = a_other.max_iterations and
				n_iter = a_other.n_iter and
				eshift = a_other.eshift and
				needtop = a_other.needtop and
				atol = a_other.atol and
				btol = a_other.btol and
				ascale = a_other.ascale and
				bscale = a_other.bscale and
				H.is_equal (a_other.H) and
				R.is_equal (a_other.R) and
				compute_s = a_other.compute_s and
				compute_t = a_other.compute_t and
				Q.is_equal (a_other.Q) and
				Z.is_equal (a_other.Z)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("size"),
					create {CE_STRUCTURE_FIELD}.make ("work"),
					create {CE_STRUCTURE_FIELD}.make ("n_evals"),
					create {CE_STRUCTURE_FIELD}.make ("max_iterations"),
					create {CE_STRUCTURE_FIELD}.make ("n_iter"),
					create {CE_STRUCTURE_FIELD}.make ("eshift"),
					create {CE_STRUCTURE_FIELD}.make ("needtop"),
					create {CE_STRUCTURE_FIELD}.make ("atol"),
					create {CE_STRUCTURE_FIELD}.make ("btol"),
					create {CE_STRUCTURE_FIELD}.make ("ascale"),
					create {CE_STRUCTURE_FIELD}.make ("bscale"),
					create {CE_STRUCTURE_FIELD}.make ("H"),
					create {CE_STRUCTURE_FIELD}.make ("R"),
					create {CE_STRUCTURE_FIELD}.make ("compute_s"),
					create {CE_STRUCTURE_FIELD}.make ("compute_t"),
					create {CE_STRUCTURE_FIELD}.make ("Q"),
					create {CE_STRUCTURE_FIELD}.make ("Z")
				>>)
			init_structure_info
		end

end
