class
	PD_GSL_FFT_COMPLEX_WAVETABLE_FLOAT

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_NATURAL_32_ARRAY} factor.make (<<64>>)
			create {CE_POINTER_ARRAY [PD_GSL_COMPLEX_FLOAT]} twiddle.make (<<64>>)
			create {CE_CLASS_POINTER [PD_GSL_COMPLEX_FLOAT]} trig.make
			Precursor
		end

feature {ANY} -- fields

	n: NATURAL_32 assign set_n

	nf: NATURAL_32 assign set_nf

	factor: CE_ARRAY [NATURAL_32]

	twiddle: CE_ARRAY [CE_POINTER [PD_GSL_COMPLEX_FLOAT]]

	trig: CE_POINTER [PD_GSL_COMPLEX_FLOAT] assign set_trig

feature {ANY} -- field setters

	set_n (a_n: NATURAL_32)
		do
			n := a_n
			update_memory_field ("n")
		end

	set_nf (a_nf: NATURAL_32)
		do
			nf := a_nf
			update_memory_field ("nf")
		end

	set_trig (a_trig: CE_POINTER [PD_GSL_COMPLEX_FLOAT])
		do
			trig := a_trig
			update_memory_field ("trig")
		end

feature {ANY} -- public routines

	gsl_fft_complex_float_memcpy (a_src: CE_POINTER [PD_GSL_FFT_COMPLEX_WAVETABLE_FLOAT]): INTEGER_32
		require
			not a_src.is_void
		local
			l_i: INTEGER_32
			l_n: INTEGER_32
			l_nf: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if n /= a_src.item.n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of src and dest do not match"), ce_string ("c_init.c"), 181, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_n := n.to_integer_32
				l_nf := nf.to_integer_32
				memcpy (create {CE_VOID_POINTER}.make_cast (trig), create {CE_VOID_POINTER}.make_cast (a_src.item.trig), l_n.to_natural_32 * {PLATFORM}.Real_64_bytes.to_natural_32).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_i := 0
				from
				until
					not (l_i < l_nf)
				loop
					twiddle[l_i] := trig + ((a_src.item.twiddle[l_i]).distance ((a_src.item.trig)))
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_fft_complex_float_transform (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_work: CE_POINTER [PD_GSL_FFT_COMPLEX_WORKSPACE_FLOAT]; a_sign: INTEGER_32): INTEGER_32
		require
			not a_work.is_void
		local
			l_nf: NATURAL_32
			l_i: NATURAL_32
			l_q___3: NATURAL_32
			l_product: NATURAL_32
			l_twiddle1: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_twiddle2: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_twiddle3: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_twiddle4: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_twiddle5: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_twiddle6: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_state: NATURAL_32
			l_scratch: CE_POINTER [REAL_32]
			l_in: CE_POINTER [REAL_32]
			l_istride: NATURAL_32
			l_out: CE_POINTER [REAL_32]
			l_ostride: NATURAL_32
			l_factor: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_nf := nf
			l_product := (1).to_natural_32
			l_state := (0).to_natural_32
			l_scratch := a_work.item.scratch
			l_in := a_data
			l_istride := a_stride
			l_out := l_scratch
			l_ostride := (1).to_natural_32
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length n must be positive integer"), ce_string ("c_main.c"), 107, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_n = {NATURAL_32} 1) then
				Result := 0
				l_return := True
			end
			if not l_return and then (a_n /= n) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("wavetable does not match length of data"), ce_string ("c_main.c"), 117, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_n /= a_work.item.n) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("workspace does not match length of data"), ce_string ("c_main.c"), 122, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nf)
				loop
					l_factor := factor[l_i.to_integer_32]
					l_product := l_product * l_factor
					l_q___3 := a_n // l_product
					if l_state = {NATURAL_32} 0 then
						l_in := a_data
						l_istride := a_stride
						l_out := l_scratch
						l_ostride := (1).to_natural_32
						l_state := (1).to_natural_32
					else
						l_in := l_scratch
						l_istride := (1).to_natural_32
						l_out := a_data
						l_ostride := a_stride
						l_state := (0).to_natural_32
					end
					if l_factor = {NATURAL_32} 2 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						fft_complex_float_pass_2 (l_in, l_istride, l_out, l_ostride, a_sign, l_product, a_n, l_twiddle1).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					elseif l_factor = {NATURAL_32} 3 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						l_twiddle2 := l_twiddle1 + l_q___3.to_integer_32
						fft_complex_float_pass_3 (l_in, l_istride, l_out, l_ostride, a_sign, l_product, a_n, l_twiddle1, l_twiddle2).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					elseif l_factor = {NATURAL_32} 4 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						l_twiddle2 := l_twiddle1 + l_q___3.to_integer_32
						l_twiddle3 := l_twiddle2 + l_q___3.to_integer_32
						fft_complex_float_pass_4 (l_in, l_istride, l_out, l_ostride, a_sign, l_product, a_n, l_twiddle1, l_twiddle2, l_twiddle3).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					elseif l_factor = {NATURAL_32} 5 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						l_twiddle2 := l_twiddle1 + l_q___3.to_integer_32
						l_twiddle3 := l_twiddle2 + l_q___3.to_integer_32
						l_twiddle4 := l_twiddle3 + l_q___3.to_integer_32
						fft_complex_float_pass_5 (l_in, l_istride, l_out, l_ostride, a_sign, l_product, a_n, l_twiddle1, l_twiddle2, l_twiddle3, l_twiddle4).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					elseif l_factor = {NATURAL_32} 6 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						l_twiddle2 := l_twiddle1 + l_q___3.to_integer_32
						l_twiddle3 := l_twiddle2 + l_q___3.to_integer_32
						l_twiddle4 := l_twiddle3 + l_q___3.to_integer_32
						l_twiddle5 := l_twiddle4 + l_q___3.to_integer_32
						fft_complex_float_pass_6 (l_in, l_istride, l_out, l_ostride, a_sign, l_product, a_n, l_twiddle1, l_twiddle2, l_twiddle3, l_twiddle4, l_twiddle5).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					elseif l_factor = {NATURAL_32} 7 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						l_twiddle2 := l_twiddle1 + l_q___3.to_integer_32
						l_twiddle3 := l_twiddle2 + l_q___3.to_integer_32
						l_twiddle4 := l_twiddle3 + l_q___3.to_integer_32
						l_twiddle5 := l_twiddle4 + l_q___3.to_integer_32
						l_twiddle6 := l_twiddle5 + l_q___3.to_integer_32
						fft_complex_float_pass_7 (l_in, l_istride, l_out, l_ostride, a_sign, l_product, a_n, l_twiddle1, l_twiddle2, l_twiddle3, l_twiddle4, l_twiddle5, l_twiddle6).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					else
						l_twiddle1 := twiddle[l_i.to_integer_32]
						fft_complex_float_pass_n (l_in, l_istride, l_out, l_ostride, a_sign, l_factor, l_product, a_n, l_twiddle1).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					l_i := l_i + 1
				end
				if l_state = {NATURAL_32} 1 then
					l_i := (0).to_natural_32
					from
					until
						not (l_i < a_n)
					loop
						a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := l_scratch[({NATURAL_32} 2 * l_i).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := l_scratch[({NATURAL_32} 2 * l_i + {NATURAL_32} 1).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				end
				Result := 0
			end
		end

	gsl_fft_complex_float_forward (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_work: CE_POINTER [PD_GSL_FFT_COMPLEX_WORKSPACE_FLOAT]): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_sign := -1
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_FFT_COMPLEX_WAVETABLE_FLOAT]}.make_shared (c_pointer))).item.gsl_fft_complex_float_transform (a_data, a_stride, a_n, a_work, l_sign)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_fft_complex_float_backward (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_work: CE_POINTER [PD_GSL_FFT_COMPLEX_WORKSPACE_FLOAT]): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_FFT_COMPLEX_WAVETABLE_FLOAT]}.make_shared (c_pointer))).item.gsl_fft_complex_float_transform (a_data, a_stride, a_n, a_work, l_sign)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_fft_complex_float_inverse (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_work: CE_POINTER [PD_GSL_FFT_COMPLEX_WORKSPACE_FLOAT]): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_norm___0: REAL_32
			l_i: NATURAL_32
			l_return: BOOLEAN
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_FFT_COMPLEX_WAVETABLE_FLOAT]}.make_shared (c_pointer))).item.gsl_fft_complex_float_transform (a_data, a_stride, a_n, a_work, l_sign)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_norm___0 := {REAL_32} 1.0 / a_n.to_real_32
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] * l_norm___0
					update_class_fields -- Current might have been changed; synchronize
					a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := a_data[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] * l_norm___0
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := l_status
			end
		end

feature {PD_GSL_FFT_COMPLEX_WAVETABLE_FLOAT} -- private routines

	fft_complex_float_pass_2 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride: NATURAL_32; a_sign: INTEGER_32; a_product, a_n: NATURAL_32; a_twiddle: CE_POINTER [PD_GSL_COMPLEX_FLOAT]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_factor: NATURAL_32
			l_m___4: NATURAL_32
			l_q___3: NATURAL_32
			l_product_1: NATURAL_32
			l_jump: NATURAL_32
			l_w_real: REAL_32
			l_w_imag: REAL_32
			l_z0_real: REAL_32
			l_z0_imag: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_x0_real: REAL_32
			l_x0_imag: REAL_32
			l_x1_real: REAL_32
			l_x1_imag: REAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (2).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_product_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_product_1
			l_k := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_k < l_q___3))
			loop
				if l_k = {NATURAL_32} 0 then
					l_w_real := (1.0).truncated_to_real
					l_w_imag := (0.0).truncated_to_real
				elseif a_sign = -1 then
					l_w_real := (a_twiddle + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w_imag := (a_twiddle + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				else
					l_w_real := (a_twiddle + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w_imag := -(a_twiddle + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				end
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_product_1)
				loop
					l_z0_real := a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32]
					l_z0_imag := a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32]
					l_z1_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4)).to_integer_32]
					l_z1_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_x0_real := l_z0_real + l_z1_real
					l_x0_imag := l_z0_imag + l_z1_imag
					l_x1_real := l_z0_real - l_z1_real
					l_x1_imag := l_z0_imag - l_z1_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j).to_integer_32] := l_x0_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j + {NATURAL_32} 1).to_integer_32] := l_x0_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_product_1)).to_integer_32] := l_w_real * l_x1_real - l_w_imag * l_x1_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_product_1) + {NATURAL_32} 1).to_integer_32] := l_w_real * l_x1_imag + l_w_imag * l_x1_real
					l_i := l_i + 1
					l_j := l_j + 1
					l_k1 := l_k1 + 1
				end
				l_j := l_j + l_jump
				l_k := l_k + 1
			end
			Result := 0
		end

	fft_complex_float_pass_3 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride: NATURAL_32; a_sign: INTEGER_32; a_product, a_n: NATURAL_32; a_twiddle1, a_twiddle2: CE_POINTER [PD_GSL_COMPLEX_FLOAT]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_factor: NATURAL_32
			l_m___4: NATURAL_32
			l_q___3: NATURAL_32
			l_product_1: NATURAL_32
			l_jump: NATURAL_32
			l_tau: REAL_32
			l_w1_real: REAL_32
			l_w1_imag: REAL_32
			l_w2_real: REAL_32
			l_w2_imag: REAL_32
			l_z0_real: REAL_32
			l_z0_imag: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_z2_real: REAL_32
			l_z2_imag: REAL_32
			l_t1_real: REAL_32
			l_t1_imag: REAL_32
			l_t2_real: REAL_32
			l_t2_imag: REAL_32
			l_t3_real: REAL_32
			l_t3_imag: REAL_32
			l_x0_real: REAL_32
			l_x0_imag: REAL_32
			l_x1_real: REAL_32
			l_x1_imag: REAL_32
			l_x2_real: REAL_32
			l_x2_imag: REAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (3).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_product_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_product_1
			l_tau := ((sqrt (3.0)) / 2.0).truncated_to_real
			l_k := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_k < l_q___3))
			loop
				if l_k = {NATURAL_32} 0 then
					l_w1_real := (1.0).truncated_to_real
					l_w1_imag := (0.0).truncated_to_real
					l_w2_real := (1.0).truncated_to_real
					l_w2_imag := (0.0).truncated_to_real
				elseif a_sign = -1 then
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				else
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := -(a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := -(a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				end
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_product_1)
				loop
					l_z0_real := a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32]
					l_z0_imag := a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32]
					l_z1_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4)).to_integer_32]
					l_z1_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z2_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4)).to_integer_32]
					l_z2_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_t1_real := l_z1_real + l_z2_real
					l_t1_imag := l_z1_imag + l_z2_imag
					l_t2_real := (l_z0_real.to_double - l_t1_real.to_double / 2.0).truncated_to_real
					l_t2_imag := (l_z0_imag.to_double - l_t1_imag.to_double / 2.0).truncated_to_real
					l_t3_real := (a_sign.to_real * l_tau) * (l_z1_real - l_z2_real)
					l_t3_imag := (a_sign.to_real * l_tau) * (l_z1_imag - l_z2_imag)
					l_x0_real := l_z0_real + l_t1_real
					l_x0_imag := l_z0_imag + l_t1_imag
					l_x1_real := l_t2_real - l_t3_imag
					l_x1_imag := l_t2_imag + l_t3_real
					l_x2_real := l_t2_real + l_t3_imag
					l_x2_imag := l_t2_imag - l_t3_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j).to_integer_32] := l_x0_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j + {NATURAL_32} 1).to_integer_32] := l_x0_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_product_1)).to_integer_32] := l_w1_real * l_x1_real - l_w1_imag * l_x1_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_product_1) + {NATURAL_32} 1).to_integer_32] := l_w1_real * l_x1_imag + l_w1_imag * l_x1_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_product_1)).to_integer_32] := l_w2_real * l_x2_real - l_w2_imag * l_x2_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_product_1) + {NATURAL_32} 1).to_integer_32] := l_w2_real * l_x2_imag + l_w2_imag * l_x2_real
					l_i := l_i + 1
					l_j := l_j + 1
					l_k1 := l_k1 + 1
				end
				l_j := l_j + l_jump
				l_k := l_k + 1
			end
			Result := 0
		end

	fft_complex_float_pass_4 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride: NATURAL_32; a_sign: INTEGER_32; a_product, a_n: NATURAL_32; a_twiddle1, a_twiddle2, a_twiddle3: CE_POINTER [PD_GSL_COMPLEX_FLOAT]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_factor: NATURAL_32
			l_m___4: NATURAL_32
			l_q___3: NATURAL_32
			l_p_1: NATURAL_32
			l_jump: NATURAL_32
			l_w1_real: REAL_32
			l_w1_imag: REAL_32
			l_w2_real: REAL_32
			l_w2_imag: REAL_32
			l_w3_real: REAL_32
			l_w3_imag: REAL_32
			l_z0_real: REAL_32
			l_z0_imag: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_z2_real: REAL_32
			l_z2_imag: REAL_32
			l_z3_real: REAL_32
			l_z3_imag: REAL_32
			l_t1_real: REAL_32
			l_t1_imag: REAL_32
			l_t2_real: REAL_32
			l_t2_imag: REAL_32
			l_t3_real: REAL_32
			l_t3_imag: REAL_32
			l_t4_real: REAL_32
			l_t4_imag: REAL_32
			l_x0_real: REAL_32
			l_x0_imag: REAL_32
			l_x1_real: REAL_32
			l_x1_imag: REAL_32
			l_x2_real: REAL_32
			l_x2_imag: REAL_32
			l_x3_real: REAL_32
			l_x3_imag: REAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (4).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_p_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_p_1
			l_k := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_k < l_q___3))
			loop
				if l_k = {NATURAL_32} 0 then
					l_w1_real := (1.0).truncated_to_real
					l_w1_imag := (0.0).truncated_to_real
					l_w2_real := (1.0).truncated_to_real
					l_w2_imag := (0.0).truncated_to_real
					l_w3_real := (1.0).truncated_to_real
					l_w3_imag := (0.0).truncated_to_real
				elseif a_sign = -1 then
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				else
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := -(a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := -(a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := -(a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				end
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_p_1)
				loop
					l_z0_real := a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32]
					l_z0_imag := a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32]
					l_z1_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4)).to_integer_32]
					l_z1_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z2_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4)).to_integer_32]
					l_z2_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z3_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 3 * l_m___4)).to_integer_32]
					l_z3_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 3 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_t1_real := l_z0_real + l_z2_real
					l_t1_imag := l_z0_imag + l_z2_imag
					l_t2_real := l_z1_real + l_z3_real
					l_t2_imag := l_z1_imag + l_z3_imag
					l_t3_real := l_z0_real - l_z2_real
					l_t3_imag := l_z0_imag - l_z2_imag
					l_t4_real := a_sign.to_real * (l_z1_real - l_z3_real)
					l_t4_imag := a_sign.to_real * (l_z1_imag - l_z3_imag)
					l_x0_real := l_t1_real + l_t2_real
					l_x0_imag := l_t1_imag + l_t2_imag
					l_x1_real := l_t3_real - l_t4_imag
					l_x1_imag := l_t3_imag + l_t4_real
					l_x2_real := l_t1_real - l_t2_real
					l_x2_imag := l_t1_imag - l_t2_imag
					l_x3_real := l_t3_real + l_t4_imag
					l_x3_imag := l_t3_imag - l_t4_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j).to_integer_32] := l_x0_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j + {NATURAL_32} 1).to_integer_32] := l_x0_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_p_1)).to_integer_32] := l_w1_real * l_x1_real - l_w1_imag * l_x1_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w1_real * l_x1_imag + l_w1_imag * l_x1_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_p_1)).to_integer_32] := l_w2_real * l_x2_real - l_w2_imag * l_x2_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w2_real * l_x2_imag + l_w2_imag * l_x2_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 3 * l_p_1)).to_integer_32] := l_w3_real * l_x3_real - l_w3_imag * l_x3_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 3 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w3_real * l_x3_imag + l_w3_imag * l_x3_real
					l_i := l_i + 1
					l_j := l_j + 1
					l_k1 := l_k1 + 1
				end
				l_j := l_j + l_jump
				l_k := l_k + 1
			end
			Result := 0
		end

	fft_complex_float_pass_5 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride: NATURAL_32; a_sign: INTEGER_32; a_product, a_n: NATURAL_32; a_twiddle1, a_twiddle2, a_twiddle3, a_twiddle4: CE_POINTER [PD_GSL_COMPLEX_FLOAT]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_factor: NATURAL_32
			l_m___4: NATURAL_32
			l_q___3: NATURAL_32
			l_p_1: NATURAL_32
			l_jump: NATURAL_32
			l_sin_2pi_by_5: REAL_32
			l_sin_2pi_by_10: REAL_32
			l_w1_real: REAL_32
			l_w1_imag: REAL_32
			l_w2_real: REAL_32
			l_w2_imag: REAL_32
			l_w3_real: REAL_32
			l_w3_imag: REAL_32
			l_w4_real: REAL_32
			l_w4_imag: REAL_32
			l_x0_real: REAL_32
			l_x0_imag: REAL_32
			l_x1_real: REAL_32
			l_x1_imag: REAL_32
			l_x2_real: REAL_32
			l_x2_imag: REAL_32
			l_x3_real: REAL_32
			l_x3_imag: REAL_32
			l_x4_real: REAL_32
			l_x4_imag: REAL_32
			l_z0_real: REAL_32
			l_z0_imag: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_z2_real: REAL_32
			l_z2_imag: REAL_32
			l_z3_real: REAL_32
			l_z3_imag: REAL_32
			l_z4_real: REAL_32
			l_z4_imag: REAL_32
			l_t1_real: REAL_32
			l_t1_imag: REAL_32
			l_t2_real: REAL_32
			l_t2_imag: REAL_32
			l_t3_real: REAL_32
			l_t3_imag: REAL_32
			l_t4_real: REAL_32
			l_t4_imag: REAL_32
			l_t5_real: REAL_32
			l_t5_imag: REAL_32
			l_t6_real: REAL_32
			l_t6_imag: REAL_32
			l_t7_real: REAL_32
			l_t7_imag: REAL_32
			l_t8_real: REAL_32
			l_t8_imag: REAL_32
			l_t9_real: REAL_32
			l_t9_imag: REAL_32
			l_t10_real: REAL_32
			l_t10_imag: REAL_32
			l_t11_real: REAL_32
			l_t11_imag: REAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (5).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_p_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_p_1
			l_sin_2pi_by_5 := (sin ((2.0 * 3.14159265358979323846) / 5.0)).truncated_to_real
			l_sin_2pi_by_10 := (sin ((2.0 * 3.14159265358979323846) / 10.0)).truncated_to_real
			l_k := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_k < l_q___3))
			loop
				if l_k = {NATURAL_32} 0 then
					l_w1_real := (1.0).truncated_to_real
					l_w1_imag := (0.0).truncated_to_real
					l_w2_real := (1.0).truncated_to_real
					l_w2_imag := (0.0).truncated_to_real
					l_w3_real := (1.0).truncated_to_real
					l_w3_imag := (0.0).truncated_to_real
					l_w4_real := (1.0).truncated_to_real
					l_w4_imag := (0.0).truncated_to_real
				elseif a_sign = -1 then
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w4_real := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w4_imag := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				else
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := -(a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := -(a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := -(a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w4_real := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w4_imag := -(a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				end
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_p_1)
				loop
					l_z0_real := a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32]
					l_z0_imag := a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32]
					l_z1_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4)).to_integer_32]
					l_z1_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z2_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4)).to_integer_32]
					l_z2_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z3_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 3 * l_m___4)).to_integer_32]
					l_z3_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 3 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z4_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 4 * l_m___4)).to_integer_32]
					l_z4_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 4 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_t1_real := l_z1_real + l_z4_real
					l_t1_imag := l_z1_imag + l_z4_imag
					l_t2_real := l_z2_real + l_z3_real
					l_t2_imag := l_z2_imag + l_z3_imag
					l_t3_real := l_z1_real - l_z4_real
					l_t3_imag := l_z1_imag - l_z4_imag
					l_t4_real := l_z2_real - l_z3_real
					l_t4_imag := l_z2_imag - l_z3_imag
					l_t5_real := l_t1_real + l_t2_real
					l_t5_imag := l_t1_imag + l_t2_imag
					l_t6_real := (((sqrt (5.0)) / 4.0) * (l_t1_real - l_t2_real).to_double).truncated_to_real
					l_t6_imag := (((sqrt (5.0)) / 4.0) * (l_t1_imag - l_t2_imag).to_double).truncated_to_real
					l_t7_real := (l_z0_real.to_double - l_t5_real.to_double / 4.0).truncated_to_real
					l_t7_imag := (l_z0_imag.to_double - l_t5_imag.to_double / 4.0).truncated_to_real
					l_t8_real := l_t7_real + l_t6_real
					l_t8_imag := l_t7_imag + l_t6_imag
					l_t9_real := l_t7_real - l_t6_real
					l_t9_imag := l_t7_imag - l_t6_imag
					l_t10_real := a_sign.to_real * (l_sin_2pi_by_5 * l_t3_real + l_sin_2pi_by_10 * l_t4_real)
					l_t10_imag := a_sign.to_real * (l_sin_2pi_by_5 * l_t3_imag + l_sin_2pi_by_10 * l_t4_imag)
					l_t11_real := a_sign.to_real * (l_sin_2pi_by_10 * l_t3_real - l_sin_2pi_by_5 * l_t4_real)
					l_t11_imag := a_sign.to_real * (l_sin_2pi_by_10 * l_t3_imag - l_sin_2pi_by_5 * l_t4_imag)
					l_x0_real := l_z0_real + l_t5_real
					l_x0_imag := l_z0_imag + l_t5_imag
					l_x1_real := l_t8_real - l_t10_imag
					l_x1_imag := l_t8_imag + l_t10_real
					l_x2_real := l_t9_real - l_t11_imag
					l_x2_imag := l_t9_imag + l_t11_real
					l_x3_real := l_t9_real + l_t11_imag
					l_x3_imag := l_t9_imag - l_t11_real
					l_x4_real := l_t8_real + l_t10_imag
					l_x4_imag := l_t8_imag - l_t10_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j).to_integer_32] := l_x0_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j + {NATURAL_32} 1).to_integer_32] := l_x0_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_p_1)).to_integer_32] := l_w1_real * l_x1_real - l_w1_imag * l_x1_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w1_real * l_x1_imag + l_w1_imag * l_x1_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_p_1)).to_integer_32] := l_w2_real * l_x2_real - l_w2_imag * l_x2_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w2_real * l_x2_imag + l_w2_imag * l_x2_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 3 * l_p_1)).to_integer_32] := l_w3_real * l_x3_real - l_w3_imag * l_x3_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 3 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w3_real * l_x3_imag + l_w3_imag * l_x3_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 4 * l_p_1)).to_integer_32] := l_w4_real * l_x4_real - l_w4_imag * l_x4_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 4 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w4_real * l_x4_imag + l_w4_imag * l_x4_real
					l_i := l_i + 1
					l_j := l_j + 1
					l_k1 := l_k1 + 1
				end
				l_j := l_j + l_jump
				l_k := l_k + 1
			end
			Result := 0
		end

	fft_complex_float_pass_6 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride: NATURAL_32; a_sign: INTEGER_32; a_product, a_n: NATURAL_32; a_twiddle1, a_twiddle2, a_twiddle3, a_twiddle4, a_twiddle5: CE_POINTER [PD_GSL_COMPLEX_FLOAT]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_factor: NATURAL_32
			l_m___4: NATURAL_32
			l_q___3: NATURAL_32
			l_p_1: NATURAL_32
			l_jump: NATURAL_32
			l_tau: REAL_32
			l_w1_real: REAL_32
			l_w1_imag: REAL_32
			l_w2_real: REAL_32
			l_w2_imag: REAL_32
			l_w3_real: REAL_32
			l_w3_imag: REAL_32
			l_w4_real: REAL_32
			l_w4_imag: REAL_32
			l_w5_real: REAL_32
			l_w5_imag: REAL_32
			l_z0_real: REAL_32
			l_z0_imag: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_z2_real: REAL_32
			l_z2_imag: REAL_32
			l_z3_real: REAL_32
			l_z3_imag: REAL_32
			l_z4_real: REAL_32
			l_z4_imag: REAL_32
			l_z5_real: REAL_32
			l_z5_imag: REAL_32
			l_ta1_real: REAL_32
			l_ta1_imag: REAL_32
			l_ta2_real: REAL_32
			l_ta2_imag: REAL_32
			l_ta3_real: REAL_32
			l_ta3_imag: REAL_32
			l_a0_real: REAL_32
			l_a0_imag: REAL_32
			l_a1_real: REAL_32
			l_a1_imag: REAL_32
			l_a2_real: REAL_32
			l_a2_imag: REAL_32
			l_tb1_real: REAL_32
			l_tb1_imag: REAL_32
			l_tb2_real: REAL_32
			l_tb2_imag: REAL_32
			l_tb3_real: REAL_32
			l_tb3_imag: REAL_32
			l_b0_real: REAL_32
			l_b0_imag: REAL_32
			l_b1_real: REAL_32
			l_b1_imag: REAL_32
			l_b2_real: REAL_32
			l_b2_imag: REAL_32
			l_x0_real: REAL_32
			l_x0_imag: REAL_32
			l_x4_real: REAL_32
			l_x4_imag: REAL_32
			l_x2_real: REAL_32
			l_x2_imag: REAL_32
			l_x3_real: REAL_32
			l_x3_imag: REAL_32
			l_x1_real: REAL_32
			l_x1_imag: REAL_32
			l_x5_real: REAL_32
			l_x5_imag: REAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (6).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_p_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_p_1
			l_tau := ((sqrt (3.0)) / 2.0).truncated_to_real
			l_k := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_k < l_q___3))
			loop
				if l_k = {NATURAL_32} 0 then
					l_w1_real := (1.0).truncated_to_real
					l_w1_imag := (0.0).truncated_to_real
					l_w2_real := (1.0).truncated_to_real
					l_w2_imag := (0.0).truncated_to_real
					l_w3_real := (1.0).truncated_to_real
					l_w3_imag := (0.0).truncated_to_real
					l_w4_real := (1.0).truncated_to_real
					l_w4_imag := (0.0).truncated_to_real
					l_w5_real := (1.0).truncated_to_real
					l_w5_imag := (0.0).truncated_to_real
				elseif a_sign = -1 then
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w4_real := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w4_imag := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w5_real := (a_twiddle5 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w5_imag := (a_twiddle5 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				else
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := -(a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := -(a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := -(a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w4_real := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w4_imag := -(a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w5_real := (a_twiddle5 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w5_imag := -(a_twiddle5 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				end
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_p_1)
				loop
					l_z0_real := a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32]
					l_z0_imag := a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32]
					l_z1_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4)).to_integer_32]
					l_z1_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z2_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4)).to_integer_32]
					l_z2_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z3_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 3 * l_m___4)).to_integer_32]
					l_z3_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 3 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z4_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 4 * l_m___4)).to_integer_32]
					l_z4_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 4 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z5_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 5 * l_m___4)).to_integer_32]
					l_z5_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 5 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_ta1_real := l_z2_real + l_z4_real
					l_ta1_imag := l_z2_imag + l_z4_imag
					l_ta2_real := l_z0_real - l_ta1_real / (2).to_real
					l_ta2_imag := l_z0_imag - l_ta1_imag / (2).to_real
					l_ta3_real := (a_sign.to_real * l_tau) * (l_z2_real - l_z4_real)
					l_ta3_imag := (a_sign.to_real * l_tau) * (l_z2_imag - l_z4_imag)
					l_a0_real := l_z0_real + l_ta1_real
					l_a0_imag := l_z0_imag + l_ta1_imag
					l_a1_real := l_ta2_real - l_ta3_imag
					l_a1_imag := l_ta2_imag + l_ta3_real
					l_a2_real := l_ta2_real + l_ta3_imag
					l_a2_imag := l_ta2_imag - l_ta3_real
					l_tb1_real := l_z5_real + l_z1_real
					l_tb1_imag := l_z5_imag + l_z1_imag
					l_tb2_real := l_z3_real - l_tb1_real / (2).to_real
					l_tb2_imag := l_z3_imag - l_tb1_imag / (2).to_real
					l_tb3_real := (a_sign.to_real * l_tau) * (l_z5_real - l_z1_real)
					l_tb3_imag := (a_sign.to_real * l_tau) * (l_z5_imag - l_z1_imag)
					l_b0_real := l_z3_real + l_tb1_real
					l_b0_imag := l_z3_imag + l_tb1_imag
					l_b1_real := l_tb2_real - l_tb3_imag
					l_b1_imag := l_tb2_imag + l_tb3_real
					l_b2_real := l_tb2_real + l_tb3_imag
					l_b2_imag := l_tb2_imag - l_tb3_real
					l_x0_real := l_a0_real + l_b0_real
					l_x0_imag := l_a0_imag + l_b0_imag
					l_x4_real := l_a1_real + l_b1_real
					l_x4_imag := l_a1_imag + l_b1_imag
					l_x2_real := l_a2_real + l_b2_real
					l_x2_imag := l_a2_imag + l_b2_imag
					l_x3_real := l_a0_real - l_b0_real
					l_x3_imag := l_a0_imag - l_b0_imag
					l_x1_real := l_a1_real - l_b1_real
					l_x1_imag := l_a1_imag - l_b1_imag
					l_x5_real := l_a2_real - l_b2_real
					l_x5_imag := l_a2_imag - l_b2_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j).to_integer_32] := l_x0_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j + {NATURAL_32} 1).to_integer_32] := l_x0_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_p_1)).to_integer_32] := l_w1_real * l_x1_real - l_w1_imag * l_x1_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w1_real * l_x1_imag + l_w1_imag * l_x1_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_p_1)).to_integer_32] := l_w2_real * l_x2_real - l_w2_imag * l_x2_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w2_real * l_x2_imag + l_w2_imag * l_x2_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 3 * l_p_1)).to_integer_32] := l_w3_real * l_x3_real - l_w3_imag * l_x3_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 3 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w3_real * l_x3_imag + l_w3_imag * l_x3_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 4 * l_p_1)).to_integer_32] := l_w4_real * l_x4_real - l_w4_imag * l_x4_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 4 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w4_real * l_x4_imag + l_w4_imag * l_x4_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 5 * l_p_1)).to_integer_32] := l_w5_real * l_x5_real - l_w5_imag * l_x5_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 5 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w5_real * l_x5_imag + l_w5_imag * l_x5_real
					l_i := l_i + 1
					l_j := l_j + 1
					l_k1 := l_k1 + 1
				end
				l_j := l_j + l_jump
				l_k := l_k + 1
			end
			Result := 0
		end

	fft_complex_float_pass_7 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride: NATURAL_32; a_sign: INTEGER_32; a_product, a_n: NATURAL_32; a_twiddle1, a_twiddle2, a_twiddle3, a_twiddle4, a_twiddle5, a_twiddle6: CE_POINTER [PD_GSL_COMPLEX_FLOAT]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_factor: NATURAL_32
			l_m___4: NATURAL_32
			l_q___3: NATURAL_32
			l_p_1: NATURAL_32
			l_jump: NATURAL_32
			l_c1___2: REAL_32
			l_c2___0: REAL_32
			l_c3___3: REAL_32
			l_s1: REAL_32
			l_s2: REAL_32
			l_s3: REAL_32
			l_w1_real: REAL_32
			l_w1_imag: REAL_32
			l_w2_real: REAL_32
			l_w2_imag: REAL_32
			l_w3_real: REAL_32
			l_w3_imag: REAL_32
			l_w4_real: REAL_32
			l_w4_imag: REAL_32
			l_w5_real: REAL_32
			l_w5_imag: REAL_32
			l_w6_real: REAL_32
			l_w6_imag: REAL_32
			l_z0_real: REAL_32
			l_z0_imag: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_z2_real: REAL_32
			l_z2_imag: REAL_32
			l_z3_real: REAL_32
			l_z3_imag: REAL_32
			l_z4_real: REAL_32
			l_z4_imag: REAL_32
			l_z5_real: REAL_32
			l_z5_imag: REAL_32
			l_z6_real: REAL_32
			l_z6_imag: REAL_32
			l_t0_real: REAL_32
			l_t0_imag: REAL_32
			l_t1_real: REAL_32
			l_t1_imag: REAL_32
			l_t2_real: REAL_32
			l_t2_imag: REAL_32
			l_t3_real: REAL_32
			l_t3_imag: REAL_32
			l_t4_real: REAL_32
			l_t4_imag: REAL_32
			l_t5_real: REAL_32
			l_t5_imag: REAL_32
			l_t6_real: REAL_32
			l_t6_imag: REAL_32
			l_t7_real: REAL_32
			l_t7_imag: REAL_32
			l_b0_real: REAL_32
			l_b0_imag: REAL_32
			l_b1_real: REAL_32
			l_b1_imag: REAL_32
			l_b2_real: REAL_32
			l_b2_imag: REAL_32
			l_b3_real: REAL_32
			l_b3_imag: REAL_32
			l_b4_real: REAL_32
			l_b4_imag: REAL_32
			l_b5_real: REAL_32
			l_b5_imag: REAL_32
			l_b6_real: REAL_32
			l_b6_imag: REAL_32
			l_b7_real: REAL_32
			l_b7_imag: REAL_32
			l_b8_real: REAL_32
			l_b8_imag: REAL_32
			l_T0_real2: REAL_32
			l_T0_imag2: REAL_32
			l_T1_real2: REAL_32
			l_T1_imag2: REAL_32
			l_T2_real2: REAL_32
			l_T2_imag2: REAL_32
			l_T3_real2: REAL_32
			l_T3_imag2: REAL_32
			l_T4_real2: REAL_32
			l_T4_imag2: REAL_32
			l_T5_real2: REAL_32
			l_T5_imag2: REAL_32
			l_T6_real2: REAL_32
			l_T6_imag2: REAL_32
			l_T7_real2: REAL_32
			l_T7_imag2: REAL_32
			l_T8_real: REAL_32
			l_T8_imag: REAL_32
			l_T9_real: REAL_32
			l_T9_imag: REAL_32
			l_T10_real: REAL_32
			l_T10_imag: REAL_32
			l_T11_real: REAL_32
			l_T11_imag: REAL_32
			l_T12_real: REAL_32
			l_T12_imag: REAL_32
			l_x0_real: REAL_32
			l_x0_imag: REAL_32
			l_x1_real: REAL_32
			l_x1_imag: REAL_32
			l_x2_real: REAL_32
			l_x2_imag: REAL_32
			l_x3_real: REAL_32
			l_x3_imag: REAL_32
			l_x4_real: REAL_32
			l_x4_imag: REAL_32
			l_x5_real: REAL_32
			l_x5_imag: REAL_32
			l_x6_real: REAL_32
			l_x6_imag: REAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (7).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_p_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_p_1
			l_c1___2 := (cos (((1.0 * 2.0) * 3.14159265358979323846) / 7.0)).truncated_to_real
			l_c2___0 := (cos (((2.0 * 2.0) * 3.14159265358979323846) / 7.0)).truncated_to_real
			l_c3___3 := (cos (((3.0 * 2.0) * 3.14159265358979323846) / 7.0)).truncated_to_real
			l_s1 := (sin (((1.0 * 2.0) * 3.14159265358979323846) / 7.0)).truncated_to_real
			l_s2 := (sin (((2.0 * 2.0) * 3.14159265358979323846) / 7.0)).truncated_to_real
			l_s3 := (sin (((3.0 * 2.0) * 3.14159265358979323846) / 7.0)).truncated_to_real
			l_k := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_k < l_q___3))
			loop
				if l_k = {NATURAL_32} 0 then
					l_w1_real := (1.0).truncated_to_real
					l_w1_imag := (0.0).truncated_to_real
					l_w2_real := (1.0).truncated_to_real
					l_w2_imag := (0.0).truncated_to_real
					l_w3_real := (1.0).truncated_to_real
					l_w3_imag := (0.0).truncated_to_real
					l_w4_real := (1.0).truncated_to_real
					l_w4_imag := (0.0).truncated_to_real
					l_w5_real := (1.0).truncated_to_real
					l_w5_imag := (0.0).truncated_to_real
					l_w6_real := (1.0).truncated_to_real
					l_w6_imag := (0.0).truncated_to_real
				elseif a_sign = -1 then
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w4_real := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w4_imag := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w5_real := (a_twiddle5 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w5_imag := (a_twiddle5 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w6_real := (a_twiddle6 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w6_imag := (a_twiddle6 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				else
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := -(a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := -(a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := -(a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w4_real := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w4_imag := -(a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w5_real := (a_twiddle5 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w5_imag := -(a_twiddle5 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w6_real := (a_twiddle6 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w6_imag := -(a_twiddle6 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
				end
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_p_1)
				loop
					l_z0_real := a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32]
					l_z0_imag := a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32]
					l_z1_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4)).to_integer_32]
					l_z1_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z2_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4)).to_integer_32]
					l_z2_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 2 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z3_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 3 * l_m___4)).to_integer_32]
					l_z3_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 3 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z4_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 4 * l_m___4)).to_integer_32]
					l_z4_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 4 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z5_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 5 * l_m___4)).to_integer_32]
					l_z5_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 5 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_z6_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 6 * l_m___4)).to_integer_32]
					l_z6_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + {NATURAL_32} 6 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_t0_real := l_z1_real + l_z6_real
					l_t0_imag := l_z1_imag + l_z6_imag
					l_t1_real := l_z1_real - l_z6_real
					l_t1_imag := l_z1_imag - l_z6_imag
					l_t2_real := l_z2_real + l_z5_real
					l_t2_imag := l_z2_imag + l_z5_imag
					l_t3_real := l_z2_real - l_z5_real
					l_t3_imag := l_z2_imag - l_z5_imag
					l_t4_real := l_z4_real + l_z3_real
					l_t4_imag := l_z4_imag + l_z3_imag
					l_t5_real := l_z4_real - l_z3_real
					l_t5_imag := l_z4_imag - l_z3_imag
					l_t6_real := l_t2_real + l_t0_real
					l_t6_imag := l_t2_imag + l_t0_imag
					l_t7_real := l_t5_real + l_t3_real
					l_t7_imag := l_t5_imag + l_t3_imag
					l_b0_real := (l_z0_real + l_t6_real) + l_t4_real
					l_b0_imag := (l_z0_imag + l_t6_imag) + l_t4_imag
					l_b1_real := ((((l_c1___2 + l_c2___0) + l_c3___3).to_double / 3.0 - 1.0) * (l_t6_real + l_t4_real).to_double).truncated_to_real
					l_b1_imag := ((((l_c1___2 + l_c2___0) + l_c3___3).to_double / 3.0 - 1.0) * (l_t6_imag + l_t4_imag).to_double).truncated_to_real
					l_b2_real := ((((2.0 * l_c1___2.to_double - l_c2___0.to_double) - l_c3___3.to_double) / 3.0) * (l_t0_real - l_t4_real).to_double).truncated_to_real
					l_b2_imag := ((((2.0 * l_c1___2.to_double - l_c2___0.to_double) - l_c3___3.to_double) / 3.0) * (l_t0_imag - l_t4_imag).to_double).truncated_to_real
					l_b3_real := ((((l_c1___2.to_double - 2.0 * l_c2___0.to_double) + l_c3___3.to_double) / 3.0) * (l_t4_real - l_t2_real).to_double).truncated_to_real
					l_b3_imag := ((((l_c1___2.to_double - 2.0 * l_c2___0.to_double) + l_c3___3.to_double) / 3.0) * (l_t4_imag - l_t2_imag).to_double).truncated_to_real
					l_b4_real := ((((l_c1___2 + l_c2___0).to_double - 2.0 * l_c3___3.to_double) / 3.0) * (l_t2_real - l_t0_real).to_double).truncated_to_real
					l_b4_imag := ((((l_c1___2 + l_c2___0).to_double - 2.0 * l_c3___3.to_double) / 3.0) * (l_t2_imag - l_t0_imag).to_double).truncated_to_real
					l_b5_real := (((-a_sign).to_double * (((l_s1 + l_s2) - l_s3).to_double / 3.0)) * (l_t7_real + l_t1_real).to_double).truncated_to_real
					l_b5_imag := (((-a_sign).to_double * (((l_s1 + l_s2) - l_s3).to_double / 3.0)) * (l_t7_imag + l_t1_imag).to_double).truncated_to_real
					l_b6_real := (((-a_sign).to_double * (((2.0 * l_s1.to_double - l_s2.to_double) + l_s3.to_double) / 3.0)) * (l_t1_real - l_t5_real).to_double).truncated_to_real
					l_b6_imag := (((-a_sign).to_double * (((2.0 * l_s1.to_double - l_s2.to_double) + l_s3.to_double) / 3.0)) * (l_t1_imag - l_t5_imag).to_double).truncated_to_real
					l_b7_real := (((-a_sign).to_double * (((l_s1.to_double - 2.0 * l_s2.to_double) - l_s3.to_double) / 3.0)) * (l_t5_real - l_t3_real).to_double).truncated_to_real
					l_b7_imag := (((-a_sign).to_double * (((l_s1.to_double - 2.0 * l_s2.to_double) - l_s3.to_double) / 3.0)) * (l_t5_imag - l_t3_imag).to_double).truncated_to_real
					l_b8_real := (((-a_sign).to_double * (((l_s1 + l_s2).to_double + 2.0 * l_s3.to_double) / 3.0)) * (l_t3_real - l_t1_real).to_double).truncated_to_real
					l_b8_imag := (((-a_sign).to_double * (((l_s1 + l_s2).to_double + 2.0 * l_s3.to_double) / 3.0)) * (l_t3_imag - l_t1_imag).to_double).truncated_to_real
					l_T0_real2 := l_b0_real + l_b1_real
					l_T0_imag2 := l_b0_imag + l_b1_imag
					l_T1_real2 := l_b2_real + l_b3_real
					l_T1_imag2 := l_b2_imag + l_b3_imag
					l_T2_real2 := l_b4_real - l_b3_real
					l_T2_imag2 := l_b4_imag - l_b3_imag
					l_T3_real2 := -l_b2_real - l_b4_real
					l_T3_imag2 := -l_b2_imag - l_b4_imag
					l_T4_real2 := l_b6_real + l_b7_real
					l_T4_imag2 := l_b6_imag + l_b7_imag
					l_T5_real2 := l_b8_real - l_b7_real
					l_T5_imag2 := l_b8_imag - l_b7_imag
					l_T6_real2 := -l_b8_real - l_b6_real
					l_T6_imag2 := -l_b8_imag - l_b6_imag
					l_T7_real2 := l_T0_real2 + l_T1_real2
					l_T7_imag2 := l_T0_imag2 + l_T1_imag2
					l_T8_real := l_T0_real2 + l_T2_real2
					l_T8_imag := l_T0_imag2 + l_T2_imag2
					l_T9_real := l_T0_real2 + l_T3_real2
					l_T9_imag := l_T0_imag2 + l_T3_imag2
					l_T10_real := l_T4_real2 + l_b5_real
					l_T10_imag := l_T4_imag2 + l_b5_imag
					l_T11_real := l_T5_real2 + l_b5_real
					l_T11_imag := l_T5_imag2 + l_b5_imag
					l_T12_real := l_T6_real2 + l_b5_real
					l_T12_imag := l_T6_imag2 + l_b5_imag
					l_x0_real := l_b0_real
					l_x0_imag := l_b0_imag
					l_x1_real := l_T7_real2 + l_T10_imag
					l_x1_imag := l_T7_imag2 - l_T10_real
					l_x2_real := l_T9_real + l_T12_imag
					l_x2_imag := l_T9_imag - l_T12_real
					l_x3_real := l_T8_real - l_T11_imag
					l_x3_imag := l_T8_imag + l_T11_real
					l_x4_real := l_T8_real + l_T11_imag
					l_x4_imag := l_T8_imag - l_T11_real
					l_x5_real := l_T9_real - l_T12_imag
					l_x5_imag := l_T9_imag + l_T12_real
					l_x6_real := l_T7_real2 - l_T10_imag
					l_x6_imag := l_T7_imag2 + l_T10_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j).to_integer_32] := l_x0_real
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j + {NATURAL_32} 1).to_integer_32] := l_x0_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_p_1)).to_integer_32] := l_w1_real * l_x1_real - l_w1_imag * l_x1_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w1_real * l_x1_imag + l_w1_imag * l_x1_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_p_1)).to_integer_32] := l_w2_real * l_x2_real - l_w2_imag * l_x2_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 2 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w2_real * l_x2_imag + l_w2_imag * l_x2_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 3 * l_p_1)).to_integer_32] := l_w3_real * l_x3_real - l_w3_imag * l_x3_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 3 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w3_real * l_x3_imag + l_w3_imag * l_x3_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 4 * l_p_1)).to_integer_32] := l_w4_real * l_x4_real - l_w4_imag * l_x4_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 4 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w4_real * l_x4_imag + l_w4_imag * l_x4_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 5 * l_p_1)).to_integer_32] := l_w5_real * l_x5_real - l_w5_imag * l_x5_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 5 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w5_real * l_x5_imag + l_w5_imag * l_x5_real
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 6 * l_p_1)).to_integer_32] := l_w6_real * l_x6_real - l_w6_imag * l_x6_imag
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + {NATURAL_32} 6 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w6_real * l_x6_imag + l_w6_imag * l_x6_real
					l_i := l_i + 1
					l_j := l_j + 1
					l_k1 := l_k1 + 1
				end
				l_j := l_j + l_jump
				l_k := l_k + 1
			end
			Result := 0
		end

	fft_complex_float_pass_n (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride: NATURAL_32; a_sign: INTEGER_32; a_factor, a_product, a_n: NATURAL_32; a_twiddle: CE_POINTER [PD_GSL_COMPLEX_FLOAT]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_m___4: NATURAL_32
			l_q___3: NATURAL_32
			l_p_1: NATURAL_32
			l_jump: NATURAL_32
			l_e: NATURAL_32
			l_e1: NATURAL_32
			l_idx: NATURAL_32
			l_idxc: NATURAL_32
			l_idx___0: NATURAL_32
			l_idx_step: NATURAL_32
			l_w_real: REAL_32
			l_w_imag: REAL_32
			l_em: NATURAL_32
			l_ecm: NATURAL_32
			l_xp_real: REAL_32
			l_xp_imag: REAL_32
			l_xm_real: REAL_32
			l_xm_imag: REAL_32
			l_ap: REAL_32
			l_am: REAL_32
			l_sum_real: REAL_32
			l_sumc_real: REAL_32
			l_bp: REAL_32
			l_bm: REAL_32
			l_sum_imag: REAL_32
			l_sumc_imag: REAL_32
			l_x_real: REAL_32
			l_x_imag: REAL_32
			l_w_real___0: REAL_32
			l_w_imag___0: REAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_m___4 := a_n // a_factor
			l_q___3 := a_n // a_product
			l_p_1 := a_product // a_factor
			l_jump := (a_factor - {NATURAL_32} 1) * l_p_1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_m___4)
			loop
				a_out[(({NATURAL_32} 2 * a_ostride) * l_i).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32]
				a_out[(({NATURAL_32} 2 * a_ostride) * l_i + {NATURAL_32} 1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32]
				l_i := l_i + 1
			end
			l_e := (1).to_natural_32
			from
			until
				not (not (l_return) and then (l_e < (a_factor - {NATURAL_32} 1) // {NATURAL_32} 2 + {NATURAL_32} 1))
			loop
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_m___4)
				loop
					l_idx := l_i + l_e * l_m___4
					l_idxc := l_i + (a_factor - l_e) * l_m___4
					a_out[(({NATURAL_32} 2 * a_ostride) * l_idx).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_idx).to_integer_32] + a_in[(({NATURAL_32} 2 * a_istride) * l_idxc).to_integer_32]
					a_out[(({NATURAL_32} 2 * a_ostride) * l_idx + {NATURAL_32} 1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_idx + {NATURAL_32} 1).to_integer_32] + a_in[(({NATURAL_32} 2 * a_istride) * l_idxc + {NATURAL_32} 1).to_integer_32]
					a_out[(({NATURAL_32} 2 * a_ostride) * l_idxc).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_idx).to_integer_32] - a_in[(({NATURAL_32} 2 * a_istride) * l_idxc).to_integer_32]
					a_out[(({NATURAL_32} 2 * a_ostride) * l_idxc + {NATURAL_32} 1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_idx + {NATURAL_32} 1).to_integer_32] - a_in[(({NATURAL_32} 2 * a_istride) * l_idxc + {NATURAL_32} 1).to_integer_32]
					l_i := l_i + 1
				end
				l_e := l_e + 1
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_m___4)
			loop
				a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32] := a_out[(({NATURAL_32} 2 * a_ostride) * l_i).to_integer_32]
				a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32] := a_out[(({NATURAL_32} 2 * a_ostride) * l_i + {NATURAL_32} 1).to_integer_32]
				l_i := l_i + 1
			end
			l_e1 := (1).to_natural_32
			from
			until
				not (not (l_return) and then (l_e1 < (a_factor - {NATURAL_32} 1) // {NATURAL_32} 2 + {NATURAL_32} 1))
			loop
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_m___4)
				loop
					a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32] + (a_out[(({NATURAL_32} 2 * a_ostride) * (l_i + l_e1 * l_m___4)).to_integer_32])
					a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32] + (a_out[(({NATURAL_32} 2 * a_ostride) * (l_i + l_e1 * l_m___4) + {NATURAL_32} 1).to_integer_32])
					l_i := l_i + 1
				end
				l_e1 := l_e1 + 1
			end
			l_e := (1).to_natural_32
			from
			until
				not (not (l_return) and then (l_e < (a_factor - {NATURAL_32} 1) // {NATURAL_32} 2 + {NATURAL_32} 1))
			loop
				l_idx___0 := l_e * l_q___3
				l_idx_step := l_e * l_q___3
				l_em := l_e * l_m___4
				l_ecm := (a_factor - l_e) * l_m___4
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_m___4)
				loop
					a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_em)).to_integer_32] := a_out[(({NATURAL_32} 2 * a_ostride) * l_i).to_integer_32]
					a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_em) + {NATURAL_32} 1).to_integer_32] := a_out[(({NATURAL_32} 2 * a_ostride) * l_i + {NATURAL_32} 1).to_integer_32]
					a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_ecm)).to_integer_32] := a_out[(({NATURAL_32} 2 * a_ostride) * l_i).to_integer_32]
					a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_ecm) + {NATURAL_32} 1).to_integer_32] := a_out[(({NATURAL_32} 2 * a_ostride) * l_i + {NATURAL_32} 1).to_integer_32]
					l_i := l_i + 1
				end
				l_e1 := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_e1 < (a_factor - {NATURAL_32} 1) // {NATURAL_32} 2 + {NATURAL_32} 1))
				loop
					if l_idx___0 = {NATURAL_32} 0 then
						l_w_real := (1).to_real
						l_w_imag := (0).to_real
					elseif a_sign = -1 then
						l_w_real := (a_twiddle + (l_idx___0 - {NATURAL_32} 1).to_integer_32).item.dat[0]
						l_w_imag := (a_twiddle + (l_idx___0 - {NATURAL_32} 1).to_integer_32).item.dat[1]
					else
						l_w_real := (a_twiddle + (l_idx___0 - {NATURAL_32} 1).to_integer_32).item.dat[0]
						l_w_imag := -(a_twiddle + (l_idx___0 - {NATURAL_32} 1).to_integer_32).item.dat[1]
					end
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_m___4)
					loop
						l_xp_real := a_out[(({NATURAL_32} 2 * a_ostride) * (l_i + l_e1 * l_m___4)).to_integer_32]
						l_xp_imag := a_out[(({NATURAL_32} 2 * a_ostride) * (l_i + l_e1 * l_m___4) + {NATURAL_32} 1).to_integer_32]
						l_xm_real := a_out[(({NATURAL_32} 2 * a_ostride) * (l_i + (a_factor - l_e1) * l_m___4)).to_integer_32]
						l_xm_imag := a_out[(({NATURAL_32} 2 * a_ostride) * (l_i + (a_factor - l_e1) * l_m___4) + {NATURAL_32} 1).to_integer_32]
						l_ap := l_w_real * l_xp_real
						l_am := l_w_imag * l_xm_imag
						l_sum_real := l_ap - l_am
						l_sumc_real := l_ap + l_am
						l_bp := l_w_real * l_xp_imag
						l_bm := l_w_imag * l_xm_real
						l_sum_imag := l_bp + l_bm
						l_sumc_imag := l_bp - l_bm
						a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_em)).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_em)).to_integer_32] + l_sum_real
						a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_em) + {NATURAL_32} 1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_em) + {NATURAL_32} 1).to_integer_32] + l_sum_imag
						a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_ecm)).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_ecm)).to_integer_32] + l_sumc_real
						a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_ecm) + {NATURAL_32} 1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_ecm) + {NATURAL_32} 1).to_integer_32] + l_sumc_imag
						l_i := l_i + 1
					end
					l_idx___0 := l_idx___0 + l_idx_step
					l_idx___0 := l_idx___0 \\ (a_factor * l_q___3)
					l_e1 := l_e1 + 1
				end
				l_e := l_e + 1
			end
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_k1 := (0).to_natural_32
			from
			until
				not (l_k1 < l_p_1)
			loop
				a_out[(({NATURAL_32} 2 * a_ostride) * l_k1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_k1).to_integer_32]
				a_out[(({NATURAL_32} 2 * a_ostride) * l_k1 + {NATURAL_32} 1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_k1 + {NATURAL_32} 1).to_integer_32]
				l_k1 := l_k1 + 1
			end
			l_e1 := (1).to_natural_32
			from
			until
				not (not (l_return) and then (l_e1 < a_factor))
			loop
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_p_1)
				loop
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_k1 + l_e1 * l_p_1)).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * (l_k1 + l_e1 * l_m___4)).to_integer_32]
					a_out[(({NATURAL_32} 2 * a_ostride) * (l_k1 + l_e1 * l_p_1) + {NATURAL_32} 1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * (l_k1 + l_e1 * l_m___4) + {NATURAL_32} 1).to_integer_32]
					l_k1 := l_k1 + 1
				end
				l_e1 := l_e1 + 1
			end
			l_i := l_p_1
			l_j := a_product
			l_k := (1).to_natural_32
			from
			until
				not (not (l_return) and then (l_k < l_q___3))
			loop
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_p_1)
				loop
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_i).to_integer_32]
					a_out[(({NATURAL_32} 2 * a_ostride) * l_j + {NATURAL_32} 1).to_integer_32] := a_in[(({NATURAL_32} 2 * a_istride) * l_i + {NATURAL_32} 1).to_integer_32]
					l_i := l_i + 1
					l_j := l_j + 1
					l_k1 := l_k1 + 1
				end
				l_j := l_j + l_jump
				l_k := l_k + 1
			end
			l_i := l_p_1
			l_j := a_product
			l_k := (1).to_natural_32
			from
			until
				not (not (l_return) and then (l_k < l_q___3))
			loop
				l_k1 := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k1 < l_p_1))
				loop
					l_e1 := (1).to_natural_32
					from
					until
						not (l_e1 < a_factor)
					loop
						l_x_real := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_e1 * l_m___4)).to_integer_32]
						l_x_imag := a_in[(({NATURAL_32} 2 * a_istride) * (l_i + l_e1 * l_m___4) + {NATURAL_32} 1).to_integer_32]
						if a_sign = -1 then
							l_w_real___0 := (a_twiddle + (((l_e1 - {NATURAL_32} 1) * l_q___3 + l_k) - {NATURAL_32} 1).to_integer_32).item.dat[0]
							l_w_imag___0 := (a_twiddle + (((l_e1 - {NATURAL_32} 1) * l_q___3 + l_k) - {NATURAL_32} 1).to_integer_32).item.dat[1]
						else
							l_w_real___0 := (a_twiddle + (((l_e1 - {NATURAL_32} 1) * l_q___3 + l_k) - {NATURAL_32} 1).to_integer_32).item.dat[0]
							l_w_imag___0 := -(a_twiddle + (((l_e1 - {NATURAL_32} 1) * l_q___3 + l_k) - {NATURAL_32} 1).to_integer_32).item.dat[1]
						end
						a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_e1 * l_p_1)).to_integer_32] := l_w_real___0 * l_x_real - l_w_imag___0 * l_x_imag
						a_out[(({NATURAL_32} 2 * a_ostride) * (l_j + l_e1 * l_p_1) + {NATURAL_32} 1).to_integer_32] := l_w_real___0 * l_x_imag + l_w_imag___0 * l_x_real
						l_e1 := l_e1 + 1
					end
					l_i := l_i + 1
					l_j := l_j + 1
					l_k1 := l_k1 + 1
				end
				l_j := l_j + l_jump
				l_k := l_k + 1
			end
			Result := 0
		end

feature {ANY}

	copy (a_other: like Current)
		do
			n := a_other.n
			nf := a_other.nf
			factor := a_other.factor.twin
			twiddle := a_other.twiddle.twin
			trig := a_other.trig
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				n = a_other.n and
				nf = a_other.nf and
				factor.is_equal (a_other.factor) and
				twiddle.is_equal (a_other.twiddle) and
				trig.is_equal (a_other.trig)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("n"),
					create {CE_STRUCTURE_FIELD}.make ("nf"),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("factor", <<64>>),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("twiddle", <<64>>),
					create {CE_STRUCTURE_FIELD}.make ("trig")
				>>)
			init_structure_info
		end

end
