class
	PD_GSL_HISTOGRAM

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDIO_STATIC
		redefine
			copy,
			is_equal
		end
	S_LIBIO_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} range.make
			create {CE_REAL_64_POINTER} bin.make
			Precursor
		end

feature {ANY} -- fields

	n: NATURAL_32 assign set_n

	range: CE_POINTER [REAL_64] assign set_range

	bin: CE_POINTER [REAL_64] assign set_bin

feature {ANY} -- field setters

	set_n (a_n: NATURAL_32)
		do
			n := a_n
			update_memory_field ("n")
		end

	set_range (a_range: CE_POINTER [REAL_64])
		do
			range := a_range
			update_memory_field ("range")
		end

	set_bin (a_bin: CE_POINTER [REAL_64])
		do
			bin := a_bin
			update_memory_field ("bin")
		end

feature {ANY} -- public routines

	gsl_histogram_get (a_i: NATURAL_32): REAL_64
		local
			l_n: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := n
			if a_i >= l_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index lies outside valid range of 0 .. n - 1"), ce_string ("get.c"), 34, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := (0).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (bin[a_i.to_integer_32])
			end
		end

	gsl_histogram_get_range (a_i: NATURAL_32; a_lower, a_upper: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_n: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := n
			if a_i >= l_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index lies outside valid range of 0 .. n - 1"), ce_string ("get.c"), 48, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				a_lower.item := range[a_i.to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				a_upper.item := range[(a_i + {NATURAL_32} 1).to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_histogram_find (a_x: REAL_64; a_i: CE_POINTER [NATURAL_32]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := find (n, range, a_x, a_i)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("x not found in range of h"), ce_string ("get.c"), 65, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_histogram_mean: REAL_64
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_wmean: REAL_96
			l_W: REAL_96
			l_xi: REAL_64
			l_wi: REAL_64
		do
			l_n := n
			l_wmean := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_xi := (range[(l_i + {NATURAL_32} 1).to_integer_32] + range[l_i.to_integer_32]) / (2).to_double
				l_wi := bin[l_i.to_integer_32]
				if l_wi > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wi)
					l_wmean := l_wmean + ((real_96_from_real_64 (l_xi) - l_wmean) * (real_96_from_real_64 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			Result := l_wmean.truncated_to_real_64
		end

	gsl_histogram_sigma: REAL_64
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_wvariance: REAL_96
			l_wmean: REAL_96
			l_W: REAL_96
			l_xi: REAL_64
			l_wi: REAL_64
			l_xi___0: REAL_64
			l_wi___0: REAL_64
			l_delta___0: REAL_96
			l_sigma: REAL_64
			l_tmp: REAL_64
		do
			l_n := n
			l_wvariance := real_96_from_integer_32 (0)
			l_wmean := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_xi := (range[(l_i + {NATURAL_32} 1).to_integer_32] + range[l_i.to_integer_32]) / (2).to_double
				l_wi := bin[l_i.to_integer_32]
				if l_wi > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wi)
					l_wmean := l_wmean + ((real_96_from_real_64 (l_xi) - l_wmean) * (real_96_from_real_64 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			l_W := real_96_from_real_64 (0.0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_xi___0 := (range[(l_i + {NATURAL_32} 1).to_integer_32] + range[l_i.to_integer_32]) / (2).to_double
				l_wi___0 := bin[l_i.to_integer_32]
				if l_wi___0 > (0).to_double then
					l_delta___0 := real_96_from_real_64 (l_xi___0) - l_wmean
					l_W := l_W + real_96_from_real_64 (l_wi___0)
					l_wvariance := l_wvariance + ((l_delta___0 * l_delta___0 - l_wvariance) * (real_96_from_real_64 (l_wi___0) / l_W))
				end
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_wvariance.truncated_to_real_64)
			update_class_fields -- Current might have been changed; synchronize
			l_sigma := l_tmp
			Result := l_sigma
		end

	gsl_histogram_sum: REAL_64
		local
			l_sum: REAL_64
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_tmp: NATURAL_32
		do
			l_sum := (0).to_double
			l_i := (0).to_natural_32
			l_n := n
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := l_i
				l_i := l_i + 1
				l_sum := l_sum + (bin[l_tmp.to_integer_32])
			end
			Result := l_sum
		end

	gsl_histogram_reset
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
		do
			l_n := n
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				bin[l_i.to_integer_32] := (0).to_double
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_histogram_max: REAL_64
		local
			l_n: INTEGER_32
		do
			l_n := n.to_integer_32
			Result := (range[l_n])
		end

	gsl_histogram_min: REAL_64
		do
			Result := (range[0])
		end

	gsl_histogram_bins: NATURAL_32
		do
			Result := n
		end

	gsl_histogram_memcpy (a_src: CE_POINTER [PD_GSL_HISTOGRAM]): INTEGER_32
		require
			not a_src.is_void
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := a_src.item.n
			if n /= a_src.item.n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different sizes, cannot copy"), ce_string ("copy.c"), 48, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i <= l_n)
				loop
					range[l_i.to_integer_32] := a_src.item.range[l_i.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					bin[l_i.to_integer_32] := a_src.item.bin[l_i.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram_clone: CE_POINTER [PD_GSL_HISTOGRAM]
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_h: CE_POINTER [PD_GSL_HISTOGRAM]
			l_return, l_break: BOOLEAN
		do
			l_n := n
			l_h := gsl_histogram_calloc_range (l_n, range)
			update_class_fields -- Current might have been changed; synchronize
			if l_h.is_void then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for histogram struct"), ce_string ("copy.c"), 82, 8)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_h.item.bin[l_i.to_integer_32] := bin[l_i.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := l_h
			end
		end

	gsl_histogram_equal_bins_p (a_h2: CE_POINTER [PD_GSL_HISTOGRAM]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_return: BOOLEAN
		do
			if n /= a_h2.item.n then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i <= n))
				loop
					if range[l_i.to_integer_32] /= a_h2.item.range[l_i.to_integer_32] then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_histogram_add (a_h2: CE_POINTER [PD_GSL_HISTOGRAM]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (c_pointer))).item.gsl_histogram_equal_bins_p (a_h2)
			update_class_fields -- Current might have been changed; synchronize
			if not l_tmp.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different binning"), ce_string ("oper.c"), 82, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < n)
				loop
					bin[l_i.to_integer_32] := bin[l_i.to_integer_32] + (a_h2.item.bin[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram_sub (a_h2: CE_POINTER [PD_GSL_HISTOGRAM]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (c_pointer))).item.gsl_histogram_equal_bins_p (a_h2)
			update_class_fields -- Current might have been changed; synchronize
			if not l_tmp.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different binning"), ce_string ("oper.c"), 105, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < n)
				loop
					bin[l_i.to_integer_32] := bin[l_i.to_integer_32] - (a_h2.item.bin[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram_mul (a_h2: CE_POINTER [PD_GSL_HISTOGRAM]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (c_pointer))).item.gsl_histogram_equal_bins_p (a_h2)
			update_class_fields -- Current might have been changed; synchronize
			if not l_tmp.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different binning"), ce_string ("oper.c"), 129, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < n)
				loop
					bin[l_i.to_integer_32] := bin[l_i.to_integer_32] * (a_h2.item.bin[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram_div (a_h2: CE_POINTER [PD_GSL_HISTOGRAM]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (c_pointer))).item.gsl_histogram_equal_bins_p (a_h2)
			update_class_fields -- Current might have been changed; synchronize
			if not l_tmp.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different binning"), ce_string ("oper.c"), 150, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < n)
				loop
					bin[l_i.to_integer_32] := bin[l_i.to_integer_32] / (a_h2.item.bin[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram_scale (a_scale: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
		do
			l_i := (0).to_natural_32
			from
			until
				not (l_i < n)
			loop
				bin[l_i.to_integer_32] := bin[l_i.to_integer_32] * a_scale
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_histogram_shift (a_shift___0: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
		do
			l_i := (0).to_natural_32
			from
			until
				not (l_i < n)
			loop
				bin[l_i.to_integer_32] := bin[l_i.to_integer_32] + a_shift___0
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_histogram_fread (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return: BOOLEAN
		do
			l_tmp := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fread (a_stream, range, n + {NATURAL_32} 1, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fread (a_stream, bin, n, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_status
			end
		end

	gsl_histogram_fwrite (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return: BOOLEAN
		do
			l_tmp := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fwrite (a_stream, range, (n + {NATURAL_32} 1), (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fwrite (a_stream, bin, n, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_status
			end
		end

	gsl_histogram_fprintf (a_stream: CE_POINTER [SD_IO_FILE]; a_range_format, a_bin_format: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_n := n
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_n))
			loop
				l_tmp := fprintf ([a_stream, a_range_format, range[l_i.to_integer_32]])
				update_class_fields -- Current might have been changed; synchronize
				l_status := l_tmp
				if l_status < 0 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file.c"), 63, 5)
						update_class_fields -- Current might have been changed; synchronize
						Result := 5
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					l_status := IO_putc ((' ').code, a_stream)
					update_class_fields -- Current might have been changed; synchronize
					if l_status = -1 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("putc failed"), ce_string ("file.c"), 70, 5)
							update_class_fields -- Current might have been changed; synchronize
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
				end
				if not l_return then
					l_status := fprintf ([a_stream, a_range_format, range[(l_i + {NATURAL_32} 1).to_integer_32]])
					update_class_fields -- Current might have been changed; synchronize
					if l_status < 0 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file.c"), 77, 5)
							update_class_fields -- Current might have been changed; synchronize
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
				end
				if not l_return then
					l_status := IO_putc ((' ').code, a_stream)
					update_class_fields -- Current might have been changed; synchronize
					if l_status = -1 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("putc failed"), ce_string ("file.c"), 84, 5)
							update_class_fields -- Current might have been changed; synchronize
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
				end
				if not l_return then
					l_status := fprintf ([a_stream, a_bin_format, bin[l_i.to_integer_32]])
					update_class_fields -- Current might have been changed; synchronize
					if l_status < 0 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file.c"), 91, 5)
							update_class_fields -- Current might have been changed; synchronize
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
				end
				if not l_return then
					l_status := IO_putc (('%N').code, a_stream)
					update_class_fields -- Current might have been changed; synchronize
					if l_status = -1 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("putc failed"), ce_string ("file.c"), 98, 5)
							update_class_fields -- Current might have been changed; synchronize
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_histogram_fscanf (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_upper: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_upper.make (<<1>>)

			l_n := n
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_n))
			loop
				l_tmp := fscanf ([a_stream, ce_string ("%%lg %%lg %%lg"), range + l_i.to_integer_32, l_upper, bin + l_i.to_integer_32])
				update_class_fields -- Current might have been changed; synchronize
				l_status := l_tmp
				if l_status /= 3 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("fscanf failed"), ce_string ("file.c"), 120, 5)
						update_class_fields -- Current might have been changed; synchronize
						Result := 5
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				range[l_n.to_integer_32] := l_upper.item
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_histogram_accumulate (a_x, a_weight: REAL_64): INTEGER_32
		local
			l_n: NATURAL_32
			l_index: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_index.make (<<1>>)

			l_n := n
			l_index.item := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_tmp := find___1 (n, range, a_x, l_index)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				Result := 1
				l_return := True
			end
			if not l_return and then (l_index.item >= l_n) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index lies outside valid range of 0 .. n - 1"), ce_string ("add.c"), 49, 7)
					update_class_fields -- Current might have been changed; synchronize
					Result := 7
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				bin[(l_index.item).to_integer_32] := bin[(l_index.item).to_integer_32] + a_weight
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_histogram_max_val: REAL_64
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_max: REAL_64
		do
			l_n := n
			l_max := bin[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				if bin[l_i.to_integer_32] > l_max then
					l_max := bin[l_i.to_integer_32]
				end
				l_i := l_i + 1
			end
			Result := l_max
		end

	gsl_histogram_max_bin: NATURAL_32
		local
			l_i: NATURAL_32
			l_imax: NATURAL_32
			l_max: REAL_64
		do
			l_imax := (0).to_natural_32
			l_max := bin[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < n)
			loop
				if bin[l_i.to_integer_32] > l_max then
					l_max := bin[l_i.to_integer_32]
					l_imax := l_i
				end
				l_i := l_i + 1
			end
			Result := l_imax
		end

	gsl_histogram_min_val: REAL_64
		local
			l_i: NATURAL_32
			l_min: REAL_64
		do
			l_min := bin[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < n)
			loop
				if bin[l_i.to_integer_32] < l_min then
					l_min := bin[l_i.to_integer_32]
				end
				l_i := l_i + 1
			end
			Result := l_min
		end

	gsl_histogram_min_bin: NATURAL_32
		local
			l_i: NATURAL_32
			l_imin: NATURAL_32
			l_min: REAL_64
		do
			l_imin := (0).to_natural_32
			l_min := bin[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < n)
			loop
				if bin[l_i.to_integer_32] < l_min then
					l_min := bin[l_i.to_integer_32]
					l_imin := l_i
				end
				l_i := l_i + 1
			end
			Result := l_imin
		end

	gsl_histogram_set_ranges_uniform (a_xmin, a_xmax: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := n
			if a_xmin >= a_xmax then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("xmin must be less than xmax"), ce_string ("init.c"), 155, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				(create {P_INIT_STATIC}).make_uniform___0 (range, l_n, a_xmin, a_xmax)
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					bin[l_i.to_integer_32] := (0).to_double
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram_set_ranges (a_range: CE_POINTER [REAL_64]; a_size: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := n
			if a_size /= l_n + {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of range must match size of histogram"), ce_string ("init.c"), 180, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i <= l_n)
				loop
					range[l_i.to_integer_32] := a_range[l_i.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					bin[l_i.to_integer_32] := (0).to_double
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram_increment (a_x: REAL_64): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (c_pointer))).item.gsl_histogram_accumulate (a_x, 1.0)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

feature {PD_GSL_HISTOGRAM} -- private routines

	find (a_n: NATURAL_32; a_range: CE_POINTER [REAL_64]; a_x: REAL_64; a_i: CE_POINTER [NATURAL_32]): INTEGER_32
		require
			not a_range.is_void
		local
			l_i_linear: NATURAL_32
			l_lower: NATURAL_32
			l_upper: NATURAL_32
			l_mid: NATURAL_32
			l_u: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_x < a_range[0] then
						Result := (-1)
						l_return := True
					end
					if not l_return and then (a_x >= a_range[a_n.to_integer_32]) then
						Result := 1
						l_return := True
					end
					if not l_return then
						l_u := (a_x - a_range[0]) / (a_range[a_n.to_integer_32] - a_range[0])
						l_i_linear := (l_u * a_n.to_real_64).truncated_to_integer_64.to_natural_32
						if a_x >= a_range[l_i_linear.to_integer_32] then
							if a_x < a_range[(l_i_linear + {NATURAL_32} 1).to_integer_32] then
								a_i.item := l_i_linear
								Result := 0
								l_return := True
							end
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						l_upper := a_n
						l_lower := (0).to_natural_32
						from
						until
							not (l_upper - l_lower > {NATURAL_32} 1)
						loop
							l_mid := (l_upper + l_lower) // {NATURAL_32} 2
							if a_x >= a_range[l_mid.to_integer_32] then
								l_lower := l_mid
							else
								l_upper := l_mid
							end
						end
						a_i.item := l_lower
						if not (a_x < a_range[l_lower.to_integer_32]) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_x >= a_range[(l_lower + {NATURAL_32} 1).to_integer_32]) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("x not found in range"), ce_string ("find.c"), 81, 7)
							Result := 7
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_histogram_calloc_range (a_n: NATURAL_32; a_range: CE_POINTER [REAL_64]): CE_POINTER [PD_GSL_HISTOGRAM]
		local
			l_i: NATURAL_32
			l_h: CE_POINTER [PD_GSL_HISTOGRAM]
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram length n must be positive integer"), ce_string ("calloc_range.c"), 46, 1)
					Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					if a_range[l_i.to_integer_32] >= a_range[(l_i + {NATURAL_32} 1).to_integer_32] then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram bin extremes  must be in increasing order"), ce_string ("calloc_range.c"), 56, 1)
							Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (default_pointer))
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				l_h := create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_cast ((malloc ((create {PD_GSL_HISTOGRAM}.make).structure_size.to_natural_32)))
				if l_h.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for histogram struct"), ce_string ("calloc_range.c"), 67, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_h.item.range := create {CE_REAL_64_POINTER}.make_cast ((malloc ((a_n + {NATURAL_32} 1) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_h.item.range).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_h))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for histogram ranges"), ce_string ("calloc_range.c"), 77, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_h.item.bin := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_n * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_h.item.bin).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_h.item.range))
					free (create {CE_VOID_POINTER}.make_cast (l_h))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for histogram bins"), ce_string ("calloc_range.c"), 88, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i <= a_n)
				loop
					l_h.item.range[l_i.to_integer_32] := a_range[l_i.to_integer_32]
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					l_h.item.bin[l_i.to_integer_32] := (0).to_double
					l_i := l_i + 1
				end
				l_h.item.n := a_n
				Result := l_h
			end
		end

	find___1 (a_n: NATURAL_32; a_range: CE_POINTER [REAL_64]; a_x: REAL_64; a_i: CE_POINTER [NATURAL_32]): INTEGER_32
		require
			not a_range.is_void
		local
			l_i_linear: NATURAL_32
			l_lower: NATURAL_32
			l_upper: NATURAL_32
			l_mid: NATURAL_32
			l_u: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_x < a_range[0] then
						Result := (-1)
						l_return := True
					end
					if not l_return and then (a_x >= a_range[a_n.to_integer_32]) then
						Result := 1
						l_return := True
					end
					if not l_return then
						l_u := (a_x - a_range[0]) / (a_range[a_n.to_integer_32] - a_range[0])
						l_i_linear := (l_u * a_n.to_real_64).truncated_to_integer_64.to_natural_32
						if a_x >= a_range[l_i_linear.to_integer_32] then
							if a_x < a_range[(l_i_linear + {NATURAL_32} 1).to_integer_32] then
								a_i.item := l_i_linear
								Result := 0
								l_return := True
							end
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						l_upper := a_n
						l_lower := (0).to_natural_32
						from
						until
							not (l_upper - l_lower > {NATURAL_32} 1)
						loop
							l_mid := (l_upper + l_lower) // {NATURAL_32} 2
							if a_x >= a_range[l_mid.to_integer_32] then
								l_lower := l_mid
							else
								l_upper := l_mid
							end
						end
						a_i.item := l_lower
						if not (a_x < a_range[l_lower.to_integer_32]) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_x >= a_range[(l_lower + {NATURAL_32} 1).to_integer_32]) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("x not found in range"), ce_string ("find.c"), 81, 7)
							Result := 7
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			n := a_other.n
			range := a_other.range
			bin := a_other.bin
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				n = a_other.n and
				range.is_equal (a_other.range) and
				bin.is_equal (a_other.bin)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("n"),
					create {CE_STRUCTURE_FIELD}.make ("range"),
					create {CE_STRUCTURE_FIELD}.make ("bin")
				>>)
			init_structure_info
		end

end
