class
	PD_GSL_HISTOGRAM2D

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDIO_STATIC
		redefine
			copy,
			is_equal
		end
	S_LIBIO_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} xrange.make
			create {CE_REAL_64_POINTER} yrange.make
			create {CE_REAL_64_POINTER} bin.make
			Precursor
		end

feature {ANY} -- fields

	nx: NATURAL_32 assign set_nx

	ny: NATURAL_32 assign set_ny

	xrange: CE_POINTER [REAL_64] assign set_xrange

	yrange: CE_POINTER [REAL_64] assign set_yrange

	bin: CE_POINTER [REAL_64] assign set_bin

feature {ANY} -- field setters

	set_nx (a_nx: NATURAL_32)
		do
			nx := a_nx
			update_memory_field ("nx")
		end

	set_ny (a_ny: NATURAL_32)
		do
			ny := a_ny
			update_memory_field ("ny")
		end

	set_xrange (a_xrange: CE_POINTER [REAL_64])
		do
			xrange := a_xrange
			update_memory_field ("xrange")
		end

	set_yrange (a_yrange: CE_POINTER [REAL_64])
		do
			yrange := a_yrange
			update_memory_field ("yrange")
		end

	set_bin (a_bin: CE_POINTER [REAL_64])
		do
			bin := a_bin
			update_memory_field ("bin")
		end

feature {ANY} -- public routines

	gsl_histogram2d_xmax: REAL_64
		local
			l_nx: INTEGER_32
		do
			l_nx := nx.to_integer_32
			Result := (xrange[l_nx])
		end

	gsl_histogram2d_xmin: REAL_64
		do
			Result := (xrange[0])
		end

	gsl_histogram2d_ymax: REAL_64
		local
			l_ny: INTEGER_32
		do
			l_ny := ny.to_integer_32
			Result := (yrange[l_ny])
		end

	gsl_histogram2d_ymin: REAL_64
		do
			Result := (yrange[0])
		end

	gsl_histogram2d_nx: NATURAL_32
		do
			Result := nx
		end

	gsl_histogram2d_ny: NATURAL_32
		do
			Result := ny
		end

	gsl_histogram2d_get (a_i, a_j: NATURAL_32): REAL_64
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_nx := nx
			l_ny := ny
			if a_i >= l_nx then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index i lies outside valid range of 0 .. nx - 1"), ce_string ("get2d.c"), 35, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := (0).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_ny) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index j lies outside valid range of 0 .. ny - 1"), ce_string ("get2d.c"), 41, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := (0).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (bin[(a_i * l_ny + a_j).to_integer_32])
			end
		end

	gsl_histogram2d_get_xrange (a_i: NATURAL_32; a_xlower, a_xupper: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_nx: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_nx := nx
			if a_i >= l_nx then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index i lies outside valid range of 0 .. nx - 1"), ce_string ("get2d.c"), 55, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				a_xlower.item := xrange[a_i.to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				a_xupper.item := xrange[(a_i + {NATURAL_32} 1).to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_histogram2d_get_yrange (a_j: NATURAL_32; a_ylower, a_yupper: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_ny: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_ny := ny
			if a_j >= l_ny then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index j lies outside valid range of 0 .. ny - 1"), ce_string ("get2d.c"), 72, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				a_ylower.item := yrange[a_j.to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				a_yupper.item := yrange[(a_j + {NATURAL_32} 1).to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_histogram2d_find (a_x, a_y: REAL_64; a_i, a_j: CE_POINTER [NATURAL_32]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := find___0 (nx, xrange, a_x, a_i)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("x not found in range of h"), ce_string ("get2d.c"), 90, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_status := find___0 (ny, yrange, a_y, a_j)
				update_class_fields -- Current might have been changed; synchronize
				if l_status.to_boolean then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("y not found in range of h"), ce_string ("get2d.c"), 97, 1)
						update_class_fields -- Current might have been changed; synchronize
						Result := 1
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_histogram2d_equal_bins_p (a_h2: CE_POINTER [PD_GSL_HISTOGRAM2D]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_return: BOOLEAN
		do
			if nx /= a_h2.item.nx then
				Result := 0
				l_return := True
			elseif ny /= a_h2.item.ny then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i <= nx))
				loop
					if xrange[l_i.to_integer_32] /= a_h2.item.xrange[l_i.to_integer_32] then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i <= ny))
				loop
					if yrange[l_i.to_integer_32] /= a_h2.item.yrange[l_i.to_integer_32] then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_histogram2d_add (a_h2: CE_POINTER [PD_GSL_HISTOGRAM2D]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (c_pointer))).item.gsl_histogram2d_equal_bins_p (a_h2)
			update_class_fields -- Current might have been changed; synchronize
			if not l_tmp.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different binning"), ce_string ("oper2d.c"), 88, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < nx * ny)
				loop
					bin[l_i.to_integer_32] := bin[l_i.to_integer_32] + (a_h2.item.bin[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram2d_sub (a_h2: CE_POINTER [PD_GSL_HISTOGRAM2D]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (c_pointer))).item.gsl_histogram2d_equal_bins_p (a_h2)
			update_class_fields -- Current might have been changed; synchronize
			if not l_tmp.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different binning"), ce_string ("oper2d.c"), 111, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < nx * ny)
				loop
					bin[l_i.to_integer_32] := bin[l_i.to_integer_32] - (a_h2.item.bin[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram2d_mul (a_h2: CE_POINTER [PD_GSL_HISTOGRAM2D]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (c_pointer))).item.gsl_histogram2d_equal_bins_p (a_h2)
			update_class_fields -- Current might have been changed; synchronize
			if not l_tmp.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different binning"), ce_string ("oper2d.c"), 134, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < nx * ny)
				loop
					bin[l_i.to_integer_32] := bin[l_i.to_integer_32] * (a_h2.item.bin[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram2d_div (a_h2: CE_POINTER [PD_GSL_HISTOGRAM2D]): INTEGER_32
		require
			not a_h2.is_void
		local
			l_i: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (c_pointer))).item.gsl_histogram2d_equal_bins_p (a_h2)
			update_class_fields -- Current might have been changed; synchronize
			if not l_tmp.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different binning"), ce_string ("oper2d.c"), 157, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < nx * ny)
				loop
					bin[l_i.to_integer_32] := bin[l_i.to_integer_32] / (a_h2.item.bin[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram2d_scale (a_scale: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
		do
			l_i := (0).to_natural_32
			from
			until
				not (l_i < nx * ny)
			loop
				bin[l_i.to_integer_32] := bin[l_i.to_integer_32] * a_scale
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_histogram2d_shift (a_shift___0: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
		do
			l_i := (0).to_natural_32
			from
			until
				not (l_i < nx * ny)
			loop
				bin[l_i.to_integer_32] := bin[l_i.to_integer_32] + a_shift___0
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_histogram2d_max_val: REAL_64
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_max: REAL_64
		do
			l_nx := nx
			l_ny := ny
			l_max := bin[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_nx * l_ny)
			loop
				if bin[l_i.to_integer_32] > l_max then
					l_max := bin[l_i.to_integer_32]
				end
				l_i := l_i + 1
			end
			Result := l_max
		end

	gsl_histogram2d_max_bin (a_imax_out, a_jmax_out: CE_POINTER [NATURAL_32])
		require
			not a_imax_out.is_void
			not a_jmax_out.is_void
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_imax: NATURAL_32
			l_jmax: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_max: REAL_64
			l_x: REAL_64
			l_return: BOOLEAN
		do
			l_nx := nx
			l_ny := ny
			l_imax := (0).to_natural_32
			l_jmax := (0).to_natural_32
			l_max := bin[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_nx))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_ny)
				loop
					l_x := bin[(l_i * l_ny + l_j).to_integer_32]
					if l_x > l_max then
						l_max := l_x
						l_imax := l_i
						l_jmax := l_j
					end
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			a_imax_out.item := l_imax
			update_class_fields -- Current might have been changed; synchronize
			a_jmax_out.item := l_jmax
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_histogram2d_min_val: REAL_64
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_min: REAL_64
		do
			l_nx := nx
			l_ny := ny
			l_min := bin[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_nx * l_ny)
			loop
				if bin[l_i.to_integer_32] < l_min then
					l_min := bin[l_i.to_integer_32]
				end
				l_i := l_i + 1
			end
			Result := l_min
		end

	gsl_histogram2d_min_bin (a_imin_out, a_jmin_out: CE_POINTER [NATURAL_32])
		require
			not a_imin_out.is_void
			not a_jmin_out.is_void
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_imin: NATURAL_32
			l_jmin: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_min: REAL_64
			l_x: REAL_64
			l_return: BOOLEAN
		do
			l_nx := nx
			l_ny := ny
			l_imin := (0).to_natural_32
			l_jmin := (0).to_natural_32
			l_min := bin[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_nx))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_ny)
				loop
					l_x := bin[(l_i * l_ny + l_j).to_integer_32]
					if l_x < l_min then
						l_min := l_x
						l_imin := l_i
						l_jmin := l_j
					end
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			a_imin_out.item := l_imin
			update_class_fields -- Current might have been changed; synchronize
			a_jmin_out.item := l_jmin
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_histogram2d_memcpy (a_src: CE_POINTER [PD_GSL_HISTOGRAM2D]): INTEGER_32
		require
			not a_src.is_void
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nx := a_src.item.nx
					l_ny := a_src.item.ny
					if nx = a_src.item.nx then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if ny = a_src.item.ny then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("histograms have different sizes, cannot copy"), ce_string ("copy2d.c"), 47, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if done
					if not l_return then
						l_i := (0).to_natural_32
						from
						until
							not (l_i <= l_nx)
						loop
							xrange[l_i.to_integer_32] := a_src.item.xrange[l_i.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_i := (0).to_natural_32
						from
						until
							not (l_i <= l_ny)
						loop
							yrange[l_i.to_integer_32] := a_src.item.yrange[l_i.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_nx * l_ny)
						loop
							bin[l_i.to_integer_32] := a_src.item.bin[l_i.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						Result := 0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_histogram2d_clone: CE_POINTER [PD_GSL_HISTOGRAM2D]
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_h: CE_POINTER [PD_GSL_HISTOGRAM2D]
			l_return, l_break: BOOLEAN
		do
			l_nx := nx
			l_ny := ny
			l_h := gsl_histogram2d_calloc_range (l_nx, l_ny, xrange, yrange)
			update_class_fields -- Current might have been changed; synchronize
			if l_h.is_void then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for histogram struct"), ce_string ("copy2d.c"), 87, 8)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nx * l_ny)
				loop
					l_h.item.bin[l_i.to_integer_32] := bin[l_i.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := l_h
			end
		end

	gsl_histogram2d_reset
		local
			l_i: NATURAL_32
			l_nx: NATURAL_32
			l_ny: NATURAL_32
		do
			l_nx := nx
			l_ny := ny
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_nx * l_ny)
			loop
				bin[l_i.to_integer_32] := (0).to_double
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_histogram2d_set_ranges_uniform (a_xmin, a_xmax, a_ymin, a_ymax: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_nx := nx
			l_ny := ny
			if a_xmin >= a_xmax then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("xmin must be less than xmax"), ce_string ("init2d.c"), 238, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_ymin >= a_ymax) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("ymin must be less than ymax"), ce_string ("init2d.c"), 243, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				(create {P_INIT2D_STATIC}).make_uniform (xrange, l_nx, a_xmin, a_xmax)
				update_class_fields; -- Current might have been changed; synchronize
				(create {P_INIT2D_STATIC}).make_uniform (yrange, l_ny, a_ymin, a_ymax)
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nx * l_ny)
				loop
					bin[l_i.to_integer_32] := (0).to_double
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram2d_set_ranges (a_xrange: CE_POINTER [REAL_64]; a_xsize: NATURAL_32; a_yrange: CE_POINTER [REAL_64]; a_ysize: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_nx := nx
			l_ny := ny
			if a_xsize /= l_nx + {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of xrange must match size of histogram"), ce_string ("init2d.c"), 272, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_ysize /= l_ny + {NATURAL_32} 1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of yrange must match size of histogram"), ce_string ("init2d.c"), 278, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i <= l_nx)
				loop
					xrange[l_i.to_integer_32] := a_xrange[l_i.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (l_i <= l_ny)
				loop
					yrange[l_i.to_integer_32] := a_yrange[l_i.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nx * l_ny)
				loop
					bin[l_i.to_integer_32] := (0).to_double
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_histogram2d_sum: REAL_64
		local
			l_n: NATURAL_32
			l_sum: REAL_64
			l_i: NATURAL_32
			l_tmp: NATURAL_32
		do
			l_n := (nx * ny)
			l_sum := (0).to_double
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := l_i
				l_i := l_i + 1
				l_sum := l_sum + (bin[l_tmp.to_integer_32])
			end
			Result := l_sum
		end

	gsl_histogram2d_xmean: REAL_64
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wmean: REAL_96
			l_W: REAL_96
			l_xi: REAL_64
			l_wi: REAL_64
			l_wij: REAL_64
			l_return: BOOLEAN
		do
			l_nx := nx
			l_ny := ny
			l_wmean := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_nx))
			loop
				l_xi := (xrange[(l_i + {NATURAL_32} 1).to_integer_32] + xrange[l_i.to_integer_32]) / 2.0
				l_wi := (0).to_double
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_ny)
				loop
					l_wij := bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_wi := l_wi + l_wij
					end
					l_j := l_j + 1
				end
				if l_wi > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wi)
					l_wmean := l_wmean + ((real_96_from_real_64 (l_xi) - l_wmean) * (real_96_from_real_64 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			Result := l_wmean.truncated_to_real_64
		end

	gsl_histogram2d_ymean: REAL_64
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wmean: REAL_96
			l_W: REAL_96
			l_yj: REAL_64
			l_wj: REAL_64
			l_wij: REAL_64
			l_return: BOOLEAN
		do
			l_nx := nx
			l_ny := ny
			l_wmean := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_ny))
			loop
				l_yj := (yrange[(l_j + {NATURAL_32} 1).to_integer_32] + yrange[l_j.to_integer_32]) / 2.0
				l_wj := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nx)
				loop
					l_wij := bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_wj := l_wj + l_wij
					end
					l_i := l_i + 1
				end
				if l_wj > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wj)
					l_wmean := l_wmean + ((real_96_from_real_64 (l_yj) - l_wmean) * (real_96_from_real_64 (l_wj) / l_W))
				end
				l_j := l_j + 1
			end
			Result := l_wmean.truncated_to_real_64
		end

	gsl_histogram2d_xsigma: REAL_64
		local
			l_xmean: REAL_64
			l_tmp: REAL_64
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wvariance: REAL_96
			l_W: REAL_96
			l_xi: REAL_64
			l_wi: REAL_64
			l_wij: REAL_64
			l_xsigma: REAL_64
			l_tmp___0: REAL_64
			l_return: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (c_pointer))).item.gsl_histogram2d_xmean
			update_class_fields -- Current might have been changed; synchronize
			l_xmean := l_tmp
			l_nx := nx
			l_ny := ny
			l_wvariance := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_nx))
			loop
				l_xi := (xrange[(l_i + {NATURAL_32} 1).to_integer_32] + xrange[l_i.to_integer_32]) / (2).to_double - l_xmean
				l_wi := (0).to_double
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_ny)
				loop
					l_wij := bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_wi := l_wi + l_wij
					end
					l_j := l_j + 1
				end
				if l_wi > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wi)
					l_wvariance := l_wvariance + ((real_96_from_real_64 ((l_xi * l_xi)) - l_wvariance) * (real_96_from_real_64 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			l_tmp___0 := sqrt (l_wvariance.truncated_to_real_64)
			update_class_fields -- Current might have been changed; synchronize
			l_xsigma := l_tmp___0
			Result := l_xsigma
		end

	gsl_histogram2d_ysigma: REAL_64
		local
			l_ymean: REAL_64
			l_tmp: REAL_64
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wvariance: REAL_96
			l_W: REAL_96
			l_yj: REAL_64
			l_wj: REAL_64
			l_wij: REAL_64
			l_ysigma: REAL_64
			l_tmp___0: REAL_64
			l_return: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (c_pointer))).item.gsl_histogram2d_ymean
			update_class_fields -- Current might have been changed; synchronize
			l_ymean := l_tmp
			l_nx := nx
			l_ny := ny
			l_wvariance := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_ny))
			loop
				l_yj := (yrange[(l_j + {NATURAL_32} 1).to_integer_32] + yrange[l_j.to_integer_32]) / 2.0 - l_ymean
				l_wj := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nx)
				loop
					l_wij := bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_wj := l_wj + l_wij
					end
					l_i := l_i + 1
				end
				if l_wj > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wj)
					l_wvariance := l_wvariance + ((real_96_from_real_64 ((l_yj * l_yj)) - l_wvariance) * (real_96_from_real_64 (l_wj) / l_W))
				end
				l_j := l_j + 1
			end
			l_tmp___0 := sqrt (l_wvariance.truncated_to_real_64)
			update_class_fields -- Current might have been changed; synchronize
			l_ysigma := l_tmp___0
			Result := l_ysigma
		end

	gsl_histogram2d_cov: REAL_64
		local
			l_xmean: REAL_64
			l_tmp: REAL_64
			l_ymean: REAL_64
			l_tmp___0: REAL_64
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wcovariance: REAL_96
			l_W: REAL_96
			l_xi: REAL_64
			l_yj: REAL_64
			l_wij: REAL_64
			l_return: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (c_pointer))).item.gsl_histogram2d_xmean
			update_class_fields -- Current might have been changed; synchronize
			l_xmean := l_tmp
			l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (c_pointer))).item.gsl_histogram2d_ymean
			update_class_fields -- Current might have been changed; synchronize
			l_ymean := l_tmp___0
			l_nx := nx
			l_ny := ny
			l_wcovariance := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_ny))
			loop
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nx)
				loop
					l_xi := (xrange[(l_i + {NATURAL_32} 1).to_integer_32] + xrange[l_i.to_integer_32]) / 2.0 - l_xmean
					l_yj := (yrange[(l_j + {NATURAL_32} 1).to_integer_32] + yrange[l_j.to_integer_32]) / 2.0 - l_ymean
					l_wij := bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_W := l_W + real_96_from_real_64 (l_wij)
						l_wcovariance := l_wcovariance + ((real_96_from_real_64 ((l_xi * l_yj)) - l_wcovariance) * (real_96_from_real_64 (l_wij) / l_W))
					end
					l_i := l_i + 1
				end
				l_j := l_j + 1
			end
			Result := l_wcovariance.truncated_to_real_64
		end

	gsl_histogram2d_fread (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return: BOOLEAN
		do
			l_tmp := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fread (a_stream, xrange, nx + {NATURAL_32} 1, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fread (a_stream, yrange, ny + {NATURAL_32} 1, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				if l_status.to_boolean then
					Result := l_status
					l_return := True
				end
			end
			if not l_return then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fread (a_stream, bin, nx * ny, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_status
			end
		end

	gsl_histogram2d_fwrite (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return: BOOLEAN
		do
			l_tmp := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fwrite (a_stream, xrange, (nx + {NATURAL_32} 1), (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fwrite (a_stream, yrange, (ny + {NATURAL_32} 1), (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				if l_status.to_boolean then
					Result := l_status
					l_return := True
				end
			end
			if not l_return then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fwrite (a_stream, bin, (nx * ny), (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_status
			end
		end

	gsl_histogram2d_fprintf (a_stream: CE_POINTER [SD_IO_FILE]; a_range_format, a_bin_format: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_nx := nx
			l_ny := ny
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_nx))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_ny))
				loop
					l_status := fprintf ([a_stream, a_range_format, xrange[l_i.to_integer_32]])
					update_class_fields -- Current might have been changed; synchronize
					if l_status < 0 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file2d.c"), 78, 5)
							update_class_fields -- Current might have been changed; synchronize
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_status := IO_putc ((' ').code, a_stream)
						update_class_fields -- Current might have been changed; synchronize
						if l_status = -1 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("putc failed"), ce_string ("file2d.c"), 85, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_status := fprintf ([a_stream, a_range_format, xrange[(l_i + {NATURAL_32} 1).to_integer_32]])
						update_class_fields -- Current might have been changed; synchronize
						if l_status < 0 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file2d.c"), 92, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_status := IO_putc ((' ').code, a_stream)
						update_class_fields -- Current might have been changed; synchronize
						if l_status = -1 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("putc failed"), ce_string ("file2d.c"), 99, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_status := fprintf ([a_stream, a_range_format, yrange[l_j.to_integer_32]])
						update_class_fields -- Current might have been changed; synchronize
						if l_status < 0 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file2d.c"), 106, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_status := IO_putc ((' ').code, a_stream)
						update_class_fields -- Current might have been changed; synchronize
						if l_status = -1 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("putc failed"), ce_string ("file2d.c"), 113, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_status := fprintf ([a_stream, a_range_format, yrange[(l_j + {NATURAL_32} 1).to_integer_32]])
						update_class_fields -- Current might have been changed; synchronize
						if l_status < 0 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file2d.c"), 120, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_status := IO_putc ((' ').code, a_stream)
						update_class_fields -- Current might have been changed; synchronize
						if l_status = -1 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("putc failed"), ce_string ("file2d.c"), 127, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_status := fprintf ([a_stream, a_bin_format, bin[(l_i * l_ny + l_j).to_integer_32]])
						update_class_fields -- Current might have been changed; synchronize
						if l_status < 0 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file2d.c"), 134, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_status := IO_putc (('%N').code, a_stream)
						update_class_fields -- Current might have been changed; synchronize
						if l_status = -1 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("putc failed"), ce_string ("file2d.c"), 141, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_status := IO_putc (('%N').code, a_stream)
					update_class_fields -- Current might have been changed; synchronize
					if l_status = -1 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("putc failed"), ce_string ("file2d.c"), 148, 5)
							update_class_fields -- Current might have been changed; synchronize
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_histogram2d_fscanf (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_xupper: CE_ARRAY [REAL_64]
			l_yupper: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_xupper.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_yupper.make (<<1>>)

			l_nx := nx
			l_ny := ny
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_nx))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_ny))
				loop
					l_tmp := fscanf ([a_stream, ce_string ("%%lg %%lg %%lg %%lg %%lg"), xrange + l_i.to_integer_32, l_xupper, yrange + l_j.to_integer_32, l_yupper, (bin + (l_i * l_ny).to_integer_32) + l_j.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_status := l_tmp
					if l_status /= 5 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("fscanf failed"), ce_string ("file2d.c"), 175, 5)
							update_class_fields -- Current might have been changed; synchronize
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					yrange[l_ny.to_integer_32] := l_yupper.item
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
			end
			if not l_return then
				xrange[l_nx.to_integer_32] := l_xupper.item
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_histogram2d_accumulate (a_x, a_y, a_weight: REAL_64): INTEGER_32
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: CE_ARRAY [NATURAL_32]
			l_j: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_i.make (<<1>>)
			create {CE_NATURAL_32_ARRAY} l_j.make (<<1>>)

			l_nx := nx
			l_ny := ny
			l_i.item := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_j.item := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_tmp := find2d (nx, xrange, ny, yrange, a_x, a_y, l_i, l_j)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				Result := 1
				l_return := True
			end
			if not l_return and then (l_i.item >= l_nx) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index lies outside valid range of 0 .. nx - 1"), ce_string ("add2d.c"), 54, 7)
					update_class_fields -- Current might have been changed; synchronize
					Result := 7
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (l_j.item >= l_ny) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index lies outside valid range of 0 .. ny - 1"), ce_string ("add2d.c"), 60, 7)
					update_class_fields -- Current might have been changed; synchronize
					Result := 7
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				bin[(l_i.item * l_ny + l_j.item).to_integer_32] := bin[(l_i.item * l_ny + l_j.item).to_integer_32] + a_weight
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_histogram2d_increment (a_x, a_y: REAL_64): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (c_pointer))).item.gsl_histogram2d_accumulate (a_x, a_y, 1.0)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

feature {PD_GSL_HISTOGRAM2D} -- private routines

	find___0 (a_n: NATURAL_32; a_range: CE_POINTER [REAL_64]; a_x: REAL_64; a_i: CE_POINTER [NATURAL_32]): INTEGER_32
		require
			not a_range.is_void
		local
			l_i_linear: NATURAL_32
			l_lower: NATURAL_32
			l_upper: NATURAL_32
			l_mid: NATURAL_32
			l_u: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_x < a_range[0] then
						Result := (-1)
						l_return := True
					end
					if not l_return and then (a_x >= a_range[a_n.to_integer_32]) then
						Result := 1
						l_return := True
					end
					if not l_return then
						l_u := (a_x - a_range[0]) / (a_range[a_n.to_integer_32] - a_range[0])
						l_i_linear := (l_u * a_n.to_real_64).truncated_to_integer_64.to_natural_32
						if a_x >= a_range[l_i_linear.to_integer_32] then
							if a_x < a_range[(l_i_linear + {NATURAL_32} 1).to_integer_32] then
								a_i.item := l_i_linear
								Result := 0
								l_return := True
							end
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						l_upper := a_n
						l_lower := (0).to_natural_32
						from
						until
							not (l_upper - l_lower > {NATURAL_32} 1)
						loop
							l_mid := (l_upper + l_lower) // {NATURAL_32} 2
							if a_x >= a_range[l_mid.to_integer_32] then
								l_lower := l_mid
							else
								l_upper := l_mid
							end
						end
						a_i.item := l_lower
						if not (a_x < a_range[l_lower.to_integer_32]) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_x >= a_range[(l_lower + {NATURAL_32} 1).to_integer_32]) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("x not found in range"), ce_string ("find.c"), 81, 7)
							Result := 7
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_histogram2d_calloc_range (a_nx, a_ny: NATURAL_32; a_xrange, a_yrange: CE_POINTER [REAL_64]): CE_POINTER [PD_GSL_HISTOGRAM2D]
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_h: CE_POINTER [PD_GSL_HISTOGRAM2D]
			l_return, l_break: BOOLEAN
		do
			if a_nx = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram length nx must be positive integer"), ce_string ("calloc_range2d.c"), 51, 1)
					Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_ny = {NATURAL_32} 0) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram length ny must be positive integer"), ce_string ("calloc_range2d.c"), 57, 1)
					Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_nx))
				loop
					if a_xrange[l_i.to_integer_32] >= a_xrange[(l_i + {NATURAL_32} 1).to_integer_32] then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram xrange not in increasing order"), ce_string ("calloc_range2d.c"), 67, 1)
							Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (default_pointer))
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < a_ny))
				loop
					if a_yrange[l_j.to_integer_32] >= a_yrange[(l_j + {NATURAL_32} 1).to_integer_32] then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram yrange not in increasing order"), ce_string ("calloc_range2d.c"), 76, 1)
							Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (default_pointer))
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
			end
			if not l_return then
				l_h := create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_cast ((malloc ((create {PD_GSL_HISTOGRAM2D}.make).structure_size.to_natural_32)))
				if l_h.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for histogram struct"), ce_string ("calloc_range2d.c"), 87, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_h.item.xrange := create {CE_REAL_64_POINTER}.make_cast ((malloc ((a_nx + {NATURAL_32} 1) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_h.item.xrange).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_h))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for histogram xrange"), ce_string ("calloc_range2d.c"), 98, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_h.item.yrange := create {CE_REAL_64_POINTER}.make_cast ((malloc ((a_ny + {NATURAL_32} 1) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_h.item.yrange).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_h))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for histogram yrange"), ce_string ("calloc_range2d.c"), 109, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_h.item.bin := create {CE_REAL_64_POINTER}.make_cast ((malloc ((a_nx * a_ny) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_h.item.bin).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_h.item.xrange))
					free (create {CE_VOID_POINTER}.make_cast (l_h.item.yrange))
					free (create {CE_VOID_POINTER}.make_cast (l_h))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for histogram bins"), ce_string ("calloc_range2d.c"), 122, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_HISTOGRAM2D]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i <= a_nx)
				loop
					l_h.item.xrange[l_i.to_integer_32] := a_xrange[l_i.to_integer_32]
					l_i := l_i + 1
				end
				l_j := (0).to_natural_32
				from
				until
					not (l_j <= a_ny)
				loop
					l_h.item.yrange[l_j.to_integer_32] := a_yrange[l_j.to_integer_32]
					l_j := l_j + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_nx))
				loop
					l_j := (0).to_natural_32
					from
					until
						not (l_j < a_ny)
					loop
						l_h.item.bin[(l_i * a_ny + l_j).to_integer_32] := (0).to_double
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
				l_h.item.nx := a_nx
				l_h.item.ny := a_ny
				Result := l_h
			end
		end

	find2d (a_nx: NATURAL_32; a_xrange: CE_POINTER [REAL_64]; a_ny: NATURAL_32; a_yrange: CE_POINTER [REAL_64]; a_x, a_y: REAL_64; a_i, a_j: CE_POINTER [NATURAL_32]): INTEGER_32
		require
			not a_xrange.is_void
		local
			l_status: INTEGER_32
			l_return: BOOLEAN
		do
			l_status := (find___4 (a_nx, a_xrange, a_x, a_i))
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_status := find___4 (a_ny, a_yrange, a_y, a_j)
				if l_status.to_boolean then
					Result := l_status
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	find___4 (a_n: NATURAL_32; a_range: CE_POINTER [REAL_64]; a_x: REAL_64; a_i: CE_POINTER [NATURAL_32]): INTEGER_32
		require
			not a_range.is_void
		local
			l_i_linear: NATURAL_32
			l_lower: NATURAL_32
			l_upper: NATURAL_32
			l_mid: NATURAL_32
			l_u: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_x < a_range[0] then
						Result := (-1)
						l_return := True
					end
					if not l_return and then (a_x >= a_range[a_n.to_integer_32]) then
						Result := 1
						l_return := True
					end
					if not l_return then
						l_u := (a_x - a_range[0]) / (a_range[a_n.to_integer_32] - a_range[0])
						l_i_linear := (l_u * a_n.to_real_64).truncated_to_integer_64.to_natural_32
						if a_x >= a_range[l_i_linear.to_integer_32] then
							if a_x < a_range[(l_i_linear + {NATURAL_32} 1).to_integer_32] then
								a_i.item := l_i_linear
								Result := 0
								l_return := True
							end
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						l_upper := a_n
						l_lower := (0).to_natural_32
						from
						until
							not (l_upper - l_lower > {NATURAL_32} 1)
						loop
							l_mid := (l_upper + l_lower) // {NATURAL_32} 2
							if a_x >= a_range[l_mid.to_integer_32] then
								l_lower := l_mid
							else
								l_upper := l_mid
							end
						end
						a_i.item := l_lower
						if not (a_x < a_range[l_lower.to_integer_32]) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_x >= a_range[(l_lower + {NATURAL_32} 1).to_integer_32]) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("x not found in range"), ce_string ("find.c"), 81, 7)
							Result := 7
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			nx := a_other.nx
			ny := a_other.ny
			xrange := a_other.xrange
			yrange := a_other.yrange
			bin := a_other.bin
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				nx = a_other.nx and
				ny = a_other.ny and
				xrange.is_equal (a_other.xrange) and
				yrange.is_equal (a_other.yrange) and
				bin.is_equal (a_other.bin)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("nx"),
					create {CE_STRUCTURE_FIELD}.make ("ny"),
					create {CE_STRUCTURE_FIELD}.make ("xrange"),
					create {CE_STRUCTURE_FIELD}.make ("yrange"),
					create {CE_STRUCTURE_FIELD}.make ("bin")
				>>)
			init_structure_info
		end

end
