class
	PD_GSL_HISTOGRAM_PDF

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} range.make
			create {CE_REAL_64_POINTER} sum.make
		end

feature {ANY} -- fields

	n: NATURAL_32 assign set_n

	range: CE_POINTER [REAL_64] assign set_range

	sum: CE_POINTER [REAL_64] assign set_sum

feature {ANY} -- field setters

	set_n (a_n: NATURAL_32)
		do
			n := a_n
			update_memory_field ("n")
		end

	set_range (a_range: CE_POINTER [REAL_64])
		do
			range := a_range
			update_memory_field ("range")
		end

	set_sum (a_sum: CE_POINTER [REAL_64])
		do
			sum := a_sum
			update_memory_field ("sum")
		end

feature {ANY} -- public routines

	gsl_histogram_pdf_sample (a_r___3: REAL_64): REAL_64
		local
			l_r___3: REAL_64
			l_i: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
			l_delta___0: REAL_64
			l_x: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_i.make (<<1>>)

			l_r___3 := a_r___3
			if l_r___3 = 1.0 then
				l_r___3 := 0.0
			end
			l_status := find___2 (n, sum, l_r___3, l_i)
			update_class_fields -- Current might have been changed; synchronize
			if l_status.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("cannot find r in cumulative pdf"), ce_string ("pdf.c"), 46, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := (0).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_delta___0 := (l_r___3 - sum[(l_i.item).to_integer_32]) / (sum[(l_i.item + {NATURAL_32} 1).to_integer_32] - sum[(l_i.item).to_integer_32])
				l_x := range[(l_i.item).to_integer_32] + l_delta___0 * (range[(l_i.item + {NATURAL_32} 1).to_integer_32] - range[(l_i.item).to_integer_32])
				Result := l_x
				l_return := True
			end
			if not l_return then
				Result := (0).to_double
			end
		end

	gsl_histogram_pdf_init (a_h: CE_POINTER [PD_GSL_HISTOGRAM]): INTEGER_32
		require
			not a_h.is_void
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_mean: REAL_64
			l_sum: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_n := n
			if l_n /= a_h.item.n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram length must match pdf length"), ce_string ("pdf.c"), 109, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_n))
				loop
					if a_h.item.bin[l_i.to_integer_32] < (0).to_double then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram bins must be non-negative to computea probability distribution"), ce_string ("pdf.c"), 117, 1)
							update_class_fields -- Current might have been changed; synchronize
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n + {NATURAL_32} 1)
				loop
					range[l_i.to_integer_32] := a_h.item.range[l_i.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_mean := (0).to_double
				l_sum := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_mean := l_mean + ((a_h.item.bin[l_i.to_integer_32] - l_mean) / (l_i + {NATURAL_32} 1).to_real_64)
					l_i := l_i + 1
				end
				sum[0] := (0).to_double
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_sum := l_sum + ((a_h.item.bin[l_i.to_integer_32] / l_mean) / l_n.to_real_64)
					sum[(l_i + {NATURAL_32} 1).to_integer_32] := l_sum
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

feature {PD_GSL_HISTOGRAM_PDF} -- private routines

	find___2 (a_n: NATURAL_32; a_range: CE_POINTER [REAL_64]; a_x: REAL_64; a_i: CE_POINTER [NATURAL_32]): INTEGER_32
		require
			not a_range.is_void
		local
			l_i_linear: NATURAL_32
			l_lower: NATURAL_32
			l_upper: NATURAL_32
			l_mid: NATURAL_32
			l_u: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_x < a_range[0] then
						Result := (-1)
						l_return := True
					end
					if not l_return and then (a_x >= a_range[a_n.to_integer_32]) then
						Result := 1
						l_return := True
					end
					if not l_return then
						l_u := (a_x - a_range[0]) / (a_range[a_n.to_integer_32] - a_range[0])
						l_i_linear := (l_u * a_n.to_real_64).truncated_to_integer_64.to_natural_32
						if a_x >= a_range[l_i_linear.to_integer_32] then
							if a_x < a_range[(l_i_linear + {NATURAL_32} 1).to_integer_32] then
								a_i.item := l_i_linear
								Result := 0
								l_return := True
							end
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						l_upper := a_n
						l_lower := (0).to_natural_32
						from
						until
							not (l_upper - l_lower > {NATURAL_32} 1)
						loop
							l_mid := (l_upper + l_lower) // {NATURAL_32} 2
							if a_x >= a_range[l_mid.to_integer_32] then
								l_lower := l_mid
							else
								l_upper := l_mid
							end
						end
						a_i.item := l_lower
						if not (a_x < a_range[l_lower.to_integer_32]) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_x >= a_range[(l_lower + {NATURAL_32} 1).to_integer_32]) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("x not found in range"), ce_string ("find.c"), 81, 7)
							Result := 7
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			n := a_other.n
			range := a_other.range
			sum := a_other.sum
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				n = a_other.n and
				range.is_equal (a_other.range) and
				sum.is_equal (a_other.sum)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("n"),
					create {CE_STRUCTURE_FIELD}.make ("range"),
					create {CE_STRUCTURE_FIELD}.make ("sum")
				>>)
			init_structure_info
		end

end
