class
	PD_GSL_INTERP_TYPE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_INTEGER_8_POINTER} name.make
			create alloc.make
			create init.make
			create eval.make
			create eval_deriv.make
			create eval_deriv2.make
			create eval_integ.make
			create free2.make
			Precursor
		end

feature {ANY} -- fields

	name: CE_POINTER [INTEGER_8] assign set_name

	min_size: NATURAL_32 assign set_min_size

	alloc: CE_ROUTINE assign set_alloc

	init: CE_ROUTINE assign set_init

	eval: CE_ROUTINE assign set_eval

	eval_deriv: CE_ROUTINE assign set_eval_deriv

	eval_deriv2: CE_ROUTINE assign set_eval_deriv2

	eval_integ: CE_ROUTINE assign set_eval_integ

	free2: CE_ROUTINE assign set_free2

feature {ANY} -- field setters

	set_name (a_name: CE_POINTER [INTEGER_8])
		do
			name := a_name
			update_memory_field ("name")
		end

	set_min_size (a_min_size: NATURAL_32)
		do
			min_size := a_min_size
			update_memory_field ("min_size")
		end

	set_alloc (a_alloc: CE_ROUTINE)
		do
			alloc := a_alloc
			update_memory_field ("alloc")
		end

	set_init (a_init: CE_ROUTINE)
		do
			init := a_init
			update_memory_field ("init")
		end

	set_eval (a_eval: CE_ROUTINE)
		do
			eval := a_eval
			update_memory_field ("eval")
		end

	set_eval_deriv (a_eval_deriv: CE_ROUTINE)
		do
			eval_deriv := a_eval_deriv
			update_memory_field ("eval_deriv")
		end

	set_eval_deriv2 (a_eval_deriv2: CE_ROUTINE)
		do
			eval_deriv2 := a_eval_deriv2
			update_memory_field ("eval_deriv2")
		end

	set_eval_integ (a_eval_integ: CE_ROUTINE)
		do
			eval_integ := a_eval_integ
			update_memory_field ("eval_integ")
		end

	set_free2 (a_free2: CE_ROUTINE)
		do
			free2 := a_free2
			update_memory_field ("free2")
		end

feature {ANY} -- public routines

	gsl_interp_alloc (a_size: NATURAL_32): CE_POINTER [PD_GSL_INTERP]
		local
			l_interp: CE_POINTER [PD_GSL_INTERP]
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			if a_size < min_size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("insufficient number of points for interpolation type"), ce_string ("interp.c"), 38, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_INTERP]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_tmp := malloc ((create {PD_GSL_INTERP}.make).structure_size.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_interp := create {CE_CLASS_POINTER [PD_GSL_INTERP]}.make_cast (l_tmp)
				if l_interp.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for interp struct"), ce_string ("interp.c"), 46, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_INTERP]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_interp.item.type := (create {CE_CLASS_POINTER [PD_GSL_INTERP_TYPE]}.make_shared (c_pointer))
				update_class_fields -- Current might have been changed; synchronize
				l_interp.item.size := a_size
				update_class_fields -- Current might have been changed; synchronize
				if ((l_interp.item.type).item.alloc).is_void then
					l_interp.item.state := create {CE_VOID_POINTER}.make_shared (default_pointer)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_interp
					l_return := True
				end
			end
			if not l_return then
				l_interp.item.state := create {CE_VOID_POINTER}.make_cast (((l_interp.item.type).item.alloc).pointer_item ([a_size]))
				update_class_fields -- Current might have been changed; synchronize
				if (l_interp.item.state).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_interp))
					update_class_fields -- Current might have been changed; synchronize
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for interp state"), ce_string ("interp.c"), 63, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_INTERP]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				Result := l_interp
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			name := a_other.name
			min_size := a_other.min_size
			alloc := a_other.alloc
			init := a_other.init
			eval := a_other.eval
			eval_deriv := a_other.eval_deriv
			eval_deriv2 := a_other.eval_deriv2
			eval_integ := a_other.eval_integ
			free2 := a_other.free2
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				name.is_equal (a_other.name) and
				min_size = a_other.min_size and
				alloc.is_equal (a_other.alloc) and
				init.is_equal (a_other.init) and
				eval.is_equal (a_other.eval) and
				eval_deriv.is_equal (a_other.eval_deriv) and
				eval_deriv2.is_equal (a_other.eval_deriv2) and
				eval_integ.is_equal (a_other.eval_integ) and
				free2.is_equal (a_other.free2)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("name"),
					create {CE_STRUCTURE_FIELD}.make ("min_size"),
					create {CE_STRUCTURE_FIELD}.make ("alloc"),
					create {CE_STRUCTURE_FIELD}.make ("init"),
					create {CE_STRUCTURE_FIELD}.make ("eval"),
					create {CE_STRUCTURE_FIELD}.make ("eval_deriv"),
					create {CE_STRUCTURE_FIELD}.make ("eval_deriv2"),
					create {CE_STRUCTURE_FIELD}.make ("eval_integ"),
					create {CE_STRUCTURE_FIELD}.make ("free2")
				>>)
			init_structure_info
		end

end
