class
	PD_GSL_MULTIMIN_FMINIMIZER

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_MULTIMIN_FMINIMIZER_TYPE]} type.make
			create {CE_CLASS_POINTER [PD_GSL_MULTIMIN_FUNCTION_STRUCT]} f.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} x.make
			create {CE_VOID_POINTER} state.make
		end

feature {ANY} -- fields

	type: CE_POINTER [PD_GSL_MULTIMIN_FMINIMIZER_TYPE] assign set_type

	f: CE_POINTER [PD_GSL_MULTIMIN_FUNCTION_STRUCT] assign set_f

	fval: REAL_64 assign set_fval

	x: CE_POINTER [PD_GSL_VECTOR] assign set_x

	size: REAL_64 assign set_size

	state: CE_POINTER [ANY] assign set_state

feature {ANY} -- field setters

	set_type (a_type: CE_POINTER [PD_GSL_MULTIMIN_FMINIMIZER_TYPE])
		do
			type := a_type
			update_memory_field ("type")
		end

	set_f (a_f: CE_POINTER [PD_GSL_MULTIMIN_FUNCTION_STRUCT])
		do
			f := a_f
			update_memory_field ("f")
		end

	set_fval (a_fval: REAL_64)
		do
			fval := a_fval
			update_memory_field ("fval")
		end

	set_x (a_x: CE_POINTER [PD_GSL_VECTOR])
		do
			x := a_x
			update_memory_field ("x")
		end

	set_size (a_size: REAL_64)
		do
			size := a_size
			update_memory_field ("size")
		end

	set_state (a_state: CE_POINTER [ANY])
		do
			state := a_state
			update_memory_field ("state")
		end

feature {ANY} -- public routines

	gsl_multimin_fminimizer_set (a_f: CE_POINTER [PD_GSL_MULTIMIN_FUNCTION_STRUCT]; a_x, a_step_size: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_f.is_void
		local
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if x.item.size /= a_f.item.n then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("function incompatible with solver size"), ce_string ("fminimizer.c"), 81, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						if a_x.item.size = a_f.item.n then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if a_step_size.item.size = a_f.item.n then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length not compatible with function"), ce_string ("fminimizer.c"), 86, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						Current.f := a_f
						update_class_fields -- Current might have been changed; synchronize
						x.item.gsl_vector_memcpy (a_x).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_tmp := (type.item.set).integer_32_item ([state, f, x, create {CE_REAL_64_POINTER}.make_shared (c_pointer + (create {PD_GSL_MULTIMIN_FMINIMIZER}.make).offset_of ("size")), a_step_size])
						update_class_fields -- Current might have been changed; synchronize
						Result := l_tmp
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_multimin_fminimizer_iterate: INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := (type.item.iterate).integer_32_item ([state, f, x, create {CE_REAL_64_POINTER}.make_shared (c_pointer + (create {PD_GSL_MULTIMIN_FMINIMIZER}.make).offset_of ("size")), create {CE_REAL_64_POINTER}.make_shared (c_pointer + (create {PD_GSL_MULTIMIN_FMINIMIZER}.make).offset_of ("fval"))])
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_multimin_fminimizer_name: CE_POINTER [INTEGER_8]
		do
			Result := (type.item.name)
		end

	gsl_multimin_fminimizer_x: CE_POINTER [PD_GSL_VECTOR]
		do
			Result := x
		end

	gsl_multimin_fminimizer_minimum: REAL_64
		do
			Result := fval
		end

	gsl_multimin_fminimizer_size: REAL_64
		do
			Result := size
		end

feature {ANY}

	copy (a_other: like Current)
		do
			type := a_other.type
			f := a_other.f
			fval := a_other.fval
			x := a_other.x
			size := a_other.size
			state := a_other.state
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				type.is_equal (a_other.type) and
				f.is_equal (a_other.f) and
				fval = a_other.fval and
				x.is_equal (a_other.x) and
				size = a_other.size and
				state.is_equal (a_other.state)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("type"),
					create {CE_STRUCTURE_FIELD}.make ("f"),
					create {CE_STRUCTURE_FIELD}.make ("fval"),
					create {CE_STRUCTURE_FIELD}.make ("x"),
					create {CE_STRUCTURE_FIELD}.make ("size"),
					create {CE_STRUCTURE_FIELD}.make ("state")
				>>)
			init_structure_info
		end

end
