class
	PD_GSL_MULTISET_STRUCT

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STDIO_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_NATURAL_32_POINTER} data.make
			Precursor
		end

feature {ANY} -- fields

	n: NATURAL_32 assign set_n

	k: NATURAL_32 assign set_k

	data: CE_POINTER [NATURAL_32] assign set_data

feature {ANY} -- field setters

	set_n (a_n: NATURAL_32)
		do
			n := a_n
			update_memory_field ("n")
		end

	set_k (a_k: NATURAL_32)
		do
			k := a_k
			update_memory_field ("k")
		end

	set_data (a_data: CE_POINTER [NATURAL_32])
		do
			data := a_data
			update_memory_field ("data")
		end

feature {ANY} -- public routines

	gsl_multiset_get (a_i: NATURAL_32): NATURAL_32
		local
			l_return, l_break: BOOLEAN
		do
			if ((create {P_VECTOR_STATIC}).gsl_check_range.item).to_boolean then
				if a_i >= k then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("index out of range"), ce_string ("../gsl/gsl_multiset.h"), 83, 4)
						update_class_fields -- Current might have been changed; synchronize
						Result := (0).to_natural_32
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				Result := (data[a_i.to_integer_32])
			end
		end

	gsl_multiset_fread (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_k: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_items: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_k := k
			l_data := data
			l_tmp := fread (create {CE_VOID_POINTER}.make_cast (l_data), {PLATFORM}.Natural_32_bytes.to_natural_32, l_k, a_stream)
			update_class_fields -- Current might have been changed; synchronize
			l_items := l_tmp
			if l_items /= l_k then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("fread failed"), ce_string ("file.c"), 41, 5)
					update_class_fields -- Current might have been changed; synchronize
					Result := 5
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_multiset_fwrite (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_k: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_items: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_k := k
			l_data := data
			l_tmp := fwrite (create {CE_VOID_POINTER}.make_cast (l_data), {PLATFORM}.Natural_32_bytes.to_natural_32, l_k, a_stream)
			update_class_fields -- Current might have been changed; synchronize
			l_items := l_tmp
			if l_items /= l_k then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("fwrite failed"), ce_string ("file.c"), 58, 5)
					update_class_fields -- Current might have been changed; synchronize
					Result := 5
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_multiset_fprintf (a_stream: CE_POINTER [SD_IO_FILE]; a_format: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_k: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_i: NATURAL_32
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_k := k
			l_data := data
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_k))
			loop
				l_tmp := fprintf ([a_stream, a_format, l_data[l_i.to_integer_32]])
				update_class_fields -- Current might have been changed; synchronize
				l_status := l_tmp
				if l_status < 0 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file.c"), 79, 5)
						update_class_fields -- Current might have been changed; synchronize
						Result := 5
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_multiset_fscanf (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_k: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_i: NATURAL_32
			l_j: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_j.make (<<1>>)

			l_k := k
			l_data := data
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_k))
			loop
				l_tmp := fscanf ([a_stream, ce_string ("%%lu"), l_j])
				update_class_fields -- Current might have been changed; synchronize
				l_status := l_tmp
				if l_status /= 1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("fscanf failed"), ce_string ("file.c"), 108, 5)
						update_class_fields -- Current might have been changed; synchronize
						Result := 5
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					l_data[l_i.to_integer_32] := (l_j.item).to_natural_32
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_multiset_n: NATURAL_32
		do
			Result := n
		end

	gsl_multiset_k: NATURAL_32
		do
			Result := k
		end

	gsl_multiset_data: CE_POINTER [NATURAL_32]
		do
			Result := data
		end

	gsl_multiset_valid: INTEGER_32
		local
			l_n: NATURAL_32
			l_k: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_ci: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := n
			l_k := k
			if l_k > l_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("multiset has k greater than n"), ce_string ("multiset.c"), 55, -1)
					update_class_fields -- Current might have been changed; synchronize
					Result := (-1)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_k))
				loop
					l_ci := data[l_i.to_integer_32]
					if l_ci >= l_n then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("multiset index outside range"), ce_string ("multiset.c"), 63, -1)
							update_class_fields -- Current might have been changed; synchronize
							Result := (-1)
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_j := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_j < l_i))
						loop
							if data[l_j.to_integer_32] > l_ci then
								from
								until
									(l_return or l_break)
								loop
									l_break := False;
									(create {P_ERROR_STATIC}).gsl_error (ce_string ("multiset indices not in increasing order"), ce_string ("multiset.c"), 71, -1)
									update_class_fields -- Current might have been changed; synchronize
									Result := (-1)
									l_return := True
									if not l_return then
										l_break := True
									end
								end
								if not l_return then
									l_break := False
								end
							end
							if not l_return then
								l_j := l_j + 1
							end
						end
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_multiset_next: INTEGER_32
		local
			l_n: NATURAL_32
			l_k: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := n
			l_k := k
			l_data := data
			if l_k = {NATURAL_32} 0 then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				l_i := l_k - {NATURAL_32} 1
				from
				until
					(l_break)
				loop
					l_break := False
					if l_i > {NATURAL_32} 0 then
						if l_data[l_i.to_integer_32] /= l_n - {NATURAL_32} 1 then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						l_i := l_i - 1
					end
				end
				l_break := False
				if l_i = {NATURAL_32} 0 then
					if l_data[0] = l_n - {NATURAL_32} 1 then
						Result := (-1)
						l_return := True
					end
				end
			end
			if not l_return then
				l_data[l_i.to_integer_32] := (l_data[l_i.to_integer_32]) + 1
				update_class_fields -- Current might have been changed; synchronize
				from
				until
					not (l_i < l_k - {NATURAL_32} 1)
				loop
					l_data[(l_i + {NATURAL_32} 1).to_integer_32] := l_data[l_i.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_multiset_prev: INTEGER_32
		local
			l_n: NATURAL_32
			l_k: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := n
			l_k := k
			l_data := data
			if l_k = {NATURAL_32} 0 then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				l_i := l_k - {NATURAL_32} 1
				from
				until
					(l_break)
				loop
					l_break := False
					if l_i > {NATURAL_32} 0 then
						if l_data[(l_i - {NATURAL_32} 1).to_integer_32] /= l_data[l_i.to_integer_32] then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						l_i := l_i - 1
					end
				end
				l_break := False
				if l_i = {NATURAL_32} 0 then
					if l_data[l_i.to_integer_32] = {NATURAL_32} 0 then
						Result := (-1)
						l_return := True
					end
				end
			end
			if not l_return then
				l_data[l_i.to_integer_32] := (l_data[l_i.to_integer_32]) - 1
				update_class_fields -- Current might have been changed; synchronize
				if l_data[l_i.to_integer_32] < l_n - {NATURAL_32} 1 then
					from
					until
						not (l_i < l_k - {NATURAL_32} 1)
					loop
						l_i := l_i + 1
						l_data[l_i.to_integer_32] := l_n - {NATURAL_32} 1
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				Result := 0
			end
		end

	gsl_multiset_memcpy (a_src: CE_POINTER [PD_GSL_MULTISET_STRUCT]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_n: NATURAL_32
			l_src_k: NATURAL_32
			l_dest_n: NATURAL_32
			l_dest_k: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_src_n := a_src.item.n
					l_src_k := a_src.item.k
					l_dest_n := n
					l_dest_k := k
					if l_src_n = l_dest_n then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if l_src_k = l_dest_k then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("multiset lengths are not equal"), ce_string ("multiset.c"), 168, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if done
					if not l_return then
						l_j := (0).to_natural_32
						from
						until
							not (l_j < l_src_k)
						loop
							data[l_j.to_integer_32] := a_src.item.data[l_j.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_j := l_j + 1
						end
						Result := 0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_multiset_init_first
		local
			l_k: NATURAL_32
			l_i: NATURAL_32
		do
			l_k := k
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_k)
			loop
				data[l_i.to_integer_32] := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_multiset_init_last
		local
			l_k: NATURAL_32
			l_i: NATURAL_32
			l_n: NATURAL_32
		do
			l_k := k
			l_n := n
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_k)
			loop
				data[l_i.to_integer_32] := l_n - {NATURAL_32} 1
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			n := a_other.n
			k := a_other.k
			data := a_other.data
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				n = a_other.n and
				k = a_other.k and
				data.is_equal (a_other.data)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("n"),
					create {CE_STRUCTURE_FIELD}.make ("k"),
					create {CE_STRUCTURE_FIELD}.make ("data")
				>>)
			init_structure_info
		end

end
