class
	PD_GSL_PERMUTATION_STRUCT

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STDIO_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_NATURAL_32_POINTER} data.make
			Precursor
		end

feature {ANY} -- fields

	size: NATURAL_32 assign set_size

	data: CE_POINTER [NATURAL_32] assign set_data

feature {ANY} -- field setters

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

	set_data (a_data: CE_POINTER [NATURAL_32])
		do
			data := a_data
			update_memory_field ("data")
		end

feature {ANY} -- public routines

	gsl_permutation_get (a_i: NATURAL_32): NATURAL_32
		do
			Result := (data[a_i.to_integer_32])
		end

	gsl_permute_vector_complex_long_double (a_v: CE_POINTER [PD_GSL_VECTOR_COMPLEX_LONG_DOUBLE]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_complex_long_double (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_complex_long_double_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_COMPLEX_LONG_DOUBLE]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_complex_long_double_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_complex (a_v: CE_POINTER [PD_GSL_VECTOR_COMPLEX]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_complex (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_complex_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_COMPLEX]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_complex_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_complex_float (a_v: CE_POINTER [PD_GSL_VECTOR_COMPLEX_FLOAT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_complex_float (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_complex_float_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_COMPLEX_FLOAT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_complex_float_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_long_double (a_v: CE_POINTER [PD_GSL_VECTOR_LONG_DOUBLE]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_long_double (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_long_double_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_LONG_DOUBLE]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_long_double_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector (a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_float (a_v: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_float (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_float_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_float_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_ulong (a_v: CE_POINTER [PD_GSL_VECTOR_ULONG]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_ulong (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_ulong_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_ULONG]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_ulong_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_long (a_v: CE_POINTER [PD_GSL_VECTOR_LONG]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_long (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_long_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_LONG]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_long_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_uint (a_v: CE_POINTER [PD_GSL_VECTOR_UINT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_uint (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_uint_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_UINT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_uint_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_int (a_v: CE_POINTER [PD_GSL_VECTOR_INT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_int (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_int_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_INT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_int_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_ushort (a_v: CE_POINTER [PD_GSL_VECTOR_USHORT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_ushort (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_ushort_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_USHORT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_ushort_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_short (a_v: CE_POINTER [PD_GSL_VECTOR_SHORT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_short (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_short_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_SHORT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_short_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_uchar (a_v: CE_POINTER [PD_GSL_VECTOR_UCHAR]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_uchar (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_uchar_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_UCHAR]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_uchar_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_char (a_v: CE_POINTER [PD_GSL_VECTOR_CHAR]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 144, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_char (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permute_vector_char_inverse (a_v: CE_POINTER [PD_GSL_VECTOR_CHAR]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_char_inverse (data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permutation_linear_to_canonical (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		require
			not a_p.is_void
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_s: NATURAL_32
			l_t: NATURAL_32
			l_pp: CE_POINTER [NATURAL_32]
			l_qq: CE_POINTER [NATURAL_32]
			l_return, l_break, l_continue: BOOLEAN
		do
			l_n := a_p.item.size
			l_t := l_n
			l_pp := a_p.item.data
			l_qq := data
			if size /= a_p.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of q does not match size of p"), ce_string ("canonical.c"), 42, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return or l_break) and then (l_i < l_n))
				loop
					l_break := False
					l_continue := False
					l_k := l_pp[l_i.to_integer_32]
					l_s := (1).to_natural_32
					from
					until
						not (l_k > l_i)
					loop
						l_k := l_pp[l_k.to_integer_32]
						l_s := l_s + 1
					end
					if l_k < l_i then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						l_t := l_t - l_s
						l_qq[l_t.to_integer_32] := l_i
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_pp[l_i.to_integer_32]
						l_s := (1).to_natural_32
						from
						until
							not (l_k > l_i)
						loop
							l_qq[(l_t + l_s).to_integer_32] := l_k
							update_class_fields -- Current might have been changed; synchronize
							l_k := l_pp[l_k.to_integer_32]
							l_s := l_s + 1
						end
						if l_t = {NATURAL_32} 0 then
							l_break := True
						end
					end
					if (not l_return and not l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
				l_continue := False
				Result := 0
			end
		end

	gsl_permutation_canonical_to_linear (a_q___3: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		require
			not a_q___3.is_void
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_kk: NATURAL_32
			l_first: NATURAL_32
			l_n: NATURAL_32
			l_pp: CE_POINTER [NATURAL_32]
			l_qq: CE_POINTER [NATURAL_32]
			l_return, l_break: BOOLEAN
		do
			l_n := size
			l_pp := data
			l_qq := a_q___3.item.data
			if a_q___3.item.size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of q does not match size of p"), ce_string ("canonical.c"), 95, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_pp[l_i.to_integer_32] := l_i
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_k := l_qq[0]
				l_first := l_pp[l_k.to_integer_32]
				l_i := (1).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_kk := l_qq[l_i.to_integer_32]
					if l_kk > l_first then
						l_pp[l_k.to_integer_32] := l_pp[l_kk.to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_kk
					else
						l_pp[l_k.to_integer_32] := l_first
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_kk
						l_first := l_pp[l_kk.to_integer_32]
					end
					l_i := l_i + 1
				end
				l_pp[l_k.to_integer_32] := l_first
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_permutation_inversions: NATURAL_32
		local
			l_count: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_size: NATURAL_32
			l_return: BOOLEAN
		do
			l_count := (0).to_natural_32
			l_size := size
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size - {NATURAL_32} 1))
			loop
				l_j := l_i + {NATURAL_32} 1
				from
				until
					not (l_j < l_size)
				loop
					if data[l_i.to_integer_32] > data[l_j.to_integer_32] then
						l_count := l_count + 1
					end
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			Result := l_count
		end

	gsl_permutation_linear_cycles: NATURAL_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_count: NATURAL_32
			l_size: NATURAL_32
			l_return, l_continue: BOOLEAN
		do
			l_count := (0).to_natural_32
			l_size := size
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size))
			loop
				l_continue := False
				l_k := data[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := data[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_count := l_count + 1
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := l_count
		end

	gsl_permutation_canonical_cycles: NATURAL_32
		local
			l_i: NATURAL_32
			l_count: NATURAL_32
			l_min: NATURAL_32
		do
			l_count := (1).to_natural_32
			l_min := data[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < size)
			loop
				if data[l_i.to_integer_32] < l_min then
					l_min := data[l_i.to_integer_32]
					l_count := l_count + 1
				end
				l_i := l_i + 1
			end
			Result := l_count
		end

	gsl_permutation_size: NATURAL_32
		do
			Result := size
		end

	gsl_permutation_data: CE_POINTER [NATURAL_32]
		do
			Result := data
		end

	gsl_permutation_swap (a_i, a_j: NATURAL_32): INTEGER_32
		local
			l_size: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_size := size
			if a_i >= l_size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first index is out of range"), ce_string ("permutation.c"), 43, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second index is out of range"), ce_string ("permutation.c"), 48, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_i /= a_j then
					l_tmp := data[a_i.to_integer_32]
					data[a_i.to_integer_32] := data[a_j.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					data[a_j.to_integer_32] := l_tmp
					update_class_fields -- Current might have been changed; synchronize
				end
				Result := 0
			end
		end

	gsl_permutation_valid: INTEGER_32
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_size := size
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size))
			loop
				if data[l_i.to_integer_32] >= l_size then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation index outside range"), ce_string ("permutation.c"), 73, -1)
						update_class_fields -- Current might have been changed; synchronize
						Result := (-1)
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					l_j := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_j < l_i))
					loop
						if data[l_i.to_integer_32] = data[l_j.to_integer_32] then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("duplicate permutation index"), ce_string ("permutation.c"), 80, -1)
								update_class_fields -- Current might have been changed; synchronize
								Result := (-1)
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
						if not l_return then
							l_j := l_j + 1
						end
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_permutation_reverse
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_tmp: NATURAL_32
		do
			l_size := size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_size // {NATURAL_32} 2)
			loop
				l_j := (l_size - l_i) - {NATURAL_32} 1
				l_tmp := data[l_i.to_integer_32]
				data[l_i.to_integer_32] := data[l_j.to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				data[l_j.to_integer_32] := l_tmp
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_permutation_inverse (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		require
			not a_p.is_void
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_size := a_p.item.size
			if size /= l_size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation lengths are not equal"), ce_string ("permutation.c"), 114, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_size)
				loop
					data[(a_p.item.data[l_i.to_integer_32]).to_integer_32] := l_i
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_permutation_next: INTEGER_32
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_tmp___0: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_size := size
			if l_size < {NATURAL_32} 2 then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				l_i := l_size - {NATURAL_32} 2
				from
				until
					(l_break)
				loop
					l_break := False
					if data[l_i.to_integer_32] > data[(l_i + {NATURAL_32} 1).to_integer_32] then
						if l_i = {NATURAL_32} 0 then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						l_i := l_i - 1
					end
				end
				l_break := False
				if l_i = {NATURAL_32} 0 then
					if data[0] > data[1] then
						Result := (-1)
						l_return := True
					end
				end
			end
			if not l_return then
				l_k := l_i + {NATURAL_32} 1
				l_j := l_i + {NATURAL_32} 2
				from
				until
					not (l_j < l_size)
				loop
					if data[l_j.to_integer_32] > data[l_i.to_integer_32] then
						if data[l_j.to_integer_32] < data[l_k.to_integer_32] then
							l_k := l_j
						end
					end
					l_j := l_j + 1
				end
				l_tmp := data[l_i.to_integer_32]
				data[l_i.to_integer_32] := data[l_k.to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				data[l_k.to_integer_32] := l_tmp
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_i + {NATURAL_32} 1
				from
				until
					not (l_j <= (l_size + l_i) // {NATURAL_32} 2)
				loop
					l_tmp___0 := data[l_j.to_integer_32]
					data[l_j.to_integer_32] := data[((l_size + l_i) - l_j).to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					data[((l_size + l_i) - l_j).to_integer_32] := l_tmp___0
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_permutation_prev: INTEGER_32
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_tmp___0: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_size := size
			if l_size < {NATURAL_32} 2 then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				l_i := l_size - {NATURAL_32} 2
				from
				until
					(l_break)
				loop
					l_break := False
					if data[l_i.to_integer_32] < data[(l_i + {NATURAL_32} 1).to_integer_32] then
						if l_i = {NATURAL_32} 0 then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						l_i := l_i - 1
					end
				end
				l_break := False
				if l_i = {NATURAL_32} 0 then
					if data[0] < data[1] then
						Result := (-1)
						l_return := True
					end
				end
			end
			if not l_return then
				l_k := l_i + {NATURAL_32} 1
				l_j := l_i + {NATURAL_32} 2
				from
				until
					not (l_j < l_size)
				loop
					if data[l_j.to_integer_32] < data[l_i.to_integer_32] then
						if data[l_j.to_integer_32] > data[l_k.to_integer_32] then
							l_k := l_j
						end
					end
					l_j := l_j + 1
				end
				l_tmp := data[l_i.to_integer_32]
				data[l_i.to_integer_32] := data[l_k.to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				data[l_k.to_integer_32] := l_tmp
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_i + {NATURAL_32} 1
				from
				until
					not (l_j <= (l_size + l_i) // {NATURAL_32} 2)
				loop
					l_tmp___0 := data[l_j.to_integer_32]
					data[l_j.to_integer_32] := data[((l_size + l_i) - l_j).to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					data[((l_size + l_i) - l_j).to_integer_32] := l_tmp___0
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_permutation_mul (a_pa, a_pb: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		require
			not a_pa.is_void
		local
			l_i: NATURAL_32
			l_size: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_size := size
			if a_pa.item.size /= l_size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of result does not match size of pa"), ce_string ("permutation.c"), 238, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_pb.item.size /= l_size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of result does not match size of pb"), ce_string ("permutation.c"), 243, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_size)
				loop
					data[l_i.to_integer_32] := a_pb.item.data[(a_pa.item.data[l_i.to_integer_32]).to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_permutation_memcpy (a_src: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_size: NATURAL_32
			l_dest_size: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_src_size := a_src.item.size
			l_dest_size := size
			if l_src_size /= l_dest_size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation lengths are not equal"), ce_string ("permutation.c"), 262, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_src_size)
				loop
					data[l_j.to_integer_32] := a_src.item.data[l_j.to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_permutation_fread (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_n: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_items: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := size
			l_data := data
			l_tmp := fread (create {CE_VOID_POINTER}.make_cast (l_data), {PLATFORM}.Natural_32_bytes.to_natural_32, l_n, a_stream)
			update_class_fields -- Current might have been changed; synchronize
			l_items := l_tmp
			if l_items /= l_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("fread failed"), ce_string ("file.c"), 38, 5)
					update_class_fields -- Current might have been changed; synchronize
					Result := 5
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_permutation_fwrite (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_n: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_items: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := size
			l_data := data
			l_tmp := fwrite (create {CE_VOID_POINTER}.make_cast (l_data), {PLATFORM}.Natural_32_bytes.to_natural_32, l_n, a_stream)
			update_class_fields -- Current might have been changed; synchronize
			l_items := l_tmp
			if l_items /= l_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("fwrite failed"), ce_string ("file.c"), 55, 5)
					update_class_fields -- Current might have been changed; synchronize
					Result := 5
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_permutation_fprintf (a_stream: CE_POINTER [SD_IO_FILE]; a_format: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_n: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_i: NATURAL_32
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_n := size
			l_data := data
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_n))
			loop
				l_tmp := fprintf ([a_stream, a_format, l_data[l_i.to_integer_32]])
				update_class_fields -- Current might have been changed; synchronize
				l_status := l_tmp
				if l_status < 0 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("fprintf failed"), ce_string ("file.c"), 76, 5)
						update_class_fields -- Current might have been changed; synchronize
						Result := 5
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_permutation_fscanf (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_n: NATURAL_32
			l_data: CE_POINTER [NATURAL_32]
			l_i: NATURAL_32
			l_j: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_j.make (<<1>>)

			l_n := size
			l_data := data
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_n))
			loop
				l_tmp := fscanf ([a_stream, ce_string ("%%lu"), l_j])
				update_class_fields -- Current might have been changed; synchronize
				l_status := l_tmp
				if l_status /= 1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("fscanf failed"), ce_string ("file.c"), 105, 5)
						update_class_fields -- Current might have been changed; synchronize
						Result := 5
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					l_data[l_i.to_integer_32] := (l_j.item).to_natural_32
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_permutation_init
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
		do
			l_n := size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				data[l_i.to_integer_32] := l_i
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_sort_vector_long_double_index (a_v: CE_POINTER [PD_GSL_VECTOR_LONG_DOUBLE]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_long_double_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_index (a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_float_index (a_v: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_float_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_ulong_index (a_v: CE_POINTER [PD_GSL_VECTOR_ULONG]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_ulong_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_long_index (a_v: CE_POINTER [PD_GSL_VECTOR_LONG]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_long_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_uint_index (a_v: CE_POINTER [PD_GSL_VECTOR_UINT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_uint_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_int_index (a_v: CE_POINTER [PD_GSL_VECTOR_INT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_int_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_ushort_index (a_v: CE_POINTER [PD_GSL_VECTOR_USHORT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_ushort_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_short_index (a_v: CE_POINTER [PD_GSL_VECTOR_SHORT]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_short_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_uchar_index (a_v: CE_POINTER [PD_GSL_VECTOR_UCHAR]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_uchar_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_sort_vector_char_index (a_v: CE_POINTER [PD_GSL_VECTOR_CHAR]): INTEGER_32
		require
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size /= a_v.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_sort_char_index (data, a_v.item.data, a_v.item.stride, a_v.item.size)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

feature {PD_GSL_PERMUTATION_STRUCT} -- private routines

	gsl_permute_complex_long_double (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_96]
			l_r1___1: REAL_96
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_96_ARRAY} l_t.make (<<2>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						l_t[l_a___5.to_integer_32] := a_data[((l_i * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 2)
						loop
							l_r1___1 := a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
							a_data[((l_k * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						a_data[((l_k * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_complex_long_double_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_96]
			l_r1___1: REAL_96
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_96_ARRAY} l_t.make (<<2>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						l_t[l_a___5.to_integer_32] := a_data[((l_k * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 2)
						loop
							l_r1___1 := a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
							a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_complex (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_64]
			l_r1___1: REAL_64
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_t.make (<<2>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						l_t[l_a___5.to_integer_32] := a_data[((l_i * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 2)
						loop
							l_r1___1 := a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
							a_data[((l_k * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						a_data[((l_k * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_complex_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_64]
			l_r1___1: REAL_64
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_t.make (<<2>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						l_t[l_a___5.to_integer_32] := a_data[((l_k * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 2)
						loop
							l_r1___1 := a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
							a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_complex_float (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_32]
			l_r1___1: REAL_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_32_ARRAY} l_t.make (<<2>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						l_t[l_a___5.to_integer_32] := a_data[((l_i * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 2)
						loop
							l_r1___1 := a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
							a_data[((l_k * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						a_data[((l_k * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_complex_float_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_32]
			l_r1___1: REAL_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_32_ARRAY} l_t.make (<<2>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						l_t[l_a___5.to_integer_32] := a_data[((l_k * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 2)
						loop
							l_r1___1 := a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32]
							a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 2)
					loop
						a_data[((l_pk * a_stride) * {NATURAL_32} 2 + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_long_double (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_96]
			l_r1___1: REAL_96
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_96_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_long_double_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_96]
			l_r1___1: REAL_96
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_96_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_64]
			l_r1___1: REAL_64
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_float (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_32]
			l_r1___1: REAL_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_float_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_32]
			l_r1___1: REAL_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_ulong (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [NATURAL_32]
			l_r1___1: NATURAL_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_ulong_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [NATURAL_32]
			l_r1___1: NATURAL_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_long (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [INTEGER_32]
			l_r1___1: INTEGER_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_long_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [INTEGER_32]
			l_r1___1: INTEGER_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_uint (a_p, a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [NATURAL_32]
			l_r1___1: NATURAL_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_uint_inverse (a_p, a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [NATURAL_32]
			l_r1___1: NATURAL_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_int (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [INTEGER_32]
			l_r1___1: INTEGER_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_int_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [INTEGER_32]
			l_r1___1: INTEGER_32
			l_return, l_continue: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_ushort (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [NATURAL_16]
			l_r1___1: NATURAL_16
			l_return, l_continue: BOOLEAN
		do
			create {CE_NATURAL_16_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_ushort_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [NATURAL_16]
			l_r1___1: NATURAL_16
			l_return, l_continue: BOOLEAN
		do
			create {CE_NATURAL_16_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_short (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [INTEGER_16]
			l_r1___1: INTEGER_16
			l_return, l_continue: BOOLEAN
		do
			create {CE_INTEGER_16_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_short_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [INTEGER_16]
			l_r1___1: INTEGER_16
			l_return, l_continue: BOOLEAN
		do
			create {CE_INTEGER_16_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_uchar (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [NATURAL_8]
			l_r1___1: NATURAL_8
			l_return, l_continue: BOOLEAN
		do
			create {CE_NATURAL_8_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_uchar_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [NATURAL_8]
			l_r1___1: NATURAL_8
			l_return, l_continue: BOOLEAN
		do
			create {CE_NATURAL_8_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_char (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [INTEGER_8]
			l_r1___1: INTEGER_8
			l_return, l_continue: BOOLEAN
		do
			create {CE_INTEGER_8_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_i * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_k * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_permute_char_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [INTEGER_8]
			l_r1___1: INTEGER_8
			l_return, l_continue: BOOLEAN
		do
			create {CE_INTEGER_8_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_sort_long_double_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_long_double_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_long_double_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_long_double_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_96]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_64]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_float_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_float_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_float_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_float_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_32]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_ulong_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_ulong_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_ulong_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_ulong_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_32]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_long_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_long_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_long_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_long_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_uint_index (a_p, a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_uint_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_uint_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_uint_downheap (a_p, a_data: CE_POINTER [NATURAL_32]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_int_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_int_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_int_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_int_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_ushort_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_ushort_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_ushort_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_ushort_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_16]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]).to_integer_32 then
						l_j := l_j + 1
					end
				end
				if not ((a_data[(l_pki * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_short_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_short_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_short_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_short_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_16]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]).to_integer_32 then
						l_j := l_j + 1
					end
				end
				if not ((a_data[(l_pki * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_uchar_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_uchar_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_uchar_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_uchar_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_8]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]).to_integer_32 then
						l_j := l_j + 1
					end
				end
				if not ((a_data[(l_pki * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_char_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					index_char_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_char_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	index_char_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_8]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_p.is_void
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]).to_integer_32 then
						l_j := l_j + 1
					end
				end
				if not ((a_data[(l_pki * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32) then
					l_break := True
				end
				if not (l_break) then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_p[l_k.to_integer_32] := l_pki
		end

feature {ANY}

	copy (a_other: like Current)
		do
			size := a_other.size
			data := a_other.data
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				size = a_other.size and
				data.is_equal (a_other.data)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("size"),
					create {CE_STRUCTURE_FIELD}.make ("data")
				>>)
			init_structure_info
		end

end
