class
	PD_GSL_POLY_COMPLEX_WORKSPACE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} matrix.make
			Precursor
		end

feature {ANY} -- fields

	nc: NATURAL_32 assign set_nc

	matrix: CE_POINTER [REAL_64] assign set_matrix

feature {ANY} -- field setters

	set_nc (a_nc: NATURAL_32)
		do
			nc := a_nc
			update_memory_field ("nc")
		end

	set_matrix (a_matrix: CE_POINTER [REAL_64])
		do
			matrix := a_matrix
			update_memory_field ("matrix")
		end

feature {ANY} -- public routines

	gsl_poly_complex_solve (a_a___5: CE_POINTER [REAL_64]; a_n: NATURAL_32; a_z: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_status: INTEGER_32
			l_m___4: CE_POINTER [REAL_64]
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of terms must be a positive integer"), ce_string ("zsolve.c"), 50, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_n = {NATURAL_32} 1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("cannot solve for only one term"), ce_string ("zsolve.c"), 55, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_a___5[(a_n - {NATURAL_32} 1).to_integer_32] = (0).to_double) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("leading term of polynomial must be non-zero"), ce_string ("zsolve.c"), 60, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (nc /= a_n - {NATURAL_32} 1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of workspace does not match polynomial"), ce_string ("zsolve.c"), 65, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_m___4 := matrix
				set_companion_matrix (a_a___5, a_n - {NATURAL_32} 1, l_m___4)
				update_class_fields -- Current might have been changed; synchronize
				balance_companion_matrix (l_m___4, a_n - {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_status := qr_companion (l_m___4, a_n - {NATURAL_32} 1, a_z)
				update_class_fields -- Current might have been changed; synchronize
				if l_status.to_boolean then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("root solving qr method failed to converge"), ce_string ("zsolve.c"), 78, 5)
						update_class_fields -- Current might have been changed; synchronize
						Result := 5
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				Result := 0
			end
		end

feature {PD_GSL_POLY_COMPLEX_WORKSPACE} -- private routines

	set_companion_matrix (a_a___5: CE_POINTER [REAL_64]; a_nc: NATURAL_32; a_m___4: CE_POINTER [REAL_64])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_nc))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < a_nc)
				loop
					a_m___4[(l_i * a_nc + l_j).to_integer_32] := 0.0
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_nc)
			loop
				a_m___4[(l_i * a_nc + (l_i - {NATURAL_32} 1)).to_integer_32] := 1.0
				l_i := l_i + 1
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_nc)
			loop
				a_m___4[(l_i * a_nc + (a_nc - {NATURAL_32} 1)).to_integer_32] := (-a_a___5[l_i.to_integer_32] / a_a___5[a_nc.to_integer_32])
				l_i := l_i + 1
			end
		end

	balance_companion_matrix (a_m___4: CE_POINTER [REAL_64]; a_nc: NATURAL_32)
		local
			l_not_converged: INTEGER_32
			l_row_norm: REAL_64
			l_col_norm: REAL_64
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_g: REAL_64
			l_f: REAL_64
			l_s: REAL_64
			l_tmp___0: REAL_64
			l_return, l_continue: BOOLEAN
		do
			l_not_converged := 1
			l_row_norm := (0).to_double
			l_col_norm := (0).to_double
			from
			until
				not (not (l_return) and then (l_not_converged.to_boolean))
			loop
				l_not_converged := 0
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_nc))
				loop
					l_continue := False
					if l_i /= a_nc - {NATURAL_32} 1 then
						l_col_norm := fabs (a_m___4[((l_i + {NATURAL_32} 1) * a_nc + l_i).to_integer_32])
					else
						l_col_norm := (0).to_double
						l_j := (0).to_natural_32
						from
						until
							not (l_j < a_nc - {NATURAL_32} 1)
						loop
							l_col_norm := l_col_norm + (fabs (a_m___4[(l_j * a_nc + (a_nc - {NATURAL_32} 1)).to_integer_32]))
							l_j := l_j + 1
						end
					end
					if l_i = {NATURAL_32} 0 then
						l_row_norm := fabs (a_m___4[(a_nc - {NATURAL_32} 1).to_integer_32])
					elseif l_i = a_nc - {NATURAL_32} 1 then
						l_row_norm := fabs (a_m___4[(l_i * a_nc + (l_i - {NATURAL_32} 1)).to_integer_32])
					else
						l_tmp___0 := fabs (a_m___4[(l_i * a_nc + (l_i - {NATURAL_32} 1)).to_integer_32])
						l_row_norm := l_tmp___0 + (fabs (a_m___4[(l_i * a_nc + (a_nc - {NATURAL_32} 1)).to_integer_32]))
					end
					if l_col_norm = (0).to_double then
						l_continue := True
					elseif l_row_norm = (0).to_double then
						l_continue := True
					end
					if not (l_return or l_continue) then
						l_g := l_row_norm / (2).to_double
						l_f := (1).to_double
						l_s := l_col_norm + l_row_norm
						from
						until
							not (l_col_norm < l_g)
						loop
							l_f := l_f * (2).to_double
							l_col_norm := l_col_norm * (4).to_double
						end
						l_g := l_row_norm * (2).to_double
						from
						until
							not (l_col_norm > l_g)
						loop
							l_f := l_f / (2).to_double
							l_col_norm := l_col_norm / (4).to_double
						end
						if l_row_norm + l_col_norm < (0.95 * l_s) * l_f then
							l_not_converged := 1
							l_g := (1).to_double / l_f
							if l_i = {NATURAL_32} 0 then
								a_m___4[(a_nc - {NATURAL_32} 1).to_integer_32] := a_m___4[(a_nc - {NATURAL_32} 1).to_integer_32] * l_g
							else
								a_m___4[(l_i * a_nc + (l_i - {NATURAL_32} 1)).to_integer_32] := a_m___4[(l_i * a_nc + (l_i - {NATURAL_32} 1)).to_integer_32] * l_g
								a_m___4[(l_i * a_nc + (a_nc - {NATURAL_32} 1)).to_integer_32] := a_m___4[(l_i * a_nc + (a_nc - {NATURAL_32} 1)).to_integer_32] * l_g
							end
							if l_i = a_nc - {NATURAL_32} 1 then
								l_j := (0).to_natural_32
								from
								until
									not (l_j < a_nc)
								loop
									a_m___4[(l_j * a_nc + l_i).to_integer_32] := a_m___4[(l_j * a_nc + l_i).to_integer_32] * l_f
									l_j := l_j + 1
								end
							else
								a_m___4[((l_i + {NATURAL_32} 1) * a_nc + l_i).to_integer_32] := a_m___4[((l_i + {NATURAL_32} 1) * a_nc + l_i).to_integer_32] * l_f
							end
						end
					end
					if (not l_return) then
						l_i := l_i + 1
					end
				end
				l_continue := False
			end
		end

	qr_companion (a_h: CE_POINTER [REAL_64]; a_nc: NATURAL_32; a_zroot: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_t: REAL_64
			l_iterations: NATURAL_32
			l_e: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_m___4: NATURAL_32
			l_w: REAL_64
			l_x: REAL_64
			l_y: REAL_64
			l_s: REAL_64
			l_z: REAL_64
			l_p: REAL_64
			l_q___3: REAL_64
			l_r___3: REAL_64
			l_notlast: INTEGER_32
			l_n: NATURAL_32
			l_a1___4: REAL_64
			l_a2___3: REAL_64
			l_a3___0: REAL_64
			l_tmp___3: REAL_64
			l_a1___5: REAL_64
			l_a2___4: REAL_64
			l_a3___1: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_return, l_break, l_continue: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_t := 0.0
					l_p := (0).to_double
					l_q___3 := (0).to_double
					l_r___3 := (0).to_double
					l_n := a_nc
				when 1 then -- next_root:
					if l_n = {NATURAL_32} 0 then
						Result := 0
						l_return := True
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 2 then -- if:
						l_iterations := (0).to_natural_32
				when 3 then -- next_iteration:
						l_e := l_n
						from
						until
							not (not (l_break) and then (l_e >= {NATURAL_32} 2))
						loop
							l_break := False
							l_a1___4 := (fabs (a_h[((l_e - {NATURAL_32} 1) * a_nc + ((l_e - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32]))
							l_a2___3 := (fabs (a_h[(((l_e - {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + ((l_e - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32]))
							l_a3___0 := (fabs (a_h[((l_e - {NATURAL_32} 1) * a_nc + (l_e - {NATURAL_32} 1)).to_integer_32]))
							if l_a1___4 <= 2.2204460492503131e-16 * (l_a2___3 + l_a3___0) then
								l_break := True
							end
							if not (l_break) then
								l_e := l_e - 1
							end
						end
						l_break := False
						l_x := a_h[((l_n - {NATURAL_32} 1) * a_nc + (l_n - {NATURAL_32} 1)).to_integer_32]
						if l_e = l_n then
							from
							until
								(l_break)
							loop
								l_break := False
								a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 1)).to_integer_32] := l_x + l_t
								a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 1) + {NATURAL_32} 1).to_integer_32] := (0).to_double
								l_break := True
							end
							l_break := False
							l_n := l_n - 1
							l_pos := -1 + 1 -- goto next_root
						end
				when 4 then
						l_y := a_h[(((l_n - {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + ((l_n - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32]
						l_w := a_h[(((l_n - {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + (l_n - {NATURAL_32} 1)).to_integer_32] * a_h[((l_n - {NATURAL_32} 1) * a_nc + ((l_n - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32]
						if l_e = l_n - {NATURAL_32} 1 then
							l_p := (l_y - l_x) / (2).to_double
							l_q___3 := l_p * l_p + l_w
							l_y := sqrt ((fabs (l_q___3)))
							l_x := l_x + l_t
							if l_q___3 > (0).to_double then
								if l_p < (0).to_double then
									l_y := -l_y
								end
								l_y := l_y + l_p
								from
								until
									(l_break)
								loop
									l_break := False
									a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 1)).to_integer_32] := l_x - l_w / l_y
									a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 1) + {NATURAL_32} 1).to_integer_32] := (0).to_double
									l_break := True
								end
								l_break := False
								from
								until
									(l_break)
								loop
									l_break := False
									a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 2)).to_integer_32] := l_x + l_y
									a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 2) + {NATURAL_32} 1).to_integer_32] := (0).to_double
									l_break := True
								end
								l_break := False
							else
								from
								until
									(l_break)
								loop
									l_break := False
									a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 1)).to_integer_32] := l_x + l_p
									a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 1) + {NATURAL_32} 1).to_integer_32] := -l_y
									l_break := True
								end
								l_break := False
								from
								until
									(l_break)
								loop
									l_break := False
									a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 2)).to_integer_32] := l_x + l_p
									a_zroot[({NATURAL_32} 2 * (l_n - {NATURAL_32} 2) + {NATURAL_32} 1).to_integer_32] := l_y
									l_break := True
								end
								l_break := False
							end
							l_n := l_n - ({NATURAL_32} 2)
							l_pos := -1 + 1 -- goto next_root
						end
				when 5 then
						if l_iterations = {NATURAL_32} 60 then
							Result := (-1)
							l_return := True
						end
				when 6 then -- if done
					if not l_return then
						if l_iterations \\ {NATURAL_32} 10 = {NATURAL_32} 0 then
							if l_iterations > {NATURAL_32} 0 then
								l_t := l_t + l_x
								l_i := (1).to_natural_32
								from
								until
									not (l_i <= l_n)
								loop
									a_h[((l_i - {NATURAL_32} 1) * a_nc + (l_i - {NATURAL_32} 1)).to_integer_32] := a_h[((l_i - {NATURAL_32} 1) * a_nc + (l_i - {NATURAL_32} 1)).to_integer_32] - l_x
									l_i := l_i + 1
								end
								l_tmp___3 := fabs (a_h[((l_n - {NATURAL_32} 1) * a_nc + ((l_n - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32])
								l_s := l_tmp___3 + (fabs (a_h[(((l_n - {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + ((l_n - {NATURAL_32} 2) - {NATURAL_32} 1)).to_integer_32]))
								l_y := 0.75 * l_s
								l_x := l_y
								l_w := (-0.4375 * l_s) * l_s
							end
						end
						l_iterations := l_iterations + 1
						l_m___4 := l_n - {NATURAL_32} 2
						from
						until
							not (not (l_break) and then (l_m___4 >= l_e))
						loop
							l_break := False
							l_z := a_h[((l_m___4 - {NATURAL_32} 1) * a_nc + (l_m___4 - {NATURAL_32} 1)).to_integer_32]
							l_r___3 := l_x - l_z
							l_s := l_y - l_z
							l_p := a_h[((l_m___4 - {NATURAL_32} 1) * a_nc + ((l_m___4 + {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32] + (l_r___3 * l_s - l_w) / a_h[(((l_m___4 + {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + (l_m___4 - {NATURAL_32} 1)).to_integer_32]
							l_q___3 := ((a_h[(((l_m___4 + {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + ((l_m___4 + {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32] - l_z) - l_r___3) - l_s
							l_r___3 := a_h[(((l_m___4 + {NATURAL_32} 2) - {NATURAL_32} 1) * a_nc + ((l_m___4 + {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32]
							l_tmp___5 := fabs (l_p)
							l_tmp___6 := fabs (l_q___3)
							l_s := (l_tmp___5 + l_tmp___6) + (fabs (l_r___3))
							l_p := l_p / l_s
							l_q___3 := l_q___3 / l_s
							l_r___3 := l_r___3 / l_s
							if l_m___4 = l_e then
								l_break := True
							end
							if not (l_break) then
								l_a1___5 := fabs (a_h[((l_m___4 - {NATURAL_32} 1) * a_nc + ((l_m___4 - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32])
								l_a2___4 := fabs (a_h[(((l_m___4 - {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + ((l_m___4 - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32])
								l_a3___1 := fabs (a_h[(((l_m___4 + {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + ((l_m___4 + {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32])
								l_tmp___8 := fabs (l_q___3)
								l_tmp___9 := fabs (l_r___3)
								if l_a1___5 * (l_tmp___8 + l_tmp___9) <= (2.2204460492503131e-16 * (fabs (l_p))) * (l_a2___4 + l_a3___1) then
									l_break := True
								end
							end
							if not (l_break) then
								l_m___4 := l_m___4 - 1
							end
						end
						l_break := False
						l_i := l_m___4 + {NATURAL_32} 2
						from
						until
							not (l_i <= l_n)
						loop
							a_h[((l_i - {NATURAL_32} 1) * a_nc + ((l_i - {NATURAL_32} 2) - {NATURAL_32} 1)).to_integer_32] := (0).to_double
							l_i := l_i + 1
						end
						l_i := l_m___4 + {NATURAL_32} 3
						from
						until
							not (l_i <= l_n)
						loop
							a_h[((l_i - {NATURAL_32} 1) * a_nc + ((l_i - {NATURAL_32} 3) - {NATURAL_32} 1)).to_integer_32] := (0).to_double
							l_i := l_i + 1
						end
						l_k := l_m___4
						from
						until
							not (not (l_return) and then (l_k <= l_n - {NATURAL_32} 1))
						loop
							l_continue := False
							l_notlast := (l_k /= l_n - {NATURAL_32} 1).to_integer
							if l_k /= l_m___4 then
								l_p := a_h[((l_k - {NATURAL_32} 1) * a_nc + ((l_k - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32]
								l_q___3 := a_h[(((l_k + {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + ((l_k - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32]
								if l_notlast.to_boolean then
									l_r___3 := a_h[(((l_k + {NATURAL_32} 2) - {NATURAL_32} 1) * a_nc + ((l_k - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32]
								else
									l_r___3 := 0.0
								end
								l_tmp___11 := fabs (l_p)
								l_tmp___12 := fabs (l_q___3)
								l_x := (l_tmp___11 + l_tmp___12) + (fabs (l_r___3))
								if l_x = (0).to_double then
									l_continue := True
								end
								if not (l_return or l_break or l_continue) then
									l_p := l_p / l_x
									l_q___3 := l_q___3 / l_x
									l_r___3 := l_r___3 / l_x
								end
							end
							if not (l_return or l_break or l_continue) then
								l_s := sqrt ((l_p * l_p + l_q___3 * l_q___3) + l_r___3 * l_r___3)
								if l_p < (0).to_double then
									l_s := -l_s
								end
								if l_k /= l_m___4 then
									a_h[((l_k - {NATURAL_32} 1) * a_nc + ((l_k - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32] := -l_s * l_x
								elseif l_e /= l_m___4 then
									a_h[((l_k - {NATURAL_32} 1) * a_nc + ((l_k - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32] := a_h[((l_k - {NATURAL_32} 1) * a_nc + ((l_k - {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32] * (-1).to_double
								end
								l_p := l_p + l_s
								l_x := l_p / l_s
								l_y := l_q___3 / l_s
								l_z := l_r___3 / l_s
								l_q___3 := l_q___3 / l_p
								l_r___3 := l_r___3 / l_p
								l_j := l_k
								from
								until
									not (l_j <= l_n)
								loop
									l_p := a_h[((l_k - {NATURAL_32} 1) * a_nc + (l_j - {NATURAL_32} 1)).to_integer_32] + l_q___3 * a_h[(((l_k + {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + (l_j - {NATURAL_32} 1)).to_integer_32]
									if l_notlast.to_boolean then
										l_p := l_p + (l_r___3 * a_h[(((l_k + {NATURAL_32} 2) - {NATURAL_32} 1) * a_nc + (l_j - {NATURAL_32} 1)).to_integer_32])
										a_h[(((l_k + {NATURAL_32} 2) - {NATURAL_32} 1) * a_nc + (l_j - {NATURAL_32} 1)).to_integer_32] := a_h[(((l_k + {NATURAL_32} 2) - {NATURAL_32} 1) * a_nc + (l_j - {NATURAL_32} 1)).to_integer_32] - (l_p * l_z)
									end
									a_h[(((l_k + {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + (l_j - {NATURAL_32} 1)).to_integer_32] := a_h[(((l_k + {NATURAL_32} 1) - {NATURAL_32} 1) * a_nc + (l_j - {NATURAL_32} 1)).to_integer_32] - (l_p * l_y)
									a_h[((l_k - {NATURAL_32} 1) * a_nc + (l_j - {NATURAL_32} 1)).to_integer_32] := a_h[((l_k - {NATURAL_32} 1) * a_nc + (l_j - {NATURAL_32} 1)).to_integer_32] - (l_p * l_x)
									l_j := l_j + 1
								end
								if l_k + {NATURAL_32} 3 < l_n then
									l_j := l_k + {NATURAL_32} 3
								else
									l_j := l_n
								end
								l_i := l_e
								from
								until
									not (l_i <= l_j)
								loop
									l_p := l_x * a_h[((l_i - {NATURAL_32} 1) * a_nc + (l_k - {NATURAL_32} 1)).to_integer_32] + l_y * a_h[((l_i - {NATURAL_32} 1) * a_nc + ((l_k + {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32]
									if l_notlast.to_boolean then
										l_p := l_p + (l_z * a_h[((l_i - {NATURAL_32} 1) * a_nc + ((l_k + {NATURAL_32} 2) - {NATURAL_32} 1)).to_integer_32])
										a_h[((l_i - {NATURAL_32} 1) * a_nc + ((l_k + {NATURAL_32} 2) - {NATURAL_32} 1)).to_integer_32] := a_h[((l_i - {NATURAL_32} 1) * a_nc + ((l_k + {NATURAL_32} 2) - {NATURAL_32} 1)).to_integer_32] - (l_p * l_r___3)
									end
									a_h[((l_i - {NATURAL_32} 1) * a_nc + ((l_k + {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32] := a_h[((l_i - {NATURAL_32} 1) * a_nc + ((l_k + {NATURAL_32} 1) - {NATURAL_32} 1)).to_integer_32] - (l_p * l_q___3)
									a_h[((l_i - {NATURAL_32} 1) * a_nc + (l_k - {NATURAL_32} 1)).to_integer_32] := a_h[((l_i - {NATURAL_32} 1) * a_nc + (l_k - {NATURAL_32} 1)).to_integer_32] - l_p
									l_i := l_i + 1
								end
							end
							if (not l_return) then
								l_k := l_k + 1
							end
						end
						l_continue := False
						l_pos := -1 + 3 -- goto next_iteration
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			nc := a_other.nc
			matrix := a_other.matrix
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				nc = a_other.nc and
				matrix.is_equal (a_other.matrix)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("nc"),
					create {CE_STRUCTURE_FIELD}.make ("matrix")
				>>)
			init_structure_info
		end

end
