class
	PD_GSL_QRNG

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_QRNG_TYPE]} type.make
			create {CE_VOID_POINTER} state.make
			Precursor
		end

feature {ANY} -- fields

	type: CE_POINTER [PD_GSL_QRNG_TYPE] assign set_type

	dimension: NATURAL_32 assign set_dimension

	state_size: NATURAL_32 assign set_state_size

	state: CE_POINTER [ANY] assign set_state

feature {ANY} -- field setters

	set_type (a_type: CE_POINTER [PD_GSL_QRNG_TYPE])
		do
			type := a_type
			update_memory_field ("type")
		end

	set_dimension (a_dimension: NATURAL_32)
		do
			dimension := a_dimension
			update_memory_field ("dimension")
		end

	set_state_size (a_state_size: NATURAL_32)
		do
			state_size := a_state_size
			update_memory_field ("state_size")
		end

	set_state (a_state: CE_POINTER [ANY])
		do
			state := a_state
			update_memory_field ("state")
		end

feature {ANY} -- public routines

	gsl_qrng_get (a_x: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := (type.item.get).integer_32_item ([state, dimension, a_x])
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_qrng_init
		do
			(type.item.init_state).call ([state, dimension])
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_qrng_memcpy (a_src: CE_POINTER [PD_GSL_QRNG]): INTEGER_32
		require
			not a_src.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if not type.is_equal (a_src.item.type) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("generators must be of the same type"), ce_string ("qrng.c"), 51, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Current.dimension := a_src.item.dimension
				update_class_fields -- Current might have been changed; synchronize
				Current.state_size := a_src.item.state_size
				update_class_fields -- Current might have been changed; synchronize
				memcpy (state, a_src.item.state, a_src.item.state_size).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_qrng_name: CE_POINTER [INTEGER_8]
		do
			Result := (type.item.name)
		end

	gsl_qrng_size: NATURAL_32
		do
			Result := state_size
		end

	gsl_qrng_state: CE_POINTER [ANY]
		do
			Result := state
		end

feature {ANY}

	copy (a_other: like Current)
		do
			type := a_other.type
			dimension := a_other.dimension
			state_size := a_other.state_size
			state := a_other.state
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				type.is_equal (a_other.type) and
				dimension = a_other.dimension and
				state_size = a_other.state_size and
				state.is_equal (a_other.state)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("type"),
					create {CE_STRUCTURE_FIELD}.make ("dimension"),
					create {CE_STRUCTURE_FIELD}.make ("state_size"),
					create {CE_STRUCTURE_FIELD}.make ("state")
				>>)
			init_structure_info
		end

end
