class
	PD_GSL_RAN_DISCRETE_T

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_NATURAL_32_POINTER} A.make
			create {CE_REAL_64_POINTER} F.make
		end

feature {ANY} -- fields

	K: NATURAL_32 assign set_K

	A: CE_POINTER [NATURAL_32] assign set_A

	F: CE_POINTER [REAL_64] assign set_F

feature {ANY} -- field setters

	set_K (a_K: NATURAL_32)
		do
			K := a_K
			update_memory_field ("K")
		end

	set_A (a_A: CE_POINTER [NATURAL_32])
		do
			A := a_A
			update_memory_field ("A")
		end

	set_F (a_F: CE_POINTER [REAL_64])
		do
			F := a_F
			update_memory_field ("F")
		end

feature {ANY} -- public routines

	gsl_ran_discrete_pdf (a_k: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_K: NATURAL_32
			l_f: REAL_64
			l_p: REAL_64
			l_return: BOOLEAN
		do
			l_p := (0).to_double
			l_K := K
			if a_k > l_K then
				Result := (0).to_double
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_K)
				loop
					l_f := F[l_i.to_integer_32]
					l_f := l_K.to_real_64 * l_f - l_i.to_real_64
					if l_i = a_k then
						l_p := l_p + l_f
					elseif a_k = A[l_i.to_integer_32] then
						l_p := l_p + (1.0 - l_f)
					end
					l_i := l_i + 1
				end
				Result := (l_p / l_K.to_real_64)
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			K := a_other.K
			A := a_other.A
			F := a_other.F
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				K = a_other.K and
				A.is_equal (a_other.A) and
				F.is_equal (a_other.F)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("K"),
					create {CE_STRUCTURE_FIELD}.make ("A"),
					create {CE_STRUCTURE_FIELD}.make ("F")
				>>)
			init_structure_info
		end

end
