class
	PD_GSL_SF_MATHIEU_WORKSPACE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} aa.make
			create {CE_REAL_64_POINTER} bb.make
			create {CE_REAL_64_POINTER} dd.make
			create {CE_REAL_64_POINTER} ee.make
			create {CE_REAL_64_POINTER} tt.make
			create {CE_REAL_64_POINTER} e2.make
			create {CE_REAL_64_POINTER} zz.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} eval.make
			create {CE_CLASS_POINTER [PD_GSL_MATRIX]} evec.make
			create {CE_CLASS_POINTER [PD_GSL_EIGEN_SYMMV_WORKSPACE]} wmat.make
			Precursor
		end

feature {ANY} -- fields

	size: NATURAL_32 assign set_size

	even_order: NATURAL_32 assign set_even_order

	odd_order: NATURAL_32 assign set_odd_order

	extra_values: INTEGER_32 assign set_extra_values

	qa: REAL_64 assign set_qa

	qb: REAL_64 assign set_qb

	aa: CE_POINTER [REAL_64] assign set_aa

	bb: CE_POINTER [REAL_64] assign set_bb

	dd: CE_POINTER [REAL_64] assign set_dd

	ee: CE_POINTER [REAL_64] assign set_ee

	tt: CE_POINTER [REAL_64] assign set_tt

	e2: CE_POINTER [REAL_64] assign set_e2

	zz: CE_POINTER [REAL_64] assign set_zz

	eval: CE_POINTER [PD_GSL_VECTOR] assign set_eval

	evec: CE_POINTER [PD_GSL_MATRIX] assign set_evec

	wmat: CE_POINTER [PD_GSL_EIGEN_SYMMV_WORKSPACE] assign set_wmat

feature {ANY} -- field setters

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

	set_even_order (a_even_order: NATURAL_32)
		do
			even_order := a_even_order
			update_memory_field ("even_order")
		end

	set_odd_order (a_odd_order: NATURAL_32)
		do
			odd_order := a_odd_order
			update_memory_field ("odd_order")
		end

	set_extra_values (a_extra_values: INTEGER_32)
		do
			extra_values := a_extra_values
			update_memory_field ("extra_values")
		end

	set_qa (a_qa: REAL_64)
		do
			qa := a_qa
			update_memory_field ("qa")
		end

	set_qb (a_qb: REAL_64)
		do
			qb := a_qb
			update_memory_field ("qb")
		end

	set_aa (a_aa: CE_POINTER [REAL_64])
		do
			aa := a_aa
			update_memory_field ("aa")
		end

	set_bb (a_bb: CE_POINTER [REAL_64])
		do
			bb := a_bb
			update_memory_field ("bb")
		end

	set_dd (a_dd: CE_POINTER [REAL_64])
		do
			dd := a_dd
			update_memory_field ("dd")
		end

	set_ee (a_ee: CE_POINTER [REAL_64])
		do
			ee := a_ee
			update_memory_field ("ee")
		end

	set_tt (a_tt: CE_POINTER [REAL_64])
		do
			tt := a_tt
			update_memory_field ("tt")
		end

	set_e2 (a_e2: CE_POINTER [REAL_64])
		do
			e2 := a_e2
			update_memory_field ("e2")
		end

	set_zz (a_zz: CE_POINTER [REAL_64])
		do
			zz := a_zz
			update_memory_field ("zz")
		end

	set_eval (a_eval: CE_POINTER [PD_GSL_VECTOR])
		do
			eval := a_eval
			update_memory_field ("eval")
		end

	set_evec (a_evec: CE_POINTER [PD_GSL_MATRIX])
		do
			evec := a_evec
			update_memory_field ("evec")
		end

	set_wmat (a_wmat: CE_POINTER [PD_GSL_EIGEN_SYMMV_WORKSPACE])
		do
			wmat := a_wmat
			update_memory_field ("wmat")
		end

feature {ANY} -- public routines

	gsl_sf_mathieu_a_array (a_order_min, a_order_max: INTEGER_32; a_qq: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_even_order: NATURAL_32
			l_odd_order: NATURAL_32
			l_extra_values: NATURAL_32
			l_ii: NATURAL_32
			l_jj: NATURAL_32
			l_status: INTEGER_32
			l_tt: CE_POINTER [REAL_64]
			l_dd: CE_POINTER [REAL_64]
			l_ee: CE_POINTER [REAL_64]
			l_e2: CE_POINTER [REAL_64]
			l_zz: CE_POINTER [REAL_64]
			l_aa: CE_POINTER [REAL_64]
			l_mat: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_evec: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_eval: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_wmat: CE_POINTER [PD_GSL_EIGEN_SYMMV_WORKSPACE]
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_mat.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_evec.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_eval.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_even_order := even_order
					l_odd_order := odd_order
					l_extra_values := extra_values.to_natural_32
					l_tt := tt
					l_dd := dd
					l_ee := ee
					l_e2 := e2
					l_zz := zz
					l_aa := aa
					l_wmat := wmat
					if not (a_order_max.to_natural_32 > size) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 3 then -- else:
						if not (a_order_max <= a_order_min) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 6 then -- else:
						if not (a_order_min < 0) then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid range [order_min,order_max]"), ce_string ("mathieu_charv.c"), 723, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						l_tt[0] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_tt[1] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_tt[2] := a_qq
						update_class_fields -- Current might have been changed; synchronize
						l_ii := {NATURAL_32} 1
						from
						until
							not (l_ii < l_even_order - {NATURAL_32} 1)
						loop
							l_tt[({NATURAL_32} 3 * l_ii).to_integer_32] := a_qq
							update_class_fields -- Current might have been changed; synchronize
							l_tt[({NATURAL_32} 3 * l_ii + {NATURAL_32} 1).to_integer_32] := (({NATURAL_32} 4 * l_ii) * l_ii).to_real_64
							update_class_fields -- Current might have been changed; synchronize
							l_tt[({NATURAL_32} 3 * l_ii + {NATURAL_32} 2).to_integer_32] := a_qq
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_tt[({NATURAL_32} 3 * l_even_order - {NATURAL_32} 3).to_integer_32] := a_qq
						update_class_fields -- Current might have been changed; synchronize
						l_tt[({NATURAL_32} 3 * l_even_order - {NATURAL_32} 2).to_integer_32] := (({NATURAL_32} 4 * (l_even_order - {NATURAL_32} 1)) * (l_even_order - {NATURAL_32} 1)).to_real_64
						update_class_fields -- Current might have been changed; synchronize
						l_tt[({NATURAL_32} 3 * l_even_order - {NATURAL_32} 1).to_integer_32] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_tt[3] := l_tt[3] * (2).to_double
						update_class_fields -- Current might have been changed; synchronize
						l_status := figi (l_even_order.to_integer_32, l_tt, l_dd, l_ee, l_e2)
						update_class_fields -- Current might have been changed; synchronize
						if l_status.to_boolean then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("Internal error in tridiagonal Mathieu matrix"), ce_string ("mathieu_charv.c"), 748, 5)
								update_class_fields -- Current might have been changed; synchronize
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						l_ii := {NATURAL_32} 0
						from
						until
							not (l_ii < l_even_order * l_even_order)
						loop
							l_zz[l_ii.to_integer_32] := 0.0
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_zz[0] := l_dd[0]
						update_class_fields -- Current might have been changed; synchronize
						l_zz[1] := l_ee[1]
						update_class_fields -- Current might have been changed; synchronize
						l_ii := {NATURAL_32} 1
						from
						until
							not (l_ii < l_even_order - {NATURAL_32} 1)
						loop
							l_zz[((l_ii * l_even_order + l_ii) - {NATURAL_32} 1).to_integer_32] := l_ee[l_ii.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_zz[(l_ii * l_even_order + l_ii).to_integer_32] := l_dd[l_ii.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_zz[((l_ii * l_even_order + l_ii) + {NATURAL_32} 1).to_integer_32] := l_ee[(l_ii + {NATURAL_32} 1).to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_zz[((l_even_order * (l_even_order - {NATURAL_32} 1) + l_even_order) - {NATURAL_32} 2).to_integer_32] := l_ee[(l_even_order - {NATURAL_32} 1).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_zz[(l_even_order * l_even_order - {NATURAL_32} 1).to_integer_32] := l_dd[(l_even_order - {NATURAL_32} 1).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_mat.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_matrix_view_array (l_zz, l_even_order, l_even_order)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_eval.item := (eval.item.gsl_vector_subvector ((0).to_natural_32, l_even_order)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_evec.item := (evec.item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_even_order, l_even_order)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_mat.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_eigen_symmv (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_evec.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_wmat).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_SORT_STATIC}).gsl_eigen_symmv_sort (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_evec.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), {PC_GSL_EIGEN_SORT_T}.GSL_EIGEN_SORT_VAL_ASC).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_ii := {NATURAL_32} 0
						from
						until
							not (l_ii < l_even_order - l_extra_values)
						loop
							l_aa[({NATURAL_32} 2 * l_ii).to_integer_32] := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_ii)
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_ii := {NATURAL_32} 0
						from
						until
							not (l_ii < l_odd_order * l_odd_order)
						loop
							l_zz[l_ii.to_integer_32] := 0.0
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_ii := {NATURAL_32} 0
						from
						until
							not (not (l_return) and then (l_ii < l_odd_order))
						loop
							l_jj := {NATURAL_32} 0
							from
							until
								not (l_jj < l_odd_order)
							loop
								if l_ii = l_jj then
									l_zz[(l_ii * l_odd_order + l_jj).to_integer_32] := (({NATURAL_32} 2 * l_ii + {NATURAL_32} 1) * ({NATURAL_32} 2 * l_ii + {NATURAL_32} 1)).to_real_64
									update_class_fields -- Current might have been changed; synchronize
								elseif l_ii = l_jj + {NATURAL_32} 1 then
									l_zz[(l_ii * l_odd_order + l_jj).to_integer_32] := a_qq
									update_class_fields -- Current might have been changed; synchronize
								elseif l_ii + {NATURAL_32} 1 = l_jj then
									l_zz[(l_ii * l_odd_order + l_jj).to_integer_32] := a_qq
									update_class_fields -- Current might have been changed; synchronize
								end
								l_jj := l_jj + 1
							end
							l_ii := l_ii + 1
						end
						l_zz[0] := l_zz[0] + a_qq
						update_class_fields -- Current might have been changed; synchronize
						l_mat.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_matrix_view_array (l_zz, l_odd_order, l_odd_order)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_eval.item := (eval.item.gsl_vector_subvector ((0).to_natural_32, l_odd_order)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_evec.item := (evec.item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_odd_order, l_odd_order)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_mat.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_eigen_symmv (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_evec.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_wmat).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_SORT_STATIC}).gsl_eigen_symmv_sort (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_evec.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), {PC_GSL_EIGEN_SORT_T}.GSL_EIGEN_SORT_VAL_ASC).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_ii := {NATURAL_32} 0
						from
						until
							not (l_ii < l_odd_order - l_extra_values)
						loop
							l_aa[({NATURAL_32} 2 * l_ii + {NATURAL_32} 1).to_integer_32] := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_ii)
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_ii := a_order_min.to_natural_32
						from
						until
							not (l_ii <= a_order_max.to_natural_32)
						loop
							a_result_array[(l_ii - a_order_min.to_natural_32).to_integer_32] := l_aa[l_ii.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_mathieu_b_array (a_order_min, a_order_max: INTEGER_32; a_qq: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_even_order: NATURAL_32
			l_odd_order: NATURAL_32
			l_extra_values: NATURAL_32
			l_ii: NATURAL_32
			l_jj: NATURAL_32
			l_zz: CE_POINTER [REAL_64]
			l_bb: CE_POINTER [REAL_64]
			l_mat: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_evec: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_eval: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_wmat: CE_POINTER [PD_GSL_EIGEN_SYMMV_WORKSPACE]
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_mat.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_evec.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_eval.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_even_order := even_order - {NATURAL_32} 1
					l_odd_order := odd_order
					l_extra_values := extra_values.to_natural_32
					l_zz := zz
					l_bb := bb
					l_wmat := wmat
					if not (a_order_max.to_natural_32 > size) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 3 then -- else:
						if not (a_order_max <= a_order_min) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 6 then -- else:
						if not (a_order_min < 0) then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid range [order_min,order_max]"), ce_string ("mathieu_charv.c"), 819, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						l_ii := {NATURAL_32} 0
						from
						until
							not (l_ii < l_even_order * l_even_order)
						loop
							l_zz[l_ii.to_integer_32] := 0.0
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_ii := {NATURAL_32} 0
						from
						until
							not (not (l_return) and then (l_ii < l_even_order))
						loop
							l_jj := {NATURAL_32} 0
							from
							until
								not (l_jj < l_even_order)
							loop
								if l_ii = l_jj then
									l_zz[(l_ii * l_even_order + l_jj).to_integer_32] := (({NATURAL_32} 4 * (l_ii + {NATURAL_32} 1)) * (l_ii + {NATURAL_32} 1)).to_real_64
									update_class_fields -- Current might have been changed; synchronize
								elseif l_ii = l_jj + {NATURAL_32} 1 then
									l_zz[(l_ii * l_even_order + l_jj).to_integer_32] := a_qq
									update_class_fields -- Current might have been changed; synchronize
								elseif l_ii + {NATURAL_32} 1 = l_jj then
									l_zz[(l_ii * l_even_order + l_jj).to_integer_32] := a_qq
									update_class_fields -- Current might have been changed; synchronize
								end
								l_jj := l_jj + 1
							end
							l_ii := l_ii + 1
						end
						l_mat.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_matrix_view_array (l_zz, l_even_order, l_even_order)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_eval.item := (eval.item.gsl_vector_subvector ((0).to_natural_32, l_even_order)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_evec.item := (evec.item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_even_order, l_even_order)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_mat.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_eigen_symmv (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_evec.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_wmat).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_SORT_STATIC}).gsl_eigen_symmv_sort (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_evec.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), {PC_GSL_EIGEN_SORT_T}.GSL_EIGEN_SORT_VAL_ASC).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_bb[0] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_ii := {NATURAL_32} 0
						from
						until
							not (l_ii < l_even_order - l_extra_values)
						loop
							l_bb[({NATURAL_32} 2 * (l_ii + {NATURAL_32} 1)).to_integer_32] := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_ii)
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_ii := {NATURAL_32} 0
						from
						until
							not (l_ii < l_odd_order * l_odd_order)
						loop
							l_zz[l_ii.to_integer_32] := 0.0
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_ii := {NATURAL_32} 0
						from
						until
							not (not (l_return) and then (l_ii < l_odd_order))
						loop
							l_jj := {NATURAL_32} 0
							from
							until
								not (l_jj < l_odd_order)
							loop
								if l_ii = l_jj then
									l_zz[(l_ii * l_odd_order + l_jj).to_integer_32] := (({NATURAL_32} 2 * l_ii + {NATURAL_32} 1) * ({NATURAL_32} 2 * l_ii + {NATURAL_32} 1)).to_real_64
									update_class_fields -- Current might have been changed; synchronize
								elseif l_ii = l_jj + {NATURAL_32} 1 then
									l_zz[(l_ii * l_odd_order + l_jj).to_integer_32] := a_qq
									update_class_fields -- Current might have been changed; synchronize
								elseif l_ii + {NATURAL_32} 1 = l_jj then
									l_zz[(l_ii * l_odd_order + l_jj).to_integer_32] := a_qq
									update_class_fields -- Current might have been changed; synchronize
								end
								l_jj := l_jj + 1
							end
							l_ii := l_ii + 1
						end
						l_zz[0] := l_zz[0] - a_qq
						update_class_fields -- Current might have been changed; synchronize
						l_mat.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_matrix_view_array (l_zz, l_odd_order, l_odd_order)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_eval.item := (eval.item.gsl_vector_subvector ((0).to_natural_32, l_odd_order)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_evec.item := (evec.item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_odd_order, l_odd_order)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_mat.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_eigen_symmv (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_evec.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_wmat).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_SORT_STATIC}).gsl_eigen_symmv_sort (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_evec.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), {PC_GSL_EIGEN_SORT_T}.GSL_EIGEN_SORT_VAL_ASC).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_ii := {NATURAL_32} 0
						from
						until
							not (l_ii < l_odd_order - l_extra_values)
						loop
							l_bb[({NATURAL_32} 2 * l_ii + {NATURAL_32} 1).to_integer_32] := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_eval.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_ii)
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						l_ii := a_order_min.to_natural_32
						from
						until
							not (l_ii <= a_order_max.to_natural_32)
						loop
							a_result_array[(l_ii - a_order_min.to_natural_32).to_integer_32] := l_bb[l_ii.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_ii := l_ii + 1
						end
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_mathieu_ce_array (a_nmin, a_nmax: INTEGER_32; a_qq, a_zz: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_even_odd: INTEGER_32
			l_order: INTEGER_32
			l_ii: INTEGER_32
			l_jj: INTEGER_32
			l_status: INTEGER_32
			l_coeff: CE_ARRAY [REAL_64]
			l_aa: CE_POINTER [REAL_64]
			l_norm___0: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_return, l_break, l_continue: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_coeff.make (<<100>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_aa := aa
					l_ii := 0
					from
					until
						not (l_ii < (a_nmax - a_nmin) + 1)
					loop
						a_result_array[l_ii] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_ii := l_ii + 1
					end
					if size < a_nmax.to_natural_32 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Work space not large enough"), ce_string ("mathieu_angfunc.c"), 211, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						if not (a_nmin < 0) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_nmax < a_nmin) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("domain error"), ce_string ("mathieu_angfunc.c"), 216, 1)
							update_class_fields -- Current might have been changed; synchronize
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						((create {CE_CLASS_POINTER [PD_GSL_SF_MATHIEU_WORKSPACE]}.make_shared (c_pointer))).item.gsl_sf_mathieu_a_array (0, a_nmax, a_qq, l_aa).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_ii := 0
						l_order := a_nmin
						from
						until
							not (not (l_return) and then (l_order <= a_nmax))
						loop
							l_continue := False
							l_norm___0 := 0.0
							l_even_odd := 0
							if l_order \\ 2 /= 0 then
								l_even_odd := 1
							end
							if a_qq = 0.0 then
								l_norm___0 := 1.0
								if l_order = 0 then
									l_norm___0 := sqrt (2.0)
									update_class_fields -- Current might have been changed; synchronize
								end
								l_tmp := cos (l_order.to_double * a_zz)
								update_class_fields -- Current might have been changed; synchronize
								a_result_array[l_ii] := l_tmp / l_norm___0
								update_class_fields -- Current might have been changed; synchronize
								l_continue := True
							end
							if not (l_return or l_break or l_continue) then
								l_status := (create {P_MATHIEU_COEFF_STATIC}).gsl_sf_mathieu_a_coeff (l_order, a_qq, l_aa[l_order], l_coeff)
								update_class_fields -- Current might have been changed; synchronize
								if l_status /= 0 then
									Result := l_status
									l_return := True
								end
								if not l_return then
									if l_even_odd = 0 then
										l_norm___0 := l_coeff[0] * l_coeff[0]
										l_jj := 0
										from
										until
											not (l_jj < 100)
										loop
											l_tmp___0 := cos ((2.0 * l_jj.to_double) * a_zz)
											update_class_fields -- Current might have been changed; synchronize
											a_result_array[l_ii] := a_result_array[l_ii] + (l_coeff[l_jj] * l_tmp___0)
											update_class_fields -- Current might have been changed; synchronize
											l_norm___0 := l_norm___0 + (l_coeff[l_jj] * l_coeff[l_jj])
											l_jj := l_jj + 1
										end
									else
										l_jj := 0
										from
										until
											not (l_jj < 100)
										loop
											l_tmp___1 := cos ((2.0 * l_jj.to_double + 1.0) * a_zz)
											update_class_fields -- Current might have been changed; synchronize
											a_result_array[l_ii] := a_result_array[l_ii] + (l_coeff[l_jj] * l_tmp___1)
											update_class_fields -- Current might have been changed; synchronize
											l_norm___0 := l_norm___0 + (l_coeff[l_jj] * l_coeff[l_jj])
											l_jj := l_jj + 1
										end
									end
									l_norm___0 := sqrt (l_norm___0)
									update_class_fields -- Current might have been changed; synchronize
									a_result_array[l_ii] := a_result_array[l_ii] / l_norm___0
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							if (not l_return) then
								l_ii := l_ii + 1
								l_order := l_order + 1
							end
						end
					end
					if not l_return then
						l_continue := False
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_mathieu_se_array (a_nmin, a_nmax: INTEGER_32; a_qq, a_zz: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_even_odd: INTEGER_32
			l_order: INTEGER_32
			l_ii: INTEGER_32
			l_jj: INTEGER_32
			l_status: INTEGER_32
			l_coeff: CE_ARRAY [REAL_64]
			l_bb: CE_POINTER [REAL_64]
			l_norm___0: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_return, l_break, l_continue: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_coeff.make (<<100>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_bb := bb
					l_ii := 0
					from
					until
						not (l_ii < (a_nmax - a_nmin) + 1)
					loop
						a_result_array[l_ii] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_ii := l_ii + 1
					end
					if size < a_nmax.to_natural_32 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Work space not large enough"), ce_string ("mathieu_angfunc.c"), 287, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						if not (a_nmin < 0) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_nmax < a_nmin) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("domain error"), ce_string ("mathieu_angfunc.c"), 292, 1)
							update_class_fields -- Current might have been changed; synchronize
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						((create {CE_CLASS_POINTER [PD_GSL_SF_MATHIEU_WORKSPACE]}.make_shared (c_pointer))).item.gsl_sf_mathieu_b_array (0, a_nmax, a_qq, l_bb).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_ii := 0
						l_order := a_nmin
						from
						until
							not (not (l_return) and then (l_order <= a_nmax))
						loop
							l_continue := False
							l_norm___0 := 0.0
							l_even_odd := 0
							if l_order \\ 2 /= 0 then
								l_even_odd := 1
							end
							if a_qq = 0.0 then
								l_norm___0 := 1.0
								a_result_array[l_ii] := sin (l_order.to_double * a_zz)
								update_class_fields -- Current might have been changed; synchronize
								l_continue := True
							end
							if not (l_return or l_break or l_continue) then
								l_status := (create {P_MATHIEU_COEFF_STATIC}).gsl_sf_mathieu_b_coeff (l_order, a_qq, l_bb[l_order], l_coeff)
								update_class_fields -- Current might have been changed; synchronize
								if l_status /= 0 then
									Result := l_status
									l_return := True
								end
								if not l_return then
									if l_even_odd = 0 then
										l_jj := 0
										from
										until
											not (l_jj < 100)
										loop
											l_tmp := sin ((2.0 * (l_jj + 1).to_double) * a_zz)
											update_class_fields -- Current might have been changed; synchronize
											a_result_array[l_ii] := a_result_array[l_ii] + (l_coeff[l_jj] * l_tmp)
											update_class_fields -- Current might have been changed; synchronize
											l_norm___0 := l_norm___0 + (l_coeff[l_jj] * l_coeff[l_jj])
											l_jj := l_jj + 1
										end
									else
										l_jj := 0
										from
										until
											not (l_jj < 100)
										loop
											l_tmp___0 := sin ((2.0 * l_jj.to_double + 1.0) * a_zz)
											update_class_fields -- Current might have been changed; synchronize
											a_result_array[l_ii] := a_result_array[l_ii] + (l_coeff[l_jj] * l_tmp___0)
											update_class_fields -- Current might have been changed; synchronize
											l_norm___0 := l_norm___0 + (l_coeff[l_jj] * l_coeff[l_jj])
											l_jj := l_jj + 1
										end
									end
									l_norm___0 := sqrt (l_norm___0)
									update_class_fields -- Current might have been changed; synchronize
									a_result_array[l_ii] := a_result_array[l_ii] / l_norm___0
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							if (not l_return) then
								l_ii := l_ii + 1
								l_order := l_order + 1
							end
						end
					end
					if not l_return then
						l_continue := False
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_mathieu_Mc_array (a_kind, a_nmin, a_nmax: INTEGER_32; a_qq, a_zz: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_even_odd: INTEGER_32
			l_order: INTEGER_32
			l_ii: INTEGER_32
			l_kk: INTEGER_32
			l_mm: INTEGER_32
			l_status: INTEGER_32
			l_maxerr: REAL_64
			l_amax: REAL_64
			l_pi: REAL_64
			l_fn: REAL_64
			l_coeff: CE_ARRAY [REAL_64]
			l_fc: REAL_64
			l_j1c: REAL_64
			l_z2c: REAL_64
			l_j1pc: REAL_64
			l_z2pc: REAL_64
			l_u1: REAL_64
			l_u2: REAL_64
			l_aa: CE_POINTER [REAL_64]
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_coeff.make (<<100>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_maxerr := 1.e-14
					l_pi := 3.14159265358979323846
					l_aa := aa
					l_ii := 0
					from
					until
						not (l_ii < (a_nmax - a_nmin) + 1)
					loop
						a_result_array[l_ii] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_ii := l_ii + 1
					end
					if a_qq <= 0.0 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("q must be greater than zero"), ce_string ("mathieu_radfunc.c"), 265, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						if not (a_kind < 1) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_kind > 2) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("kind must be 1 or 2"), ce_string ("mathieu_radfunc.c"), 269, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						l_mm := 0
						l_amax := 0.0
						l_fn := 0.0
						l_tmp := sqrt (a_qq)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := exp (-1.0 * a_zz)
						update_class_fields -- Current might have been changed; synchronize
						l_u1 := l_tmp * l_tmp___0
						l_tmp___1 := sqrt (a_qq)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := exp (a_zz)
						update_class_fields -- Current might have been changed; synchronize
						l_u2 := l_tmp___1 * l_tmp___2;
						((create {CE_CLASS_POINTER [PD_GSL_SF_MATHIEU_WORKSPACE]}.make_shared (c_pointer))).item.gsl_sf_mathieu_a_array (0, a_nmax, a_qq, l_aa).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_ii := 0
						l_order := a_nmin
						from
						until
							not (not (l_return) and then (l_order <= a_nmax))
						loop
							l_even_odd := 0
							if l_order \\ 2 /= 0 then
								l_even_odd := 1
							end
							l_status := (create {P_MATHIEU_COEFF_STATIC}).gsl_sf_mathieu_a_coeff (l_order, a_qq, l_aa[l_order], l_coeff)
							update_class_fields -- Current might have been changed; synchronize
							if l_status /= 0 then
								Result := l_status
								l_return := True
							end
							if not l_return then
								if l_even_odd = 0 then
									l_kk := 0
									from
									until
										not (not (l_break) and then (l_kk < 100))
									loop
										l_break := False
										l_tmp___5 := fabs (l_coeff[l_kk])
										update_class_fields -- Current might have been changed; synchronize
										if l_amax > l_tmp___5 then
											l_amax := l_amax
										else
											l_tmp___4 := fabs (l_coeff[l_kk])
											update_class_fields -- Current might have been changed; synchronize
											l_amax := l_tmp___4
										end
										l_tmp___6 := fabs (l_coeff[l_kk])
										update_class_fields -- Current might have been changed; synchronize
										if l_tmp___6 / l_amax < l_maxerr then
											l_break := True
										end
										if not (l_return or l_break) then
											l_j1c := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk, l_u1)
											update_class_fields -- Current might have been changed; synchronize
											if a_kind = 1 then
												l_z2c := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk, l_u2)
												update_class_fields -- Current might have been changed; synchronize
											else
												l_z2c := (create {P_BESSEL_YN_STATIC}).gsl_sf_bessel_Yn (l_kk, l_u2)
												update_class_fields -- Current might have been changed; synchronize
											end
											l_tmp___7 := pow (-1.0, 0.5 * l_order.to_double + l_kk.to_double)
											update_class_fields -- Current might have been changed; synchronize
											l_fc := l_tmp___7 * l_coeff[l_kk]
											l_fn := l_fn + ((l_fc * l_j1c) * l_z2c)
											l_kk := l_kk + 1
										end
									end
									l_break := False
									l_tmp___8 := sqrt (l_pi / 2.0)
									update_class_fields -- Current might have been changed; synchronize
									l_fn := l_fn * (l_tmp___8 / l_coeff[0])
								else
									l_kk := 0
									from
									until
										not (not (l_break) and then (l_kk < 100))
									loop
										l_break := False
										l_tmp___11 := fabs (l_coeff[l_kk])
										update_class_fields -- Current might have been changed; synchronize
										if l_amax > l_tmp___11 then
											l_amax := l_amax
										else
											l_tmp___10 := fabs (l_coeff[l_kk])
											update_class_fields -- Current might have been changed; synchronize
											l_amax := l_tmp___10
										end
										l_tmp___12 := fabs (l_coeff[l_kk])
										update_class_fields -- Current might have been changed; synchronize
										if l_tmp___12 / l_amax < l_maxerr then
											l_break := True
										end
										if not (l_return or l_break) then
											l_j1c := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk, l_u1)
											update_class_fields -- Current might have been changed; synchronize
											l_j1pc := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk + 1, l_u1)
											update_class_fields -- Current might have been changed; synchronize
											if a_kind = 1 then
												l_z2c := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk, l_u2)
												update_class_fields -- Current might have been changed; synchronize
												l_z2pc := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk + 1, l_u2)
												update_class_fields -- Current might have been changed; synchronize
											else
												l_z2c := (create {P_BESSEL_YN_STATIC}).gsl_sf_bessel_Yn (l_kk, l_u2)
												update_class_fields -- Current might have been changed; synchronize
												l_z2pc := (create {P_BESSEL_YN_STATIC}).gsl_sf_bessel_Yn (l_kk + 1, l_u2)
												update_class_fields -- Current might have been changed; synchronize
											end
											l_tmp___13 := pow (-1.0, 0.5 * (l_order - 1).to_double + l_kk.to_double)
											update_class_fields -- Current might have been changed; synchronize
											l_fc := l_tmp___13 * l_coeff[l_kk]
											l_fn := l_fn + (l_fc * (l_j1c * l_z2pc + l_j1pc * l_z2c))
											l_kk := l_kk + 1
										end
									end
									l_break := False
									l_tmp___14 := sqrt (l_pi / 2.0)
									update_class_fields -- Current might have been changed; synchronize
									l_fn := l_fn * (l_tmp___14 / l_coeff[0])
								end
								a_result_array[l_ii] := l_fn
								update_class_fields -- Current might have been changed; synchronize
								l_ii := l_ii + 1
								l_order := l_order + 1
							end
						end
					end
					if not l_return then
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_mathieu_Ms_array (a_kind, a_nmin, a_nmax: INTEGER_32; a_qq, a_zz: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_even_odd: INTEGER_32
			l_order: INTEGER_32
			l_ii: INTEGER_32
			l_kk: INTEGER_32
			l_mm: INTEGER_32
			l_status: INTEGER_32
			l_maxerr: REAL_64
			l_amax: REAL_64
			l_pi: REAL_64
			l_fn: REAL_64
			l_coeff: CE_ARRAY [REAL_64]
			l_fc: REAL_64
			l_j1c: REAL_64
			l_z2c: REAL_64
			l_j1mc: REAL_64
			l_z2mc: REAL_64
			l_j1pc: REAL_64
			l_z2pc: REAL_64
			l_u1: REAL_64
			l_u2: REAL_64
			l_bb: CE_POINTER [REAL_64]
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_coeff.make (<<100>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_maxerr := 1.e-14
					l_pi := 3.14159265358979323846
					l_bb := bb
					l_ii := 0
					from
					until
						not (l_ii < (a_nmax - a_nmin) + 1)
					loop
						a_result_array[l_ii] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_ii := l_ii + 1
					end
					if a_qq <= 0.0 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("q must be greater than zero"), ce_string ("mathieu_radfunc.c"), 371, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						if not (a_kind < 1) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_kind > 2) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("kind must be 1 or 2"), ce_string ("mathieu_radfunc.c"), 375, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						l_mm := 0
						l_amax := 0.0
						l_fn := 0.0
						l_tmp := sqrt (a_qq)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := exp (-1.0 * a_zz)
						update_class_fields -- Current might have been changed; synchronize
						l_u1 := l_tmp * l_tmp___0
						l_tmp___1 := sqrt (a_qq)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := exp (a_zz)
						update_class_fields -- Current might have been changed; synchronize
						l_u2 := l_tmp___1 * l_tmp___2;
						((create {CE_CLASS_POINTER [PD_GSL_SF_MATHIEU_WORKSPACE]}.make_shared (c_pointer))).item.gsl_sf_mathieu_b_array (0, a_nmax, a_qq, l_bb).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_ii := 0
						l_order := a_nmin
						from
						until
							not (not (l_return) and then (l_order <= a_nmax))
						loop
							l_even_odd := 0
							if l_order \\ 2 /= 0 then
								l_even_odd := 1
							end
							l_status := (create {P_MATHIEU_COEFF_STATIC}).gsl_sf_mathieu_b_coeff (l_order, a_qq, l_bb[l_order], l_coeff)
							update_class_fields -- Current might have been changed; synchronize
							if l_status /= 0 then
								Result := l_status
								l_return := True
							end
							if not l_return then
								if l_even_odd = 0 then
									l_kk := 0
									from
									until
										not (not (l_break) and then (l_kk < 100))
									loop
										l_break := False
										l_tmp___5 := fabs (l_coeff[l_kk])
										update_class_fields -- Current might have been changed; synchronize
										if l_amax > l_tmp___5 then
											l_amax := l_amax
										else
											l_tmp___4 := fabs (l_coeff[l_kk])
											update_class_fields -- Current might have been changed; synchronize
											l_amax := l_tmp___4
										end
										l_tmp___6 := fabs (l_coeff[l_kk])
										update_class_fields -- Current might have been changed; synchronize
										if l_tmp___6 / l_amax < l_maxerr then
											l_break := True
										end
										if not (l_return or l_break) then
											l_j1mc := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk, l_u1)
											update_class_fields -- Current might have been changed; synchronize
											l_j1pc := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk + 2, l_u1)
											update_class_fields -- Current might have been changed; synchronize
											if a_kind = 1 then
												l_z2mc := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk, l_u2)
												update_class_fields -- Current might have been changed; synchronize
												l_z2pc := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk + 2, l_u2)
												update_class_fields -- Current might have been changed; synchronize
											else
												l_z2mc := (create {P_BESSEL_YN_STATIC}).gsl_sf_bessel_Yn (l_kk, l_u2)
												update_class_fields -- Current might have been changed; synchronize
												l_z2pc := (create {P_BESSEL_YN_STATIC}).gsl_sf_bessel_Yn (l_kk + 2, l_u2)
												update_class_fields -- Current might have been changed; synchronize
											end
											l_tmp___7 := pow (-1.0, (0.5 * l_order.to_double + l_kk.to_double) + (1).to_double)
											update_class_fields -- Current might have been changed; synchronize
											l_fc := l_tmp___7 * l_coeff[l_kk]
											l_fn := l_fn + (l_fc * (l_j1mc * l_z2pc - l_j1pc * l_z2mc))
											l_kk := l_kk + 1
										end
									end
									l_break := False
									l_tmp___8 := sqrt (l_pi / 2.0)
									update_class_fields -- Current might have been changed; synchronize
									l_fn := l_fn * (l_tmp___8 / l_coeff[0])
								else
									l_kk := 0
									from
									until
										not (not (l_break) and then (l_kk < 100))
									loop
										l_break := False
										l_tmp___11 := fabs (l_coeff[l_kk])
										update_class_fields -- Current might have been changed; synchronize
										if l_amax > l_tmp___11 then
											l_amax := l_amax
										else
											l_tmp___10 := fabs (l_coeff[l_kk])
											update_class_fields -- Current might have been changed; synchronize
											l_amax := l_tmp___10
										end
										l_tmp___12 := fabs (l_coeff[l_kk])
										update_class_fields -- Current might have been changed; synchronize
										if l_tmp___12 / l_amax < l_maxerr then
											l_break := True
										end
										if not (l_return or l_break) then
											l_j1c := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk, l_u1)
											update_class_fields -- Current might have been changed; synchronize
											l_j1pc := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk + 1, l_u1)
											update_class_fields -- Current might have been changed; synchronize
											if a_kind = 1 then
												l_z2c := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk, l_u2)
												update_class_fields -- Current might have been changed; synchronize
												l_z2pc := (create {P_BESSEL_JN_STATIC}).gsl_sf_bessel_Jn (l_kk + 1, l_u2)
												update_class_fields -- Current might have been changed; synchronize
											else
												l_z2c := (create {P_BESSEL_YN_STATIC}).gsl_sf_bessel_Yn (l_kk, l_u2)
												update_class_fields -- Current might have been changed; synchronize
												l_z2pc := (create {P_BESSEL_YN_STATIC}).gsl_sf_bessel_Yn (l_kk + 1, l_u2)
												update_class_fields -- Current might have been changed; synchronize
											end
											l_tmp___13 := pow (-1.0, 0.5 * (l_order - 1).to_double + l_kk.to_double)
											update_class_fields -- Current might have been changed; synchronize
											l_fc := l_tmp___13 * l_coeff[l_kk]
											l_fn := l_fn + (l_fc * (l_j1c * l_z2pc - l_j1pc * l_z2c))
											l_kk := l_kk + 1
										end
									end
									l_break := False
									l_tmp___14 := sqrt (l_pi / 2.0)
									update_class_fields -- Current might have been changed; synchronize
									l_fn := l_fn * (l_tmp___14 / l_coeff[0])
								end
								a_result_array[l_ii] := l_fn
								update_class_fields -- Current might have been changed; synchronize
								l_ii := l_ii + 1
								l_order := l_order + 1
							end
						end
					end
					if not l_return then
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {PD_GSL_SF_MATHIEU_WORKSPACE} -- private routines

	figi (a_nn: INTEGER_32; a_tt, a_dd, a_ee, a_e2: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_ii: INTEGER_32
			l_return: BOOLEAN
		do
			l_ii := 0
			from
			until
				not (not (l_return) and then (l_ii < a_nn))
			loop
				if l_ii /= 0 then
					a_e2[l_ii] := a_tt[3 * l_ii] * a_tt[(3 * (l_ii - 1) + 2)]
					if a_e2[l_ii] < 0.0 then
						Result := (a_nn + l_ii)
						l_return := True
					end
					if not l_return and then (a_e2[l_ii] = 0.0) then
						if a_tt[3 * l_ii] /= 0.0 then
							Result := (-1 * (3 * a_nn + l_ii))
							l_return := True
						elseif a_tt[(3 * (l_ii - 1) + 2)] /= 0.0 then
							Result := (-1 * (3 * a_nn + l_ii))
							l_return := True
						end
					end
					if not l_return then
						a_ee[l_ii] := sqrt (a_e2[l_ii])
					end
				end
				if not l_return then
					a_dd[l_ii] := a_tt[(3 * l_ii + 1)]
					l_ii := l_ii + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			size := a_other.size
			even_order := a_other.even_order
			odd_order := a_other.odd_order
			extra_values := a_other.extra_values
			qa := a_other.qa
			qb := a_other.qb
			aa := a_other.aa
			bb := a_other.bb
			dd := a_other.dd
			ee := a_other.ee
			tt := a_other.tt
			e2 := a_other.e2
			zz := a_other.zz
			eval := a_other.eval
			evec := a_other.evec
			wmat := a_other.wmat
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				size = a_other.size and
				even_order = a_other.even_order and
				odd_order = a_other.odd_order and
				extra_values = a_other.extra_values and
				qa = a_other.qa and
				qb = a_other.qb and
				aa.is_equal (a_other.aa) and
				bb.is_equal (a_other.bb) and
				dd.is_equal (a_other.dd) and
				ee.is_equal (a_other.ee) and
				tt.is_equal (a_other.tt) and
				e2.is_equal (a_other.e2) and
				zz.is_equal (a_other.zz) and
				eval.is_equal (a_other.eval) and
				evec.is_equal (a_other.evec) and
				wmat.is_equal (a_other.wmat)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("size"),
					create {CE_STRUCTURE_FIELD}.make ("even_order"),
					create {CE_STRUCTURE_FIELD}.make ("odd_order"),
					create {CE_STRUCTURE_FIELD}.make ("extra_values"),
					create {CE_STRUCTURE_FIELD}.make ("qa"),
					create {CE_STRUCTURE_FIELD}.make ("qb"),
					create {CE_STRUCTURE_FIELD}.make ("aa"),
					create {CE_STRUCTURE_FIELD}.make ("bb"),
					create {CE_STRUCTURE_FIELD}.make ("dd"),
					create {CE_STRUCTURE_FIELD}.make ("ee"),
					create {CE_STRUCTURE_FIELD}.make ("tt"),
					create {CE_STRUCTURE_FIELD}.make ("e2"),
					create {CE_STRUCTURE_FIELD}.make ("zz"),
					create {CE_STRUCTURE_FIELD}.make ("eval"),
					create {CE_STRUCTURE_FIELD}.make ("evec"),
					create {CE_STRUCTURE_FIELD}.make ("wmat")
				>>)
			init_structure_info
		end

end
