class
	PD_GSL_VECTOR

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} data.make
			create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]} block.make
			Precursor
		end

feature {ANY} -- fields

	size: NATURAL_32 assign set_size

	stride: NATURAL_32 assign set_stride

	data: CE_POINTER [REAL_64] assign set_data

	block: CE_POINTER [PD_GSL_BLOCK_STRUCT] assign set_block

	owner: INTEGER_32 assign set_owner

feature {ANY} -- field setters

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

	set_stride (a_stride: NATURAL_32)
		do
			stride := a_stride
			update_memory_field ("stride")
		end

	set_data (a_data: CE_POINTER [REAL_64])
		do
			data := a_data
			update_memory_field ("data")
		end

	set_block (a_block: CE_POINTER [PD_GSL_BLOCK_STRUCT])
		do
			block := a_block
			update_memory_field ("block")
		end

	set_owner (a_owner: INTEGER_32)
		do
			owner := a_owner
			update_memory_field ("owner")
		end

feature {ANY} -- public routines

	gsl_vector_get (a_i: NATURAL_32): REAL_64
		do
			Result := (data[(a_i * stride).to_integer_32])
		end

	gsl_vector_set (a_i: NATURAL_32; a_x: REAL_64)
		do
			data[(a_i * stride).to_integer_32] := a_x
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_vector_ptr (a_i: NATURAL_32): CE_POINTER [REAL_64]
		do
			Result := (data + (a_i * stride).to_integer_32)
		end

	gsl_vector_const_ptr (a_i: NATURAL_32): CE_POINTER [REAL_64]
		do
			Result := (data + (a_i * stride).to_integer_32)
		end

	gsl_blas_ddot (a_Y: CE_POINTER [PD_GSL_VECTOR]; a_result: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				a_result.item := cblas_ddot (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 107, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_dnrm2: REAL_64
		local
			l_tmp: REAL_64
		do
			l_tmp := cblas_dnrm2 (size.to_integer_32, data, stride.to_integer_32)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_blas_idamax: NATURAL_32
		local
			l_tmp: NATURAL_32
		do
			l_tmp := cblas_idamax (size.to_integer_32, data, stride.to_integer_32)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_blas_dswap (a_Y: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_dswap (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 286, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_dcopy (a_Y: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_dcopy (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 349, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_daxpy (a_alpha: REAL_64; a_Y: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_daxpy (size.to_integer_32, a_alpha, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 413, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_drot (a_Y: CE_POINTER [PD_GSL_VECTOR]; a_c___1, a_s: REAL_64): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_drot (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32, a_c___1, a_s)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 494, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_drotm (a_Y: CE_POINTER [PD_GSL_VECTOR]; a_P___2: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_drotm (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32, a_P___2)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 544, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_dscal (a_alpha: REAL_64)
		do
			cblas_dscal (size.to_integer_32, a_alpha, data, stride.to_integer_32)
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_blas_dger (a_alpha: REAL_64; a_Y: CE_POINTER [PD_GSL_VECTOR]; a_A___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_A___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_A___0.item.size1
					l_N := a_A___0.item.size2
					if size /= l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if a_Y.item.size = l_N then
							cblas_dger ({PC_CBLAS_ORDER}.CblasRowMajor, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32, a_A___0.item.data, (a_A___0.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_013, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 4 then
					if not l_return then
						Result := 0
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_bspline_knots (a_w: CE_POINTER [PD_GSL_BSPLINE_WORKSPACE]): INTEGER_32
		require
			not a_w.is_void
		local
			l_i: NATURAL_32
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_return, l_break: BOOLEAN
		do
			if size /= a_w.item.nbreak then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("breakpts vector has wrong size"), ce_string ("bspline.c"), 300, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_w.item.k)
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get ((0).to_natural_32)
					update_class_fields; -- Current might have been changed; synchronize
					(a_w.item.knots).item.gsl_vector_set (l_i, l_tmp)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (1).to_natural_32
				from
				until
					not (l_i < a_w.item.l)
				loop
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get (l_i)
					update_class_fields; -- Current might have been changed; synchronize
					(a_w.item.knots).item.gsl_vector_set ((a_w.item.k - {NATURAL_32} 1) + l_i, l_tmp___0)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := a_w.item.n
				from
				until
					not (l_i < a_w.item.n + a_w.item.k)
				loop
					l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get (a_w.item.l)
					update_class_fields; -- Current might have been changed; synchronize
					(a_w.item.knots).item.gsl_vector_set (l_i, l_tmp___1)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_bspline_eval (a_x: REAL_64; a_w: CE_POINTER [PD_GSL_BSPLINE_WORKSPACE]): INTEGER_32
		require
			not a_w.is_void
		local
			l_i: NATURAL_32
			l_istart: CE_ARRAY [NATURAL_32]
			l_iend: CE_ARRAY [NATURAL_32]
			l_error: INTEGER_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_istart.make (<<1>>)
			create {CE_NATURAL_32_ARRAY} l_iend.make (<<1>>)

			if size /= a_w.item.n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector B not of length n"), ce_string ("bspline.c"), 394, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_error := (a_w.item.B).item.gsl_bspline_eval_nonzero (a_x, l_istart, l_iend, a_w)
				update_class_fields -- Current might have been changed; synchronize
				if l_error.to_boolean then
					Result := l_error
					l_return := True
				end
				if not l_return then
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_istart.item)
					loop
						((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_set (l_i, 0.0)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
					l_i := l_istart.item
					from
					until
						not (l_i <= l_iend.item)
					loop
						l_tmp := (a_w.item.B).item.gsl_vector_get (l_i - l_istart.item)
						update_class_fields; -- Current might have been changed; synchronize
						((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_set (l_i, l_tmp)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
					l_i := l_iend.item + {NATURAL_32} 1
					from
					until
						not (l_i < a_w.item.n)
					loop
						((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_set (l_i, 0.0)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_bspline_eval_nonzero (a_x: REAL_64; a_istart, a_iend: CE_POINTER [NATURAL_32]; a_w: CE_POINTER [PD_GSL_BSPLINE_WORKSPACE]): INTEGER_32
		require
			not a_w.is_void
		local
			l_i: CE_ARRAY [NATURAL_32]
			l_j: CE_ARRAY [NATURAL_32]
			l_flag: CE_ARRAY [INTEGER_32]
			l_error: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_i.make (<<1>>)
			create {CE_NATURAL_32_ARRAY} l_j.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_flag.make (<<1>>)

			if size /= a_w.item.k then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("Bk vector length does not match order k"), ce_string ("bspline.c"), 458, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_flag.item := 0
				update_class_fields -- Current might have been changed; synchronize
				l_error := 0
				l_i.item := a_w.item.bspline_find_interval (a_x, l_flag)
				update_class_fields -- Current might have been changed; synchronize
				l_error := a_w.item.bspline_process_interval_for_eval (a_x, l_i, l_flag.item)
				update_class_fields -- Current might have been changed; synchronize
				if l_error.to_boolean then
					Result := l_error
					l_return := True
				end
				if not l_return then
					a_istart.item := (l_i.item - a_w.item.k) + {NATURAL_32} 1
					update_class_fields -- Current might have been changed; synchronize
					a_iend.item := l_i.item
					update_class_fields; -- Current might have been changed; synchronize
					(create {P_BSPLINE_STATIC}).bspline_pppack_bsplvb (a_w.item.knots, a_w.item.k, (1).to_natural_32, a_x, a_iend.item, l_j, a_w.item.deltal, a_w.item.deltar, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer)))
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_solve_symm_tridiag (a_offdiag, a_rhs, a_solution: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_rhs.is_void
		local
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of diag must match rhs"), ce_string ("tridiag.c"), 453, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offdiag.item.size /= a_rhs.item.size - {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of offdiag must match rhs-1"), ce_string ("tridiag.c"), 457, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_solution.item.size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of solution must match rhs"), ce_string ("tridiag.c"), 461, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_tmp := solve_tridiag (data, stride, a_offdiag.item.data, a_offdiag.item.stride, a_rhs.item.data, a_rhs.item.stride, a_solution.item.data, a_solution.item.stride, size)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_tmp
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_solve_tridiag (a_abovediag, a_belowdiag, a_rhs, a_solution: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_rhs.is_void
		local
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of diag must match rhs"), ce_string ("tridiag.c"), 484, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_abovediag.item.size /= a_rhs.item.size - {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of abovediag must match rhs-1"), ce_string ("tridiag.c"), 488, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_belowdiag.item.size /= a_rhs.item.size - {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of belowdiag must match rhs-1"), ce_string ("tridiag.c"), 492, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_solution.item.size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of solution must match rhs"), ce_string ("tridiag.c"), 496, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_tmp := solve_tridiag_nonsym (data, stride, a_abovediag.item.data, a_abovediag.item.stride, a_belowdiag.item.data, a_belowdiag.item.stride, a_rhs.item.data, a_rhs.item.stride, a_solution.item.data, a_solution.item.stride, size)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_tmp
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_solve_symm_cyc_tridiag (a_offdiag, a_rhs, a_solution: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_rhs.is_void
		local
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of diag must match rhs"), ce_string ("tridiag.c"), 519, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offdiag.item.size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of offdiag must match rhs"), ce_string ("tridiag.c"), 523, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_solution.item.size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of solution must match rhs"), ce_string ("tridiag.c"), 527, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size < {NATURAL_32} 3 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of cyclic system must be 3 or more"), ce_string ("tridiag.c"), 531, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_tmp := solve_cyc_tridiag (data, stride, a_offdiag.item.data, a_offdiag.item.stride, a_rhs.item.data, a_rhs.item.stride, a_solution.item.data, a_solution.item.stride, size)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_tmp
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_solve_cyc_tridiag (a_abovediag, a_belowdiag, a_rhs, a_solution: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_rhs.is_void
		local
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of diag must match rhs"), ce_string ("tridiag.c"), 553, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_abovediag.item.size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of abovediag must match rhs"), ce_string ("tridiag.c"), 557, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_belowdiag.item.size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of belowdiag must match rhs"), ce_string ("tridiag.c"), 561, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_solution.item.size /= a_rhs.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of solution must match rhs"), ce_string ("tridiag.c"), 565, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size < {NATURAL_32} 3 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of cyclic system must be 3 or more"), ce_string ("tridiag.c"), 569, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_tmp := solve_cyc_tridiag_nonsym (data, stride, a_abovediag.item.data, a_abovediag.item.stride, a_belowdiag.item.data, a_belowdiag.item.stride, a_rhs.item.data, a_rhs.item.stride, a_solution.item.data, a_solution.item.stride, size)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_tmp
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	svd2 (a_f: CE_POINTER [PD_GSL_VECTOR]; a_U, a_V: CE_POINTER [PD_GSL_MATRIX])
		require
			not a_f.is_void
			not a_U.is_void
			not a_V.is_void
		local
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_a11: REAL_64
			l_a12: REAL_64
			l_a21: REAL_64
			l_a22: REAL_64
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_d0: REAL_64
			l_tmp: REAL_64
			l_f0: REAL_64
			l_tmp___0: REAL_64
			l_d1: REAL_64
			l_tmp___1: REAL_64
			l_Uip: REAL_64
			l_tmp___2: REAL_64
			l_Uiq: REAL_64
			l_tmp___3: REAL_64
			l_Vip: REAL_64
			l_tmp___4: REAL_64
			l_Viq: REAL_64
			l_tmp___5: REAL_64
			l_Vip___0: REAL_64
			l_tmp___6: REAL_64
			l_Viq___0: REAL_64
			l_tmp___7: REAL_64
			l_t1: REAL_64
			l_t2: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_Uip___0: REAL_64
			l_tmp___10: REAL_64
			l_Uiq___0: REAL_64
			l_tmp___11: REAL_64
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)

			l_M := a_U.item.size1
			l_N := a_V.item.size1
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get ((0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_d0 := l_tmp
			l_tmp___0 := a_f.item.gsl_vector_get ((0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_f0 := l_tmp___0
			l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get ((1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_d1 := l_tmp___1
			if l_d0 = 0.0 then
				(create {P_QRSTEP_STATIC}).create_givens (l_f0, l_d1, l_c___1, l_s)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_set ((0).to_natural_32, l_c___1.item * l_f0 - l_s.item * l_d1)
				update_class_fields -- Current might have been changed; synchronize
				a_f.item.gsl_vector_set ((0).to_natural_32, l_s.item * l_f0 + l_c___1.item * l_d1)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_set ((1).to_natural_32, 0.0)
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_M)
				loop
					l_tmp___2 := a_U.item.gsl_matrix_get (l_i, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_Uip := l_tmp___2
					l_tmp___3 := a_U.item.gsl_matrix_get (l_i, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_Uiq := l_tmp___3
					a_U.item.gsl_matrix_set (l_i, (0).to_natural_32, l_c___1.item * l_Uip - l_s.item * l_Uiq)
					update_class_fields -- Current might have been changed; synchronize
					a_U.item.gsl_matrix_set (l_i, (1).to_natural_32, l_s.item * l_Uip + l_c___1.item * l_Uiq)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				a_V.item.gsl_matrix_swap_columns ((0).to_natural_32, (1).to_natural_32).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			elseif l_d1 = 0.0 then
				(create {P_QRSTEP_STATIC}).create_givens (l_d0, l_f0, l_c___1, l_s)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_set ((0).to_natural_32, l_d0 * l_c___1.item - l_f0 * l_s.item)
				update_class_fields -- Current might have been changed; synchronize
				a_f.item.gsl_vector_set ((0).to_natural_32, 0.0)
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp___4 := a_V.item.gsl_matrix_get (l_i, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_Vip := l_tmp___4
					l_tmp___5 := a_V.item.gsl_matrix_get (l_i, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_Viq := l_tmp___5
					a_V.item.gsl_matrix_set (l_i, (0).to_natural_32, l_c___1.item * l_Vip - l_s.item * l_Viq)
					update_class_fields -- Current might have been changed; synchronize
					a_V.item.gsl_matrix_set (l_i, (1).to_natural_32, l_s.item * l_Vip + l_c___1.item * l_Viq)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
			else
				create_schur (l_d0, l_f0, l_d1, l_c___1, l_s)
				update_class_fields -- Current might have been changed; synchronize
				l_a11 := l_c___1.item * l_d0 - l_s.item * l_f0
				l_a21 := -l_s.item * l_d1
				l_a12 := l_s.item * l_d0 + l_c___1.item * l_f0
				l_a22 := l_c___1.item * l_d1
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp___6 := a_V.item.gsl_matrix_get (l_i, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_Vip___0 := l_tmp___6
					l_tmp___7 := a_V.item.gsl_matrix_get (l_i, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_Viq___0 := l_tmp___7
					a_V.item.gsl_matrix_set (l_i, (0).to_natural_32, l_c___1.item * l_Vip___0 - l_s.item * l_Viq___0)
					update_class_fields -- Current might have been changed; synchronize
					a_V.item.gsl_matrix_set (l_i, (1).to_natural_32, l_s.item * l_Vip___0 + l_c___1.item * l_Viq___0)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_tmp___8 := hypot (l_a11, l_a21)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___9 := hypot (l_a12, l_a22)
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp___8 < l_tmp___9 then
					l_t1 := l_a11
					l_a11 := l_a12
					l_a12 := l_t1
					l_t2 := l_a21
					l_a21 := l_a22
					l_a22 := l_t2
					a_V.item.gsl_matrix_swap_columns ((0).to_natural_32, (1).to_natural_32).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
				(create {P_QRSTEP_STATIC}).create_givens (l_a11, l_a21, l_c___1, l_s)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_set ((0).to_natural_32, l_c___1.item * l_a11 - l_s.item * l_a21)
				update_class_fields -- Current might have been changed; synchronize
				a_f.item.gsl_vector_set ((0).to_natural_32, l_c___1.item * l_a12 - l_s.item * l_a22)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_set ((1).to_natural_32, l_s.item * l_a12 + l_c___1.item * l_a22)
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_M)
				loop
					l_tmp___10 := a_U.item.gsl_matrix_get (l_i, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_Uip___0 := l_tmp___10
					l_tmp___11 := a_U.item.gsl_matrix_get (l_i, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_Uiq___0 := l_tmp___11
					a_U.item.gsl_matrix_set (l_i, (0).to_natural_32, l_c___1.item * l_Uip___0 - l_s.item * l_Uiq___0)
					update_class_fields -- Current might have been changed; synchronize
					a_U.item.gsl_matrix_set (l_i, (1).to_natural_32, l_s.item * l_Uip___0 + l_c___1.item * l_Uiq___0)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
			end
		end

	gsl_matrix_view_vector (a_n1, a_n2: NATURAL_32): PD_GSL_MATRIX_VIEW
		local
			l_view: PD_GSL_MATRIX_VIEW
			l_m___4: PD_GSL_MATRIX
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_m___4.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("view_source.c"), 102, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("view_source.c"), 107, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif stride /= {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must have unit stride"), ce_string ("view_source.c"), 112, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 * a_n2 > size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size exceeds size of original"), ce_string ("view_source.c"), 117, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_m___4.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := data
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_m___4.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_view_vector_with_tda (a_n1, a_n2, a_tda: NATURAL_32): PD_GSL_MATRIX_VIEW
		local
			l_view: PD_GSL_MATRIX_VIEW
			l_m___4: PD_GSL_MATRIX
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_m___4.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("view_source.c"), 147, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("view_source.c"), 152, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif stride /= {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must have unit stride"), ce_string ("view_source.c"), 157, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 > a_tda then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must not exceed tda"), ce_string ("view_source.c"), 162, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 * a_tda > size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size exceeds size of original"), ce_string ("view_source.c"), 167, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_m___4.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := data
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := a_tda
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_m___4.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_const_view_vector (a_n1, a_n2: NATURAL_32): PD_GSL_MATRIX_CONST_VIEW
		local
			l_view: PD_GSL_MATRIX_CONST_VIEW
			l_m___4: PD_GSL_MATRIX
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_m___4.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("view_source.c"), 102, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("view_source.c"), 107, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif stride /= {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must have unit stride"), ce_string ("view_source.c"), 112, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 * a_n2 > size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size exceeds size of original"), ce_string ("view_source.c"), 117, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_m___4.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := data
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_m___4.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_const_view_vector_with_tda (a_n1, a_n2, a_tda: NATURAL_32): PD_GSL_MATRIX_CONST_VIEW
		local
			l_view: PD_GSL_MATRIX_CONST_VIEW
			l_m___4: PD_GSL_MATRIX
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_m___4.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("view_source.c"), 147, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("view_source.c"), 152, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif stride /= {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must have unit stride"), ce_string ("view_source.c"), 157, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 > a_tda then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must not exceed tda"), ce_string ("view_source.c"), 162, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 * a_tda > size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size exceeds size of original"), ce_string ("view_source.c"), 167, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_m___4.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := data
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := a_tda
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_m___4.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	enorm: REAL_64
		local
			l_tmp: REAL_64
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_blas_dnrm2
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	compute_trial_step (a_dx, a_x_trial: CE_POINTER [PD_GSL_VECTOR])
		local
			l_i: NATURAL_32
			l_N: NATURAL_32
			l_pi: REAL_64
			l_tmp: REAL_64
			l_xi: REAL_64
			l_tmp___0: REAL_64
		do
			l_N := size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_tmp := a_dx.item.gsl_vector_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_pi := l_tmp
				l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_xi := l_tmp___0
				a_x_trial.item.gsl_vector_set (l_i, l_xi + l_pi)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_multifit_test_gradient (a_epsabs: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
			l_residual: REAL_64
			l_n: NATURAL_32
			l_gi: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_residual := (0).to_double
			l_n := size
			if a_epsabs < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("absolute tolerance is negative"), ce_string ("convergence.c"), 72, 13)
					update_class_fields -- Current might have been changed; synchronize
					Result := 13
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_gi := l_tmp
					l_tmp___0 := fabs (l_gi)
					update_class_fields -- Current might have been changed; synchronize
					l_residual := l_residual + l_tmp___0
					l_i := l_i + 1
				end
				if l_residual < a_epsabs then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := (-2)
			end
		end

	gsl_multifit_linear_est (a_c___1: CE_POINTER [PD_GSL_VECTOR]; a_cov: CE_POINTER [PD_GSL_MATRIX]; a_y, a_y_err: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_c___1.is_void
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_var: REAL_64
			l_xi: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_xj: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_return, l_break: BOOLEAN
		do
			if size /= a_c___1.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of parameters c does not match number of observations x"), ce_string ("multilinear.c"), 474, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_cov.item.size1 /= a_cov.item.size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("covariance matrix is not square"), ce_string ("multilinear.c"), 478, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_c___1.item.size /= a_cov.item.size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of parameters c does not match size of covariance matrix cov"), ce_string ("multilinear.c"), 483, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_var := (0).to_double;
				((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_blas_ddot (a_c___1, a_y).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < size))
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_xi := l_tmp
					l_tmp___0 := a_cov.item.gsl_matrix_get (l_i, l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_var := l_var + ((l_xi * l_xi) * l_tmp___0)
					l_j := (0).to_natural_32
					from
					until
						not (l_j < l_i)
					loop
						l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get (l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_xj := l_tmp___1
						l_tmp___2 := a_cov.item.gsl_matrix_get (l_i, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_var := l_var + ((((2).to_double * l_xi) * l_xj) * l_tmp___2)
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
				a_y_err.item := sqrt (l_var)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_multimin_test_gradient (a_epsabs: REAL_64): INTEGER_32
		local
			l_norm___0: REAL_64
			l_return, l_break: BOOLEAN
		do
			if a_epsabs < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("absolute tolerance is negative"), ce_string ("convergence.c"), 31, 13)
					update_class_fields -- Current might have been changed; synchronize
					Result := 13
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_norm___0 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_blas_dnrm2
				update_class_fields -- Current might have been changed; synchronize
				if l_norm___0 < a_epsabs then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := (-2)
			end
		end

	enorm___1: REAL_64
		local
			l_e2: REAL_64
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_fi: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
		do
			l_e2 := (0).to_double
			l_n := size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_fi := l_tmp
				l_e2 := l_e2 + (l_fi * l_fi)
				l_i := l_i + 1
			end
			l_tmp___0 := sqrt (l_e2)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp___0
		end

	gsl_multiroot_test_residual (a_epsabs: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
			l_residual: REAL_64
			l_n: NATURAL_32
			l_fi: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_residual := (0).to_double
			l_n := size
			if a_epsabs < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("absolute tolerance is negative"), ce_string ("convergence.c"), 72, 13)
					update_class_fields -- Current might have been changed; synchronize
					Result := 13
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_fi := l_tmp
					l_tmp___0 := fabs (l_fi)
					update_class_fields -- Current might have been changed; synchronize
					l_residual := l_residual + l_tmp___0
					l_i := l_i + 1
				end
				if l_residual < a_epsabs then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := (-2)
			end
		end

	enorm___2: REAL_64
		local
			l_e2: REAL_64
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_fi: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
		do
			l_e2 := (0).to_double
			l_n := size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (c_pointer))).item.gsl_vector_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_fi := l_tmp
				l_e2 := l_e2 + (l_fi * l_fi)
				l_i := l_i + 1
			end
			l_tmp___0 := sqrt (l_e2)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp___0
		end

	gsl_sort_vector_smallest (a_dest: CE_POINTER [REAL_64]; a_k: NATURAL_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_sort_smallest (a_dest, a_k, data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_sort_vector_largest (a_dest: CE_POINTER [REAL_64]; a_k: NATURAL_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_sort_largest (a_dest, a_k, data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_sort_vector
		do
			gsl_sort (data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_sort_vector_smallest_index (a_p: CE_POINTER [NATURAL_32]; a_k: NATURAL_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_sort_smallest_index (a_p, a_k, data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_sort_vector_largest_index (a_p: CE_POINTER [NATURAL_32]; a_k: NATURAL_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_sort_largest_index (a_p, a_k, data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_vector_ispos: INTEGER_32
		local
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_n := size
			l_stride := stride
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_k := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < {NATURAL_32} 1))
				loop
					if data[(l_stride * l_j + l_k).to_integer_32] <= 0.0 then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_k := l_k + 1
					end
				end
				if not l_return then
					l_j := l_j + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_vector_isneg: INTEGER_32
		local
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_n := size
			l_stride := stride
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_k := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < {NATURAL_32} 1))
				loop
					if data[(l_stride * l_j + l_k).to_integer_32] >= 0.0 then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_k := l_k + 1
					end
				end
				if not l_return then
					l_j := l_j + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_vector_isnonneg: INTEGER_32
		local
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_n := size
			l_stride := stride
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_k := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < {NATURAL_32} 1))
				loop
					if data[(l_stride * l_j + l_k).to_integer_32] < 0.0 then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_k := l_k + 1
					end
				end
				if not l_return then
					l_j := l_j + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_vector_max: REAL_64
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_max: REAL_64
			l_i: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x > l_max then
					l_max := l_x
				end
				if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x.truncated_to_real)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					Result := l_x
					l_return := True
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_max
			end
		end

	gsl_vector_min: REAL_64
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_min: REAL_64
			l_i: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x < l_min then
					l_min := l_x
				end
				if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x.truncated_to_real)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					Result := l_x
					l_return := True
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_min
			end
		end

	gsl_vector_minmax (a_min_out, a_max_out: CE_POINTER [REAL_64])
		require
			not a_min_out.is_void
			not a_max_out.is_void
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_max: REAL_64
			l_min: REAL_64
			l_i: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_break: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_break) and then (l_i < l_N))
			loop
				l_break := False
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x < l_min then
					l_min := l_x
				end
				if l_x > l_max then
					l_max := l_x
				end
				if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x.truncated_to_real)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					l_min := l_x
					l_max := l_x
					l_break := True
				end
				if not (l_break) then
					l_i := l_i + 1
				end
			end
			l_break := False
			a_min_out.item := l_min
			update_class_fields -- Current might have been changed; synchronize
			a_max_out.item := l_max
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_vector_max_index: NATURAL_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_max: REAL_64
			l_imax: NATURAL_32
			l_i: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_imax := (0).to_natural_32
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x > l_max then
					l_max := l_x
					l_imax := l_i
				end
				if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x.truncated_to_real)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					Result := l_i
					l_return := True
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_imax
			end
		end

	gsl_vector_min_index: NATURAL_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_min: REAL_64
			l_imin: NATURAL_32
			l_i: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_imin := (0).to_natural_32
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x < l_min then
					l_min := l_x
					l_imin := l_i
				end
				if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x.truncated_to_real)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					Result := l_i
					l_return := True
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_imin
			end
		end

	gsl_vector_minmax_index (a_imin_out, a_imax_out: CE_POINTER [NATURAL_32])
		require
			not a_imin_out.is_void
			not a_imax_out.is_void
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_imin: NATURAL_32
			l_imax: NATURAL_32
			l_max: REAL_64
			l_min: REAL_64
			l_i: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_break: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_imin := (0).to_natural_32
			l_imax := (0).to_natural_32
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_break) and then (l_i < l_N))
			loop
				l_break := False
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x < l_min then
					l_min := l_x
					l_imin := l_i
				end
				if l_x > l_max then
					l_max := l_x
					l_imax := l_i
				end
				if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x.truncated_to_real)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					l_imin := l_i
					l_imax := l_i
					l_break := True
				end
				if not (l_break) then
					l_i := l_i + 1
				end
			end
			l_break := False
			a_imin_out.item := l_imin
			update_class_fields -- Current might have been changed; synchronize
			a_imax_out.item := l_imax
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_vector_swap (a_w: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_w.is_void
		local
			l_d1: CE_POINTER [REAL_64]
			l_d2: CE_POINTER [REAL_64]
			l_size: NATURAL_32
			l_s1: NATURAL_32
			l_s2: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_d1 := data
			l_d2 := a_w.item.data
			l_size := size
			l_s1 := stride
			l_s2 := a_w.item.stride
			if size /= a_w.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector lengths must be equal"), ce_string ("swap_source.c"), 32, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_size))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_tmp := l_d1[(l_i * l_s1 + l_k).to_integer_32]
						l_d1[(l_i * l_s1 + l_k).to_integer_32] := l_d2[(l_i * l_s2 + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_d2[(l_i * l_s2 + l_k).to_integer_32] := l_tmp
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_vector_swap_elements (a_i, a_j: NATURAL_32): INTEGER_32
		local
			l_data: CE_POINTER [REAL_64]
			l_size: NATURAL_32
			l_stride: NATURAL_32
			l_s: NATURAL_32
			l_k: NATURAL_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_data := data
			l_size := size
			l_stride := stride
			if a_i >= l_size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first index is out of range"), ce_string ("swap_source.c"), 57, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second index is out of range"), ce_string ("swap_source.c"), 62, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_i /= a_j then
					l_s := l_stride
					l_k := (0).to_natural_32
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_tmp := l_data[(a_j * l_s + l_k).to_integer_32]
						l_data[(a_j * l_s + l_k).to_integer_32] := l_data[(a_i * l_s + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_data[(a_i * l_s + l_k).to_integer_32] := l_tmp
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
				end
				Result := 0
			end
		end

	gsl_vector_reverse: INTEGER_32
		local
			l_data: CE_POINTER [REAL_64]
			l_size: NATURAL_32
			l_stride: NATURAL_32
			l_s: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_tmp: REAL_64
			l_return: BOOLEAN
		do
			l_data := data
			l_size := size
			l_stride := stride
			l_s := l_stride
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size // {NATURAL_32} 2))
			loop
				l_j := (l_size - l_i) - {NATURAL_32} 1
				l_k := (0).to_natural_32
				from
				until
					not (l_k < {NATURAL_32} 1)
				loop
					l_tmp := l_data[(l_j * l_s + l_k).to_integer_32]
					l_data[(l_j * l_s + l_k).to_integer_32] := l_data[(l_i * l_s + l_k).to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_data[(l_i * l_s + l_k).to_integer_32] := l_tmp
					update_class_fields -- Current might have been changed; synchronize
					l_k := l_k + 1
				end
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_vector_memcpy (a_src: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_size: NATURAL_32
			l_dest_size: NATURAL_32
			l_src_stride: NATURAL_32
			l_dest_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_src_size := a_src.item.size
			l_dest_size := size
			if l_src_size /= l_dest_size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector lengths are not equal"), ce_string ("copy_source.c"), 29, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_src_stride := a_src.item.stride
				l_dest_stride := stride
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_src_size))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						data[(l_dest_stride * l_j + l_k).to_integer_32] := a_src.item.data[(l_src_stride * l_j + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_vector_sub (a_b___0: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_N: NATURAL_32
			l_stride_a: NATURAL_32
			l_stride_b: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_N := size
			if a_b___0.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vectors must have same length"), ce_string ("oper_source.c"), 52, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_stride_a := stride
				l_stride_b := a_b___0.item.stride
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					data[(l_i * l_stride_a).to_integer_32] := data[(l_i * l_stride_a).to_integer_32] - (a_b___0.item.data[(l_i * l_stride_b).to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_vector_mul (a_b___0: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_N: NATURAL_32
			l_stride_a: NATURAL_32
			l_stride_b: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_N := size
			if a_b___0.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vectors must have same length"), ce_string ("oper_source.c"), 77, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_stride_a := stride
				l_stride_b := a_b___0.item.stride
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					data[(l_i * l_stride_a).to_integer_32] := data[(l_i * l_stride_a).to_integer_32] * (a_b___0.item.data[(l_i * l_stride_b).to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_vector_scale (a_x: REAL_64): INTEGER_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_i: NATURAL_32
		do
			l_N := size
			l_stride := stride
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				data[(l_i * l_stride).to_integer_32] := data[(l_i * l_stride).to_integer_32] * a_x
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_vector_add_constant (a_x: REAL_64): INTEGER_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_i: NATURAL_32
		do
			l_N := size
			l_stride := stride
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				data[(l_i * l_stride).to_integer_32] := data[(l_i * l_stride).to_integer_32] + a_x
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_vector_fread (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fread (a_stream, data, size, stride)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_vector_fwrite (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fwrite (a_stream, data, size, stride)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_vector_fprintf (a_stream: CE_POINTER [SD_IO_FILE]; a_format: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_raw_fprintf (a_stream, data, size, stride, a_format)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_vector_fscanf (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_raw_fscanf (a_stream, data, size, stride)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_vector_subvector (a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_s: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("subvector_source.c"), 28, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) >= size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("view would extend past end of vector"), ce_string ("subvector_source.c"), 34, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := data + (stride * a_offset).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := stride
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_vector_subvector_with_stride (a_offset, a_stride, a_n: NATURAL_32): PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_s: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("subvector_source.c"), 59, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_stride = {NATURAL_32} 0) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("stride must be positive integer"), ce_string ("subvector_source.c"), 65, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) * a_stride >= size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("view would extend past end of vector"), ce_string ("subvector_source.c"), 71, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := data + (stride * a_offset).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := stride * a_stride
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_vector_const_subvector_with_stride (a_offset, a_stride, a_n: NATURAL_32): PD_GSL_VECTOR_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_CONST_VIEW
			l_s: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("subvector_source.c"), 59, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_stride = {NATURAL_32} 0) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("stride must be positive integer"), ce_string ("subvector_source.c"), 65, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) * a_stride >= size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("view would extend past end of vector"), ce_string ("subvector_source.c"), 71, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := (data + (stride * a_offset).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_s.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := (stride * a_stride)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_vector_alloc_from_vector (a_offset, a_n, a_stride: NATURAL_32): CE_POINTER [PD_GSL_VECTOR]
		local
			l_v: CE_POINTER [PD_GSL_VECTOR]
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("init_source.c"), 131, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_stride = {NATURAL_32} 0) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("stride must be positive integer"), ce_string ("init_source.c"), 136, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) * a_stride >= size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector would extend past end of block"), ce_string ("init_source.c"), 141, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_tmp := malloc ((create {PD_GSL_VECTOR}.make).structure_size.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v := create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_cast (l_tmp)
				if l_v.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for vector struct"), ce_string ("init_source.c"), 149, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_v.item.data := data + (stride * a_offset).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.stride := a_stride * stride
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				Result := l_v
			end
		end

	gsl_vector_set_all (a_x: REAL_64)
		local
			l_data: CE_POINTER [REAL_64]
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_i: NATURAL_32
		do
			l_data := data
			l_n := size
			l_stride := stride
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_data[(l_i * l_stride).to_integer_32] := a_x
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_vector_set_zero
		local
			l_data: CE_POINTER [REAL_64]
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_zero: REAL_64
			l_i: NATURAL_32
		do
			l_data := data
			l_n := size
			l_stride := stride
			l_zero := 0.0
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_data[(l_i * l_stride).to_integer_32] := l_zero
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_vector_set_basis (a_i: NATURAL_32): INTEGER_32
		local
			l_data: CE_POINTER [REAL_64]
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_zero: REAL_64
			l_one: REAL_64
			l_k: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_data := data
			l_n := size
			l_stride := stride
			l_zero := 0.0
			l_one := 1.0
			if a_i >= l_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index out of range"), ce_string ("init_source.c"), 219, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_k := (0).to_natural_32
				from
				until
					not (l_k < l_n)
				loop
					l_data[(l_k * l_stride).to_integer_32] := l_zero
					update_class_fields -- Current might have been changed; synchronize
					l_k := l_k + 1
				end
				l_data[(a_i * l_stride).to_integer_32] := l_one
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

feature {PD_GSL_VECTOR} -- private routines

	cblas_drot (a_N: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32; a_c___1, a_s: REAL_64)
		local
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
			l_x: REAL_64
			l_y: REAL_64
		do
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				l_x := a_X[l_ix]
				l_y := a_Y[l_iy]
				a_X[l_ix] := a_c___1 * l_x + a_s * l_y
				a_Y[l_iy] := -a_s * l_x + a_c___1 * l_y
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
		end

	solve_tridiag (a_diag: CE_POINTER [REAL_64]; a_d_stride: NATURAL_32; a_offdiag: CE_POINTER [REAL_64]; a_o_stride: NATURAL_32; a_b___0: CE_POINTER [REAL_64]; a_b_stride: NATURAL_32; a_x: CE_POINTER [REAL_64]; a_x_stride, a_N: NATURAL_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_gamma___0: CE_POINTER [REAL_64]
			l_alpha: CE_POINTER [REAL_64]
			l_c___1: CE_POINTER [REAL_64]
			l_z: CE_POINTER [REAL_64]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_status := 0
					l_gamma___0 := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_alpha := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_c___1 := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_z := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					if not (l_gamma___0.is_void) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if not (l_alpha.is_void) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if not (l_c___1.is_void) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if not (l_z.is_void) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate working space"), ce_string ("tridiag.c"), 53, 8)
							Result := 8
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						l_alpha[0] := a_diag[0]
						l_gamma___0[0] := (a_offdiag[0] / l_alpha[0])
						if l_alpha[0] = (0).to_double then
							l_status := 12
						end
						l_i := (1).to_natural_32
						from
						until
							not (l_i < a_N - {NATURAL_32} 1)
						loop
							l_alpha[l_i.to_integer_32] := (a_diag[(a_d_stride * l_i).to_integer_32] - a_offdiag[(a_o_stride * (l_i - {NATURAL_32} 1)).to_integer_32] * l_gamma___0[(l_i - {NATURAL_32} 1).to_integer_32])
							l_gamma___0[l_i.to_integer_32] := (a_offdiag[(a_o_stride * l_i).to_integer_32] / l_alpha[l_i.to_integer_32])
							if l_alpha[l_i.to_integer_32] = (0).to_double then
								l_status := 12
							end
							l_i := l_i + 1
						end
						if a_N > {NATURAL_32} 1 then
							l_alpha[(a_N - {NATURAL_32} 1).to_integer_32] := (a_diag[(a_d_stride * (a_N - {NATURAL_32} 1)).to_integer_32] - a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 2)).to_integer_32] * l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32])
						end
						l_z[0] := a_b___0[0]
						l_i := (1).to_natural_32
						from
						until
							not (l_i < a_N)
						loop
							l_z[l_i.to_integer_32] := (a_b___0[(a_b_stride * l_i).to_integer_32] - (l_gamma___0[(l_i - {NATURAL_32} 1).to_integer_32] * l_z[(l_i - {NATURAL_32} 1).to_integer_32]))
							l_i := l_i + 1
						end
						l_i := (0).to_natural_32
						from
						until
							not (l_i < a_N)
						loop
							l_c___1[l_i.to_integer_32] := l_z[l_i.to_integer_32] / l_alpha[l_i.to_integer_32]
							l_i := l_i + 1
						end
						a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32] := l_c___1[(a_N - {NATURAL_32} 1).to_integer_32]
						if a_N >= {NATURAL_32} 2 then
							l_i := a_N - {NATURAL_32} 2
							l_j := (0).to_natural_32
							from
							until
								not (l_j <= a_N - {NATURAL_32} 2)
							loop
								a_x[(a_x_stride * l_i).to_integer_32] := l_c___1[l_i.to_integer_32] - l_gamma___0[l_i.to_integer_32] * a_x[(a_x_stride * (l_i + {NATURAL_32} 1)).to_integer_32]
								l_j := l_j + 1
								l_i := l_i - 1
							end
						end
				when 13 then -- if done | if done | if done
					if not l_return then
						if not l_z.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_z))
						end
						if not l_c___1.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_c___1))
						end
						if not l_alpha.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_alpha))
						end
						if not l_gamma___0.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_gamma___0))
						end
						if l_status = 12 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be positive definite"), ce_string ("tridiag.c"), 117, l_status)
								Result := l_status
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						Result := l_status
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	solve_tridiag_nonsym (a_diag: CE_POINTER [REAL_64]; a_d_stride: NATURAL_32; a_abovediag: CE_POINTER [REAL_64]; a_a_stride: NATURAL_32; a_belowdiag: CE_POINTER [REAL_64]; a_b_stride: NATURAL_32; a_rhs: CE_POINTER [REAL_64]; a_r_stride: NATURAL_32; a_x: CE_POINTER [REAL_64]; a_x_stride, a_N: NATURAL_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_alpha: CE_POINTER [REAL_64]
			l_z: CE_POINTER [REAL_64]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_t: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_status := 0
					l_alpha := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_z := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					if not (l_alpha.is_void) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (l_z.is_void) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate working space"), ce_string ("tridiag.c"), 146, 8)
							Result := 8
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_alpha[0] := a_diag[0]
						l_z[0] := a_rhs[0]
						if l_alpha[0] = (0).to_double then
							l_status := 12
						end
						l_i := (1).to_natural_32
						from
						until
							not (l_i < a_N)
						loop
							l_t := (a_belowdiag[(a_b_stride * (l_i - {NATURAL_32} 1)).to_integer_32] / l_alpha[(l_i - {NATURAL_32} 1).to_integer_32])
							l_alpha[l_i.to_integer_32] := (a_diag[(a_d_stride * l_i).to_integer_32] - (l_t * a_abovediag[(a_a_stride * (l_i - {NATURAL_32} 1)).to_integer_32]))
							l_z[l_i.to_integer_32] := (a_rhs[(a_r_stride * l_i).to_integer_32] - (l_t * l_z[(l_i - {NATURAL_32} 1).to_integer_32]))
							if l_alpha[l_i.to_integer_32] = (0).to_double then
								l_status := 12
							end
							l_i := l_i + 1
						end
						a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32] := l_z[(a_N - {NATURAL_32} 1).to_integer_32] / l_alpha[(a_N - {NATURAL_32} 1).to_integer_32]
						if a_N >= {NATURAL_32} 2 then
							l_i := a_N - {NATURAL_32} 2
							l_j := (0).to_natural_32
							from
							until
								not (l_j <= a_N - {NATURAL_32} 2)
							loop
								a_x[(a_x_stride * l_i).to_integer_32] := (l_z[l_i.to_integer_32] - (a_abovediag[(a_a_stride * l_i).to_integer_32] * a_x[(a_x_stride * (l_i + {NATURAL_32} 1)).to_integer_32])) / l_alpha[l_i.to_integer_32]
								l_j := l_j + 1
								l_i := l_i - 1
							end
						end
				when 7 then -- if done
					if not l_return then
						if not l_z.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_z))
						end
						if not l_alpha.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_alpha))
						end
						if l_status = 12 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be positive definite"), ce_string ("tridiag.c"), 191, l_status)
								Result := l_status
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						Result := l_status
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	solve_cyc_tridiag (a_diag: CE_POINTER [REAL_64]; a_d_stride: NATURAL_32; a_offdiag: CE_POINTER [REAL_64]; a_o_stride: NATURAL_32; a_b___0: CE_POINTER [REAL_64]; a_b_stride: NATURAL_32; a_x: CE_POINTER [REAL_64]; a_x_stride, a_N: NATURAL_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_delta___0: CE_POINTER [REAL_64]
			l_gamma___0: CE_POINTER [REAL_64]
			l_alpha: CE_POINTER [REAL_64]
			l_c___1: CE_POINTER [REAL_64]
			l_z: CE_POINTER [REAL_64]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_sum: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_status := 0
					l_delta___0 := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_gamma___0 := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_alpha := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_c___1 := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_z := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					if not (l_delta___0.is_void) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 3 then -- else:
						if not (l_gamma___0.is_void) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 6 then -- else:
						if not (l_alpha.is_void) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 9 then -- else:
						if not (l_c___1.is_void) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 12 then -- else:
						if not (l_z.is_void) then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate working space"), ce_string ("tridiag.c"), 225, 8)
							Result := 8
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 15 then -- else:
						l_sum := 0.0
						if a_N = {NATURAL_32} 1 then
							a_x[0] := (a_b___0[0] / a_diag[0])
							Result := 0
							l_return := True
						end
						if not l_return then
							l_alpha[0] := a_diag[0]
							l_gamma___0[0] := (a_offdiag[0] / l_alpha[0])
							l_delta___0[0] := (a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 1)).to_integer_32] / l_alpha[0])
							if l_alpha[0] = (0).to_double then
								l_status := 12
							end
							l_i := (1).to_natural_32
							from
							until
								not (l_i < a_N - {NATURAL_32} 2)
							loop
								l_alpha[l_i.to_integer_32] := (a_diag[(a_d_stride * l_i).to_integer_32] - a_offdiag[(a_o_stride * (l_i - {NATURAL_32} 1)).to_integer_32] * l_gamma___0[(l_i - {NATURAL_32} 1).to_integer_32])
								l_gamma___0[l_i.to_integer_32] := (a_offdiag[(a_o_stride * l_i).to_integer_32] / l_alpha[l_i.to_integer_32])
								l_delta___0[l_i.to_integer_32] := (-l_delta___0[(l_i - {NATURAL_32} 1).to_integer_32] * a_offdiag[(a_o_stride * (l_i - {NATURAL_32} 1)).to_integer_32]) / l_alpha[l_i.to_integer_32]
								if l_alpha[l_i.to_integer_32] = (0).to_double then
									l_status := 12
								end
								l_i := l_i + 1
							end
							l_i := (0).to_natural_32
							from
							until
								not (l_i < a_N - {NATURAL_32} 2)
							loop
								l_sum := l_sum + ((l_alpha[l_i.to_integer_32] * l_delta___0[l_i.to_integer_32]) * l_delta___0[l_i.to_integer_32])
								l_i := l_i + 1
							end
							l_alpha[(a_N - {NATURAL_32} 2).to_integer_32] := (a_diag[(a_d_stride * (a_N - {NATURAL_32} 2)).to_integer_32] - a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 3)).to_integer_32] * l_gamma___0[(a_N - {NATURAL_32} 3).to_integer_32])
							l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32] := ((a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 2)).to_integer_32] - a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 3)).to_integer_32] * l_delta___0[(a_N - {NATURAL_32} 3).to_integer_32]) / l_alpha[(a_N - {NATURAL_32} 2).to_integer_32])
							l_alpha[(a_N - {NATURAL_32} 1).to_integer_32] := ((a_diag[(a_d_stride * (a_N - {NATURAL_32} 1)).to_integer_32] - l_sum) - ((l_alpha[(a_N - {NATURAL_32} 2).to_integer_32] * l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32]) * l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32]))
							l_z[0] := a_b___0[0]
							l_i := (1).to_natural_32
							from
							until
								not (l_i < a_N - {NATURAL_32} 1)
							loop
								l_z[l_i.to_integer_32] := (a_b___0[(a_b_stride * l_i).to_integer_32] - (l_z[(l_i - {NATURAL_32} 1).to_integer_32] * l_gamma___0[(l_i - {NATURAL_32} 1).to_integer_32]))
								l_i := l_i + 1
							end
							l_sum := 0.0
							l_i := (0).to_natural_32
							from
							until
								not (l_i < a_N - {NATURAL_32} 2)
							loop
								l_sum := l_sum + (l_delta___0[l_i.to_integer_32] * l_z[l_i.to_integer_32])
								l_i := l_i + 1
							end
							l_z[(a_N - {NATURAL_32} 1).to_integer_32] := ((a_b___0[(a_b_stride * (a_N - {NATURAL_32} 1)).to_integer_32] - l_sum) - (l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32] * l_z[(a_N - {NATURAL_32} 2).to_integer_32]))
							l_i := (0).to_natural_32
							from
							until
								not (l_i < a_N)
							loop
								l_c___1[l_i.to_integer_32] := l_z[l_i.to_integer_32] / l_alpha[l_i.to_integer_32]
								l_i := l_i + 1
							end
							a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32] := l_c___1[(a_N - {NATURAL_32} 1).to_integer_32]
							a_x[(a_x_stride * (a_N - {NATURAL_32} 2)).to_integer_32] := l_c___1[(a_N - {NATURAL_32} 2).to_integer_32] - l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32] * a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32]
							if a_N >= {NATURAL_32} 3 then
								l_i := a_N - {NATURAL_32} 3
								l_j := (0).to_natural_32
								from
								until
									not (l_j <= a_N - {NATURAL_32} 3)
								loop
									a_x[(a_x_stride * l_i).to_integer_32] := (l_c___1[l_i.to_integer_32] - l_gamma___0[l_i.to_integer_32] * a_x[(a_x_stride * (l_i + {NATURAL_32} 1)).to_integer_32]) - l_delta___0[l_i.to_integer_32] * a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32]
									l_j := l_j + 1
									l_i := l_i - 1
								end
							end
						end
				when 16 then -- if done | if done | if done | if done
					if not l_return then
						if not l_z.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_z))
						end
						if not l_c___1.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_c___1))
						end
						if not l_alpha.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_alpha))
						end
						if not l_gamma___0.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_gamma___0))
						end
						if not l_delta___0.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_delta___0))
						end
						if l_status = 12 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be positive definite"), ce_string ("tridiag.c"), 310, l_status)
								Result := l_status
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						Result := l_status
					end
				when 17 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	solve_cyc_tridiag_nonsym (a_diag: CE_POINTER [REAL_64]; a_d_stride: NATURAL_32; a_abovediag: CE_POINTER [REAL_64]; a_a_stride: NATURAL_32; a_belowdiag: CE_POINTER [REAL_64]; a_b_stride: NATURAL_32; a_rhs: CE_POINTER [REAL_64]; a_r_stride: NATURAL_32; a_x: CE_POINTER [REAL_64]; a_x_stride, a_N: NATURAL_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_alpha: CE_POINTER [REAL_64]
			l_zb: CE_POINTER [REAL_64]
			l_zu: CE_POINTER [REAL_64]
			l_w: CE_POINTER [REAL_64]
			l_beta: REAL_64
			l_q___3: REAL_64
			l_tmp___4: REAL_64
			l_i: NATURAL_32
			l_t: REAL_64
			l_i___0: NATURAL_32
			l_t___0: REAL_64
			l_i___1: NATURAL_32
			l_j: NATURAL_32
			l_vw: REAL_64
			l_vx: REAL_64
			l_i___2: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_status := 0
					l_alpha := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_zb := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_zu := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_w := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					if not (l_alpha.is_void) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if not (l_zb.is_void) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if not (l_zu.is_void) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if not (l_w.is_void) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate working space"), ce_string ("tridiag.c"), 343, 8)
							Result := 8
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						l_zb[0] := a_rhs[0]
						if a_diag[0] /= (0).to_double then
							l_beta := (-a_diag[0])
						else
							l_beta := (1).to_double
						end
						l_q___3 := ((1).to_double - (a_abovediag[0] * a_belowdiag[0]) / (a_diag[0] * a_diag[a_d_stride.to_integer_32]))
						if (fabs (l_q___3 / l_beta)) > 0.5 then
							if (fabs (l_q___3 / l_beta)) < (2).to_double then
								if (fabs (l_q___3 / l_beta)) < (1).to_double then
									l_tmp___4 := 0.5
								else
									l_tmp___4 := (2).to_double
								end
								l_beta := l_beta * l_tmp___4
							end
						end
						l_zu[0] := l_beta
						l_alpha[0] := (a_diag[0] - l_beta)
						if l_alpha[0] = (0).to_double then
							l_status := 12
						end
						l_i := (1).to_natural_32
						from
						until
							not (l_i + {NATURAL_32} 1 < a_N)
						loop
							l_t := (a_belowdiag[(a_b_stride * (l_i - {NATURAL_32} 1)).to_integer_32] / l_alpha[(l_i - {NATURAL_32} 1).to_integer_32])
							l_alpha[l_i.to_integer_32] := (a_diag[(a_d_stride * l_i).to_integer_32] - (l_t * a_abovediag[(a_a_stride * (l_i - {NATURAL_32} 1)).to_integer_32]))
							l_zb[l_i.to_integer_32] := (a_rhs[(a_r_stride * l_i).to_integer_32] - (l_t * l_zb[(l_i - {NATURAL_32} 1).to_integer_32]))
							l_zu[l_i.to_integer_32] := -l_t * l_zu[(l_i - {NATURAL_32} 1).to_integer_32]
							if l_alpha[l_i.to_integer_32] = (0).to_double then
								l_status := 12
							end
							l_i := l_i + 1
						end
						l_i___0 := a_N - {NATURAL_32} 1
						l_t___0 := (a_belowdiag[(a_b_stride * (l_i___0 - {NATURAL_32} 1)).to_integer_32] / l_alpha[(l_i___0 - {NATURAL_32} 1).to_integer_32])
						l_alpha[l_i___0.to_integer_32] := ((a_diag[(a_d_stride * l_i___0).to_integer_32] - (a_abovediag[(a_a_stride * l_i___0).to_integer_32] * a_belowdiag[(a_b_stride * l_i___0).to_integer_32]) / l_beta) - (l_t___0 * a_abovediag[(a_a_stride * (l_i___0 - {NATURAL_32} 1)).to_integer_32]))
						l_zb[l_i___0.to_integer_32] := (a_rhs[(a_r_stride * l_i___0).to_integer_32] - (l_t___0 * l_zb[(l_i___0 - {NATURAL_32} 1).to_integer_32]))
						l_zu[l_i___0.to_integer_32] := (a_abovediag[(a_a_stride * l_i___0).to_integer_32] - (l_t___0 * l_zu[(l_i___0 - {NATURAL_32} 1).to_integer_32]))
						if l_alpha[l_i___0.to_integer_32] = (0).to_double then
							l_status := 12
						end
						l_w[(a_N - {NATURAL_32} 1).to_integer_32] := l_zu[(a_N - {NATURAL_32} 1).to_integer_32] / l_alpha[(a_N - {NATURAL_32} 1).to_integer_32]
						a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32] := l_zb[(a_N - {NATURAL_32} 1).to_integer_32] / l_alpha[(a_N - {NATURAL_32} 1).to_integer_32]
						l_i___1 := a_N - {NATURAL_32} 2
						l_j := (0).to_natural_32
						from
						until
							not (l_j <= a_N - {NATURAL_32} 2)
						loop
							l_w[l_i___1.to_integer_32] := (l_zu[l_i___1.to_integer_32] - (a_abovediag[(a_a_stride * l_i___1).to_integer_32] * l_w[(l_i___1 + {NATURAL_32} 1).to_integer_32])) / l_alpha[l_i___1.to_integer_32]
							a_x[(l_i___1 * a_x_stride).to_integer_32] := (l_zb[l_i___1.to_integer_32] - (a_abovediag[(a_a_stride * l_i___1).to_integer_32] * a_x[(a_x_stride * (l_i___1 + {NATURAL_32} 1)).to_integer_32])) / l_alpha[l_i___1.to_integer_32]
							l_j := l_j + 1
							l_i___1 := l_i___1 - 1
						end
						l_vw := l_w[0] + ((a_belowdiag[(a_b_stride * (a_N - {NATURAL_32} 1)).to_integer_32] / l_beta) * l_w[(a_N - {NATURAL_32} 1).to_integer_32])
						l_vx := a_x[0] + ((a_belowdiag[(a_b_stride * (a_N - {NATURAL_32} 1)).to_integer_32] / l_beta) * a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32])
						if l_vw + (1).to_double = (0).to_double then
							l_status := 12
						end
						l_i___2 := (0).to_natural_32
						from
						until
							not (l_i___2 < a_N)
						loop
							a_x[(l_i___2 * a_x_stride).to_integer_32] := a_x[(l_i___2 * a_x_stride).to_integer_32] - ((l_vx / ((1).to_double + l_vw)) * l_w[l_i___2.to_integer_32])
							l_i___2 := l_i___2 + 1
						end
				when 13 then -- if done | if done | if done
					if not l_return then
						if not l_zb.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_zb))
						end
						if not l_zu.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_zu))
						end
						if not l_w.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_w))
						end
						if not l_alpha.is_void then
							free (create {CE_VOID_POINTER}.make_cast (l_alpha))
						end
						if l_status = 12 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be positive definite"), ce_string ("tridiag.c"), 438, l_status)
								Result := l_status
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
					end
					if not l_return then
						Result := l_status
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	create_schur (a_d0, a_f0, a_d1: REAL_64; a_c___1, a_s: CE_POINTER [REAL_64])
		local
			l_d0: REAL_64
			l_f0: REAL_64
			l_d1: REAL_64
			l_apq: REAL_64
			l_scale: REAL_64
			l_d0_exp: CE_ARRAY [INTEGER_32]
			l_f0_exp: CE_ARRAY [INTEGER_32]
			l_t: REAL_64
			l_tau: REAL_64
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_32_ARRAY} l_d0_exp.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_f0_exp.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_d0 := a_d0
					l_f0 := a_f0
					l_d1 := a_d1
					l_apq := (2.0 * l_d0) * l_f0
					if l_d0 = (0).to_double then
						a_c___1.item := 1.0
						a_s.item := 0.0
						l_return := True
					elseif l_f0 = (0).to_double then
						a_c___1.item := 1.0
						a_s.item := 0.0
						l_return := True
					end
					if l_return then
						l_pos := -1 + 19 -- skip if block
					end
				when 1 then -- if:
						if not ((fabs (l_d0)) < 1.4916681462400413e-154) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 17 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 4 then -- else:
						if not ((fabs (l_d0)) > 1.3407807929942596e+154) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						l_pos := -1 + 17 -- goto _L
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 7 then -- else:
						if not ((fabs (l_f0)) < 1.4916681462400413e-154) then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						l_pos := -1 + 17 -- goto _L
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 10 then -- else:
						if not ((fabs (l_f0)) > 1.3407807929942596e+154) then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						l_pos := -1 + 17 -- goto _L
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 13 then -- else:
						if not ((fabs (l_d1)) < 1.4916681462400413e-154) then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						l_pos := -1 + 17 -- goto _L
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 16 then -- else:
						if not ((fabs (l_d1)) > 1.3407807929942596e+154) then
							l_pos := -1 + 18 -- skip if block
						end
				when 17 then -- if: | _L:
						frexp (l_d0, l_d0_exp).do_nothing
						frexp (l_f0, l_f0_exp).do_nothing
						l_scale := ldexp (1.0, -(l_d0_exp.item + l_f0_exp.item) // 4)
						l_d0 := l_d0 * l_scale
						l_f0 := l_f0 * l_scale
						l_d1 := l_d1 * l_scale
						l_apq := (2.0 * l_d0) * l_f0
				when 18 then -- if done | if done | if done | if done | if done
						if l_apq /= 0.0 then
							l_tau := (l_f0 * l_f0 + (l_d1 + l_d0) * (l_d1 - l_d0)) / l_apq
							if l_tau >= 0.0 then
								l_t := 1.0 / (l_tau + (hypot (1.0, l_tau)))
							else
								l_t := -1.0 / (-l_tau + (hypot (1.0, l_tau)))
							end
							a_c___1.item := 1.0 / (hypot (1.0, l_t))
							a_s.item := l_t * a_c___1.item
						else
							a_c___1.item := 1.0
							a_s.item := 0.0
						end
				when 19 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sort_smallest (a_dest: CE_POINTER [REAL_64]; a_k: NATURAL_32; a_src: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_xbound: REAL_64
			l_i1: NATURAL_32
			l_xi: REAL_64
			l_return, l_break, l_continue: BOOLEAN
		do
			if a_k > a_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subset length k exceeds vector length n"), ce_string ("subset_source.c"), 28, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_k = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				elseif a_n = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_j := (1).to_natural_32
				l_xbound := a_src[({NATURAL_32} 0).to_integer_32]
				a_dest[0] := l_xbound
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_continue := False
					l_xi := a_src[(l_i * a_stride).to_integer_32]
					if l_j < a_k then
						l_j := l_j + 1
					elseif l_xi >= l_xbound then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						l_i1 := l_j - {NATURAL_32} 1
						from
						until
							not (not (l_break) and then (l_i1 > {NATURAL_32} 0))
						loop
							l_break := False
							if l_xi > a_dest[(l_i1 - {NATURAL_32} 1).to_integer_32] then
								l_break := True
							end
							if not (l_return or l_break) then
								a_dest[l_i1.to_integer_32] := a_dest[(l_i1 - {NATURAL_32} 1).to_integer_32]
								l_i1 := l_i1 - 1
							end
						end
						l_break := False
						a_dest[l_i1.to_integer_32] := l_xi
						l_xbound := a_dest[(l_j - {NATURAL_32} 1).to_integer_32]
					end
					if (not l_return) then
						l_i := l_i + 1
					end
				end
				l_continue := False
				Result := 0
			end
		end

	gsl_sort_largest (a_dest: CE_POINTER [REAL_64]; a_k: NATURAL_32; a_src: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_xbound: REAL_64
			l_i1: NATURAL_32
			l_xi: REAL_64
			l_return, l_break, l_continue: BOOLEAN
		do
			if a_k > a_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subset length k exceeds vector length n"), ce_string ("subset_source.c"), 93, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_k = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				elseif a_n = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_j := (1).to_natural_32
				l_xbound := a_src[({NATURAL_32} 0).to_integer_32]
				a_dest[0] := l_xbound
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_continue := False
					l_xi := a_src[(l_i * a_stride).to_integer_32]
					if l_j < a_k then
						l_j := l_j + 1
					elseif l_xi <= l_xbound then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						l_i1 := l_j - {NATURAL_32} 1
						from
						until
							not (not (l_break) and then (l_i1 > {NATURAL_32} 0))
						loop
							l_break := False
							if l_xi < a_dest[(l_i1 - {NATURAL_32} 1).to_integer_32] then
								l_break := True
							end
							if not (l_return or l_break) then
								a_dest[l_i1.to_integer_32] := a_dest[(l_i1 - {NATURAL_32} 1).to_integer_32]
								l_i1 := l_i1 - 1
							end
						end
						l_break := False
						a_dest[l_i1.to_integer_32] := l_xi
						l_xbound := a_dest[(l_j - {NATURAL_32} 1).to_integer_32]
					end
					if (not l_return) then
						l_i := l_i + 1
					end
				end
				l_continue := False
				Result := 0
			end
		end

	gsl_sort (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_k: NATURAL_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					my_downheap (a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_data[({NATURAL_32} 0).to_integer_32]
					a_data[({NATURAL_32} 0).to_integer_32] := a_data[(l_N * a_stride).to_integer_32]
					a_data[(l_N * a_stride).to_integer_32] := l_tmp
					l_N := l_N - 1
					my_downheap (a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	my_downheap (a_data: CE_POINTER [REAL_64]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_data.is_void
		local
			l_k: NATURAL_32
			l_v: REAL_64
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_v := a_data[(l_k * a_stride).to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(l_j * a_stride).to_integer_32] < a_data[((l_j + {NATURAL_32} 1) * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (l_v < a_data[(l_j * a_stride).to_integer_32]) then
					l_break := True
				end
				if not (l_break) then
					a_data[(l_k * a_stride).to_integer_32] := a_data[(l_j * a_stride).to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_data[(l_k * a_stride).to_integer_32] := l_v
		end

	gsl_sort_smallest_index (a_p: CE_POINTER [NATURAL_32]; a_k: NATURAL_32; a_src: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_xbound: REAL_64
			l_i1: NATURAL_32
			l_xi: REAL_64
			l_return, l_break, l_continue: BOOLEAN
		do
			if a_k > a_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subset length k exceeds vector length n"), ce_string ("subsetind_source.c"), 28, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_k = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				elseif a_n = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_j := (1).to_natural_32
				l_xbound := a_src[({NATURAL_32} 0).to_integer_32]
				a_p[0] := (0).to_natural_32
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_continue := False
					l_xi := a_src[(l_i * a_stride).to_integer_32]
					if l_j < a_k then
						l_j := l_j + 1
					elseif l_xi >= l_xbound then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						l_i1 := l_j - {NATURAL_32} 1
						from
						until
							not (not (l_break) and then (l_i1 > {NATURAL_32} 0))
						loop
							l_break := False
							if l_xi > a_src[(a_p[(l_i1 - {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
								l_break := True
							end
							if not (l_return or l_break) then
								a_p[l_i1.to_integer_32] := a_p[(l_i1 - {NATURAL_32} 1).to_integer_32]
								l_i1 := l_i1 - 1
							end
						end
						l_break := False
						a_p[l_i1.to_integer_32] := l_i
						l_xbound := a_src[(a_p[(l_j - {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]
					end
					if (not l_return) then
						l_i := l_i + 1
					end
				end
				l_continue := False
				Result := 0
			end
		end

	gsl_sort_largest_index (a_p: CE_POINTER [NATURAL_32]; a_k: NATURAL_32; a_src: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_xbound: REAL_64
			l_i1: NATURAL_32
			l_xi: REAL_64
			l_return, l_break, l_continue: BOOLEAN
		do
			if a_k > a_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subset length k exceeds vector length n"), ce_string ("subsetind_source.c"), 93, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_k = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				elseif a_n = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_j := (1).to_natural_32
				l_xbound := a_src[({NATURAL_32} 0).to_integer_32]
				a_p[0] := (0).to_natural_32
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_continue := False
					l_xi := a_src[(l_i * a_stride).to_integer_32]
					if l_j < a_k then
						l_j := l_j + 1
					elseif l_xi <= l_xbound then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						l_i1 := l_j - {NATURAL_32} 1
						from
						until
							not (not (l_break) and then (l_i1 > {NATURAL_32} 0))
						loop
							l_break := False
							if l_xi < a_src[(a_stride * a_p[(l_i1 - {NATURAL_32} 1).to_integer_32]).to_integer_32] then
								l_break := True
							end
							if not (l_return or l_break) then
								a_p[l_i1.to_integer_32] := a_p[(l_i1 - {NATURAL_32} 1).to_integer_32]
								l_i1 := l_i1 - 1
							end
						end
						l_break := False
						a_p[l_i1.to_integer_32] := l_i
						l_xbound := a_src[(a_stride * a_p[(l_j - {NATURAL_32} 1).to_integer_32]).to_integer_32]
					end
					if (not l_return) then
						l_i := l_i + 1
					end
				end
				l_continue := False
				Result := 0
			end
		end

	cblas_daxpy (a_N: INTEGER_32; a_alpha: REAL_64; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_m___4: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha = 0.0 then
						l_return := True
					end
					if l_return then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						if a_incX /= 1 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if a_incY = 1 then
							l_m___4 := a_N \\ 4
							l_i := 0
							from
							until
								not (l_i < l_m___4)
							loop
								a_Y[l_i] := a_Y[l_i] + (a_alpha * a_X[l_i])
								l_i := l_i + 1
							end
							l_i := l_m___4
							from
							until
								not (l_i + 3 < a_N)
							loop
								a_Y[l_i] := a_Y[l_i] + (a_alpha * a_X[l_i])
								a_Y[(l_i + 1)] := a_Y[(l_i + 1)] + (a_alpha * a_X[(l_i + 1)])
								a_Y[(l_i + 2)] := a_Y[(l_i + 2)] + (a_alpha * a_X[(l_i + 2)])
								a_Y[(l_i + 3)] := a_Y[(l_i + 3)] + (a_alpha * a_X[(l_i + 3)])
								l_i := l_i + (4)
							end
						else
							l_pos := -1 + 4 -- goto _L
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 4 then -- else: | _L:
						if a_incX > 0 then
							l_tmp := 0
						else
							l_tmp := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp
						if a_incY > 0 then
							l_tmp___0 := 0
						else
							l_tmp___0 := (a_N - 1) * -a_incY
						end
						l_iy := l_tmp___0
						l_i := 0
						from
						until
							not (l_i < a_N)
						loop
							a_Y[l_iy] := a_Y[l_iy] + (a_alpha * a_X[l_ix])
							l_ix := l_ix + a_incX
							l_iy := l_iy + a_incY
							l_i := l_i + 1
						end
				when 5 then -- if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dcopy (a_N: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
		do
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				a_Y[l_iy] := a_X[l_ix]
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
		end

	cblas_ddot (a_N: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32): REAL_64
		local
			l_r___3: REAL_64
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
		do
			l_r___3 := 0.0
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				l_r___3 := l_r___3 + (a_X[l_ix] * a_Y[l_iy])
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
			Result := l_r___3
		end

	cblas_dger (a_order: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_64; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_tmp___0: REAL_64
			l_jy: INTEGER_32
			l_tmp___1: INTEGER_32
			l_jy___0: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: REAL_64
			l_ix___0: INTEGER_32
			l_tmp___4: INTEGER_32
			l_return: BOOLEAN
		do
			if a_order = {NATURAL_32} 101 then
				if a_incX > 0 then
					l_tmp := 0
				else
					l_tmp := (a_M - 1) * -a_incX
				end
				l_ix := l_tmp
				l_i := 0
				from
				until
					not (not (l_return) and then (l_i < a_M))
				loop
					l_tmp___0 := a_alpha * a_X[l_ix]
					if a_incY > 0 then
						l_tmp___1 := 0
					else
						l_tmp___1 := (a_N - 1) * -a_incY
					end
					l_jy := l_tmp___1
					l_j := 0
					from
					until
						not (l_j < a_N)
					loop
						a_A___0[(a_lda * l_i + l_j)] := a_A___0[(a_lda * l_i + l_j)] + (a_Y[l_jy] * l_tmp___0)
						l_jy := l_jy + a_incY
						l_j := l_j + 1
					end
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
			elseif a_order = {NATURAL_32} 102 then
				if a_incY > 0 then
					l_tmp___2 := 0
				else
					l_tmp___2 := (a_N - 1) * -a_incY
				end
				l_jy___0 := l_tmp___2
				l_j := 0
				from
				until
					not (not (l_return) and then (l_j < a_N))
				loop
					l_tmp___3 := a_alpha * a_Y[l_jy___0]
					if a_incX > 0 then
						l_tmp___4 := 0
					else
						l_tmp___4 := (a_M - 1) * -a_incX
					end
					l_ix___0 := l_tmp___4
					l_i := 0
					from
					until
						not (l_i < a_M)
					loop
						a_A___0[(l_i + a_lda * l_j)] := a_A___0[(l_i + a_lda * l_j)] + (a_X[l_ix___0] * l_tmp___3)
						l_ix___0 := l_ix___0 + a_incX
						l_i := l_i + 1
					end
					l_jy___0 := l_jy___0 + a_incY
					l_j := l_j + 1
				end
			else
				(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_ger.h"), ce_string ("unrecognized operation")])
			end
		end

	cblas_dnrm2 (a_N: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32): REAL_64
		local
			l_scale: REAL_64
			l_ssq: REAL_64
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_x: REAL_64
			l_ax: REAL_64
			l_return: BOOLEAN
		do
			l_scale := 0.0
			l_ssq := 1.0
			l_ix := 0
			if a_N <= 0 then
				Result := (0).to_double
				l_return := True
			elseif a_incX <= 0 then
				Result := (0).to_double
				l_return := True
			elseif a_N = 1 then
				Result := (fabs (a_X[0]))
				l_return := True
			end
			if not l_return then
				l_i := 0
				from
				until
					not (l_i < a_N)
				loop
					l_x := a_X[l_ix]
					if l_x /= 0.0 then
						l_ax := (fabs (l_x))
						if l_scale < l_ax then
							l_ssq := 1.0 + (l_ssq * (l_scale / l_ax)) * (l_scale / l_ax)
							l_scale := l_ax
						else
							l_ssq := l_ssq + ((l_ax / l_scale) * (l_ax / l_scale))
						end
					end
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
				Result := (l_scale * (sqrt (l_ssq)))
			end
		end

	cblas_drotm (a_N: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32; a_P___2: CE_POINTER [REAL_64])
		require
			not a_P___2.is_void
		local
			l_n: INTEGER_32
			l_i: INTEGER_32
			l_tmp: INTEGER_32
			l_j: INTEGER_32
			l_tmp___0: INTEGER_32
			l_h11: REAL_64
			l_h21: REAL_64
			l_h12: REAL_64
			l_h22: REAL_64
			l_w: REAL_64
			l_z: REAL_64
			l_return: BOOLEAN
		do
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_i := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_j := l_tmp___0
			if a_P___2[0] = (-1.0) then
				l_h11 := a_P___2[1]
				l_h21 := a_P___2[2]
				l_h12 := a_P___2[3]
				l_h22 := a_P___2[4]
			elseif a_P___2[0] = 0.0 then
				l_h11 := 1.0
				l_h21 := a_P___2[2]
				l_h12 := a_P___2[3]
				l_h22 := 1.0
			elseif a_P___2[0] = 1.0 then
				l_h11 := a_P___2[1]
				l_h21 := -1.0
				l_h12 := 1.0
				l_h22 := a_P___2[4]
			elseif a_P___2[0] = (-2.0) then
				l_return := True
			else
				(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_rotm.h"), ce_string ("unrecognized value of P[0]")])
				l_return := True
			end
			if not l_return then
				l_n := 0
				from
				until
					not (l_n < a_N)
				loop
					l_w := a_X[l_i]
					l_z := a_Y[l_j]
					a_X[l_i] := l_h11 * l_w + l_h12 * l_z
					a_Y[l_j] := l_h21 * l_w + l_h22 * l_z
					l_i := l_i + a_incX
					l_j := l_j + a_incY
					l_n := l_n + 1
				end
			end
		end

	cblas_dscal (a_N: INTEGER_32; a_alpha: REAL_64; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32)
		local
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_return: BOOLEAN
		do
			l_ix := 0
			if a_incX <= 0 then
				l_return := True
			end
			if not l_return then
				l_i := 0
				from
				until
					not (l_i < a_N)
				loop
					a_X[l_ix] := a_X[l_ix] * a_alpha
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
			end
		end

	cblas_dswap (a_N: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: REAL_64
		do
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				l_tmp___1 := a_X[l_ix]
				a_X[l_ix] := a_Y[l_iy]
				a_Y[l_iy] := l_tmp___1
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
		end

	cblas_idamax (a_N: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32): NATURAL_32
		local
			l_max: REAL_64
			l_ix: INTEGER_32
			l_i: INTEGER_32
			l_result: NATURAL_32
			l_return: BOOLEAN
		do
			l_max := 0.0
			l_ix := 0
			l_result := (0).to_natural_32
			if a_incX <= 0 then
				Result := (0).to_natural_32
				l_return := True
			end
			if not l_return then
				l_i := 0
				from
				until
					not (l_i < a_N)
				loop
					if (fabs (a_X[l_ix])) > l_max then
						l_max := fabs (a_X[l_ix])
						l_result := l_i.to_natural_32
					end
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
				Result := l_result
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			size := a_other.size
			stride := a_other.stride
			data := a_other.data
			block := a_other.block
			owner := a_other.owner
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				size = a_other.size and
				stride = a_other.stride and
				data.is_equal (a_other.data) and
				block.is_equal (a_other.block) and
				owner = a_other.owner
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("size"),
					create {CE_STRUCTURE_FIELD}.make ("stride"),
					create {CE_STRUCTURE_FIELD}.make ("data"),
					create {CE_STRUCTURE_FIELD}.make ("block"),
					create {CE_STRUCTURE_FIELD}.make ("owner")
				>>)
			init_structure_info
		end

end
