class
	PD_RANLXS_STATE_T

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_ARRAY} xdbl.make (<<12>>)
			create {CE_REAL_64_ARRAY} ydbl.make (<<12>>)
			create {CE_REAL_32_ARRAY} xflt.make (<<24>>)
		end

feature {ANY} -- fields

	xdbl: CE_ARRAY [REAL_64]

	ydbl: CE_ARRAY [REAL_64]

	carry: REAL_64 assign set_carry

	xflt: CE_ARRAY [REAL_32]

	ir: NATURAL_32 assign set_ir

	jr: NATURAL_32 assign set_jr

	is: NATURAL_32 assign set_is

	is_old: NATURAL_32 assign set_is_old

	pr: NATURAL_32 assign set_pr

feature {ANY} -- field setters

	set_carry (a_carry: REAL_64)
		do
			carry := a_carry
			update_memory_field ("carry")
		end

	set_ir (a_ir: NATURAL_32)
		do
			ir := a_ir
			update_memory_field ("ir")
		end

	set_jr (a_jr: NATURAL_32)
		do
			jr := a_jr
			update_memory_field ("jr")
		end

	set_is (a_is: NATURAL_32)
		do
			is := a_is
			update_memory_field ("is")
		end

	set_is_old (a_is_old: NATURAL_32)
		do
			is_old := a_is_old
			update_memory_field ("is_old")
		end

	set_pr (a_pr: NATURAL_32)
		do
			pr := a_pr
			update_memory_field ("pr")
		end

feature {PD_RANLXS_STATE_T} -- private attributes

	next: CE_STATIC [CE_ARRAY [INTEGER_32]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_32_ARRAY}.make_initialized (<<
					1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 0
				>>)
		end

	sbase: CE_STATIC [REAL_64]
		once ("PROCESS")
			Result.item := 16777216.0
		end

	sone_bit: CE_STATIC [REAL_64]
		once ("PROCESS")
			Result.item := (1.0 / 16777216.0)
		end

	shift: CE_STATIC [REAL_64]
		once ("PROCESS")
			Result.item := 268435456.0
		end

feature {ANY} -- public routines

	increment_state
		local
			l_k: INTEGER_32
			l_kmax___0: INTEGER_32
			l_m___4: INTEGER_32
			l_x: REAL_64
			l_y1: REAL_64
			l_y2: REAL_64
			l_y3: REAL_64
			l_xflt: CE_POINTER [REAL_32]
			l_xdbl: CE_POINTER [REAL_64]
			l_ydbl: CE_POINTER [REAL_64]
			l_carry: REAL_64
			l_ir: NATURAL_32
			l_jr: NATURAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
		do
			l_xflt := xflt
			l_xdbl := xdbl
			l_ydbl := ydbl
			l_carry := carry
			l_ir := ir
			l_jr := jr
			l_k := 0
			from
			until
				not (l_ir > {NATURAL_32} 0)
			loop
				l_y1 := l_xdbl[l_jr.to_integer_32] - l_xdbl[l_ir.to_integer_32]
				l_y2 := l_y1 - l_carry
				if l_y2 < (0).to_double then
					l_carry := (create {P_RANLXS_STATIC}).one_bit.item
					l_y2 := l_y2 + (1).to_double
				else
					l_carry := (0).to_double
				end
				l_xdbl[l_ir.to_integer_32] := l_y2
				update_class_fields -- Current might have been changed; synchronize
				l_ir := (next.item[l_ir.to_integer_32]).to_natural_32
				l_jr := (next.item[l_jr.to_integer_32]).to_natural_32
				l_k := l_k + 1
			end
			l_kmax___0 := (pr - {NATURAL_32} 12).to_integer_32
			from
			until
				not (l_k <= l_kmax___0)
			loop
				l_y1 := l_xdbl[7] - l_xdbl[0]
				l_y1 := l_y1 - l_carry
				l_y2 := l_xdbl[8] - l_xdbl[1]
				if l_y1 < (0).to_double then
					l_y2 := l_y2 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y1 := l_y1 + (1).to_double
				end
				l_xdbl[0] := l_y1
				update_class_fields -- Current might have been changed; synchronize
				l_y3 := l_xdbl[9] - l_xdbl[2]
				if l_y2 < (0).to_double then
					l_y3 := l_y3 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y2 := l_y2 + (1).to_double
				end
				l_xdbl[1] := l_y2
				update_class_fields -- Current might have been changed; synchronize
				l_y1 := l_xdbl[10] - l_xdbl[3]
				if l_y3 < (0).to_double then
					l_y1 := l_y1 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y3 := l_y3 + (1).to_double
				end
				l_xdbl[2] := l_y3
				update_class_fields -- Current might have been changed; synchronize
				l_y2 := l_xdbl[11] - l_xdbl[4]
				if l_y1 < (0).to_double then
					l_y2 := l_y2 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y1 := l_y1 + (1).to_double
				end
				l_xdbl[3] := l_y1
				update_class_fields -- Current might have been changed; synchronize
				l_y3 := l_xdbl[0] - l_xdbl[5]
				if l_y2 < (0).to_double then
					l_y3 := l_y3 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y2 := l_y2 + (1).to_double
				end
				l_xdbl[4] := l_y2
				update_class_fields -- Current might have been changed; synchronize
				l_y1 := l_xdbl[1] - l_xdbl[6]
				if l_y3 < (0).to_double then
					l_y1 := l_y1 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y3 := l_y3 + (1).to_double
				end
				l_xdbl[5] := l_y3
				update_class_fields -- Current might have been changed; synchronize
				l_y2 := l_xdbl[2] - l_xdbl[7]
				if l_y1 < (0).to_double then
					l_y2 := l_y2 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y1 := l_y1 + (1).to_double
				end
				l_xdbl[6] := l_y1
				update_class_fields -- Current might have been changed; synchronize
				l_y3 := l_xdbl[3] - l_xdbl[8]
				if l_y2 < (0).to_double then
					l_y3 := l_y3 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y2 := l_y2 + (1).to_double
				end
				l_xdbl[7] := l_y2
				update_class_fields -- Current might have been changed; synchronize
				l_y1 := l_xdbl[4] - l_xdbl[9]
				if l_y3 < (0).to_double then
					l_y1 := l_y1 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y3 := l_y3 + (1).to_double
				end
				l_xdbl[8] := l_y3
				update_class_fields -- Current might have been changed; synchronize
				l_y2 := l_xdbl[5] - l_xdbl[10]
				if l_y1 < (0).to_double then
					l_y2 := l_y2 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y1 := l_y1 + (1).to_double
				end
				l_xdbl[9] := l_y1
				update_class_fields -- Current might have been changed; synchronize
				l_y3 := l_xdbl[6] - l_xdbl[11]
				if l_y2 < (0).to_double then
					l_y3 := l_y3 - ((create {P_RANLXS_STATIC}).one_bit.item)
					l_y2 := l_y2 + (1).to_double
				end
				l_xdbl[10] := l_y2
				update_class_fields -- Current might have been changed; synchronize
				if l_y3 < (0).to_double then
					l_carry := (create {P_RANLXS_STATIC}).one_bit.item
					l_y3 := l_y3 + (1).to_double
				else
					l_carry := (0).to_double
				end
				l_xdbl[11] := l_y3
				update_class_fields -- Current might have been changed; synchronize
				l_k := l_k + (12)
			end
			l_kmax___0 := pr.to_integer_32
			from
			until
				not (l_k < l_kmax___0)
			loop
				l_y1 := l_xdbl[l_jr.to_integer_32] - l_xdbl[l_ir.to_integer_32]
				l_y2 := l_y1 - l_carry
				if l_y2 < (0).to_double then
					l_carry := (create {P_RANLXS_STATIC}).one_bit.item
					l_y2 := l_y2 + (1).to_double
				else
					l_carry := (0).to_double
				end
				l_xdbl[l_ir.to_integer_32] := l_y2
				update_class_fields -- Current might have been changed; synchronize
				l_ydbl[l_ir.to_integer_32] := l_y2 + shift.item
				update_class_fields -- Current might have been changed; synchronize
				l_ir := (next.item[l_ir.to_integer_32]).to_natural_32
				l_jr := (next.item[l_jr.to_integer_32]).to_natural_32
				l_k := l_k + 1
			end
			l_ydbl[l_ir.to_integer_32] := l_xdbl[l_ir.to_integer_32] + shift.item
			update_class_fields -- Current might have been changed; synchronize
			l_k := next.item[l_ir.to_integer_32]
			from
			until
				not (l_k > 0)
			loop
				l_ydbl[l_k] := l_xdbl[l_k] + shift.item
				update_class_fields -- Current might have been changed; synchronize
				l_k := next.item[l_k]
			end
			l_k := 0
			l_m___4 := 0
			from
			until
				not (l_k < 12)
			loop
				l_x := l_xdbl[l_k]
				l_y2 := l_ydbl[l_k] - shift.item
				if l_y2 > l_x then
					l_y2 := l_y2 - sone_bit.item
				end
				l_y1 := (l_x - l_y2) * sbase.item
				l_tmp := l_m___4
				l_m___4 := l_m___4 + 1
				l_xflt[l_tmp] := l_y1.truncated_to_real
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := l_m___4
				l_m___4 := l_m___4 + 1
				l_xflt[l_tmp___0] := l_y2.truncated_to_real
				update_class_fields -- Current might have been changed; synchronize
				l_k := l_k + 1
			end
			Current.ir := l_ir
			update_class_fields -- Current might have been changed; synchronize
			Current.is := {NATURAL_32} 2 * l_ir
			update_class_fields -- Current might have been changed; synchronize
			Current.is_old := {NATURAL_32} 2 * l_ir
			update_class_fields -- Current might have been changed; synchronize
			Current.jr := l_jr
			update_class_fields -- Current might have been changed; synchronize
			Current.carry := l_carry
			update_class_fields -- Current might have been changed; synchronize
		end

feature {ANY}

	copy (a_other: like Current)
		do
			xdbl := a_other.xdbl.twin
			ydbl := a_other.ydbl.twin
			carry := a_other.carry
			xflt := a_other.xflt.twin
			ir := a_other.ir
			jr := a_other.jr
			is := a_other.is
			is_old := a_other.is_old
			pr := a_other.pr
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				xdbl.is_equal (a_other.xdbl) and
				ydbl.is_equal (a_other.ydbl) and
				carry = a_other.carry and
				xflt.is_equal (a_other.xflt) and
				ir = a_other.ir and
				jr = a_other.jr and
				is = a_other.is and
				is_old = a_other.is_old and
				pr = a_other.pr
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("xdbl", <<12>>),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("ydbl", <<12>>),
					create {CE_STRUCTURE_FIELD}.make ("carry"),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("xflt", <<24>>),
					create {CE_STRUCTURE_FIELD}.make ("ir"),
					create {CE_STRUCTURE_FIELD}.make ("jr"),
					create {CE_STRUCTURE_FIELD}.make ("is"),
					create {CE_STRUCTURE_FIELD}.make ("is_old"),
					create {CE_STRUCTURE_FIELD}.make ("pr")
				>>)
			init_structure_info
		end

end
