class
	PD_GSL_MATRIX

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end
	S_GCC_BUILTIN_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} data.make
			create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]} block.make
			Precursor
		end

feature {ANY} -- fields

	size1: NATURAL_32 assign set_size1

	size2: NATURAL_32 assign set_size2

	tda: NATURAL_32 assign set_tda

	data: CE_POINTER [REAL_64] assign set_data

	block: CE_POINTER [PD_GSL_BLOCK_STRUCT] assign set_block

	owner: INTEGER_32 assign set_owner

feature {ANY} -- field setters

	set_size1 (a_size1: NATURAL_32)
		do
			size1 := a_size1
			update_memory_field ("size1")
		end

	set_size2 (a_size2: NATURAL_32)
		do
			size2 := a_size2
			update_memory_field ("size2")
		end

	set_tda (a_tda: NATURAL_32)
		do
			tda := a_tda
			update_memory_field ("tda")
		end

	set_data (a_data: CE_POINTER [REAL_64])
		do
			data := a_data
			update_memory_field ("data")
		end

	set_block (a_block: CE_POINTER [PD_GSL_BLOCK_STRUCT])
		do
			block := a_block
			update_memory_field ("block")
		end

	set_owner (a_owner: INTEGER_32)
		do
			owner := a_owner
			update_memory_field ("owner")
		end

feature {PD_GSL_MATRIX} -- private attributes

	mvl_tab: CE_STATIC [CE_ARRAY [CE_ARRAY [PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION]]]
		once ("PROCESS")
			Result.item := create {CE_ARRAY_ARRAY [PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION]}.make_initialized (<<
					create {CE_CLASS_ARRAY [PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION]}.make_initialized (<<
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([5, 1]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([5, 4]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([7, 5]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([9, 7]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([10, 10]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([8, 14])
						>>),
					create {CE_CLASS_ARRAY [PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION]}.make_initialized (<<
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([2, 1]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([4, 0]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([7, 1]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([6, 5]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([5, 9]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([7, 11])
						>>),
					create {CE_CLASS_ARRAY [PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION]}.make_initialized (<<
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([1, 0]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([3, 0]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([5, 1]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([4, 5]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([4, 8]),
							create {PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION}.make_initialized ([2, 11])
						>>)
				>>)
		end

feature {ANY} -- public routines

	gsl_matrix_get (a_i, a_j: NATURAL_32): REAL_64
		do
			Result := (data[(a_i * tda + a_j).to_integer_32])
		end

	gsl_matrix_set (a_i, a_j: NATURAL_32; a_x: REAL_64)
		do
			data[(a_i * tda + a_j).to_integer_32] := a_x
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_matrix_ptr (a_i, a_j: NATURAL_32): CE_POINTER [REAL_64]
		do
			Result := (data + (a_i * tda + a_j).to_integer_32)
		end

	gsl_matrix_const_ptr (a_i, a_j: NATURAL_32): CE_POINTER [REAL_64]
		do
			Result := (data + (a_i * tda + a_j).to_integer_32)
		end

	gsl_blas_dgemv (a_TransA: NATURAL_32; a_alpha: REAL_64; a_X: CE_POINTER [PD_GSL_VECTOR]; a_beta: REAL_64; a_Y: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_TransA /= {NATURAL_32} 111 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_N = a_X.item.size then
							if l_M = a_Y.item.size then
								cblas_dgemv ({PC_CBLAS_ORDER}.CblasRowMajor, a_TransA, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32, a_beta, a_Y.item.data, (a_Y.item.stride).to_integer_32)
								update_class_fields -- Current might have been changed; synchronize
								Result := 0
								l_return := True
							else
								l_pos := -1 + 3 -- goto _L___2
							end
						else
							l_pos := -1 + 3 -- goto _L___2
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else: | _L___2:
						if a_TransA /= {NATURAL_32} 112 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if l_M = a_X.item.size then
							if l_N = a_Y.item.size then
								cblas_dgemv ({PC_CBLAS_ORDER}.CblasRowMajor, a_TransA, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32, a_beta, a_Y.item.data, (a_Y.item.stride).to_integer_32)
								update_class_fields -- Current might have been changed; synchronize
								Result := 0
								l_return := True
							else
								l_pos := -1 + 6 -- goto _L___0
							end
						else
							l_pos := -1 + 6 -- goto _L___0
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 636, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_dsymv (a_Uplo: NATURAL_32; a_alpha: REAL_64; a_X: CE_POINTER [PD_GSL_VECTOR]; a_beta: REAL_64; a_Y: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if l_M = l_N then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square"), ce_string ("blas.c"), 775, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 3 then -- else:
						if l_N = a_X.item.size then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 6 then -- else:
						if l_N = a_Y.item.size then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 779, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						cblas_dsymv ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32, a_beta, a_Y.item.data, (a_Y.item.stride).to_integer_32)
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_dtrmv (a_Uplo, a_TransA, a_Diag: NATURAL_32; a_X: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			l_N := size2
			if l_M /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square"), ce_string ("blas.c"), 822, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_X.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 826, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				cblas_dtrmv ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, a_TransA, a_Diag, l_N.to_integer_32, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_blas_dtrsv (a_Uplo, a_TransA, a_Diag: NATURAL_32; a_X: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			l_N := size2
			if l_M /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square"), ce_string ("blas.c"), 915, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_X.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 919, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				cblas_dtrsv ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, a_TransA, a_Diag, l_N.to_integer_32, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_blas_dgemm (a_TransA, a_TransB: NATURAL_32; a_alpha: REAL_64; a_B: CE_POINTER [PD_GSL_MATRIX]; a_beta: REAL_64; a_C: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_B.is_void
			not a_C.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_tmp: NATURAL_32
			l_NA: NATURAL_32
			l_tmp___0: NATURAL_32
			l_MB: NATURAL_32
			l_tmp___1: NATURAL_32
			l_NB: NATURAL_32
			l_tmp___2: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_C.item.size1
					l_N := a_C.item.size2
					if a_TransA = {NATURAL_32} 111 then
						l_tmp := size1
					else
						l_tmp := size2
					end
					l_MA := l_tmp
					if a_TransA = {NATURAL_32} 111 then
						l_tmp___0 := size2
					else
						l_tmp___0 := size1
					end
					l_NA := l_tmp___0
					if a_TransB = {NATURAL_32} 111 then
						l_tmp___1 := a_B.item.size1
					else
						l_tmp___1 := a_B.item.size2
					end
					l_MB := l_tmp___1
					if a_TransB = {NATURAL_32} 111 then
						l_tmp___2 := a_B.item.size2
					else
						l_tmp___2 := a_B.item.size1
					end
					l_NB := l_tmp___2
					if l_M /= l_MA then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_N = l_NB then
							if l_NA = l_MB then
								cblas_dgemm ({PC_CBLAS_ORDER}.CblasRowMajor, a_TransA, a_TransB, l_M.to_integer_32, l_N.to_integer_32, l_NA.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
								update_class_fields -- Current might have been changed; synchronize
								Result := 0
								l_return := True
							else
								l_pos := -1 + 3 -- goto _L___0
							end
						else
							l_pos := -1 + 3 -- goto _L___0
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_354, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 4 then
					if not l_return then
						Result := 0
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_dsymm (a_Side, a_Uplo: NATURAL_32; a_alpha: REAL_64; a_B: CE_POINTER [PD_GSL_MATRIX]; a_beta: REAL_64; a_C: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_B.is_void
			not a_C.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_NA: NATURAL_32
			l_MB: NATURAL_32
			l_NB: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_C.item.size1
					l_N := a_C.item.size2
					l_MA := size1
					l_NA := size2
					l_MB := a_B.item.size1
					l_NB := a_B.item.size2
					if l_MA /= l_NA then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must be square"), ce_string ("blas.c"), 1_465, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 8 -- skip if block
					end
				when 1 then -- if:
						if a_Side /= {NATURAL_32} 141 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_M = l_MA then
							if l_N = l_NB then
								if l_NA = l_MB then
									cblas_dsymm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
									update_class_fields -- Current might have been changed; synchronize
									Result := 0
									l_return := True
								else
									l_pos := -1 + 4 -- goto _L___0
								end
							else
								l_pos := -1 + 4 -- goto _L___0
							end
						else
							l_pos := -1 + 4 -- goto _L___0
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else: | _L___0:
						if a_Side /= {NATURAL_32} 142 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_M = l_MB then
							if l_N = l_NA then
								if l_NB = l_MA then
									cblas_dsymm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
									update_class_fields -- Current might have been changed; synchronize
									Result := 0
									l_return := True
								else
									l_pos := -1 + 7 -- goto _L
								end
							else
								l_pos := -1 + 7 -- goto _L
							end
						else
							l_pos := -1 + 7 -- goto _L
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_478, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_dsyrk (a_Uplo, a_Trans: NATURAL_32; a_alpha, a_beta: REAL_64; a_C: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_C.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_J: NATURAL_32
			l_tmp: NATURAL_32
			l_K: NATURAL_32
			l_tmp___0: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := a_C.item.size1
			l_N := a_C.item.size2
			if a_Trans = {NATURAL_32} 111 then
				l_tmp := size1
			else
				l_tmp := size2
			end
			l_J := l_tmp
			if a_Trans = {NATURAL_32} 111 then
				l_tmp___0 := size2
			else
				l_tmp___0 := size1
			end
			l_K := l_tmp___0
			if l_M /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix C must be square"), ce_string ("blas.c"), 1_659, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= l_J then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_663, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				cblas_dsyrk ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, a_Trans, l_N.to_integer_32, l_K.to_integer_32, a_alpha, data, tda.to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_blas_dsyr2k (a_Uplo, a_Trans: NATURAL_32; a_alpha: REAL_64; a_B: CE_POINTER [PD_GSL_MATRIX]; a_beta: REAL_64; a_C: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_B.is_void
			not a_C.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_tmp: NATURAL_32
			l_NA: NATURAL_32
			l_tmp___0: NATURAL_32
			l_MB: NATURAL_32
			l_tmp___1: NATURAL_32
			l_NB: NATURAL_32
			l_tmp___2: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_C.item.size1
					l_N := a_C.item.size2
					if a_Trans = {NATURAL_32} 111 then
						l_tmp := size1
					else
						l_tmp := size2
					end
					l_MA := l_tmp
					if a_Trans = {NATURAL_32} 111 then
						l_tmp___0 := size2
					else
						l_tmp___0 := size1
					end
					l_NA := l_tmp___0
					if a_Trans = {NATURAL_32} 111 then
						l_tmp___1 := a_B.item.size1
					else
						l_tmp___1 := a_B.item.size2
					end
					l_MB := l_tmp___1
					if a_Trans = {NATURAL_32} 111 then
						l_tmp___2 := a_B.item.size2
					else
						l_tmp___2 := a_B.item.size1
					end
					l_NB := l_tmp___2
					if l_M = l_N then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix C must be square"), ce_string ("blas.c"), 1_820, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 3 then -- else:
						if l_N = l_MA then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 6 then -- else:
						if l_N = l_MB then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 9 then -- else:
						if l_NA = l_NB then
							l_pos := -1 + 11 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_824, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if done | if done | if done
					if not l_return then
						cblas_dsyr2k ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, a_Trans, l_N.to_integer_32, l_NA.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
					end
				when 12 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_dtrmm (a_Side, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_alpha: REAL_64; a_B: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_B.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_NA: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_B.item.size1
					l_N := a_B.item.size2
					l_MA := size1
					l_NA := size2
					if l_MA /= l_NA then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must be square"), ce_string ("blas.c"), 1_997, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 8 -- skip if block
					end
				when 1 then -- if:
						if a_Side /= {NATURAL_32} 141 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_M = l_MA then
							cblas_dtrmm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, a_TransA, a_Diag, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 4 -- goto _L___0
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else: | _L___0:
						if a_Side /= {NATURAL_32} 142 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_N = l_MA then
							cblas_dtrmm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, a_TransA, a_Diag, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 7 -- goto _L
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 2_008, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_dtrsm (a_Side, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_alpha: REAL_64; a_B: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_B.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_NA: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_B.item.size1
					l_N := a_B.item.size2
					l_MA := size1
					l_NA := size2
					if l_MA /= l_NA then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must be square"), ce_string ("blas.c"), 2_116, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 8 -- skip if block
					end
				when 1 then -- if:
						if a_Side /= {NATURAL_32} 141 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_M = l_MA then
							cblas_dtrsm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, a_TransA, a_Diag, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 4 -- goto _L___0
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else: | _L___0:
						if a_Side /= {NATURAL_32} 142 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_N = l_MA then
							cblas_dtrsm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, a_TransA, a_Diag, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 7 -- goto _L
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 2_127, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_bspline_deriv_eval (a_x: REAL_64; a_nderiv: NATURAL_32; a_w: CE_POINTER [PD_GSL_BSPLINE_WORKSPACE]; a_dw: CE_POINTER [PD_GSL_BSPLINE_DERIV_WORKSPACE]): INTEGER_32
		require
			not a_w.is_void
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_istart: CE_ARRAY [NATURAL_32]
			l_iend: CE_ARRAY [NATURAL_32]
			l_error: INTEGER_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_istart.make (<<1>>)
			create {CE_NATURAL_32_ARRAY} l_iend.make (<<1>>)

			if size1 /= a_w.item.n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dB matrix first dimension not of length n"), ce_string ("bspline.c"), 513, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 < a_nderiv + {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dB matrix second dimension must be at least length nderiv+1"), ce_string ("bspline.c"), 519, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_dw.item.k < a_w.item.k then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("derivative workspace is too small"), ce_string ("bspline.c"), 523, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_error := (a_dw.item.dB).item.gsl_bspline_deriv_eval_nonzero (a_x, a_nderiv, l_istart, l_iend, a_w, a_dw)
				update_class_fields -- Current might have been changed; synchronize
				if l_error.to_boolean then
					Result := l_error
					l_return := True
				end
				if not l_return then
					l_j := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_j <= a_nderiv))
					loop
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_istart.item)
						loop
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, 0.0)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_i := l_istart.item
						from
						until
							not (l_i <= l_iend.item)
						loop
							l_tmp := (a_dw.item.dB).item.gsl_matrix_get (l_i - l_istart.item, l_j)
							update_class_fields; -- Current might have been changed; synchronize
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, l_tmp)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_i := l_iend.item + {NATURAL_32} 1
						from
						until
							not (l_i < a_w.item.n)
						loop
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, 0.0)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_j := l_j + 1
					end
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_bspline_deriv_eval_nonzero (a_x: REAL_64; a_nderiv: NATURAL_32; a_istart, a_iend: CE_POINTER [NATURAL_32]; a_w: CE_POINTER [PD_GSL_BSPLINE_WORKSPACE]; a_dw: CE_POINTER [PD_GSL_BSPLINE_DERIV_WORKSPACE]): INTEGER_32
		require
			not a_w.is_void
		local
			l_i: CE_ARRAY [NATURAL_32]
			l_j: NATURAL_32
			l_flag: CE_ARRAY [INTEGER_32]
			l_error: INTEGER_32
			l_min_nderivk: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_i.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_flag.make (<<1>>)

			if size1 /= a_w.item.k then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dB matrix first dimension not of length k"), ce_string ("bspline.c"), 607, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 < a_nderiv + {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dB matrix second dimension must be at least length nderiv+1"), ce_string ("bspline.c"), 613, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_dw.item.k < a_w.item.k then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("derivative workspace is too small"), ce_string ("bspline.c"), 617, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_flag.item := 0
				update_class_fields -- Current might have been changed; synchronize
				l_error := 0
				l_i.item := a_w.item.bspline_find_interval (a_x, l_flag)
				update_class_fields -- Current might have been changed; synchronize
				l_error := a_w.item.bspline_process_interval_for_eval (a_x, l_i, l_flag.item)
				update_class_fields -- Current might have been changed; synchronize
				if l_error.to_boolean then
					Result := l_error
					l_return := True
				end
				if not l_return then
					a_istart.item := (l_i.item - a_w.item.k) + {NATURAL_32} 1
					update_class_fields -- Current might have been changed; synchronize
					a_iend.item := l_i.item
					update_class_fields -- Current might have been changed; synchronize
					bspline_pppack_bsplvd (a_w.item.knots, a_w.item.k, a_x, a_iend.item, a_w.item.deltal, a_w.item.deltar, a_dw.item.A, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)), a_nderiv)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp := GSL_MIN_INT (a_nderiv.to_integer_32, (a_w.item.k - {NATURAL_32} 1).to_integer_32)
					update_class_fields -- Current might have been changed; synchronize
					l_min_nderivk := l_tmp.to_natural_32
					l_j := l_min_nderivk + {NATURAL_32} 1
					from
					until
						not (not (l_return) and then (l_j <= a_nderiv))
					loop
						l_i.item := (0).to_natural_32
						update_class_fields -- Current might have been changed; synchronize
						from
						until
							not (l_i.item < a_w.item.k)
						loop
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i.item, l_j, 0.0)
							update_class_fields -- Current might have been changed; synchronize
							l_i.item := l_i.item + 1
							update_class_fields -- Current might have been changed; synchronize
						end
						l_j := l_j + 1
					end
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_eigen_nonsymmv (a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_s: INTEGER_32
			l_Z: CE_ARRAY [PD_GSL_MATRIX]
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX]} l_Z.make (<<1>>)

			l_N := size1
			if l_N /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("nonsymmv.c"), 165, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_eval.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("nonsymmv.c"), 169, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_evec.item.size1 /= a_evec.item.size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvector matrix must be square"), ce_string ("nonsymmv.c"), 173, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_evec.item.size1 /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvector matrix has wrong size"), ce_string ("nonsymmv.c"), 177, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_Z.item.size1 := l_N
				update_class_fields -- Current might have been changed; synchronize
				l_Z.item.size2 := l_N
				update_class_fields -- Current might have been changed; synchronize
				l_Z.item.tda := {NATURAL_32} 2 * l_N
				update_class_fields -- Current might have been changed; synchronize
				l_Z.item.data := a_evec.item.data
				update_class_fields -- Current might have been changed; synchronize
				l_Z.item.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_Z.item.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_nonsymm_Z (a_eval, l_Z, a_w.item.nonsymm_workspace_p)
				update_class_fields -- Current might have been changed; synchronize
				if (not (a_w.item.Z).is_void) then
					(a_w.item.Z).item.gsl_matrix_memcpy (l_Z).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
				if l_s = 0 then
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.nonsymmv_get_right_eigenvectors (l_Z, a_eval, a_evec, a_w)
					update_class_fields -- Current might have been changed; synchronize
					nonsymmv_normalize_eigenvectors (a_eval, a_evec)
					update_class_fields -- Current might have been changed; synchronize
				end
				Result := l_s
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_eigen_nonsymmv_Z (a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_Z: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]): INTEGER_32
		local
			l_s: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if size1 = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues/eigenvectors"), ce_string ("nonsymmv.c"), 247, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 3 then -- else:
						if a_eval.item.size = size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("nonsymmv.c"), 251, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 6 then -- else:
						if a_evec.item.size1 = a_evec.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvector matrix must be square"), ce_string ("nonsymmv.c"), 255, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 9 then -- else:
						if a_evec.item.size1 = size1 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvector matrix has wrong size"), ce_string ("nonsymmv.c"), 259, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 12 then -- else:
						if a_Z.item.size1 = a_Z.item.size2 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						l_pos := -1 + 16 -- goto _L
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 15 then -- else:
						if a_Z.item.size1 = size1 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Z matrix has wrong dimensions"), ce_string ("nonsymmv.c"), 263, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 18 then -- else:
						a_w.item.Z := a_Z
						update_class_fields -- Current might have been changed; synchronize
						l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_nonsymmv (a_eval, a_evec, a_w)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.Z := create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer)
						update_class_fields -- Current might have been changed; synchronize
						Result := l_s
						l_return := True
				when 19 then -- if done | if done | if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 20 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	nonsymmv_get_right_eigenvectors (a_Z: CE_POINTER [PD_GSL_MATRIX]; a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE])
		require
			not a_w.is_void
		local
			l_N: NATURAL_32
			l_smlnum: REAL_64
			l_bignum: REAL_64
			l_i: INTEGER_32
			l_iu: NATURAL_32
			l_ju: NATURAL_32
			l_ii: NATURAL_32
			l_lambda: CE_ARRAY [PD_GSL_COMPLEX]
			l_lambda_re: REAL_64
			l_lambda_im: REAL_64
			l_Tv: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_Zv: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_y: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_y2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_ev: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_ev2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_dat: CE_ARRAY [REAL_64]
			l_dat_X: CE_ARRAY [REAL_64]
			l_scale: CE_ARRAY [REAL_64]
			l_xnorm: CE_ARRAY [REAL_64]
			l_ecol: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_ecol2: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_complex_pair: INTEGER_32
			l_smin: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_k: INTEGER_32
			l_l: INTEGER_32
			l_bv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___15: REAL_64
			l_lu: NATURAL_32
			l_tmp___16: REAL_64
			l_x: REAL_64
			l_tmp___17: REAL_64
			l_tmp___18: REAL_64
			l_wv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_x11___0: REAL_64
			l_x21: REAL_64
			l_tmp___19: REAL_64
			l_tmp___20: REAL_64
			l_beta: REAL_64
			l_tmp___23: REAL_64
			l_tmp___24: REAL_64
			l_tmp___25: REAL_64
			l_tmp___26: REAL_64
			l_wv___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v1___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v2___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_x___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___27: REAL_64
			l_a___5: REAL_64
			l_tmp___28: REAL_64
			l_tmp___29: REAL_64
			l_bv___0: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_xv___0: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_k___0: NATURAL_32
			l_l___0: INTEGER_32
			l_lambda2: PD_GSL_COMPLEX
			l_tmp___30: REAL_64
			l_tmp___31: REAL_64
			l_tmp___32: REAL_64
			l_tmp___33: REAL_64
			l_tmp___34: REAL_64
			l_tmp___35: REAL_64
			l_tmp___36: REAL_64
			l_tmp___37: REAL_64
			l_tmp___38: REAL_64
			l_tmp___39: REAL_64
			l_lu___0: NATURAL_32
			l_tmp___40: REAL_64
			l_bval: PD_GSL_COMPLEX
			l_x___1: PD_GSL_COMPLEX
			l_tmp___41: REAL_64
			l_wv___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v1___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v2___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_b1___0: PD_GSL_COMPLEX
			l_b2: PD_GSL_COMPLEX
			l_x1___0: PD_GSL_COMPLEX
			l_x2___1: PD_GSL_COMPLEX
			l_beta___0: REAL_64
			l_tmp___44: REAL_64
			l_tmp___45: REAL_64
			l_tmp___46: REAL_64
			l_tmp___47: REAL_64
			l_wv___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v1___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v2___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v3: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v4: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_x___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___48: REAL_64
			l_tmp___49: REAL_64
			l_tmp___50: REAL_64
			l_tmp___51: REAL_64
			l_a___6: REAL_64
			l_tmp___52: REAL_64
			l_tmp___56: REAL_64
			l_tmp___57: REAL_64
			l_tmp___58: REAL_64
			l_tmp___59: REAL_64
			l_tmp___60: REAL_64
			l_tmp___61: REAL_64
			l_a___7: REAL_64
			l_tmp___62: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_COMPLEX]} l_lambda.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_Tv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_Zv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_y.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_y2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_ev.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_ev2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_dat.make (<<4>>)
			create {CE_REAL_64_ARRAY} l_dat_X.make (<<4>>)
			create {CE_REAL_64_ARRAY} l_scale.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_xnorm.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]} l_ecol.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]} l_ecol2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_bv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_wv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_wv___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v1___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v2___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_x___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]} l_bv___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]} l_xv___0.make (<<1>>)
			create l_lambda2.make
			create l_bval.make
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_wv___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v1___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v2___1.make (<<1>>)
			create l_b1___0.make
			create l_b2.make
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_wv___2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v1___2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v2___2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v3.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v4.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_x___2.make (<<1>>)

			l_N := size1
			l_smlnum := (2.2250738585072014e-308 * l_N.to_real_64) / 2.2204460492503131e-16
			l_bignum := (1.0 - 2.2204460492503131e-16) / l_smlnum;
			(a_w.item.work3).item.gsl_vector_set ((0).to_natural_32, 0.0)
			update_class_fields -- Current might have been changed; synchronize
			l_ju := (1).to_natural_32
			from
			until
				not (not (l_return) and then (l_ju < l_N))
			loop
				(a_w.item.work3).item.gsl_vector_set (l_ju, 0.0)
				update_class_fields -- Current might have been changed; synchronize
				l_iu := (0).to_natural_32
				from
				until
					not (l_iu < l_ju)
				loop
					l_tmp := (a_w.item.work3).item.gsl_vector_get (l_ju)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_iu, l_ju)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___1 := fabs (l_tmp___0)
					update_class_fields; -- Current might have been changed; synchronize
					(a_w.item.work3).item.gsl_vector_set (l_ju, l_tmp + l_tmp___1)
					update_class_fields -- Current might have been changed; synchronize
					l_iu := l_iu + 1
				end
				l_ju := l_ju + 1
			end
			l_i := l_N.to_integer_32 - 1
			from
			until
				not (not (l_return) and then (l_i >= 0))
			loop
				l_iu := l_i.to_natural_32
				l_lambda_re := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_iu, l_iu)
				update_class_fields -- Current might have been changed; synchronize
				if l_iu /= {NATURAL_32} 0 then
					l_tmp___8 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_iu, l_iu - {NATURAL_32} 1)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___8 /= 0.0 then
						l_tmp___2 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_iu, l_iu - {NATURAL_32} 1)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := fabs (l_tmp___2)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___4 := sqrt (l_tmp___3)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_iu - {NATURAL_32} 1, l_iu)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___6 := fabs (l_tmp___5)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___7 := sqrt (l_tmp___6)
						update_class_fields -- Current might have been changed; synchronize
						l_lambda_im := l_tmp___4 * l_tmp___7
					else
						l_lambda_im := 0.0
					end
				else
					l_lambda_im := 0.0
				end
				from
				until
					(l_break)
				loop
					l_break := False
					l_lambda.item.dat[0] := l_lambda_re
					update_class_fields -- Current might have been changed; synchronize
					l_lambda.item.dat[1] := l_lambda_im
					update_class_fields -- Current might have been changed; synchronize
					l_break := True
				end
				l_break := False
				l_tmp___13 := fabs (l_lambda_re)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___14 := fabs (l_lambda_im)
				update_class_fields -- Current might have been changed; synchronize
				if 2.2204460492503131e-16 * (l_tmp___13 + l_tmp___14) > l_smlnum then
					l_tmp___11 := fabs (l_lambda_re)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___12 := fabs (l_lambda_im)
					update_class_fields -- Current might have been changed; synchronize
					l_smin := 2.2204460492503131e-16 * (l_tmp___11 + l_tmp___12)
				else
					l_smin := l_smlnum
				end
				if l_smin > 2.0 * 2.2250738585072014e-308 then
					l_smin := l_smin
				else
					l_smin := 2.0 * 2.2250738585072014e-308
				end
				if l_lambda_im = 0.0 then
					a_eval.item.gsl_vector_complex_set (l_iu, (l_lambda.item).twin)
					update_class_fields -- Current might have been changed; synchronize
					l_k := 0
					from
					until
						not (l_k < l_i)
					loop
						l_tmp___15 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_k.to_natural_32, l_iu)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.work).item.gsl_vector_set (l_k.to_natural_32, -l_tmp___15)
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					(a_w.item.work).item.gsl_vector_set (l_iu, 1.0)
					update_class_fields -- Current might have been changed; synchronize
					l_l := l_i - 1
					from
					until
						not (l_l >= 0)
					loop
						l_lu := l_l.to_natural_32
						if l_lu = {NATURAL_32} 0 then
							l_complex_pair := 0
						else
							l_tmp___16 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_lu, l_lu - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_complex_pair := (l_tmp___16 /= 0.0).to_integer
						end
						if not l_complex_pair.to_boolean then
							l_Tv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_lu, l_lu, (1).to_natural_32, (1).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_bv.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_dat, (1).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___17 := (a_w.item.work).item.gsl_vector_get (l_lu)
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set ((0).to_natural_32, l_tmp___17)
							update_class_fields -- Current might have been changed; synchronize
							l_xv.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_dat_X, (1).to_natural_32)).twin
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_Tv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_schur_solve_equation (1.0, l_lambda_re, 1.0, 1.0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_scale, l_xnorm, l_smin).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_x := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							if l_xnorm.item > 1.0 then
								l_tmp___18 := (a_w.item.work3).item.gsl_vector_get (l_lu)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___18 > l_bignum / l_xnorm.item then
									l_x := l_x / (l_xnorm.item)
									l_scale.item := l_scale.item / (l_xnorm.item)
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							if l_scale.item /= 1.0 then
								l_wv.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale.item)
								update_class_fields -- Current might have been changed; synchronize
							end
							(a_w.item.work).item.gsl_vector_set (l_lu, l_x)
							update_class_fields -- Current might have been changed; synchronize
							if l_lu > {NATURAL_32} 0 then
								l_v1.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_lu, (0).to_natural_32, l_lu)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_v2.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_lu)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-l_x, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							end
						else
							l_Tv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_lu - {NATURAL_32} 1, l_lu - {NATURAL_32} 1, (2).to_natural_32, (2).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_bv.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_dat, (2).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___19 := (a_w.item.work).item.gsl_vector_get (l_lu - {NATURAL_32} 1)
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set ((0).to_natural_32, l_tmp___19)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___20 := (a_w.item.work).item.gsl_vector_get (l_lu)
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set ((1).to_natural_32, l_tmp___20)
							update_class_fields -- Current might have been changed; synchronize
							l_xv.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_dat_X, (2).to_natural_32)).twin
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_Tv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_schur_solve_equation (1.0, l_lambda_re, 1.0, 1.0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_scale, l_xnorm, l_smin).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_x11___0 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_x21 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((1).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							if l_xnorm.item > 1.0 then
								l_tmp___25 := (a_w.item.work3).item.gsl_vector_get (l_lu - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___26 := (a_w.item.work3).item.gsl_vector_get (l_lu)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___25 > l_tmp___26 then
									l_tmp___23 := (a_w.item.work3).item.gsl_vector_get (l_lu - {NATURAL_32} 1)
									update_class_fields -- Current might have been changed; synchronize
									l_beta := l_tmp___23
								else
									l_tmp___24 := (a_w.item.work3).item.gsl_vector_get (l_lu)
									update_class_fields -- Current might have been changed; synchronize
									l_beta := l_tmp___24
								end
								if l_beta > l_bignum / l_xnorm.item then
									l_x11___0 := l_x11___0 / (l_xnorm.item)
									l_x21 := l_x21 / (l_xnorm.item)
									l_scale.item := l_scale.item / (l_xnorm.item)
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							if l_scale.item /= 1.0 then
								l_wv___0.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale.item)
								update_class_fields -- Current might have been changed; synchronize
							end
							(a_w.item.work).item.gsl_vector_set (l_lu - {NATURAL_32} 1, l_x11___0)
							update_class_fields; -- Current might have been changed; synchronize
							(a_w.item.work).item.gsl_vector_set (l_lu, l_x21)
							update_class_fields -- Current might have been changed; synchronize
							if l_lu > {NATURAL_32} 1 then
								l_v1___0.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_lu - {NATURAL_32} 1, (0).to_natural_32, l_lu - {NATURAL_32} 1)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_v2___0.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_lu - {NATURAL_32} 1)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-l_x11___0, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								l_v1___0.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_lu, (0).to_natural_32, l_lu - {NATURAL_32} 1)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-l_x21, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							end
							l_l := l_l - 1
						end
						l_l := l_l - 1
					end
					l_ecol.item := (a_evec.item.gsl_matrix_complex_column (l_iu)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_y.item := (a_Z.item.gsl_matrix_column (l_iu)).twin
					update_class_fields -- Current might have been changed; synchronize
					if l_iu > {NATURAL_32} 0 then
						l_Zv.item := (a_Z.item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_iu)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_x___0.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_iu)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___27 := (a_w.item.work).item.gsl_vector_get (l_iu)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_Zv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_tmp___27, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					l_ev.item := (gsl_vector_complex_real (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					update_class_fields -- Current might have been changed; synchronize
					l_ev2.item := (gsl_vector_complex_imag (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					update_class_fields -- Current might have been changed; synchronize
					l_scale.item := 0.0
					update_class_fields -- Current might have been changed; synchronize
					l_ii := (0).to_natural_32
					from
					until
						not (l_ii < l_N)
					loop
						l_tmp___28 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_ii)
						update_class_fields -- Current might have been changed; synchronize
						l_a___5 := l_tmp___28;
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_ii, l_a___5)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_ii, 0.0)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___29 := fabs (l_a___5)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___29 > l_scale.item then
							l_scale.item := fabs (l_a___5)
							update_class_fields -- Current might have been changed; synchronize
						end
						l_ii := l_ii + 1
					end
					if l_scale.item /= 0.0 then
						l_scale.item := 1.0 / l_scale.item
						update_class_fields -- Current might have been changed; synchronize
					end
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale.item)
					update_class_fields -- Current might have been changed; synchronize
				else
					from
					until
						(l_break)
					loop
						l_break := False
						l_lambda2.dat[0] := l_lambda.item.dat[0]
						update_class_fields -- Current might have been changed; synchronize
						l_break := True
					end
					l_break := False
					from
					until
						(l_break)
					loop
						l_break := False
						l_lambda2.dat[1] := -l_lambda.item.dat[1]
						update_class_fields -- Current might have been changed; synchronize
						l_break := True
					end
					l_break := False
					a_eval.item.gsl_vector_complex_set (l_iu - {NATURAL_32} 1, (l_lambda.item).twin)
					update_class_fields -- Current might have been changed; synchronize
					a_eval.item.gsl_vector_complex_set (l_iu, l_lambda2.twin)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___32 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_iu - {NATURAL_32} 1, l_iu)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___33 := fabs (l_tmp___32)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___34 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_iu, l_iu - {NATURAL_32} 1)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___35 := fabs (l_tmp___34)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___33 >= l_tmp___35 then
						(a_w.item.work).item.gsl_vector_set (l_iu - {NATURAL_32} 1, 1.0)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___30 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_iu - {NATURAL_32} 1, l_iu)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.work2).item.gsl_vector_set (l_iu, l_lambda_im / l_tmp___30)
						update_class_fields -- Current might have been changed; synchronize
					else
						l_tmp___31 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_iu, l_iu - {NATURAL_32} 1)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.work).item.gsl_vector_set (l_iu - {NATURAL_32} 1, -l_lambda_im / l_tmp___31)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.work2).item.gsl_vector_set (l_iu, 1.0)
						update_class_fields -- Current might have been changed; synchronize
					end
					(a_w.item.work).item.gsl_vector_set (l_iu, 0.0)
					update_class_fields; -- Current might have been changed; synchronize
					(a_w.item.work2).item.gsl_vector_set (l_iu - {NATURAL_32} 1, 0.0)
					update_class_fields -- Current might have been changed; synchronize
					l_k___0 := (0).to_natural_32
					from
					until
						not (l_k___0 < l_iu - {NATURAL_32} 1)
					loop
						l_tmp___36 := (a_w.item.work).item.gsl_vector_get (l_iu - {NATURAL_32} 1)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___37 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_k___0, l_iu - {NATURAL_32} 1)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.work).item.gsl_vector_set (l_k___0, -l_tmp___36 * l_tmp___37)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___38 := (a_w.item.work2).item.gsl_vector_get (l_iu)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___39 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_k___0, l_iu)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.work2).item.gsl_vector_set (l_k___0, -l_tmp___38 * l_tmp___39)
						update_class_fields -- Current might have been changed; synchronize
						l_k___0 := l_k___0 + 1
					end
					l_l___0 := l_i - 2
					from
					until
						not (not (l_return) and then (l_l___0 >= 0))
					loop
						l_lu___0 := l_l___0.to_natural_32
						if l_lu___0 = {NATURAL_32} 0 then
							l_complex_pair := 0
						else
							l_tmp___40 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_lu___0, l_lu___0 - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_complex_pair := (l_tmp___40 /= 0.0).to_integer
						end
						if not l_complex_pair.to_boolean then
							l_Tv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_lu___0, l_lu___0, (1).to_natural_32, (1).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_bv___0.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_complex_view_array (l_dat, (1).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_xv___0.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_complex_view_array (l_dat_X, (1).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							from
							until
								(l_break)
							loop
								l_break := False
								l_bval.dat[0] := (a_w.item.work).item.gsl_vector_get (l_lu___0)
								update_class_fields -- Current might have been changed; synchronize
								l_bval.dat[1] := (a_w.item.work2).item.gsl_vector_get (l_lu___0)
								update_class_fields -- Current might have been changed; synchronize
								l_break := True
							end
							l_break := False;
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_vector_complex_set ((0).to_natural_32, l_bval.twin)
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_Tv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_schur_solve_equation_z (1.0, l_lambda, 1.0, 1.0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")), l_scale, l_xnorm, l_smin).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							if l_xnorm.item > 1.0 then
								l_tmp___41 := (a_w.item.work3).item.gsl_vector_get (l_lu___0)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___41 > l_bignum / l_xnorm.item then
									(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_blas_zdscal (1.0 / l_xnorm.item)
									update_class_fields -- Current might have been changed; synchronize
									l_scale.item := l_scale.item / (l_xnorm.item)
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							if l_scale.item /= 1.0 then
								l_wv___1.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale.item)
								update_class_fields -- Current might have been changed; synchronize
								l_wv___1.item := ((a_w.item.work2).item.gsl_vector_subvector ((0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale.item)
								update_class_fields -- Current might have been changed; synchronize
							end
							l_x___1 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_vector_complex_get ((0).to_natural_32)).twin
							update_class_fields; -- Current might have been changed; synchronize
							(a_w.item.work).item.gsl_vector_set (l_lu___0, l_x___1.dat[0])
							update_class_fields; -- Current might have been changed; synchronize
							(a_w.item.work2).item.gsl_vector_set (l_lu___0, l_x___1.dat[1])
							update_class_fields -- Current might have been changed; synchronize
							if l_lu___0 > {NATURAL_32} 0 then
								l_v1___1.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_lu___0, (0).to_natural_32, l_lu___0)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_v2___1.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_lu___0)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-l_x___1.dat[0], create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								l_v2___1.item := ((a_w.item.work2).item.gsl_vector_subvector ((0).to_natural_32, l_lu___0)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-l_x___1.dat[1], create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							end
						else
							l_Tv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_lu___0 - {NATURAL_32} 1, l_lu___0 - {NATURAL_32} 1, (2).to_natural_32, (2).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_bv___0.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_complex_view_array (l_dat, (2).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_xv___0.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_complex_view_array (l_dat_X, (2).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							from
							until
								(l_break)
							loop
								l_break := False
								l_b1___0.dat[0] := (a_w.item.work).item.gsl_vector_get (l_lu___0 - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_b1___0.dat[1] := (a_w.item.work2).item.gsl_vector_get (l_lu___0 - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_break := True
							end
							l_break := False
							from
							until
								(l_break)
							loop
								l_break := False
								l_b2.dat[0] := (a_w.item.work).item.gsl_vector_get (l_lu___0)
								update_class_fields -- Current might have been changed; synchronize
								l_b2.dat[1] := (a_w.item.work2).item.gsl_vector_get (l_lu___0)
								update_class_fields -- Current might have been changed; synchronize
								l_break := True
							end
							l_break := False;
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_vector_complex_set ((0).to_natural_32, l_b1___0.twin)
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_vector_complex_set ((1).to_natural_32, l_b2.twin)
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_Tv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_schur_solve_equation_z (1.0, l_lambda, 1.0, 1.0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")), l_scale, l_xnorm, l_smin).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_x1___0 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_vector_complex_get ((0).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_x2___1 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_vector_complex_get ((1).to_natural_32)).twin
							update_class_fields -- Current might have been changed; synchronize
							if l_xnorm.item > 1.0 then
								l_tmp___46 := (a_w.item.work3).item.gsl_vector_get (l_lu___0 - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___47 := (a_w.item.work3).item.gsl_vector_get (l_lu___0)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___46 > l_tmp___47 then
									l_tmp___44 := (a_w.item.work3).item.gsl_vector_get (l_lu___0 - {NATURAL_32} 1)
									update_class_fields -- Current might have been changed; synchronize
									l_beta___0 := l_tmp___44
								else
									l_tmp___45 := (a_w.item.work3).item.gsl_vector_get (l_lu___0)
									update_class_fields -- Current might have been changed; synchronize
									l_beta___0 := l_tmp___45
								end
								if l_beta___0 > l_bignum / l_xnorm.item then
									(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_blas_zdscal (1.0 / l_xnorm.item)
									update_class_fields -- Current might have been changed; synchronize
									l_scale.item := l_scale.item / (l_xnorm.item)
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							if l_scale.item /= 1.0 then
								l_wv___2.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale.item)
								update_class_fields -- Current might have been changed; synchronize
								l_wv___2.item := ((a_w.item.work2).item.gsl_vector_subvector ((0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale.item)
								update_class_fields -- Current might have been changed; synchronize
							end
							(a_w.item.work).item.gsl_vector_set (l_lu___0 - {NATURAL_32} 1, l_x1___0.dat[0])
							update_class_fields; -- Current might have been changed; synchronize
							(a_w.item.work).item.gsl_vector_set (l_lu___0, l_x2___1.dat[0])
							update_class_fields; -- Current might have been changed; synchronize
							(a_w.item.work2).item.gsl_vector_set (l_lu___0 - {NATURAL_32} 1, l_x1___0.dat[1])
							update_class_fields; -- Current might have been changed; synchronize
							(a_w.item.work2).item.gsl_vector_set (l_lu___0, l_x2___1.dat[1])
							update_class_fields -- Current might have been changed; synchronize
							if l_lu___0 > {NATURAL_32} 1 then
								l_v1___2.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_lu___0 - {NATURAL_32} 1, (0).to_natural_32, l_lu___0 - {NATURAL_32} 1)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_v4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_lu___0, (0).to_natural_32, l_lu___0 - {NATURAL_32} 1)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_v2___2.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_lu___0 - {NATURAL_32} 1)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_v3.item := ((a_w.item.work2).item.gsl_vector_subvector ((0).to_natural_32, l_lu___0 - {NATURAL_32} 1)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-l_x1___0.dat[0], create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v4.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-l_x2___1.dat[0], create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-l_x1___0.dat[1], create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v4.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-l_x2___1.dat[1], create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							end
							l_l___0 := l_l___0 - 1
						end
						l_l___0 := l_l___0 - 1
					end
					l_y.item := (a_Z.item.gsl_matrix_column (l_iu - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_y2.item := (a_Z.item.gsl_matrix_column (l_iu)).twin
					update_class_fields -- Current might have been changed; synchronize
					if l_iu > {NATURAL_32} 1 then
						l_Zv.item := (a_Z.item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_iu - {NATURAL_32} 1)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_x___2.item := ((a_w.item.work).item.gsl_vector_subvector ((0).to_natural_32, l_iu - {NATURAL_32} 1)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___48 := (a_w.item.work).item.gsl_vector_get (l_iu - {NATURAL_32} 1)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_Zv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_tmp___48, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_x___2.item := ((a_w.item.work2).item.gsl_vector_subvector ((0).to_natural_32, l_iu - {NATURAL_32} 1)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___49 := (a_w.item.work2).item.gsl_vector_get (l_iu)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_Zv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_tmp___49, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					else
						l_tmp___50 := (a_w.item.work).item.gsl_vector_get (l_iu - {NATURAL_32} 1)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_tmp___50)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___51 := (a_w.item.work2).item.gsl_vector_get (l_iu)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_tmp___51)
						update_class_fields -- Current might have been changed; synchronize
					end
					l_ecol.item := (a_evec.item.gsl_matrix_complex_column (l_iu - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_ecol2.item := (a_evec.item.gsl_matrix_complex_column (l_iu)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_ev.item := (gsl_vector_complex_imag (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					update_class_fields -- Current might have been changed; synchronize
					l_ev2.item := (gsl_vector_complex_imag (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol2.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					update_class_fields -- Current might have been changed; synchronize
					l_scale.item := 0.0
					update_class_fields -- Current might have been changed; synchronize
					l_ii := (0).to_natural_32
					from
					until
						not (l_ii < l_N)
					loop
						l_tmp___52 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_ii)
						update_class_fields -- Current might have been changed; synchronize
						l_a___6 := l_tmp___52
						l_tmp___59 := fabs (l_a___6)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___60 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_ii)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___61 := fabs (l_tmp___60)
						update_class_fields -- Current might have been changed; synchronize
						if l_scale.item > l_tmp___59 + l_tmp___61 then
							l_scale.item := l_scale.item
							update_class_fields -- Current might have been changed; synchronize
						else
							l_tmp___56 := fabs (l_a___6)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___57 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_ii)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___58 := fabs (l_tmp___57)
							update_class_fields -- Current might have been changed; synchronize
							l_scale.item := l_tmp___56 + l_tmp___58
							update_class_fields -- Current might have been changed; synchronize
						end
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_ii, l_a___6)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_ii, -l_a___6)
						update_class_fields -- Current might have been changed; synchronize
						l_ii := l_ii + 1
					end
					l_ev.item := (gsl_vector_complex_real (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					update_class_fields -- Current might have been changed; synchronize
					l_ev2.item := (gsl_vector_complex_real (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol2.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					update_class_fields -- Current might have been changed; synchronize
					l_ii := (0).to_natural_32
					from
					until
						not (l_ii < l_N)
					loop
						l_tmp___62 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_ii)
						update_class_fields -- Current might have been changed; synchronize
						l_a___7 := l_tmp___62;
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_ii, l_a___7)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_ii, l_a___7)
						update_class_fields -- Current might have been changed; synchronize
						l_ii := l_ii + 1
					end
					if l_scale.item /= 0.0 then
						l_scale.item := 1.0 / l_scale.item
						update_class_fields -- Current might have been changed; synchronize
					end
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_blas_zdscal (l_scale.item)
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol2.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_blas_zdscal (l_scale.item)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i - 1
				end
				l_i := l_i - 1
			end
		end

	gsl_eigen_symmv (a_eval: CE_POINTER [PD_GSL_VECTOR]; a_evec: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_SYMMV_WORKSPACE]): INTEGER_32
		local
			l_d: CE_POINTER [REAL_64]
			l_sd: CE_POINTER [REAL_64]
			l_N: NATURAL_32
			l_a___5: NATURAL_32
			l_b___0: NATURAL_32
			l_A00: REAL_64
			l_tmp: REAL_64
			l_d_vec: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_sd_vec: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: PD_GSL_VECTOR_VIEW
			l_tau: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___2: PD_GSL_VECTOR_VIEW
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_tmp___5: INTEGER_32
			l_tmp___6: INTEGER_32
			l_tmp___7: INTEGER_32
			l_i: NATURAL_32
			l_n_block: NATURAL_32
			l_d_block: CE_POINTER [REAL_64]
			l_sd_block: CE_POINTER [REAL_64]
			l_gc: CE_POINTER [REAL_64]
			l_gs: CE_POINTER [REAL_64]
			l_c___1: REAL_64
			l_s: REAL_64
			l_k: NATURAL_32
			l_qki: REAL_64
			l_tmp___8: REAL_64
			l_qkj: REAL_64
			l_tmp___9: REAL_64
			l_d_vec___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___10: PD_GSL_VECTOR_VIEW
			l_return, l_break, l_continue: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_d_vec.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_sd_vec.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_tau.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_d_vec___0.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if size1 = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("symmv.c"), 105, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if a_eval.item.size = size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("symmv.c"), 109, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if a_evec.item.size1 = size1 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if a_evec.item.size2 = size1 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvector matrix must match matrix size"), ce_string ("symmv.c"), 113, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						l_d := a_w.item.d
						l_sd := a_w.item.sd
						l_N := size1
						if l_N = {NATURAL_32} 1 then
							l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_A00 := l_tmp
							a_eval.item.gsl_vector_set ((0).to_natural_32, l_A00)
							update_class_fields -- Current might have been changed; synchronize
							a_evec.item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, 1.0)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						end
						if not l_return then
							l_tmp___0 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_d, l_N)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_d_vec.item := l_tmp___0.twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___1 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_sd, l_N - {NATURAL_32} 1)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_sd_vec.item := l_tmp___1.twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_sd, l_N - {NATURAL_32} 1)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_tau.item := l_tmp___2.twin
							update_class_fields; -- Current might have been changed; synchronize
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_symmtd_decomp (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_tau.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
							update_class_fields; -- Current might have been changed; synchronize
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_symmtd_unpack ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_tau.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), a_evec, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_d_vec.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_sd_vec.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							chop_small_elements___0 (l_N, l_d, l_sd)
							update_class_fields -- Current might have been changed; synchronize
							l_b___0 := l_N - {NATURAL_32} 1
							from
							until
								not (not (l_return) and then (l_b___0 > {NATURAL_32} 0))
							loop
								l_continue := False
								if l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32] = 0.0 then
									l_b___0 := l_b___0 - 1
									l_continue := True
								else
									if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
										l_tmp___3 := isnanf ((l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32]).truncated_to_real)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___7 := l_tmp___3
									else
										if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
											l_tmp___4 := isnan (l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32])
											update_class_fields -- Current might have been changed; synchronize
											l_tmp___6 := l_tmp___4
										else
											l_tmp___5 := isnanl (real_96_from_real_64 (l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32]))
											update_class_fields -- Current might have been changed; synchronize
											l_tmp___6 := l_tmp___5
										end
										l_tmp___7 := l_tmp___6
									end
									if l_tmp___7.to_boolean then
										l_b___0 := l_b___0 - 1
										l_continue := True
									end
								end
								if not (l_return or l_break or l_continue) then
									l_a___5 := l_b___0 - {NATURAL_32} 1
									from
									until
										not (not (l_break) and then (l_a___5 > {NATURAL_32} 0))
									loop
										l_break := False
										if l_sd[(l_a___5 - {NATURAL_32} 1).to_integer_32] = 0.0 then
											l_break := True
										end
										if not (l_return or l_break) then
											l_a___5 := l_a___5 - 1
										end
									end
									l_break := False
									l_n_block := (l_b___0 - l_a___5) + {NATURAL_32} 1
									l_d_block := l_d + l_a___5.to_integer_32
									l_sd_block := l_sd + l_a___5.to_integer_32
									l_gc := a_w.item.gc
									l_gs := a_w.item.gs
									qrstep___0 (l_n_block, l_d_block, l_sd_block, l_gc, l_gs)
									update_class_fields -- Current might have been changed; synchronize
									l_i := (0).to_natural_32
									from
									until
										not (not (l_return) and then (l_i < l_n_block - {NATURAL_32} 1))
									loop
										l_c___1 := l_gc[l_i.to_integer_32]
										l_s := l_gs[l_i.to_integer_32]
										l_k := (0).to_natural_32
										from
										until
											not (l_k < l_N)
										loop
											l_tmp___8 := a_evec.item.gsl_matrix_get (l_k, l_a___5 + l_i)
											update_class_fields -- Current might have been changed; synchronize
											l_qki := l_tmp___8
											l_tmp___9 := a_evec.item.gsl_matrix_get (l_k, (l_a___5 + l_i) + {NATURAL_32} 1)
											update_class_fields -- Current might have been changed; synchronize
											l_qkj := l_tmp___9
											a_evec.item.gsl_matrix_set (l_k, l_a___5 + l_i, l_qki * l_c___1 - l_qkj * l_s)
											update_class_fields -- Current might have been changed; synchronize
											a_evec.item.gsl_matrix_set (l_k, (l_a___5 + l_i) + {NATURAL_32} 1, l_qki * l_s + l_qkj * l_c___1)
											update_class_fields -- Current might have been changed; synchronize
											l_k := l_k + 1
										end
										l_i := l_i + 1
									end
									chop_small_elements___0 (l_N, l_d, l_sd)
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							l_continue := False
							l_tmp___10 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_d, l_N)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_d_vec___0.item := l_tmp___10.twin
							update_class_fields -- Current might have been changed; synchronize
							a_eval.item.gsl_vector_memcpy ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_d_vec___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						end
				when 13 then -- if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	symschur2 (a_p, a_q___3: NATURAL_32; a_c___1, a_s: CE_POINTER [REAL_64]): REAL_64
		require
			not a_c___1.is_void
			not a_s.is_void
		local
			l_Apq: REAL_64
			l_tmp: REAL_64
			l_App: REAL_64
			l_tmp___0: REAL_64
			l_Aqq: REAL_64
			l_tmp___1: REAL_64
			l_tau: REAL_64
			l_t: REAL_64
			l_c1___2: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (a_p, a_q___3)
			update_class_fields -- Current might have been changed; synchronize
			l_Apq := l_tmp
			if l_Apq /= 0.0 then
				l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (a_p, a_p)
				update_class_fields -- Current might have been changed; synchronize
				l_App := l_tmp___0
				l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (a_q___3, a_q___3)
				update_class_fields -- Current might have been changed; synchronize
				l_Aqq := l_tmp___1
				l_tau := (l_Aqq - l_App) / (2.0 * l_Apq)
				if l_tau >= 0.0 then
					l_tmp___2 := hypot (1.0, l_tau)
					update_class_fields -- Current might have been changed; synchronize
					l_t := 1.0 / (l_tau + l_tmp___2)
				else
					l_tmp___3 := hypot (1.0, l_tau)
					update_class_fields -- Current might have been changed; synchronize
					l_t := -1.0 / (-l_tau + l_tmp___3)
				end
				l_tmp___4 := hypot (1.0, l_t)
				update_class_fields -- Current might have been changed; synchronize
				l_c1___2 := 1.0 / l_tmp___4
				a_c___1.item := l_c1___2
				update_class_fields -- Current might have been changed; synchronize
				a_s.item := l_t * l_c1___2
				update_class_fields -- Current might have been changed; synchronize
			else
				a_c___1.item := 1.0
				update_class_fields -- Current might have been changed; synchronize
				a_s.item := 0.0
				update_class_fields -- Current might have been changed; synchronize
			end
			l_tmp___5 := fabs (l_Apq)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp___5
		end

	apply_jacobi_L (a_p, a_q___3: NATURAL_32; a_c___1, a_s: REAL_64)
		local
			l_j: NATURAL_32
			l_N: NATURAL_32
			l_Apj: REAL_64
			l_tmp: REAL_64
			l_Aqj: REAL_64
			l_tmp___0: REAL_64
		do
			l_N := size2
			l_j := (0).to_natural_32
			from
			until
				not (l_j < l_N)
			loop
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (a_p, l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_Apj := l_tmp
				l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (a_q___3, l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_Aqj := l_tmp___0;
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (a_p, l_j, l_Apj * a_c___1 - l_Aqj * a_s)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (a_q___3, l_j, l_Apj * a_s + l_Aqj * a_c___1)
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_j + 1
			end
		end

	apply_jacobi_R (a_p, a_q___3: NATURAL_32; a_c___1, a_s: REAL_64)
		local
			l_i: NATURAL_32
			l_M: NATURAL_32
			l_Aip: REAL_64
			l_tmp: REAL_64
			l_Aiq: REAL_64
			l_tmp___0: REAL_64
		do
			l_M := size1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_M)
			loop
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, a_p)
				update_class_fields -- Current might have been changed; synchronize
				l_Aip := l_tmp
				l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, a_q___3)
				update_class_fields -- Current might have been changed; synchronize
				l_Aiq := l_tmp___0;
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, a_p, l_Aip * a_c___1 - l_Aiq * a_s)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, a_q___3, l_Aip * a_s + l_Aiq * a_c___1)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	norm: REAL_64
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_sum: REAL_64
			l_scale: REAL_64
			l_ssq: REAL_64
			l_Aij: REAL_64
			l_tmp: REAL_64
			l_ax: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_sum := 0.0
			l_scale := 0.0
			l_ssq := 1.0
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N)
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_Aij := l_tmp
					if l_Aij /= 0.0 then
						l_tmp___0 := fabs (l_Aij)
						update_class_fields -- Current might have been changed; synchronize
						l_ax := l_tmp___0
						if l_scale < l_ax then
							l_ssq := 1.0 + (l_ssq * (l_scale / l_ax)) * (l_scale / l_ax)
							l_scale := l_ax
						else
							l_ssq := l_ssq + ((l_ax / l_scale) * (l_ax / l_scale))
						end
					end
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			l_tmp___1 := sqrt (l_ssq)
			update_class_fields -- Current might have been changed; synchronize
			l_sum := l_scale * l_tmp___1
			Result := l_sum
		end

	gsl_eigen_jacobi (a_eval: CE_POINTER [PD_GSL_VECTOR]; a_evec: CE_POINTER [PD_GSL_MATRIX]; a_max_rot: NATURAL_32; a_nrot: CE_POINTER [NATURAL_32]): INTEGER_32
		local
			l_i: NATURAL_32
			l_p: NATURAL_32
			l_q___3: NATURAL_32
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_red: REAL_64
			l_redsum: REAL_64
			l_nrm: REAL_64
			l_tmp: REAL_64
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_ep: REAL_64
			l_tmp___0: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					l_redsum := 0.0
					if l_M = l_N then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenproblem requires square matrix"), ce_string ("jacobi.c"), 146, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if l_M = a_evec.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if l_M = a_evec.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvector matrix must match input matrix"), ce_string ("jacobi.c"), 150, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						if l_M /= a_eval.item.size then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match input matrix"), ce_string ("jacobi.c"), 154, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
				when 10 then -- if done | if done
					if not l_return then
						a_eval.item.gsl_vector_set_zero
						update_class_fields -- Current might have been changed; synchronize
						a_evec.item.gsl_matrix_set_identity
						update_class_fields -- Current might have been changed; synchronize
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return or l_break) and then (l_i < a_max_rot))
						loop
							l_break := False
							l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.norm
							update_class_fields -- Current might have been changed; synchronize
							l_nrm := l_tmp
							if l_nrm = 0.0 then
								l_break := True
							end
							if not (l_return or l_break) then
								l_p := (0).to_natural_32
								from
								until
									not (not (l_return) and then (l_p < l_N))
								loop
									l_q___3 := l_p + {NATURAL_32} 1
									from
									until
										not (l_q___3 < l_N)
									loop
										l_red := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.symschur2 (l_p, l_q___3, l_c___1, l_s)
										update_class_fields -- Current might have been changed; synchronize
										l_redsum := l_redsum + l_red;
										((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.apply_jacobi_L (l_p, l_q___3, l_c___1.item, l_s.item)
										update_class_fields; -- Current might have been changed; synchronize
										((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.apply_jacobi_R (l_p, l_q___3, l_c___1.item, l_s.item)
										update_class_fields -- Current might have been changed; synchronize
										a_evec.item.apply_jacobi_R (l_p, l_q___3, l_c___1.item, l_s.item)
										update_class_fields -- Current might have been changed; synchronize
										l_q___3 := l_q___3 + 1
									end
									l_p := l_p + 1
								end
								l_i := l_i + 1
							end
						end
						l_break := False
						a_nrot.item := l_i
						update_class_fields -- Current might have been changed; synchronize
						l_p := (0).to_natural_32
						from
						until
							not (l_p < l_N)
						loop
							l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_p, l_p)
							update_class_fields -- Current might have been changed; synchronize
							l_ep := l_tmp___0
							a_eval.item.gsl_vector_set (l_p, l_ep)
							update_class_fields -- Current might have been changed; synchronize
							l_p := l_p + 1
						end
						if l_i = a_max_rot then
							Result := 11
							l_return := True
						end
					end
					if not l_return then
						Result := 0
					end
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_eigen_invert_jacobi (a_ainv: CE_POINTER [PD_GSL_MATRIX]; a_max_rot: NATURAL_32): INTEGER_32
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_nrot: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
			l_eval: CE_POINTER [PD_GSL_VECTOR]
			l_tmp: CE_POINTER [PD_GSL_VECTOR]
			l_evec: CE_POINTER [PD_GSL_MATRIX]
			l_tmp___0: CE_POINTER [PD_GSL_MATRIX]
			l_tmp___1: CE_POINTER [PD_GSL_MATRIX]
			l_tmp___2: CE_POINTER [PD_GSL_MATRIX]
			l_ainv_ij: REAL_64
			l_f: REAL_64
			l_tmp___3: REAL_64
			l_vik: REAL_64
			l_tmp___4: REAL_64
			l_vjk: REAL_64
			l_tmp___5: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_NATURAL_32_ARRAY} l_nrot.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if size1 = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_ainv.item.size1 = a_ainv.item.size2 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("jacobi method requires square matrix"), ce_string ("jacobi.c"), 208, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if size1 /= a_ainv.item.size2 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("inverse matrix must match input matrix"), ce_string ("jacobi.c"), 212, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
				when 7 then -- if done
					if not l_return then
						l_n := size2
						l_nrot.item := {NATURAL_32} 0
						update_class_fields -- Current might have been changed; synchronize
						l_tmp := (create {P_INIT_SOURCE_STATIC}).gsl_vector_alloc (l_n)
						update_class_fields -- Current might have been changed; synchronize
						l_eval := l_tmp
						l_tmp___0 := (create {P_INIT_SOURCE_STATIC}).gsl_matrix_alloc (l_n, l_n)
						update_class_fields -- Current might have been changed; synchronize
						l_evec := l_tmp___0
						l_tmp___2 := (create {P_INIT_SOURCE_STATIC}).gsl_matrix_alloc (l_n, l_n)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := l_tmp___2
						l_tmp___1.item.gsl_matrix_memcpy ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_status := l_tmp___1.item.gsl_eigen_jacobi (l_eval, l_evec, a_max_rot, l_nrot)
						update_class_fields -- Current might have been changed; synchronize
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_n))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (not (l_return) and then (l_j < l_n))
							loop
								l_ainv_ij := 0.0
								l_k := (0).to_natural_32
								from
								until
									not (l_k < l_n)
								loop
									l_tmp___3 := l_eval.item.gsl_vector_get (l_k)
									update_class_fields -- Current might have been changed; synchronize
									l_f := 1.0 / l_tmp___3
									l_tmp___4 := l_evec.item.gsl_matrix_get (l_i, l_k)
									update_class_fields -- Current might have been changed; synchronize
									l_vik := l_tmp___4
									l_tmp___5 := l_evec.item.gsl_matrix_get (l_j, l_k)
									update_class_fields -- Current might have been changed; synchronize
									l_vjk := l_tmp___5
									l_ainv_ij := l_ainv_ij + ((l_vik * l_vjk) * l_f)
									l_k := l_k + 1
								end
								a_ainv.item.gsl_matrix_set (l_i, l_j, l_ainv_ij)
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						(create {P_INIT_SOURCE_STATIC}).gsl_vector_free (l_eval)
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_INIT_SOURCE_STATIC}).gsl_matrix_free (l_evec)
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_INIT_SOURCE_STATIC}).gsl_matrix_free (l_tmp___1)
						update_class_fields -- Current might have been changed; synchronize
						if l_status.to_boolean then
							Result := l_status
						else
							Result := 0
						end
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_schur_gen_eigvals (a_B: CE_POINTER [PD_GSL_MATRIX]; a_wr1, a_wr2, a_wi, a_scale1, a_scale2: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_B.is_void
			not a_wr1.is_void
			not a_wr2.is_void
			not a_wi.is_void
			not a_scale1.is_void
		local
			l_safemin: REAL_64
			l_safemax: REAL_64
			l_rtmin: REAL_64
			l_tmp: REAL_64
			l_rtmax: REAL_64
			l_anorm: REAL_64
			l_bnorm: REAL_64
			l_ascale: REAL_64
			l_bscale: REAL_64
			l_bsize: REAL_64
			l_s1: REAL_64
			l_s2: REAL_64
			l_A11: REAL_64
			l_A12: REAL_64
			l_A21: REAL_64
			l_A22: REAL_64
			l_B11: REAL_64
			l_B12: REAL_64
			l_B22: REAL_64
			l_binv11: REAL_64
			l_binv22: REAL_64
			l_bmin: REAL_64
			l_as11: REAL_64
			l_as12: REAL_64
			l_as22: REAL_64
			l_abi22: REAL_64
			l_pp: REAL_64
			l_qq: REAL_64
			l_shift___0: REAL_64
			l_ss: REAL_64
			l_discr: REAL_64
			l_r___3: REAL_64
			l_tmp___33: REAL_64
			l_tmp___34: REAL_64
			l_tmp___35: REAL_64
			l_tmp___36: REAL_64
			l_tmp___37: REAL_64
			l_tmp___38: REAL_64
			l_tmp___39: REAL_64
			l_tmp___40: REAL_64
			l_tmp___41: REAL_64
			l_tmp___42: REAL_64
			l_tmp___43: REAL_64
			l_tmp___44: REAL_64
			l_tmp___45: REAL_64
			l_tmp___46: REAL_64
			l_tmp___47: REAL_64
			l_tmp___48: REAL_64
			l_tmp___49: REAL_64
			l_tmp___58: REAL_64
			l_tmp___59: REAL_64
			l_tmp___60: REAL_64
			l_tmp___61: REAL_64
			l_tmp___62: REAL_64
			l_tmp___63: REAL_64
			l_tmp___64: REAL_64
			l_tmp___65: REAL_64
			l_tmp___66: REAL_64
			l_tmp___67: REAL_64
			l_tmp___68: REAL_64
			l_tmp___69: REAL_64
			l_tmp___70: REAL_64
			l_tmp___71: REAL_64
			l_tmp___72: REAL_64
			l_tmp___73: REAL_64
			l_tmp___74: REAL_64
			l_tmp___75: REAL_64
			l_tmp___76: REAL_64
			l_tmp___77: REAL_64
			l_tmp___78: REAL_64
			l_tmp___96: REAL_64
			l_tmp___102: REAL_64
			l_tmp___104: REAL_64
			l_tmp___105: REAL_64
			l_tmp___106: REAL_64
			l_tmp___107: REAL_64
			l_tmp___108: REAL_64
			l_tmp___110: REAL_64
			l_tmp___111: REAL_64
			l_tmp___112: REAL_64
			l_tmp___113: REAL_64
			l_tmp___114: REAL_64
			l_tmp___120: REAL_64
			l_tmp___122: REAL_64
			l_tmp___123: REAL_64
			l_tmp___124: REAL_64
			l_tmp___125: REAL_64
			l_tmp___126: REAL_64
			l_tmp___128: REAL_64
			l_tmp___129: REAL_64
			l_tmp___130: REAL_64
			l_tmp___131: INTEGER_32
			l_tmp___132: REAL_64
			l_tmp___133: INTEGER_32
			l_tmp___134: REAL_64
			l_tmp___143: REAL_64
			l_tmp___146: REAL_64
			l_tmp___147: REAL_64
			l_tmp___148: REAL_64
			l_tmp___149: REAL_64
			l_tmp___150: REAL_64
			l_tmp___151: REAL_64
			l_tmp___154: REAL_64
			l_tmp___155: REAL_64
			l_tmp___156: REAL_64
			l_tmp___157: REAL_64
			l_tmp___158: REAL_64
			l_tmp___161: REAL_64
			l_tmp___162: REAL_64
			l_tmp___163: REAL_64
			l_tmp___164: REAL_64
			l_tmp___165: REAL_64
			l_tmp___166: REAL_64
			l_tmp___167: REAL_64
			l_tmp___168: REAL_64
			l_tmp___169: REAL_64
			l_tmp___170: REAL_64
			l_tmp___171: REAL_64
			l_tmp___172: REAL_64
			l_tmp___173: REAL_64
			l_sum: REAL_64
			l_tmp___174: INTEGER_32
			l_diff: REAL_64
			l_tmp___175: INTEGER_32
			l_wbig: REAL_64
			l_wsmall: REAL_64
			l_wdet: REAL_64
			l_tmp___176: REAL_64
			l_tmp___178: REAL_64
			l_tmp___179: REAL_64
			l_tmp___180: REAL_64
			l_fuzzy1: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_wabs: REAL_64
			l_wsize: REAL_64
			l_wscale: REAL_64
			l_tmp___181: REAL_64
			l_tmp___182: REAL_64
			l_tmp___183: REAL_64
			l_tmp___184: REAL_64
			l_tmp___243: REAL_64
			l_tmp___244: REAL_64
			l_tmp___245: REAL_64
			l_tmp___246: REAL_64
			l_tmp___248: REAL_64
			l_tmp___249: REAL_64
			l_tmp___250: REAL_64
			l_tmp___251: REAL_64
			l_tmp___257: REAL_64
			l_tmp___258: REAL_64
			l_tmp___259: REAL_64
			l_tmp___260: REAL_64
			l_tmp___262: REAL_64
			l_tmp___263: REAL_64
			l_tmp___264: REAL_64
			l_tmp___283: REAL_64
			l_tmp___284: REAL_64
			l_tmp___285: REAL_64
			l_tmp___286: REAL_64
			l_tmp___288: REAL_64
			l_tmp___289: REAL_64
			l_tmp___290: REAL_64
			l_tmp___291: REAL_64
			l_tmp___297: REAL_64
			l_tmp___298: REAL_64
			l_tmp___299: REAL_64
			l_tmp___300: REAL_64
			l_tmp___302: REAL_64
			l_tmp___303: REAL_64
			l_tmp___304: REAL_64
			l_tmp___305: REAL_64
			l_tmp___306: REAL_64
			l_tmp___307: REAL_64
			l_tmp___308: REAL_64
			l_tmp___496: REAL_64
			l_tmp___502: REAL_64
			l_tmp___503: REAL_64
			l_tmp___504: REAL_64
			l_tmp___505: REAL_64
			l_tmp___507: REAL_64
			l_tmp___508: REAL_64
			l_tmp___509: REAL_64
			l_tmp___510: REAL_64
			l_tmp___511: REAL_64
			l_tmp___517: REAL_64
			l_tmp___518: REAL_64
			l_tmp___519: REAL_64
			l_tmp___520: REAL_64
			l_tmp___522: REAL_64
			l_tmp___523: REAL_64
			l_tmp___524: REAL_64
			l_tmp___525: REAL_64
			l_tmp___540: REAL_64
			l_tmp___546: REAL_64
			l_tmp___547: REAL_64
			l_tmp___548: REAL_64
			l_tmp___549: REAL_64
			l_tmp___551: REAL_64
			l_tmp___552: REAL_64
			l_tmp___553: REAL_64
			l_tmp___554: REAL_64
			l_tmp___555: REAL_64
			l_tmp___561: REAL_64
			l_tmp___562: REAL_64
			l_tmp___563: REAL_64
			l_tmp___564: REAL_64
			l_tmp___566: REAL_64
			l_tmp___567: REAL_64
			l_tmp___568: REAL_64
			l_tmp___626: REAL_64
			l_tmp___632: REAL_64
			l_tmp___633: REAL_64
			l_tmp___634: REAL_64
			l_tmp___635: REAL_64
			l_tmp___637: REAL_64
			l_tmp___638: REAL_64
			l_tmp___639: REAL_64
			l_tmp___640: REAL_64
			l_tmp___641: REAL_64
			l_tmp___647: REAL_64
			l_tmp___648: REAL_64
			l_tmp___649: REAL_64
			l_tmp___650: REAL_64
			l_tmp___652: REAL_64
			l_tmp___653: REAL_64
			l_tmp___654: REAL_64
			l_tmp___655: REAL_64
			l_tmp___670: REAL_64
			l_tmp___676: REAL_64
			l_tmp___677: REAL_64
			l_tmp___678: REAL_64
			l_tmp___679: REAL_64
			l_tmp___681: REAL_64
			l_tmp___682: REAL_64
			l_tmp___683: REAL_64
			l_tmp___684: REAL_64
			l_tmp___685: REAL_64
			l_tmp___691: REAL_64
			l_tmp___692: REAL_64
			l_tmp___693: REAL_64
			l_tmp___694: REAL_64
			l_tmp___696: REAL_64
			l_tmp___697: REAL_64
			l_tmp___698: REAL_64
			l_tmp___699: REAL_64
			l_tmp___700: REAL_64
			l_tmp___701: REAL_64
			l_tmp___702: REAL_64
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_safemin := 2.2250738585072014e-308 * 1.0e2
					l_safemax := 1.0 / l_safemin
					l_tmp := sqrt (l_safemin)
					update_class_fields -- Current might have been changed; synchronize
					l_rtmin := l_tmp
					l_rtmax := 1.0 / l_rtmin
					l_tmp___67 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___68 := fabs (l_tmp___67)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___69 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___70 := fabs (l_tmp___69)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___71 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___72 := fabs (l_tmp___71)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___73 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___74 := fabs (l_tmp___73)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___68 + l_tmp___70 > l_tmp___72 + l_tmp___74 then
						l_tmp___58 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___59 := fabs (l_tmp___58)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___60 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___61 := fabs (l_tmp___60)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___66 := l_tmp___59 + l_tmp___61
					else
						l_tmp___62 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___63 := fabs (l_tmp___62)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___64 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___65 := fabs (l_tmp___64)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___66 := l_tmp___63 + l_tmp___65
					end
					if l_tmp___66 > l_safemin then
						l_tmp___42 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___43 := fabs (l_tmp___42)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___44 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___45 := fabs (l_tmp___44)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___46 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___47 := fabs (l_tmp___46)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___48 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___49 := fabs (l_tmp___48)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___43 + l_tmp___45 > l_tmp___47 + l_tmp___49 then
							l_tmp___33 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___34 := fabs (l_tmp___33)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___35 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___36 := fabs (l_tmp___35)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___41 := l_tmp___34 + l_tmp___36
						else
							l_tmp___37 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___38 := fabs (l_tmp___37)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___39 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___40 := fabs (l_tmp___39)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___41 := l_tmp___38 + l_tmp___40
						end
						l_anorm := l_tmp___41
					else
						l_anorm := l_safemin
					end
					l_ascale := 1.0 / l_anorm
					l_tmp___75 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_A11 := l_ascale * l_tmp___75
					l_tmp___76 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_A12 := l_ascale * l_tmp___76
					l_tmp___77 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_A21 := l_ascale * l_tmp___77
					l_tmp___78 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_A22 := l_ascale * l_tmp___78
					l_B11 := a_B.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_B12 := a_B.item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_B22 := a_B.item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___114 := fabs (l_B11)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___126 := fabs (l_B12)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___130 := fabs (l_B22)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___130 > l_rtmin then
						l_tmp___128 := fabs (l_B22)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___129 := l_tmp___128
					else
						l_tmp___129 := l_rtmin
					end
					if l_tmp___126 > l_tmp___129 then
						l_tmp___120 := fabs (l_B12)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___125 := l_tmp___120
					else
						l_tmp___124 := fabs (l_B22)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___124 > l_rtmin then
							l_tmp___122 := fabs (l_B22)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___123 := l_tmp___122
						else
							l_tmp___123 := l_rtmin
						end
						l_tmp___125 := l_tmp___123
					end
					if l_tmp___114 > l_tmp___125 then
						l_tmp___96 := fabs (l_B11)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___113 := l_tmp___96
					else
						l_tmp___108 := fabs (l_B12)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___112 := fabs (l_B22)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___112 > l_rtmin then
							l_tmp___110 := fabs (l_B22)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___111 := l_tmp___110
						else
							l_tmp___111 := l_rtmin
						end
						if l_tmp___108 > l_tmp___111 then
							l_tmp___102 := fabs (l_B12)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___107 := l_tmp___102
						else
							l_tmp___106 := fabs (l_B22)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___106 > l_rtmin then
								l_tmp___104 := fabs (l_B22)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___105 := l_tmp___104
							else
								l_tmp___105 := l_rtmin
							end
							l_tmp___107 := l_tmp___105
						end
						l_tmp___113 := l_tmp___107
					end
					l_bmin := l_rtmin * l_tmp___113
					l_tmp___132 := fabs (l_B11)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___132 < l_bmin then
						if l_B11 >= 0.0 then
							l_tmp___131 := 1
						else
							l_tmp___131 := -1
						end
						l_B11 := l_tmp___131.to_double * l_bmin
					end
					l_tmp___134 := fabs (l_B22)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___134 < l_bmin then
						if l_B22 >= 0.0 then
							l_tmp___133 := 1
						else
							l_tmp___133 := -1
						end
						l_B22 := l_tmp___133.to_double * l_bmin
					end
					l_tmp___151 := fabs (l_B11)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___157 := fabs (l_B12)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___158 := fabs (l_B22)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___157 + l_tmp___158 > l_safemin then
						l_tmp___154 := fabs (l_B12)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___155 := fabs (l_B22)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___156 := l_tmp___154 + l_tmp___155
					else
						l_tmp___156 := l_safemin
					end
					if l_tmp___151 > l_tmp___156 then
						l_tmp___143 := fabs (l_B11)
						update_class_fields -- Current might have been changed; synchronize
						l_bnorm := l_tmp___143
					else
						l_tmp___149 := fabs (l_B12)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___150 := fabs (l_B22)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___149 + l_tmp___150 > l_safemin then
							l_tmp___146 := fabs (l_B12)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___147 := fabs (l_B22)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___148 := l_tmp___146 + l_tmp___147
						else
							l_tmp___148 := l_safemin
						end
						l_bnorm := l_tmp___148
					end
					l_tmp___163 := fabs (l_B11)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___164 := fabs (l_B22)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___163 > l_tmp___164 then
						l_tmp___161 := fabs (l_B11)
						update_class_fields -- Current might have been changed; synchronize
						l_bsize := l_tmp___161
					else
						l_tmp___162 := fabs (l_B22)
						update_class_fields -- Current might have been changed; synchronize
						l_bsize := l_tmp___162
					end
					l_bscale := 1.0 / l_bsize
					l_B11 := l_B11 * l_bscale
					l_B12 := l_B12 * l_bscale
					l_B22 := l_B22 * l_bscale
					l_binv11 := 1.0 / l_B11
					l_binv22 := 1.0 / l_B22
					l_s1 := l_A11 * l_binv11
					l_s2 := l_A22 * l_binv22
					l_tmp___165 := fabs (l_s1)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___166 := fabs (l_s2)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___165 <= l_tmp___166 then
						l_as12 := l_A12 - l_s1 * l_B12
						l_as22 := l_A22 - l_s1 * l_B22
						l_ss := l_A21 * (l_binv11 * l_binv22)
						l_abi22 := l_as22 * l_binv22 - l_ss * l_B12
						l_pp := 0.5 * l_abi22
						l_shift___0 := l_s1
					else
						l_as12 := l_A12 - l_s2 * l_B12
						l_as11 := l_A11 - l_s2 * l_B11
						l_ss := l_A21 * (l_binv11 * l_binv22)
						l_abi22 := -l_ss * l_B12
						l_pp := 0.5 * (l_as11 * l_binv11 + l_abi22)
						l_shift___0 := l_s2
					end
					l_qq := l_ss * l_as12
					l_tmp___173 := fabs (l_pp * l_rtmin)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___173 >= 1.0 then
						l_discr := (l_rtmin * l_pp) * (l_rtmin * l_pp) + l_qq * l_safemin
						l_tmp___167 := fabs (l_discr)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___168 := sqrt (l_tmp___167)
						update_class_fields -- Current might have been changed; synchronize
						l_r___3 := l_tmp___168 * l_rtmax
					else
						l_tmp___172 := fabs (l_qq)
						update_class_fields -- Current might have been changed; synchronize
						if l_pp * l_pp + l_tmp___172 <= l_safemin then
							l_discr := (l_rtmax * l_pp) * (l_rtmax * l_pp) + l_qq * l_safemax
							l_tmp___169 := fabs (l_discr)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___170 := sqrt (l_tmp___169)
							update_class_fields -- Current might have been changed; synchronize
							l_r___3 := l_tmp___170 * l_rtmin
						else
							l_discr := l_pp * l_pp + l_qq
							l_tmp___171 := fabs (l_discr)
							update_class_fields -- Current might have been changed; synchronize
							l_r___3 := sqrt (l_tmp___171)
							update_class_fields -- Current might have been changed; synchronize
						end
					end
					if not (l_discr >= 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if l_r___3 /= 0.0 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						if l_pp >= 0.0 then
							l_tmp___174 := 1
						else
							l_tmp___174 := -1
						end
						l_sum := l_pp + l_tmp___174.to_double * l_r___3
						if l_pp >= 0.0 then
							l_tmp___175 := 1
						else
							l_tmp___175 := -1
						end
						l_diff := l_pp - l_tmp___175.to_double * l_r___3
						l_wbig := l_shift___0 + l_sum
						l_wsmall := l_shift___0 + l_diff
						l_tmp___176 := fabs (l_wbig)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___180 := fabs (l_wsmall)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___180 > l_safemin then
							l_tmp___178 := fabs (l_wsmall)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___179 := l_tmp___178
						else
							l_tmp___179 := l_safemin
						end
						if 0.5 * l_tmp___176 > l_tmp___179 then
							l_wdet := (l_A11 * l_A22 - l_A12 * l_A21) * (l_binv11 * l_binv22)
							l_wsmall := l_wdet / l_wbig
						end
						if l_pp > l_abi22 then
							if l_wbig < l_wsmall then
								a_wr1.item := l_wbig
								update_class_fields -- Current might have been changed; synchronize
							else
								a_wr1.item := l_wsmall
								update_class_fields -- Current might have been changed; synchronize
							end
							if l_wbig > l_wsmall then
								a_wr2.item := l_wbig
								update_class_fields -- Current might have been changed; synchronize
							else
								a_wr2.item := l_wsmall
								update_class_fields -- Current might have been changed; synchronize
							end
						else
							if l_wbig > l_wsmall then
								a_wr1.item := l_wbig
								update_class_fields -- Current might have been changed; synchronize
							else
								a_wr1.item := l_wsmall
								update_class_fields -- Current might have been changed; synchronize
							end
							if l_wbig < l_wsmall then
								a_wr2.item := l_wbig
								update_class_fields -- Current might have been changed; synchronize
							else
								a_wr2.item := l_wsmall
								update_class_fields -- Current might have been changed; synchronize
							end
						end
						a_wi.item := 0.0
						update_class_fields -- Current might have been changed; synchronize
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						a_wr1.item := l_shift___0 + l_pp
						update_class_fields -- Current might have been changed; synchronize
						a_wr2.item := a_wr1.item
						update_class_fields -- Current might have been changed; synchronize
						a_wi.item := l_r___3
						update_class_fields -- Current might have been changed; synchronize
				when 7 then -- if done
					l_fuzzy1 := 1.0 + 1.0e-5
					if 1.0 > l_ascale then
						l_tmp___181 := 1.0
					else
						l_tmp___181 := l_ascale
					end
					l_c1___2 := l_bsize * (l_safemin * l_tmp___181)
					if 1.0 > l_bnorm then
						l_tmp___182 := 1.0
					else
						l_tmp___182 := l_bnorm
					end
					l_c2___0 := l_safemin * l_tmp___182
					l_c3___3 := l_bsize * l_safemin
					if l_ascale <= 1.0 then
						if l_bsize <= 1.0 then
							if 1.0 < (l_ascale / l_safemin) * l_bsize then
								l_c4___2 := 1.0
							else
								l_c4___2 := (l_ascale / l_safemin) * l_bsize
							end
						else
							l_c4___2 := 1.0
						end
					else
						l_c4___2 := 1.0
					end
					if not (l_ascale <= 1.0) then
						l_pos := -1 + 10 -- skip if block
					end
				when 8 then -- if:
						l_pos := -1 + 11 -- goto _L___0
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 14
				when 10 then -- else:
						if not (l_bsize <= 1.0) then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if: | _L___0:
						if 1.0 < l_ascale * l_bsize then
							l_c5___1 := 1.0
						else
							l_c5___1 := l_ascale * l_bsize
						end
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 14
				when 13 then -- else:
						l_c5___1 := 1.0
				when 14 then -- if done
					l_tmp___183 := fabs (a_wr1.item)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___184 := fabs (a_wi.item)
					update_class_fields -- Current might have been changed; synchronize
					l_wabs := l_tmp___183 + l_tmp___184
					if l_wabs > l_c5___1 then
						l_tmp___304 := l_wabs
					else
						l_tmp___304 := l_c5___1
					end
					if l_c4___2 < 0.5 * l_tmp___304 then
						l_tmp___303 := l_c4___2
					else
						if l_wabs > l_c5___1 then
							l_tmp___302 := l_wabs
						else
							l_tmp___302 := l_c5___1
						end
						l_tmp___303 := 0.5 * l_tmp___302
					end
					if l_fuzzy1 * (l_wabs * l_c2___0 + l_c3___3) > l_tmp___303 then
						l_tmp___300 := l_fuzzy1 * (l_wabs * l_c2___0 + l_c3___3)
					else
						if l_wabs > l_c5___1 then
							l_tmp___299 := l_wabs
						else
							l_tmp___299 := l_c5___1
						end
						if l_c4___2 < 0.5 * l_tmp___299 then
							l_tmp___298 := l_c4___2
						else
							if l_wabs > l_c5___1 then
								l_tmp___297 := l_wabs
							else
								l_tmp___297 := l_c5___1
							end
							l_tmp___298 := 0.5 * l_tmp___297
						end
						l_tmp___300 := l_tmp___298
					end
					if l_c1___2 > l_tmp___300 then
						l_tmp___291 := l_c1___2
					else
						if l_wabs > l_c5___1 then
							l_tmp___290 := l_wabs
						else
							l_tmp___290 := l_c5___1
						end
						if l_c4___2 < 0.5 * l_tmp___290 then
							l_tmp___289 := l_c4___2
						else
							if l_wabs > l_c5___1 then
								l_tmp___288 := l_wabs
							else
								l_tmp___288 := l_c5___1
							end
							l_tmp___289 := 0.5 * l_tmp___288
						end
						if l_fuzzy1 * (l_wabs * l_c2___0 + l_c3___3) > l_tmp___289 then
							l_tmp___286 := l_fuzzy1 * (l_wabs * l_c2___0 + l_c3___3)
						else
							if l_wabs > l_c5___1 then
								l_tmp___285 := l_wabs
							else
								l_tmp___285 := l_c5___1
							end
							if l_c4___2 < 0.5 * l_tmp___285 then
								l_tmp___284 := l_c4___2
							else
								if l_wabs > l_c5___1 then
									l_tmp___283 := l_wabs
								else
									l_tmp___283 := l_c5___1
								end
								l_tmp___284 := 0.5 * l_tmp___283
							end
							l_tmp___286 := l_tmp___284
						end
						l_tmp___291 := l_tmp___286
					end
					if l_safemin > l_tmp___291 then
						l_wsize := l_safemin
					else
						if l_wabs > l_c5___1 then
							l_tmp___264 := l_wabs
						else
							l_tmp___264 := l_c5___1
						end
						if l_c4___2 < 0.5 * l_tmp___264 then
							l_tmp___263 := l_c4___2
						else
							if l_wabs > l_c5___1 then
								l_tmp___262 := l_wabs
							else
								l_tmp___262 := l_c5___1
							end
							l_tmp___263 := 0.5 * l_tmp___262
						end
						if l_fuzzy1 * (l_wabs * l_c2___0 + l_c3___3) > l_tmp___263 then
							l_tmp___260 := l_fuzzy1 * (l_wabs * l_c2___0 + l_c3___3)
						else
							if l_wabs > l_c5___1 then
								l_tmp___259 := l_wabs
							else
								l_tmp___259 := l_c5___1
							end
							if l_c4___2 < 0.5 * l_tmp___259 then
								l_tmp___258 := l_c4___2
							else
								if l_wabs > l_c5___1 then
									l_tmp___257 := l_wabs
								else
									l_tmp___257 := l_c5___1
								end
								l_tmp___258 := 0.5 * l_tmp___257
							end
							l_tmp___260 := l_tmp___258
						end
						if l_c1___2 > l_tmp___260 then
							l_tmp___251 := l_c1___2
						else
							if l_wabs > l_c5___1 then
								l_tmp___250 := l_wabs
							else
								l_tmp___250 := l_c5___1
							end
							if l_c4___2 < 0.5 * l_tmp___250 then
								l_tmp___249 := l_c4___2
							else
								if l_wabs > l_c5___1 then
									l_tmp___248 := l_wabs
								else
									l_tmp___248 := l_c5___1
								end
								l_tmp___249 := 0.5 * l_tmp___248
							end
							if l_fuzzy1 * (l_wabs * l_c2___0 + l_c3___3) > l_tmp___249 then
								l_tmp___246 := l_fuzzy1 * (l_wabs * l_c2___0 + l_c3___3)
							else
								if l_wabs > l_c5___1 then
									l_tmp___245 := l_wabs
								else
									l_tmp___245 := l_c5___1
								end
								if l_c4___2 < 0.5 * l_tmp___245 then
									l_tmp___244 := l_c4___2
								else
									if l_wabs > l_c5___1 then
										l_tmp___243 := l_wabs
									else
										l_tmp___243 := l_c5___1
									end
									l_tmp___244 := 0.5 * l_tmp___243
								end
								l_tmp___246 := l_tmp___244
							end
							l_tmp___251 := l_tmp___246
						end
						l_wsize := l_tmp___251
					end
					if l_wsize /= 1.0 then
						l_wscale := 1.0 / l_wsize
						if l_wsize > 1.0 then
							if l_ascale > l_bsize then
								l_tmp___305 := l_ascale
							else
								l_tmp___305 := l_bsize
							end
							if l_ascale < l_bsize then
								l_tmp___306 := l_ascale
							else
								l_tmp___306 := l_bsize
							end
							a_scale1.item := (l_tmp___305 * l_wscale) * l_tmp___306
							update_class_fields -- Current might have been changed; synchronize
						else
							if l_ascale < l_bsize then
								l_tmp___307 := l_ascale
							else
								l_tmp___307 := l_bsize
							end
							if l_ascale > l_bsize then
								l_tmp___308 := l_ascale
							else
								l_tmp___308 := l_bsize
							end
							a_scale1.item := (l_tmp___307 * l_wscale) * l_tmp___308
							update_class_fields -- Current might have been changed; synchronize
						end
						a_wr1.item := a_wr1.item * l_wscale
						update_class_fields -- Current might have been changed; synchronize
						if a_wi.item /= 0.0 then
							a_wi.item := a_wi.item * l_wscale
							update_class_fields -- Current might have been changed; synchronize
							a_wr2.item := a_wr1.item
							update_class_fields -- Current might have been changed; synchronize
							a_scale2.item := a_scale1.item
							update_class_fields -- Current might have been changed; synchronize
						end
					else
						a_scale1.item := l_ascale * l_bsize
						update_class_fields -- Current might have been changed; synchronize
						a_scale2.item := a_scale1.item
						update_class_fields -- Current might have been changed; synchronize
					end
					if a_wi.item = 0.0 then
						l_tmp___685 := fabs (a_wr2.item)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___698 := fabs (a_wr2.item)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___698 > l_c5___1 then
							l_tmp___696 := fabs (a_wr2.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___697 := l_tmp___696
						else
							l_tmp___697 := l_c5___1
						end
						if l_c4___2 < 0.5 * l_tmp___697 then
							l_tmp___694 := l_c4___2
						else
							l_tmp___693 := fabs (a_wr2.item)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___693 > l_c5___1 then
								l_tmp___691 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___692 := l_tmp___691
							else
								l_tmp___692 := l_c5___1
							end
							l_tmp___694 := 0.5 * l_tmp___692
						end
						if l_fuzzy1 * (l_tmp___685 * l_c2___0 + l_c3___3) > l_tmp___694 then
							l_tmp___670 := fabs (a_wr2.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___684 := l_fuzzy1 * (l_tmp___670 * l_c2___0 + l_c3___3)
						else
							l_tmp___683 := fabs (a_wr2.item)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___683 > l_c5___1 then
								l_tmp___681 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___682 := l_tmp___681
							else
								l_tmp___682 := l_c5___1
							end
							if l_c4___2 < 0.5 * l_tmp___682 then
								l_tmp___679 := l_c4___2
							else
								l_tmp___678 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___678 > l_c5___1 then
									l_tmp___676 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___677 := l_tmp___676
								else
									l_tmp___677 := l_c5___1
								end
								l_tmp___679 := 0.5 * l_tmp___677
							end
							l_tmp___684 := l_tmp___679
						end
						if l_c1___2 > l_tmp___684 then
							l_tmp___655 := l_c1___2
						else
							l_tmp___641 := fabs (a_wr2.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___654 := fabs (a_wr2.item)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___654 > l_c5___1 then
								l_tmp___652 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___653 := l_tmp___652
							else
								l_tmp___653 := l_c5___1
							end
							if l_c4___2 < 0.5 * l_tmp___653 then
								l_tmp___650 := l_c4___2
							else
								l_tmp___649 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___649 > l_c5___1 then
									l_tmp___647 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___648 := l_tmp___647
								else
									l_tmp___648 := l_c5___1
								end
								l_tmp___650 := 0.5 * l_tmp___648
							end
							if l_fuzzy1 * (l_tmp___641 * l_c2___0 + l_c3___3) > l_tmp___650 then
								l_tmp___626 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___640 := l_fuzzy1 * (l_tmp___626 * l_c2___0 + l_c3___3)
							else
								l_tmp___639 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___639 > l_c5___1 then
									l_tmp___637 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___638 := l_tmp___637
								else
									l_tmp___638 := l_c5___1
								end
								if l_c4___2 < 0.5 * l_tmp___638 then
									l_tmp___635 := l_c4___2
								else
									l_tmp___634 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									if l_tmp___634 > l_c5___1 then
										l_tmp___632 := fabs (a_wr2.item)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___633 := l_tmp___632
									else
										l_tmp___633 := l_c5___1
									end
									l_tmp___635 := 0.5 * l_tmp___633
								end
								l_tmp___640 := l_tmp___635
							end
							l_tmp___655 := l_tmp___640
						end
						if l_safemin > l_tmp___655 then
							l_wsize := l_safemin
						else
							l_tmp___555 := fabs (a_wr2.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___568 := fabs (a_wr2.item)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___568 > l_c5___1 then
								l_tmp___566 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___567 := l_tmp___566
							else
								l_tmp___567 := l_c5___1
							end
							if l_c4___2 < 0.5 * l_tmp___567 then
								l_tmp___564 := l_c4___2
							else
								l_tmp___563 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___563 > l_c5___1 then
									l_tmp___561 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___562 := l_tmp___561
								else
									l_tmp___562 := l_c5___1
								end
								l_tmp___564 := 0.5 * l_tmp___562
							end
							if l_fuzzy1 * (l_tmp___555 * l_c2___0 + l_c3___3) > l_tmp___564 then
								l_tmp___540 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___554 := l_fuzzy1 * (l_tmp___540 * l_c2___0 + l_c3___3)
							else
								l_tmp___553 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___553 > l_c5___1 then
									l_tmp___551 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___552 := l_tmp___551
								else
									l_tmp___552 := l_c5___1
								end
								if l_c4___2 < 0.5 * l_tmp___552 then
									l_tmp___549 := l_c4___2
								else
									l_tmp___548 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									if l_tmp___548 > l_c5___1 then
										l_tmp___546 := fabs (a_wr2.item)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___547 := l_tmp___546
									else
										l_tmp___547 := l_c5___1
									end
									l_tmp___549 := 0.5 * l_tmp___547
								end
								l_tmp___554 := l_tmp___549
							end
							if l_c1___2 > l_tmp___554 then
								l_tmp___525 := l_c1___2
							else
								l_tmp___511 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___524 := fabs (a_wr2.item)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___524 > l_c5___1 then
									l_tmp___522 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___523 := l_tmp___522
								else
									l_tmp___523 := l_c5___1
								end
								if l_c4___2 < 0.5 * l_tmp___523 then
									l_tmp___520 := l_c4___2
								else
									l_tmp___519 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									if l_tmp___519 > l_c5___1 then
										l_tmp___517 := fabs (a_wr2.item)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___518 := l_tmp___517
									else
										l_tmp___518 := l_c5___1
									end
									l_tmp___520 := 0.5 * l_tmp___518
								end
								if l_fuzzy1 * (l_tmp___511 * l_c2___0 + l_c3___3) > l_tmp___520 then
									l_tmp___496 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___510 := l_fuzzy1 * (l_tmp___496 * l_c2___0 + l_c3___3)
								else
									l_tmp___509 := fabs (a_wr2.item)
									update_class_fields -- Current might have been changed; synchronize
									if l_tmp___509 > l_c5___1 then
										l_tmp___507 := fabs (a_wr2.item)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___508 := l_tmp___507
									else
										l_tmp___508 := l_c5___1
									end
									if l_c4___2 < 0.5 * l_tmp___508 then
										l_tmp___505 := l_c4___2
									else
										l_tmp___504 := fabs (a_wr2.item)
										update_class_fields -- Current might have been changed; synchronize
										if l_tmp___504 > l_c5___1 then
											l_tmp___502 := fabs (a_wr2.item)
											update_class_fields -- Current might have been changed; synchronize
											l_tmp___503 := l_tmp___502
										else
											l_tmp___503 := l_c5___1
										end
										l_tmp___505 := 0.5 * l_tmp___503
									end
									l_tmp___510 := l_tmp___505
								end
								l_tmp___525 := l_tmp___510
							end
							l_wsize := l_tmp___525
						end
						if l_wsize /= 1.0 then
							l_wscale := 1.0 / l_wsize
							if l_wsize > 1.0 then
								if l_ascale > l_bsize then
									l_tmp___699 := l_ascale
								else
									l_tmp___699 := l_bsize
								end
								if l_ascale < l_bsize then
									l_tmp___700 := l_ascale
								else
									l_tmp___700 := l_bsize
								end
								a_scale2.item := (l_tmp___699 * l_wscale) * l_tmp___700
								update_class_fields -- Current might have been changed; synchronize
							else
								if l_ascale < l_bsize then
									l_tmp___701 := l_ascale
								else
									l_tmp___701 := l_bsize
								end
								if l_ascale > l_bsize then
									l_tmp___702 := l_ascale
								else
									l_tmp___702 := l_bsize
								end
								a_scale2.item := (l_tmp___701 * l_wscale) * l_tmp___702
								update_class_fields -- Current might have been changed; synchronize
							end
							a_wr2.item := a_wr2.item * l_wscale
							update_class_fields -- Current might have been changed; synchronize
						else
							a_scale2.item := l_ascale * l_bsize
							update_class_fields -- Current might have been changed; synchronize
						end
					end
					Result := 0
				when 15 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_schur_solve_equation (a_ca, a_z, a_d1, a_d2: REAL_64; a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]; a_s, a_xnorm: CE_POINTER [REAL_64]; a_smin: REAL_64): INTEGER_32
		require
			not a_b___0.is_void
			not a_x.is_void
			not a_s.is_void
			not a_xnorm.is_void
		local
			l_N: NATURAL_32
			l_bnorm: REAL_64
			l_scale: REAL_64
			l_c___1: REAL_64
			l_cnorm: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_cr: CE_ARRAY [CE_ARRAY [REAL_64]]
			l_crv: CE_POINTER [REAL_64]
			l_cmax: REAL_64
			l_icmax: NATURAL_32
			l_j: NATURAL_32
			l_bval1: REAL_64
			l_bval2: REAL_64
			l_ur11: REAL_64
			l_ur12: REAL_64
			l_ur22: REAL_64
			l_ur11r: REAL_64
			l_cr21: REAL_64
			l_cr22: REAL_64
			l_lr21: REAL_64
			l_b1___0: REAL_64
			l_b2: REAL_64
			l_bbnd: REAL_64
			l_x1___0: REAL_64
			l_x2___1: REAL_64
			l_temp: REAL_64
			l_ipivot: CE_ARRAY [CE_ARRAY [NATURAL_32]]
			l_rswap: CE_ARRAY [INTEGER_32]
			l_zswap: CE_ARRAY [INTEGER_32]
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_tmp___17: REAL_64
			l_tmp___18: REAL_64
			l_tmp___19: REAL_64
			l_tmp___20: REAL_64
			l_tmp___21: REAL_64
			l_tmp___22: REAL_64
			l_tmp___25: REAL_64
			l_tmp___26: REAL_64
			l_tmp___27: REAL_64
			l_tmp___28: REAL_64
			l_return: BOOLEAN
		do
			create {CE_ARRAY_ARRAY [REAL_64]} l_cr.make (<<2, 2>>)
			create {CE_ARRAY_ARRAY [NATURAL_32]} l_ipivot.make (<<4, 4>>)
			create {CE_INTEGER_32_ARRAY} l_rswap.make (<<4>>)
			create {CE_INTEGER_32_ARRAY} l_zswap.make (<<4>>)

			l_N := size1
			l_scale := 1.0
			if l_N = {NATURAL_32} 1 then
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_c___1 := a_ca * l_tmp - a_z * a_d1
				l_cnorm := fabs (l_c___1)
				update_class_fields -- Current might have been changed; synchronize
				if l_cnorm < a_smin then
					l_c___1 := a_smin
					l_cnorm := a_smin
				end
				l_tmp___0 := a_b___0.item.gsl_vector_get ((0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_bnorm := fabs (l_tmp___0)
				update_class_fields -- Current might have been changed; synchronize
				if l_cnorm < 1.0 then
					if l_bnorm > 1.0 then
						if l_bnorm > ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308)) * l_cnorm then
							l_scale := 1.0 / l_bnorm
						end
					end
				end
				l_tmp___1 := a_b___0.item.gsl_vector_get ((0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				a_x.item.gsl_vector_set ((0).to_natural_32, (l_tmp___1 * l_scale) / l_c___1)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___2 := a_x.item.gsl_vector_get ((0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				a_xnorm.item := fabs (l_tmp___2)
				update_class_fields -- Current might have been changed; synchronize
			else
				(l_ipivot[0])[0] := (0).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[0])[1] := (1).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[0])[2] := (2).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[0])[3] := (3).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[1])[0] := (1).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[1])[1] := (0).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[1])[2] := (3).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[1])[3] := (2).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[2])[0] := (2).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[2])[1] := (3).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[2])[2] := (0).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[2])[3] := (1).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[3])[0] := (3).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[3])[1] := (2).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[3])[2] := (1).to_natural_32
				update_class_fields; -- Current might have been changed; synchronize
				(l_ipivot[3])[3] := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_rswap[0] := 0
				update_class_fields -- Current might have been changed; synchronize
				l_rswap[1] := 1
				update_class_fields -- Current might have been changed; synchronize
				l_rswap[2] := 0
				update_class_fields -- Current might have been changed; synchronize
				l_rswap[3] := 1
				update_class_fields -- Current might have been changed; synchronize
				l_zswap[0] := 0
				update_class_fields -- Current might have been changed; synchronize
				l_zswap[1] := 0
				update_class_fields -- Current might have been changed; synchronize
				l_zswap[2] := 1
				update_class_fields -- Current might have been changed; synchronize
				l_zswap[3] := 1
				update_class_fields -- Current might have been changed; synchronize
				l_crv := create {CE_REAL_64_POINTER}.make_cast (l_cr)
				l_tmp___3 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
				update_class_fields; -- Current might have been changed; synchronize
				(l_cr[0])[0] := a_ca * l_tmp___3 - a_z * a_d1
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
				update_class_fields; -- Current might have been changed; synchronize
				(l_cr[1])[1] := a_ca * l_tmp___4 - a_z * a_d2
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
				update_class_fields; -- Current might have been changed; synchronize
				(l_cr[0])[1] := a_ca * l_tmp___5
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___6 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
				update_class_fields; -- Current might have been changed; synchronize
				(l_cr[1])[0] := a_ca * l_tmp___6
				update_class_fields -- Current might have been changed; synchronize
				l_cmax := 0.0
				l_icmax := (0).to_natural_32
				l_j := (0).to_natural_32
				from
				until
					not (l_j < {NATURAL_32} 4)
				loop
					l_tmp___7 := fabs (l_crv[l_j.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___7 > l_cmax then
						l_cmax := fabs (l_crv[l_j.to_integer_32])
						update_class_fields -- Current might have been changed; synchronize
						l_icmax := l_j
					end
					l_j := l_j + 1
				end
				l_bval1 := a_b___0.item.gsl_vector_get ((0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_bval2 := a_b___0.item.gsl_vector_get ((1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				if l_cmax < a_smin then
					l_tmp___12 := fabs (l_bval1)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___13 := fabs (l_bval2)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___12 > l_tmp___13 then
						l_tmp___10 := fabs (l_bval1)
						update_class_fields -- Current might have been changed; synchronize
						l_bnorm := l_tmp___10
					else
						l_tmp___11 := fabs (l_bval2)
						update_class_fields -- Current might have been changed; synchronize
						l_bnorm := l_tmp___11
					end
					if a_smin < 1.0 then
						if l_bnorm > 1.0 then
							if l_bnorm > ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308)) * a_smin then
								l_scale := 1.0 / l_bnorm
							end
						end
					end
					l_temp := l_scale / a_smin
					a_x.item.gsl_vector_set ((0).to_natural_32, l_temp * l_bval1)
					update_class_fields -- Current might have been changed; synchronize
					a_x.item.gsl_vector_set ((1).to_natural_32, l_temp * l_bval2)
					update_class_fields -- Current might have been changed; synchronize
					a_xnorm.item := l_temp * l_bnorm
					update_class_fields -- Current might have been changed; synchronize
					a_s.item := l_scale
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
				if not l_return then
					l_ur11 := l_crv[l_icmax.to_integer_32]
					l_cr21 := l_crv[((l_ipivot[1])[l_icmax.to_integer_32]).to_integer_32]
					l_ur12 := l_crv[((l_ipivot[2])[l_icmax.to_integer_32]).to_integer_32]
					l_cr22 := l_crv[((l_ipivot[3])[l_icmax.to_integer_32]).to_integer_32]
					l_ur11r := 1.0 / l_ur11
					l_lr21 := l_ur11r * l_cr21
					l_ur22 := l_cr22 - l_ur12 * l_lr21
					l_tmp___14 := fabs (l_ur22)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___14 < a_smin then
						l_ur22 := a_smin
					end
					if (l_rswap[l_icmax.to_integer_32]).to_boolean then
						l_b1___0 := l_bval2
						l_b2 := l_bval1
					else
						l_b1___0 := l_bval1
						l_b2 := l_bval2
					end
					l_b2 := l_b2 - (l_lr21 * l_b1___0)
					l_tmp___19 := fabs (l_b1___0 * (l_ur22 * l_ur11r))
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___20 := fabs (l_b2)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___19 > l_tmp___20 then
						l_tmp___17 := fabs (l_b1___0 * (l_ur22 * l_ur11r))
						update_class_fields -- Current might have been changed; synchronize
						l_bbnd := l_tmp___17
					else
						l_tmp___18 := fabs (l_b2)
						update_class_fields -- Current might have been changed; synchronize
						l_bbnd := l_tmp___18
					end
					if l_bbnd > 1.0 then
						l_tmp___22 := fabs (l_ur22)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___22 < 1.0 then
							l_tmp___21 := fabs (l_ur22)
							update_class_fields -- Current might have been changed; synchronize
							if l_bbnd >= ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308)) * l_tmp___21 then
								l_scale := 1.0 / l_bbnd
							end
						end
					end
					l_x2___1 := (l_b2 * l_scale) / l_ur22
					l_x1___0 := (l_scale * l_b1___0) * l_ur11r - l_x2___1 * (l_ur11r * l_ur12)
					if (l_zswap[l_icmax.to_integer_32]).to_boolean then
						a_x.item.gsl_vector_set ((0).to_natural_32, l_x2___1)
						update_class_fields -- Current might have been changed; synchronize
						a_x.item.gsl_vector_set ((1).to_natural_32, l_x1___0)
						update_class_fields -- Current might have been changed; synchronize
					else
						a_x.item.gsl_vector_set ((0).to_natural_32, l_x1___0)
						update_class_fields -- Current might have been changed; synchronize
						a_x.item.gsl_vector_set ((1).to_natural_32, l_x2___1)
						update_class_fields -- Current might have been changed; synchronize
					end
					l_tmp___27 := fabs (l_x1___0)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___28 := fabs (l_x2___1)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___27 > l_tmp___28 then
						l_tmp___25 := fabs (l_x1___0)
						update_class_fields -- Current might have been changed; synchronize
						a_xnorm.item := l_tmp___25
						update_class_fields -- Current might have been changed; synchronize
					else
						l_tmp___26 := fabs (l_x2___1)
						update_class_fields -- Current might have been changed; synchronize
						a_xnorm.item := l_tmp___26
						update_class_fields -- Current might have been changed; synchronize
					end
					if a_xnorm.item > 1.0 then
						if l_cmax > 1.0 then
							if a_xnorm.item > ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308)) / l_cmax then
								l_temp := l_cmax / ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308))
								a_x.item.gsl_blas_dscal (l_temp)
								update_class_fields -- Current might have been changed; synchronize
								a_xnorm.item := a_xnorm.item * l_temp
								update_class_fields -- Current might have been changed; synchronize
								l_scale := l_scale * l_temp
							end
						end
					end
				end
			end
			if not l_return then
				a_s.item := l_scale
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_schur_solve_equation_z (a_ca: REAL_64; a_z: CE_POINTER [PD_GSL_COMPLEX]; a_d1, a_d2: REAL_64; a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_s, a_xnorm: CE_POINTER [REAL_64]; a_smin: REAL_64): INTEGER_32
		require
			not a_z.is_void
			not a_b___0.is_void
			not a_s.is_void
		local
			l_N: NATURAL_32
			l_scale: REAL_64
			l_bnorm: REAL_64
			l_cr: REAL_64
			l_ci: REAL_64
			l_cnorm: REAL_64
			l_bval: PD_GSL_COMPLEX
			l_c___1: PD_GSL_COMPLEX
			l_xval: PD_GSL_COMPLEX
			l_tmp: PD_GSL_COMPLEX
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_cr___0: CE_ARRAY [CE_ARRAY [REAL_64]]
			l_ci___0: CE_ARRAY [CE_ARRAY [REAL_64]]
			l_civ: CE_POINTER [REAL_64]
			l_crv: CE_POINTER [REAL_64]
			l_cmax: REAL_64
			l_bval1: PD_GSL_COMPLEX
			l_bval2: PD_GSL_COMPLEX
			l_xval1: PD_GSL_COMPLEX
			l_xval2: PD_GSL_COMPLEX
			l_xr1: REAL_64
			l_xi1: REAL_64
			l_icmax: NATURAL_32
			l_j: NATURAL_32
			l_temp: REAL_64
			l_ur11: REAL_64
			l_ur12: REAL_64
			l_ur22: REAL_64
			l_ui11: REAL_64
			l_ui12: REAL_64
			l_ui22: REAL_64
			l_ur11r: REAL_64
			l_ui11r: REAL_64
			l_ur12s: REAL_64
			l_ui12s: REAL_64
			l_u22abs: REAL_64
			l_lr21: REAL_64
			l_li21: REAL_64
			l_cr21: REAL_64
			l_cr22: REAL_64
			l_ci21: REAL_64
			l_ci22: REAL_64
			l_br1: REAL_64
			l_bi1: REAL_64
			l_br2: REAL_64
			l_bi2: REAL_64
			l_bbnd: REAL_64
			l_b1___0: PD_GSL_COMPLEX
			l_b2: PD_GSL_COMPLEX
			l_ipivot: CE_ARRAY [CE_ARRAY [NATURAL_32]]
			l_rswap: CE_ARRAY [INTEGER_32]
			l_zswap: CE_ARRAY [INTEGER_32]
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_tmp___19: REAL_64
			l_tmp___20: REAL_64
			l_tmp___21: REAL_64
			l_tmp___22: REAL_64
			l_tmp___23: REAL_64
			l_tmp___24: REAL_64
			l_tmp___25: REAL_64
			l_tmp___26: REAL_64
			l_tmp___27: REAL_64
			l_tmp___28: REAL_64
			l_tmp___29: REAL_64
			l_tmp___30: REAL_64
			l_tmp___37: REAL_64
			l_tmp___38: REAL_64
			l_tmp___39: REAL_64
			l_tmp___40: REAL_64
			l_tmp___41: REAL_64
			l_tmp___42: REAL_64
			l_tmp___43: REAL_64
			l_tmp___44: REAL_64
			l_tmp___45: REAL_64
			l_tmp___46: REAL_64
			l_tmp___47: REAL_64
			l_tmp___48: REAL_64
			l_tmp___53: REAL_64
			l_tmp___54: REAL_64
			l_tmp___55: REAL_64
			l_tmp___56: REAL_64
			l_tmp___57: REAL_64
			l_tmp___58: REAL_64
			l_tmp___59: REAL_64
			l_tmp___60: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create l_c___1.make
			create l_tmp.make
			create {CE_ARRAY_ARRAY [REAL_64]} l_cr___0.make (<<2, 2>>)
			create {CE_ARRAY_ARRAY [REAL_64]} l_ci___0.make (<<2, 2>>)
			create l_xval1.make
			create l_b1___0.make
			create l_b2.make
			create {CE_ARRAY_ARRAY [NATURAL_32]} l_ipivot.make (<<4, 4>>)
			create {CE_INTEGER_32_ARRAY} l_rswap.make (<<4>>)
			create {CE_INTEGER_32_ARRAY} l_zswap.make (<<4>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := size1
					l_scale := 1.0
					if l_N /= {NATURAL_32} 1 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_cr := a_ca * l_tmp___0 - a_z.item.dat[0] * a_d1
						l_ci := -a_z.item.dat[1] * a_d1
						l_tmp___1 := fabs (l_cr)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := fabs (l_ci)
						update_class_fields -- Current might have been changed; synchronize
						l_cnorm := l_tmp___1 + l_tmp___2
						if l_cnorm < a_smin then
							l_cr := a_smin
							l_ci := 0.0
							l_cnorm := a_smin
						end
						l_bval := (a_b___0.item.gsl_vector_complex_get ((0).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := fabs (l_bval.dat[0])
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___4 := fabs (l_bval.dat[1])
						update_class_fields -- Current might have been changed; synchronize
						l_bnorm := l_tmp___3 + l_tmp___4
						if l_cnorm < 1.0 then
							if l_bnorm > 1.0 then
								if l_bnorm > ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308)) * l_cnorm then
									l_scale := 1.0 / l_bnorm
								end
							end
						end
						from
						until
							(l_break)
						loop
							l_break := False
							l_tmp.dat[0] := l_scale * l_bval.dat[0]
							update_class_fields -- Current might have been changed; synchronize
							l_tmp.dat[1] := l_scale * l_bval.dat[1]
							update_class_fields -- Current might have been changed; synchronize
							l_break := True
						end
						l_break := False
						from
						until
							(l_break)
						loop
							l_break := False
							l_c___1.dat[0] := l_cr
							update_class_fields -- Current might have been changed; synchronize
							l_c___1.dat[1] := l_ci
							update_class_fields -- Current might have been changed; synchronize
							l_break := True
						end
						l_break := False
						l_xval := ((create {P_MATH_STATIC}).gsl_complex_div (l_tmp.twin, l_c___1.twin)).twin
						update_class_fields -- Current might have been changed; synchronize
						a_x.item.gsl_vector_complex_set ((0).to_natural_32, l_xval.twin)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___5 := fabs (l_xval.dat[0])
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___6 := fabs (l_xval.dat[1])
						update_class_fields -- Current might have been changed; synchronize
						a_xnorm.item := l_tmp___5 + l_tmp___6
						update_class_fields -- Current might have been changed; synchronize
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 3 then -- else:;
						(l_ipivot[0])[0] := (0).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[0])[1] := (1).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[0])[2] := (2).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[0])[3] := (3).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[1])[0] := (1).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[1])[1] := (0).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[1])[2] := (3).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[1])[3] := (2).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[2])[0] := (2).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[2])[1] := (3).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[2])[2] := (0).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[2])[3] := (1).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[3])[0] := (3).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[3])[1] := (2).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[3])[2] := (1).to_natural_32
						update_class_fields; -- Current might have been changed; synchronize
						(l_ipivot[3])[3] := (0).to_natural_32
						update_class_fields -- Current might have been changed; synchronize
						l_rswap[0] := 0
						update_class_fields -- Current might have been changed; synchronize
						l_rswap[1] := 1
						update_class_fields -- Current might have been changed; synchronize
						l_rswap[2] := 0
						update_class_fields -- Current might have been changed; synchronize
						l_rswap[3] := 1
						update_class_fields -- Current might have been changed; synchronize
						l_zswap[0] := 0
						update_class_fields -- Current might have been changed; synchronize
						l_zswap[1] := 0
						update_class_fields -- Current might have been changed; synchronize
						l_zswap[2] := 1
						update_class_fields -- Current might have been changed; synchronize
						l_zswap[3] := 1
						update_class_fields -- Current might have been changed; synchronize
						l_civ := create {CE_REAL_64_POINTER}.make_cast (l_ci___0)
						l_crv := create {CE_REAL_64_POINTER}.make_cast (l_cr___0)
						l_tmp___7 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
						update_class_fields; -- Current might have been changed; synchronize
						(l_cr___0[0])[0] := a_ca * l_tmp___7 - a_z.item.dat[0] * a_d1
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___8 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
						update_class_fields; -- Current might have been changed; synchronize
						(l_cr___0[1])[1] := a_ca * l_tmp___8 - a_z.item.dat[0] * a_d2
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___9 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
						update_class_fields; -- Current might have been changed; synchronize
						(l_cr___0[0])[1] := a_ca * l_tmp___9
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___10 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
						update_class_fields; -- Current might have been changed; synchronize
						(l_cr___0[1])[0] := a_ca * l_tmp___10
						update_class_fields; -- Current might have been changed; synchronize
						(l_ci___0[0])[0] := -a_z.item.dat[1] * a_d1
						update_class_fields; -- Current might have been changed; synchronize
						(l_ci___0[0])[1] := 0.0
						update_class_fields; -- Current might have been changed; synchronize
						(l_ci___0[1])[0] := 0.0
						update_class_fields; -- Current might have been changed; synchronize
						(l_ci___0[1])[1] := -a_z.item.dat[1] * a_d2
						update_class_fields -- Current might have been changed; synchronize
						l_cmax := 0.0
						l_icmax := (0).to_natural_32
						l_j := (0).to_natural_32
						from
						until
							not (l_j < {NATURAL_32} 4)
						loop
							l_tmp___13 := fabs (l_crv[l_j.to_integer_32])
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___14 := fabs (l_civ[l_j.to_integer_32])
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___13 + l_tmp___14 > l_cmax then
								l_tmp___11 := fabs (l_crv[l_j.to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___12 := fabs (l_civ[l_j.to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_cmax := l_tmp___11 + l_tmp___12
								l_icmax := l_j
							end
							l_j := l_j + 1
						end
						l_bval1 := (a_b___0.item.gsl_vector_complex_get ((0).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_bval2 := (a_b___0.item.gsl_vector_complex_get ((1).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						if l_cmax < a_smin then
							l_tmp___23 := fabs (l_bval1.dat[0])
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___24 := fabs (l_bval1.dat[1])
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___25 := fabs (l_bval2.dat[0])
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___26 := fabs (l_bval2.dat[1])
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___23 + l_tmp___24 > l_tmp___25 + l_tmp___26 then
								l_tmp___19 := fabs (l_bval1.dat[0])
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___20 := fabs (l_bval1.dat[1])
								update_class_fields -- Current might have been changed; synchronize
								l_bnorm := l_tmp___19 + l_tmp___20
							else
								l_tmp___21 := fabs (l_bval2.dat[0])
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___22 := fabs (l_bval2.dat[1])
								update_class_fields -- Current might have been changed; synchronize
								l_bnorm := l_tmp___21 + l_tmp___22
							end
							if a_smin < 1.0 then
								if l_bnorm > 1.0 then
									if l_bnorm > ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308)) * a_smin then
										l_scale := 1.0 / l_bnorm
									end
								end
							end
							l_temp := l_scale / a_smin
							l_xval1 := ((create {P_MATH_STATIC}).gsl_complex_mul_real (l_bval1.twin, l_temp)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_xval2 := ((create {P_MATH_STATIC}).gsl_complex_mul_real (l_bval2.twin, l_temp)).twin
							update_class_fields -- Current might have been changed; synchronize
							a_x.item.gsl_vector_complex_set ((0).to_natural_32, l_xval1.twin)
							update_class_fields -- Current might have been changed; synchronize
							a_x.item.gsl_vector_complex_set ((1).to_natural_32, l_xval2.twin)
							update_class_fields -- Current might have been changed; synchronize
							a_xnorm.item := l_temp * l_bnorm
							update_class_fields -- Current might have been changed; synchronize
							a_s.item := l_scale
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						end
						if l_return then
							l_pos := -1 + 12 -- skip if block
						end
				when 4 then -- if:
						l_ur11 := l_crv[l_icmax.to_integer_32]
						l_ui11 := l_civ[l_icmax.to_integer_32]
						l_cr21 := l_crv[((l_ipivot[1])[l_icmax.to_integer_32]).to_integer_32]
						l_ci21 := l_civ[((l_ipivot[1])[l_icmax.to_integer_32]).to_integer_32]
						l_ur12 := l_crv[((l_ipivot[2])[l_icmax.to_integer_32]).to_integer_32]
						l_ui12 := l_civ[((l_ipivot[2])[l_icmax.to_integer_32]).to_integer_32]
						l_cr22 := l_crv[((l_ipivot[3])[l_icmax.to_integer_32]).to_integer_32]
						l_ci22 := l_civ[((l_ipivot[3])[l_icmax.to_integer_32]).to_integer_32]
						if l_icmax /= {NATURAL_32} 0 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						l_pos := -1 + 8 -- goto _L
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else:
						if l_icmax /= {NATURAL_32} 3 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if: | _L:
						l_tmp___27 := fabs (l_ur11)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___28 := fabs (l_ui11)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___27 > l_tmp___28 then
							l_temp := l_ui11 / l_ur11
							l_ur11r := 1.0 / (l_ur11 * (1.0 + l_temp * l_temp))
							l_ui11r := -l_temp * l_ur11r
						else
							l_temp := l_ur11 / l_ui11
							l_ui11r := -1.0 / (l_ui11 * (1.0 + l_temp * l_temp))
							l_ur11r := -l_temp * l_ui11r
						end
						l_lr21 := l_cr21 * l_ur11r
						l_li21 := l_cr21 * l_ui11r
						l_ur12s := l_ur12 * l_ur11r
						l_ui12s := l_ur12 * l_ui11r
						l_ur22 := l_cr22 - l_ur12 * l_lr21
						l_ui22 := l_ci22 - l_ur12 * l_li21
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else:
						l_ur11r := 1.0 / l_ur11
						l_ui11r := 0.0
						l_lr21 := l_cr21 * l_ur11r
						l_li21 := l_ci21 * l_ur11r
						l_ur12s := l_ur12 * l_ur11r
						l_ui12s := l_ui12 * l_ur11r
						l_ur22 := (l_cr22 - l_ur12 * l_lr21) + l_ui12 * l_li21
						l_ui22 := -l_ur12 * l_li21 - l_ui12 * l_lr21
				when 11 then -- if done
						l_tmp___29 := fabs (l_ur22)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___30 := fabs (l_ui22)
						update_class_fields -- Current might have been changed; synchronize
						l_u22abs := l_tmp___29 + l_tmp___30
						if l_u22abs < a_smin then
							l_ur22 := a_smin
							l_ui22 := 0.0
						end
						if (l_rswap[l_icmax.to_integer_32]).to_boolean then
							l_br2 := l_bval1.dat[0]
							l_bi2 := l_bval1.dat[1]
							l_br1 := l_bval2.dat[0]
							l_bi1 := l_bval2.dat[1]
						else
							l_br1 := l_bval1.dat[0]
							l_bi1 := l_bval1.dat[1]
							l_br2 := l_bval2.dat[0]
							l_bi2 := l_bval2.dat[1]
						end
						l_br2 := l_br2 + (l_li21 * l_bi1 - l_lr21 * l_br1)
						l_bi2 := l_bi2 - (l_li21 * l_br1 + l_lr21 * l_bi1)
						l_tmp___43 := fabs (l_br1)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___44 := fabs (l_bi1)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___45 := fabs (l_ur11r)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___46 := fabs (l_ui11r)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___47 := fabs (l_br2)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___48 := fabs (l_bi2)
						update_class_fields -- Current might have been changed; synchronize
						if (l_tmp___43 + l_tmp___44) * (l_u22abs * (l_tmp___45 + l_tmp___46)) > l_tmp___47 + l_tmp___48 then
							l_tmp___37 := fabs (l_br1)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___38 := fabs (l_bi1)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___39 := fabs (l_ur11r)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___40 := fabs (l_ui11r)
							update_class_fields -- Current might have been changed; synchronize
							l_bbnd := (l_tmp___37 + l_tmp___38) * (l_u22abs * (l_tmp___39 + l_tmp___40))
						else
							l_tmp___41 := fabs (l_br2)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___42 := fabs (l_bi2)
							update_class_fields -- Current might have been changed; synchronize
							l_bbnd := l_tmp___41 + l_tmp___42
						end
						if l_bbnd > 1.0 then
							if l_u22abs < 1.0 then
								if l_bbnd >= ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308)) * l_u22abs then
									l_scale := 1.0 / l_bbnd
									l_br1 := l_br1 * l_scale
									l_bi1 := l_bi1 * l_scale
									l_br2 := l_br2 * l_scale
									l_bi2 := l_bi2 * l_scale
								end
							end
						end
						from
						until
							(l_break)
						loop
							l_break := False
							l_b1___0.dat[0] := l_br2
							update_class_fields -- Current might have been changed; synchronize
							l_b1___0.dat[1] := l_bi2
							update_class_fields -- Current might have been changed; synchronize
							l_break := True
						end
						l_break := False
						from
						until
							(l_break)
						loop
							l_break := False
							l_b2.dat[0] := l_ur22
							update_class_fields -- Current might have been changed; synchronize
							l_b2.dat[1] := l_ui22
							update_class_fields -- Current might have been changed; synchronize
							l_break := True
						end
						l_break := False
						l_xval2 := ((create {P_MATH_STATIC}).gsl_complex_div (l_b1___0.twin, l_b2.twin)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_xr1 := ((l_ur11r * l_br1 - l_ui11r * l_bi1) - l_ur12s * l_xval2.dat[0]) + l_ui12s * l_xval2.dat[1]
						l_xi1 := ((l_ui11r * l_br1 + l_ur11r * l_bi1) - l_ui12s * l_xval2.dat[0]) - l_ur12s * l_xval2.dat[1]
						from
						until
							(l_break)
						loop
							l_break := False
							l_xval1.dat[0] := l_xr1
							update_class_fields -- Current might have been changed; synchronize
							l_xval1.dat[1] := l_xi1
							update_class_fields -- Current might have been changed; synchronize
							l_break := True
						end
						l_break := False
						if (l_zswap[l_icmax.to_integer_32]).to_boolean then
							a_x.item.gsl_vector_complex_set ((0).to_natural_32, l_xval2.twin)
							update_class_fields -- Current might have been changed; synchronize
							a_x.item.gsl_vector_complex_set ((1).to_natural_32, l_xval1.twin)
							update_class_fields -- Current might have been changed; synchronize
						else
							a_x.item.gsl_vector_complex_set ((0).to_natural_32, l_xval1.twin)
							update_class_fields -- Current might have been changed; synchronize
							a_x.item.gsl_vector_complex_set ((1).to_natural_32, l_xval2.twin)
							update_class_fields -- Current might have been changed; synchronize
						end
						l_tmp___57 := fabs (l_xval1.dat[0])
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___58 := fabs (l_xval1.dat[1])
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___59 := fabs (l_xval2.dat[0])
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___60 := fabs (l_xval2.dat[1])
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___57 + l_tmp___58 > l_tmp___59 + l_tmp___60 then
							l_tmp___53 := fabs (l_xval1.dat[0])
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___54 := fabs (l_xval1.dat[1])
							update_class_fields -- Current might have been changed; synchronize
							a_xnorm.item := l_tmp___53 + l_tmp___54
							update_class_fields -- Current might have been changed; synchronize
						else
							l_tmp___55 := fabs (l_xval2.dat[0])
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___56 := fabs (l_xval2.dat[1])
							update_class_fields -- Current might have been changed; synchronize
							a_xnorm.item := l_tmp___55 + l_tmp___56
							update_class_fields -- Current might have been changed; synchronize
						end
						if a_xnorm.item > 1.0 then
							if l_cmax > 1.0 then
								if a_xnorm.item > ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308)) / l_cmax then
									l_temp := l_cmax / ((1.0 - 2.2204460492503131e-16) / (2.0 * 2.2250738585072014e-308))
									a_x.item.gsl_blas_zdscal (l_temp)
									update_class_fields -- Current might have been changed; synchronize
									a_xnorm.item := a_xnorm.item * l_temp
									update_class_fields -- Current might have been changed; synchronize
									l_scale := l_scale * l_temp
								end
							end
						end
				when 12 then -- if done | if done
					if not l_return then
						a_s.item := l_scale
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
					end
				when 13 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_eigen_nonsymm (a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_NONSYMM_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_s: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_N := size1
			if l_N /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("nonsymm.c"), 181, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_eval.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("nonsymm.c"), 185, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				if (a_w.item.do_balance).to_boolean then
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_balance_matrix (a_w.item.diag).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_hessenberg_decomp (a_w.item.tau).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				if (not (a_w.item.Z).is_void) then
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_hessenberg_unpack (a_w.item.tau, a_w.item.Z).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_francis_Z (a_eval, a_w.item.Z, a_w.item.francis_workspace_p)
					update_class_fields -- Current might have been changed; synchronize
					if (a_w.item.do_balance).to_boolean then
						(a_w.item.Z).item.gsl_linalg_balance_accum (a_w.item.diag).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
				else
					l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_francis (a_eval, a_w.item.francis_workspace_p)
					update_class_fields -- Current might have been changed; synchronize
				end
				a_w.item.n_evals := (a_w.item.francis_workspace_p).item.n_evals
				update_class_fields -- Current might have been changed; synchronize
				Result := l_s
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_eigen_nonsymm_Z (a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_Z: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_NONSYMM_WORKSPACE]): INTEGER_32
		local
			l_s: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if size1 = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("nonsymm.c"), 271, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if a_eval.item.size = size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("nonsymm.c"), 275, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if a_Z.item.size1 = a_Z.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if a_Z.item.size1 = size1 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Z matrix has wrong dimensions"), ce_string ("nonsymm.c"), 279, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						a_w.item.Z := a_Z
						update_class_fields -- Current might have been changed; synchronize
						l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_nonsymm (a_eval, a_w)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.Z := create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer)
						update_class_fields -- Current might have been changed; synchronize
						Result := l_s
						l_return := True
				when 13 then -- if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_eigen_francis (a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_FRANCIS_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_j: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("francis.c"), 159, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_eval.item.size /= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("francis.c"), 163, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_N := size1
				a_w.item.size := l_N
				update_class_fields -- Current might have been changed; synchronize
				a_w.item.max_iterations := {NATURAL_32} 30 * l_N
				update_class_fields -- Current might have been changed; synchronize
				a_w.item.H := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))
				update_class_fields -- Current might have been changed; synchronize
				a_w.item.n_iter := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				a_w.item.n_evals := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_j := 0
				from
				until
					not (l_j < l_N.to_integer_32 - 3)
				loop
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j.to_natural_32 + {NATURAL_32} 2, l_j.to_natural_32, 0.0)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j.to_natural_32 + {NATURAL_32} 3, l_j.to_natural_32, 0.0)
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				if l_N > {NATURAL_32} 2 then
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 3, 0.0)
					update_class_fields -- Current might have been changed; synchronize
				end
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.francis_schur_decomp (a_eval, a_w)
				update_class_fields -- Current might have been changed; synchronize
				if a_w.item.n_evals /= l_N then
					Result := 11
					l_return := True
				end
				if not l_return then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_eigen_francis_Z (a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_Z: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_FRANCIS_WORKSPACE]): INTEGER_32
		require
			not a_w.is_void
		local
			l_s: INTEGER_32
		do
			a_w.item.Z := a_Z
			update_class_fields -- Current might have been changed; synchronize
			l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_francis (a_eval, a_w)
			update_class_fields -- Current might have been changed; synchronize
			a_w.item.Z := create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_s
		end

	francis_schur_decomp (a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_FRANCIS_WORKSPACE])
		local
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_N: NATURAL_32
			l_q___3: NATURAL_32
			l_lambda1: CE_ARRAY [PD_GSL_COMPLEX]
			l_lambda2: CE_ARRAY [PD_GSL_COMPLEX]
			l_v: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_v___0: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_v___1: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp: NATURAL_32
			l_return, l_break, l_continue: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_COMPLEX]} l_lambda1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_COMPLEX]} l_lambda2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_v.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_v___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_v___1.make (<<1>>)

			l_N := size1
			l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
			update_class_fields -- Current might have been changed; synchronize
			from
			until
				(l_return or l_break)
			loop
				l_break := False
				l_continue := False
				if l_N > {NATURAL_32} 2 then
					l_tmp := a_w.item.n_iter
					a_w.item.n_iter := (a_w.item.n_iter) + 1
					update_class_fields -- Current might have been changed; synchronize
					if not (l_tmp < a_w.item.max_iterations) then
						l_break := True
					end
				else
					l_break := True
				end
				if not (l_return or l_break or l_continue) then
					l_q___3 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.francis_search_subdiag_small_elements
					update_class_fields -- Current might have been changed; synchronize
					if l_q___3 = {NATURAL_32} 0 then
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.francis_qrstep (a_w).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_continue := True
					end
				end
				if not (l_return or l_break or l_continue) then
					if l_q___3 = l_N - {NATURAL_32} 1 then
						from
						until
							(l_break)
						loop
							l_break := False
							l_lambda1.item.dat[0] := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get (l_q___3, l_q___3)
							update_class_fields -- Current might have been changed; synchronize
							l_lambda1.item.dat[1] := 0.0
							update_class_fields -- Current might have been changed; synchronize
							l_break := True
						end
						l_break := False
						a_eval.item.gsl_vector_complex_set (a_w.item.n_evals, (l_lambda1.item).twin)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.n_evals := (a_w.item.n_evals) + 1
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.n_iter := (0).to_natural_32
						update_class_fields -- Current might have been changed; synchronize
						l_N := l_N - 1
						l_m___4.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
					elseif l_q___3 = l_N - {NATURAL_32} 2 then
						l_v.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix (l_q___3, l_q___3, (2).to_natural_32, (2).to_natural_32)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.francis_schur_standardize (l_lambda1, l_lambda2, a_w)
						update_class_fields -- Current might have been changed; synchronize
						a_eval.item.gsl_vector_complex_set (a_w.item.n_evals, (l_lambda1.item).twin)
						update_class_fields -- Current might have been changed; synchronize
						a_eval.item.gsl_vector_complex_set (a_w.item.n_evals + {NATURAL_32} 1, (l_lambda2.item).twin)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.n_evals := a_w.item.n_evals + ({NATURAL_32} 2)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.n_iter := (0).to_natural_32
						update_class_fields -- Current might have been changed; synchronize
						l_N := l_N - ({NATURAL_32} 2)
						l_m___4.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
					elseif l_q___3 = {NATURAL_32} 1 then
						from
						until
							(l_break)
						loop
							l_break := False
							l_lambda1.item.dat[0] := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_lambda1.item.dat[1] := 0.0
							update_class_fields -- Current might have been changed; synchronize
							l_break := True
						end
						l_break := False
						a_eval.item.gsl_vector_complex_set (a_w.item.n_evals, (l_lambda1.item).twin)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.n_evals := (a_w.item.n_evals) + 1
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.n_iter := (0).to_natural_32
						update_class_fields -- Current might have been changed; synchronize
						l_N := l_N - 1
						l_m___4.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((1).to_natural_32, (1).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
					elseif l_q___3 = {NATURAL_32} 2 then
						l_v___0.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, (2).to_natural_32, (2).to_natural_32)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_v___0.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.francis_schur_standardize (l_lambda1, l_lambda2, a_w)
						update_class_fields -- Current might have been changed; synchronize
						a_eval.item.gsl_vector_complex_set (a_w.item.n_evals, (l_lambda1.item).twin)
						update_class_fields -- Current might have been changed; synchronize
						a_eval.item.gsl_vector_complex_set (a_w.item.n_evals + {NATURAL_32} 1, (l_lambda2.item).twin)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.n_evals := a_w.item.n_evals + ({NATURAL_32} 2)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.n_iter := (0).to_natural_32
						update_class_fields -- Current might have been changed; synchronize
						l_N := l_N - ({NATURAL_32} 2)
						l_m___4.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((2).to_natural_32, (2).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
					else
						l_v___1.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix (l_q___3, l_q___3, l_N - l_q___3, l_N - l_q___3)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_v___1.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.francis_schur_decomp (a_eval, a_w)
						update_class_fields -- Current might have been changed; synchronize
						l_v___1.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_q___3, l_q___3)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_v___1.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.francis_schur_decomp (a_eval, a_w)
						update_class_fields -- Current might have been changed; synchronize
						l_N := (0).to_natural_32
					end
				end
			end
			l_break := False
			l_continue := False
			if l_N = {NATURAL_32} 1 then
				from
				until
					(l_break)
				loop
					l_break := False
					l_lambda1.item.dat[0] := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_lambda1.item.dat[1] := 0.0
					update_class_fields -- Current might have been changed; synchronize
					l_break := True
				end
				l_break := False
				a_eval.item.gsl_vector_complex_set (a_w.item.n_evals, (l_lambda1.item).twin)
				update_class_fields -- Current might have been changed; synchronize
				a_w.item.n_evals := (a_w.item.n_evals) + 1
				update_class_fields -- Current might have been changed; synchronize
				a_w.item.n_iter := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
			elseif l_N = {NATURAL_32} 2 then
				(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.francis_schur_standardize (l_lambda1, l_lambda2, a_w)
				update_class_fields -- Current might have been changed; synchronize
				a_eval.item.gsl_vector_complex_set (a_w.item.n_evals, (l_lambda1.item).twin)
				update_class_fields -- Current might have been changed; synchronize
				a_eval.item.gsl_vector_complex_set (a_w.item.n_evals + {NATURAL_32} 1, (l_lambda2.item).twin)
				update_class_fields -- Current might have been changed; synchronize
				a_w.item.n_evals := a_w.item.n_evals + ({NATURAL_32} 2)
				update_class_fields -- Current might have been changed; synchronize
				a_w.item.n_iter := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	francis_qrstep (a_w: CE_POINTER [PD_GSL_EIGEN_FRANCIS_WORKSPACE]): INTEGER_32
		require
			not a_w.is_void
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tau_i: REAL_64
			l_dat: CE_ARRAY [REAL_64]
			l_scale: REAL_64
			l_v2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v3: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_q___3: NATURAL_32
			l_r___3: NATURAL_32
			l_top: NATURAL_32
			l_s: REAL_64
			l_disc: REAL_64
			l_h_nn: REAL_64
			l_h_nm1nm1: REAL_64
			l_h_cross: REAL_64
			l_h_tmp1: REAL_64
			l_h_tmp2: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_ave: REAL_64
			l_tmp___6: INTEGER_32
			l_tmp___7: INTEGER_32
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_tmp___15: REAL_64
			l_tmp___16: REAL_64
			l_tmp___17: REAL_64
			l_tmp___18: REAL_64
			l_tmp___19: REAL_64
			l_tmp___20: REAL_64
			l_tmp___21: REAL_64
			l_tmp___22: REAL_64
			l_tmp___23: REAL_64
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_dat.make (<<3>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v3.make (<<1>>)

			l_N := size1
			l_top := (0).to_natural_32
			l_v2.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_dat, (2).to_natural_32)).twin
			update_class_fields -- Current might have been changed; synchronize
			l_v3.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_dat, (3).to_natural_32)).twin
			update_class_fields -- Current might have been changed; synchronize
			if a_w.item.n_iter \\ {NATURAL_32} 10 = {NATURAL_32} 0 then
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 2)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := fabs (l_tmp)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 3)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___2 := fabs (l_tmp___1)
				update_class_fields -- Current might have been changed; synchronize
				l_s := l_tmp___0 + l_tmp___2
				l_tmp___3 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_h_nn := l_tmp___3 + 0.75 * l_s
				l_h_nm1nm1 := l_h_nn
				l_h_cross := (-0.4375 * l_s) * l_s
			else
				l_h_nn := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_h_nm1nm1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 2)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_h_cross := l_tmp___4 * l_tmp___5
				l_disc := 0.5 * (l_h_nm1nm1 - l_h_nn)
				l_disc := l_disc * l_disc + l_h_cross
				if l_disc > 0.0 then
					l_disc := sqrt (l_disc)
					update_class_fields -- Current might have been changed; synchronize
					l_ave := 0.5 * (l_h_nm1nm1 + l_h_nn)
					l_tmp___8 := fabs (l_h_nm1nm1)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___9 := fabs (l_h_nn)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___8 - l_tmp___9 > 0.0 then
						l_h_nm1nm1 := l_h_nm1nm1 * l_h_nn - l_h_cross
						if l_ave >= 0.0 then
							l_tmp___6 := 1
						else
							l_tmp___6 := -1
						end
						l_h_nn := l_h_nm1nm1 / (l_disc * l_tmp___6.to_double + l_ave)
					else
						if l_ave >= 0.0 then
							l_tmp___7 := 1
						else
							l_tmp___7 := -1
						end
						l_h_nn := l_disc * l_tmp___7.to_double + l_ave
					end
					l_h_nm1nm1 := l_h_nn
					l_h_cross := 0.0
				end
			end
			l_tmp___10 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_h_tmp1 := l_h_nm1nm1 - l_tmp___10
			l_tmp___11 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_h_tmp2 := l_h_nn - l_tmp___11
			l_tmp___12 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___13 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_dat[0] := (l_h_tmp1 * l_h_tmp2 - l_h_cross) / l_tmp___12 + l_tmp___13
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___14 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___15 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_dat[1] := ((l_tmp___14 - l_tmp___15) - l_h_tmp1) - l_h_tmp2
			update_class_fields -- Current might have been changed; synchronize
			l_dat[2] := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((2).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___16 := fabs (l_dat[0])
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___17 := fabs (l_dat[1])
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___18 := fabs (l_dat[2])
			update_class_fields -- Current might have been changed; synchronize
			l_scale := (l_tmp___16 + l_tmp___17) + l_tmp___18
			if l_scale /= 0.0 then
				l_dat[0] := l_dat[0] / l_scale
				update_class_fields -- Current might have been changed; synchronize
				l_dat[1] := l_dat[1] / l_scale
				update_class_fields -- Current might have been changed; synchronize
				l_dat[2] := l_dat[2] / l_scale
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not (a_w.item.Z).is_void) then
				l_top := (a_w.item.H).item.francis_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
				update_class_fields -- Current might have been changed; synchronize
			elseif (a_w.item.compute_t).to_boolean then
				l_top := (a_w.item.H).item.francis_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
				update_class_fields -- Current might have been changed; synchronize
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N - {NATURAL_32} 2)
			loop
				l_tau_i := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
				update_class_fields -- Current might have been changed; synchronize
				if l_tau_i /= 0.0 then
					if 1 > l_i.to_integer_32 - 1 then
						l_q___3 := (0).to_natural_32
					else
						l_q___3 := l_i - {NATURAL_32} 1
					end
					if l_i + {NATURAL_32} 3 < l_N - {NATURAL_32} 1 then
						l_r___3 := l_i + {NATURAL_32} 3
					else
						l_r___3 := l_N - {NATURAL_32} 1
					end
					if (a_w.item.compute_t).to_boolean then
						l_m___4.item := ((a_w.item.H).item.gsl_matrix_submatrix (l_top + l_i, l_top + l_q___3, (3).to_natural_32, (a_w.item.size - l_top) - l_q___3)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_m___4.item := ((a_w.item.H).item.gsl_matrix_submatrix ((0).to_natural_32, l_top + l_i, (l_top + l_r___3) + {NATURAL_32} 1, (3).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					else
						l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i, l_q___3, (3).to_natural_32, l_N - l_q___3)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, l_i, l_r___3 + {NATURAL_32} 1, (3).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					if (not (a_w.item.Z).is_void) then
						l_m___4.item := ((a_w.item.Z).item.gsl_matrix_submatrix ((0).to_natural_32, l_top + l_i, a_w.item.size, (3).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				l_dat[0] := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i + {NATURAL_32} 1, l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_dat[1] := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i + {NATURAL_32} 2, l_i)
				update_class_fields -- Current might have been changed; synchronize
				if l_i < l_N - {NATURAL_32} 3 then
					l_dat[2] := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i + {NATURAL_32} 3, l_i)
					update_class_fields -- Current might have been changed; synchronize
				end
				l_tmp___19 := fabs (l_dat[0])
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___20 := fabs (l_dat[1])
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___21 := fabs (l_dat[2])
				update_class_fields -- Current might have been changed; synchronize
				l_scale := (l_tmp___19 + l_tmp___20) + l_tmp___21
				if l_scale /= 0.0 then
					l_dat[0] := l_dat[0] / l_scale
					update_class_fields -- Current might have been changed; synchronize
					l_dat[1] := l_dat[1] / l_scale
					update_class_fields -- Current might have been changed; synchronize
					l_dat[2] := l_dat[2] / l_scale
					update_class_fields -- Current might have been changed; synchronize
				end
				l_i := l_i + 1
			end
			l_tmp___22 := fabs (l_dat[0])
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___23 := fabs (l_dat[1])
			update_class_fields -- Current might have been changed; synchronize
			l_scale := l_tmp___22 + l_tmp___23
			if l_scale /= 0.0 then
				l_dat[0] := l_dat[0] / l_scale
				update_class_fields -- Current might have been changed; synchronize
				l_dat[1] := l_dat[1] / l_scale
				update_class_fields -- Current might have been changed; synchronize
			end
			l_tau_i := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
			update_class_fields -- Current might have been changed; synchronize
			if (a_w.item.compute_t).to_boolean then
				l_m___4.item := ((a_w.item.H).item.gsl_matrix_submatrix ((l_top + l_N) - {NATURAL_32} 2, (l_top + l_N) - {NATURAL_32} 3, (2).to_natural_32, ((a_w.item.size - l_top) - l_N) + {NATURAL_32} 3)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item := ((a_w.item.H).item.gsl_matrix_submatrix ((0).to_natural_32, (l_top + l_N) - {NATURAL_32} 2, l_top + l_N, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			else
				l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 3, (2).to_natural_32, (3).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, l_N - {NATURAL_32} 2, l_N, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not (a_w.item.Z).is_void) then
				l_m___4.item := ((a_w.item.Z).item.gsl_matrix_submatrix ((0).to_natural_32, (l_top + l_N) - {NATURAL_32} 2, a_w.item.size, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			Result := 0
		end

	francis_search_subdiag_small_elements: NATURAL_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_dpel: REAL_64
			l_tmp: REAL_64
			l_sel: REAL_64
			l_tmp___0: REAL_64
			l_del: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_return: BOOLEAN
		do
			l_N := size1
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2)
			update_class_fields -- Current might have been changed; synchronize
			l_dpel := l_tmp
			l_i := l_N - {NATURAL_32} 1
			from
			until
				not (not (l_return) and then (l_i > {NATURAL_32} 0))
			loop
				l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i - {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_sel := l_tmp___0
				l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_del := l_tmp___1
				if l_sel = 0.0 then
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_i - {NATURAL_32} 1, 0.0)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_i
					l_return := True
				else
					l_tmp___2 := fabs (l_sel)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := fabs (l_del)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___4 := fabs (l_dpel)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___2 < 2.2204460492503131e-16 * (l_tmp___3 + l_tmp___4) then
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_i - {NATURAL_32} 1, 0.0)
						update_class_fields -- Current might have been changed; synchronize
						Result := l_i
						l_return := True
					end
				end
				if not l_return then
					l_dpel := l_del
					l_i := l_i - 1
				end
			end
			if not l_return then
				Result := (0).to_natural_32
			end
		end

	francis_schur_standardize (a_eval1, a_eval2: CE_POINTER [PD_GSL_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_FRANCIS_WORKSPACE])
		require
			not a_eval1.is_void
			not a_eval2.is_void
			not a_w.is_void
		local
			l_N: NATURAL_32
			l_cs: CE_ARRAY [REAL_64]
			l_sn: CE_ARRAY [REAL_64]
			l_top: NATURAL_32
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_yv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_xv___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_yv___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_cs.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_sn.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_yv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_yv___0.make (<<1>>)

			l_N := a_w.item.size
			l_top := (a_w.item.H).item.francis_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
			update_class_fields; -- Current might have been changed; synchronize
			((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.francis_standard_form (l_cs, l_sn)
			update_class_fields -- Current might have been changed; synchronize
			from
			until
				(l_break)
			loop
				l_break := False
				a_eval1.item.dat[0] := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_break := True
			end
			l_break := False
			from
			until
				(l_break)
			loop
				l_break := False
				a_eval2.item.dat[0] := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_break := True
			end
			l_break := False
			l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			if l_tmp___5 = 0.0 then
				from
				until
					(l_break)
				loop
					l_break := False
					a_eval1.item.dat[1] := 0.0
					update_class_fields -- Current might have been changed; synchronize
					l_break := True
				end
				l_break := False
				from
				until
					(l_break)
				loop
					l_break := False
					a_eval2.item.dat[1] := 0.0
					update_class_fields -- Current might have been changed; synchronize
					l_break := True
				end
				l_break := False
			else
				l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___1 := fabs (l_tmp___0)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___2 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___3 := fabs (l_tmp___2)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___4 := sqrt (l_tmp___1 * l_tmp___3)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp := l_tmp___4
				from
				until
					(l_break)
				loop
					l_break := False
					a_eval1.item.dat[1] := l_tmp
					update_class_fields -- Current might have been changed; synchronize
					l_break := True
				end
				l_break := False
				from
				until
					(l_break)
				loop
					l_break := False
					a_eval2.item.dat[1] := -l_tmp
					update_class_fields -- Current might have been changed; synchronize
					l_break := True
				end
				l_break := False
			end
			if (a_w.item.compute_t).to_boolean then
				if l_top < l_N - {NATURAL_32} 2 then
					l_xv.item := ((a_w.item.H).item.gsl_matrix_subrow (l_top, l_top + {NATURAL_32} 2, (l_N - l_top) - {NATURAL_32} 2)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.H).item.gsl_matrix_subrow (l_top + {NATURAL_32} 1, l_top + {NATURAL_32} 2, (l_N - l_top) - {NATURAL_32} 2)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
				if l_top > {NATURAL_32} 0 then
					l_xv.item := ((a_w.item.H).item.gsl_matrix_subcolumn (l_top, (0).to_natural_32, l_top)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.H).item.gsl_matrix_subcolumn (l_top + {NATURAL_32} 1, (0).to_natural_32, l_top)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
			end
			if (not (a_w.item.Z).is_void) then
				l_xv___0.item := ((a_w.item.Z).item.gsl_matrix_column (l_top)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv___0.item := ((a_w.item.Z).item.gsl_matrix_column (l_top + {NATURAL_32} 1)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	francis_get_submatrix (a_B: CE_POINTER [PD_GSL_MATRIX]): NATURAL_32
		require
			not a_B.is_void
		local
			l_diff: NATURAL_32
			l_ratio: REAL_64
			l_top: NATURAL_32
			l_tmp: REAL_64
		do
			l_diff := ((a_B.item.data).distance (data)).to_natural_32
			l_ratio := l_diff.to_real_64 / (tda + {NATURAL_32} 1).to_real_64
			l_tmp := floor (l_ratio)
			update_class_fields -- Current might have been changed; synchronize
			l_top := l_tmp.truncated_to_integer_64.to_natural_32
			Result := l_top
		end

	francis_standard_form (a_cs, a_sn: CE_POINTER [REAL_64])
		require
			not a_cs.is_void
			not a_sn.is_void
		local
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_c___1: REAL_64
			l_d: REAL_64
			l_tmp: REAL_64
			l_p: REAL_64
			l_z: REAL_64
			l_bcmax: REAL_64
			l_bcmis: REAL_64
			l_scale: REAL_64
			l_tau: REAL_64
			l_sigma: REAL_64
			l_cs1: REAL_64
			l_sn1: REAL_64
			l_aa: REAL_64
			l_bb: REAL_64
			l_cc: REAL_64
			l_dd: REAL_64
			l_sab: REAL_64
			l_sac: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___13: INTEGER_32
			l_tmp___14: INTEGER_32
			l_tmp___16: REAL_64
			l_tmp___17: REAL_64
			l_tmp___18: INTEGER_32
			l_tmp___19: REAL_64
			l_tmp___20: REAL_64
			l_tmp___21: REAL_64
			l_tmp___22: REAL_64
			l_tmp___23: INTEGER_32
			l_tmp___24: REAL_64
			l_tmp___25: REAL_64
			l_tmp___26: INTEGER_32
			l_tmp___27: REAL_64
			l_tmp___28: REAL_64
			l_tmp___29: REAL_64
			l_tmp___30: INTEGER_32
			l_tmp___31: INTEGER_32
			l_tmp___32: INTEGER_32
			l_tmp___33: INTEGER_32
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_a___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_b___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_c___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_d := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					if l_c___1 /= 0.0 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						a_cs.item := 1.0
						update_class_fields -- Current might have been changed; synchronize
						a_sn.item := 0.0
						update_class_fields -- Current might have been changed; synchronize
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if l_b___0 /= 0.0 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						a_cs.item := 0.0
						update_class_fields -- Current might have been changed; synchronize
						a_sn.item := 1.0
						update_class_fields -- Current might have been changed; synchronize
						l_tmp := l_d
						l_d := l_a___5
						l_a___5 := l_tmp
						l_b___0 := -l_c___1
						l_c___1 := 0.0
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if l_a___5 - l_d /= 0.0 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						if l_b___0 >= 0.0 then
							l_tmp___32 := 1
						else
							l_tmp___32 := -1
						end
						if l_c___1 >= 0.0 then
							l_tmp___33 := 1
						else
							l_tmp___33 := -1
						end
						if l_tmp___32 /= l_tmp___33 then
							a_cs.item := 1.0
							update_class_fields -- Current might have been changed; synchronize
							a_sn.item := 0.0
							update_class_fields -- Current might have been changed; synchronize
						else
							l_pos := -1 + 9 -- goto _L
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else: | _L:
						l_tmp := l_a___5 - l_d
						l_p := 0.5 * l_tmp
						l_tmp___4 := fabs (l_b___0)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___5 := fabs (l_c___1)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___4 > l_tmp___5 then
							l_tmp___2 := fabs (l_b___0)
							update_class_fields -- Current might have been changed; synchronize
							l_bcmax := l_tmp___2
						else
							l_tmp___3 := fabs (l_c___1)
							update_class_fields -- Current might have been changed; synchronize
							l_bcmax := l_tmp___3
						end
						l_tmp___11 := fabs (l_b___0)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___12 := fabs (l_c___1)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___11 < l_tmp___12 then
							l_tmp___8 := fabs (l_b___0)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___10 := l_tmp___8
						else
							l_tmp___9 := fabs (l_c___1)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___10 := l_tmp___9
						end
						if l_b___0 >= 0.0 then
							l_tmp___13 := 1
						else
							l_tmp___13 := -1
						end
						if l_c___1 >= 0.0 then
							l_tmp___14 := 1
						else
							l_tmp___14 := -1
						end
						l_bcmis := (l_tmp___10 * l_tmp___13.to_double) * l_tmp___14.to_double
						l_tmp___17 := fabs (l_p)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___17 > l_bcmax then
							l_tmp___16 := fabs (l_p)
							update_class_fields -- Current might have been changed; synchronize
							l_scale := l_tmp___16
						else
							l_scale := l_bcmax
						end
						l_z := (l_p / l_scale) * l_p + (l_bcmax / l_scale) * l_bcmis
						if l_z >= 4.0 * 2.2204460492503131e-16 then
							if l_p >= 0.0 then
								l_tmp___18 := 1
							else
								l_tmp___18 := -1
							end
							l_tmp___19 := sqrt (l_scale)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___20 := sqrt (l_z)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___21 := fabs (l_tmp___19 * l_tmp___20)
							update_class_fields -- Current might have been changed; synchronize
							l_z := l_p + l_tmp___18.to_double * l_tmp___21
							l_a___5 := l_d + l_z
							l_d := l_d - ((l_bcmax / l_z) * l_bcmis)
							l_tau := (create {P_HYPOT_STATIC}).gsl_hypot (l_c___1, l_z)
							update_class_fields -- Current might have been changed; synchronize
							a_cs.item := l_z / l_tau
							update_class_fields -- Current might have been changed; synchronize
							a_sn.item := l_c___1 / l_tau
							update_class_fields -- Current might have been changed; synchronize
							l_b___0 := l_b___0 - l_c___1
							l_c___1 := 0.0
						else
							l_sigma := l_b___0 + l_c___1
							l_tau := (create {P_HYPOT_STATIC}).gsl_hypot (l_sigma, l_tmp)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___22 := fabs (l_sigma)
							update_class_fields -- Current might have been changed; synchronize
							a_cs.item := sqrt (0.5 * (1.0 + l_tmp___22 / l_tau))
							update_class_fields -- Current might have been changed; synchronize
							if l_sigma >= 0.0 then
								l_tmp___23 := 1
							else
								l_tmp___23 := -1
							end
							a_sn.item := -(l_p / (l_tau * a_cs.item)) * l_tmp___23.to_double
							update_class_fields -- Current might have been changed; synchronize
							l_aa := l_a___5 * a_cs.item + l_b___0 * a_sn.item
							l_bb := -l_a___5 * a_sn.item + l_b___0 * a_cs.item
							l_cc := l_c___1 * a_cs.item + l_d * a_sn.item
							l_dd := -l_c___1 * a_sn.item + l_d * a_cs.item
							l_a___5 := l_aa * a_cs.item + l_cc * a_sn.item
							l_b___0 := l_bb * a_cs.item + l_dd * a_sn.item
							l_c___1 := -l_aa * a_sn.item + l_cc * a_cs.item
							l_d := -l_bb * a_sn.item + l_dd * a_cs.item
							l_tmp := 0.5 * (l_a___5 + l_d)
							l_d := l_tmp
							l_a___5 := l_d
							if l_c___1 /= 0.0 then
								if l_b___0 /= 0.0 then
									if l_b___0 >= 0.0 then
										l_tmp___30 := 1
									else
										l_tmp___30 := -1
									end
									if l_c___1 >= 0.0 then
										l_tmp___31 := 1
									else
										l_tmp___31 := -1
									end
									if l_tmp___30 = l_tmp___31 then
										l_tmp___24 := fabs (l_b___0)
										update_class_fields -- Current might have been changed; synchronize
										l_sab := sqrt (l_tmp___24)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___25 := fabs (l_c___1)
										update_class_fields -- Current might have been changed; synchronize
										l_sac := sqrt (l_tmp___25)
										update_class_fields -- Current might have been changed; synchronize
										if l_c___1 >= 0.0 then
											l_tmp___26 := 1
										else
											l_tmp___26 := -1
										end
										l_tmp___27 := fabs (l_sab * l_sac)
										update_class_fields -- Current might have been changed; synchronize
										l_p := l_tmp___26.to_double * l_tmp___27
										l_tmp___28 := fabs (l_b___0 + l_c___1)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___29 := sqrt (l_tmp___28)
										update_class_fields -- Current might have been changed; synchronize
										l_tau := 1.0 / l_tmp___29
										l_a___5 := l_tmp + l_p
										l_d := l_tmp - l_p
										l_b___0 := l_b___0 - l_c___1
										l_c___1 := 0.0
										l_cs1 := l_sab * l_tau
										l_sn1 := l_sac * l_tau
										l_tmp := a_cs.item * l_cs1 - a_sn.item * l_sn1
										a_sn.item := a_cs.item * l_sn1 + a_sn.item * l_cs1
										update_class_fields -- Current might have been changed; synchronize
										a_cs.item := l_tmp
										update_class_fields -- Current might have been changed; synchronize
									end
								else
									l_b___0 := -l_c___1
									l_c___1 := 0.0
									l_tmp := a_cs.item
									a_cs.item := -a_sn.item
									update_class_fields -- Current might have been changed; synchronize
									a_sn.item := l_tmp
									update_class_fields -- Current might have been changed; synchronize
								end
							end
						end
				when 10 then -- if done | if done;
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, l_a___5)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((0).to_natural_32, (1).to_natural_32, l_b___0)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((1).to_natural_32, (0).to_natural_32, l_c___1)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((1).to_natural_32, (1).to_natural_32, l_d)
					update_class_fields -- Current might have been changed; synchronize
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_eigen_genv (a_B: CE_POINTER [PD_GSL_MATRIX]; a_alpha: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_beta: CE_POINTER [PD_GSL_VECTOR]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_GENV_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_s: INTEGER_32
			l_Z: CE_ARRAY [PD_GSL_MATRIX]
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX]} l_Z.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := size1
					if l_N = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("genv.c"), 174, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 3 then -- else:
						if l_N = a_B.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L___0
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 6 then -- else:
						if l_N = a_B.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("B matrix dimensions must match A"), ce_string ("genv.c"), 178, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 9 then -- else:
						if a_alpha.item.size = l_N then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 12 then -- else:
						if a_beta.item.size = l_N then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("genv.c"), 182, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 15 then -- else:
						if a_w.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size does not match workspace"), ce_string ("genv.c"), 186, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_evec.item.size1 /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvector matrix has wrong size"), ce_string ("genv.c"), 190, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_Z.item.size1 := l_N
							update_class_fields -- Current might have been changed; synchronize
							l_Z.item.size2 := l_N
							update_class_fields -- Current might have been changed; synchronize
							l_Z.item.tda := {NATURAL_32} 2 * l_N
							update_class_fields -- Current might have been changed; synchronize
							l_Z.item.data := a_evec.item.data
							update_class_fields -- Current might have been changed; synchronize
							l_Z.item.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
							update_class_fields -- Current might have been changed; synchronize
							l_Z.item.owner := 0
							update_class_fields -- Current might have been changed; synchronize
							l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_gen_QZ (a_B, a_alpha, a_beta, a_w.item.Q, l_Z, a_w.item.gen_workspace_p)
							update_class_fields -- Current might have been changed; synchronize
							if (not (a_w.item.Z).is_void) then
								(a_w.item.Z).item.gsl_matrix_memcpy (l_Z).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							end
							if l_s = 0 then
								l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.genv_get_right_eigenvectors (a_B, l_Z, a_evec, a_w)
								update_class_fields -- Current might have been changed; synchronize
								if l_s = 0 then
									genv_normalize_eigenvectors (a_alpha, a_evec)
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							Result := l_s
							l_return := True
						end
				when 16 then -- if done | if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 17 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_eigen_genv_QZ (a_B: CE_POINTER [PD_GSL_MATRIX]; a_alpha: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_beta: CE_POINTER [PD_GSL_VECTOR]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_Q___1, a_Z: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_GENV_WORKSPACE]): INTEGER_32
		local
			l_s: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_Q___1.is_void then
						l_pos := -1 + 9 -- skip if block
					end
				when 1 then -- if:
						if size1 = a_Q___1.item.size1 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L___2
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else:
						if size1 = a_Q___1.item.size2 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if: | _L___2:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Q matrix has wrong dimensions"), ce_string ("genv.c"), 268, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else:
						l_pos := -1 + 9 -- goto _L___1
				when 8 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 19
				when 9 then -- else: | _L___1:
						if a_Z.is_void then
							l_pos := -1 + 18 -- skip if block
						end
				when 10 then -- if:
						if size1 = a_Z.item.size1 then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						l_pos := -1 + 14 -- goto _L___0
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 13 then -- else:
						if size1 = a_Z.item.size2 then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Z matrix has wrong dimensions"), ce_string ("genv.c"), 272, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 16 then -- else:
						l_pos := -1 + 18 -- goto _L
				when 17 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 19
				when 18 then -- else: | _L:
						a_w.item.Q := a_Q___1
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.Z := a_Z
						update_class_fields -- Current might have been changed; synchronize
						l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_genv (a_B, a_alpha, a_beta, a_evec, a_w)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.Q := create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.Z := create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer)
						update_class_fields -- Current might have been changed; synchronize
						Result := l_s
						l_return := True
				when 19 then -- if done
					if not l_return then
						Result := 0
					end
				when 20 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	genv_get_right_eigenvectors (a_T, a_Z: CE_POINTER [PD_GSL_MATRIX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_GENV_WORKSPACE]): INTEGER_32
		require
			not a_T.is_void
			not a_w.is_void
		local
			l_N: NATURAL_32
			l_small___0: REAL_64
			l_big: REAL_64
			l_bignum: REAL_64
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_end: NATURAL_32
			l_is: INTEGER_32
			l_anorm: REAL_64
			l_bnorm: REAL_64
			l_temp: CE_ARRAY [REAL_64]
			l_temp2: CE_ARRAY [REAL_64]
			l_temp2r: REAL_64
			l_temp2i: REAL_64
			l_ascale: REAL_64
			l_bscale: REAL_64
			l_salfar: REAL_64
			l_sbeta: REAL_64
			l_acoef: CE_ARRAY [REAL_64]
			l_bcoefr: CE_ARRAY [REAL_64]
			l_bcoefi: CE_ARRAY [REAL_64]
			l_acoefa: REAL_64
			l_bcoefa: REAL_64
			l_creala: REAL_64
			l_cimaga: REAL_64
			l_crealb: REAL_64
			l_cimagb: REAL_64
			l_cre2a: REAL_64
			l_cim2a: REAL_64
			l_cre2b: REAL_64
			l_cim2b: REAL_64
			l_dmin: REAL_64
			l_xmax: REAL_64
			l_scale: CE_ARRAY [REAL_64]
			l_nw: NATURAL_32
			l_na: NATURAL_32
			l_lsa: INTEGER_32
			l_lsb: INTEGER_32
			l_complex_pair: INTEGER_32
			l_z_zero: PD_GSL_COMPLEX
			l_z_one: PD_GSL_COMPLEX
			l_bdiag: CE_ARRAY [REAL_64]
			l_sum: CE_ARRAY [REAL_64]
			l_il2by2: INTEGER_32
			l_jr: NATURAL_32
			l_jc: NATURAL_32
			l_ja: NATURAL_32
			l_xscale: REAL_64
			l_ecol: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_re: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_im: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_re2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_im2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: NATURAL_32
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_je: NATURAL_32
			l_tmp___15: REAL_64
			l_tmp___16: REAL_64
			l_tmp___17: REAL_64
			l_tmp___18: REAL_64
			l_tmp___19: REAL_64
			l_tmp___37: REAL_64
			l_tmp___38: REAL_64
			l_tmp___39: REAL_64
			l_tmp___40: REAL_64
			l_tmp___41: REAL_64
			l_tmp___42: REAL_64
			l_tmp___43: REAL_64
			l_tmp___44: REAL_64
			l_tmp___45: REAL_64
			l_tmp___46: REAL_64
			l_tmp___51: REAL_64
			l_tmp___52: REAL_64
			l_tmp___53: REAL_64
			l_tmp___54: REAL_64
			l_tmp___55: REAL_64
			l_tmp___56: REAL_64
			l_tmp___57: REAL_64
			l_tmp___58: REAL_64
			l_tmp___59: REAL_64
			l_tmp___60: REAL_64
			l_tmp___61: REAL_64
			l_tmp___62: REAL_64
			l_tmp___63: REAL_64
			l_tmp___64: INTEGER_32
			l_tmp___65: REAL_64
			l_tmp___66: REAL_64
			l_tmp___67: INTEGER_32
			l_tmp___68: REAL_64
			l_tmp___69: REAL_64
			l_tmp___72: REAL_64
			l_tmp___73: REAL_64
			l_tmp___74: REAL_64
			l_tmp___75: REAL_64
			l_tmp___107: REAL_64
			l_tmp___108: REAL_64
			l_tmp___109: REAL_64
			l_tmp___110: REAL_64
			l_tmp___111: REAL_64
			l_tmp___112: REAL_64
			l_tmp___115: REAL_64
			l_tmp___116: REAL_64
			l_tmp___117: REAL_64
			l_tmp___118: REAL_64
			l_tmp___119: REAL_64
			l_tmp___129: REAL_64
			l_tmp___130: REAL_64
			l_tmp___131: REAL_64
			l_tmp___132: REAL_64
			l_tmp___133: REAL_64
			l_tmp___134: REAL_64
			l_tmp___137: REAL_64
			l_tmp___138: REAL_64
			l_tmp___139: REAL_64
			l_tmp___140: REAL_64
			l_tmp___141: REAL_64
			l_tmp___142: REAL_64
			l_tmp___143: REAL_64
			l_vs: CE_ARRAY [PD_GSL_MATRIX_CONST_VIEW]
			l_tmp___144: PD_GSL_MATRIX_CONST_VIEW
			l_vt: CE_ARRAY [PD_GSL_MATRIX_CONST_VIEW]
			l_tmp___145: PD_GSL_MATRIX_CONST_VIEW
			l_tmp___146: REAL_64
			l_tmp___147: REAL_64
			l_tmp___148: REAL_64
			l_tmp___149: REAL_64
			l_tmp___150: REAL_64
			l_tmp___151: REAL_64
			l_tmp___152: REAL_64
			l_tmp___153: REAL_64
			l_tmp___154: REAL_64
			l_tmp___155: REAL_64
			l_tmp___156: REAL_64
			l_tmp___157: REAL_64
			l_tmp___158: REAL_64
			l_tmp___159: REAL_64
			l_tmp___160: REAL_64
			l_tmp___169: REAL_64
			l_tmp___170: REAL_64
			l_tmp___171: REAL_64
			l_tmp___172: REAL_64
			l_tmp___173: REAL_64
			l_tmp___174: REAL_64
			l_tmp___175: REAL_64
			l_tmp___176: REAL_64
			l_tmp___177: REAL_64
			l_tmp___178: REAL_64
			l_tmp___179: REAL_64
			l_tmp___180: REAL_64
			l_tmp___181: REAL_64
			l_tmp___182: REAL_64
			l_tmp___183: REAL_64
			l_tmp___184: REAL_64
			l_tmp___185: REAL_64
			l_tmp___186: REAL_64
			l_tmp___187: REAL_64
			l_tmp___188: REAL_64
			l_tmp___189: REAL_64
			l_tmp___190: REAL_64
			l_tmp___191: REAL_64
			l_tmp___192: REAL_64
			l_tmp___193: REAL_64
			l_tmp___194: REAL_64
			l_tmp___195: REAL_64
			l_tmp___196: REAL_64
			l_tmp___197: REAL_64
			l_tmp___198: REAL_64
			l_tmp___199: REAL_64
			l_tmp___200: REAL_64
			l_tmp___201: REAL_64
			l_tmp___202: REAL_64
			l_tmp___203: REAL_64
			l_tmp___204: REAL_64
			l_tmp___206: REAL_64
			l_tmp___207: REAL_64
			l_tmp___208: REAL_64
			l_sv: CE_ARRAY [PD_GSL_MATRIX_CONST_VIEW]
			l_tmp___209: PD_GSL_MATRIX_CONST_VIEW
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_bv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_bdat: CE_ARRAY [REAL_64]
			l_sv___0: CE_ARRAY [PD_GSL_MATRIX_CONST_VIEW]
			l_tmp___210: PD_GSL_MATRIX_CONST_VIEW
			l_xv___0: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_tmp___211: PD_GSL_VECTOR_COMPLEX_VIEW
			l_bv___0: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_tmp___212: PD_GSL_VECTOR_COMPLEX_VIEW
			l_z: CE_ARRAY [PD_GSL_COMPLEX]
			l_tmp___213: REAL_64
			l_tmp___214: REAL_64
			l_tmp___215: REAL_64
			l_tmp___216: REAL_64
			l_tmp___217: REAL_64
			l_tmp___220: REAL_64
			l_tmp___221: REAL_64
			l_tmp___222: REAL_64
			l_tmp___223: REAL_64
			l_tmp___225: REAL_64
			l_tmp___226: REAL_64
			l_tmp___227: REAL_64
			l_tmp___228: REAL_64
			l_tmp___229: REAL_64
			l_tmp___230: REAL_64
			l_tmp___231: REAL_64
			l_tmp___232: REAL_64
			l_tmp___233: REAL_64
			l_tmp___234: REAL_64
			l_tmp___235: REAL_64
			l_tmp___236: REAL_64
			l_tmp___237: REAL_64
			l_tmp___238: REAL_64
			l_tmp___239: REAL_64
			l_tmp___240: REAL_64
			l_tmp___241: REAL_64
			l_tmp___242: REAL_64
			l_tmp___243: REAL_64
			l_tmp___244: REAL_64
			l_tmp___245: REAL_64
			l_tmp___246: REAL_64
			l_tmp___247: REAL_64
			l_tmp___248: REAL_64
			l_tmp___249: REAL_64
			l_tmp___250: REAL_64
			l_tmp___251: REAL_64
			l_tmp___252: REAL_64
			l_tmp___253: REAL_64
			l_tmp___254: REAL_64
			l_tmp___255: REAL_64
			l_tmp___256: REAL_64
			l_tmp___257: REAL_64
			l_tmp___258: REAL_64
			l_tmp___259: REAL_64
			l_tmp___264: REAL_64
			l_tmp___265: REAL_64
			l_tmp___266: REAL_64
			l_tmp___267: REAL_64
			l_tmp___268: REAL_64
			l_tmp___269: REAL_64
			l_tmp___270: REAL_64
			l_tmp___271: REAL_64
			l_tmp___274: REAL_64
			l_tmp___275: REAL_64
			l_tmp___276: REAL_64
			l_tmp___277: REAL_64
			l_tmp___278: REAL_64
			l_tmp___279: REAL_64
			l_tmp___280: REAL_64
			l_tmp___281: REAL_64
			l_return, l_break, l_continue: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_temp.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_temp2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_acoef.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_bcoefr.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_bcoefi.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_scale.make (<<1>>)
			create l_z_zero.make
			create l_z_one.make
			create {CE_REAL_64_ARRAY} l_bdiag.make (<<2>>)
			create {CE_REAL_64_ARRAY} l_sum.make (<<4>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]} l_ecol.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_re.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_im.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_re2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_im2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_CONST_VIEW]} l_vs.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_CONST_VIEW]} l_vt.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_CONST_VIEW]} l_sv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_bv.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_bdat.make (<<4>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_CONST_VIEW]} l_sv___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]} l_xv___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]} l_bv___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_COMPLEX]} l_z.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := a_w.item.size
					l_small___0 := (2.2250738585072014e-308 * l_N.to_real_64) / 2.2204460492503131e-16
					l_big := 1.0 / l_small___0
					l_bignum := 1.0 / (2.2250738585072014e-308 * l_N.to_real_64)
					l_bdiag[0] := 0.0
					update_class_fields -- Current might have been changed; synchronize
					l_bdiag[1] := 0.0
					update_class_fields -- Current might have been changed; synchronize
					from
					until
						(l_break)
					loop
						l_break := False
						l_z_zero.dat[0] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_z_zero.dat[1] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_break := True
					end
					l_break := False
					from
					until
						(l_break)
					loop
						l_break := False
						l_z_one.dat[0] := 1.0
						update_class_fields -- Current might have been changed; synchronize
						l_z_one.dat[1] := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_break := True
					end
					l_break := False
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_anorm := fabs (l_tmp)
					update_class_fields -- Current might have been changed; synchronize
					if l_N > {NATURAL_32} 1 then
						l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := fabs (l_tmp___0)
						update_class_fields -- Current might have been changed; synchronize
						l_anorm := l_anorm + l_tmp___1
					end
					l_tmp___2 := a_T.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_bnorm := fabs (l_tmp___2)
					update_class_fields; -- Current might have been changed; synchronize
					(a_w.item.work1).item.gsl_vector_set ((0).to_natural_32, 0.0)
					update_class_fields; -- Current might have been changed; synchronize
					(a_w.item.work2).item.gsl_vector_set ((0).to_natural_32, 0.0)
					update_class_fields -- Current might have been changed; synchronize
					l_j := (1).to_natural_32
					from
					until
						not (not (l_return) and then (l_j < l_N))
					loop
						l_temp2.item := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_temp.item := l_temp2.item
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j - {NATURAL_32} 1)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___3 = 0.0 then
							l_end := l_j
						else
							l_end := l_j - {NATURAL_32} 1
						end
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_end)
						loop
							l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___5 := fabs (l_tmp___4)
							update_class_fields -- Current might have been changed; synchronize
							l_temp.item := l_temp.item + l_tmp___5
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___6 := a_T.item.gsl_matrix_get (l_i, l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___7 := fabs (l_tmp___6)
							update_class_fields -- Current might have been changed; synchronize
							l_temp2.item := l_temp2.item + l_tmp___7
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						(a_w.item.work1).item.gsl_vector_set (l_j, l_temp.item)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.work2).item.gsl_vector_set (l_j, l_temp2.item)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_end
						from
						until
							(l_break)
						loop
							l_break := False
							if l_j + {NATURAL_32} 2 < l_N then
								l_tmp___12 := l_j + {NATURAL_32} 2
							else
								l_tmp___12 := l_N
							end
							if not (l_i < l_tmp___12) then
								l_break := True
							end
							if not (l_break) then
								l_tmp___8 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___9 := fabs (l_tmp___8)
								update_class_fields -- Current might have been changed; synchronize
								l_temp.item := l_temp.item + l_tmp___9
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___10 := a_T.item.gsl_matrix_get (l_i, l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___11 := fabs (l_tmp___10)
								update_class_fields -- Current might have been changed; synchronize
								l_temp2.item := l_temp2.item + l_tmp___11
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
						end
						l_break := False
						if l_anorm > l_temp.item then
							l_anorm := l_anorm
						else
							l_anorm := l_temp.item
						end
						if l_bnorm > l_temp2.item then
							l_bnorm := l_bnorm
						else
							l_bnorm := l_temp2.item
						end
						l_j := l_j + 1
					end
					if l_anorm > 2.2250738585072014e-308 then
						l_tmp___13 := l_anorm
					else
						l_tmp___13 := 2.2250738585072014e-308
					end
					l_ascale := 1.0 / l_tmp___13
					if l_bnorm > 2.2250738585072014e-308 then
						l_tmp___14 := l_bnorm
					else
						l_tmp___14 := 2.2250738585072014e-308
					end
					l_bscale := 1.0 / l_tmp___14
					l_complex_pair := 0
					l_k := (0).to_natural_32
				when 1 then -- loop condition:
					if not (not (l_return) and then (l_k < l_N)) then
						l_pos := -1 + 14 -- skip loop
					end
				when 2 then -- loop body:
						l_continue := False
						l_je := (l_N - {NATURAL_32} 1) - l_k
						if l_complex_pair.to_boolean then
							l_complex_pair := 0
							l_continue := True
						end
						if not (l_return or l_break or l_continue) then
							l_nw := (1).to_natural_32
							if l_je > {NATURAL_32} 0 then
								l_tmp___15 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je, l_je - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___15 /= 0.0 then
									l_complex_pair := 1
									l_nw := (2).to_natural_32
								end
							end
							if not l_complex_pair.to_boolean then
								l_tmp___16 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je, l_je)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___17 := fabs (l_tmp___16)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___17 <= 2.2250738585072014e-308 then
									l_tmp___18 := a_T.item.gsl_matrix_get (l_je, l_je)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___19 := fabs (l_tmp___18)
									update_class_fields -- Current might have been changed; synchronize
									if l_tmp___19 <= 2.2250738585072014e-308 then
										l_i := (0).to_natural_32
										from
										until
											not (l_i < l_N)
										loop
											a_evec.item.gsl_matrix_complex_set (l_i, l_je, l_z_zero.twin)
											update_class_fields -- Current might have been changed; synchronize
											l_i := l_i + 1
										end
										a_evec.item.gsl_matrix_complex_set (l_je, l_je, l_z_one.twin)
										update_class_fields -- Current might have been changed; synchronize
										l_continue := True
									end
								end
								if not (l_return or l_break or l_continue) then
									l_i := (0).to_natural_32
									from
									until
										not (l_i < l_N)
									loop
										(a_w.item.work3).item.gsl_vector_set (l_i, 0.0)
										update_class_fields -- Current might have been changed; synchronize
										l_i := l_i + 1
									end
								end
							else
								l_i := (0).to_natural_32
								from
								until
									not (l_i < l_N)
								loop
									(a_w.item.work3).item.gsl_vector_set (l_i, 0.0)
									update_class_fields; -- Current might have been changed; synchronize
									(a_w.item.work4).item.gsl_vector_set (l_i, 0.0)
									update_class_fields -- Current might have been changed; synchronize
									l_i := l_i + 1
								end
							end
						end
						if (l_return or l_break or l_continue) then
							l_pos := -1 + 13 -- skip if block
						end
				when 3 then -- if:
						if l_complex_pair.to_boolean then
							l_pos := -1 + 11 -- skip if block
						end
				when 4 then -- if:
						l_tmp___56 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je, l_je)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___57 := fabs (l_tmp___56)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___58 := a_T.item.gsl_matrix_get (l_je, l_je)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___59 := fabs (l_tmp___58)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___57 * l_ascale > l_tmp___59 * l_bscale then
							l_tmp___51 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je, l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___52 := fabs (l_tmp___51)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___55 := l_tmp___52 * l_ascale
						else
							l_tmp___53 := a_T.item.gsl_matrix_get (l_je, l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___54 := fabs (l_tmp___53)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___55 := l_tmp___54 * l_bscale
						end
						if 2.2250738585072014e-308 > l_tmp___55 then
							l_tmp___46 := 2.2250738585072014e-308
						else
							l_tmp___42 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je, l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___43 := fabs (l_tmp___42)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___44 := a_T.item.gsl_matrix_get (l_je, l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___45 := fabs (l_tmp___44)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___43 * l_ascale > l_tmp___45 * l_bscale then
								l_tmp___37 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je, l_je)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___38 := fabs (l_tmp___37)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___41 := l_tmp___38 * l_ascale
							else
								l_tmp___39 := a_T.item.gsl_matrix_get (l_je, l_je)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___40 := fabs (l_tmp___39)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___41 := l_tmp___40 * l_bscale
							end
							l_tmp___46 := l_tmp___41
						end
						l_temp.item := 1.0 / l_tmp___46
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___60 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je, l_je)
						update_class_fields -- Current might have been changed; synchronize
						l_salfar := (l_temp.item * l_tmp___60) * l_ascale
						l_tmp___61 := a_T.item.gsl_matrix_get (l_je, l_je)
						update_class_fields -- Current might have been changed; synchronize
						l_sbeta := (l_temp.item * l_tmp___61) * l_bscale
						l_acoef.item := l_sbeta * l_ascale
						update_class_fields -- Current might have been changed; synchronize
						l_bcoefr.item := l_salfar * l_bscale
						update_class_fields -- Current might have been changed; synchronize
						l_bcoefi.item := 0.0
						update_class_fields -- Current might have been changed; synchronize
						l_scale.item := 1.0
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___62 := fabs (l_sbeta)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___62 >= 2.2250738585072014e-308 then
							l_tmp___63 := fabs (l_acoef.item)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___63 < l_small___0 then
								l_tmp___64 := 1
							else
								l_tmp___64 := 0
							end
						else
							l_tmp___64 := 0
						end
						l_lsa := l_tmp___64
						l_tmp___65 := fabs (l_salfar)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___65 >= 2.2250738585072014e-308 then
							l_tmp___66 := fabs (l_bcoefr.item)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___66 < l_small___0 then
								l_tmp___67 := 1
							else
								l_tmp___67 := 0
							end
						else
							l_tmp___67 := 0
						end
						l_lsb := l_tmp___67
						if l_lsa.to_boolean then
							l_tmp___68 := fabs (l_sbeta)
							update_class_fields -- Current might have been changed; synchronize
							if l_anorm < l_big then
								l_tmp___69 := l_anorm
							else
								l_tmp___69 := l_big
							end
							l_scale.item := (l_small___0 / l_tmp___68) * l_tmp___69
							update_class_fields -- Current might have been changed; synchronize
						end
						if l_lsb.to_boolean then
							l_tmp___74 := fabs (l_salfar)
							update_class_fields -- Current might have been changed; synchronize
							if l_bnorm < l_big then
								l_tmp___75 := l_bnorm
							else
								l_tmp___75 := l_big
							end
							if l_scale.item > (l_small___0 / l_tmp___74) * l_tmp___75 then
								l_scale.item := l_scale.item
								update_class_fields -- Current might have been changed; synchronize
							else
								l_tmp___72 := fabs (l_salfar)
								update_class_fields -- Current might have been changed; synchronize
								if l_bnorm < l_big then
									l_tmp___73 := l_bnorm
								else
									l_tmp___73 := l_big
								end
								l_scale.item := (l_small___0 / l_tmp___72) * l_tmp___73
								update_class_fields -- Current might have been changed; synchronize
							end
						end
						if not (l_lsa.to_boolean) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						l_pos := -1 + 8 -- goto _L
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 7 then -- else:
						if not (l_lsb.to_boolean) then
							l_pos := -1 + 9 -- skip if block
						end
				when 8 then -- if: | _L:
						l_tmp___140 := fabs (l_acoef.item)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___141 := fabs (l_bcoefr.item)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___140 > l_tmp___141 then
							l_tmp___137 := fabs (l_acoef.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___139 := l_tmp___137
						else
							l_tmp___138 := fabs (l_bcoefr.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___139 := l_tmp___138
						end
						if 1.0 > l_tmp___139 then
							l_tmp___134 := 1.0
						else
							l_tmp___132 := fabs (l_acoef.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___133 := fabs (l_bcoefr.item)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___132 > l_tmp___133 then
								l_tmp___129 := fabs (l_acoef.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___131 := l_tmp___129
							else
								l_tmp___130 := fabs (l_bcoefr.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___131 := l_tmp___130
							end
							l_tmp___134 := l_tmp___131
						end
						if l_scale.item < 1.0 / (2.2250738585072014e-308 * l_tmp___134) then
							l_scale.item := l_scale.item
							update_class_fields -- Current might have been changed; synchronize
						else
							l_tmp___118 := fabs (l_acoef.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___119 := fabs (l_bcoefr.item)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___118 > l_tmp___119 then
								l_tmp___115 := fabs (l_acoef.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___117 := l_tmp___115
							else
								l_tmp___116 := fabs (l_bcoefr.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___117 := l_tmp___116
							end
							if 1.0 > l_tmp___117 then
								l_tmp___112 := 1.0
							else
								l_tmp___110 := fabs (l_acoef.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___111 := fabs (l_bcoefr.item)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___110 > l_tmp___111 then
									l_tmp___107 := fabs (l_acoef.item)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___109 := l_tmp___107
								else
									l_tmp___108 := fabs (l_bcoefr.item)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___109 := l_tmp___108
								end
								l_tmp___112 := l_tmp___109
							end
							l_scale.item := 1.0 / (2.2250738585072014e-308 * l_tmp___112)
							update_class_fields -- Current might have been changed; synchronize
						end
						if l_lsa.to_boolean then
							l_acoef.item := l_ascale * (l_scale.item * l_sbeta)
							update_class_fields -- Current might have been changed; synchronize
						else
							l_acoef.item := l_acoef.item * (l_scale.item)
							update_class_fields -- Current might have been changed; synchronize
						end
						if l_lsb.to_boolean then
							l_bcoefr.item := l_bscale * (l_scale.item * l_salfar)
							update_class_fields -- Current might have been changed; synchronize
						else
							l_bcoefr.item := l_bcoefr.item * (l_scale.item)
							update_class_fields -- Current might have been changed; synchronize
						end
				when 9 then -- if done
						l_acoefa := fabs (l_acoef.item)
						update_class_fields -- Current might have been changed; synchronize
						l_bcoefa := fabs (l_bcoefr.item)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.work3).item.gsl_vector_set (l_je, 1.0)
						update_class_fields -- Current might have been changed; synchronize
						l_xmax := 1.0
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_je)
						loop
							l_tmp___142 := a_T.item.gsl_matrix_get (l_i, l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___143 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_je)
							update_class_fields; -- Current might have been changed; synchronize
							(a_w.item.work3).item.gsl_vector_set (l_i, l_bcoefr.item * l_tmp___142 - l_acoef.item * l_tmp___143)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 11 then -- else:
						l_tmp___144 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_submatrix (l_je - {NATURAL_32} 1, l_je - {NATURAL_32} 1, (2).to_natural_32, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_vs.item := l_tmp___144.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___145 := (a_T.item.gsl_matrix_const_submatrix (l_je - {NATURAL_32} 1, l_je - {NATURAL_32} 1, (2).to_natural_32, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_vt.item := l_tmp___145.twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vs.item.c_pointer + (create {PD_GSL_MATRIX_CONST_VIEW}.make).offset_of ("matrix"))).item.gsl_schur_gen_eigvals ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vt.item.c_pointer + (create {PD_GSL_MATRIX_CONST_VIEW}.make).offset_of ("matrix"))), l_bcoefr, l_temp2, l_bcoefi, l_acoef, l_temp).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						if l_bcoefi.item = 0.0 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_schur_gen_eigvals failed on complex block"), ce_string ("genv.c"), 514, -1)
								update_class_fields -- Current might have been changed; synchronize
								Result := (-1)
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
						if not l_return then
							l_acoefa := fabs (l_acoef.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___146 := fabs (l_bcoefr.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___147 := fabs (l_bcoefi.item)
							update_class_fields -- Current might have been changed; synchronize
							l_bcoefa := l_tmp___146 + l_tmp___147
							l_scale.item := 1.0
							update_class_fields -- Current might have been changed; synchronize
							if l_acoefa * 2.2204460492503131e-16 < 2.2250738585072014e-308 then
								if l_acoefa >= 2.2250738585072014e-308 then
									l_scale.item := (2.2250738585072014e-308 / 2.2204460492503131e-16) / l_acoefa
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							if l_bcoefa * 2.2204460492503131e-16 < 2.2250738585072014e-308 then
								if l_bcoefa >= 2.2250738585072014e-308 then
									if l_scale.item > (2.2250738585072014e-308 / 2.2204460492503131e-16) / l_bcoefa then
										l_scale.item := l_scale.item
										update_class_fields -- Current might have been changed; synchronize
									else
										l_scale.item := (2.2250738585072014e-308 / 2.2204460492503131e-16) / l_bcoefa
										update_class_fields -- Current might have been changed; synchronize
									end
								end
							end
							if 2.2250738585072014e-308 * l_acoefa > l_ascale then
								l_scale.item := l_ascale / (2.2250738585072014e-308 * l_acoefa)
								update_class_fields -- Current might have been changed; synchronize
							end
							if 2.2250738585072014e-308 * l_bcoefa > l_bscale then
								if l_scale.item < l_bscale / (2.2250738585072014e-308 * l_bcoefa) then
									l_scale.item := l_scale.item
									update_class_fields -- Current might have been changed; synchronize
								else
									l_scale.item := l_bscale / (2.2250738585072014e-308 * l_bcoefa)
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							if l_scale.item /= 1.0 then
								l_acoef.item := l_acoef.item * (l_scale.item)
								update_class_fields -- Current might have been changed; synchronize
								l_acoefa := fabs (l_acoef.item)
								update_class_fields -- Current might have been changed; synchronize
								l_bcoefr.item := l_bcoefr.item * (l_scale.item)
								update_class_fields -- Current might have been changed; synchronize
								l_bcoefi.item := l_bcoefi.item * (l_scale.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___148 := fabs (l_bcoefr.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___149 := fabs (l_bcoefi.item)
								update_class_fields -- Current might have been changed; synchronize
								l_bcoefa := l_tmp___148 + l_tmp___149
							end
							l_tmp___150 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je, l_je - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_temp.item := l_acoef.item * l_tmp___150
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___151 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je, l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___152 := a_T.item.gsl_matrix_get (l_je, l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_temp2r := l_acoef.item * l_tmp___151 - l_bcoefr.item * l_tmp___152
							l_tmp___153 := a_T.item.gsl_matrix_get (l_je, l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_temp2i := -l_bcoefi.item * l_tmp___153
							l_tmp___158 := fabs (l_temp.item)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___159 := fabs (l_temp2r)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___160 := fabs (l_temp2i)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___158 >= l_tmp___159 + l_tmp___160 then
								(a_w.item.work3).item.gsl_vector_set (l_je, 1.0)
								update_class_fields; -- Current might have been changed; synchronize
								(a_w.item.work4).item.gsl_vector_set (l_je, 0.0)
								update_class_fields; -- Current might have been changed; synchronize
								(a_w.item.work3).item.gsl_vector_set (l_je - {NATURAL_32} 1, -l_temp2r / l_temp.item)
								update_class_fields; -- Current might have been changed; synchronize
								(a_w.item.work4).item.gsl_vector_set (l_je - {NATURAL_32} 1, -l_temp2i / l_temp.item)
								update_class_fields -- Current might have been changed; synchronize
							else
								(a_w.item.work3).item.gsl_vector_set (l_je - {NATURAL_32} 1, 1.0)
								update_class_fields; -- Current might have been changed; synchronize
								(a_w.item.work4).item.gsl_vector_set (l_je - {NATURAL_32} 1, 0.0)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___154 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je - {NATURAL_32} 1, l_je)
								update_class_fields -- Current might have been changed; synchronize
								l_temp.item := l_acoef.item * l_tmp___154
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___155 := a_T.item.gsl_matrix_get (l_je - {NATURAL_32} 1, l_je - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___156 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_je - {NATURAL_32} 1, l_je - {NATURAL_32} 1)
								update_class_fields; -- Current might have been changed; synchronize
								(a_w.item.work3).item.gsl_vector_set (l_je, (l_bcoefr.item * l_tmp___155 - l_acoef.item * l_tmp___156) / l_temp.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___157 := a_T.item.gsl_matrix_get (l_je - {NATURAL_32} 1, l_je - {NATURAL_32} 1)
								update_class_fields; -- Current might have been changed; synchronize
								(a_w.item.work4).item.gsl_vector_set (l_je, (l_bcoefi.item * l_tmp___157) / l_temp.item)
								update_class_fields -- Current might have been changed; synchronize
							end
							l_tmp___177 := (a_w.item.work3).item.gsl_vector_get (l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___178 := fabs (l_tmp___177)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___179 := (a_w.item.work4).item.gsl_vector_get (l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___180 := fabs (l_tmp___179)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___181 := (a_w.item.work3).item.gsl_vector_get (l_je - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___182 := fabs (l_tmp___181)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___183 := (a_w.item.work4).item.gsl_vector_get (l_je - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___184 := fabs (l_tmp___183)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___178 + l_tmp___180 > l_tmp___182 + l_tmp___184 then
								l_tmp___169 := (a_w.item.work3).item.gsl_vector_get (l_je)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___170 := fabs (l_tmp___169)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___171 := (a_w.item.work4).item.gsl_vector_get (l_je)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___172 := fabs (l_tmp___171)
								update_class_fields -- Current might have been changed; synchronize
								l_xmax := l_tmp___170 + l_tmp___172
							else
								l_tmp___173 := (a_w.item.work3).item.gsl_vector_get (l_je - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___174 := fabs (l_tmp___173)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___175 := (a_w.item.work4).item.gsl_vector_get (l_je - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___176 := fabs (l_tmp___175)
								update_class_fields -- Current might have been changed; synchronize
								l_xmax := l_tmp___174 + l_tmp___176
							end
							l_tmp___185 := (a_w.item.work3).item.gsl_vector_get (l_je - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_creala := l_acoef.item * l_tmp___185
							l_tmp___186 := (a_w.item.work4).item.gsl_vector_get (l_je - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_cimaga := l_acoef.item * l_tmp___186
							l_tmp___187 := (a_w.item.work3).item.gsl_vector_get (l_je - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___188 := (a_w.item.work4).item.gsl_vector_get (l_je - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_crealb := l_bcoefr.item * l_tmp___187 - l_bcoefi.item * l_tmp___188
							l_tmp___189 := (a_w.item.work3).item.gsl_vector_get (l_je - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___190 := (a_w.item.work4).item.gsl_vector_get (l_je - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_cimagb := l_bcoefi.item * l_tmp___189 + l_bcoefr.item * l_tmp___190
							l_tmp___191 := (a_w.item.work3).item.gsl_vector_get (l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_cre2a := l_acoef.item * l_tmp___191
							l_tmp___192 := (a_w.item.work4).item.gsl_vector_get (l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_cim2a := l_acoef.item * l_tmp___192
							l_tmp___193 := (a_w.item.work3).item.gsl_vector_get (l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___194 := (a_w.item.work4).item.gsl_vector_get (l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_cre2b := l_bcoefr.item * l_tmp___193 - l_bcoefi.item * l_tmp___194
							l_tmp___195 := (a_w.item.work3).item.gsl_vector_get (l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___196 := (a_w.item.work4).item.gsl_vector_get (l_je)
							update_class_fields -- Current might have been changed; synchronize
							l_cim2b := l_bcoefi.item * l_tmp___195 + l_bcoefr.item * l_tmp___196
							l_i := (0).to_natural_32
							from
							until
								not (l_i < l_je - {NATURAL_32} 1)
							loop
								l_tmp___197 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_je - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___198 := a_T.item.gsl_matrix_get (l_i, l_je - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___199 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_je)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___200 := a_T.item.gsl_matrix_get (l_i, l_je)
								update_class_fields; -- Current might have been changed; synchronize
								(a_w.item.work3).item.gsl_vector_set (l_i, ((-l_creala * l_tmp___197 + l_crealb * l_tmp___198) - l_cre2a * l_tmp___199) + l_cre2b * l_tmp___200)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___201 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_je - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___202 := a_T.item.gsl_matrix_get (l_i, l_je - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___203 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_je)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___204 := a_T.item.gsl_matrix_get (l_i, l_je)
								update_class_fields; -- Current might have been changed; synchronize
								(a_w.item.work4).item.gsl_vector_set (l_i, ((-l_cimaga * l_tmp___201 + l_cimagb * l_tmp___202) - l_cim2a * l_tmp___203) + l_cim2b * l_tmp___204)
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
						end
				when 12 then -- if done
						if not l_return then
							if (2.2204460492503131e-16 * l_acoefa) * l_anorm > (2.2204460492503131e-16 * l_bcoefa) * l_bnorm then
								l_tmp___207 := (2.2204460492503131e-16 * l_acoefa) * l_anorm
							else
								l_tmp___207 := (2.2204460492503131e-16 * l_bcoefa) * l_bnorm
							end
							if 2.2250738585072014e-308 > l_tmp___207 then
								l_dmin := 2.2250738585072014e-308
							else
								if (2.2204460492503131e-16 * l_acoefa) * l_anorm > (2.2204460492503131e-16 * l_bcoefa) * l_bnorm then
									l_tmp___206 := (2.2204460492503131e-16 * l_acoefa) * l_anorm
								else
									l_tmp___206 := (2.2204460492503131e-16 * l_bcoefa) * l_bnorm
								end
								l_dmin := l_tmp___206
							end
							l_il2by2 := 0
							l_is := l_je.to_integer_32 - l_nw.to_integer_32
							from
							until
								not (not (l_return) and then (l_is >= 0))
							loop
								l_continue := False
								l_j := l_is.to_natural_32
								if not l_il2by2.to_boolean then
									if l_j > {NATURAL_32} 0 then
										l_tmp___208 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j - {NATURAL_32} 1)
										update_class_fields -- Current might have been changed; synchronize
										if l_tmp___208 /= 0.0 then
											l_il2by2 := 1
											l_continue := True
										end
									end
								end
								if not (l_return or l_break or l_continue) then
									l_bdiag[0] := a_T.item.gsl_matrix_get (l_j, l_j)
									update_class_fields -- Current might have been changed; synchronize
									if l_il2by2.to_boolean then
										l_na := (2).to_natural_32
										l_bdiag[1] := a_T.item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j + {NATURAL_32} 1)
										update_class_fields -- Current might have been changed; synchronize
									else
										l_na := (1).to_natural_32
									end
									if l_nw = {NATURAL_32} 1 then
										l_tmp___209 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_submatrix (l_j, l_j, l_na, l_na)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_sv.item := l_tmp___209.twin
										update_class_fields -- Current might have been changed; synchronize
										l_bv.item := ((a_w.item.work3).item.gsl_vector_subvector (l_j, l_na)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_xv.item := (gsl_vector_view_array_with_stride (l_sum, (2).to_natural_32, l_na)).twin
										update_class_fields; -- Current might have been changed; synchronize
										(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_sv.item.c_pointer + (create {PD_GSL_MATRIX_CONST_VIEW}.make).offset_of ("matrix"))).item.gsl_schur_solve_equation (l_acoef.item, l_bcoefr.item, l_bdiag[0], l_bdiag[1], (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_scale, l_temp, l_dmin).do_nothing
										update_class_fields -- Current might have been changed; synchronize
									else
										l_tmp___210 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_submatrix (l_j, l_j, l_na, l_na)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_sv___0.item := l_tmp___210.twin
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___211 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_complex_view_array (l_sum, l_na)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_xv___0.item := l_tmp___211.twin
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___212 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_complex_view_array (l_bdat, l_na)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_bv___0.item := l_tmp___212.twin
										update_class_fields -- Current might have been changed; synchronize
										l_bdat[0] := (a_w.item.work3).item.gsl_vector_get (l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_bdat[1] := (a_w.item.work4).item.gsl_vector_get (l_j)
										update_class_fields -- Current might have been changed; synchronize
										if l_na = {NATURAL_32} 2 then
											l_bdat[2] := (a_w.item.work3).item.gsl_vector_get (l_j + {NATURAL_32} 1)
											update_class_fields -- Current might have been changed; synchronize
											l_bdat[3] := (a_w.item.work4).item.gsl_vector_get (l_j + {NATURAL_32} 1)
											update_class_fields -- Current might have been changed; synchronize
										end
										from
										until
											(l_break)
										loop
											l_break := False
											l_z.item.dat[0] := l_bcoefr.item
											update_class_fields -- Current might have been changed; synchronize
											l_z.item.dat[1] := l_bcoefi.item
											update_class_fields -- Current might have been changed; synchronize
											l_break := True
										end
										l_break := False;
										(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_sv___0.item.c_pointer + (create {PD_GSL_MATRIX_CONST_VIEW}.make).offset_of ("matrix"))).item.gsl_schur_solve_equation_z (l_acoef.item, l_z, l_bdiag[0], l_bdiag[1], (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")), l_scale, l_temp, l_dmin).do_nothing
										update_class_fields -- Current might have been changed; synchronize
									end
									if l_scale.item < 1.0 then
										l_jr := (0).to_natural_32
										from
										until
											not (l_jr <= l_je)
										loop
											l_tmp___213 := (a_w.item.work3).item.gsl_vector_get (l_jr)
											update_class_fields; -- Current might have been changed; synchronize
											(a_w.item.work3).item.gsl_vector_set (l_jr, l_scale.item * l_tmp___213)
											update_class_fields -- Current might have been changed; synchronize
											if l_nw = {NATURAL_32} 2 then
												l_tmp___214 := (a_w.item.work4).item.gsl_vector_get (l_jr)
												update_class_fields; -- Current might have been changed; synchronize
												(a_w.item.work4).item.gsl_vector_set (l_jr, l_scale.item * l_tmp___214)
												update_class_fields -- Current might have been changed; synchronize
											end
											l_jr := l_jr + 1
										end
									end
									if l_scale.item * l_xmax > l_temp.item then
										l_xmax := l_scale.item * l_xmax
									else
										l_xmax := l_temp.item
									end
									l_jr := (0).to_natural_32
									from
									until
										not (l_jr < l_na)
									loop
										(a_w.item.work3).item.gsl_vector_set (l_j + l_jr, l_sum[(l_jr * l_na).to_integer_32])
										update_class_fields -- Current might have been changed; synchronize
										if l_nw = {NATURAL_32} 2 then
											(a_w.item.work4).item.gsl_vector_set (l_j + l_jr, l_sum[(l_jr * l_na + {NATURAL_32} 1).to_integer_32])
											update_class_fields -- Current might have been changed; synchronize
										end
										l_jr := l_jr + 1
									end
									if l_j > {NATURAL_32} 0 then
										if 1.0 > l_xmax then
											l_tmp___215 := 1.0
										else
											l_tmp___215 := l_xmax
										end
										l_xscale := 1.0 / l_tmp___215
										l_tmp___216 := (a_w.item.work1).item.gsl_vector_get (l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___217 := (a_w.item.work2).item.gsl_vector_get (l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_temp.item := l_acoefa * l_tmp___216 + l_bcoefa * l_tmp___217
										update_class_fields -- Current might have been changed; synchronize
										if l_il2by2.to_boolean then
											l_tmp___222 := (a_w.item.work1).item.gsl_vector_get (l_j + {NATURAL_32} 1)
											update_class_fields -- Current might have been changed; synchronize
											l_tmp___223 := (a_w.item.work2).item.gsl_vector_get (l_j + {NATURAL_32} 1)
											update_class_fields -- Current might have been changed; synchronize
											if l_temp.item > l_acoefa * l_tmp___222 + l_bcoefa * l_tmp___223 then
												l_temp.item := l_temp.item
												update_class_fields -- Current might have been changed; synchronize
											else
												l_tmp___220 := (a_w.item.work1).item.gsl_vector_get (l_j + {NATURAL_32} 1)
												update_class_fields -- Current might have been changed; synchronize
												l_tmp___221 := (a_w.item.work2).item.gsl_vector_get (l_j + {NATURAL_32} 1)
												update_class_fields -- Current might have been changed; synchronize
												l_temp.item := l_acoefa * l_tmp___220 + l_bcoefa * l_tmp___221
												update_class_fields -- Current might have been changed; synchronize
											end
										end
										if l_acoefa > l_bcoefa then
											l_tmp___226 := l_acoefa
										else
											l_tmp___226 := l_bcoefa
										end
										if l_temp.item > l_tmp___226 then
											l_temp.item := l_temp.item
											update_class_fields -- Current might have been changed; synchronize
										else
											if l_acoefa > l_bcoefa then
												l_tmp___225 := l_acoefa
											else
												l_tmp___225 := l_bcoefa
											end
											l_temp.item := l_tmp___225
											update_class_fields -- Current might have been changed; synchronize
										end
										if l_temp.item > l_bignum * l_xscale then
											l_jr := (0).to_natural_32
											from
											until
												not (l_jr <= l_je)
											loop
												l_tmp___227 := (a_w.item.work3).item.gsl_vector_get (l_jr)
												update_class_fields; -- Current might have been changed; synchronize
												(a_w.item.work3).item.gsl_vector_set (l_jr, l_xscale * l_tmp___227)
												update_class_fields -- Current might have been changed; synchronize
												if l_nw = {NATURAL_32} 2 then
													l_tmp___228 := (a_w.item.work4).item.gsl_vector_get (l_jr)
													update_class_fields; -- Current might have been changed; synchronize
													(a_w.item.work4).item.gsl_vector_set (l_jr, l_xscale * l_tmp___228)
													update_class_fields -- Current might have been changed; synchronize
												end
												l_jr := l_jr + 1
											end
											l_xmax := l_xmax * l_xscale
										end
										l_ja := (0).to_natural_32
										from
										until
											not (not (l_return) and then (l_ja < l_na))
										loop
											if l_complex_pair.to_boolean then
												l_tmp___229 := (a_w.item.work3).item.gsl_vector_get (l_j + l_ja)
												update_class_fields -- Current might have been changed; synchronize
												l_creala := l_acoef.item * l_tmp___229
												l_tmp___230 := (a_w.item.work4).item.gsl_vector_get (l_j + l_ja)
												update_class_fields -- Current might have been changed; synchronize
												l_cimaga := l_acoef.item * l_tmp___230
												l_tmp___231 := (a_w.item.work3).item.gsl_vector_get (l_j + l_ja)
												update_class_fields -- Current might have been changed; synchronize
												l_tmp___232 := (a_w.item.work4).item.gsl_vector_get (l_j + l_ja)
												update_class_fields -- Current might have been changed; synchronize
												l_crealb := l_bcoefr.item * l_tmp___231 - l_bcoefi.item * l_tmp___232
												l_tmp___233 := (a_w.item.work3).item.gsl_vector_get (l_j + l_ja)
												update_class_fields -- Current might have been changed; synchronize
												l_tmp___234 := (a_w.item.work4).item.gsl_vector_get (l_j + l_ja)
												update_class_fields -- Current might have been changed; synchronize
												l_cimagb := l_bcoefi.item * l_tmp___233 + l_bcoefr.item * l_tmp___234
												l_jr := (0).to_natural_32
												from
												until
													not (l_jr <= l_j - {NATURAL_32} 1)
												loop
													l_tmp___235 := (a_w.item.work3).item.gsl_vector_get (l_jr)
													update_class_fields -- Current might have been changed; synchronize
													l_tmp___236 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_jr, l_j + l_ja)
													update_class_fields -- Current might have been changed; synchronize
													l_tmp___237 := a_T.item.gsl_matrix_get (l_jr, l_j + l_ja)
													update_class_fields; -- Current might have been changed; synchronize
													(a_w.item.work3).item.gsl_vector_set (l_jr, (l_tmp___235 - l_creala * l_tmp___236) + l_crealb * l_tmp___237)
													update_class_fields -- Current might have been changed; synchronize
													l_tmp___238 := (a_w.item.work4).item.gsl_vector_get (l_jr)
													update_class_fields -- Current might have been changed; synchronize
													l_tmp___239 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_jr, l_j + l_ja)
													update_class_fields -- Current might have been changed; synchronize
													l_tmp___240 := a_T.item.gsl_matrix_get (l_jr, l_j + l_ja)
													update_class_fields; -- Current might have been changed; synchronize
													(a_w.item.work4).item.gsl_vector_set (l_jr, (l_tmp___238 - l_cimaga * l_tmp___239) + l_cimagb * l_tmp___240)
													update_class_fields -- Current might have been changed; synchronize
													l_jr := l_jr + 1
												end
											else
												l_tmp___241 := (a_w.item.work3).item.gsl_vector_get (l_j + l_ja)
												update_class_fields -- Current might have been changed; synchronize
												l_creala := l_acoef.item * l_tmp___241
												l_tmp___242 := (a_w.item.work3).item.gsl_vector_get (l_j + l_ja)
												update_class_fields -- Current might have been changed; synchronize
												l_crealb := l_bcoefr.item * l_tmp___242
												l_jr := (0).to_natural_32
												from
												until
													not (l_jr <= l_j - {NATURAL_32} 1)
												loop
													l_tmp___243 := (a_w.item.work3).item.gsl_vector_get (l_jr)
													update_class_fields -- Current might have been changed; synchronize
													l_tmp___244 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_jr, l_j + l_ja)
													update_class_fields -- Current might have been changed; synchronize
													l_tmp___245 := a_T.item.gsl_matrix_get (l_jr, l_j + l_ja)
													update_class_fields; -- Current might have been changed; synchronize
													(a_w.item.work3).item.gsl_vector_set (l_jr, (l_tmp___243 - l_creala * l_tmp___244) + l_crealb * l_tmp___245)
													update_class_fields -- Current might have been changed; synchronize
													l_jr := l_jr + 1
												end
											end
											l_ja := l_ja + 1
										end
									end
									l_il2by2 := 0
								end
								if (not l_return) then
									l_is := l_is - 1
								end
							end
							l_continue := False
							l_jr := (0).to_natural_32
							from
							until
								not (l_jr < l_N)
							loop
								l_tmp___246 := (a_w.item.work3).item.gsl_vector_get ((0).to_natural_32)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___247 := a_Z.item.gsl_matrix_get (l_jr, (0).to_natural_32)
								update_class_fields; -- Current might have been changed; synchronize
								(a_w.item.work5).item.gsl_vector_set (l_jr, l_tmp___246 * l_tmp___247)
								update_class_fields -- Current might have been changed; synchronize
								if l_nw = {NATURAL_32} 2 then
									l_tmp___248 := (a_w.item.work4).item.gsl_vector_get ((0).to_natural_32)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___249 := a_Z.item.gsl_matrix_get (l_jr, (0).to_natural_32)
									update_class_fields; -- Current might have been changed; synchronize
									(a_w.item.work6).item.gsl_vector_set (l_jr, l_tmp___248 * l_tmp___249)
									update_class_fields -- Current might have been changed; synchronize
								end
								l_jr := l_jr + 1
							end
							l_jc := (1).to_natural_32
							from
							until
								not (not (l_return) and then (l_jc <= l_je))
							loop
								l_jr := (0).to_natural_32
								from
								until
									not (l_jr < l_N)
								loop
									l_tmp___250 := (a_w.item.work5).item.gsl_vector_get (l_jr)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___251 := (a_w.item.work3).item.gsl_vector_get (l_jc)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___252 := a_Z.item.gsl_matrix_get (l_jr, l_jc)
									update_class_fields; -- Current might have been changed; synchronize
									(a_w.item.work5).item.gsl_vector_set (l_jr, l_tmp___250 + l_tmp___251 * l_tmp___252)
									update_class_fields -- Current might have been changed; synchronize
									if l_nw = {NATURAL_32} 2 then
										l_tmp___253 := (a_w.item.work6).item.gsl_vector_get (l_jr)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___254 := (a_w.item.work4).item.gsl_vector_get (l_jc)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___255 := a_Z.item.gsl_matrix_get (l_jr, l_jc)
										update_class_fields; -- Current might have been changed; synchronize
										(a_w.item.work6).item.gsl_vector_set (l_jr, l_tmp___253 + l_tmp___254 * l_tmp___255)
										update_class_fields -- Current might have been changed; synchronize
									end
									l_jr := l_jr + 1
								end
								l_jc := l_jc + 1
							end
							if l_complex_pair.to_boolean then
								l_ecol.item := (a_evec.item.gsl_matrix_complex_column (l_je - {NATURAL_32} 1)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_re.item := (gsl_vector_complex_real (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
								update_class_fields -- Current might have been changed; synchronize
								l_im.item := (gsl_vector_complex_imag (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
								update_class_fields -- Current might have been changed; synchronize
								l_ecol.item := (a_evec.item.gsl_matrix_complex_column (l_je)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_re2.item := (gsl_vector_complex_real (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
								update_class_fields -- Current might have been changed; synchronize
								l_im2.item := (gsl_vector_complex_imag (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
								update_class_fields -- Current might have been changed; synchronize
							else
								l_ecol.item := (a_evec.item.gsl_matrix_complex_column (l_je)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_re.item := (gsl_vector_complex_real (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
								update_class_fields -- Current might have been changed; synchronize
								l_im.item := (gsl_vector_complex_imag (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
								update_class_fields -- Current might have been changed; synchronize
							end
							l_jr := (0).to_natural_32
							from
							until
								not (l_jr < l_N)
							loop
								l_tmp___256 := (a_w.item.work5).item.gsl_vector_get (l_jr)
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_jr, l_tmp___256)
								update_class_fields -- Current might have been changed; synchronize
								if l_complex_pair.to_boolean then
									l_tmp___257 := (a_w.item.work6).item.gsl_vector_get (l_jr)
									update_class_fields; -- Current might have been changed; synchronize
									(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_jr, l_tmp___257)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___258 := (a_w.item.work5).item.gsl_vector_get (l_jr)
									update_class_fields; -- Current might have been changed; synchronize
									(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_jr, l_tmp___258)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___259 := (a_w.item.work6).item.gsl_vector_get (l_jr)
									update_class_fields; -- Current might have been changed; synchronize
									(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_jr, -l_tmp___259)
									update_class_fields -- Current might have been changed; synchronize
								else
									(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_jr, 0.0)
									update_class_fields -- Current might have been changed; synchronize
								end
								l_jr := l_jr + 1
							end
							l_xmax := 0.0
							if l_complex_pair.to_boolean then
								l_j := (0).to_natural_32
								from
								until
									not (l_j < l_N)
								loop
									l_tmp___268 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___269 := fabs (l_tmp___268)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___270 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___271 := fabs (l_tmp___270)
									update_class_fields -- Current might have been changed; synchronize
									if l_xmax > l_tmp___269 + l_tmp___271 then
										l_xmax := l_xmax
									else
										l_tmp___264 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___265 := fabs (l_tmp___264)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___266 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___267 := fabs (l_tmp___266)
										update_class_fields -- Current might have been changed; synchronize
										l_xmax := l_tmp___265 + l_tmp___267
									end
									l_j := l_j + 1
								end
							else
								l_j := (0).to_natural_32
								from
								until
									not (l_j < l_N)
								loop
									l_tmp___276 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___277 := fabs (l_tmp___276)
									update_class_fields -- Current might have been changed; synchronize
									if l_xmax > l_tmp___277 then
										l_xmax := l_xmax
									else
										l_tmp___274 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___275 := fabs (l_tmp___274)
										update_class_fields -- Current might have been changed; synchronize
										l_xmax := l_tmp___275
									end
									l_j := l_j + 1
								end
							end
							if l_xmax > 2.2250738585072014e-308 then
								l_xscale := 1.0 / l_xmax
								l_j := (0).to_natural_32
								from
								until
									not (l_j < l_N)
								loop
									l_tmp___278 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
									update_class_fields; -- Current might have been changed; synchronize
									(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_j, l_tmp___278 * l_xscale)
									update_class_fields -- Current might have been changed; synchronize
									if l_complex_pair.to_boolean then
										l_tmp___279 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
										update_class_fields; -- Current might have been changed; synchronize
										(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_j, l_tmp___279 * l_xscale)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___280 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
										update_class_fields; -- Current might have been changed; synchronize
										(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_j, l_tmp___280 * l_xscale)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___281 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
										update_class_fields; -- Current might have been changed; synchronize
										(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set (l_j, l_tmp___281 * l_xscale)
										update_class_fields -- Current might have been changed; synchronize
									end
									l_j := l_j + 1
								end
							end
						end
				when 13 then -- if done
						if (not l_return) then
							l_k := l_k + 1
						end
						l_pos := -1 + 1 -- back to loop condition
				when 14 then -- loop done
					if not l_return then
						l_continue := False
						Result := 0
					end
				when 15 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_eigen_gen (a_B: CE_POINTER [PD_GSL_MATRIX]; a_alpha: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_beta: CE_POINTER [PD_GSL_VECTOR]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_anorm: REAL_64
			l_bnorm: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := size1
					if l_N = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("gen.c"), 220, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 3 then -- else:
						if l_N = a_B.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L___0
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 6 then -- else:
						if l_N = a_B.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("B matrix dimensions must match A"), ce_string ("gen.c"), 224, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 9 then -- else:
						if a_alpha.item.size = l_N then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 12 then -- else:
						if a_beta.item.size = l_N then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("gen.c"), 228, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 15 then -- else:
						if a_w.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size does not match workspace"), ce_string ("gen.c"), 232, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_hesstri_decomp (a_B, a_w.item.Q, a_w.item.Z, a_w.item.work).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_w.item.H := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))
							update_class_fields -- Current might have been changed; synchronize
							a_w.item.R := a_B
							update_class_fields -- Current might have been changed; synchronize
							a_w.item.n_evals := (0).to_natural_32
							update_class_fields -- Current might have been changed; synchronize
							a_w.item.n_iter := (0).to_natural_32
							update_class_fields -- Current might have been changed; synchronize
							a_w.item.eshift := 0.0
							update_class_fields -- Current might have been changed; synchronize
							if not (a_w.item.Q).is_void then
								l_tmp := 1
							elseif not (a_w.item.Z).is_void then
								l_tmp := 1
							elseif (a_w.item.compute_t).to_boolean then
								l_tmp := 1
							elseif (a_w.item.compute_s).to_boolean then
								l_tmp := 1
							else
								l_tmp := 0
							end
							a_w.item.needtop := l_tmp
							update_class_fields -- Current might have been changed; synchronize
							l_anorm := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.norm
							update_class_fields -- Current might have been changed; synchronize
							l_bnorm := a_B.item.norm
							update_class_fields -- Current might have been changed; synchronize
							if 2.2250738585072014e-308 > 2.2204460492503131e-16 * l_anorm then
								a_w.item.atol := 2.2250738585072014e-308
								update_class_fields -- Current might have been changed; synchronize
							else
								a_w.item.atol := 2.2204460492503131e-16 * l_anorm
								update_class_fields -- Current might have been changed; synchronize
							end
							if 2.2250738585072014e-308 > 2.2204460492503131e-16 * l_bnorm then
								a_w.item.btol := 2.2250738585072014e-308
								update_class_fields -- Current might have been changed; synchronize
							else
								a_w.item.btol := 2.2204460492503131e-16 * l_bnorm
								update_class_fields -- Current might have been changed; synchronize
							end
							if 2.2250738585072014e-308 > l_anorm then
								l_tmp___0 := 2.2250738585072014e-308
							else
								l_tmp___0 := l_anorm
							end
							a_w.item.ascale := 1.0 / l_tmp___0
							update_class_fields -- Current might have been changed; synchronize
							if 2.2250738585072014e-308 > l_bnorm then
								l_tmp___1 := 2.2250738585072014e-308
							else
								l_tmp___1 := l_bnorm
							end
							a_w.item.bscale := 1.0 / l_tmp___1
							update_class_fields; -- Current might have been changed; synchronize
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gen_schur_decomp (a_B, a_alpha, a_beta, a_w)
							update_class_fields -- Current might have been changed; synchronize
							if a_w.item.n_evals /= l_N then
								Result := 11
								l_return := True
							end
							if not l_return then
								Result := 0
								l_return := True
							end
						end
				when 16 then -- if done | if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 17 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_eigen_gen_QZ (a_B: CE_POINTER [PD_GSL_MATRIX]; a_alpha: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_beta: CE_POINTER [PD_GSL_VECTOR]; a_Q___1, a_Z: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE]): INTEGER_32
		local
			l_s: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_Q___1.is_void then
						l_pos := -1 + 9 -- skip if block
					end
				when 1 then -- if:
						if size1 = a_Q___1.item.size1 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L___2
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else:
						if size1 = a_Q___1.item.size2 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if: | _L___2:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Q matrix has wrong dimensions"), ce_string ("gen.c"), 306, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else:
						l_pos := -1 + 9 -- goto _L___1
				when 8 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 19
				when 9 then -- else: | _L___1:
						if a_Z.is_void then
							l_pos := -1 + 18 -- skip if block
						end
				when 10 then -- if:
						if size1 = a_Z.item.size1 then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						l_pos := -1 + 14 -- goto _L___0
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 13 then -- else:
						if size1 = a_Z.item.size2 then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Z matrix has wrong dimensions"), ce_string ("gen.c"), 310, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 16 then -- else:
						l_pos := -1 + 18 -- goto _L
				when 17 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 19
				when 18 then -- else: | _L:
						a_w.item.Q := a_Q___1
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.Z := a_Z
						update_class_fields -- Current might have been changed; synchronize
						l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_gen (a_B, a_alpha, a_beta, a_w)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.Q := create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.Z := create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer)
						update_class_fields -- Current might have been changed; synchronize
						Result := l_s
						l_return := True
				when 19 then -- if done
					if not l_return then
						Result := 0
					end
				when 20 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gen_schur_decomp (a_R: CE_POINTER [PD_GSL_MATRIX]; a_alpha: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_beta: CE_POINTER [PD_GSL_VECTOR]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE])
		require
			not a_R.is_void
		local
			l_N: NATURAL_32
			l_h: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_r___3: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_vh: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_vr: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_q___3: NATURAL_32
			l_z1: CE_ARRAY [PD_GSL_COMPLEX]
			l_z2: CE_ARRAY [PD_GSL_COMPLEX]
			l_a___5: CE_ARRAY [REAL_64]
			l_b___0: CE_ARRAY [REAL_64]
			l_s: INTEGER_32
			l_flag: CE_ARRAY [INTEGER_32]
			l_tmp: NATURAL_32
			l_break, l_continue: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_r___3.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_vh.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_vr.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_COMPLEX]} l_z1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_COMPLEX]} l_z2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_a___5.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_b___0.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_flag.make (<<1>>)

			l_N := size1
			l_h.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
			update_class_fields -- Current might have been changed; synchronize
			l_r___3.item := (a_R.item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
			update_class_fields -- Current might have been changed; synchronize
			from
			until
				(l_break)
			loop
				l_break := False
				l_continue := False
				if l_N > {NATURAL_32} 1 then
					l_tmp := a_w.item.n_iter
					a_w.item.n_iter := (a_w.item.n_iter) + 1
					update_class_fields -- Current might have been changed; synchronize
					if not (l_tmp < a_w.item.max_iterations) then
						l_break := True
					end
				else
					l_break := True
				end
				if not (l_break or l_continue) then
					l_q___3 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_search_small_elements (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_flag, a_w)
					update_class_fields -- Current might have been changed; synchronize
					if l_flag.item = 0 then
						l_s := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_qzstep (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), a_w)
						update_class_fields -- Current might have been changed; synchronize
						if l_s = -2 then
							l_s := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_standardize2 (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_z1, l_z2, l_a___5, l_b___0, a_w)
							update_class_fields -- Current might have been changed; synchronize
							if l_s /= 0 then
								l_continue := True
							end
							if not (l_break or l_continue) then
								gen_store_eigval2 ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_z1, l_a___5.item, l_z2, l_b___0.item, a_alpha, a_beta, a_w)
								update_class_fields -- Current might have been changed; synchronize
								l_N := (0).to_natural_32
							end
						end
						if not (l_break or l_continue) then
							l_continue := True
						end
					elseif l_flag.item = 2 then
						if l_q___3 = {NATURAL_32} 0 then
							(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_tri_split_top (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), a_w)
							update_class_fields -- Current might have been changed; synchronize
						else
							if l_q___3 /= l_N - {NATURAL_32} 1 then
								(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_tri_chase_zero (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_q___3, a_w)
								update_class_fields -- Current might have been changed; synchronize
							end
							(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_tri_zero_H (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), a_w)
							update_class_fields -- Current might have been changed; synchronize
						end
						l_continue := True
					end
				end
				if not (l_break or l_continue) then
					if l_q___3 = l_N - {NATURAL_32} 1 then
						l_vh.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix (l_q___3, l_q___3, (1).to_natural_32, (1).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_vr.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix (l_q___3, l_q___3, (1).to_natural_32, (1).to_natural_32)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_standardize1 (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vr.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_a___5, l_b___0, a_w).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						gen_store_eigval1 ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_a___5.item, l_b___0.item, a_alpha, a_beta, a_w)
						update_class_fields -- Current might have been changed; synchronize
						l_N := l_N - 1
						l_h.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_r___3.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
					elseif l_q___3 = l_N - {NATURAL_32} 2 then
						l_vh.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix (l_q___3, l_q___3, (2).to_natural_32, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_vr.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix (l_q___3, l_q___3, (2).to_natural_32, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_s := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_standardize2 (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vr.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_z1, l_z2, l_a___5, l_b___0, a_w)
						update_class_fields -- Current might have been changed; synchronize
						if l_s /= 0 then
							(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_decomp (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vr.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), a_alpha, a_beta, a_w)
							update_class_fields -- Current might have been changed; synchronize
						else
							gen_store_eigval2 ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_z1, l_a___5.item, l_z2, l_b___0.item, a_alpha, a_beta, a_w)
							update_class_fields -- Current might have been changed; synchronize
						end
						l_N := l_N - ({NATURAL_32} 2)
						l_h.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_r___3.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
					elseif l_q___3 = {NATURAL_32} 1 then
						l_vh.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, (1).to_natural_32, (1).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_vr.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, (1).to_natural_32, (1).to_natural_32)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_standardize1 (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vr.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_a___5, l_b___0, a_w).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						gen_store_eigval1 ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_a___5.item, l_b___0.item, a_alpha, a_beta, a_w)
						update_class_fields -- Current might have been changed; synchronize
						l_N := l_N - 1
						l_h.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((1).to_natural_32, (1).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_r___3.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((1).to_natural_32, (1).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
					elseif l_q___3 = {NATURAL_32} 2 then
						l_vh.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, (2).to_natural_32, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_vr.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, (2).to_natural_32, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_s := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_standardize2 (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vr.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_z1, l_z2, l_a___5, l_b___0, a_w)
						update_class_fields -- Current might have been changed; synchronize
						if l_s /= 0 then
							(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_decomp (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vr.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), a_alpha, a_beta, a_w)
							update_class_fields -- Current might have been changed; synchronize
						else
							gen_store_eigval2 ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_z1, l_a___5.item, l_z2, l_b___0.item, a_alpha, a_beta, a_w)
							update_class_fields -- Current might have been changed; synchronize
						end
						l_N := l_N - ({NATURAL_32} 2)
						l_h.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((2).to_natural_32, (2).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_r___3.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((2).to_natural_32, (2).to_natural_32, l_N, l_N)).twin
						update_class_fields -- Current might have been changed; synchronize
					else
						l_vh.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix (l_q___3, l_q___3, l_N - l_q___3, l_N - l_q___3)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_vr.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix (l_q___3, l_q___3, l_N - l_q___3, l_N - l_q___3)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_decomp (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vr.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), a_alpha, a_beta, a_w)
						update_class_fields -- Current might have been changed; synchronize
						l_vh.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_q___3, l_q___3)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_vr.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_submatrix ((0).to_natural_32, (0).to_natural_32, l_q___3, l_q___3)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_decomp (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vr.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), a_alpha, a_beta, a_w)
						update_class_fields -- Current might have been changed; synchronize
						l_N := (0).to_natural_32
					end
				end
			end
			l_break := False
			l_continue := False
			if l_N = {NATURAL_32} 1 then
				(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gen_schur_standardize1 (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), l_a___5, l_b___0, a_w).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				gen_store_eigval1 ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_a___5.item, l_b___0.item, a_alpha, a_beta, a_w)
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gen_qzstep (a_R: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE]): INTEGER_32
		require
			not a_R.is_void
			not a_w.is_void
		local
			l_N: NATURAL_32
			l_vh: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_vr: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_wr1: CE_ARRAY [REAL_64]
			l_wr2: CE_ARRAY [REAL_64]
			l_wi: CE_ARRAY [REAL_64]
			l_scale1: CE_ARRAY [REAL_64]
			l_scale2: CE_ARRAY [REAL_64]
			l_scale: REAL_64
			l_cs: CE_ARRAY [REAL_64]
			l_sn: CE_ARRAY [REAL_64]
			l_temp: REAL_64
			l_temp2: REAL_64
			l_j: NATURAL_32
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_yv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_top: NATURAL_32
			l_rows: NATURAL_32
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___14: REAL_64
			l_tmp___15: REAL_64
			l_tmp___16: REAL_64
			l_tmp___17: REAL_64
			l_tmp___18: REAL_64
			l_tmp___19: REAL_64
			l_tmp___20: REAL_64
			l_tmp___21: REAL_64
			l_return: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_vh.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_vr.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_wr1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_wr2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_wi.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_scale1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_scale2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_cs.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_sn.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_yv.make (<<1>>)

			l_N := size1
			if a_w.item.n_iter \\ {NATURAL_32} 10 = {NATURAL_32} 0 then
				l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___2 := fabs (l_tmp___1)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___3 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___4 := fabs (l_tmp___3)
				update_class_fields -- Current might have been changed; synchronize
				if (2.2250738585072014e-308 * (a_w.item.max_iterations).to_real_64) * l_tmp___2 < l_tmp___4 then
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 1)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2)
					update_class_fields -- Current might have been changed; synchronize
					a_w.item.eshift := a_w.item.eshift + (l_tmp / l_tmp___0)
					update_class_fields -- Current might have been changed; synchronize
				else
					a_w.item.eshift := a_w.item.eshift + (1.0 / (2.2250738585072014e-308 * (a_w.item.max_iterations).to_real_64))
					update_class_fields -- Current might have been changed; synchronize
				end
				if a_w.item.eshift < 2.2204460492503131e-16 then
					l_tmp___7 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 2)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___8 := fabs (l_tmp___7)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___9 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___10 := fabs (l_tmp___9)
					update_class_fields -- Current might have been changed; synchronize
					if (2.2250738585072014e-308 * (a_w.item.max_iterations).to_real_64) * l_tmp___8 < l_tmp___10 then
						l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 2)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___6 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2)
						update_class_fields -- Current might have been changed; synchronize
						a_w.item.eshift := (1.736 * (a_w.item.ascale * l_tmp___5)) / (a_w.item.bscale * l_tmp___6)
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				l_scale1.item := 1.0
				update_class_fields -- Current might have been changed; synchronize
				l_wr1.item := a_w.item.eshift
				update_class_fields -- Current might have been changed; synchronize
			else
				l_vh.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2, (2).to_natural_32, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_vr.item := (a_R.item.gsl_matrix_submatrix (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2, (2).to_natural_32, (2).to_natural_32)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vh.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_schur_gen_eigvals ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vr.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_wr1, l_wr2, l_wi, l_scale1, l_scale2).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				if l_wi.item /= 0.0 then
					if l_N = {NATURAL_32} 2 then
						Result := (-2)
						l_return := True
					else
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gen_qzstep_d (a_R, a_w)
						update_class_fields -- Current might have been changed; synchronize
					end
					if not l_return then
						Result := 0
						l_return := True
					end
				end
			end
			if not l_return then
				if a_w.item.ascale < 1.0 then
					l_tmp___11 := a_w.item.ascale
				else
					l_tmp___11 := 1.0
				end
				l_temp := l_tmp___11 * (0.5 / 2.2250738585072014e-308)
				if l_scale1.item > l_temp then
					l_scale := l_temp / l_scale1.item
				else
					l_scale := 1.0
				end
				if a_w.item.bscale < 1.0 then
					l_tmp___12 := a_w.item.bscale
				else
					l_tmp___12 := 1.0
				end
				l_temp := l_tmp___12 * (0.5 / 2.2250738585072014e-308)
				l_tmp___16 := fabs (l_wr1.item)
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp___16 > l_temp then
					l_tmp___15 := fabs (l_wr1.item)
					update_class_fields -- Current might have been changed; synchronize
					if l_scale < l_temp / l_tmp___15 then
						l_scale := l_scale
					else
						l_tmp___14 := fabs (l_wr1.item)
						update_class_fields -- Current might have been changed; synchronize
						l_scale := l_temp / l_tmp___14
					end
				end
				l_scale1.item := l_scale1.item * l_scale
				update_class_fields -- Current might have been changed; synchronize
				l_wr1.item := l_wr1.item * l_scale
				update_class_fields -- Current might have been changed; synchronize
				if (a_w.item.needtop).to_boolean then
					l_top := (a_w.item.H).item.gen_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
					update_class_fields -- Current might have been changed; synchronize
				end
				l_tmp___17 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___18 := a_R.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_temp := l_scale1.item * l_tmp___17 - l_wr1.item * l_tmp___18
				l_tmp___19 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_temp2 := l_scale1.item * l_tmp___19;
				(create {P_QRSTEP_STATIC}).create_givens (l_temp, l_temp2, l_cs, l_sn)
				update_class_fields -- Current might have been changed; synchronize
				l_sn.item := -l_sn.item
				update_class_fields -- Current might have been changed; synchronize
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N - {NATURAL_32} 1)
				loop
					if l_j > {NATURAL_32} 0 then
						l_temp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j - {NATURAL_32} 1)
						update_class_fields -- Current might have been changed; synchronize
						l_temp2 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j - {NATURAL_32} 1)
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_QRSTEP_STATIC}).create_givens (l_temp, l_temp2, l_cs, l_sn)
						update_class_fields -- Current might have been changed; synchronize
						l_sn.item := -l_sn.item
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___20 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j - {NATURAL_32} 1)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___21 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j - {NATURAL_32} 1)
						update_class_fields -- Current might have been changed; synchronize
						l_temp := l_cs.item * l_tmp___20 + l_sn.item * l_tmp___21;
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j, l_j - {NATURAL_32} 1, l_temp)
						update_class_fields; -- Current might have been changed; synchronize
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j + {NATURAL_32} 1, l_j - {NATURAL_32} 1, 0.0)
						update_class_fields -- Current might have been changed; synchronize
					end
					if (a_w.item.compute_s).to_boolean then
						l_xv.item := ((a_w.item.H).item.gsl_matrix_subrow (l_top + l_j, l_top + l_j, (a_w.item.size - l_top) - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := ((a_w.item.H).item.gsl_matrix_subrow ((l_top + l_j) + {NATURAL_32} 1, l_top + l_j, (a_w.item.size - l_top) - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
					else
						l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subrow (l_j, l_j, l_N - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subrow (l_j + {NATURAL_32} 1, l_j, l_N - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
					end
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					if (a_w.item.compute_t).to_boolean then
						l_xv.item := ((a_w.item.R).item.gsl_matrix_subrow (l_top + l_j, l_top + l_j, (a_w.item.size - l_top) - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := ((a_w.item.R).item.gsl_matrix_subrow ((l_top + l_j) + {NATURAL_32} 1, l_top + l_j, (a_w.item.size - l_top) - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
					else
						l_xv.item := (a_R.item.gsl_matrix_subrow (l_j, l_j, l_N - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := (a_R.item.gsl_matrix_subrow (l_j + {NATURAL_32} 1, l_j, l_N - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
					end
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					if (not (a_w.item.Q).is_void) then
						l_xv.item := ((a_w.item.Q).item.gsl_matrix_column (l_top + l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := ((a_w.item.Q).item.gsl_matrix_column ((l_top + l_j) + {NATURAL_32} 1)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					l_temp := a_R.item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j + {NATURAL_32} 1)
					update_class_fields -- Current might have been changed; synchronize
					l_temp2 := a_R.item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j)
					update_class_fields; -- Current might have been changed; synchronize
					(create {P_QRSTEP_STATIC}).create_givens (l_temp, l_temp2, l_cs, l_sn)
					update_class_fields -- Current might have been changed; synchronize
					if l_j + {NATURAL_32} 3 < l_N then
						l_rows := l_j + {NATURAL_32} 3
					else
						l_rows := l_N
					end
					if (a_w.item.compute_s).to_boolean then
						l_xv.item := ((a_w.item.H).item.gsl_matrix_subcolumn (l_top + l_j, (0).to_natural_32, l_top + l_rows)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := ((a_w.item.H).item.gsl_matrix_subcolumn ((l_top + l_j) + {NATURAL_32} 1, (0).to_natural_32, l_top + l_rows)).twin
						update_class_fields -- Current might have been changed; synchronize
					else
						l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_j, (0).to_natural_32, l_rows)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_j + {NATURAL_32} 1, (0).to_natural_32, l_rows)).twin
						update_class_fields -- Current might have been changed; synchronize
					end
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					if l_j + {NATURAL_32} 2 < l_N then
						l_rows := l_j + {NATURAL_32} 2
					else
						l_rows := l_N
					end
					if (a_w.item.compute_t).to_boolean then
						l_xv.item := ((a_w.item.R).item.gsl_matrix_subcolumn (l_top + l_j, (0).to_natural_32, l_top + l_rows)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := ((a_w.item.R).item.gsl_matrix_subcolumn ((l_top + l_j) + {NATURAL_32} 1, (0).to_natural_32, l_top + l_rows)).twin
						update_class_fields -- Current might have been changed; synchronize
					else
						l_xv.item := (a_R.item.gsl_matrix_subcolumn (l_j, (0).to_natural_32, l_rows)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := (a_R.item.gsl_matrix_subcolumn (l_j + {NATURAL_32} 1, (0).to_natural_32, l_rows)).twin
						update_class_fields -- Current might have been changed; synchronize
					end
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					if (not (a_w.item.Z).is_void) then
						l_xv.item := ((a_w.item.Z).item.gsl_matrix_column (l_top + l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_yv.item := ((a_w.item.Z).item.gsl_matrix_column ((l_top + l_j) + {NATURAL_32} 1)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gen_qzstep_d (a_R: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE])
		require
			not a_R.is_void
			not a_w.is_void
		local
			l_N: NATURAL_32
			l_j: NATURAL_32
			l_dat: CE_ARRAY [REAL_64]
			l_tau: REAL_64
			l_v2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v3: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp: REAL_64
			l_q___3: NATURAL_32
			l_r___3: NATURAL_32
			l_top: NATURAL_32
			l_scale: REAL_64
			l_AB11: REAL_64
			l_AB22: REAL_64
			l_ABNN: REAL_64
			l_ABMM: REAL_64
			l_AMNBNN: REAL_64
			l_ANMBMM: REAL_64
			l_A21B11: REAL_64
			l_A12B22: REAL_64
			l_A32B22: REAL_64
			l_B12B22: REAL_64
			l_BMNBNN: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_tmp___15: REAL_64
			l_tmp___16: REAL_64
			l_tmp___17: REAL_64
			l_tmp___18: REAL_64
			l_tmp___19: REAL_64
			l_tmp___20: REAL_64
			l_tmp___21: REAL_64
			l_tmp___22: REAL_64
			l_tmp___23: REAL_64
			l_tmp___24: REAL_64
			l_tmp___25: INTEGER_32
			l_tmp___26: REAL_64
			l_tmp___27: REAL_64
			l_tmp___28: REAL_64
			l_tmp___29: REAL_64
			l_tmp___30: REAL_64
			l_tmp___31: REAL_64
			l_tmp___32: REAL_64
			l_tmp___33: REAL_64
			l_tmp___34: REAL_64
			l_tmp___35: REAL_64
			l_tmp___36: REAL_64
			l_tmp___37: REAL_64
			l_tmp___38: REAL_64
		do
			create {CE_REAL_64_ARRAY} l_dat.make (<<3>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v3.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			l_N := size1
			l_v2.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_dat, (2).to_natural_32)).twin
			update_class_fields -- Current might have been changed; synchronize
			l_v3.item := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_dat, (3).to_natural_32)).twin
			update_class_fields -- Current might have been changed; synchronize
			if (a_w.item.needtop).to_boolean then
				l_top := (a_w.item.H).item.gen_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
				update_class_fields -- Current might have been changed; synchronize
			end
			l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___1 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2)
			update_class_fields -- Current might have been changed; synchronize
			l_ABMM := (a_w.item.ascale * l_tmp___0) / (a_w.item.bscale * l_tmp___1)
			l_tmp___2 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 1)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___3 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 1)
			update_class_fields -- Current might have been changed; synchronize
			l_ABNN := (a_w.item.ascale * l_tmp___2) / (a_w.item.bscale * l_tmp___3)
			l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___5 := a_R.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_AB11 := (a_w.item.ascale * l_tmp___4) / (a_w.item.bscale * l_tmp___5)
			l_tmp___6 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___7 := a_R.item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_AB22 := (a_w.item.ascale * l_tmp___6) / (a_w.item.bscale * l_tmp___7)
			l_tmp___8 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 1)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___9 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 1)
			update_class_fields -- Current might have been changed; synchronize
			l_AMNBNN := (a_w.item.ascale * l_tmp___8) / (a_w.item.bscale * l_tmp___9)
			l_tmp___10 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 2)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___11 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2)
			update_class_fields -- Current might have been changed; synchronize
			l_ANMBMM := (a_w.item.ascale * l_tmp___10) / (a_w.item.bscale * l_tmp___11)
			l_tmp___12 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 1)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___13 := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 1)
			update_class_fields -- Current might have been changed; synchronize
			l_BMNBNN := l_tmp___12 / l_tmp___13
			l_tmp___14 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___15 := a_R.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_A21B11 := (a_w.item.ascale * l_tmp___14) / (a_w.item.bscale * l_tmp___15)
			l_tmp___16 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___17 := a_R.item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_A12B22 := (a_w.item.ascale * l_tmp___16) / (a_w.item.bscale * l_tmp___17)
			l_tmp___18 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((2).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___19 := a_R.item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_A32B22 := (a_w.item.ascale * l_tmp___18) / (a_w.item.bscale * l_tmp___19)
			l_tmp___20 := a_R.item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___21 := a_R.item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_B12B22 := l_tmp___20 / l_tmp___21
			l_dat[0] := (((l_ABMM - l_AB11) * (l_ABNN - l_AB11) - l_AMNBNN * l_ANMBMM) + (l_ANMBMM * l_BMNBNN) * l_AB11) + (l_A12B22 - l_AB11 * l_B12B22) * l_A21B11
			update_class_fields -- Current might have been changed; synchronize
			l_dat[1] := (((((l_AB22 - l_AB11) - l_A21B11 * l_B12B22) - (l_ABMM - l_AB11)) - (l_ABNN - l_AB11)) + l_ANMBMM * l_BMNBNN) * l_A21B11
			update_class_fields -- Current might have been changed; synchronize
			l_dat[2] := l_A32B22 * l_A21B11
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___22 := fabs (l_dat[0])
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___23 := fabs (l_dat[1])
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___24 := fabs (l_dat[2])
			update_class_fields -- Current might have been changed; synchronize
			l_scale := (l_tmp___22 + l_tmp___23) + l_tmp___24
			if l_scale /= 0.0 then
				l_dat[0] := l_dat[0] / l_scale
				update_class_fields -- Current might have been changed; synchronize
				l_dat[1] := l_dat[1] / l_scale
				update_class_fields -- Current might have been changed; synchronize
				l_dat[2] := l_dat[2] / l_scale
				update_class_fields -- Current might have been changed; synchronize
			end
			l_j := (0).to_natural_32
			from
			until
				not (l_j < l_N - {NATURAL_32} 2)
			loop
				if l_j + {NATURAL_32} 4 < l_N then
					l_r___3 := l_j + {NATURAL_32} 4
				else
					l_r___3 := l_N
				end
				l_tau := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
				update_class_fields -- Current might have been changed; synchronize
				if l_tau /= 0.0 then
					if 0 > l_j.to_integer_32 - 1 then
						l_tmp___25 := 0
					else
						l_tmp___25 := l_j.to_integer_32 - 1
					end
					l_q___3 := l_tmp___25.to_natural_32
					if (a_w.item.compute_s).to_boolean then
						l_m___4.item := ((a_w.item.H).item.gsl_matrix_submatrix (l_top + l_j, l_top + l_q___3, (3).to_natural_32, (a_w.item.size - l_top) - l_q___3)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_hm (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					else
						l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_j, l_q___3, (3).to_natural_32, l_N - l_q___3)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_hm (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					if (a_w.item.compute_t).to_boolean then
						l_m___4.item := ((a_w.item.R).item.gsl_matrix_submatrix (l_top + l_j, l_top + l_j, (3).to_natural_32, (a_w.item.size - l_top) - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_hm (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					else
						l_m___4.item := (a_R.item.gsl_matrix_submatrix (l_j, l_j, (3).to_natural_32, l_N - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_hm (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					if (not (a_w.item.Q).is_void) then
						l_m___4.item := ((a_w.item.Q).item.gsl_matrix_submatrix ((0).to_natural_32, l_top + l_j, a_w.item.size, (3).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				l_dat[0] := a_R.item.gsl_matrix_get (l_j + {NATURAL_32} 2, l_j + {NATURAL_32} 2)
				update_class_fields -- Current might have been changed; synchronize
				l_dat[1] := a_R.item.gsl_matrix_get (l_j + {NATURAL_32} 2, l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_dat[2] := a_R.item.gsl_matrix_get (l_j + {NATURAL_32} 2, l_j + {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___26 := fabs (l_dat[0])
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___27 := fabs (l_dat[1])
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___28 := fabs (l_dat[2])
				update_class_fields -- Current might have been changed; synchronize
				l_scale := (l_tmp___26 + l_tmp___27) + l_tmp___28
				if l_scale /= 0.0 then
					l_dat[0] := l_dat[0] / l_scale
					update_class_fields -- Current might have been changed; synchronize
					l_dat[1] := l_dat[1] / l_scale
					update_class_fields -- Current might have been changed; synchronize
					l_dat[2] := l_dat[2] / l_scale
					update_class_fields -- Current might have been changed; synchronize
				end
				l_tau := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
				update_class_fields -- Current might have been changed; synchronize
				if l_tau /= 0.0 then
					l_tmp := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___29 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((2).to_natural_32)
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set ((1).to_natural_32, l_tmp___29 / l_tmp)
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set ((2).to_natural_32, 1.0 / l_tmp)
					update_class_fields -- Current might have been changed; synchronize
					l_tau := l_tau * (l_tmp * l_tmp)
					if (a_w.item.compute_s).to_boolean then
						l_m___4.item := ((a_w.item.H).item.gsl_matrix_submatrix ((0).to_natural_32, l_top + l_j, l_top + l_r___3, (3).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					else
						l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, l_j, l_r___3, (3).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					if (a_w.item.compute_t).to_boolean then
						l_m___4.item := ((a_w.item.R).item.gsl_matrix_submatrix ((0).to_natural_32, l_top + l_j, (l_top + l_j) + {NATURAL_32} 3, (3).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					else
						l_m___4.item := (a_R.item.gsl_matrix_submatrix ((0).to_natural_32, l_j, l_j + {NATURAL_32} 3, (3).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					if (not (a_w.item.Z).is_void) then
						l_m___4.item := ((a_w.item.Z).item.gsl_matrix_submatrix ((0).to_natural_32, l_top + l_j, a_w.item.size, (3).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				l_dat[0] := a_R.item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j + {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_dat[1] := a_R.item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___30 := fabs (l_dat[0])
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___31 := fabs (l_dat[1])
				update_class_fields -- Current might have been changed; synchronize
				l_scale := l_tmp___30 + l_tmp___31
				if l_scale /= 0.0 then
					l_dat[0] := l_dat[0] / l_scale
					update_class_fields -- Current might have been changed; synchronize
					l_dat[1] := l_dat[1] / l_scale
					update_class_fields -- Current might have been changed; synchronize
				end
				l_tau := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
				update_class_fields -- Current might have been changed; synchronize
				if l_tau /= 0.0 then
					l_tmp := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((1).to_natural_32)
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set ((1).to_natural_32, 1.0 / l_tmp)
					update_class_fields -- Current might have been changed; synchronize
					l_tau := l_tau * (l_tmp * l_tmp)
					if (a_w.item.compute_s).to_boolean then
						l_m___4.item := ((a_w.item.H).item.gsl_matrix_submatrix ((0).to_natural_32, l_top + l_j, l_top + l_r___3, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					else
						l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, l_j, l_r___3, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					if (a_w.item.compute_t).to_boolean then
						l_m___4.item := ((a_w.item.R).item.gsl_matrix_submatrix ((0).to_natural_32, l_top + l_j, (l_top + l_j) + {NATURAL_32} 3, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					else
						l_m___4.item := (a_R.item.gsl_matrix_submatrix ((0).to_natural_32, l_j, l_j + {NATURAL_32} 3, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					if (not (a_w.item.Z).is_void) then
						l_m___4.item := ((a_w.item.Z).item.gsl_matrix_submatrix ((0).to_natural_32, l_top + l_j, a_w.item.size, (2).to_natural_32)).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				l_dat[0] := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_dat[1] := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j + {NATURAL_32} 2, l_j)
				update_class_fields -- Current might have been changed; synchronize
				if l_j < l_N - {NATURAL_32} 3 then
					l_dat[2] := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j + {NATURAL_32} 3, l_j)
					update_class_fields -- Current might have been changed; synchronize
				end
				l_tmp___32 := fabs (l_dat[0])
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___33 := fabs (l_dat[1])
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___34 := fabs (l_dat[2])
				update_class_fields -- Current might have been changed; synchronize
				l_scale := (l_tmp___32 + l_tmp___33) + l_tmp___34
				if l_scale /= 0.0 then
					l_dat[0] := l_dat[0] / l_scale
					update_class_fields -- Current might have been changed; synchronize
					l_dat[1] := l_dat[1] / l_scale
					update_class_fields -- Current might have been changed; synchronize
					l_dat[2] := l_dat[2] / l_scale
					update_class_fields -- Current might have been changed; synchronize
				end
				l_j := l_j + 1
			end
			l_tmp___35 := fabs (l_dat[0])
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___36 := fabs (l_dat[1])
			update_class_fields -- Current might have been changed; synchronize
			l_scale := l_tmp___35 + l_tmp___36
			if l_scale /= 0.0 then
				l_dat[0] := l_dat[0] / l_scale
				update_class_fields -- Current might have been changed; synchronize
				l_dat[1] := l_dat[1] / l_scale
				update_class_fields -- Current might have been changed; synchronize
			end
			l_tau := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
			update_class_fields -- Current might have been changed; synchronize
			if (a_w.item.compute_s).to_boolean then
				l_m___4.item := ((a_w.item.H).item.gsl_matrix_submatrix ((l_top + l_N) - {NATURAL_32} 2, (l_top + l_N) - {NATURAL_32} 3, (2).to_natural_32, ((a_w.item.size - l_top) - l_N) + {NATURAL_32} 3)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_hm (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			else
				l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 3, (2).to_natural_32, (3).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_hm (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			if (a_w.item.compute_t).to_boolean then
				l_m___4.item := ((a_w.item.R).item.gsl_matrix_submatrix ((l_top + l_N) - {NATURAL_32} 2, (l_top + l_N) - {NATURAL_32} 2, (2).to_natural_32, ((a_w.item.size - l_top) - l_N) + {NATURAL_32} 2)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_hm (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			else
				l_m___4.item := (a_R.item.gsl_matrix_submatrix (l_N - {NATURAL_32} 2, l_N - {NATURAL_32} 2, (2).to_natural_32, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_hm (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not (a_w.item.Q).is_void) then
				l_m___4.item := ((a_w.item.Q).item.gsl_matrix_submatrix ((0).to_natural_32, (l_top + l_N) - {NATURAL_32} 2, a_w.item.size, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			l_dat[0] := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 1)
			update_class_fields -- Current might have been changed; synchronize
			l_dat[1] := a_R.item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 2)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___37 := fabs (l_dat[0])
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___38 := fabs (l_dat[1])
			update_class_fields -- Current might have been changed; synchronize
			l_scale := l_tmp___37 + l_tmp___38
			if l_scale /= 0.0 then
				l_dat[0] := l_dat[0] / l_scale
				update_class_fields -- Current might have been changed; synchronize
				l_dat[1] := l_dat[1] / l_scale
				update_class_fields -- Current might have been changed; synchronize
			end
			l_tau := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
			update_class_fields -- Current might have been changed; synchronize
			l_tmp := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((1).to_natural_32)
			update_class_fields; -- Current might have been changed; synchronize
			(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set ((1).to_natural_32, 1.0 / l_tmp)
			update_class_fields -- Current might have been changed; synchronize
			l_tau := l_tau * (l_tmp * l_tmp)
			if (a_w.item.compute_s).to_boolean then
				l_m___4.item := ((a_w.item.H).item.gsl_matrix_submatrix ((0).to_natural_32, (l_top + l_N) - {NATURAL_32} 2, l_top + l_N, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			else
				l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, l_N - {NATURAL_32} 2, l_N, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			if (a_w.item.compute_t).to_boolean then
				l_m___4.item := ((a_w.item.R).item.gsl_matrix_submatrix ((0).to_natural_32, (l_top + l_N) - {NATURAL_32} 2, l_top + l_N, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			else
				l_m___4.item := (a_R.item.gsl_matrix_submatrix ((0).to_natural_32, l_N - {NATURAL_32} 2, l_N, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not (a_w.item.Z).is_void) then
				l_m___4.item := ((a_w.item.Z).item.gsl_matrix_submatrix ((0).to_natural_32, (l_top + l_N) - {NATURAL_32} 2, a_w.item.size, (2).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				gsl_linalg_householder_mh (l_tau, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gen_tri_split_top (a_R: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE])
		require
			not a_w.is_void
		local
			l_N: NATURAL_32
			l_j: NATURAL_32
			l_top: NATURAL_32
			l_cs: CE_ARRAY [REAL_64]
			l_sn: CE_ARRAY [REAL_64]
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_yv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: REAL_64
			l_tmp___0: REAL_64
		do
			create {CE_REAL_64_ARRAY} l_cs.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_sn.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_yv.make (<<1>>)

			l_N := size1
			if (a_w.item.needtop).to_boolean then
				l_top := (a_w.item.H).item.gen_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
				update_class_fields -- Current might have been changed; synchronize
			end
			l_j := (0).to_natural_32
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
			update_class_fields; -- Current might have been changed; synchronize
			(create {P_QRSTEP_STATIC}).create_givens (l_tmp___0, l_tmp, l_cs, l_sn)
			update_class_fields -- Current might have been changed; synchronize
			l_sn.item := -l_sn.item
			update_class_fields -- Current might have been changed; synchronize
			if (a_w.item.compute_s).to_boolean then
				l_xv.item := ((a_w.item.H).item.gsl_matrix_subrow (l_top + l_j, l_top, a_w.item.size - l_top)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.H).item.gsl_matrix_subrow ((l_top + l_j) + {NATURAL_32} 1, l_top, a_w.item.size - l_top)).twin
				update_class_fields -- Current might have been changed; synchronize
			else
				l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_j)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_j + {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
			end
			(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
			update_class_fields; -- Current might have been changed; synchronize
			((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j + {NATURAL_32} 1, l_j, 0.0)
			update_class_fields -- Current might have been changed; synchronize
			if (a_w.item.compute_t).to_boolean then
				l_xv.item := ((a_w.item.R).item.gsl_matrix_subrow (l_top + l_j, l_top + {NATURAL_32} 1, (a_w.item.size - l_top) - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.R).item.gsl_matrix_subrow ((l_top + l_j) + {NATURAL_32} 1, l_top + {NATURAL_32} 1, (a_w.item.size - l_top) - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
			else
				l_xv.item := (a_R.item.gsl_matrix_subrow (l_j, (1).to_natural_32, l_N - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := (a_R.item.gsl_matrix_subrow (l_j + {NATURAL_32} 1, (1).to_natural_32, l_N - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
			end
			(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			if (not (a_w.item.Q).is_void) then
				l_xv.item := ((a_w.item.Q).item.gsl_matrix_column (l_top + l_j)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.Q).item.gsl_matrix_column ((l_top + l_j) + {NATURAL_32} 1)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gen_tri_chase_zero (a_R: CE_POINTER [PD_GSL_MATRIX]; a_q___3: NATURAL_32; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE])
		require
			not a_w.is_void
		local
			l_N: NATURAL_32
			l_j: NATURAL_32
			l_top: NATURAL_32
			l_cs: CE_ARRAY [REAL_64]
			l_sn: CE_ARRAY [REAL_64]
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_yv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
		do
			create {CE_REAL_64_ARRAY} l_cs.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_sn.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_yv.make (<<1>>)

			l_N := size1
			if (a_w.item.needtop).to_boolean then
				l_top := (a_w.item.H).item.gen_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
				update_class_fields -- Current might have been changed; synchronize
			end
			l_j := a_q___3
			from
			until
				not (l_j < l_N - {NATURAL_32} 1)
			loop
				l_tmp := a_R.item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j + {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := a_R.item.gsl_matrix_get (l_j, l_j + {NATURAL_32} 1)
				update_class_fields; -- Current might have been changed; synchronize
				(create {P_QRSTEP_STATIC}).create_givens (l_tmp___0, l_tmp, l_cs, l_sn)
				update_class_fields -- Current might have been changed; synchronize
				l_sn.item := -l_sn.item
				update_class_fields -- Current might have been changed; synchronize
				if (a_w.item.compute_t).to_boolean then
					l_xv.item := ((a_w.item.R).item.gsl_matrix_subrow (l_top + l_j, (l_top + l_j) + {NATURAL_32} 1, ((a_w.item.size - l_top) - l_j) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.R).item.gsl_matrix_subrow ((l_top + l_j) + {NATURAL_32} 1, (l_top + l_j) + {NATURAL_32} 1, ((a_w.item.size - l_top) - l_j) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
				else
					l_xv.item := (a_R.item.gsl_matrix_subrow (l_j, l_j + {NATURAL_32} 1, (l_N - l_j) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := (a_R.item.gsl_matrix_subrow (l_j + {NATURAL_32} 1, l_j + {NATURAL_32} 1, (l_N - l_j) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
				end
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				a_R.item.gsl_matrix_set (l_j + {NATURAL_32} 1, l_j + {NATURAL_32} 1, 0.0)
				update_class_fields -- Current might have been changed; synchronize
				if (a_w.item.compute_s).to_boolean then
					l_xv.item := ((a_w.item.H).item.gsl_matrix_subrow (l_top + l_j, (l_top + l_j) - {NATURAL_32} 1, ((a_w.item.size - l_top) - l_j) + {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.H).item.gsl_matrix_subrow ((l_top + l_j) + {NATURAL_32} 1, (l_top + l_j) - {NATURAL_32} 1, ((a_w.item.size - l_top) - l_j) + {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
				else
					l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subrow (l_j, l_j - {NATURAL_32} 1, (l_N - l_j) + {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subrow (l_j + {NATURAL_32} 1, l_j - {NATURAL_32} 1, (l_N - l_j) + {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
				end
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				if (not (a_w.item.Q).is_void) then
					l_xv.item := ((a_w.item.Q).item.gsl_matrix_column (l_top + l_j)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.Q).item.gsl_matrix_column ((l_top + l_j) + {NATURAL_32} 1)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
				l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j - {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___2 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j + {NATURAL_32} 1, l_j)
				update_class_fields; -- Current might have been changed; synchronize
				(create {P_QRSTEP_STATIC}).create_givens (l_tmp___2, l_tmp___1, l_cs, l_sn)
				update_class_fields -- Current might have been changed; synchronize
				l_sn.item := -l_sn.item
				update_class_fields -- Current might have been changed; synchronize
				if (a_w.item.compute_s).to_boolean then
					l_xv.item := ((a_w.item.H).item.gsl_matrix_subcolumn (l_top + l_j, (0).to_natural_32, (l_top + l_j) + {NATURAL_32} 2)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.H).item.gsl_matrix_subcolumn ((l_top + l_j) - {NATURAL_32} 1, (0).to_natural_32, (l_top + l_j) + {NATURAL_32} 2)).twin
					update_class_fields -- Current might have been changed; synchronize
				else
					l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_j, (0).to_natural_32, l_j + {NATURAL_32} 2)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_j - {NATURAL_32} 1, (0).to_natural_32, l_j + {NATURAL_32} 2)).twin
					update_class_fields -- Current might have been changed; synchronize
				end
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j + {NATURAL_32} 1, l_j - {NATURAL_32} 1, 0.0)
				update_class_fields -- Current might have been changed; synchronize
				if (a_w.item.compute_t).to_boolean then
					l_xv.item := ((a_w.item.R).item.gsl_matrix_subcolumn (l_top + l_j, (0).to_natural_32, (l_top + l_j) + {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.R).item.gsl_matrix_subcolumn ((l_top + l_j) - {NATURAL_32} 1, (0).to_natural_32, (l_top + l_j) + {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
				else
					l_xv.item := (a_R.item.gsl_matrix_subcolumn (l_j, (0).to_natural_32, l_j + {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := (a_R.item.gsl_matrix_subcolumn (l_j - {NATURAL_32} 1, (0).to_natural_32, l_j + {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
				end
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				if (not (a_w.item.Z).is_void) then
					l_xv.item := ((a_w.item.Z).item.gsl_matrix_column (l_top + l_j)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.Z).item.gsl_matrix_column ((l_top + l_j) - {NATURAL_32} 1)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
				l_j := l_j + 1
			end
		end

	gen_tri_zero_H (a_R: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE])
		require
			not a_w.is_void
		local
			l_N: NATURAL_32
			l_top: NATURAL_32
			l_cs: CE_ARRAY [REAL_64]
			l_sn: CE_ARRAY [REAL_64]
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_yv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: REAL_64
			l_tmp___0: REAL_64
		do
			create {CE_REAL_64_ARRAY} l_cs.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_sn.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_yv.make (<<1>>)

			l_N := size1
			if (a_w.item.needtop).to_boolean then
				l_top := (a_w.item.H).item.gen_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
				update_class_fields -- Current might have been changed; synchronize
			end
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 2)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 1)
			update_class_fields; -- Current might have been changed; synchronize
			(create {P_QRSTEP_STATIC}).create_givens (l_tmp___0, l_tmp, l_cs, l_sn)
			update_class_fields -- Current might have been changed; synchronize
			l_sn.item := -l_sn.item
			update_class_fields -- Current might have been changed; synchronize
			if (a_w.item.compute_s).to_boolean then
				l_xv.item := ((a_w.item.H).item.gsl_matrix_subcolumn ((l_top + l_N) - {NATURAL_32} 1, (0).to_natural_32, l_top + l_N)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.H).item.gsl_matrix_subcolumn ((l_top + l_N) - {NATURAL_32} 2, (0).to_natural_32, l_top + l_N)).twin
				update_class_fields -- Current might have been changed; synchronize
			else
				l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_N - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_N - {NATURAL_32} 2)).twin
				update_class_fields -- Current might have been changed; synchronize
			end
			(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
			update_class_fields; -- Current might have been changed; synchronize
			((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_N - {NATURAL_32} 1, l_N - {NATURAL_32} 2, 0.0)
			update_class_fields -- Current might have been changed; synchronize
			if (a_w.item.compute_t).to_boolean then
				l_xv.item := ((a_w.item.R).item.gsl_matrix_subcolumn ((l_top + l_N) - {NATURAL_32} 1, (0).to_natural_32, (l_top + l_N) - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.R).item.gsl_matrix_subcolumn ((l_top + l_N) - {NATURAL_32} 2, (0).to_natural_32, (l_top + l_N) - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
			else
				l_xv.item := (a_R.item.gsl_matrix_subcolumn (l_N - {NATURAL_32} 1, (0).to_natural_32, l_N - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := (a_R.item.gsl_matrix_subcolumn (l_N - {NATURAL_32} 2, (0).to_natural_32, l_N - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
			end
			(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			if (not (a_w.item.Z).is_void) then
				l_xv.item := ((a_w.item.Z).item.gsl_matrix_column ((l_top + l_N) - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.Z).item.gsl_matrix_column ((l_top + l_N) - {NATURAL_32} 2)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gen_search_small_elements (a_R: CE_POINTER [PD_GSL_MATRIX]; a_flag: CE_POINTER [INTEGER_32]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE]): NATURAL_32
		require
			not a_flag.is_void
		local
			l_N: NATURAL_32
			l_k: INTEGER_32
			l_i: NATURAL_32
			l_pass1: INTEGER_32
			l_pass2: INTEGER_32
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := size1
					l_pass1 := 0
					l_pass2 := 0
					l_k := l_N.to_integer_32 - 1
				when 1 then -- loop condition:
					if not (not (l_return) and then (l_k >= 0)) then
						l_pos := -1 + 10 -- skip loop
					end
				when 2 then -- loop body:
						l_i := l_k.to_natural_32
						if l_i /= {NATURAL_32} 0 then
							l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___0 := fabs (l_tmp)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___0 <= a_w.item.atol then
								((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_i - {NATURAL_32} 1, 0.0)
								update_class_fields -- Current might have been changed; synchronize
								l_pass1 := 1
							end
						end
						l_tmp___1 := a_R.item.gsl_matrix_get (l_i, l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := fabs (l_tmp___1)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___2 < a_w.item.btol then
							a_R.item.gsl_matrix_set (l_i, l_i, 0.0)
							update_class_fields -- Current might have been changed; synchronize
							l_pass2 := 1
						end
						if not (l_pass1.to_boolean) then
							l_pos := -1 + 5 -- skip if block
						end
				when 3 then -- if:
						if not l_pass2.to_boolean then
							a_flag.item := 1
							update_class_fields -- Current might have been changed; synchronize
							Result := l_i
							l_return := True
						else
							l_pos := -1 + 5 -- goto _L___0
						end
				when 4 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 5 then -- else: | _L___0:
						if l_pass1.to_boolean then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if:
						if l_pass2.to_boolean then
							a_flag.item := 2
							update_class_fields -- Current might have been changed; synchronize
							Result := l_i
							l_return := True
						else
							l_pos := -1 + 8 -- goto _L
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else: | _L:
						if l_pass1.to_boolean then
							if l_pass2.to_boolean then
								a_flag.item := 3
								update_class_fields -- Current might have been changed; synchronize
								Result := l_i
								l_return := True
							end
						end
				when 9 then -- if done
						if not l_return then
							l_k := l_k - 1
						end
						l_pos := -1 + 1 -- back to loop condition
				when 10 then -- loop done
					if not l_return then
						a_flag.item := 0
						update_class_fields -- Current might have been changed; synchronize
						Result := (0).to_natural_32
					end
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gen_schur_standardize1 (a_B: CE_POINTER [PD_GSL_MATRIX]; a_alphar, a_beta: CE_POINTER [REAL_64]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE]): INTEGER_32
		require
			not a_B.is_void
			not a_alphar.is_void
			not a_beta.is_void
		local
			l_i: NATURAL_32
			l_top: NATURAL_32
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
		do
			l_tmp___4 := a_B.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			if l_tmp___4 < 0.0 then
				if (a_w.item.needtop).to_boolean then
					l_top := (a_w.item.H).item.gen_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
					update_class_fields -- Current might have been changed; synchronize
				end
				if (a_w.item.compute_t).to_boolean then
					l_i := (0).to_natural_32
					from
					until
						not (l_i <= l_top)
					loop
						l_tmp := (a_w.item.R).item.gsl_matrix_get (l_i, l_top)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.R).item.gsl_matrix_set (l_i, l_top, -l_tmp)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				else
					l_tmp___0 := a_B.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					a_B.item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, -l_tmp___0)
					update_class_fields -- Current might have been changed; synchronize
				end
				if (a_w.item.compute_s).to_boolean then
					l_i := (0).to_natural_32
					from
					until
						not (l_i <= l_top)
					loop
						l_tmp___1 := (a_w.item.H).item.gsl_matrix_get (l_i, l_top)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.H).item.gsl_matrix_set (l_i, l_top, -l_tmp___1)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				else
					l_tmp___2 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, -l_tmp___2)
					update_class_fields -- Current might have been changed; synchronize
				end
				if (not (a_w.item.Z).is_void) then
					l_i := (0).to_natural_32
					from
					until
						not (l_i < a_w.item.size)
					loop
						l_tmp___3 := (a_w.item.Z).item.gsl_matrix_get (l_i, l_top)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.Z).item.gsl_matrix_set (l_i, l_top, -l_tmp___3)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				end
			end
			a_alphar.item := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			a_beta.item := a_B.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			Result := 0
		end

	gen_compute_eigenvals (a_B: CE_POINTER [PD_GSL_MATRIX]; a_alpha1, a_alpha2: CE_POINTER [PD_GSL_COMPLEX]; a_beta1, a_beta2: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_B.is_void
		local
			l_wr1: CE_ARRAY [REAL_64]
			l_wr2: CE_ARRAY [REAL_64]
			l_wi: CE_ARRAY [REAL_64]
			l_scale1: CE_ARRAY [REAL_64]
			l_scale2: CE_ARRAY [REAL_64]
			l_s1inv: REAL_64
			l_A11: REAL_64
			l_A12: REAL_64
			l_A21: REAL_64
			l_A22: REAL_64
			l_B11: REAL_64
			l_B22: REAL_64
			l_c11r: REAL_64
			l_c11i: REAL_64
			l_c12: REAL_64
			l_c21: REAL_64
			l_c22r: REAL_64
			l_c22i: REAL_64
			l_cz: REAL_64
			l_cq: REAL_64
			l_szr: REAL_64
			l_szi: REAL_64
			l_sqr: REAL_64
			l_sqi: REAL_64
			l_a1r: REAL_64
			l_a1i: REAL_64
			l_a2r: REAL_64
			l_a2i: REAL_64
			l_b1r: REAL_64
			l_b1i: REAL_64
			l_b1a: REAL_64
			l_b2r: REAL_64
			l_b2i: REAL_64
			l_b2a: REAL_64
			l_alphar: REAL_64
			l_alphai: REAL_64
			l_t1: REAL_64
			l_an: REAL_64
			l_bn: REAL_64
			l_tempr: REAL_64
			l_tempi: REAL_64
			l_wabs: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_wr1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_wr2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_wi.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_scale1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_scale2.make (<<1>>);

			((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_schur_gen_eigvals (a_B, l_wr1, l_wr2, l_wi, l_scale1, l_scale2).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			if l_wi.item = 0.0 then
				Result := (-2)
				l_return := True
			end
			if not l_return then
				l_s1inv := 1.0 / l_scale1.item
				l_A11 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_A12 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_A21 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_A22 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_B11 := a_B.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_B22 := a_B.item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_c11r := l_scale1.item * l_A11 - l_wr1.item * l_B11
				l_c11i := -l_wi.item * l_B11
				l_c12 := l_scale1.item * l_A12
				l_c21 := l_scale1.item * l_A21
				l_c22r := l_scale1.item * l_A22 - l_wr1.item * l_B22
				l_c22i := -l_wi.item * l_B22
				l_tmp := fabs (l_c11r)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := fabs (l_c11i)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___1 := fabs (l_c12)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___2 := fabs (l_c21)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___3 := fabs (l_c22r)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___4 := fabs (l_c22i)
				update_class_fields -- Current might have been changed; synchronize
				if (l_tmp + l_tmp___0) + l_tmp___1 > (l_tmp___2 + l_tmp___3) + l_tmp___4 then
					l_t1 := gsl_hypot3 (l_c12, l_c11r, l_c11i)
					update_class_fields -- Current might have been changed; synchronize
					if l_t1 /= 0.0 then
						l_cz := l_c12 / l_t1
						l_szr := -l_c11r / l_t1
						l_szi := -l_c11i / l_t1
					else
						l_cz := 0.0
						l_szr := 1.0
						l_szi := 0.0
					end
				else
					l_cz := hypot (l_c22r, l_c22i)
					update_class_fields -- Current might have been changed; synchronize
					if l_cz <= 2.2250738585072014e-308 then
						l_cz := 0.0
						l_szr := 1.0
						l_szi := 0.0
					else
						l_tempr := l_c22r / l_cz
						l_tempi := l_c22i / l_cz
						l_t1 := hypot (l_cz, l_c21)
						update_class_fields -- Current might have been changed; synchronize
						l_cz := l_cz / l_t1
						l_szr := (-l_c21 * l_tempr) / l_t1
						l_szi := (l_c21 * l_tempi) / l_t1
					end
				end
				l_tmp___5 := fabs (l_A11)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___6 := fabs (l_A12)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___7 := fabs (l_A21)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___8 := fabs (l_A22)
				update_class_fields -- Current might have been changed; synchronize
				l_an := ((l_tmp___5 + l_tmp___6) + l_tmp___7) + l_tmp___8
				l_tmp___9 := fabs (l_B11)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___10 := fabs (l_B22)
				update_class_fields -- Current might have been changed; synchronize
				l_bn := l_tmp___9 + l_tmp___10
				l_tmp___11 := fabs (l_wr1.item)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___12 := fabs (l_wi.item)
				update_class_fields -- Current might have been changed; synchronize
				l_wabs := l_tmp___11 + l_tmp___12
				if l_scale1.item * l_an > l_wabs * l_bn then
					l_cq := l_cz * l_B11
					if l_cq <= 2.2250738585072014e-308 then
						l_cq := 0.0
						l_sqr := 1.0
						l_sqi := 0.0
					else
						l_sqr := l_szr * l_B22
						l_sqi := -l_szi * l_B22
					end
				else
					l_a1r := l_cz * l_A11 + l_szr * l_A12
					l_a1i := l_szi * l_A12
					l_a2r := l_cz * l_A21 + l_szr * l_A22
					l_a2i := l_szi * l_A22
					l_cq := hypot (l_a1r, l_a1i)
					update_class_fields -- Current might have been changed; synchronize
					if l_cq <= 2.2250738585072014e-308 then
						l_cq := 0.0
						l_sqr := 1.0
						l_sqi := 0.0
					else
						l_tempr := l_a1r / l_cq
						l_tempi := l_a1i / l_cq
						l_sqr := l_tempr * l_a2r + l_tempi * l_a2i
						l_sqi := l_tempi * l_a2r - l_tempr * l_a2i
					end
				end
				l_t1 := gsl_hypot3 (l_cq, l_sqr, l_sqi)
				update_class_fields -- Current might have been changed; synchronize
				l_cq := l_cq / l_t1
				l_sqr := l_sqr / l_t1
				l_sqi := l_sqi / l_t1
				l_tempr := l_sqr * l_szr - l_sqi * l_szi
				l_tempi := l_sqr * l_szi + l_sqi * l_szr
				l_b1r := (l_cq * l_cz) * l_B11 + l_tempr * l_B22
				l_b1i := l_tempi * l_B22
				l_b1a := hypot (l_b1r, l_b1i)
				update_class_fields -- Current might have been changed; synchronize
				l_b2r := (l_cq * l_cz) * l_B22 + l_tempr * l_B11
				l_b2i := -l_tempi * l_B11
				l_b2a := hypot (l_b2r, l_b2i)
				update_class_fields -- Current might have been changed; synchronize
				a_beta1.item := l_b1a
				update_class_fields -- Current might have been changed; synchronize
				a_beta2.item := l_b2a
				update_class_fields -- Current might have been changed; synchronize
				l_alphar := (l_wr1.item * l_b1a) * l_s1inv
				l_alphai := (l_wi.item * l_b1a) * l_s1inv
				from
				until
					(l_break)
				loop
					l_break := False
					a_alpha1.item.dat[0] := l_alphar
					update_class_fields -- Current might have been changed; synchronize
					a_alpha1.item.dat[1] := l_alphai
					update_class_fields -- Current might have been changed; synchronize
					l_break := True
				end
				l_break := False
				l_alphar := (l_wr1.item * l_b2a) * l_s1inv
				l_alphai := -(l_wi.item * l_b2a) * l_s1inv
				from
				until
					(l_break)
				loop
					l_break := False
					a_alpha2.item.dat[0] := l_alphar
					update_class_fields -- Current might have been changed; synchronize
					a_alpha2.item.dat[1] := l_alphai
					update_class_fields -- Current might have been changed; synchronize
					l_break := True
				end
				l_break := False
				Result := 0
			end
		end

	gen_get_submatrix (a_B: CE_POINTER [PD_GSL_MATRIX]): NATURAL_32
		require
			not a_B.is_void
		local
			l_diff: NATURAL_32
			l_ratio: REAL_64
			l_top: NATURAL_32
			l_tmp: REAL_64
		do
			l_diff := ((a_B.item.data).distance (data)).to_natural_32
			l_ratio := l_diff.to_real_64 / (tda + {NATURAL_32} 1).to_real_64
			l_tmp := floor (l_ratio)
			update_class_fields -- Current might have been changed; synchronize
			l_top := l_tmp.truncated_to_integer_64.to_natural_32
			Result := l_top
		end

	gsl_eigen_symm (a_eval: CE_POINTER [PD_GSL_VECTOR]; a_w: CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_d: CE_POINTER [REAL_64]
			l_sd: CE_POINTER [REAL_64]
			l_a___5: NATURAL_32
			l_b___0: NATURAL_32
			l_A00: REAL_64
			l_tmp: REAL_64
			l_d_vec: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_sd_vec: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: PD_GSL_VECTOR_VIEW
			l_tau: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___2: PD_GSL_VECTOR_VIEW
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_tmp___5: INTEGER_32
			l_tmp___6: INTEGER_32
			l_tmp___7: INTEGER_32
			l_n_block: NATURAL_32
			l_d_block: CE_POINTER [REAL_64]
			l_sd_block: CE_POINTER [REAL_64]
			l_d_vec___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___8: PD_GSL_VECTOR_VIEW
			l_return, l_break, l_continue: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_d_vec.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_sd_vec.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_tau.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_d_vec___0.make (<<1>>)

			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("symm.c"), 91, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_eval.item.size /= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("symm.c"), 95, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_N := size1
				l_d := a_w.item.d
				l_sd := a_w.item.sd
				if l_N = {NATURAL_32} 1 then
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_A00 := l_tmp
					a_eval.item.gsl_vector_set ((0).to_natural_32, l_A00)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
				if not l_return then
					l_tmp___0 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_d, l_N)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_d_vec.item := l_tmp___0.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___1 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_sd, l_N - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_sd_vec.item := l_tmp___1.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___2 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_sd, l_N - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_tau.item := l_tmp___2.twin
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_symmtd_decomp (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_tau.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_symmtd_unpack_T (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_d_vec.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_sd_vec.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					chop_small_elements___2 (l_N, l_d, l_sd)
					update_class_fields -- Current might have been changed; synchronize
					l_b___0 := l_N - {NATURAL_32} 1
					from
					until
						not (not (l_return) and then (l_b___0 > {NATURAL_32} 0))
					loop
						l_continue := False
						if l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32] = 0.0 then
							l_b___0 := l_b___0 - 1
							l_continue := True
						else
							if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
								l_tmp___3 := isnanf ((l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32]).truncated_to_real)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___7 := l_tmp___3
							else
								if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
									l_tmp___4 := isnan (l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32])
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___6 := l_tmp___4
								else
									l_tmp___5 := isnanl (real_96_from_real_64 (l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32]))
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___6 := l_tmp___5
								end
								l_tmp___7 := l_tmp___6
							end
							if l_tmp___7.to_boolean then
								l_b___0 := l_b___0 - 1
								l_continue := True
							end
						end
						if not (l_return or l_break or l_continue) then
							l_a___5 := l_b___0 - {NATURAL_32} 1
							from
							until
								not (not (l_break) and then (l_a___5 > {NATURAL_32} 0))
							loop
								l_break := False
								if l_sd[(l_a___5 - {NATURAL_32} 1).to_integer_32] = 0.0 then
									l_break := True
								end
								if not (l_return or l_break) then
									l_a___5 := l_a___5 - 1
								end
							end
							l_break := False
							l_n_block := (l_b___0 - l_a___5) + {NATURAL_32} 1
							l_d_block := l_d + l_a___5.to_integer_32
							l_sd_block := l_sd + l_a___5.to_integer_32
							qrstep___2 (l_n_block, l_d_block, l_sd_block, create {CE_REAL_64_POINTER}.make_shared (default_pointer), create {CE_REAL_64_POINTER}.make_shared (default_pointer))
							update_class_fields -- Current might have been changed; synchronize
							chop_small_elements___2 (l_n_block, l_d_block, l_sd_block)
							update_class_fields -- Current might have been changed; synchronize
						end
					end
					l_continue := False
					l_tmp___8 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_d, l_N)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_d_vec___0.item := l_tmp___8.twin
					update_class_fields -- Current might have been changed; synchronize
					a_eval.item.gsl_vector_memcpy ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_d_vec___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_eigen_gensymmv (a_B: CE_POINTER [PD_GSL_MATRIX]; a_eval: CE_POINTER [PD_GSL_VECTOR]; a_evec: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_GENSYMMV_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_s: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := size1
					if l_N = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("gensymmv.c"), 123, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if l_N = a_B.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if l_N = a_B.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("B matrix dimensions must match A"), ce_string ("gensymmv.c"), 127, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						if a_eval.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("gensymmv.c"), 131, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_evec.item.size1 /= a_evec.item.size2 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvector matrix must be square"), ce_string ("gensymmv.c"), 135, 20)
								update_class_fields -- Current might have been changed; synchronize
								Result := 20
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_evec.item.size1 /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvector matrix has wrong size"), ce_string ("gensymmv.c"), 139, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_w.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size does not match workspace"), ce_string ("gensymmv.c"), 143, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_s := a_B.item.gsl_linalg_cholesky_decomp
							update_class_fields -- Current might have been changed; synchronize
							if l_s /= 0 then
								Result := l_s
								l_return := True
							end
							if not l_return then
								((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_gensymm_standardize (a_B).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_symmv (a_eval, a_evec, a_w.item.symmv_workspace_p)
								update_class_fields -- Current might have been changed; synchronize
								if l_s /= 0 then
									Result := l_s
									l_return := True
								end
							end
							if not l_return then
								a_B.item.gsl_blas_dtrsm ({PC_CBLAS_SIDE}.CblasLeft, {PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, 1.0, a_evec).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								a_evec.item.gensymmv_normalize_eigenvectors
								update_class_fields -- Current might have been changed; synchronize
								Result := 0
								l_return := True
							end
						end
				when 10 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gensymmv_normalize_eigenvectors
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_vi: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_scale: REAL_64
			l_tmp___0: REAL_64
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_vi.make (<<1>>)

			l_N := size1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_vi.item := l_tmp.twin
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
				update_class_fields -- Current might have been changed; synchronize
				l_scale := 1.0 / l_tmp___0;
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_eigen_gensymm (a_B: CE_POINTER [PD_GSL_MATRIX]; a_eval: CE_POINTER [PD_GSL_VECTOR]; a_w: CE_POINTER [PD_GSL_EIGEN_GENSYMM_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_s: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := size1
					if l_N = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("gensymm.c"), 120, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if l_N = a_B.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if l_N = a_B.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("B matrix dimensions must match A"), ce_string ("gensymm.c"), 124, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						if a_eval.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("gensymm.c"), 128, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_w.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size does not match workspace"), ce_string ("gensymm.c"), 132, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_s := a_B.item.gsl_linalg_cholesky_decomp
							update_class_fields -- Current might have been changed; synchronize
							if l_s /= 0 then
								Result := l_s
								l_return := True
							end
							if not l_return then
								((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_gensymm_standardize (a_B).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_eigen_symm (a_eval, a_w.item.symm_workspace_p)
								update_class_fields -- Current might have been changed; synchronize
								Result := l_s
								l_return := True
							end
						end
				when 10 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_eigen_gensymm_standardize (a_B: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_c___1: REAL_64
			l_ai: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_ma: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___0: PD_GSL_MATRIX_VIEW
			l_bi: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___1: PD_GSL_VECTOR_CONST_VIEW
			l_mb: CE_ARRAY [PD_GSL_MATRIX_CONST_VIEW]
			l_tmp___2: PD_GSL_MATRIX_CONST_VIEW
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_ai.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_ma.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_bi.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_CONST_VIEW]} l_mb.make (<<1>>)

			l_N := size1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_a___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_b___0 := a_B.item.gsl_matrix_get (l_i, l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_a___5 := l_a___5 / (l_b___0 * l_b___0);
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_i, l_a___5)
				update_class_fields -- Current might have been changed; synchronize
				if l_i < l_N - {NATURAL_32} 1 then
					l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_i, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_ai.item := l_tmp.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i + {NATURAL_32} 1, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_ma.item := l_tmp___0.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___1 := (a_B.item.gsl_matrix_const_subcolumn (l_i, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_bi.item := l_tmp___1.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___2 := (a_B.item.gsl_matrix_const_submatrix (l_i + {NATURAL_32} 1, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_mb.item := l_tmp___2.twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (1.0 / l_b___0)
					update_class_fields -- Current might have been changed; synchronize
					l_c___1 := -0.5 * l_a___5;
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_bi.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (l_c___1, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					gsl_blas_dsyr2 ({PC_CBLAS_UPLO}.CblasLower, -1.0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_bi.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_ma.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_bi.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (l_c___1, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_mb.item.c_pointer + (create {PD_GSL_MATRIX_CONST_VIEW}.make).offset_of ("matrix"))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ai.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_linalg_LU_decomp (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_signum: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_ajj: REAL_64
			l_max: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_i_pivot: NATURAL_32
			l_aij: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_aij___0: REAL_64
			l_tmp___3: REAL_64
			l_aik: REAL_64
			l_tmp___4: REAL_64
			l_ajk: REAL_64
			l_tmp___5: REAL_64
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LU decomposition requires square matrix"), ce_string ("lu.c"), 64, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_p.item.size /= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation length must match matrix size"), ce_string ("lu.c"), 68, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_N := size1
				a_signum.item := 1
				update_class_fields -- Current might have been changed; synchronize
				a_p.item.gsl_permutation_init
				update_class_fields -- Current might have been changed; synchronize
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N - {NATURAL_32} 1))
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := fabs (l_tmp)
					update_class_fields -- Current might have been changed; synchronize
					l_max := l_tmp___0
					l_i_pivot := l_j
					l_i := l_j + {NATURAL_32} 1
					from
					until
						not (l_i < l_N)
					loop
						l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := fabs (l_tmp___1)
						update_class_fields -- Current might have been changed; synchronize
						l_aij := l_tmp___2
						if l_aij > l_max then
							l_max := l_aij
							l_i_pivot := l_i
						end
						l_i := l_i + 1
					end
					if l_i_pivot /= l_j then
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_swap_rows (l_j, l_i_pivot).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						a_p.item.gsl_permutation_swap (l_j, l_i_pivot).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						a_signum.item := -a_signum.item
						update_class_fields -- Current might have been changed; synchronize
					end
					l_ajj := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
					update_class_fields -- Current might have been changed; synchronize
					if l_ajj /= 0.0 then
						l_i := l_j + {NATURAL_32} 1
						from
						until
							not (not (l_return) and then (l_i < l_N))
						loop
							l_tmp___3 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_aij___0 := l_tmp___3 / l_ajj;
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, l_aij___0)
							update_class_fields -- Current might have been changed; synchronize
							l_k := l_j + {NATURAL_32} 1
							from
							until
								not (l_k < l_N)
							loop
								l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_aik := l_tmp___4
								l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_ajk := l_tmp___5;
								((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_k, l_aik - l_aij___0 * l_ajk)
								update_class_fields -- Current might have been changed; synchronize
								l_k := l_k + 1
							end
							l_i := l_i + 1
						end
					end
					l_j := l_j + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LU_solve (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LU matrix must be square"), ce_string ("lu.c"), 131, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_p.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation length must match matrix size"), ce_string ("lu.c"), 135, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("lu.c"), 139, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("lu.c"), 143, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.singular
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp.to_boolean then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix is singular"), ce_string ("lu.c"), 147, 1)
						update_class_fields -- Current might have been changed; synchronize
						Result := 1
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				else
					a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_status := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_LU_svx (a_p, a_x)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_status
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LU_svx (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LU matrix must be square"), ce_string ("lu.c"), 171, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_p.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation length must match matrix size"), ce_string ("lu.c"), 175, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution/rhs size"), ce_string ("lu.c"), 179, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.singular
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp.to_boolean then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix is singular"), ce_string ("lu.c"), 183, 1)
						update_class_fields -- Current might have been changed; synchronize
						Result := 1
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				else
					a_p.item.gsl_permute_vector (a_x).do_nothing
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasUnit, a_x).do_nothing
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LU_refine (a_LU: CE_POINTER [PD_GSL_MATRIX]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_b___0, a_x, a_residual: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix a must be square"), ce_string ("lu.c"), 209, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_LU.item.size1 /= a_LU.item.size2 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("LU matrix must be square"), ce_string ("lu.c"), 213, 20)
						update_class_fields -- Current might have been changed; synchronize
						Result := 20
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				elseif size1 /= a_LU.item.size2 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("LU matrix must be decomposition of a"), ce_string ("lu.c"), 217, 20)
						update_class_fields -- Current might have been changed; synchronize
						Result := 20
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				elseif a_LU.item.size1 /= a_p.item.size then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation length must match matrix size"), ce_string ("lu.c"), 221, 19)
						update_class_fields -- Current might have been changed; synchronize
						Result := 19
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				elseif a_LU.item.size1 /= a_b___0.item.size then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("lu.c"), 225, 19)
						update_class_fields -- Current might have been changed; synchronize
						Result := 19
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				elseif a_LU.item.size1 /= a_x.item.size then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("lu.c"), 229, 19)
						update_class_fields -- Current might have been changed; synchronize
						Result := 19
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				else
					l_tmp := a_LU.item.singular
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp.to_boolean then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix is singular"), ce_string ("lu.c"), 233, 1)
							update_class_fields -- Current might have been changed; synchronize
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					else
						a_residual.item.gsl_vector_memcpy (a_b___0).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, a_x, -1.0, a_residual).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_status := a_LU.item.gsl_linalg_LU_svx (a_p, a_residual)
						update_class_fields -- Current might have been changed; synchronize
						a_residual.item.gsl_blas_daxpy (-1.0, a_x).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						Result := l_status
						l_return := True
					end
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LU_det (a_signum: INTEGER_32): REAL_64
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_det: REAL_64
			l_tmp: REAL_64
		do
			l_n := size1
			l_det := a_signum.to_double
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_det := l_det * l_tmp
				l_i := l_i + 1
			end
			Result := l_det
		end

	gsl_linalg_LU_lndet: REAL_64
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_lndet: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
		do
			l_n := size1
			l_lndet := 0.0
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := fabs (l_tmp)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___1 := log (l_tmp___0)
				update_class_fields -- Current might have been changed; synchronize
				l_lndet := l_lndet + l_tmp___1
				l_i := l_i + 1
			end
			Result := l_lndet
		end

	gsl_linalg_LU_sgndet (a_signum: INTEGER_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_s: INTEGER_32
			l_u: REAL_64
			l_tmp: REAL_64
			l_break: BOOLEAN
		do
			l_n := size1
			l_s := a_signum
			l_i := (0).to_natural_32
			from
			until
				not (not (l_break) and then (l_i < l_n))
			loop
				l_break := False
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_u := l_tmp
				if l_u < (0).to_double then
					l_s := l_s * (-1)
				elseif l_u = (0).to_double then
					l_s := 0
					l_break := True
				end
				if not (l_break) then
					l_i := l_i + 1
				end
			end
			l_break := False
			Result := l_s
		end

	singular: INTEGER_32
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_u: REAL_64
			l_tmp: REAL_64
			l_return: BOOLEAN
		do
			l_n := size1
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_n))
			loop
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_u := l_tmp
				if l_u = (0).to_double then
					Result := 1
					l_return := True
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_bidiag_decomp (a_tau_U, a_tau_V: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_v: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_tau_i: REAL_64
			l_tmp___1: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___2: PD_GSL_MATRIX_VIEW
			l_r___3: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___3: PD_GSL_VECTOR_VIEW
			l_v___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___4: PD_GSL_VECTOR_VIEW
			l_tau_i___0: REAL_64
			l_tmp___5: REAL_64
			l_m___5: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___6: PD_GSL_MATRIX_VIEW
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_r___3.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___5.make (<<1>>)

			if size1 < size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("bidiagonal decomposition requires M>=N"), ce_string ("bidiag.c"), 79, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_tau_U.item.size /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau_U must be N"), ce_string ("bidiag.c"), 83, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_tau_V.item.size + {NATURAL_32} 1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau_V must be (N - 1)"), ce_string ("bidiag.c"), 87, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_M := size1
				l_N := size2
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_c___1.item := l_tmp.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i, l_M - l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_v.item := l_tmp___0.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___1 := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
					update_class_fields -- Current might have been changed; synchronize
					l_tau_i := l_tmp___1
					if l_i + {NATURAL_32} 1 < l_N then
						l_tmp___2 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i, l_i + {NATURAL_32} 1, l_M - l_i, l_N - (l_i + {NATURAL_32} 1))).twin
						update_class_fields -- Current might have been changed; synchronize
						l_m___4.item := l_tmp___2.twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					a_tau_U.item.gsl_vector_set (l_i, l_tau_i)
					update_class_fields -- Current might have been changed; synchronize
					if l_i + {NATURAL_32} 1 < l_N then
						l_tmp___3 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_r___3.item := l_tmp___3.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1))).twin
						update_class_fields -- Current might have been changed; synchronize
						l_v___0.item := l_tmp___4.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___5 := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
						update_class_fields -- Current might have been changed; synchronize
						l_tau_i___0 := l_tmp___5
						if l_i + {NATURAL_32} 1 < l_M then
							l_tmp___6 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i + {NATURAL_32} 1, l_i + {NATURAL_32} 1, l_M - (l_i + {NATURAL_32} 1), l_N - (l_i + {NATURAL_32} 1))).twin
							update_class_fields -- Current might have been changed; synchronize
							l_m___5.item := l_tmp___6.twin
							update_class_fields -- Current might have been changed; synchronize
							gsl_linalg_householder_mh (l_tau_i___0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___5.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						a_tau_V.item.gsl_vector_set (l_i, l_tau_i___0)
						update_class_fields -- Current might have been changed; synchronize
					end
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_bidiag_unpack (a_tau_U: CE_POINTER [PD_GSL_VECTOR]; a_U: CE_POINTER [PD_GSL_MATRIX]; a_tau_V: CE_POINTER [PD_GSL_VECTOR]; a_V: CE_POINTER [PD_GSL_MATRIX]; a_diag, a_superdiag: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_K: NATURAL_32
			l_tmp: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_Aii: REAL_64
			l_tmp___0: REAL_64
			l_Aij: REAL_64
			l_tmp___1: REAL_64
			l_r___3: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___2: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___3: PD_GSL_VECTOR_CONST_VIEW
			l_ti: REAL_64
			l_tmp___4: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___5: PD_GSL_MATRIX_VIEW
			l_tmp___6: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___7: PD_GSL_VECTOR_CONST_VIEW
			l_h___0: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___8: PD_GSL_VECTOR_CONST_VIEW
			l_tj: REAL_64
			l_tmp___9: REAL_64
			l_m___5: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___10: PD_GSL_MATRIX_VIEW
			l_tmp___11: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_r___3.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___5.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if l_M < l_N then
						l_tmp := l_M
					else
						l_tmp := l_N
					end
					l_K := l_tmp
					if not (l_M < l_N) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must have M >= N"), ce_string ("bidiag.c"), 161, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 3 then -- else:
						if a_tau_U.item.size = l_K then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("bidiag.c"), 165, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 6 then -- else:
						if a_tau_V.item.size + {NATURAL_32} 1 = l_K then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N) - 1"), ce_string ("bidiag.c"), 169, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 9 then -- else:
						if a_U.item.size1 = l_M then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						l_pos := -1 + 13 -- goto _L___0
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 12 then -- else:
						if a_U.item.size2 = l_N then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of U must be M x N"), ce_string ("bidiag.c"), 173, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 15 then -- else:
						if a_V.item.size1 = l_N then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						l_pos := -1 + 19 -- goto _L
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 18 then -- else:
						if a_V.item.size2 = l_N then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of V must be N x N"), ce_string ("bidiag.c"), 177, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else:
						if a_diag.item.size /= l_K then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of diagonal must match size of A"), ce_string ("bidiag.c"), 181, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_superdiag.item.size + {NATURAL_32} 1 /= l_K then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of subdiagonal must be (diagonal size - 1)"), ce_string ("bidiag.c"), 185, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_i := (0).to_natural_32
							from
							until
								not (l_i < l_N)
							loop
								l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_Aii := l_tmp___0
								a_diag.item.gsl_vector_set (l_i, l_Aii)
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
							l_i := (0).to_natural_32
							from
							until
								not (l_i < l_N - {NATURAL_32} 1)
							loop
								l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i + {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_Aij := l_tmp___1
								a_superdiag.item.gsl_vector_set (l_i, l_Aij)
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
							a_V.item.gsl_matrix_set_identity
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_N - {NATURAL_32} 1
							from
							until
								(l_break)
							loop
								l_break := False
								l_tmp___6 := l_i
								l_i := l_i - 1
								if not (l_tmp___6 > {NATURAL_32} 0) then
									l_break := True
								end
								if not (l_return or l_break) then
									l_tmp___2 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_row (l_i)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_r___3.item := l_tmp___2.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___3 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1))).twin
									update_class_fields -- Current might have been changed; synchronize
									l_h.item := l_tmp___3.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___4 := a_tau_V.item.gsl_vector_get (l_i)
									update_class_fields -- Current might have been changed; synchronize
									l_ti := l_tmp___4
									l_tmp___5 := (a_V.item.gsl_matrix_submatrix (l_i + {NATURAL_32} 1, l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1), l_N - (l_i + {NATURAL_32} 1))).twin
									update_class_fields -- Current might have been changed; synchronize
									l_m___4.item := l_tmp___5.twin
									update_class_fields -- Current might have been changed; synchronize
									gsl_linalg_householder_hm (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							l_break := False
							a_U.item.gsl_matrix_set_identity
							update_class_fields -- Current might have been changed; synchronize
							l_j := l_N
							from
							until
								(l_break)
							loop
								l_break := False
								l_tmp___11 := l_j
								l_j := l_j - 1
								if not (l_tmp___11 > {NATURAL_32} 0) then
									l_break := True
								end
								if not (l_return or l_break) then
									l_tmp___7 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_column (l_j)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_c___1.item := l_tmp___7.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___8 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_j, l_M - l_j)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_h___0.item := l_tmp___8.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___9 := a_tau_U.item.gsl_vector_get (l_j)
									update_class_fields -- Current might have been changed; synchronize
									l_tj := l_tmp___9
									l_tmp___10 := (a_U.item.gsl_matrix_submatrix (l_j, l_j, l_M - l_j, l_N - l_j)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_m___5.item := l_tmp___10.twin
									update_class_fields -- Current might have been changed; synchronize
									gsl_linalg_householder_hm (l_tj, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h___0.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___5.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							l_break := False
							Result := 0
							l_return := True
						end
				when 22 then -- if done | if done | if done | if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 23 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_bidiag_unpack2 (a_tau_U, a_tau_V: CE_POINTER [PD_GSL_VECTOR]; a_V: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_K: NATURAL_32
			l_tmp: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_r___3: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___1: PD_GSL_VECTOR_CONST_VIEW
			l_ti: REAL_64
			l_tmp___2: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___3: PD_GSL_MATRIX_VIEW
			l_tmp___4: NATURAL_32
			l_Aij: REAL_64
			l_tmp___5: REAL_64
			l_tj: REAL_64
			l_tmp___6: REAL_64
			l_Ajj: REAL_64
			l_tmp___7: REAL_64
			l_m___5: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___8: PD_GSL_MATRIX_VIEW
			l_tmp___9: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_r___3.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___5.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if l_M < l_N then
						l_tmp := l_M
					else
						l_tmp := l_N
					end
					l_K := l_tmp
					if not (l_M < l_N) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must have M >= N"), ce_string ("bidiag.c"), 260, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 3 then -- else:
						if a_tau_U.item.size = l_K then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("bidiag.c"), 264, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 6 then -- else:
						if a_tau_V.item.size + {NATURAL_32} 1 = l_K then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N) - 1"), ce_string ("bidiag.c"), 268, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 9 then -- else:
						if a_V.item.size1 = l_N then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 12 then -- else:
						if a_V.item.size2 = l_N then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of V must be N x N"), ce_string ("bidiag.c"), 272, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 15 then -- else:
						a_V.item.gsl_matrix_set_identity
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_N - {NATURAL_32} 1
						from
						until
							(l_break)
						loop
							l_break := False
							l_tmp___4 := l_i
							l_i := l_i - 1
							if not (l_tmp___4 > {NATURAL_32} 0) then
								l_break := True
							end
							if not (l_return or l_break) then
								l_tmp___0 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_row (l_i)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_r___3.item := l_tmp___0.twin
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___1 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1))).twin
								update_class_fields -- Current might have been changed; synchronize
								l_h.item := l_tmp___1.twin
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___2 := a_tau_V.item.gsl_vector_get (l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_ti := l_tmp___2
								l_tmp___3 := (a_V.item.gsl_matrix_submatrix (l_i + {NATURAL_32} 1, l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1), l_N - (l_i + {NATURAL_32} 1))).twin
								update_class_fields -- Current might have been changed; synchronize
								l_m___4.item := l_tmp___3.twin
								update_class_fields -- Current might have been changed; synchronize
								gsl_linalg_householder_hm (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							end
						end
						l_break := False
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_N - {NATURAL_32} 1)
						loop
							l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i + {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_Aij := l_tmp___5
							a_tau_V.item.gsl_vector_set (l_i, l_Aij)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_j := l_N
						from
						until
							(l_break)
						loop
							l_break := False
							l_tmp___9 := l_j
							l_j := l_j - 1
							if not (l_tmp___9 > {NATURAL_32} 0) then
								l_break := True
							end
							if not (l_return or l_break) then
								l_tmp___6 := a_tau_U.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_tj := l_tmp___6
								l_tmp___7 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_Ajj := l_tmp___7
								l_tmp___8 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_j, l_j, l_M - l_j, l_N - l_j)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_m___5.item := l_tmp___8.twin
								update_class_fields -- Current might have been changed; synchronize
								a_tau_U.item.gsl_vector_set (l_j, l_Ajj)
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___5.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_linalg_householder_hm1 (l_tj).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							end
						end
						l_break := False
						Result := 0
						l_return := True
				when 16 then -- if done | if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 17 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_bidiag_unpack_B (a_diag, a_superdiag: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_diag.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_K: NATURAL_32
			l_tmp: NATURAL_32
			l_i: NATURAL_32
			l_Aii: REAL_64
			l_tmp___0: REAL_64
			l_Aij: REAL_64
			l_tmp___1: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			l_N := size2
			if l_M < l_N then
				l_tmp := l_M
			else
				l_tmp := l_N
			end
			l_K := l_tmp
			if a_diag.item.size /= l_K then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of diagonal must match size of A"), ce_string ("bidiag.c"), 336, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_superdiag.item.size + {NATURAL_32} 1 /= l_K then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of subdiagonal must be (matrix size - 1)"), ce_string ("bidiag.c"), 340, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_K)
				loop
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_Aii := l_tmp___0
					a_diag.item.gsl_vector_set (l_i, l_Aii)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_K - {NATURAL_32} 1)
				loop
					l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i + {NATURAL_32} 1)
					update_class_fields -- Current might have been changed; synchronize
					l_Aij := l_tmp___1
					a_superdiag.item.gsl_vector_set (l_i, l_Aij)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_symmtd_decomp (a_tau: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_v: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_tau_i: REAL_64
			l_tmp___1: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___2: PD_GSL_MATRIX_VIEW
			l_ei: REAL_64
			l_tmp___3: REAL_64
			l_x: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___4: PD_GSL_VECTOR_VIEW
			l_xv: CE_ARRAY [REAL_64]
			l_alpha: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_x.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_xv.make (<<1>>)

			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("symmetric tridiagonal decomposition requires square matrix"), ce_string ("symmtd.c"), 69, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_tau.item.size + {NATURAL_32} 1 /= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be (matrix size - 1)"), ce_string ("symmtd.c"), 73, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_N := size1
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N - {NATURAL_32} 2)
				loop
					l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_c___1.item := l_tmp.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1))).twin
					update_class_fields -- Current might have been changed; synchronize
					l_v.item := l_tmp___0.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___1 := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
					update_class_fields -- Current might have been changed; synchronize
					l_tau_i := l_tmp___1
					if l_tau_i /= 0.0 then
						l_tmp___2 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i + {NATURAL_32} 1, l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1), l_N - (l_i + {NATURAL_32} 1))).twin
						update_class_fields -- Current might have been changed; synchronize
						l_m___4.item := l_tmp___2.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_ei := l_tmp___3
						l_tmp___4 := (a_tau.item.gsl_vector_subvector (l_i, l_N - (l_i + {NATURAL_32} 1))).twin
						update_class_fields -- Current might have been changed; synchronize
						l_x.item := l_tmp___4.twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set ((0).to_natural_32, 1.0)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_blas_dsymv ({PC_CBLAS_UPLO}.CblasLower, l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), 0.0, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_xv).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_alpha := -(l_tau_i / 2.0) * l_xv.item;
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (l_alpha, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						gsl_blas_dsyr2 ({PC_CBLAS_UPLO}.CblasLower, -1.0, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set ((0).to_natural_32, l_ei)
						update_class_fields -- Current might have been changed; synchronize
					end
					a_tau.item.gsl_vector_set (l_i, l_tau_i)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_symmtd_unpack (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_Q___1: CE_POINTER [PD_GSL_MATRIX]; a_diag, a_sdiag: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_ti: REAL_64
			l_tmp___1: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___2: PD_GSL_MATRIX_VIEW
			l_tmp___3: NATURAL_32
			l_Aii: REAL_64
			l_tmp___4: REAL_64
			l_Aji: REAL_64
			l_tmp___5: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if size1 = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must be square"), ce_string ("symmtd.c"), 132, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if a_tau.item.size + {NATURAL_32} 1 = size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be (matrix size - 1)"), ce_string ("symmtd.c"), 136, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if a_Q___1.item.size1 = size1 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if a_Q___1.item.size2 = size1 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of Q must match size of A"), ce_string ("symmtd.c"), 140, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						if a_diag.item.size /= size1 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of diagonal must match size of A"), ce_string ("symmtd.c"), 144, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_sdiag.item.size + {NATURAL_32} 1 /= size1 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of subdiagonal must be (matrix size - 1)"), ce_string ("symmtd.c"), 148, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_N := size1
							a_Q___1.item.gsl_matrix_set_identity
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_N - {NATURAL_32} 2
							from
							until
								(l_break)
							loop
								l_break := False
								l_tmp___3 := l_i
								l_i := l_i - 1
								if not (l_tmp___3 > {NATURAL_32} 0) then
									l_break := True
								end
								if not (l_return or l_break) then
									l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_column (l_i)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_c___1.item := l_tmp.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___0 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1))).twin
									update_class_fields -- Current might have been changed; synchronize
									l_h.item := l_tmp___0.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___1 := a_tau.item.gsl_vector_get (l_i)
									update_class_fields -- Current might have been changed; synchronize
									l_ti := l_tmp___1
									l_tmp___2 := (a_Q___1.item.gsl_matrix_submatrix (l_i + {NATURAL_32} 1, l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1), l_N - (l_i + {NATURAL_32} 1))).twin
									update_class_fields -- Current might have been changed; synchronize
									l_m___4.item := l_tmp___2.twin
									update_class_fields -- Current might have been changed; synchronize
									gsl_linalg_householder_hm (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							l_break := False
							l_i := (0).to_natural_32
							from
							until
								not (l_i < l_N)
							loop
								l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_Aii := l_tmp___4
								a_diag.item.gsl_vector_set (l_i, l_Aii)
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
							l_i := (0).to_natural_32
							from
							until
								not (l_i < l_N - {NATURAL_32} 1)
							loop
								l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i + {NATURAL_32} 1, l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_Aji := l_tmp___5
								a_sdiag.item.gsl_vector_set (l_i, l_Aji)
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
							Result := 0
							l_return := True
						end
				when 13 then -- if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_symmtd_unpack_T (a_diag, a_sdiag: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_Aii: REAL_64
			l_tmp: REAL_64
			l_Aij: REAL_64
			l_tmp___0: REAL_64
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must be square"), ce_string ("symmtd.c"), 198, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_diag.item.size /= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of diagonal must match size of A"), ce_string ("symmtd.c"), 202, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_sdiag.item.size + {NATURAL_32} 1 /= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of subdiagonal must be (matrix size - 1)"), ce_string ("symmtd.c"), 206, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_N := size1
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_Aii := l_tmp
					a_diag.item.gsl_vector_set (l_i, l_Aii)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N - {NATURAL_32} 1)
				loop
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i + {NATURAL_32} 1, l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_Aij := l_tmp___0
					a_sdiag.item.gsl_vector_set (l_i, l_Aij)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_balance_matrix (a_D: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_D.is_void
		local
			l_N: NATURAL_32
			l_row_norm: REAL_64
			l_col_norm: REAL_64
			l_not_converged: INTEGER_32
			l_v: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_g: REAL_64
			l_f: REAL_64
			l_s: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_return, l_break, l_continue: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v.make (<<1>>)

			l_N := size1
			if l_N /= a_D.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must match matrix size"), ce_string ("balancemat.c"), 54, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_D.item.gsl_vector_set_all (1.0)
				update_class_fields -- Current might have been changed; synchronize
				l_not_converged := 1
				from
				until
					not (not (l_return) and then (l_not_converged.to_boolean))
				loop
					l_not_converged := 0
					l_i := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_i < l_N))
					loop
						l_continue := False
						l_row_norm := 0.0
						l_col_norm := 0.0
						l_j := (0).to_natural_32
						from
						until
							not (l_j < l_N)
						loop
							if l_j /= l_i then
								l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___0 := fabs (l_tmp)
								update_class_fields -- Current might have been changed; synchronize
								l_col_norm := l_col_norm + l_tmp___0
								l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___2 := fabs (l_tmp___1)
								update_class_fields -- Current might have been changed; synchronize
								l_row_norm := l_row_norm + l_tmp___2
							end
							l_j := l_j + 1
						end
						if l_col_norm = 0.0 then
							l_continue := True
						elseif l_row_norm = 0.0 then
							l_continue := True
						end
						if not (l_return or l_break or l_continue) then
							l_g := l_row_norm / 2.0
							l_f := 1.0
							l_s := l_col_norm + l_row_norm
							from
							until
								not (l_col_norm < l_g)
							loop
								l_f := l_f * (2.0)
								l_col_norm := l_col_norm * (2.0 * 2.0)
							end
							l_g := l_row_norm * 2.0
							from
							until
								not (l_col_norm > l_g)
							loop
								l_f := l_f / (2.0)
								l_col_norm := l_col_norm / (2.0 * 2.0)
							end
							if l_row_norm + l_col_norm < (0.95 * l_s) * l_f then
								l_not_converged := 1
								l_g := 1.0 / l_f
								l_v.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_i)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_g)
								update_class_fields -- Current might have been changed; synchronize
								l_v.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i)).twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_f)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___3 := a_D.item.gsl_vector_get (l_i)
								update_class_fields -- Current might have been changed; synchronize
								a_D.item.gsl_vector_set (l_i, l_tmp___3 * l_f)
								update_class_fields -- Current might have been changed; synchronize
							end
						end
						if (not l_return) then
							l_i := l_i + 1
						end
					end
					l_continue := False
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_balance_accum (a_D: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_D.is_void
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_s: REAL_64
			l_r___3: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_r___3.make (<<1>>)

			l_N := size1
			if l_N /= a_D.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must match matrix size"), ce_string ("balancemat.c"), 168, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_s := a_D.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_r___3.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_i)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_r___3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_s)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	matrix_exp_series (a_eB: CE_POINTER [PD_GSL_MATRIX]; a_number_of_terms: INTEGER_32)
		require
			not a_eB.is_void
		local
			l_count: INTEGER_32
			l_temp: CE_POINTER [PD_GSL_MATRIX]
			l_tmp: CE_POINTER [PD_GSL_MATRIX]
		do
			l_tmp := (create {P_INIT_SOURCE_STATIC}).gsl_matrix_calloc (size1, size2)
			update_class_fields -- Current might have been changed; synchronize
			l_temp := l_tmp
			a_eB.item.gsl_matrix_memcpy ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			a_eB.item.gsl_matrix_scale (1.0 / a_number_of_terms.to_double).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			a_eB.item.gsl_matrix_add_diagonal (1.0).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			l_count := a_number_of_terms - 1
			from
			until
				not (l_count >= 1)
			loop
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dgemm ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, a_eB, 0.0, l_temp).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_temp.item.gsl_matrix_scale (1.0 / l_count.to_double).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_temp.item.gsl_matrix_add_diagonal (1.0).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				a_eB.item.gsl_matrix_memcpy (l_temp).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_count := l_count - 1
			end
			(create {P_INIT_SOURCE_STATIC}).gsl_matrix_free (l_temp)
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_linalg_exponential_ss (a_eA: CE_POINTER [PD_GSL_MATRIX]; a_mode: NATURAL_32): INTEGER_32
		local
			l_i: INTEGER_32
			l_sugg: PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION
			l_tmp: PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION
			l_divisor: REAL_64
			l_tmp___0: REAL_64
			l_reduced_A: CE_POINTER [PD_GSL_MATRIX]
			l_tmp___1: CE_POINTER [PD_GSL_MATRIX]
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if size1 = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("cannot exponentiate a non-square matrix"), ce_string ("exponential.c"), 154, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if size1 = a_eA.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if size2 = a_eA.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("exponential of matrix must have same dimension as matrix"), ce_string ("exponential.c"), 158, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.obtain_suggestion (a_mode)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_sugg := l_tmp.twin
						l_tmp___0 := exp (0.69314718055994530942 * (l_sugg.j).to_double)
						update_class_fields -- Current might have been changed; synchronize
						l_divisor := l_tmp___0
						l_tmp___1 := (create {P_INIT_SOURCE_STATIC}).gsl_matrix_alloc (size1, size2)
						update_class_fields -- Current might have been changed; synchronize
						l_reduced_A := l_tmp___1
						l_reduced_A.item.gsl_matrix_memcpy ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_reduced_A.item.gsl_matrix_scale (1.0 / l_divisor).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_reduced_A.item.matrix_exp_series (a_eA, l_sugg.k)
						update_class_fields -- Current might have been changed; synchronize
						l_i := 0
						from
						until
							not (l_i < l_sugg.j)
						loop
							a_eA.item.gsl_blas_dgemm ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, a_eA, 0.0, l_reduced_A).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_eA.item.gsl_matrix_memcpy (l_reduced_A).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						(create {P_INIT_SOURCE_STATIC}).gsl_matrix_free (l_reduced_A)
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
						l_return := True
				when 10 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_hessenberg_decomp (a_tau: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_hv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tau_i: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_hv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			l_N := size1
			if l_N /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("Hessenberg reduction requires square matrix"), ce_string ("hessenberg.c"), 74, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_tau.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("tau vector must match matrix size"), ce_string ("hessenberg.c"), 78, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N < {NATURAL_32} 3 then
				Result := 0
				l_return := True
			else
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N - {NATURAL_32} 2)
				loop
					l_c___1.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_i, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_hv.item := (a_tau.item.gsl_vector_subvector (l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1))).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_memcpy ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_tau_i := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
					update_class_fields -- Current might have been changed; synchronize
					l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i + {NATURAL_32} 1, l_i, l_N - (l_i + {NATURAL_32} 1), l_N - l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, l_i + {NATURAL_32} 1, l_N, l_N - (l_i + {NATURAL_32} 1))).twin
					update_class_fields -- Current might have been changed; synchronize
					gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					a_tau.item.gsl_vector_set (l_i, l_tau_i)
					update_class_fields -- Current might have been changed; synchronize
					l_c___1.item := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector ((1).to_natural_32, l_c___1.item.vector.size - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_hv.item := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector ((1).to_natural_32, l_hv.item.vector.size - {NATURAL_32} 1)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_memcpy ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_hessenberg_unpack_accum (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_V: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_N: NATURAL_32
			l_j: NATURAL_32
			l_tau_j: REAL_64
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_hv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_hv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			l_N := size1
			if l_N /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("Hessenberg reduction requires square matrix"), ce_string ("hessenberg.c"), 215, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_tau.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("tau vector must match matrix size"), ce_string ("hessenberg.c"), 219, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_V.item.size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("V matrix has wrong dimension"), ce_string ("hessenberg.c"), 223, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				if l_N < {NATURAL_32} 3 then
					Result := 0
					l_return := True
				end
				if not l_return then
					l_j := (0).to_natural_32
					from
					until
						not (l_j < l_N - {NATURAL_32} 2)
					loop
						l_c___1.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_tau_j := a_tau.item.gsl_vector_get (l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_hv.item := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_j + {NATURAL_32} 1, l_N - (l_j + {NATURAL_32} 1))).twin
						update_class_fields -- Current might have been changed; synchronize
						l_m___4.item := (a_V.item.gsl_matrix_submatrix ((0).to_natural_32, l_j + {NATURAL_32} 1, a_V.item.size1, l_N - (l_j + {NATURAL_32} 1))).twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_mh (l_tau_j, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_j := l_j + 1
					end
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_hessenberg_set_zero: INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return: BOOLEAN
		do
			l_N := size1
			if l_N < {NATURAL_32} 3 then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N - {NATURAL_32} 2))
				loop
					l_i := l_j + {NATURAL_32} 2
					from
					until
						not (l_i < l_N)
					loop
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, 0.0)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_linalg_hessenberg_submatrix (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_top: NATURAL_32; a_tau: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_A___0.is_void
		local
			l_N: NATURAL_32
			l_N_M: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_hv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tau_i: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_hv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			l_N := a_A___0.item.size1
			l_N_M := size1
			if l_N /= a_A___0.item.size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("Hessenberg reduction requires square matrix"), ce_string ("hessenberg.c"), 360, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_tau.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("tau vector must match matrix size"), ce_string ("hessenberg.c"), 364, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N < {NATURAL_32} 3 then
				Result := 0
				l_return := True
			else
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N - {NATURAL_32} 2)
				loop
					l_c___1.item := (a_A___0.item.gsl_matrix_subcolumn (l_i, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_hv.item := (a_tau.item.gsl_vector_subvector (l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1))).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_memcpy ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_tau_i := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
					update_class_fields -- Current might have been changed; synchronize
					l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((a_top + l_i) + {NATURAL_32} 1, a_top + l_i, l_N - (l_i + {NATURAL_32} 1), (l_N_M - a_top) - l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, (a_top + l_i) + {NATURAL_32} 1, a_top + l_N, l_N - (l_i + {NATURAL_32} 1))).twin
					update_class_fields -- Current might have been changed; synchronize
					gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					a_tau.item.gsl_vector_set (l_i, l_tau_i)
					update_class_fields -- Current might have been changed; synchronize
					l_c___1.item := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector ((1).to_natural_32, l_c___1.item.vector.size - {NATURAL_32} 1)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_hv.item := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector ((1).to_natural_32, l_hv.item.vector.size - {NATURAL_32} 1)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_memcpy ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_hessenberg (a_tau: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_hessenberg_decomp (a_tau)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_linalg_hesstri_decomp (a_B, a_U, a_V: CE_POINTER [PD_GSL_MATRIX]; a_work: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_cs: CE_ARRAY [REAL_64]
			l_sn: CE_ARRAY [REAL_64]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_yv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_cs.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_sn.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_yv.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := size1
					if l_N = size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if l_N = a_B.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if l_N = a_B.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Hessenberg-triangular reduction requires square matrices"), ce_string ("hesstri.c"), 65, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						if l_N /= a_work.item.size then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of workspace must match matrix dimension"), ce_string ("hesstri.c"), 70, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							a_B.item.gsl_linalg_QR_decomp (a_work).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_B.item.gsl_linalg_QR_QTmat (a_work, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							if (not a_U.is_void) then
								a_B.item.gsl_linalg_QR_unpack (a_work, a_U, a_B).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							else
								l_j := (0).to_natural_32
								from
								until
									not (not (l_return) and then (l_j < l_N - {NATURAL_32} 1))
								loop
									l_i := l_j + {NATURAL_32} 1
									from
									until
										not (l_i < l_N)
									loop
										a_B.item.gsl_matrix_set (l_i, l_j, 0.0)
										update_class_fields -- Current might have been changed; synchronize
										l_i := l_i + 1
									end
									l_j := l_j + 1
								end
							end
							if (not a_V.is_void) then
								a_V.item.gsl_matrix_set_identity
								update_class_fields -- Current might have been changed; synchronize
							end
							if l_N < {NATURAL_32} 3 then
								Result := 0
								l_return := True
							end
							if not l_return then
								l_j := (0).to_natural_32
								from
								until
									not (not (l_return) and then (l_j < l_N - {NATURAL_32} 2))
								loop
									l_i := l_N - {NATURAL_32} 1
									from
									until
										not (l_i >= l_j + {NATURAL_32} 2)
									loop
										l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i - {NATURAL_32} 1, l_j)
										update_class_fields; -- Current might have been changed; synchronize
										(create {P_QRSTEP_STATIC}).create_givens (l_tmp___0, l_tmp, l_cs, l_sn)
										update_class_fields -- Current might have been changed; synchronize
										l_sn.item := -l_sn.item
										update_class_fields -- Current might have been changed; synchronize
										l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subrow (l_i - {NATURAL_32} 1, l_j, l_N - l_j)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subrow (l_i, l_j, l_N - l_j)).twin
										update_class_fields; -- Current might have been changed; synchronize
										(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
										update_class_fields -- Current might have been changed; synchronize
										l_xv.item := (a_B.item.gsl_matrix_subrow (l_i - {NATURAL_32} 1, l_i - {NATURAL_32} 1, (l_N - l_i) + {NATURAL_32} 1)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_yv.item := (a_B.item.gsl_matrix_subrow (l_i, l_i - {NATURAL_32} 1, (l_N - l_i) + {NATURAL_32} 1)).twin
										update_class_fields; -- Current might have been changed; synchronize
										(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
										update_class_fields -- Current might have been changed; synchronize
										if (not a_U.is_void) then
											l_xv.item := (a_U.item.gsl_matrix_column (l_i - {NATURAL_32} 1)).twin
											update_class_fields -- Current might have been changed; synchronize
											l_yv.item := (a_U.item.gsl_matrix_column (l_i)).twin
											update_class_fields; -- Current might have been changed; synchronize
											(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
											update_class_fields -- Current might have been changed; synchronize
										end
										l_tmp___1 := a_B.item.gsl_matrix_get (l_i, l_i - {NATURAL_32} 1)
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___2 := a_B.item.gsl_matrix_get (l_i, l_i)
										update_class_fields; -- Current might have been changed; synchronize
										(create {P_QRSTEP_STATIC}).create_givens (-l_tmp___2, l_tmp___1, l_cs, l_sn)
										update_class_fields -- Current might have been changed; synchronize
										l_sn.item := -l_sn.item
										update_class_fields -- Current might have been changed; synchronize
										l_xv.item := (a_B.item.gsl_matrix_subcolumn (l_i - {NATURAL_32} 1, (0).to_natural_32, l_i + {NATURAL_32} 1)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_yv.item := (a_B.item.gsl_matrix_subcolumn (l_i, (0).to_natural_32, l_i + {NATURAL_32} 1)).twin
										update_class_fields; -- Current might have been changed; synchronize
										(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
										update_class_fields -- Current might have been changed; synchronize
										l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i - {NATURAL_32} 1)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i)).twin
										update_class_fields; -- Current might have been changed; synchronize
										(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
										update_class_fields -- Current might have been changed; synchronize
										if (not a_V.is_void) then
											l_xv.item := (a_V.item.gsl_matrix_column (l_i - {NATURAL_32} 1)).twin
											update_class_fields -- Current might have been changed; synchronize
											l_yv.item := (a_V.item.gsl_matrix_column (l_i)).twin
											update_class_fields; -- Current might have been changed; synchronize
											(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cs.item, l_sn.item).do_nothing
											update_class_fields -- Current might have been changed; synchronize
										end
										l_i := l_i - 1
									end
									l_j := l_j + 1
								end
								Result := 0
								l_return := True
							end
						end
				when 10 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_QR_decomp (a_tau: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_tau.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c_full: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_tau_i: REAL_64
			l_tmp___1: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___2: PD_GSL_MATRIX_VIEW
			l_tmp___3: NATURAL_32
			l_tmp___4: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c_full.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			l_M := size1
			l_N := size2
			if l_M < l_N then
				l_tmp___4 := l_M
			else
				l_tmp___4 := l_N
			end
			if a_tau.item.size /= l_tmp___4 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("qr.c"), 68, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := (0).to_natural_32
				from
				until
					(l_break)
				loop
					l_break := False
					if l_M < l_N then
						l_tmp___3 := l_M
					else
						l_tmp___3 := l_N
					end
					if not (l_i < l_tmp___3) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c_full.item := l_tmp.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c_full.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c___1.item := l_tmp___0.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
						update_class_fields -- Current might have been changed; synchronize
						l_tau_i := l_tmp___1
						a_tau.item.gsl_vector_set (l_i, l_tau_i)
						update_class_fields -- Current might have been changed; synchronize
						if l_i + {NATURAL_32} 1 < l_N then
							l_tmp___2 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i, l_i + {NATURAL_32} 1, l_M - l_i, l_N - (l_i + {NATURAL_32} 1))).twin
							update_class_fields -- Current might have been changed; synchronize
							l_m___4.item := l_tmp___2.twin
							update_class_fields -- Current might have been changed; synchronize
							gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						l_i := l_i + 1
					end
				end
				l_break := False
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QR_solve (a_tau, a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must be square"), ce_string ("qr.c"), 112, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("qr.c"), 116, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("qr.c"), 120, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_QR_svx (a_tau, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QR_svx (a_tau, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must be square"), ce_string ("qr.c"), 149, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match x/rhs size"), ce_string ("qr.c"), 153, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_QR_QTvec (a_tau, a_x).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QR_lssolve (a_tau, a_b___0, a_x, a_residual: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_R: CE_ARRAY [PD_GSL_MATRIX_CONST_VIEW]
			l_tmp: PD_GSL_MATRIX_CONST_VIEW
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_CONST_VIEW]} l_R.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)

			l_M := size1
			l_N := size2
			if l_M < l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must have M>=N"), ce_string ("qr.c"), 185, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_M /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("qr.c"), 189, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("qr.c"), 193, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_M /= a_residual.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match residual size"), ce_string ("qr.c"), 197, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_R.item := l_tmp.twin
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := (a_residual.item.gsl_vector_subvector ((0).to_natural_32, l_N)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_c___1.item := l_tmp___0.twin
				update_class_fields -- Current might have been changed; synchronize
				a_residual.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_QR_QTvec (a_tau, a_residual).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				a_x.item.gsl_vector_memcpy ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_R.item.c_pointer + (create {PD_GSL_MATRIX_CONST_VIEW}.make).offset_of ("matrix"))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set_zero
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_QR_Qvec (a_tau, a_residual).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QR_Rsolve (a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must be square"), ce_string ("qr.c"), 232, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("qr.c"), 236, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match x size"), ce_string ("qr.c"), 240, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QR_Rsvx (a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must be square"), ce_string ("qr.c"), 262, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match rhs size"), ce_string ("qr.c"), 266, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_R_solve (a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("R matrix must be square"), ce_string ("qr.c"), 283, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("qr.c"), 287, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("qr.c"), 291, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_R_svx (a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("R matrix must be square"), ce_string ("qr.c"), 312, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("qr.c"), 316, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QR_QTvec (a_tau, a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_tau.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_w: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: PD_GSL_VECTOR_VIEW
			l_ti: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: NATURAL_32
			l_tmp___4: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_w.make (<<1>>)

			l_M := size1
			l_N := size2
			if l_M < l_N then
				l_tmp___4 := l_M
			else
				l_tmp___4 := l_N
			end
			if a_tau.item.size /= l_tmp___4 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("qr.c"), 341, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_v.item.size /= l_M then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector size must be N"), ce_string ("qr.c"), 345, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := (0).to_natural_32
				from
				until
					(l_break)
				loop
					l_break := False
					if l_M < l_N then
						l_tmp___3 := l_M
					else
						l_tmp___3 := l_N
					end
					if not (l_i < l_tmp___3) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_column (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c___1.item := l_tmp.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_h.item := l_tmp___0.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := (a_v.item.gsl_vector_subvector (l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_w.item := l_tmp___1.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := a_tau.item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_ti := l_tmp___2
						gsl_linalg_householder_hv (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_w.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				end
				l_break := False
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QR_Qvec (a_tau, a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_tau.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_w: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: PD_GSL_VECTOR_VIEW
			l_ti: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: NATURAL_32
			l_tmp___4: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_w.make (<<1>>)

			l_M := size1
			l_N := size2
			if l_M < l_N then
				l_tmp___4 := l_M
			else
				l_tmp___4 := l_N
			end
			if a_tau.item.size /= l_tmp___4 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("qr.c"), 374, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_v.item.size /= l_M then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector size must be N"), ce_string ("qr.c"), 378, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				if l_M < l_N then
					l_i := l_M
				else
					l_i := l_N
				end
				from
				until
					(l_break)
				loop
					l_break := False
					l_tmp___3 := l_i
					l_i := l_i - 1
					if not (l_tmp___3 > {NATURAL_32} 0) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_column (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c___1.item := l_tmp.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_h.item := l_tmp___0.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := (a_v.item.gsl_vector_subvector (l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_w.item := l_tmp___1.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := a_tau.item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_ti := l_tmp___2
						gsl_linalg_householder_hv (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_w.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				l_break := False
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QR_QTmat (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_A___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_tau.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___1: PD_GSL_MATRIX_VIEW
			l_ti: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: NATURAL_32
			l_tmp___4: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			l_M := size1
			l_N := size2
			if l_M < l_N then
				l_tmp___4 := l_M
			else
				l_tmp___4 := l_N
			end
			if a_tau.item.size /= l_tmp___4 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("qr.c"), 409, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_A___0.item.size1 /= l_M then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must have M rows"), ce_string ("qr.c"), 413, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := (0).to_natural_32
				from
				until
					(l_break)
				loop
					l_break := False
					if l_M < l_N then
						l_tmp___3 := l_M
					else
						l_tmp___3 := l_N
					end
					if not (l_i < l_tmp___3) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_column (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c___1.item := l_tmp.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_h.item := l_tmp___0.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := (a_A___0.item.gsl_matrix_submatrix (l_i, (0).to_natural_32, l_M - l_i, a_A___0.item.size2)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_m___4.item := l_tmp___1.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := a_tau.item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_ti := l_tmp___2
						gsl_linalg_householder_hm (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				end
				l_break := False
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QR_unpack (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_Q___1, a_R: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_Q___1.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___1: PD_GSL_MATRIX_VIEW
			l_ti: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: NATURAL_32
			l_tmp___4: REAL_64
			l_tmp___5: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_Q___1.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L___0
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if a_Q___1.item.size2 = l_M then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Q matrix must be M x M"), ce_string ("qr.c"), 444, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if a_R.item.size1 = l_M then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if a_R.item.size2 = l_N then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("R matrix must be M x N"), ce_string ("qr.c"), 448, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						if l_M < l_N then
							l_tmp___5 := l_M
						else
							l_tmp___5 := l_N
						end
						if a_tau.item.size /= l_tmp___5 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("qr.c"), 452, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							a_Q___1.item.gsl_matrix_set_identity
							update_class_fields -- Current might have been changed; synchronize
							if l_M < l_N then
								l_i := l_M
							else
								l_i := l_N
							end
							from
							until
								(l_break)
							loop
								l_break := False
								l_tmp___3 := l_i
								l_i := l_i - 1
								if not (l_tmp___3 > {NATURAL_32} 0) then
									l_break := True
								end
								if not (l_return or l_break) then
									l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_column (l_i)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_c___1.item := l_tmp.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___0 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i, l_M - l_i)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_h.item := l_tmp___0.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___1 := (a_Q___1.item.gsl_matrix_submatrix (l_i, l_i, l_M - l_i, l_M - l_i)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_m___4.item := l_tmp___1.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___2 := a_tau.item.gsl_vector_get (l_i)
									update_class_fields -- Current might have been changed; synchronize
									l_ti := l_tmp___2
									gsl_linalg_householder_hm (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							l_break := False
							l_i := (0).to_natural_32
							from
							until
								not (not (l_return) and then (l_i < l_M))
							loop
								l_j := (0).to_natural_32
								from
								until
									(l_break)
								loop
									l_break := False
									if l_j < l_i then
										if not (l_j < l_N) then
											l_break := True
										end
									else
										l_break := True
									end
									if not (l_return or l_break) then
										a_R.item.gsl_matrix_set (l_i, l_j, 0.0)
										update_class_fields -- Current might have been changed; synchronize
										l_j := l_j + 1
									end
								end
								l_break := False
								l_j := l_i
								from
								until
									not (l_j < l_N)
								loop
									l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
									update_class_fields -- Current might have been changed; synchronize
									a_R.item.gsl_matrix_set (l_i, l_j, l_tmp___4)
									update_class_fields -- Current might have been changed; synchronize
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
							Result := 0
							l_return := True
						end
				when 13 then -- if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_QR_update (a_R: CE_POINTER [PD_GSL_MATRIX]; a_w, a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_R.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_w0: REAL_64
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_wk: REAL_64
			l_tmp: REAL_64
			l_wkm1: REAL_64
			l_tmp___0: REAL_64
			l_r0j: REAL_64
			l_tmp___1: REAL_64
			l_vj: REAL_64
			l_tmp___2: REAL_64
			l_c___2: CE_ARRAY [REAL_64]
			l_s___0: CE_ARRAY [REAL_64]
			l_diag: REAL_64
			l_tmp___3: REAL_64
			l_offdiag: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_c___2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s___0.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_R.item.size1
					l_N := a_R.item.size2
					if size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if size2 = l_M then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Q matrix must be M x M if R is M x N"), ce_string ("qr.c"), 509, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_w.item.size /= l_M then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("w must be length M if R is M x N"), ce_string ("qr.c"), 513, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_v.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("v must be length N if R is M x N"), ce_string ("qr.c"), 517, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_k := l_M - {NATURAL_32} 1
							from
							until
								not (l_k > {NATURAL_32} 0)
							loop
								l_tmp := a_w.item.gsl_vector_get (l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_wk := l_tmp
								l_tmp___0 := a_w.item.gsl_vector_get (l_k - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_wkm1 := l_tmp___0;
								(create {P_QRSTEP_STATIC}).create_givens (l_wkm1, l_wk, l_c___1, l_s)
								update_class_fields -- Current might have been changed; synchronize
								apply_givens_vec (a_w, l_k - {NATURAL_32} 1, l_k, l_c___1.item, l_s.item)
								update_class_fields -- Current might have been changed; synchronize
								apply_givens_qr (l_M, l_N, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)), a_R, l_k - {NATURAL_32} 1, l_k, l_c___1.item, l_s.item)
								update_class_fields -- Current might have been changed; synchronize
								l_k := l_k - 1
							end
							l_w0 := a_w.item.gsl_vector_get ((0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								l_tmp___1 := a_R.item.gsl_matrix_get ((0).to_natural_32, l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_r0j := l_tmp___1
								l_tmp___2 := a_v.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_vj := l_tmp___2
								a_R.item.gsl_matrix_set ((0).to_natural_32, l_j, l_r0j + l_w0 * l_vj)
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_k := (1).to_natural_32
							from
							until
								(l_break)
							loop
								l_break := False
								if l_M < l_N + {NATURAL_32} 1 then
									l_tmp___5 := l_M
								else
									l_tmp___5 := l_N + {NATURAL_32} 1
								end
								if not (l_k < l_tmp___5) then
									l_break := True
								end
								if not (l_return or l_break) then
									l_tmp___3 := a_R.item.gsl_matrix_get (l_k - {NATURAL_32} 1, l_k - {NATURAL_32} 1)
									update_class_fields -- Current might have been changed; synchronize
									l_diag := l_tmp___3
									l_tmp___4 := a_R.item.gsl_matrix_get (l_k, l_k - {NATURAL_32} 1)
									update_class_fields -- Current might have been changed; synchronize
									l_offdiag := l_tmp___4;
									(create {P_QRSTEP_STATIC}).create_givens (l_diag, l_offdiag, l_c___2, l_s___0)
									update_class_fields -- Current might have been changed; synchronize
									apply_givens_qr (l_M, l_N, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)), a_R, l_k - {NATURAL_32} 1, l_k, l_c___2.item, l_s___0.item)
									update_class_fields -- Current might have been changed; synchronize
									a_R.item.gsl_matrix_set (l_k, l_k - {NATURAL_32} 1, 0.0)
									update_class_fields -- Current might have been changed; synchronize
									l_k := l_k + 1
								end
							end
							l_break := False
							Result := 0
							l_return := True
						end
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_matmult (a_B, a_C: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_B.is_void
		local
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_temp: REAL_64
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if size2 = a_B.item.size1 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if size1 = a_C.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if a_B.item.size2 = a_C.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix sizes are not conformant"), ce_string ("multiply.c"), 34, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < a_C.item.size1))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (not (l_return) and then (l_j < a_C.item.size2))
							loop
								l_a___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, (0).to_natural_32)
								update_class_fields -- Current might have been changed; synchronize
								l_b___0 := a_B.item.gsl_matrix_get ((0).to_natural_32, l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_temp := l_a___5 * l_b___0
								l_k := (1).to_natural_32
								from
								until
									not (l_k < size2)
								loop
									l_a___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_k)
									update_class_fields -- Current might have been changed; synchronize
									l_b___0 := a_B.item.gsl_matrix_get (l_k, l_j)
									update_class_fields -- Current might have been changed; synchronize
									l_temp := l_temp + (l_a___5 * l_b___0)
									l_k := l_k + 1
								end
								a_C.item.gsl_matrix_set (l_i, l_j, l_temp)
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 10 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_matmult_mod (a_modA: NATURAL_32; a_B: CE_POINTER [PD_GSL_MATRIX]; a_modB: NATURAL_32; a_C: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_B.is_void
		local
			l_tmp: INTEGER_32
			l_dim1_A: NATURAL_32
			l_dim2_A: NATURAL_32
			l_dim1_B: NATURAL_32
			l_dim2_B: NATURAL_32
			l_dim1_C: NATURAL_32
			l_dim2_C: NATURAL_32
			l_swap_tmp: NATURAL_32
			l_swap_tmp___0: NATURAL_32
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_temp: REAL_64
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_a1___4: NATURAL_32
			l_a2___3: NATURAL_32
			l_b1___0: NATURAL_32
			l_b2: NATURAL_32
			l_swap_tmp___1: NATURAL_32
			l_swap_tmp___2: NATURAL_32
			l_swap_tmp___3: NATURAL_32
			l_swap_tmp___4: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_modA /= {NATURAL_32} 0 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if a_modB = {NATURAL_32} 0 then
							l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_matmult (a_B, a_C)
							update_class_fields -- Current might have been changed; synchronize
							Result := l_tmp
							l_return := True
						else
							l_pos := -1 + 3 -- goto _L___0
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else: | _L___0:
						l_dim1_A := size1
						l_dim2_A := size2
						l_dim1_B := a_B.item.size1
						l_dim2_B := a_B.item.size2
						l_dim1_C := a_C.item.size1
						l_dim2_C := a_C.item.size2
						if ((a_modA).bit_and ({NATURAL_32} 1)).to_boolean then
							from
							until
								(l_break)
							loop
								l_break := False
								l_swap_tmp := l_dim1_A
								l_dim1_A := l_dim2_A
								l_dim2_A := l_swap_tmp
								l_break := True
							end
							l_break := False
						end
						if ((a_modB).bit_and ({NATURAL_32} 1)).to_boolean then
							from
							until
								(l_break)
							loop
								l_break := False
								l_swap_tmp___0 := l_dim1_B
								l_dim1_B := l_dim2_B
								l_dim2_B := l_swap_tmp___0
								l_break := True
							end
							l_break := False
						end
						if l_dim2_A = l_dim1_B then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if l_dim1_A = l_dim1_C then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if l_dim2_B = l_dim2_C then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix sizes are not conformant"), ce_string ("multiply.c"), 89, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_dim1_C))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (not (l_return) and then (l_j < l_dim2_C))
							loop
								l_a1___4 := l_i
								l_a2___3 := (0).to_natural_32
								l_b1___0 := (0).to_natural_32
								l_b2 := l_j
								if ((a_modA).bit_and ({NATURAL_32} 1)).to_boolean then
									from
									until
										(l_break)
									loop
										l_break := False
										l_swap_tmp___1 := l_a1___4
										l_a1___4 := l_a2___3
										l_a2___3 := l_swap_tmp___1
										l_break := True
									end
									l_break := False
								end
								if ((a_modB).bit_and ({NATURAL_32} 1)).to_boolean then
									from
									until
										(l_break)
									loop
										l_break := False
										l_swap_tmp___2 := l_b1___0
										l_b1___0 := l_b2
										l_b2 := l_swap_tmp___2
										l_break := True
									end
									l_break := False
								end
								l_a___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_a1___4, l_a2___3)
								update_class_fields -- Current might have been changed; synchronize
								l_b___0 := a_B.item.gsl_matrix_get (l_b1___0, l_b2)
								update_class_fields -- Current might have been changed; synchronize
								l_temp := l_a___5 * l_b___0
								l_k := (1).to_natural_32
								from
								until
									not (not (l_return) and then (l_k < l_dim2_A))
								loop
									l_a1___4 := l_i
									l_a2___3 := l_k
									l_b1___0 := l_k
									l_b2 := l_j
									if ((a_modA).bit_and ({NATURAL_32} 1)).to_boolean then
										from
										until
											(l_break)
										loop
											l_break := False
											l_swap_tmp___3 := l_a1___4
											l_a1___4 := l_a2___3
											l_a2___3 := l_swap_tmp___3
											l_break := True
										end
										l_break := False
									end
									if ((a_modB).bit_and ({NATURAL_32} 1)).to_boolean then
										from
										until
											(l_break)
										loop
											l_break := False
											l_swap_tmp___4 := l_b1___0
											l_b1___0 := l_b2
											l_b2 := l_swap_tmp___4
											l_break := True
										end
										l_break := False
									end
									l_a___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_a1___4, l_a2___3)
									update_class_fields -- Current might have been changed; synchronize
									l_b___0 := a_B.item.gsl_matrix_get (l_b1___0, l_b2)
									update_class_fields -- Current might have been changed; synchronize
									l_temp := l_temp + (l_a___5 * l_b___0)
									l_k := l_k + 1
								end
								a_C.item.gsl_matrix_set (l_i, l_j, l_temp)
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 13 then -- if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_balance_columns (a_D: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_D.is_void
		local
			l_N: NATURAL_32
			l_j: NATURAL_32
			l_A_j: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_s: REAL_64
			l_tmp___0: REAL_64
			l_f: REAL_64
			l_tmp___1: INTEGER_32
			l_return, l_break, l_continue: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_A_j.make (<<1>>)

			l_N := size2
			if a_D.item.size /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of D must match second dimension of A"), ce_string ("balance.c"), 44, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				a_D.item.gsl_vector_set_all (1.0)
				update_class_fields -- Current might have been changed; synchronize
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_continue := False
					l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_j)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_A_j.item := l_tmp.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := gsl_blas_dasum ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_A_j.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))))
					update_class_fields -- Current might have been changed; synchronize
					l_s := l_tmp___0
					l_f := 1.0
					if l_s = 0.0 then
						a_D.item.gsl_vector_set (l_j, l_f)
						update_class_fields -- Current might have been changed; synchronize
						l_continue := True
					else
						l_tmp___1 := (create {P_INFNAN_STATIC}).gsl_finite (l_s)
						update_class_fields -- Current might have been changed; synchronize
						if not l_tmp___1.to_boolean then
							a_D.item.gsl_vector_set (l_j, l_f)
							update_class_fields -- Current might have been changed; synchronize
							l_continue := True
						end
					end
					if not (l_return or l_break or l_continue) then
						from
						until
							not (l_s > 1.0)
						loop
							l_s := l_s / (2.0)
							l_f := l_f * (2.0)
						end
						from
						until
							not (l_s < 0.5)
						loop
							l_s := l_s * (2.0)
							l_f := l_f / (2.0)
						end
						a_D.item.gsl_vector_set (l_j, l_f)
						update_class_fields -- Current might have been changed; synchronize
						if l_f /= 1.0 then
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_A_j.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (1.0 / l_f)
							update_class_fields -- Current might have been changed; synchronize
						end
					end
					if (not l_return) then
						l_j := l_j + 1
					end
				end
				l_continue := False
				Result := 0
			end
		end

	gsl_linalg_householder_hm1 (a_tau: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_i___0: NATURAL_32
			l_j___0: NATURAL_32
			l_wj: REAL_64
			l_vi: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_vi___0: REAL_64
			l_tmp___1: REAL_64
			l_Aij: REAL_64
			l_tmp___2: REAL_64
			l_vi___1: REAL_64
			l_tmp___3: REAL_64
			l_return: BOOLEAN
		do
			if a_tau = (0).to_double then
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, 1.0)
				update_class_fields -- Current might have been changed; synchronize
				l_j := (1).to_natural_32
				from
				until
					not (l_j < size2)
				loop
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((0).to_natural_32, l_j, 0.0)
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_i := (1).to_natural_32
				from
				until
					not (l_i < size1)
				loop
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, (0).to_natural_32, 0.0)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				l_j___0 := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_j___0 < size2))
				loop
					l_wj := 0.0
					l_i___0 := (1).to_natural_32
					from
					until
						not (l_i___0 < size1)
					loop
						l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i___0, (0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_vi := l_tmp
						l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i___0, l_j___0)
						update_class_fields -- Current might have been changed; synchronize
						l_wj := l_wj + (l_tmp___0 * l_vi)
						l_i___0 := l_i___0 + 1
					end
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((0).to_natural_32, l_j___0, -a_tau * l_wj)
					update_class_fields -- Current might have been changed; synchronize
					l_i___0 := (1).to_natural_32
					from
					until
						not (l_i___0 < size1)
					loop
						l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i___0, (0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_vi___0 := l_tmp___1
						l_tmp___2 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i___0, l_j___0)
						update_class_fields -- Current might have been changed; synchronize
						l_Aij := l_tmp___2;
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i___0, l_j___0, l_Aij - (a_tau * l_vi___0) * l_wj)
						update_class_fields -- Current might have been changed; synchronize
						l_i___0 := l_i___0 + 1
					end
					l_j___0 := l_j___0 + 1
				end
				l_i___0 := (1).to_natural_32
				from
				until
					not (l_i___0 < size1)
				loop
					l_tmp___3 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i___0, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_vi___1 := l_tmp___3;
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i___0, (0).to_natural_32, -a_tau * l_vi___1)
					update_class_fields -- Current might have been changed; synchronize
					l_i___0 := l_i___0 + 1
				end
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, 1.0 - a_tau)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_linalg_LQ_decomp (a_tau: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_tau.is_void
		local
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_i: NATURAL_32
			l_c_full: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_tau_i: REAL_64
			l_tmp___1: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___2: PD_GSL_MATRIX_VIEW
			l_tmp___3: NATURAL_32
			l_tmp___4: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c_full.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			l_N := size1
			l_M := size2
			if l_M < l_N then
				l_tmp___4 := l_M
			else
				l_tmp___4 := l_N
			end
			if a_tau.item.size /= l_tmp___4 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("lq.c"), 90, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := (0).to_natural_32
				from
				until
					(l_break)
				loop
					l_break := False
					if l_M < l_N then
						l_tmp___3 := l_M
					else
						l_tmp___3 := l_N
					end
					if not (l_i < l_tmp___3) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c_full.item := l_tmp.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c_full.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c___1.item := l_tmp___0.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
						update_class_fields -- Current might have been changed; synchronize
						l_tau_i := l_tmp___1
						a_tau.item.gsl_vector_set (l_i, l_tau_i)
						update_class_fields -- Current might have been changed; synchronize
						if l_i + {NATURAL_32} 1 < l_N then
							l_tmp___2 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i + {NATURAL_32} 1, l_i, l_N - (l_i + {NATURAL_32} 1), l_M - l_i)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_m___4.item := l_tmp___2.twin
							update_class_fields -- Current might have been changed; synchronize
							gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						l_i := l_i + 1
					end
				end
				l_break := False
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LQ_solve_T (a_tau, a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LQ matrix must be square"), ce_string ("lq.c"), 135, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("lq.c"), 139, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("lq.c"), 143, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_LQ_svx_T (a_tau, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LQ_svx_T (a_tau, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LQ matrix must be square"), ce_string ("lq.c"), 172, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match x/rhs size"), ce_string ("lq.c"), 176, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_LQ_vecQT (a_tau, a_x).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LQ_lssolve_T (a_tau, a_b___0, a_x, a_residual: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_L: CE_ARRAY [PD_GSL_MATRIX_CONST_VIEW]
			l_tmp: PD_GSL_MATRIX_CONST_VIEW
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_CONST_VIEW]} l_L.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)

			l_N := size1
			l_M := size2
			if l_M < l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LQ matrix must have M>=N"), ce_string ("lq.c"), 208, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_M /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("lq.c"), 212, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("lq.c"), 216, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_M /= a_residual.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match residual size"), ce_string ("lq.c"), 220, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_submatrix ((0).to_natural_32, (0).to_natural_32, l_N, l_N)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_L.item := l_tmp.twin
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := (a_residual.item.gsl_vector_subvector ((0).to_natural_32, l_N)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_c___1.item := l_tmp___0.twin
				update_class_fields -- Current might have been changed; synchronize
				a_residual.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_LQ_vecQT (a_tau, a_residual).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				a_x.item.gsl_vector_memcpy ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_L.item.c_pointer + (create {PD_GSL_MATRIX_CONST_VIEW}.make).offset_of ("matrix"))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set_zero
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_LQ_vecQ (a_tau, a_residual).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LQ_Lsolve_T (a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LQ matrix must be square"), ce_string ("lq.c"), 255, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("lq.c"), 259, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match x size"), ce_string ("lq.c"), 263, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LQ_Lsvx_T (a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LQ matrix must be square"), ce_string ("lq.c"), 285, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match rhs size"), ce_string ("lq.c"), 289, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_L_solve_T (a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("R matrix must be square"), ce_string ("lq.c"), 306, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("lq.c"), 310, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("lq.c"), 314, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LQ_vecQT (a_tau, a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_tau.is_void
		local
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_w: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: PD_GSL_VECTOR_VIEW
			l_ti: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: NATURAL_32
			l_tmp___4: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_w.make (<<1>>)

			l_N := size1
			l_M := size2
			if l_M < l_N then
				l_tmp___4 := l_M
			else
				l_tmp___4 := l_N
			end
			if a_tau.item.size /= l_tmp___4 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("lq.c"), 341, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_v.item.size /= l_M then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector size must be M"), ce_string ("lq.c"), 345, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := (0).to_natural_32
				from
				until
					(l_break)
				loop
					l_break := False
					if l_M < l_N then
						l_tmp___3 := l_M
					else
						l_tmp___3 := l_N
					end
					if not (l_i < l_tmp___3) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_row (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c___1.item := l_tmp.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_h.item := l_tmp___0.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := (a_v.item.gsl_vector_subvector (l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_w.item := l_tmp___1.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := a_tau.item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_ti := l_tmp___2
						gsl_linalg_householder_hv (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_w.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				end
				l_break := False
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LQ_vecQ (a_tau, a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_tau.is_void
		local
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_w: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: PD_GSL_VECTOR_VIEW
			l_ti: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: NATURAL_32
			l_tmp___4: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_w.make (<<1>>)

			l_N := size1
			l_M := size2
			if l_M < l_N then
				l_tmp___4 := l_M
			else
				l_tmp___4 := l_N
			end
			if a_tau.item.size /= l_tmp___4 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("lq.c"), 374, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_v.item.size /= l_M then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector size must be M"), ce_string ("lq.c"), 378, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				if l_M < l_N then
					l_i := l_M
				else
					l_i := l_N
				end
				from
				until
					(l_break)
				loop
					l_break := False
					l_tmp___3 := l_i
					l_i := l_i - 1
					if not (l_tmp___3 > {NATURAL_32} 0) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_row (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c___1.item := l_tmp.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_h.item := l_tmp___0.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := (a_v.item.gsl_vector_subvector (l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_w.item := l_tmp___1.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := a_tau.item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_ti := l_tmp___2
						gsl_linalg_householder_hv (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_w.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				l_break := False
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_LQ_unpack (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_Q___1, a_L: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_Q___1.is_void
		local
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_l_border: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp: PD_GSL_VECTOR_CONST_VIEW
			l_h: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___1: PD_GSL_MATRIX_VIEW
			l_ti: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: NATURAL_32
			l_tmp___4: REAL_64
			l_tmp___5: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_h.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := size1
					l_M := size2
					if a_Q___1.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L___0
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if a_Q___1.item.size2 = l_M then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Q matrix must be M x M"), ce_string ("lq.c"), 410, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if a_L.item.size1 = l_N then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if a_L.item.size2 = l_M then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("R matrix must be N x M"), ce_string ("lq.c"), 414, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						if l_M < l_N then
							l_tmp___5 := l_M
						else
							l_tmp___5 := l_N
						end
						if a_tau.item.size /= l_tmp___5 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("lq.c"), 418, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							a_Q___1.item.gsl_matrix_set_identity
							update_class_fields -- Current might have been changed; synchronize
							if l_M < l_N then
								l_i := l_M
							else
								l_i := l_N
							end
							from
							until
								(l_break)
							loop
								l_break := False
								l_tmp___3 := l_i
								l_i := l_i - 1
								if not (l_tmp___3 > {NATURAL_32} 0) then
									l_break := True
								end
								if not (l_return or l_break) then
									l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_row (l_i)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_c___1.item := l_tmp.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___0 := (gsl_vector_const_subvector ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), l_i, l_M - l_i)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_h.item := l_tmp___0.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___1 := (a_Q___1.item.gsl_matrix_submatrix (l_i, l_i, l_M - l_i, l_M - l_i)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_m___4.item := l_tmp___1.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___2 := a_tau.item.gsl_vector_get (l_i)
									update_class_fields -- Current might have been changed; synchronize
									l_ti := l_tmp___2
									gsl_linalg_householder_mh (l_ti, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_h.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
									update_class_fields -- Current might have been changed; synchronize
								end
							end
							l_break := False
							l_i := (0).to_natural_32
							from
							until
								not (not (l_return) and then (l_i < l_N))
							loop
								if l_i < l_M - {NATURAL_32} 1 then
									l_l_border := l_i
								else
									l_l_border := l_M - {NATURAL_32} 1
								end
								l_j := (0).to_natural_32
								from
								until
									not (l_j <= l_l_border)
								loop
									l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
									update_class_fields -- Current might have been changed; synchronize
									a_L.item.gsl_matrix_set (l_i, l_j, l_tmp___4)
									update_class_fields -- Current might have been changed; synchronize
									l_j := l_j + 1
								end
								l_j := l_l_border + {NATURAL_32} 1
								from
								until
									not (l_j < l_M)
								loop
									a_L.item.gsl_matrix_set (l_i, l_j, 0.0)
									update_class_fields -- Current might have been changed; synchronize
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
							Result := 0
							l_return := True
						end
				when 13 then -- if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_LQ_update (a_L: CE_POINTER [PD_GSL_MATRIX]; a_v, a_w: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_L.is_void
		local
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_w0: REAL_64
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_wk: REAL_64
			l_tmp: REAL_64
			l_wkm1: REAL_64
			l_tmp___0: REAL_64
			l_lj0: REAL_64
			l_tmp___1: REAL_64
			l_vj: REAL_64
			l_tmp___2: REAL_64
			l_c___2: CE_ARRAY [REAL_64]
			l_s___0: CE_ARRAY [REAL_64]
			l_diag: REAL_64
			l_tmp___3: REAL_64
			l_offdiag: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_c___2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s___0.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := a_L.item.size1
					l_M := a_L.item.size2
					if size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if size2 = l_M then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Q matrix must be N x N if L is M x N"), ce_string ("lq.c"), 476, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_w.item.size /= l_M then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("w must be length N if L is M x N"), ce_string ("lq.c"), 480, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_v.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("v must be length M if L is M x N"), ce_string ("lq.c"), 484, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_k := l_M - {NATURAL_32} 1
							from
							until
								not (l_k > {NATURAL_32} 0)
							loop
								l_tmp := a_w.item.gsl_vector_get (l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_wk := l_tmp
								l_tmp___0 := a_w.item.gsl_vector_get (l_k - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_wkm1 := l_tmp___0;
								(create {P_QRSTEP_STATIC}).create_givens (l_wkm1, l_wk, l_c___1, l_s)
								update_class_fields -- Current might have been changed; synchronize
								apply_givens_vec (a_w, l_k - {NATURAL_32} 1, l_k, l_c___1.item, l_s.item)
								update_class_fields -- Current might have been changed; synchronize
								apply_givens_lq (l_M, l_N, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)), a_L, l_k - {NATURAL_32} 1, l_k, l_c___1.item, l_s.item)
								update_class_fields -- Current might have been changed; synchronize
								l_k := l_k - 1
							end
							l_w0 := a_w.item.gsl_vector_get ((0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								l_tmp___1 := a_L.item.gsl_matrix_get (l_j, (0).to_natural_32)
								update_class_fields -- Current might have been changed; synchronize
								l_lj0 := l_tmp___1
								l_tmp___2 := a_v.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_vj := l_tmp___2
								a_L.item.gsl_matrix_set (l_j, (0).to_natural_32, l_lj0 + l_w0 * l_vj)
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_k := (1).to_natural_32
							from
							until
								(l_break)
							loop
								l_break := False
								if l_M < l_N + {NATURAL_32} 1 then
									l_tmp___5 := l_M
								else
									l_tmp___5 := l_N + {NATURAL_32} 1
								end
								if not (l_k < l_tmp___5) then
									l_break := True
								end
								if not (l_return or l_break) then
									l_tmp___3 := a_L.item.gsl_matrix_get (l_k - {NATURAL_32} 1, l_k - {NATURAL_32} 1)
									update_class_fields -- Current might have been changed; synchronize
									l_diag := l_tmp___3
									l_tmp___4 := a_L.item.gsl_matrix_get (l_k - {NATURAL_32} 1, l_k)
									update_class_fields -- Current might have been changed; synchronize
									l_offdiag := l_tmp___4;
									(create {P_QRSTEP_STATIC}).create_givens (l_diag, l_offdiag, l_c___2, l_s___0)
									update_class_fields -- Current might have been changed; synchronize
									apply_givens_lq (l_M, l_N, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)), a_L, l_k - {NATURAL_32} 1, l_k, l_c___2.item, l_s___0.item)
									update_class_fields -- Current might have been changed; synchronize
									a_L.item.gsl_matrix_set (l_k - {NATURAL_32} 1, l_k, 0.0)
									update_class_fields -- Current might have been changed; synchronize
									l_k := l_k + 1
								end
							end
							l_break := False
							Result := 0
							l_return := True
						end
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_HH_solve (a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if size1 > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("System is underdetermined"), ce_string ("hh.c"), 41, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix and vector sizes must be equal"), ce_string ("hh.c"), 45, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_status := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_HH_svx (a_x)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_status
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_HH_svx (a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_d: CE_POINTER [REAL_64]
			l_tmp: CE_POINTER [ANY]
			l_aii: REAL_64
			l_tmp___0: REAL_64
			l_alpha: REAL_64
			l_f: REAL_64
			l_ak: REAL_64
			l_max_norm: REAL_64
			l_r___3: REAL_64
			l_aki: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: INTEGER_32
			l_norm___0: REAL_64
			l_ajk: REAL_64
			l_tmp___4: REAL_64
			l_aji: REAL_64
			l_tmp___5: REAL_64
			l_ajk___0: REAL_64
			l_tmp___6: REAL_64
			l_aji___0: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_xj: REAL_64
			l_tmp___12: REAL_64
			l_aji___1: REAL_64
			l_tmp___13: REAL_64
			l_xi: REAL_64
			l_tmp___14: REAL_64
			l_sum: REAL_64
			l_tmp___15: REAL_64
			l_tmp___16: REAL_64
			l_tmp___17: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			if size1 > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("System is underdetermined"), ce_string ("hh.c"), 66, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix and vector sizes must be equal"), ce_string ("hh.c"), 70, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_N := size1
				l_M := size2
				l_tmp := malloc (l_N * {PLATFORM}.Real_64_bytes.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_d := create {CE_REAL_64_POINTER}.make_cast (l_tmp)
				if l_d.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("could not allocate memory for workspace"), ce_string ("hh.c"), 81, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := 8
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					l_i := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_i < l_N))
					loop
						l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_aii := l_tmp___0
						l_max_norm := 0.0
						l_r___3 := 0.0
						l_k := l_i
						from
						until
							not (l_k < l_M)
						loop
							l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_k, l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_aki := l_tmp___1
							l_r___3 := l_r___3 + (l_aki * l_aki)
							l_k := l_k + 1
						end
						if l_r___3 = 0.0 then
							free (create {CE_VOID_POINTER}.make_cast (l_d))
							update_class_fields -- Current might have been changed; synchronize
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix is rank deficient"), ce_string ("hh.c"), 105, 21)
								update_class_fields -- Current might have been changed; synchronize
								Result := 21
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
						if not l_return then
							l_tmp___2 := sqrt (l_r___3)
							update_class_fields -- Current might have been changed; synchronize
							if l_aii >= 0.0 then
								l_tmp___3 := 1
							else
								l_tmp___3 := -1
							end
							l_alpha := l_tmp___2 * l_tmp___3.to_double
							l_ak := 1.0 / (l_r___3 + l_alpha * l_aii);
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_i, l_aii + l_alpha)
							update_class_fields -- Current might have been changed; synchronize
							l_d[l_i.to_integer_32] := -l_alpha
							update_class_fields -- Current might have been changed; synchronize
							l_k := l_i + {NATURAL_32} 1
							from
							until
								not (not (l_return) and then (l_k < l_N))
							loop
								l_norm___0 := 0.0
								l_f := 0.0
								l_j := l_i
								from
								until
									not (l_j < l_M)
								loop
									l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_k)
									update_class_fields -- Current might have been changed; synchronize
									l_ajk := l_tmp___4
									l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_i)
									update_class_fields -- Current might have been changed; synchronize
									l_aji := l_tmp___5
									l_norm___0 := l_norm___0 + (l_ajk * l_ajk)
									l_f := l_f + (l_ajk * l_aji)
									l_j := l_j + 1
								end
								if l_max_norm > l_norm___0 then
									l_max_norm := l_max_norm
								else
									l_max_norm := l_norm___0
								end
								l_f := l_f * l_ak
								l_j := l_i
								from
								until
									not (l_j < l_M)
								loop
									l_tmp___6 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_k)
									update_class_fields -- Current might have been changed; synchronize
									l_ajk___0 := l_tmp___6
									l_tmp___7 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_i)
									update_class_fields -- Current might have been changed; synchronize
									l_aji___0 := l_tmp___7;
									((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j, l_k, l_ajk___0 - l_f * l_aji___0)
									update_class_fields -- Current might have been changed; synchronize
									l_j := l_j + 1
								end
								l_k := l_k + 1
							end
							l_tmp___8 := fabs (l_alpha)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___9 := sqrt (l_max_norm)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___8 < (2.0 * 2.2204460492503131e-16) * l_tmp___9 then
								free (create {CE_VOID_POINTER}.make_cast (l_d))
								update_class_fields -- Current might have been changed; synchronize
								from
								until
									(l_return or l_break)
								loop
									l_break := False;
									(create {P_ERROR_STATIC}).gsl_error (ce_string ("apparent singularity detected"), ce_string ("hh.c"), 142, 21)
									update_class_fields -- Current might have been changed; synchronize
									Result := 21
									l_return := True
									if not l_return then
										l_break := True
									end
								end
								if not l_return then
									l_break := False
								end
							end
						end
						if not l_return then
							l_f := 0.0
							l_j := l_i
							from
							until
								not (l_j < l_M)
							loop
								l_tmp___10 := a_x.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___11 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_f := l_f + (l_tmp___10 * l_tmp___11)
								l_j := l_j + 1
							end
							l_f := l_f * l_ak
							l_j := l_i
							from
							until
								not (l_j < l_M)
							loop
								l_tmp___12 := a_x.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_xj := l_tmp___12
								l_tmp___13 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_aji___1 := l_tmp___13
								a_x.item.gsl_vector_set (l_j, l_xj - l_f * l_aji___1)
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
					end
				end
				if not l_return then
					l_i := l_N
					from
					until
						(l_return or l_break)
					loop
						l_break := False
						l_tmp___17 := l_i
						l_i := l_i - 1
						if not (l_tmp___17 > {NATURAL_32} 0) then
							l_break := True
						end
						if not (l_return or l_break) then
							l_tmp___14 := a_x.item.gsl_vector_get (l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_xi := l_tmp___14
							l_sum := 0.0
							l_k := l_i + {NATURAL_32} 1
							from
							until
								not (l_k < l_N)
							loop
								l_tmp___15 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___16 := a_x.item.gsl_vector_get (l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_sum := l_sum + (l_tmp___15 * l_tmp___16)
								l_k := l_k + 1
							end
							a_x.item.gsl_vector_set (l_i, (l_xi - l_sum) / l_d[l_i.to_integer_32])
							update_class_fields -- Current might have been changed; synchronize
						end
					end
					l_break := False
					free (create {CE_VOID_POINTER}.make_cast (l_d))
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QRPT_decomp (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_signum: CE_POINTER [INTEGER_32]; a_norm___0: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_tau.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_x: REAL_64
			l_tmp___0: REAL_64
			l_max_norm: REAL_64
			l_tmp___1: REAL_64
			l_j: NATURAL_32
			l_kmax___0: NATURAL_32
			l_x___0: REAL_64
			l_tmp___2: REAL_64
			l_c_full: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___3: PD_GSL_VECTOR_VIEW
			l_c___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___4: PD_GSL_VECTOR_VIEW
			l_tau_i: REAL_64
			l_tmp___5: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___6: PD_GSL_MATRIX_VIEW
			l_x___1: REAL_64
			l_tmp___7: REAL_64
			l_y: REAL_64
			l_temp: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_c_full___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___11: PD_GSL_VECTOR_VIEW
			l_c___3: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___12: PD_GSL_VECTOR_VIEW
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_tmp___15: NATURAL_32
			l_tmp___16: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c_full.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c_full___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___3.make (<<1>>)

			l_M := size1
			l_N := size2
			if l_M < l_N then
				l_tmp___16 := l_M
			else
				l_tmp___16 := l_N
			end
			if a_tau.item.size /= l_tmp___16 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("qrpt.c"), 78, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_p.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation size must be N"), ce_string ("qrpt.c"), 82, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_norm___0.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("norm size must be N"), ce_string ("qrpt.c"), 86, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_signum.item := 1
				update_class_fields -- Current might have been changed; synchronize
				a_p.item.gsl_permutation_init
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_c___1.item := l_tmp.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
					update_class_fields -- Current might have been changed; synchronize
					l_x := l_tmp___0
					a_norm___0.item.gsl_vector_set (l_i, l_x)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					(l_return or l_break)
				loop
					l_break := False
					if l_M < l_N then
						l_tmp___15 := l_M
					else
						l_tmp___15 := l_N
					end
					if not (l_i < l_tmp___15) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp___1 := a_norm___0.item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_max_norm := l_tmp___1
						l_kmax___0 := l_i
						l_j := l_i + {NATURAL_32} 1
						from
						until
							not (l_j < l_N)
						loop
							l_tmp___2 := a_norm___0.item.gsl_vector_get (l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_x___0 := l_tmp___2
							if l_x___0 > l_max_norm then
								l_max_norm := l_x___0
								l_kmax___0 := l_j
							end
							l_j := l_j + 1
						end
						if l_kmax___0 /= l_i then
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_swap_columns (l_i, l_kmax___0).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_p.item.gsl_permutation_swap (l_i, l_kmax___0).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_norm___0.item.gsl_vector_swap_elements (l_i, l_kmax___0).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_signum.item := -a_signum.item
							update_class_fields -- Current might have been changed; synchronize
						end
						l_tmp___3 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c_full.item := l_tmp___3.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c_full.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c___2.item := l_tmp___4.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___5 := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
						update_class_fields -- Current might have been changed; synchronize
						l_tau_i := l_tmp___5
						a_tau.item.gsl_vector_set (l_i, l_tau_i)
						update_class_fields -- Current might have been changed; synchronize
						if l_i + {NATURAL_32} 1 < l_N then
							l_tmp___6 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i, l_i + {NATURAL_32} 1, l_M - l_i, l_N - (l_i + {NATURAL_32} 1))).twin
							update_class_fields -- Current might have been changed; synchronize
							l_m___4.item := l_tmp___6.twin
							update_class_fields -- Current might have been changed; synchronize
							gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						if l_i + {NATURAL_32} 1 < l_M then
							l_j := l_i + {NATURAL_32} 1
							from
							until
								not (l_j < l_N)
							loop
								l_tmp___7 := a_norm___0.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_x___1 := l_tmp___7
								if l_x___1 > 0.0 then
									l_y := (0).to_double
									l_tmp___8 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
									update_class_fields -- Current might have been changed; synchronize
									l_temp := l_tmp___8 / l_x___1
									l_tmp___10 := fabs (l_temp)
									update_class_fields -- Current might have been changed; synchronize
									if l_tmp___10 >= (1).to_double then
										l_y := 0.0
									else
										l_tmp___9 := sqrt ((1).to_double - l_temp * l_temp)
										update_class_fields -- Current might have been changed; synchronize
										l_y := l_x___1 * l_tmp___9
									end
									l_tmp___13 := fabs (l_y / l_x___1)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___14 := sqrt (20.0)
									update_class_fields -- Current might have been changed; synchronize
									if l_tmp___13 < l_tmp___14 * 1.4901161193847656e-08 then
										l_tmp___11 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_j)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_c_full___0.item := l_tmp___11.twin
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___12 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c_full___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i + {NATURAL_32} 1, l_M - (l_i + {NATURAL_32} 1))).twin
										update_class_fields -- Current might have been changed; synchronize
										l_c___3.item := l_tmp___12.twin
										update_class_fields -- Current might have been changed; synchronize
										l_y := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
										update_class_fields -- Current might have been changed; synchronize
									end
									a_norm___0.item.gsl_vector_set (l_j, l_y)
									update_class_fields -- Current might have been changed; synchronize
								end
								l_j := l_j + 1
							end
						end
						l_i := l_i + 1
					end
				end
				l_break := False
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QRPT_decomp2 (a_q___3, a_r___3: CE_POINTER [PD_GSL_MATRIX]; a_tau: CE_POINTER [PD_GSL_VECTOR]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_signum: CE_POINTER [INTEGER_32]; a_norm___0: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_q___3.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_q___3.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L___0
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if a_q___3.item.size2 = l_M then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("q must be M x M"), ce_string ("qrpt.c"), 200, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if a_r___3.item.size1 = l_M then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if a_r___3.item.size2 = l_N then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("r must be M x N"), ce_string ("qrpt.c"), 204, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						if l_M < l_N then
							l_tmp := l_M
						else
							l_tmp := l_N
						end
						if a_tau.item.size /= l_tmp then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("qrpt.c"), 208, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_p.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation size must be N"), ce_string ("qrpt.c"), 212, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_norm___0.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("norm size must be N"), ce_string ("qrpt.c"), 216, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
				when 13 then -- if done | if done | if done
					if not l_return then
						a_r___3.item.gsl_matrix_memcpy ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						a_r___3.item.gsl_linalg_QRPT_decomp (a_tau, a_p, a_signum, a_norm___0).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						a_r___3.item.gsl_linalg_QR_unpack (a_tau, a_q___3, a_r___3).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_QRPT_solve (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must be square"), ce_string ("qrpt.c"), 248, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_p.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match permutation size"), ce_string ("qrpt.c"), 252, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("qrpt.c"), 256, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("qrpt.c"), 260, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_QRPT_svx (a_tau, a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QRPT_svx (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must be square"), ce_string ("qrpt.c"), 280, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_p.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match permutation size"), ce_string ("qrpt.c"), 284, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("qrpt.c"), 288, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_QR_QTvec (a_tau, a_x).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				gsl_permute_vector_inverse (a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QRPT_QRsolve (a_R: CE_POINTER [PD_GSL_MATRIX]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		do
			if size1 /= size2 then
				Result := 20
			elseif a_R.item.size1 /= a_R.item.size2 then
				Result := 20
			elseif size1 /= a_p.item.size then
				Result := 19
			elseif size1 /= a_R.item.size1 then
				Result := 19
			elseif size1 /= a_b___0.item.size then
				Result := 19
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasTrans, 1.0, a_b___0, 0.0, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				a_R.item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				gsl_permute_vector_inverse (a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_linalg_QRPT_Rsolve (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must be square"), ce_string ("qrpt.c"), 348, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("qrpt.c"), 352, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match x size"), ce_string ("qrpt.c"), 356, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_p.item.size /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation size must match x size"), ce_string ("qrpt.c"), 360, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				gsl_permute_vector_inverse (a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QRPT_Rsvx (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must be square"), ce_string ("qrpt.c"), 386, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match x size"), ce_string ("qrpt.c"), 390, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_p.item.size /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation size must match x size"), ce_string ("qrpt.c"), 394, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				gsl_permute_vector_inverse (a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_QRPT_update (a_R: CE_POINTER [PD_GSL_MATRIX]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_w, a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_R.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_w0: REAL_64
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_wk: REAL_64
			l_tmp: REAL_64
			l_wkm1: REAL_64
			l_tmp___0: REAL_64
			l_r0j: REAL_64
			l_tmp___1: REAL_64
			l_p_j: NATURAL_32
			l_tmp___2: NATURAL_32
			l_vj: REAL_64
			l_tmp___3: REAL_64
			l_c___2: CE_ARRAY [REAL_64]
			l_s___0: CE_ARRAY [REAL_64]
			l_diag: REAL_64
			l_tmp___4: REAL_64
			l_offdiag: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_c___2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s___0.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_R.item.size1
					l_N := a_R.item.size2
					if size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if size2 = l_M then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Q matrix must be M x M if R is M x N"), ce_string ("qrpt.c"), 431, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_w.item.size /= l_M then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("w must be length M if R is M x N"), ce_string ("qrpt.c"), 435, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_v.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("v must be length N if R is M x N"), ce_string ("qrpt.c"), 439, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_k := l_M - {NATURAL_32} 1
							from
							until
								not (l_k > {NATURAL_32} 0)
							loop
								l_tmp := a_w.item.gsl_vector_get (l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_wk := l_tmp
								l_tmp___0 := a_w.item.gsl_vector_get (l_k - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								l_wkm1 := l_tmp___0;
								(create {P_QRSTEP_STATIC}).create_givens (l_wkm1, l_wk, l_c___1, l_s)
								update_class_fields -- Current might have been changed; synchronize
								apply_givens_vec (a_w, l_k - {NATURAL_32} 1, l_k, l_c___1.item, l_s.item)
								update_class_fields -- Current might have been changed; synchronize
								apply_givens_qr (l_M, l_N, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)), a_R, l_k - {NATURAL_32} 1, l_k, l_c___1.item, l_s.item)
								update_class_fields -- Current might have been changed; synchronize
								l_k := l_k - 1
							end
							l_w0 := a_w.item.gsl_vector_get ((0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								l_tmp___1 := a_R.item.gsl_matrix_get ((0).to_natural_32, l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_r0j := l_tmp___1
								l_tmp___2 := a_p.item.gsl_permutation_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_p_j := l_tmp___2
								l_tmp___3 := a_v.item.gsl_vector_get (l_p_j)
								update_class_fields -- Current might have been changed; synchronize
								l_vj := l_tmp___3
								a_R.item.gsl_matrix_set ((0).to_natural_32, l_j, l_r0j + l_w0 * l_vj)
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_k := (1).to_natural_32
							from
							until
								(l_break)
							loop
								l_break := False
								if l_M < l_N + {NATURAL_32} 1 then
									l_tmp___6 := l_M
								else
									l_tmp___6 := l_N + {NATURAL_32} 1
								end
								if not (l_k < l_tmp___6) then
									l_break := True
								end
								if not (l_return or l_break) then
									l_tmp___4 := a_R.item.gsl_matrix_get (l_k - {NATURAL_32} 1, l_k - {NATURAL_32} 1)
									update_class_fields -- Current might have been changed; synchronize
									l_diag := l_tmp___4
									l_tmp___5 := a_R.item.gsl_matrix_get (l_k, l_k - {NATURAL_32} 1)
									update_class_fields -- Current might have been changed; synchronize
									l_offdiag := l_tmp___5;
									(create {P_QRSTEP_STATIC}).create_givens (l_diag, l_offdiag, l_c___2, l_s___0)
									update_class_fields -- Current might have been changed; synchronize
									apply_givens_qr (l_M, l_N, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)), a_R, l_k - {NATURAL_32} 1, l_k, l_c___2.item, l_s___0.item)
									update_class_fields -- Current might have been changed; synchronize
									a_R.item.gsl_matrix_set (l_k, l_k - {NATURAL_32} 1, 0.0)
									update_class_fields -- Current might have been changed; synchronize
									l_k := l_k + 1
								end
							end
							l_break := False
							Result := 0
							l_return := True
						end
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_PTLQ_decomp (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_signum: CE_POINTER [INTEGER_32]; a_norm___0: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_tau.is_void
		local
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_i: NATURAL_32
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_x: REAL_64
			l_tmp___0: REAL_64
			l_max_norm: REAL_64
			l_tmp___1: REAL_64
			l_j: NATURAL_32
			l_kmax___0: NATURAL_32
			l_x___0: REAL_64
			l_tmp___2: REAL_64
			l_c_full: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___3: PD_GSL_VECTOR_VIEW
			l_c___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___4: PD_GSL_VECTOR_VIEW
			l_tau_i: REAL_64
			l_tmp___5: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___6: PD_GSL_MATRIX_VIEW
			l_x___1: REAL_64
			l_tmp___7: REAL_64
			l_y: REAL_64
			l_temp: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_c_full___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___11: PD_GSL_VECTOR_VIEW
			l_c___3: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___12: PD_GSL_VECTOR_VIEW
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_tmp___15: NATURAL_32
			l_tmp___16: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c_full.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c_full___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___3.make (<<1>>)

			l_N := size1
			l_M := size2
			if l_M < l_N then
				l_tmp___16 := l_M
			else
				l_tmp___16 := l_N
			end
			if a_tau.item.size /= l_tmp___16 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("ptlq.c"), 83, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_p.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation size must be N"), ce_string ("ptlq.c"), 87, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_norm___0.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("norm size must be N"), ce_string ("ptlq.c"), 91, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_signum.item := 1
				update_class_fields -- Current might have been changed; synchronize
				a_p.item.gsl_permutation_init
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_c___1.item := l_tmp.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
					update_class_fields -- Current might have been changed; synchronize
					l_x := l_tmp___0
					a_norm___0.item.gsl_vector_set (l_i, l_x)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					(l_return or l_break)
				loop
					l_break := False
					if l_M < l_N then
						l_tmp___15 := l_M
					else
						l_tmp___15 := l_N
					end
					if not (l_i < l_tmp___15) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp___1 := a_norm___0.item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_max_norm := l_tmp___1
						l_kmax___0 := l_i
						l_j := l_i + {NATURAL_32} 1
						from
						until
							not (l_j < l_N)
						loop
							l_tmp___2 := a_norm___0.item.gsl_vector_get (l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_x___0 := l_tmp___2
							if l_x___0 > l_max_norm then
								l_max_norm := l_x___0
								l_kmax___0 := l_j
							end
							l_j := l_j + 1
						end
						if l_kmax___0 /= l_i then
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_swap_rows (l_i, l_kmax___0).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_p.item.gsl_permutation_swap (l_i, l_kmax___0).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_norm___0.item.gsl_vector_swap_elements (l_i, l_kmax___0).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_signum.item := -a_signum.item
							update_class_fields -- Current might have been changed; synchronize
						end
						l_tmp___3 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c_full.item := l_tmp___3.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c_full.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i, l_M - l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_c___2.item := l_tmp___4.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___5 := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
						update_class_fields -- Current might have been changed; synchronize
						l_tau_i := l_tmp___5
						a_tau.item.gsl_vector_set (l_i, l_tau_i)
						update_class_fields -- Current might have been changed; synchronize
						if l_i + {NATURAL_32} 1 < l_N then
							l_tmp___6 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i + {NATURAL_32} 1, l_i, l_N - (l_i + {NATURAL_32} 1), l_M - l_i)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_m___4.item := l_tmp___6.twin
							update_class_fields -- Current might have been changed; synchronize
							gsl_linalg_householder_mh (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						if l_i + {NATURAL_32} 1 < l_M then
							l_j := l_i + {NATURAL_32} 1
							from
							until
								not (l_j < l_N)
							loop
								l_tmp___7 := a_norm___0.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_x___1 := l_tmp___7
								if l_x___1 > 0.0 then
									l_y := (0).to_double
									l_tmp___8 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_i)
									update_class_fields -- Current might have been changed; synchronize
									l_temp := l_tmp___8 / l_x___1
									l_tmp___10 := fabs (l_temp)
									update_class_fields -- Current might have been changed; synchronize
									if l_tmp___10 >= (1).to_double then
										l_y := 0.0
									else
										l_tmp___9 := sqrt ((1).to_double - l_temp * l_temp)
										update_class_fields -- Current might have been changed; synchronize
										l_y := l_x___1 * l_tmp___9
									end
									l_tmp___13 := fabs (l_y / l_x___1)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___14 := sqrt (20.0)
									update_class_fields -- Current might have been changed; synchronize
									if l_tmp___13 < l_tmp___14 * 1.4901161193847656e-08 then
										l_tmp___11 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_j)).twin
										update_class_fields -- Current might have been changed; synchronize
										l_c_full___0.item := l_tmp___11.twin
										update_class_fields -- Current might have been changed; synchronize
										l_tmp___12 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c_full___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i + {NATURAL_32} 1, l_M - (l_i + {NATURAL_32} 1))).twin
										update_class_fields -- Current might have been changed; synchronize
										l_c___3.item := l_tmp___12.twin
										update_class_fields -- Current might have been changed; synchronize
										l_y := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
										update_class_fields -- Current might have been changed; synchronize
									end
									a_norm___0.item.gsl_vector_set (l_j, l_y)
									update_class_fields -- Current might have been changed; synchronize
								end
								l_j := l_j + 1
							end
						end
						l_i := l_i + 1
					end
				end
				l_break := False
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_PTLQ_decomp2 (a_q___3, a_r___3: CE_POINTER [PD_GSL_MATRIX]; a_tau: CE_POINTER [PD_GSL_VECTOR]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_signum: CE_POINTER [INTEGER_32]; a_norm___0: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_q___3.is_void
		local
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_tmp: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := size1
					l_M := size2
					if a_q___3.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L___0
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if a_q___3.item.size2 = l_M then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("q must be M x M"), ce_string ("ptlq.c"), 205, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if a_r___3.item.size1 = l_N then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if a_r___3.item.size2 = l_M then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("r must be N x M"), ce_string ("ptlq.c"), 209, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						if l_M < l_N then
							l_tmp := l_M
						else
							l_tmp := l_N
						end
						if a_tau.item.size /= l_tmp then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of tau must be MIN(M,N)"), ce_string ("ptlq.c"), 213, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_p.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation size must be N"), ce_string ("ptlq.c"), 217, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						elseif a_norm___0.item.size /= l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("norm size must be N"), ce_string ("ptlq.c"), 221, 19)
								update_class_fields -- Current might have been changed; synchronize
								Result := 19
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
				when 13 then -- if done | if done | if done
					if not l_return then
						a_r___3.item.gsl_matrix_memcpy ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						a_r___3.item.gsl_linalg_PTLQ_decomp (a_tau, a_p, a_signum, a_norm___0).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						a_r___3.item.gsl_linalg_LQ_unpack (a_tau, a_q___3, a_r___3).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_linalg_PTLQ_solve_T (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("QR matrix must be square"), ce_string ("ptlq.c"), 253, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_p.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match permutation size"), ce_string ("ptlq.c"), 257, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("ptlq.c"), 261, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("ptlq.c"), 265, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_PTLQ_svx_T (a_tau, a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_PTLQ_svx_T (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LQ matrix must be square"), ce_string ("ptlq.c"), 285, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_p.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match permutation size"), ce_string ("ptlq.c"), 289, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("ptlq.c"), 293, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_LQ_vecQT (a_tau, a_x).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				gsl_permute_vector_inverse (a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_PTLQ_LQsolve_T (a_L: CE_POINTER [PD_GSL_MATRIX]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		do
			if size1 /= size2 then
				Result := 20
			elseif a_L.item.size1 /= a_L.item.size2 then
				Result := 20
			elseif size1 /= a_p.item.size then
				Result := 19
			elseif size1 /= a_L.item.size1 then
				Result := 19
			elseif size1 /= a_b___0.item.size then
				Result := 19
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, a_b___0, 0.0, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				a_L.item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				gsl_permute_vector_inverse (a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_linalg_PTLQ_Lsolve_T (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LQ matrix must be square"), ce_string ("ptlq.c"), 353, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("ptlq.c"), 357, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match x size"), ce_string ("ptlq.c"), 361, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_p.item.size /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation size must match x size"), ce_string ("ptlq.c"), 365, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				gsl_permute_vector_inverse (a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_PTLQ_Lsvx_T (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("LQ matrix must be square"), ce_string ("ptlq.c"), 391, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match x size"), ce_string ("ptlq.c"), 395, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_p.item.size /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("permutation size must match x size"), ce_string ("ptlq.c"), 399, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				gsl_permute_vector_inverse (a_p, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_PTLQ_update (a_L: CE_POINTER [PD_GSL_MATRIX]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v, a_w: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_N: NATURAL_32
			l_M: NATURAL_32
			l_w0: REAL_64
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_wk: REAL_64
			l_tmp: REAL_64
			l_wkm1: REAL_64
			l_tmp___0: REAL_64
			l_lj0: REAL_64
			l_tmp___1: REAL_64
			l_p_j: NATURAL_32
			l_tmp___2: NATURAL_32
			l_vj: REAL_64
			l_tmp___3: REAL_64
			l_c___2: CE_ARRAY [REAL_64]
			l_s___0: CE_ARRAY [REAL_64]
			l_diag: REAL_64
			l_tmp___4: REAL_64
			l_offdiag: REAL_64
			l_tmp___5: REAL_64
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_c___2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s___0.make (<<1>>)

			if size1 /= size2 then
				Result := 20
			elseif a_L.item.size1 /= a_L.item.size2 then
				Result := 20
			elseif a_L.item.size1 /= size2 then
				Result := 19
			elseif a_v.item.size /= size2 then
				Result := 19
			elseif a_w.item.size /= size2 then
				Result := 19
			else
				l_N := size1
				l_M := size2
				l_k := l_M - {NATURAL_32} 1
				from
				until
					not (l_k > {NATURAL_32} 0)
				loop
					l_tmp := a_w.item.gsl_vector_get (l_k)
					update_class_fields -- Current might have been changed; synchronize
					l_wk := l_tmp
					l_tmp___0 := a_w.item.gsl_vector_get (l_k - {NATURAL_32} 1)
					update_class_fields -- Current might have been changed; synchronize
					l_wkm1 := l_tmp___0;
					(create {P_QRSTEP_STATIC}).create_givens (l_wkm1, l_wk, l_c___1, l_s)
					update_class_fields -- Current might have been changed; synchronize
					apply_givens_vec (a_w, l_k - {NATURAL_32} 1, l_k, l_c___1.item, l_s.item)
					update_class_fields -- Current might have been changed; synchronize
					apply_givens_lq (l_M, l_N, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)), a_L, l_k - {NATURAL_32} 1, l_k, l_c___1.item, l_s.item)
					update_class_fields -- Current might have been changed; synchronize
					l_k := l_k - 1
				end
				l_w0 := a_w.item.gsl_vector_get ((0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N)
				loop
					l_tmp___1 := a_L.item.gsl_matrix_get (l_j, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_lj0 := l_tmp___1
					l_tmp___2 := a_p.item.gsl_permutation_get (l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_p_j := l_tmp___2
					l_tmp___3 := a_v.item.gsl_vector_get (l_p_j)
					update_class_fields -- Current might have been changed; synchronize
					l_vj := l_tmp___3
					a_L.item.gsl_matrix_set (l_j, (0).to_natural_32, l_lj0 + l_w0 * l_vj)
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_k := (1).to_natural_32
				from
				until
					not (l_k < l_N)
				loop
					l_tmp___4 := a_L.item.gsl_matrix_get (l_k - {NATURAL_32} 1, l_k - {NATURAL_32} 1)
					update_class_fields -- Current might have been changed; synchronize
					l_diag := l_tmp___4
					l_tmp___5 := a_L.item.gsl_matrix_get (l_k - {NATURAL_32} 1, l_k)
					update_class_fields -- Current might have been changed; synchronize
					l_offdiag := l_tmp___5;
					(create {P_QRSTEP_STATIC}).create_givens (l_diag, l_offdiag, l_c___2, l_s___0)
					update_class_fields -- Current might have been changed; synchronize
					apply_givens_lq (l_M, l_N, (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)), a_L, l_k - {NATURAL_32} 1, l_k, l_c___2.item, l_s___0.item)
					update_class_fields -- Current might have been changed; synchronize
					l_k := l_k + 1
				end
				Result := 0
			end
		end

	gsl_linalg_cholesky_decomp: INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_status: INTEGER_32
			l_A_00: REAL_64
			l_tmp: REAL_64
			l_L_00: REAL_64
			l_tmp___0: REAL_64
			l_A_10: REAL_64
			l_tmp___1: REAL_64
			l_A_11: REAL_64
			l_tmp___2: REAL_64
			l_L_10: REAL_64
			l_diag: REAL_64
			l_L_11: REAL_64
			l_tmp___3: REAL_64
			l_A_kk: REAL_64
			l_tmp___4: REAL_64
			l_sum: CE_ARRAY [REAL_64]
			l_A_ki: REAL_64
			l_tmp___5: REAL_64
			l_A_ii: REAL_64
			l_tmp___6: REAL_64
			l_ci: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___7: PD_GSL_VECTOR_VIEW
			l_ck: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___8: PD_GSL_VECTOR_VIEW
			l_di: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___9: PD_GSL_VECTOR_VIEW
			l_dk: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___10: PD_GSL_VECTOR_VIEW
			l_ck___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___11: PD_GSL_VECTOR_VIEW
			l_dk___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___12: PD_GSL_VECTOR_VIEW
			l_sum___0: REAL_64
			l_tmp___13: REAL_64
			l_diag___0: REAL_64
			l_L_kk: REAL_64
			l_tmp___14: REAL_64
			l_A_ij: REAL_64
			l_tmp___15: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_sum.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_ci.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_ck.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_di.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_dk.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_ck___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_dk___0.make (<<1>>)

			l_M := size1
			l_N := size2
			if l_M /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("cholesky decomposition requires square matrix"), ce_string ("cholesky.c"), 59, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_status := 0
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_A_00 := l_tmp
				l_tmp___0 := quiet_sqrt (l_A_00)
				update_class_fields -- Current might have been changed; synchronize
				l_L_00 := l_tmp___0
				if l_A_00 <= (0).to_double then
					l_status := 1
				end
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, l_L_00)
				update_class_fields -- Current might have been changed; synchronize
				if l_M > {NATURAL_32} 1 then
					l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_A_10 := l_tmp___1
					l_tmp___2 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					l_A_11 := l_tmp___2
					l_L_10 := l_A_10 / l_L_00
					l_diag := l_A_11 - l_L_10 * l_L_10
					l_tmp___3 := quiet_sqrt (l_diag)
					update_class_fields -- Current might have been changed; synchronize
					l_L_11 := l_tmp___3
					if l_diag <= (0).to_double then
						l_status := 1
					end
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((1).to_natural_32, (0).to_natural_32, l_L_10)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((1).to_natural_32, (1).to_natural_32, l_L_11)
					update_class_fields -- Current might have been changed; synchronize
				end
				l_k := (2).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < l_M))
				loop
					l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_k, l_k)
					update_class_fields -- Current might have been changed; synchronize
					l_A_kk := l_tmp___4
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_k)
					loop
						l_sum.item := (0).to_double
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_k, l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_A_ki := l_tmp___5
						l_tmp___6 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_A_ii := l_tmp___6
						l_tmp___7 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_i)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_ci.item := l_tmp___7.twin
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___8 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_k)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_ck.item := l_tmp___8.twin
						update_class_fields -- Current might have been changed; synchronize
						if l_i > {NATURAL_32} 0 then
							l_tmp___9 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ci.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector ((0).to_natural_32, l_i)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_di.item := l_tmp___9.twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___10 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ck.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector ((0).to_natural_32, l_i)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_dk.item := l_tmp___10.twin
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_di.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_dk.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_sum).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						l_A_ki := (l_A_ki - l_sum.item) / l_A_ii;
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_k, l_i, l_A_ki)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
					l_tmp___11 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_k)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_ck___0.item := l_tmp___11.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___12 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ck___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector ((0).to_natural_32, l_k)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_dk___0.item := l_tmp___12.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___13 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_dk___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
					update_class_fields -- Current might have been changed; synchronize
					l_sum___0 := l_tmp___13
					l_diag___0 := l_A_kk - l_sum___0 * l_sum___0
					l_tmp___14 := quiet_sqrt (l_diag___0)
					update_class_fields -- Current might have been changed; synchronize
					l_L_kk := l_tmp___14
					if l_diag___0 <= (0).to_double then
						l_status := 1
					end
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_k, l_k, l_L_kk)
					update_class_fields -- Current might have been changed; synchronize
					l_k := l_k + 1
				end
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_M))
				loop
					l_j := (0).to_natural_32
					from
					until
						not (l_j < l_i)
					loop
						l_tmp___15 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_A_ij := l_tmp___15;
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j, l_i, l_A_ij)
						update_class_fields -- Current might have been changed; synchronize
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
				if l_status = 1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be positive definite"), ce_string ("cholesky.c"), 157, 1)
						update_class_fields -- Current might have been changed; synchronize
						Result := 1
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_cholesky_solve (a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("cholesky matrix must be square"), ce_string ("cholesky.c"), 172, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match b size"), ce_string ("cholesky.c"), 176, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("cholesky.c"), 180, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				a_x.item.gsl_vector_memcpy (a_b___0).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_cholesky_svx (a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("cholesky matrix must be square"), ce_string ("cholesky.c"), 207, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size must match solution size"), ce_string ("cholesky.c"), 211, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dtrsv ({PC_CBLAS_UPLO}.CblasUpper, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_cholesky_invert: INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_sum: CE_ARRAY [REAL_64]
			l_v1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_ajj: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_A_ij: REAL_64
			l_tmp___1: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_sum.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)

			if size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("cholesky matrix must be square"), ce_string ("cholesky.c"), 243, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_N := size1
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_j := (l_N - l_i) - {NATURAL_32} 1
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j, l_j, 1.0 / l_tmp)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_ajj := -l_tmp___0
					if l_j < l_N - {NATURAL_32} 1 then
						l_m___4.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_j + {NATURAL_32} 1, l_j + {NATURAL_32} 1, (l_N - l_j) - {NATURAL_32} 1, (l_N - l_j) - {NATURAL_32} 1)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_v1.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_j, l_j + {NATURAL_32} 1, (l_N - l_j) - {NATURAL_32} 1)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_blas_dtrmv ({PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasNoTrans, {PC_CBLAS_DIAG}.CblasNonUnit, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_ajj)
						update_class_fields -- Current might have been changed; synchronize
					end
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_N))
				loop
					l_j := l_i + {NATURAL_32} 1
					from
					until
						not (l_j < l_N)
					loop
						l_v1.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_i, l_j, l_N - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_v2.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_j, l_j, l_N - l_j)).twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_sum).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, l_sum.item)
						update_class_fields -- Current might have been changed; synchronize
						l_j := l_j + 1
					end
					l_v1.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_subcolumn (l_i, l_i, l_N - l_i)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_sum).do_nothing
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_i, l_sum.item)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_j := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_j)
					loop
						l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_A_ij := l_tmp___1;
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j, l_i, l_A_ij)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
					l_j := l_j + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_cholesky_decomp_unit (a_D: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_stat_chol: INTEGER_32
			l_tmp: INTEGER_32
			l_C_ii: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_return: BOOLEAN
		do
			l_N := size1
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_cholesky_decomp
			update_class_fields -- Current might have been changed; synchronize
			l_stat_chol := l_tmp
			if l_stat_chol = 0 then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_C_ii := l_tmp___0
					a_D.item.gsl_vector_set (l_i, l_C_ii * l_C_ii)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_N))
				loop
					l_j := (0).to_natural_32
					from
					until
						not (l_j < l_N)
					loop
						l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := a_D.item.gsl_vector_get (l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := sqrt (l_tmp___2)
						update_class_fields; -- Current might have been changed; synchronize
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, l_tmp___1 / l_tmp___3)
						update_class_fields -- Current might have been changed; synchronize
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_N))
				loop
					l_j := l_i + {NATURAL_32} 1
					from
					until
						not (l_j < l_N)
					loop
						l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_i)
						update_class_fields; -- Current might have been changed; synchronize
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, l_tmp___4)
						update_class_fields -- Current might have been changed; synchronize
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
			end
			Result := l_stat_chol
		end

	gsl_linalg_SV_decomp (a_V: CE_POINTER [PD_GSL_MATRIX]; a_S, a_work: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_a___5: NATURAL_32
			l_b___0: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_iter: NATURAL_32
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_K: NATURAL_32
			l_tmp: NATURAL_32
			l_column: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_norm___0: REAL_64
			l_tmp___1: REAL_64
			l_f: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___2: PD_GSL_VECTOR_VIEW
			l_fbm1: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: INTEGER_32
			l_fam1: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: INTEGER_32
			l_n_block: NATURAL_32
			l_S_block: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___7: PD_GSL_VECTOR_VIEW
			l_f_block: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___8: PD_GSL_VECTOR_VIEW
			l_U_block: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___9: PD_GSL_MATRIX_VIEW
			l_V_block: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___10: PD_GSL_MATRIX_VIEW
			l_rescale: INTEGER_32
			l_scale: REAL_64
			l_norm___1: REAL_64
			l_s_i: REAL_64
			l_tmp___11: REAL_64
			l_a___6: REAL_64
			l_tmp___12: REAL_64
			l_f_i: REAL_64
			l_tmp___13: REAL_64
			l_a___7: REAL_64
			l_tmp___14: REAL_64
			l_Sj: REAL_64
			l_tmp___15: REAL_64
			l_Vij: REAL_64
			l_tmp___16: REAL_64
			l_S_max: REAL_64
			l_tmp___17: REAL_64
			l_i_max: NATURAL_32
			l_Sj___0: REAL_64
			l_tmp___18: REAL_64
			l_return, l_break, l_continue: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_column.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_f.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_S_block.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_f_block.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_U_block.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_V_block.make (<<1>>)

			l_M := size1
			l_N := size2
			if l_M < l_N then
				l_tmp := l_M
			else
				l_tmp := l_N
			end
			l_K := l_tmp
			if l_M < l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("svd of MxN matrix, M<N, is not implemented"), ce_string ("svd.c"), 61, 24)
					update_class_fields -- Current might have been changed; synchronize
					Result := 24
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_V.item.size1 /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("square matrix V must match second dimension of matrix A"), ce_string ("svd.c"), 66, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_V.item.size1 /= a_V.item.size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix V must be square"), ce_string ("svd.c"), 70, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_S.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of vector S must match second dimension of matrix A"), ce_string ("svd.c"), 75, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_work.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of workspace must match second dimension of matrix A"), ce_string ("svd.c"), 80, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (l_N = {NATURAL_32} 1) then
				l_tmp___0 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column ((0).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_column.item := l_tmp___0.twin
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___1 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
				update_class_fields -- Current might have been changed; synchronize
				l_norm___0 := l_tmp___1
				a_S.item.gsl_vector_set ((0).to_natural_32, l_norm___0)
				update_class_fields -- Current might have been changed; synchronize
				a_V.item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, 1.0)
				update_class_fields -- Current might have been changed; synchronize
				if l_norm___0 /= 0.0 then
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (1.0 / l_norm___0)
					update_class_fields -- Current might have been changed; synchronize
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				l_tmp___2 := (a_work.item.gsl_vector_subvector ((0).to_natural_32, l_K - {NATURAL_32} 1)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_f.item := l_tmp___2.twin
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_bidiag_decomp (a_S, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_bidiag_unpack2 (a_S, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), a_V).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				chop_small_elements___3 (a_S, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
				update_class_fields -- Current might have been changed; synchronize
				l_b___0 := l_N - {NATURAL_32} 1
				l_iter := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_b___0 > {NATURAL_32} 0))
				loop
					l_continue := False
					l_tmp___3 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_b___0 - {NATURAL_32} 1)
					update_class_fields -- Current might have been changed; synchronize
					l_fbm1 := l_tmp___3
					if l_fbm1 = 0.0 then
						l_b___0 := l_b___0 - 1
						l_continue := True
					else
						l_tmp___4 := (create {P_INFNAN_STATIC}).gsl_isnan (l_fbm1)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___4.to_boolean then
							l_b___0 := l_b___0 - 1
							l_continue := True
						end
					end
					if not (l_return or l_break or l_continue) then
						l_a___5 := l_b___0 - {NATURAL_32} 1
						from
						until
							not (not (l_break) and then (l_a___5 > {NATURAL_32} 0))
						loop
							l_break := False
							l_tmp___5 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_a___5 - {NATURAL_32} 1)
							update_class_fields -- Current might have been changed; synchronize
							l_fam1 := l_tmp___5
							if l_fam1 = 0.0 then
								l_break := True
							else
								l_tmp___6 := (create {P_INFNAN_STATIC}).gsl_isnan (l_fam1)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___6.to_boolean then
									l_break := True
								end
							end
							if not (l_return or l_break) then
								l_a___5 := l_a___5 - 1
							end
						end
						l_break := False
						l_iter := l_iter + 1
						if l_iter > {NATURAL_32} 100 * l_N then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("SVD decomposition failed to converge"), ce_string ("svd.c"), 149, 11)
								update_class_fields -- Current might have been changed; synchronize
								Result := 11
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
						if not l_return then
							l_n_block := (l_b___0 - l_a___5) + {NATURAL_32} 1
							l_tmp___7 := (a_S.item.gsl_vector_subvector (l_a___5, l_n_block)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_S_block.item := l_tmp___7.twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___8 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_a___5, l_n_block - {NATURAL_32} 1)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_f_block.item := l_tmp___8.twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___9 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix ((0).to_natural_32, l_a___5, size1, l_n_block)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_U_block.item := l_tmp___9.twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___10 := (a_V.item.gsl_matrix_submatrix ((0).to_natural_32, l_a___5, a_V.item.size1, l_n_block)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_V_block.item := l_tmp___10.twin
							update_class_fields -- Current might have been changed; synchronize
							l_rescale := 0
							l_scale := (1).to_double
							l_norm___1 := (0).to_double
							l_i := (0).to_natural_32
							from
							until
								not (l_i < l_n_block)
							loop
								l_tmp___11 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_S_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_s_i := l_tmp___11
								l_tmp___12 := fabs (l_s_i)
								update_class_fields -- Current might have been changed; synchronize
								l_a___6 := l_tmp___12
								if l_a___6 > l_norm___1 then
									l_norm___1 := l_a___6
								end
								l_i := l_i + 1
							end
							l_i := (0).to_natural_32
							from
							until
								not (l_i < l_n_block - {NATURAL_32} 1)
							loop
								l_tmp___13 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_f_i := l_tmp___13
								l_tmp___14 := fabs (l_f_i)
								update_class_fields -- Current might have been changed; synchronize
								l_a___7 := l_tmp___14
								if l_a___7 > l_norm___1 then
									l_norm___1 := l_a___7
								end
								l_i := l_i + 1
							end
							if l_norm___1 > 1.3407807929942596e+154 then
								l_scale := l_norm___1 / 1.3407807929942596e+154
								l_rescale := 1
							elseif l_norm___1 < 1.4916681462400413e-154 then
								if l_norm___1 > (0).to_double then
									l_scale := l_norm___1 / 1.4916681462400413e-154
									l_rescale := 1
								end
							end
							if l_rescale.to_boolean then
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_S_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (1.0 / l_scale)
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (1.0 / l_scale)
								update_class_fields -- Current might have been changed; synchronize
							end
							qrstep___3 (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_S_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_U_block.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_V_block.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")))
							update_class_fields -- Current might have been changed; synchronize
							chop_small_elements___3 (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_S_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
							update_class_fields -- Current might have been changed; synchronize
							if l_rescale.to_boolean then
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_S_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale)
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_f_block.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale)
								update_class_fields -- Current might have been changed; synchronize
							end
						end
					end
				end
			end
			if not l_return then
				l_continue := False
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_K))
				loop
					l_tmp___15 := a_S.item.gsl_vector_get (l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_Sj := l_tmp___15
					if l_Sj < 0.0 then
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_N)
						loop
							l_tmp___16 := a_V.item.gsl_matrix_get (l_i, l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_Vij := l_tmp___16
							a_V.item.gsl_matrix_set (l_i, l_j, -l_Vij)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						a_S.item.gsl_vector_set (l_j, -l_Sj)
						update_class_fields -- Current might have been changed; synchronize
					end
					l_j := l_j + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_K))
				loop
					l_tmp___17 := a_S.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_S_max := l_tmp___17
					l_i_max := l_i
					l_j := l_i + {NATURAL_32} 1
					from
					until
						not (l_j < l_K)
					loop
						l_tmp___18 := a_S.item.gsl_vector_get (l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_Sj___0 := l_tmp___18
						if l_Sj___0 > l_S_max then
							l_S_max := l_Sj___0
							l_i_max := l_j
						end
						l_j := l_j + 1
					end
					if l_i_max /= l_i then
						a_S.item.gsl_vector_swap_elements (l_i, l_i_max).do_nothing
						update_class_fields; -- Current might have been changed; synchronize
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_swap_columns (l_i, l_i_max).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						a_V.item.gsl_matrix_swap_columns (l_i, l_i_max).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_linalg_SV_decomp_mod (a_X, a_V: CE_POINTER [PD_GSL_MATRIX]; a_S, a_work: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_column: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_norm___0: REAL_64
			l_tmp___0: REAL_64
			l_c___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: PD_GSL_VECTOR_VIEW
			l_v: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___2: PD_GSL_VECTOR_VIEW
			l_tau_i: REAL_64
			l_tmp___3: REAL_64
			l_m___4: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___4: PD_GSL_MATRIX_VIEW
			l_Aii: REAL_64
			l_tmp___5: REAL_64
			l_Aij: REAL_64
			l_tmp___6: REAL_64
			l_tj: REAL_64
			l_tmp___7: REAL_64
			l_m___5: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___8: PD_GSL_MATRIX_VIEW
			l_tmp___9: NATURAL_32
			l_sum: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___10: PD_GSL_VECTOR_VIEW
			l_L_i: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___11: PD_GSL_VECTOR_VIEW
			l_Lij: REAL_64
			l_tmp___12: REAL_64
			l_X_j: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___13: PD_GSL_VECTOR_VIEW
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_column.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_c___1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_v.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___4.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_m___5.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_sum.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_L_i.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_X_j.make (<<1>>)

			l_M := size1
			l_N := size2
			if l_M < l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("svd of MxN matrix, M<N, is not implemented"), ce_string ("svd.c"), 286, 24)
					update_class_fields -- Current might have been changed; synchronize
					Result := 24
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_V.item.size1 /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("square matrix V must match second dimension of matrix A"), ce_string ("svd.c"), 291, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_V.item.size1 /= a_V.item.size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix V must be square"), ce_string ("svd.c"), 295, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_X.item.size1 /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("square matrix X must match second dimension of matrix A"), ce_string ("svd.c"), 300, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_X.item.size1 /= a_X.item.size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix X must be square"), ce_string ("svd.c"), 304, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_S.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of vector S must match second dimension of matrix A"), ce_string ("svd.c"), 309, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_work.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of workspace must match second dimension of matrix A"), ce_string ("svd.c"), 314, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (l_N = {NATURAL_32} 1) then
				l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column ((0).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_column.item := l_tmp.twin
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
				update_class_fields -- Current might have been changed; synchronize
				l_norm___0 := l_tmp___0
				a_S.item.gsl_vector_set ((0).to_natural_32, l_norm___0)
				update_class_fields -- Current might have been changed; synchronize
				a_V.item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, 1.0)
				update_class_fields -- Current might have been changed; synchronize
				if l_norm___0 /= 0.0 then
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (1.0 / l_norm___0)
					update_class_fields -- Current might have been changed; synchronize
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp___1 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_c___1.item := l_tmp___1.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___2 := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_subvector (l_i, l_M - l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_v.item := l_tmp___2.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := gsl_linalg_householder_transform (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
					update_class_fields -- Current might have been changed; synchronize
					l_tau_i := l_tmp___3
					if l_i + {NATURAL_32} 1 < l_N then
						l_tmp___4 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_i, l_i + {NATURAL_32} 1, l_M - l_i, l_N - (l_i + {NATURAL_32} 1))).twin
						update_class_fields -- Current might have been changed; synchronize
						l_m___4.item := l_tmp___4.twin
						update_class_fields -- Current might have been changed; synchronize
						gsl_linalg_householder_hm (l_tau_i, (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					a_S.item.gsl_vector_set (l_i, l_tau_i)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_N))
				loop
					l_j := (0).to_natural_32
					from
					until
						not (l_j < l_i)
					loop
						a_X.item.gsl_matrix_set (l_i, l_j, 0.0)
						update_class_fields -- Current might have been changed; synchronize
						l_j := l_j + 1
					end
					l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_Aii := l_tmp___5
					a_X.item.gsl_matrix_set (l_i, l_i, l_Aii)
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_i + {NATURAL_32} 1
					from
					until
						not (l_j < l_N)
					loop
						l_tmp___6 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_Aij := l_tmp___6
						a_X.item.gsl_matrix_set (l_i, l_j, l_Aij)
						update_class_fields -- Current might have been changed; synchronize
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
				l_j := l_N
				from
				until
					(l_break)
				loop
					l_break := False
					l_tmp___9 := l_j
					l_j := l_j - 1
					if not (l_tmp___9 > {NATURAL_32} 0) then
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp___7 := a_S.item.gsl_vector_get (l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_tj := l_tmp___7
						l_tmp___8 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_submatrix (l_j, l_j, l_M - l_j, l_N - l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_m___5.item := l_tmp___8.twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___5.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_linalg_householder_hm1 (l_tj).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				l_break := False
				a_X.item.gsl_linalg_SV_decomp (a_V, a_S, a_work).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___10 := (a_work.item.gsl_vector_subvector ((0).to_natural_32, l_N)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_sum.item := l_tmp___10.twin
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_M))
				loop
					l_tmp___11 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row (l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_L_i.item := l_tmp___11.twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_sum.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set_zero
					update_class_fields -- Current might have been changed; synchronize
					l_j := (0).to_natural_32
					from
					until
						not (l_j < l_N)
					loop
						l_tmp___12 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_L_i.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get (l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_Lij := l_tmp___12
						l_tmp___13 := (a_X.item.gsl_matrix_row (l_j)).twin
						update_class_fields -- Current might have been changed; synchronize
						l_X_j.item := l_tmp___13.twin
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_X_j.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (l_Lij, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_sum.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_j := l_j + 1
					end
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_L_i.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_memcpy ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_sum.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_linalg_SV_solve (a_V: CE_POINTER [PD_GSL_MATRIX]; a_S, a_b___0, a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_w: CE_POINTER [PD_GSL_VECTOR]
			l_tmp: CE_POINTER [PD_GSL_VECTOR]
			l_wi: REAL_64
			l_tmp___0: REAL_64
			l_alpha: REAL_64
			l_tmp___1: REAL_64
			l_return, l_break: BOOLEAN
		do
			if size1 /= a_b___0.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first dimension of matrix U must size of vector b"), ce_string ("svd.c"), 430, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_S.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of vector S must match second dimension of matrix U"), ce_string ("svd.c"), 435, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_V.item.size1 /= a_V.item.size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix V must be square"), ce_string ("svd.c"), 439, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_S.item.size /= a_V.item.size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of vector S must match size of matrix V"), ce_string ("svd.c"), 444, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_V.item.size2 /= a_x.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of matrix V must match size of vector x"), ce_string ("svd.c"), 448, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_N := size2
				l_tmp := (create {P_INIT_SOURCE_STATIC}).gsl_vector_calloc (l_N)
				update_class_fields -- Current might have been changed; synchronize
				l_w := l_tmp;
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasTrans, 1.0, a_b___0, 0.0, l_w).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp___0 := l_w.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_wi := l_tmp___0
					l_tmp___1 := a_S.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_alpha := l_tmp___1
					if l_alpha /= (0).to_double then
						l_alpha := 1.0 / l_alpha
					end
					l_w.item.gsl_vector_set (l_i, l_alpha * l_wi)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				a_V.item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, l_w, 0.0, a_x).do_nothing
				update_class_fields; -- Current might have been changed; synchronize
				(create {P_INIT_SOURCE_STATIC}).gsl_vector_free (l_w)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_linalg_SV_decomp_jacobi (a_Q___1: CE_POINTER [PD_GSL_MATRIX]; a_S: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_count: INTEGER_32
			l_sweep: INTEGER_32
			l_sweepmax: INTEGER_32
			l_tolerance: REAL_64
			l_cj: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp: PD_GSL_VECTOR_VIEW
			l_sj: REAL_64
			l_tmp___0: REAL_64
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_p: CE_ARRAY [REAL_64]
			l_q___3: REAL_64
			l_cosine: REAL_64
			l_sine: REAL_64
			l_v: REAL_64
			l_abserr_a: REAL_64
			l_abserr_b: REAL_64
			l_sorted: INTEGER_32
			l_orthog: INTEGER_32
			l_noisya: INTEGER_32
			l_noisyb: INTEGER_32
			l_cj___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: PD_GSL_VECTOR_VIEW
			l_ck: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___2: PD_GSL_VECTOR_VIEW
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_Aik: REAL_64
			l_tmp___7: REAL_64
			l_Aij: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_Qij: REAL_64
			l_tmp___13: REAL_64
			l_Qik: REAL_64
			l_tmp___14: REAL_64
			l_prev_norm: REAL_64
			l_column: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___15: PD_GSL_VECTOR_VIEW
			l_norm___0: REAL_64
			l_tmp___16: REAL_64
			l_return, l_break, l_continue: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_cj.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_p.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_cj___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_ck.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_column.make (<<1>>)

			if size1 < size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("svd of MxN matrix, M<N, is not implemented"), ce_string ("svd.c"), 504, 24)
					update_class_fields -- Current might have been changed; synchronize
					Result := 24
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_Q___1.item.size1 /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("square matrix Q must match second dimension of matrix A"), ce_string ("svd.c"), 509, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_Q___1.item.size1 /= a_Q___1.item.size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix Q must be square"), ce_string ("svd.c"), 513, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_S.item.size /= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length of vector S must match second dimension of matrix A"), ce_string ("svd.c"), 518, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_M := size1
				l_N := size2
				l_count := 1
				l_sweep := 0
				l_sweepmax := ({NATURAL_32} 5 * l_N).to_integer_32
				l_tolerance := ({NATURAL_32} 10 * l_M).to_real_64 * 2.2204460492503131e-16
				if l_sweepmax > 12 then
					l_sweepmax := l_sweepmax
				else
					l_sweepmax := 12
				end
				a_Q___1.item.gsl_matrix_set_identity
				update_class_fields -- Current might have been changed; synchronize
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N)
				loop
					l_tmp := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_j)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_cj.item := l_tmp.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_cj.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
					update_class_fields -- Current might have been changed; synchronize
					l_sj := l_tmp___0
					a_S.item.gsl_vector_set (l_j, 2.2204460492503131e-16 * l_sj)
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				from
				until
					(l_return or l_break)
				loop
					l_break := False
					if l_count > 0 then
						if not (l_sweep <= l_sweepmax) then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_return or l_break or l_continue) then
						l_count := ((l_N * (l_N - {NATURAL_32} 1)) // {NATURAL_32} 2).to_integer_32
						l_j := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_j < l_N - {NATURAL_32} 1))
						loop
							l_k := l_j + {NATURAL_32} 1
							from
							until
								not (not (l_return) and then (l_k < l_N))
							loop
								l_continue := False
								l_a___5 := 0.0
								l_b___0 := 0.0
								l_p.item := 0.0
								update_class_fields -- Current might have been changed; synchronize
								l_q___3 := 0.0
								l_tmp___1 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_j)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_cj___0.item := l_tmp___1.twin
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___2 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_k)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_ck.item := l_tmp___2.twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_cj___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ck.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_p).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								l_p.item := l_p.item * (2.0)
								update_class_fields -- Current might have been changed; synchronize
								l_a___5 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_cj___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
								update_class_fields -- Current might have been changed; synchronize
								l_b___0 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_ck.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
								update_class_fields -- Current might have been changed; synchronize
								l_q___3 := l_a___5 * l_a___5 - l_b___0 * l_b___0
								l_v := hypot (l_p.item, l_q___3)
								update_class_fields -- Current might have been changed; synchronize
								l_abserr_a := a_S.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_abserr_b := a_S.item.gsl_vector_get (l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___3 := (create {P_COERCE_STATIC}).gsl_coerce_double (l_a___5)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___4 := (create {P_COERCE_STATIC}).gsl_coerce_double (l_b___0)
								update_class_fields -- Current might have been changed; synchronize
								l_sorted := (l_tmp___3 >= l_tmp___4).to_integer
								l_tmp___5 := fabs (l_p.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___6 := (create {P_COERCE_STATIC}).gsl_coerce_double (l_a___5 * l_b___0)
								update_class_fields -- Current might have been changed; synchronize
								l_orthog := (l_tmp___5 <= l_tolerance * l_tmp___6).to_integer
								l_noisya := (l_a___5 < l_abserr_a).to_integer
								l_noisyb := (l_b___0 < l_abserr_b).to_integer
								if l_sorted.to_boolean then
									if l_orthog.to_boolean then
										l_count := l_count - 1
										l_continue := True
									elseif l_noisya.to_boolean then
										l_count := l_count - 1
										l_continue := True
									elseif l_noisyb.to_boolean then
										l_count := l_count - 1
										l_continue := True
									end
								end
								if not (l_return or l_break or l_continue) then
									if l_v = (0).to_double then
										l_cosine := 0.0
										l_sine := 1.0
									elseif not l_sorted.to_boolean then
										l_cosine := 0.0
										l_sine := 1.0
									else
										l_cosine := sqrt ((l_v + l_q___3) / (2.0 * l_v))
										update_class_fields -- Current might have been changed; synchronize
										l_sine := l_p.item / ((2.0 * l_v) * l_cosine)
									end
									l_i := (0).to_natural_32
									from
									until
										not (l_i < l_M)
									loop
										l_tmp___7 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_k)
										update_class_fields -- Current might have been changed; synchronize
										l_Aik := l_tmp___7
										l_tmp___8 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_Aij := l_tmp___8;
										((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, l_Aij * l_cosine + l_Aik * l_sine)
										update_class_fields; -- Current might have been changed; synchronize
										((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_k, -l_Aij * l_sine + l_Aik * l_cosine)
										update_class_fields -- Current might have been changed; synchronize
										l_i := l_i + 1
									end
									l_tmp___9 := fabs (l_cosine)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___10 := fabs (l_sine)
									update_class_fields -- Current might have been changed; synchronize
									a_S.item.gsl_vector_set (l_j, l_tmp___9 * l_abserr_a + l_tmp___10 * l_abserr_b)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___11 := fabs (l_sine)
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___12 := fabs (l_cosine)
									update_class_fields -- Current might have been changed; synchronize
									a_S.item.gsl_vector_set (l_k, l_tmp___11 * l_abserr_a + l_tmp___12 * l_abserr_b)
									update_class_fields -- Current might have been changed; synchronize
									l_i := (0).to_natural_32
									from
									until
										not (l_i < l_N)
									loop
										l_tmp___13 := a_Q___1.item.gsl_matrix_get (l_i, l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_Qij := l_tmp___13
										l_tmp___14 := a_Q___1.item.gsl_matrix_get (l_i, l_k)
										update_class_fields -- Current might have been changed; synchronize
										l_Qik := l_tmp___14
										a_Q___1.item.gsl_matrix_set (l_i, l_j, l_Qij * l_cosine + l_Qik * l_sine)
										update_class_fields -- Current might have been changed; synchronize
										a_Q___1.item.gsl_matrix_set (l_i, l_k, -l_Qij * l_sine + l_Qik * l_cosine)
										update_class_fields -- Current might have been changed; synchronize
										l_i := l_i + 1
									end
								end
								if (not l_return) then
									l_k := l_k + 1
								end
							end
							l_continue := False
							l_j := l_j + 1
						end
						l_sweep := l_sweep + 1
					end
				end
				l_break := False
				l_prev_norm := -1.0
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N)
				loop
					l_tmp___15 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column (l_j)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_column.item := l_tmp___15.twin
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___16 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
					update_class_fields -- Current might have been changed; synchronize
					l_norm___0 := l_tmp___16
					if l_norm___0 = 0.0 then
						a_S.item.gsl_vector_set (l_j, 0.0)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set_zero
						update_class_fields -- Current might have been changed; synchronize
						l_prev_norm := 0.0
					elseif l_prev_norm = 0.0 then
						a_S.item.gsl_vector_set (l_j, 0.0)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set_zero
						update_class_fields -- Current might have been changed; synchronize
						l_prev_norm := 0.0
					elseif l_j > {NATURAL_32} 0 then
						if l_norm___0 <= l_tolerance * l_prev_norm then
							a_S.item.gsl_vector_set (l_j, 0.0)
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_set_zero
							update_class_fields -- Current might have been changed; synchronize
							l_prev_norm := 0.0
						else
							a_S.item.gsl_vector_set (l_j, l_norm___0)
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_scale (1.0 / l_norm___0).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_prev_norm := l_norm___0
						end
					else
						a_S.item.gsl_vector_set (l_j, l_norm___0)
						update_class_fields; -- Current might have been changed; synchronize
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_scale (1.0 / l_norm___0).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_prev_norm := l_norm___0
					end
					l_j := l_j + 1
				end
				if l_count > 0 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("Jacobi iterations did not reach desired tolerance"), ce_string ("svd.c"), 674, 14)
						update_class_fields -- Current might have been changed; synchronize
						Result := 14
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
				if not l_return then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_matrix_isnull: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size1))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_size2))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_k < {NATURAL_32} 1))
					loop
						if data[((l_i * l_tda + l_j) + l_k).to_integer_32] /= 0.0 then
							Result := 0
							l_return := True
						end
						if not l_return then
							l_k := l_k + 1
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_matrix_ispos: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size1))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_size2))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_k < {NATURAL_32} 1))
					loop
						if data[((l_i * l_tda + l_j) + l_k).to_integer_32] <= 0.0 then
							Result := 0
							l_return := True
						end
						if not l_return then
							l_k := l_k + 1
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_matrix_isneg: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size1))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_size2))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_k < {NATURAL_32} 1))
					loop
						if data[((l_i * l_tda + l_j) + l_k).to_integer_32] >= 0.0 then
							Result := 0
							l_return := True
						end
						if not l_return then
							l_k := l_k + 1
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_matrix_isnonneg: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size1))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_size2))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_k < {NATURAL_32} 1))
					loop
						if data[((l_i * l_tda + l_j) + l_k).to_integer_32] < 0.0 then
							Result := 0
							l_return := True
						end
						if not l_return then
							l_k := l_k + 1
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_matrix_max: REAL_64
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_max: REAL_64
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x > l_max then
						l_max := l_x
					end
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x.truncated_to_real)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						Result := l_x
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_max
			end
		end

	gsl_matrix_min: REAL_64
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_min: REAL_64
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x < l_min then
						l_min := l_x
					end
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x.truncated_to_real)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						Result := l_x
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_min
			end
		end

	gsl_matrix_minmax (a_min_out, a_max_out: CE_POINTER [REAL_64])
		require
			not a_min_out.is_void
			not a_max_out.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_max: REAL_64
			l_min: REAL_64
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x < l_min then
						l_min := l_x
					end
					if l_x > l_max then
						l_max := l_x
					end
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x.truncated_to_real)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						a_min_out.item := l_x
						update_class_fields -- Current might have been changed; synchronize
						a_max_out.item := l_x
						update_class_fields -- Current might have been changed; synchronize
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				a_min_out.item := l_min
				update_class_fields -- Current might have been changed; synchronize
				a_max_out.item := l_max
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gsl_matrix_max_index (a_imax_out, a_jmax_out: CE_POINTER [NATURAL_32])
		require
			not a_imax_out.is_void
			not a_jmax_out.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_max: REAL_64
			l_imax: NATURAL_32
			l_jmax: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_imax := (0).to_natural_32
			l_jmax := (0).to_natural_32
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x > l_max then
						l_max := l_x
						l_imax := l_i
						l_jmax := l_j
					end
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x.truncated_to_real)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						a_imax_out.item := l_i
						update_class_fields -- Current might have been changed; synchronize
						a_jmax_out.item := l_j
						update_class_fields -- Current might have been changed; synchronize
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				a_imax_out.item := l_imax
				update_class_fields -- Current might have been changed; synchronize
				a_jmax_out.item := l_jmax
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gsl_matrix_min_index (a_imin_out, a_jmin_out: CE_POINTER [NATURAL_32])
		require
			not a_imin_out.is_void
			not a_jmin_out.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_min: REAL_64
			l_imin: NATURAL_32
			l_jmin: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_imin := (0).to_natural_32
			l_jmin := (0).to_natural_32
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x < l_min then
						l_min := l_x
						l_imin := l_i
						l_jmin := l_j
					end
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x.truncated_to_real)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						a_imin_out.item := l_i
						update_class_fields -- Current might have been changed; synchronize
						a_jmin_out.item := l_j
						update_class_fields -- Current might have been changed; synchronize
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				a_imin_out.item := l_imin
				update_class_fields -- Current might have been changed; synchronize
				a_jmin_out.item := l_jmin
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gsl_matrix_minmax_index (a_imin_out, a_jmin_out, a_imax_out, a_jmax_out: CE_POINTER [NATURAL_32])
		require
			not a_imin_out.is_void
			not a_jmin_out.is_void
			not a_imax_out.is_void
			not a_jmax_out.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_imin: NATURAL_32
			l_jmin: NATURAL_32
			l_imax: NATURAL_32
			l_jmax: NATURAL_32
			l_max: REAL_64
			l_min: REAL_64
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_imin := (0).to_natural_32
			l_jmin := (0).to_natural_32
			l_imax := (0).to_natural_32
			l_jmax := (0).to_natural_32
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x < l_min then
						l_min := l_x
						l_imin := l_i
						l_jmin := l_j
					end
					if l_x > l_max then
						l_max := l_x
						l_imax := l_i
						l_jmax := l_j
					end
					if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x.truncated_to_real)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_64_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_64 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						a_imin_out.item := l_i
						update_class_fields -- Current might have been changed; synchronize
						a_jmin_out.item := l_j
						update_class_fields -- Current might have been changed; synchronize
						a_imax_out.item := l_i
						update_class_fields -- Current might have been changed; synchronize
						a_jmax_out.item := l_j
						update_class_fields -- Current might have been changed; synchronize
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				a_imin_out.item := l_imin
				update_class_fields -- Current might have been changed; synchronize
				a_jmin_out.item := l_jmin
				update_class_fields -- Current might have been changed; synchronize
				a_imax_out.item := l_imax
				update_class_fields -- Current might have been changed; synchronize
				a_jmax_out.item := l_jmax
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gsl_matrix_row (a_i: NATURAL_32): PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("rowcol_source.c"), 27, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + (a_i * tda).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := size2
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_column (a_j: NATURAL_32): PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 51, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + a_j.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := size1
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_diagonal: PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_v: PD_GSL_VECTOR
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_v.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_v.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_v.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_v.data := data
			update_class_fields -- Current might have been changed; synchronize
			if size1 < size2 then
				l_v.size := size1
				update_class_fields -- Current might have been changed; synchronize
			else
				l_v.size := size2
				update_class_fields -- Current might have been changed; synchronize
			end
			l_v.stride := tda + {NATURAL_32} 1
			update_class_fields -- Current might have been changed; synchronize
			l_v.block := block
			update_class_fields -- Current might have been changed; synchronize
			l_v.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector := l_v.twin
			update_class_fields -- Current might have been changed; synchronize
			Result := l_view.twin
		end

	gsl_matrix_subdiagonal (a_k: NATURAL_32): PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_k >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subdiagonal index is out of range"), ce_string ("rowcol_source.c"), 92, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + (a_k * tda).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				if size1 - a_k < size2 then
					l_v.size := size1 - a_k
					update_class_fields -- Current might have been changed; synchronize
				else
					l_v.size := size2
					update_class_fields -- Current might have been changed; synchronize
				end
				l_v.stride := tda + {NATURAL_32} 1
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_superdiagonal (a_k: NATURAL_32): PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_k >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 118, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + a_k.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				if size1 < size2 - a_k then
					l_v.size := size1
					update_class_fields -- Current might have been changed; synchronize
				else
					l_v.size := size2 - a_k
					update_class_fields -- Current might have been changed; synchronize
				end
				l_v.stride := tda + {NATURAL_32} 1
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_subrow (a_i, a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("rowcol_source.c"), 142, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("rowcol_source.c"), 147, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offset + a_n > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimension n overflows matrix"), ce_string ("rowcol_source.c"), 151, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + (a_i * tda + a_offset).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_subcolumn (a_j, a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 175, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("rowcol_source.c"), 180, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offset + a_n > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimension n overflows matrix"), ce_string ("rowcol_source.c"), 184, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + (a_offset * tda + a_j).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_const_row (a_i: NATURAL_32): PD_GSL_VECTOR_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_CONST_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("rowcol_source.c"), 27, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + (a_i * tda).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := size2
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_const_column (a_j: NATURAL_32): PD_GSL_VECTOR_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_CONST_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 51, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + a_j.to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := size1
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_const_diagonal: PD_GSL_VECTOR_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_CONST_VIEW
			l_v: PD_GSL_VECTOR
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_v.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_v.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_v.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_v.data := data
			update_class_fields -- Current might have been changed; synchronize
			if size1 < size2 then
				l_v.size := size1
				update_class_fields -- Current might have been changed; synchronize
			else
				l_v.size := size2
				update_class_fields -- Current might have been changed; synchronize
			end
			l_v.stride := (tda + {NATURAL_32} 1)
			update_class_fields -- Current might have been changed; synchronize
			l_v.block := block
			update_class_fields -- Current might have been changed; synchronize
			l_v.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector := l_v.twin
			update_class_fields -- Current might have been changed; synchronize
			Result := l_view.twin
		end

	gsl_matrix_const_subdiagonal (a_k: NATURAL_32): PD_GSL_VECTOR_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_CONST_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_k >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subdiagonal index is out of range"), ce_string ("rowcol_source.c"), 92, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + (a_k * tda).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				if size1 - a_k < size2 then
					l_v.size := (size1 - a_k)
					update_class_fields -- Current might have been changed; synchronize
				else
					l_v.size := size2
					update_class_fields -- Current might have been changed; synchronize
				end
				l_v.stride := (tda + {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_const_superdiagonal (a_k: NATURAL_32): PD_GSL_VECTOR_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_CONST_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_k >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 118, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + a_k.to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				if size1 < size2 - a_k then
					l_v.size := size1
					update_class_fields -- Current might have been changed; synchronize
				else
					l_v.size := (size2 - a_k)
					update_class_fields -- Current might have been changed; synchronize
				end
				l_v.stride := (tda + {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_const_subrow (a_i, a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_CONST_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("rowcol_source.c"), 142, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("rowcol_source.c"), 147, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offset + a_n > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimension n overflows matrix"), ce_string ("rowcol_source.c"), 151, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + (a_i * tda + a_offset).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_const_subcolumn (a_j, a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_CONST_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 175, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("rowcol_source.c"), 180, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offset + a_n > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimension n overflows matrix"), ce_string ("rowcol_source.c"), 184, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + (a_offset * tda + a_j).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_swap_rows (a_i, a_j: NATURAL_32): INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_row1: CE_POINTER [REAL_64]
			l_row2: CE_POINTER [REAL_64]
			l_k: NATURAL_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			if a_i >= l_size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first row index is out of range"), ce_string ("swap_source.c"), 29, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_size1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second row index is out of range"), ce_string ("swap_source.c"), 34, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_i /= a_j then
					l_row1 := data + (a_i * tda).to_integer_32
					l_row2 := data + (a_j * tda).to_integer_32
					l_k := (0).to_natural_32
					from
					until
						not (l_k < l_size2)
					loop
						l_tmp := l_row1[l_k.to_integer_32]
						l_row1[l_k.to_integer_32] := l_row2[l_k.to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_row2[l_k.to_integer_32] := l_tmp
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
				end
				Result := 0
			end
		end

	gsl_matrix_swap_columns (a_i, a_j: NATURAL_32): INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_col1: CE_POINTER [REAL_64]
			l_col2: CE_POINTER [REAL_64]
			l_p: NATURAL_32
			l_k: NATURAL_32
			l_n: NATURAL_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			if a_i >= l_size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first column index is out of range"), ce_string ("swap_source.c"), 64, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_size2) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second column index is out of range"), ce_string ("swap_source.c"), 69, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_i /= a_j then
					l_col1 := data + a_i.to_integer_32
					l_col2 := data + a_j.to_integer_32
					l_p := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_p < l_size1))
					loop
						l_n := l_p * tda
						l_k := (0).to_natural_32
						from
						until
							not (l_k < {NATURAL_32} 1)
						loop
							l_tmp := l_col1[(l_n + l_k).to_integer_32]
							l_col1[(l_n + l_k).to_integer_32] := l_col2[(l_n + l_k).to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_col2[(l_n + l_k).to_integer_32] := l_tmp
							update_class_fields -- Current might have been changed; synchronize
							l_k := l_k + 1
						end
						l_p := l_p + 1
					end
				end
				Result := 0
			end
		end

	gsl_matrix_swap_rowcol (a_i, a_j: NATURAL_32): INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_row: CE_POINTER [REAL_64]
			l_col: CE_POINTER [REAL_64]
			l_p: NATURAL_32
			l_k: NATURAL_32
			l_r___3: NATURAL_32
			l_c___1: NATURAL_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			if l_size1 /= l_size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to swap row and column"), ce_string ("swap_source.c"), 106, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_i >= l_size1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("swap_source.c"), 111, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_size2) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("swap_source.c"), 116, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_row := data + (a_i * tda).to_integer_32
				l_col := data + a_j.to_integer_32
				l_p := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_p < l_size1))
				loop
					l_r___3 := l_p
					l_c___1 := l_p * tda
					l_k := (0).to_natural_32
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_tmp := l_col[(l_c___1 + l_k).to_integer_32]
						l_col[(l_c___1 + l_k).to_integer_32] := l_row[(l_r___3 + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_row[(l_r___3 + l_k).to_integer_32] := l_tmp
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_p := l_p + 1
				end
				Result := 0
			end
		end

	gsl_matrix_transpose: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_e1: NATURAL_32
			l_e2: NATURAL_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			if l_size1 /= l_size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to take transpose"), ce_string ("swap_source.c"), 154, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_size1))
				loop
					l_j := l_i + {NATURAL_32} 1
					from
					until
						not (not (l_return) and then (l_j < l_size2))
					loop
						l_k := (0).to_natural_32
						from
						until
							not (l_k < {NATURAL_32} 1)
						loop
							l_e1 := (l_i * tda + l_j) + l_k
							l_e2 := (l_j * tda + l_i) + l_k
							l_tmp := data[l_e1.to_integer_32]
							data[l_e1.to_integer_32] := data[l_e2.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							data[l_e2.to_integer_32] := l_tmp
							update_class_fields -- Current might have been changed; synchronize
							l_k := l_k + 1
						end
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_matrix_transpose_memcpy (a_src: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_size1: NATURAL_32
			l_src_size2: NATURAL_32
			l_dest_size1: NATURAL_32
			l_dest_size2: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_e1: NATURAL_32
			l_e2: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_src_size1 := a_src.item.size1
					l_src_size2 := a_src.item.size2
					l_dest_size1 := size1
					l_dest_size2 := size2
					if l_dest_size2 = l_src_size1 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if l_dest_size1 = l_src_size2 then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimensions of dest matrix must be transpose of src matrix"), ce_string ("swap_source.c"), 192, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if done
					if not l_return then
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_dest_size1))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (not (l_return) and then (l_j < l_dest_size2))
							loop
								l_k := (0).to_natural_32
								from
								until
									not (l_k < {NATURAL_32} 1)
								loop
									l_e1 := (l_i * tda + l_j) + l_k
									l_e2 := (l_j * a_src.item.tda + l_i) + l_k
									data[l_e1.to_integer_32] := a_src.item.data[l_e2.to_integer_32]
									update_class_fields -- Current might have been changed; synchronize
									l_k := l_k + 1
								end
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_memcpy (a_src: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_size1: NATURAL_32
			l_src_size2: NATURAL_32
			l_dest_size1: NATURAL_32
			l_dest_size2: NATURAL_32
			l_src_tda: NATURAL_32
			l_dest_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_src_size1 := a_src.item.size1
					l_src_size2 := a_src.item.size2
					l_dest_size1 := size1
					l_dest_size2 := size2
					if l_src_size1 = l_dest_size1 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if l_src_size2 = l_dest_size2 then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix sizes are different"), ce_string ("copy_source.c"), 31, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if done
					if not l_return then
						l_src_tda := a_src.item.tda
						l_dest_tda := tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_src_size1))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_src_size2)
							loop
								data[(l_dest_tda * l_i + l_j).to_integer_32] := a_src.item.data[(l_src_tda * l_i + l_j).to_integer_32]
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_swap (a_src: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_size1: NATURAL_32
			l_src_size2: NATURAL_32
			l_dest_size1: NATURAL_32
			l_dest_size2: NATURAL_32
			l_src_tda: NATURAL_32
			l_dest_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_tmp: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_src_size1 := a_src.item.size1
					l_src_size2 := a_src.item.size2
					l_dest_size1 := size1
					l_dest_size2 := size2
					if l_src_size1 = l_dest_size1 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if l_src_size2 = l_dest_size2 then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix sizes are different"), ce_string ("copy_source.c"), 63, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if done
					if not l_return then
						l_src_tda := a_src.item.tda
						l_dest_tda := tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_src_size1))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_src_size2)
							loop
								l_tmp := a_src.item.data[(l_src_tda * l_i + l_j).to_integer_32]
								a_src.item.data[(l_src_tda * l_i + l_j).to_integer_32] := data[(l_dest_tda * l_i + l_j).to_integer_32]
								update_class_fields -- Current might have been changed; synchronize
								data[(l_dest_tda * l_i + l_j).to_integer_32] := l_tmp
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_add (a_b___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda_a: NATURAL_32
			l_tda_b: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_b___0.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_b___0.item.size2 = l_N then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrices must have same dimensions"), ce_string ("oper_source.c"), 28, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_tda_a := tda
						l_tda_b := a_b___0.item.tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_M))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								data[(l_i * l_tda_a + l_j).to_integer_32] := data[(l_i * l_tda_a + l_j).to_integer_32] + (a_b___0.item.data[(l_i * l_tda_b + l_j).to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_sub (a_b___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda_a: NATURAL_32
			l_tda_b: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_b___0.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_b___0.item.size2 = l_N then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrices must have same dimensions"), ce_string ("oper_source.c"), 57, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_tda_a := tda
						l_tda_b := a_b___0.item.tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_M))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								data[(l_i * l_tda_a + l_j).to_integer_32] := data[(l_i * l_tda_a + l_j).to_integer_32] - (a_b___0.item.data[(l_i * l_tda_b + l_j).to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_mul_elements (a_b___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda_a: NATURAL_32
			l_tda_b: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_b___0.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_b___0.item.size2 = l_N then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrices must have same dimensions"), ce_string ("oper_source.c"), 86, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_tda_a := tda
						l_tda_b := a_b___0.item.tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_M))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								data[(l_i * l_tda_a + l_j).to_integer_32] := data[(l_i * l_tda_a + l_j).to_integer_32] * (a_b___0.item.data[(l_i * l_tda_b + l_j).to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_div_elements (a_b___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda_a: NATURAL_32
			l_tda_b: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_b___0.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_b___0.item.size2 = l_N then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrices must have same dimensions"), ce_string ("oper_source.c"), 115, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_tda_a := tda
						l_tda_b := a_b___0.item.tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_M))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								data[(l_i * l_tda_a + l_j).to_integer_32] := data[(l_i * l_tda_a + l_j).to_integer_32] / (a_b___0.item.data[(l_i * l_tda_b + l_j).to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_scale (a_x: REAL_64): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N)
				loop
					data[(l_i * l_tda + l_j).to_integer_32] := data[(l_i * l_tda + l_j).to_integer_32] * a_x
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_matrix_add_constant (a_x: REAL_64): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N)
				loop
					data[(l_i * l_tda + l_j).to_integer_32] := data[(l_i * l_tda + l_j).to_integer_32] + a_x
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_matrix_add_diagonal (a_x: REAL_64): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_loop_lim: NATURAL_32
			l_tmp: NATURAL_32
			l_i: NATURAL_32
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			if l_M < l_N then
				l_tmp := l_M
			else
				l_tmp := l_N
			end
			l_loop_lim := l_tmp
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_loop_lim)
			loop
				data[(l_i * l_tda + l_i).to_integer_32] := data[(l_i * l_tda + l_i).to_integer_32] + a_x
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_matrix_set_row (a_i: NATURAL_32; a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_v_data: CE_POINTER [REAL_64]
			l_row_data: CE_POINTER [REAL_64]
			l_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			if a_i >= l_M then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("getset_source.c"), 99, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_v.item.size /= l_N) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix row size and vector length are not equal"), ce_string ("getset_source.c"), 105, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v_data := a_v.item.data
				l_row_data := data + (a_i * l_tda).to_integer_32
				l_stride := a_v.item.stride
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_k := {NATURAL_32} 0
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_row_data[(l_j + l_k).to_integer_32] := l_v_data[(l_stride * l_j + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_matrix_set_col (a_j: NATURAL_32; a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_v_data: CE_POINTER [REAL_64]
			l_column_data: CE_POINTER [REAL_64]
			l_stride: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			if a_j >= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("getset_source.c"), 140, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_v.item.size /= l_M) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix column size and vector length are not equal"), ce_string ("getset_source.c"), 146, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v_data := a_v.item.data
				l_column_data := data + a_j.to_integer_32
				l_stride := a_v.item.stride
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_M))
				loop
					l_k := {NATURAL_32} 0
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_column_data[(l_i * l_tda + l_k).to_integer_32] := l_v_data[(l_stride * l_i + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_vector_alloc_row_from_matrix (a_i: NATURAL_32): CE_POINTER [PD_GSL_VECTOR]
		local
			l_v: CE_POINTER [PD_GSL_VECTOR]
			l_M: NATURAL_32
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			if a_i >= l_M then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("getset_source.c"), 181, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_tmp := malloc ((create {PD_GSL_VECTOR}.make).structure_size.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v := create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_cast (l_tmp)
				if l_v.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for vector struct"), ce_string ("getset_source.c"), 189, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_v.item.data := data + (a_i * tda).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.size := size2
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_v
			end
		end

	gsl_vector_alloc_col_from_matrix (a_j: NATURAL_32): CE_POINTER [PD_GSL_VECTOR]
		local
			l_v: CE_POINTER [PD_GSL_VECTOR]
			l_N: NATURAL_32
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			l_N := size2
			if a_j >= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("getset_source.c"), 210, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_tmp := malloc ((create {PD_GSL_VECTOR}.make).structure_size.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v := create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_cast (l_tmp)
				if l_v.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for vector struct"), ce_string ("getset_source.c"), 218, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_v.item.data := data + a_j.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.size := size1
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_v
			end
		end

	gsl_matrix_fread (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_break: BOOLEAN
		do
			l_status := 0
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			if l_tda = l_size2 then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fread (a_stream, data, l_size1 * l_size2, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
			else
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_size1))
				loop
					l_break := False
					l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fread (a_stream, data + (l_i * l_tda).to_integer_32, l_size2, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					if l_status.to_boolean then
						l_break := True
					end
					if not (l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
			end
			Result := l_status
		end

	gsl_matrix_fwrite (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_break: BOOLEAN
		do
			l_status := 0
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			if l_tda = l_size2 then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fwrite (a_stream, data, l_size1 * l_size2, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
			else
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_size1))
				loop
					l_break := False
					l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_raw_fwrite (a_stream, (data + (l_i * l_tda).to_integer_32), l_size2, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					if l_status.to_boolean then
						l_break := True
					end
					if not (l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
			end
			Result := l_status
		end

	gsl_matrix_fprintf (a_stream: CE_POINTER [SD_IO_FILE]; a_format: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_status: INTEGER_32
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_break: BOOLEAN
		do
			l_status := 0
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			if l_tda = l_size2 then
				l_status := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_raw_fprintf (a_stream, data, l_size1 * l_size2, (1).to_natural_32, a_format)
				update_class_fields -- Current might have been changed; synchronize
			else
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_size1))
				loop
					l_break := False
					l_status := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_raw_fprintf (a_stream, (data + (l_i * l_tda).to_integer_32), l_size2, (1).to_natural_32, a_format)
					update_class_fields -- Current might have been changed; synchronize
					if l_status.to_boolean then
						l_break := True
					end
					if not (l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
			end
			Result := l_status
		end

	gsl_matrix_fscanf (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_break: BOOLEAN
		do
			l_status := 0
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			if l_tda = l_size2 then
				l_status := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_raw_fscanf (a_stream, data, l_size1 * l_size2, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
			else
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_size1))
				loop
					l_break := False
					l_status := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_raw_fscanf (a_stream, data + (l_i * l_tda).to_integer_32, l_size2, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					if l_status.to_boolean then
						l_break := True
					end
					if not (l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
			end
			Result := l_status
		end

	gsl_matrix_submatrix (a_i, a_j, a_n1, a_n2: NATURAL_32): PD_GSL_MATRIX_VIEW
		local
			l_view: PD_GSL_MATRIX_VIEW
			l_s: PD_GSL_MATRIX
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("submatrix_source.c"), 29, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("submatrix_source.c"), 33, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first dimension must be non-zero"), ce_string ("submatrix_source.c"), 37, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second dimension must be non-zero"), ce_string ("submatrix_source.c"), 41, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_i + a_n1 > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first dimension overflows matrix"), ce_string ("submatrix_source.c"), 45, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_j + a_n2 > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second dimension overflows matrix"), ce_string ("submatrix_source.c"), 49, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := data + (a_i * tda + a_j).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_s.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_s.tda := tda
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_const_submatrix (a_i, a_j, a_n1, a_n2: NATURAL_32): PD_GSL_MATRIX_CONST_VIEW
		local
			l_view: PD_GSL_MATRIX_CONST_VIEW
			l_s: PD_GSL_MATRIX
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("submatrix_source.c"), 29, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("submatrix_source.c"), 33, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first dimension must be non-zero"), ce_string ("submatrix_source.c"), 37, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second dimension must be non-zero"), ce_string ("submatrix_source.c"), 41, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_i + a_n1 > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first dimension overflows matrix"), ce_string ("submatrix_source.c"), 45, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_j + a_n2 > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second dimension overflows matrix"), ce_string ("submatrix_source.c"), 49, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := (data + (a_i * tda + a_j).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_s.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_s.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_s.tda := tda
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_alloc_from_matrix (a_k1, a_k2, a_n1, a_n2: NATURAL_32): CE_POINTER [PD_GSL_MATRIX]
		local
			l_m___4: CE_POINTER [PD_GSL_MATRIX]
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("init_source.c"), 146, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("init_source.c"), 151, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_k1 + a_n1 > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("submatrix dimension 1 exceeds size of original"), ce_string ("init_source.c"), 156, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_k2 + a_n2 > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("submatrix dimension 2 exceeds size of original"), ce_string ("init_source.c"), 161, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_tmp := malloc ((create {PD_GSL_MATRIX}.make).structure_size.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4 := create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_cast (l_tmp)
				if l_m___4.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for matrix struct"), ce_string ("init_source.c"), 169, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_m___4.item.data := (data + (a_k1 * tda).to_integer_32) + a_k2.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.tda := tda
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				Result := l_m___4
			end
		end

	gsl_matrix_set_identity
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_data: CE_POINTER [REAL_64]
			l_p: NATURAL_32
			l_q___3: NATURAL_32
			l_tda: NATURAL_32
			l_zero: REAL_64
			l_one: REAL_64
			l_return: BOOLEAN
		do
			l_data := data
			l_p := size1
			l_q___3 := size2
			l_tda := tda
			l_zero := 0.0
			l_one := 1.0
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_p))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_q___3)
				loop
					if l_i = l_j then
						l_data[(l_i * l_tda + l_j).to_integer_32] := l_one
						update_class_fields -- Current might have been changed; synchronize
					else
						l_data[(l_i * l_tda + l_j).to_integer_32] := l_zero
						update_class_fields -- Current might have been changed; synchronize
					end
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
		end

	gsl_matrix_set_zero
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_data: CE_POINTER [REAL_64]
			l_p: NATURAL_32
			l_q___3: NATURAL_32
			l_tda: NATURAL_32
			l_zero: REAL_64
			l_return: BOOLEAN
		do
			l_data := data
			l_p := size1
			l_q___3 := size2
			l_tda := tda
			l_zero := 0.0
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_p))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_q___3)
				loop
					l_data[(l_i * l_tda + l_j).to_integer_32] := l_zero
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
		end

	gsl_matrix_set_all (a_x: REAL_64)
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_data: CE_POINTER [REAL_64]
			l_p: NATURAL_32
			l_q___3: NATURAL_32
			l_tda: NATURAL_32
			l_return: BOOLEAN
		do
			l_data := data
			l_p := size1
			l_q___3 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_p))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_q___3)
				loop
					l_data[(l_i * l_tda + l_j).to_integer_32] := a_x
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
		end

	qrsolv (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_lambda: REAL_64; a_diag, a_qtb, a_x, a_sdiag, a_wa: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_nsing: NATURAL_32
			l_rjj: REAL_64
			l_tmp: REAL_64
			l_qtbj: REAL_64
			l_tmp___0: REAL_64
			l_rji: REAL_64
			l_tmp___1: REAL_64
			l_qtbpj: REAL_64
			l_pj: NATURAL_32
			l_tmp___2: NATURAL_32
			l_diagpj: REAL_64
			l_tmp___3: REAL_64
			l_sine: REAL_64
			l_cosine: REAL_64
			l_wak: REAL_64
			l_tmp___4: REAL_64
			l_rkk: REAL_64
			l_tmp___5: REAL_64
			l_sdiagk: REAL_64
			l_tmp___6: REAL_64
			l_cotangent: REAL_64
			l_tmp___7: REAL_64
			l_tangent: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_new_rkk: REAL_64
			l_new_wak: REAL_64
			l_rik: REAL_64
			l_tmp___11: REAL_64
			l_sdiagi: REAL_64
			l_tmp___12: REAL_64
			l_new_rik: REAL_64
			l_new_sdiagi: REAL_64
			l_rjj___0: REAL_64
			l_tmp___13: REAL_64
			l_xj: REAL_64
			l_tmp___14: REAL_64
			l_sdiagj: REAL_64
			l_tmp___15: REAL_64
			l_sum: REAL_64
			l_tmp___16: REAL_64
			l_tmp___17: REAL_64
			l_waj: REAL_64
			l_tmp___18: REAL_64
			l_sdiagj___0: REAL_64
			l_tmp___19: REAL_64
			l_pj___0: NATURAL_32
			l_tmp___20: NATURAL_32
			l_waj___0: REAL_64
			l_tmp___21: REAL_64
			l_return, l_break, l_continue: BOOLEAN
		do
			l_n := size2
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_rjj := l_tmp
				l_tmp___0 := a_qtb.item.gsl_vector_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_qtbj := l_tmp___0
				l_i := l_j + {NATURAL_32} 1
				from
				until
					not (l_i < l_n)
				loop
					l_tmp___1 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_rji := l_tmp___1;
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_j, l_rji)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				a_x.item.gsl_vector_set (l_j, l_rjj)
				update_class_fields -- Current might have been changed; synchronize
				a_wa.item.gsl_vector_set (l_j, l_qtbj)
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_j + 1
			end
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_continue := False
				l_tmp___2 := a_p.item.gsl_permutation_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_pj := l_tmp___2
				l_tmp___3 := a_diag.item.gsl_vector_get (l_pj)
				update_class_fields -- Current might have been changed; synchronize
				l_diagpj := a_lambda * l_tmp___3
				if l_diagpj = (0).to_double then
					l_continue := True
				end
				if not (l_return or l_continue) then
					a_sdiag.item.gsl_vector_set (l_j, l_diagpj)
					update_class_fields -- Current might have been changed; synchronize
					l_k := l_j + {NATURAL_32} 1
					from
					until
						not (l_k < l_n)
					loop
						a_sdiag.item.gsl_vector_set (l_k, 0.0)
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_qtbpj := (0).to_double
					l_k := l_j
					from
					until
						not (not (l_return) and then (l_k < l_n))
					loop
						l_continue := False
						l_tmp___4 := a_wa.item.gsl_vector_get (l_k)
						update_class_fields -- Current might have been changed; synchronize
						l_wak := l_tmp___4
						l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_k, l_k)
						update_class_fields -- Current might have been changed; synchronize
						l_rkk := l_tmp___5
						l_tmp___6 := a_sdiag.item.gsl_vector_get (l_k)
						update_class_fields -- Current might have been changed; synchronize
						l_sdiagk := l_tmp___6
						if l_sdiagk = (0).to_double then
							l_continue := True
						end
						if not (l_return or l_continue) then
							l_tmp___9 := fabs (l_rkk)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___10 := fabs (l_sdiagk)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___9 < l_tmp___10 then
								l_cotangent := l_rkk / l_sdiagk
								l_tmp___7 := sqrt (0.25 + (0.25 * l_cotangent) * l_cotangent)
								update_class_fields -- Current might have been changed; synchronize
								l_sine := 0.5 / l_tmp___7
								l_cosine := l_sine * l_cotangent
							else
								l_tangent := l_sdiagk / l_rkk
								l_tmp___8 := sqrt (0.25 + (0.25 * l_tangent) * l_tangent)
								update_class_fields -- Current might have been changed; synchronize
								l_cosine := 0.5 / l_tmp___8
								l_sine := l_cosine * l_tangent
							end
							l_new_rkk := l_cosine * l_rkk + l_sine * l_sdiagk
							l_new_wak := l_cosine * l_wak + l_sine * l_qtbpj
							l_qtbpj := -l_sine * l_wak + l_cosine * l_qtbpj;
							((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_k, l_k, l_new_rkk)
							update_class_fields -- Current might have been changed; synchronize
							a_wa.item.gsl_vector_set (l_k, l_new_wak)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_k + {NATURAL_32} 1
							from
							until
								not (l_i < l_n)
							loop
								l_tmp___11 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_rik := l_tmp___11
								l_tmp___12 := a_sdiag.item.gsl_vector_get (l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_sdiagi := l_tmp___12
								l_new_rik := l_cosine * l_rik + l_sine * l_sdiagi
								l_new_sdiagi := -l_sine * l_rik + l_cosine * l_sdiagi;
								((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_i, l_k, l_new_rik)
								update_class_fields -- Current might have been changed; synchronize
								a_sdiag.item.gsl_vector_set (l_i, l_new_sdiagi)
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
						end
						if (not l_return) then
							l_k := l_k + 1
						end
					end
					l_continue := False
					l_tmp___13 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_rjj___0 := l_tmp___13
					l_tmp___14 := a_x.item.gsl_vector_get (l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_xj := l_tmp___14
					a_sdiag.item.gsl_vector_set (l_j, l_rjj___0)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set (l_j, l_j, l_xj)
					update_class_fields -- Current might have been changed; synchronize
				end
				if (not l_return) then
					l_j := l_j + 1
				end
			end
			l_continue := False
			l_nsing := l_n
			l_j := (0).to_natural_32
			from
			until
				not (not (l_break) and then (l_j < l_n))
			loop
				l_break := False
				l_tmp___15 := a_sdiag.item.gsl_vector_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_sdiagj := l_tmp___15
				if l_sdiagj = (0).to_double then
					l_nsing := l_j
					l_break := True
				end
				if not (l_return or l_break or l_continue) then
					l_j := l_j + 1
				end
			end
			l_break := False
			l_j := l_nsing
			from
			until
				not (l_j < l_n)
			loop
				a_wa.item.gsl_vector_set (l_j, 0.0)
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_j + 1
			end
			l_k := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_k < l_nsing))
			loop
				l_sum := (0).to_double
				l_j := (l_nsing - {NATURAL_32} 1) - l_k
				l_i := l_j + {NATURAL_32} 1
				from
				until
					not (l_i < l_nsing)
				loop
					l_tmp___16 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___17 := a_wa.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_sum := l_sum + (l_tmp___16 * l_tmp___17)
					l_i := l_i + 1
				end
				l_tmp___18 := a_wa.item.gsl_vector_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_waj := l_tmp___18
				l_tmp___19 := a_sdiag.item.gsl_vector_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_sdiagj___0 := l_tmp___19
				a_wa.item.gsl_vector_set (l_j, (l_waj - l_sum) / l_sdiagj___0)
				update_class_fields -- Current might have been changed; synchronize
				l_k := l_k + 1
			end
			l_j := (0).to_natural_32
			from
			until
				not (l_j < l_n)
			loop
				l_tmp___20 := a_p.item.gsl_permutation_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_pj___0 := l_tmp___20
				l_tmp___21 := a_wa.item.gsl_vector_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_waj___0 := l_tmp___21
				a_x.item.gsl_vector_set (l_pj___0, l_waj___0)
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_j + 1
			end
			Result := 0
		end

	count_nsing: NATURAL_32
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_rii: REAL_64
			l_tmp: REAL_64
			l_break: BOOLEAN
		do
			l_n := size2
			l_i := (0).to_natural_32
			from
			until
				not (not (l_break) and then (l_i < l_n))
			loop
				l_break := False
				l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_rii := l_tmp
				if l_rii = (0).to_double then
					l_break := True
				end
				if not (l_break) then
					l_i := l_i + 1
				end
			end
			l_break := False
			Result := l_i
		end

	compute_newton_direction (a_perm: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_qtf, a_x: CE_POINTER [PD_GSL_VECTOR])
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_nsing: NATURAL_32
			l_qtfi: REAL_64
			l_tmp: REAL_64
			l_rjj: REAL_64
			l_tmp___0: REAL_64
			l_temp: REAL_64
			l_tmp___1: REAL_64
			l_rij: REAL_64
			l_tmp___2: REAL_64
			l_xi: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_n := size2
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := a_qtf.item.gsl_vector_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_qtfi := l_tmp
				a_x.item.gsl_vector_set (l_i, l_qtfi)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			l_nsing := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.count_nsing
			update_class_fields -- Current might have been changed; synchronize
			l_i := l_nsing
			from
			until
				not (l_i < l_n)
			loop
				a_x.item.gsl_vector_set (l_i, 0.0)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			if l_nsing > {NATURAL_32} 0 then
				l_j := l_nsing
				from
				until
					(l_return or l_break)
				loop
					l_break := False
					if l_j > {NATURAL_32} 0 then
						l_tmp___4 := l_j
						l_j := l_j - 1
						if not l_tmp___4.to_boolean then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_return or l_break) then
						l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_rjj := l_tmp___0
						l_tmp___1 := a_x.item.gsl_vector_get (l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_temp := l_tmp___1 / l_rjj
						a_x.item.gsl_vector_set (l_j, l_temp)
						update_class_fields -- Current might have been changed; synchronize
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_j)
						loop
							l_tmp___2 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_rij := l_tmp___2
							l_tmp___3 := a_x.item.gsl_vector_get (l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_xi := l_tmp___3
							a_x.item.gsl_vector_set (l_i, l_xi - l_rij * l_temp)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
					end
				end
				l_break := False
			end
			gsl_permute_vector_inverse (a_perm, a_x).do_nothing
			update_class_fields -- Current might have been changed; synchronize
		end

	compute_newton_correction (a_sdiag: CE_POINTER [PD_GSL_VECTOR]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_x: CE_POINTER [PD_GSL_VECTOR]; a_dxnorm: REAL_64; a_diag, a_w: CE_POINTER [PD_GSL_VECTOR])
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_pi: NATURAL_32
			l_tmp: NATURAL_32
			l_dpi: REAL_64
			l_tmp___0: REAL_64
			l_xpi: REAL_64
			l_tmp___1: REAL_64
			l_sj: REAL_64
			l_tmp___2: REAL_64
			l_wj: REAL_64
			l_tmp___3: REAL_64
			l_tj: REAL_64
			l_rij: REAL_64
			l_tmp___4: REAL_64
			l_wi: REAL_64
			l_tmp___5: REAL_64
			l_return: BOOLEAN
		do
			l_n := size2
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := a_p.item.gsl_permutation_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_pi := l_tmp
				l_tmp___0 := a_diag.item.gsl_vector_get (l_pi)
				update_class_fields -- Current might have been changed; synchronize
				l_dpi := l_tmp___0
				l_tmp___1 := a_x.item.gsl_vector_get (l_pi)
				update_class_fields -- Current might have been changed; synchronize
				l_xpi := l_tmp___1
				a_w.item.gsl_vector_set (l_i, (l_dpi * (l_dpi * l_xpi)) / a_dxnorm)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_tmp___2 := a_sdiag.item.gsl_vector_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_sj := l_tmp___2
				l_tmp___3 := a_w.item.gsl_vector_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_wj := l_tmp___3
				l_tj := l_wj / l_sj
				a_w.item.gsl_vector_set (l_j, l_tj)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_j + {NATURAL_32} 1
				from
				until
					not (l_i < l_n)
				loop
					l_tmp___4 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_rij := l_tmp___4
					l_tmp___5 := a_w.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_wi := l_tmp___5
					a_w.item.gsl_vector_set (l_i, l_wi - l_rij * l_tj)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_j := l_j + 1
			end
		end

	compute_newton_bound (a_x: CE_POINTER [PD_GSL_VECTOR]; a_dxnorm: REAL_64; a_perm: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_diag, a_w: CE_POINTER [PD_GSL_VECTOR])
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_nsing: NATURAL_32
			l_tmp: NATURAL_32
			l_pi: NATURAL_32
			l_tmp___0: NATURAL_32
			l_dpi: REAL_64
			l_tmp___1: REAL_64
			l_xpi: REAL_64
			l_tmp___2: REAL_64
			l_sum: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_rjj: REAL_64
			l_tmp___5: REAL_64
			l_wj: REAL_64
			l_tmp___6: REAL_64
			l_return: BOOLEAN
		do
			l_n := size2
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.count_nsing
			update_class_fields -- Current might have been changed; synchronize
			l_nsing := l_tmp
			if l_nsing < l_n then
				a_w.item.gsl_vector_set_zero
				update_class_fields -- Current might have been changed; synchronize
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_tmp___0 := a_perm.item.gsl_permutation_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_pi := l_tmp___0
					l_tmp___1 := a_diag.item.gsl_vector_get (l_pi)
					update_class_fields -- Current might have been changed; synchronize
					l_dpi := l_tmp___1
					l_tmp___2 := a_x.item.gsl_vector_get (l_pi)
					update_class_fields -- Current might have been changed; synchronize
					l_xpi := l_tmp___2
					a_w.item.gsl_vector_set (l_i, l_dpi * ((l_dpi * l_xpi) / a_dxnorm))
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_n))
				loop
					l_sum := (0).to_double
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_j)
					loop
						l_tmp___3 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___4 := a_w.item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_sum := l_sum + (l_tmp___3 * l_tmp___4)
						l_i := l_i + 1
					end
					l_tmp___5 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_j, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_rjj := l_tmp___5
					l_tmp___6 := a_w.item.gsl_vector_get (l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_wj := l_tmp___6
					a_w.item.gsl_vector_set (l_j, (l_wj - l_sum) / l_rjj)
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
			end
		end

	compute_gradient_direction (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_qtf, a_diag, a_g: CE_POINTER [PD_GSL_VECTOR])
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_sum: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_pj: NATURAL_32
			l_tmp___1: NATURAL_32
			l_dpj: REAL_64
			l_tmp___2: REAL_64
			l_return: BOOLEAN
		do
			l_n := size2
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_sum := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i <= l_j)
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := a_qtf.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_sum := l_sum + (l_tmp * l_tmp___0)
					l_i := l_i + 1
				end
				l_tmp___1 := a_p.item.gsl_permutation_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_pj := l_tmp___1
				l_tmp___2 := a_diag.item.gsl_vector_get (l_pj)
				update_class_fields -- Current might have been changed; synchronize
				l_dpj := l_tmp___2
				a_g.item.gsl_vector_set (l_j, l_sum / l_dpj)
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_j + 1
			end
		end

	lmpar (a_perm: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_qtf, a_diag: CE_POINTER [PD_GSL_VECTOR]; a_delta___0: REAL_64; a_par_inout: CE_POINTER [REAL_64]; a_newton, a_gradient, a_sdiag, a_x, a_w: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_par_inout.is_void
			not a_newton.is_void
		local
			l_dxnorm: REAL_64
			l_gnorm: REAL_64
			l_fp: REAL_64
			l_fp_old: REAL_64
			l_par_lower: REAL_64
			l_par_upper: REAL_64
			l_par_c: REAL_64
			l_par: REAL_64
			l_iter: NATURAL_32
			l_wnorm: REAL_64
			l_tmp: REAL_64
			l_phider: REAL_64
			l_tmp___0: REAL_64
			l_sqrt_par: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_wnorm___0: REAL_64
			l_tmp___3: REAL_64
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_par := a_par_inout.item
					l_iter := (0).to_natural_32;
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.compute_newton_direction (a_perm, a_qtf, a_newton)
					update_class_fields -- Current might have been changed; synchronize
					l_dxnorm := (create {P_LMUTIL_STATIC}).scaled_enorm (a_diag, a_newton)
					update_class_fields -- Current might have been changed; synchronize
					l_fp := l_dxnorm - a_delta___0
					if l_fp <= 0.1 * a_delta___0 then
						a_x.item.gsl_vector_memcpy (a_newton).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						a_par_inout.item := (0).to_double
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
						l_return := True
					end
					if l_return then
						l_pos := -1 + 7 -- skip if block
					end
				when 1 then -- if:;
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.compute_newton_bound (a_newton, l_dxnorm, a_perm, a_diag, a_w)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp := a_w.item.enorm
						update_class_fields -- Current might have been changed; synchronize
						l_wnorm := l_tmp
						l_phider := l_wnorm * l_wnorm
						if l_wnorm > (0).to_double then
							l_par_lower := l_fp / (a_delta___0 * l_phider)
						else
							l_par_lower := 0.0
						end
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.compute_gradient_direction (a_perm, a_qtf, a_diag, a_gradient)
						update_class_fields -- Current might have been changed; synchronize
						l_gnorm := a_gradient.item.enorm
						update_class_fields -- Current might have been changed; synchronize
						l_par_upper := l_gnorm / a_delta___0
						if l_par_upper = (0).to_double then
							l_tmp___0 := (create {P_GSL_GSL_MINMAX_STATIC}).GSL_MIN_DBL (a_delta___0, 0.1)
							update_class_fields -- Current might have been changed; synchronize
							l_par_upper := 2.2250738585072014e-308 / l_tmp___0
						end
						if l_par > l_par_upper then
							l_par := l_par_upper
						elseif l_par < l_par_lower then
							l_par := l_par_lower
						end
						if l_par = (0).to_double then
							l_par := l_gnorm / l_dxnorm
						end
				when 2 then -- iteration:
						l_iter := l_iter + 1
						if l_par = (0).to_double then
							l_par := (create {P_GSL_GSL_MINMAX_STATIC}).GSL_MAX_DBL (0.001 * l_par_upper, 2.2250738585072014e-308)
							update_class_fields -- Current might have been changed; synchronize
						end
						l_tmp___1 := sqrt (l_par)
						update_class_fields -- Current might have been changed; synchronize
						l_sqrt_par := l_tmp___1;
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.qrsolv (a_perm, l_sqrt_par, a_diag, a_qtf, a_x, a_sdiag, a_w).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_dxnorm := (create {P_LMUTIL_STATIC}).scaled_enorm (a_diag, a_x)
						update_class_fields -- Current might have been changed; synchronize
						l_fp_old := l_fp
						l_fp := l_dxnorm - a_delta___0
						l_tmp___2 := fabs (l_fp)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___2 <= 0.1 * a_delta___0 then
							l_pos := -1 + 6 -- goto line220
						end
				when 3 then
						if l_par_lower = (0).to_double then
							if l_fp <= l_fp_old then
								if l_fp_old < (0).to_double then
									l_pos := -1 + 6 -- goto line220
								end
							end
						end
				when 4 then
						if l_iter = {NATURAL_32} 10 then
							l_pos := -1 + 6 -- goto line220
						end
				when 5 then;
						((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.compute_newton_correction (a_sdiag, a_perm, a_x, l_dxnorm, a_diag, a_w)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := a_w.item.enorm
						update_class_fields -- Current might have been changed; synchronize
						l_wnorm___0 := l_tmp___3
						l_par_c := l_fp / ((a_delta___0 * l_wnorm___0) * l_wnorm___0)
						if l_fp > (0).to_double then
							if l_par > l_par_lower then
								l_par_lower := l_par
							end
						elseif l_fp < (0).to_double then
							if l_par < l_par_upper then
								l_par_upper := l_par
							end
						end
						l_par := (create {P_GSL_GSL_MINMAX_STATIC}).GSL_MAX_DBL (l_par_lower, l_par + l_par_c)
						update_class_fields -- Current might have been changed; synchronize
						l_pos := -1 + 2 -- goto iteration
				when 6 then -- line220:
						a_par_inout.item := l_par
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_multifit_gradient (a_f, a_g: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasTrans, 1.0, a_f, 0.0, a_g)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	multifit_linear_svd (a_y: CE_POINTER [PD_GSL_VECTOR]; a_tol: REAL_64; a_balance: INTEGER_32; a_rank: CE_POINTER [NATURAL_32]; a_c___1: CE_POINTER [PD_GSL_VECTOR]; a_cov: CE_POINTER [PD_GSL_MATRIX]; a_chisq: CE_POINTER [REAL_64]; a_work: CE_POINTER [PD_GSL_MULTIFIT_LINEAR_WORKSPACE]): INTEGER_32
		require
			not a_y.is_void
		local
			l_n: NATURAL_32
			l_p: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_p_eff: NATURAL_32
			l_A___0: CE_POINTER [PD_GSL_MATRIX]
			l_Q___1: CE_POINTER [PD_GSL_MATRIX]
			l_QSI: CE_POINTER [PD_GSL_MATRIX]
			l_S: CE_POINTER [PD_GSL_VECTOR]
			l_xt: CE_POINTER [PD_GSL_VECTOR]
			l_D: CE_POINTER [PD_GSL_VECTOR]
			l_alpha0: REAL_64
			l_tmp: REAL_64
			l_column: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_alpha: REAL_64
			l_tmp___1: REAL_64
			l_s2: REAL_64
			l_r2___0: REAL_64
			l_yi: REAL_64
			l_tmp___2: REAL_64
			l_row: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___3: PD_GSL_VECTOR_CONST_VIEW
			l_y_est: CE_ARRAY [REAL_64]
			l_ri: REAL_64
			l_row_i: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___4: PD_GSL_VECTOR_VIEW
			l_d_i: REAL_64
			l_tmp___5: REAL_64
			l_row_j: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___6: PD_GSL_VECTOR_VIEW
			l_d_j: REAL_64
			l_tmp___7: REAL_64
			l_s3: CE_ARRAY [REAL_64]
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_column.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_row.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_y_est.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_row_i.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_row_j.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s3.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if size1 = a_y.item.size then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of observations in y does not match rows of matrix X"), ce_string ("multilinear.c"), 49, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 3 then -- else:
						if size2 = a_c___1.item.size then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of parameters c does not match columns of matrix X"), ce_string ("multilinear.c"), 54, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 6 then -- else:
						if a_cov.item.size1 = a_cov.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("covariance matrix is not square"), ce_string ("multilinear.c"), 58, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 9 then -- else:
						if a_c___1.item.size = a_cov.item.size1 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of parameters does not match size of covariance matrix"), ce_string ("multilinear.c"), 64, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 12 then -- else:
						if size1 = a_work.item.n then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						l_pos := -1 + 16 -- goto _L
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 15 then -- else:
						if size2 = a_work.item.p then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of workspace does not match size of observation matrix"), ce_string ("multilinear.c"), 70, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 18 then -- else:
						if a_tol <= (0).to_double then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("tolerance must be positive"), ce_string ("multilinear.c"), 74, 4)
								update_class_fields -- Current might have been changed; synchronize
								Result := 4
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_n := size1
							l_p := size2
							l_A___0 := a_work.item.A
							l_Q___1 := a_work.item.Q
							l_QSI := a_work.item.QSI
							l_S := a_work.item.S
							l_xt := a_work.item.xt
							l_D := a_work.item.D
							l_A___0.item.gsl_matrix_memcpy ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							if a_balance.to_boolean then
								l_A___0.item.gsl_linalg_balance_columns (l_D).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							else
								l_D.item.gsl_vector_set_all (1.0)
								update_class_fields -- Current might have been changed; synchronize
							end
							l_A___0.item.gsl_linalg_SV_decomp_mod (l_QSI, l_Q___1, l_S, l_xt).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_A___0.item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasTrans, 1.0, a_y, 0.0, l_xt).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_QSI.item.gsl_matrix_memcpy (l_Q___1).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_tmp := l_S.item.gsl_vector_get ((0).to_natural_32)
							update_class_fields -- Current might have been changed; synchronize
							l_alpha0 := l_tmp
							l_p_eff := (0).to_natural_32
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_p)
							loop
								l_tmp___0 := (l_QSI.item.gsl_matrix_column (l_j)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_column.item := l_tmp___0.twin
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___1 := l_S.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_alpha := l_tmp___1
								if l_alpha <= a_tol * l_alpha0 then
									l_alpha := 0.0
								else
									l_alpha := 1.0 / l_alpha
									l_p_eff := l_p_eff + 1
								end
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_scale (l_alpha).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							a_rank.item := l_p_eff
							update_class_fields -- Current might have been changed; synchronize
							a_c___1.item.gsl_vector_set_zero
							update_class_fields -- Current might have been changed; synchronize
							l_QSI.item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, l_xt, 0.0, a_c___1).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							gsl_vector_div (a_c___1, l_D).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_s2 := (0).to_double
							l_r2___0 := (0).to_double
							l_i := (0).to_natural_32
							from
							until
								not (l_i < l_n)
							loop
								l_tmp___2 := a_y.item.gsl_vector_get (l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_yi := l_tmp___2
								l_tmp___3 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_row (l_i)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_row.item := l_tmp___3.twin
								update_class_fields; -- Current might have been changed; synchronize
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_row.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot (a_c___1, l_y_est).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								l_ri := l_yi - l_y_est.item
								l_r2___0 := l_r2___0 + (l_ri * l_ri)
								l_i := l_i + 1
							end
							l_s2 := l_r2___0 / (l_n - l_p_eff).to_real_64
							a_chisq.item := l_r2___0
							update_class_fields -- Current might have been changed; synchronize
							l_i := (0).to_natural_32
							from
							until
								not (not (l_return) and then (l_i < l_p))
							loop
								l_tmp___4 := (l_QSI.item.gsl_matrix_row (l_i)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_row_i.item := l_tmp___4.twin
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___5 := l_D.item.gsl_vector_get (l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_d_i := l_tmp___5
								l_j := l_i
								from
								until
									not (l_j < l_p)
								loop
									l_tmp___6 := (l_QSI.item.gsl_matrix_row (l_j)).twin
									update_class_fields -- Current might have been changed; synchronize
									l_row_j.item := l_tmp___6.twin
									update_class_fields -- Current might have been changed; synchronize
									l_tmp___7 := l_D.item.gsl_vector_get (l_j)
									update_class_fields -- Current might have been changed; synchronize
									l_d_j := l_tmp___7;
									(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_row_i.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_row_j.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_s3).do_nothing
									update_class_fields -- Current might have been changed; synchronize
									a_cov.item.gsl_matrix_set (l_i, l_j, (l_s3.item * l_s2) / (l_d_i * l_d_j))
									update_class_fields -- Current might have been changed; synchronize
									a_cov.item.gsl_matrix_set (l_j, l_i, (l_s3.item * l_s2) / (l_d_i * l_d_j))
									update_class_fields -- Current might have been changed; synchronize
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
							Result := 0
							l_return := True
						end
				when 19 then -- if done | if done | if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 20 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_multifit_linear (a_y, a_c___1: CE_POINTER [PD_GSL_VECTOR]; a_cov: CE_POINTER [PD_GSL_MATRIX]; a_chisq: CE_POINTER [REAL_64]; a_work: CE_POINTER [PD_GSL_MULTIFIT_LINEAR_WORKSPACE]): INTEGER_32
		require
			not a_y.is_void
		local
			l_rank: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			create {CE_NATURAL_32_ARRAY} l_rank.make (<<1>>)

			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.multifit_linear_svd (a_y, 2.2204460492503131e-16, 1, l_rank, a_c___1, a_cov, a_chisq, a_work)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_multifit_linear_svd (a_y: CE_POINTER [PD_GSL_VECTOR]; a_tol: REAL_64; a_rank: CE_POINTER [NATURAL_32]; a_c___1: CE_POINTER [PD_GSL_VECTOR]; a_cov: CE_POINTER [PD_GSL_MATRIX]; a_chisq: CE_POINTER [REAL_64]; a_work: CE_POINTER [PD_GSL_MULTIFIT_LINEAR_WORKSPACE]): INTEGER_32
		require
			not a_y.is_void
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.multifit_linear_svd (a_y, a_tol, 1, a_rank, a_c___1, a_cov, a_chisq, a_work)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_multifit_linear_usvd (a_y: CE_POINTER [PD_GSL_VECTOR]; a_tol: REAL_64; a_rank: CE_POINTER [NATURAL_32]; a_c___1: CE_POINTER [PD_GSL_VECTOR]; a_cov: CE_POINTER [PD_GSL_MATRIX]; a_chisq: CE_POINTER [REAL_64]; a_work: CE_POINTER [PD_GSL_MULTIFIT_LINEAR_WORKSPACE]): INTEGER_32
		require
			not a_y.is_void
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.multifit_linear_svd (a_y, a_tol, 0, a_rank, a_c___1, a_cov, a_chisq, a_work)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	multifit_wlinear_svd (a_w, a_y: CE_POINTER [PD_GSL_VECTOR]; a_tol: REAL_64; a_balance: INTEGER_32; a_rank: CE_POINTER [NATURAL_32]; a_c___1: CE_POINTER [PD_GSL_VECTOR]; a_cov: CE_POINTER [PD_GSL_MATRIX]; a_chisq: CE_POINTER [REAL_64]; a_work: CE_POINTER [PD_GSL_MULTIFIT_LINEAR_WORKSPACE]): INTEGER_32
		require
			not a_y.is_void
		local
			l_n: NATURAL_32
			l_p: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_p_eff: NATURAL_32
			l_A___0: CE_POINTER [PD_GSL_MATRIX]
			l_Q___1: CE_POINTER [PD_GSL_MATRIX]
			l_QSI: CE_POINTER [PD_GSL_MATRIX]
			l_S: CE_POINTER [PD_GSL_VECTOR]
			l_t: CE_POINTER [PD_GSL_VECTOR]
			l_xt: CE_POINTER [PD_GSL_VECTOR]
			l_D: CE_POINTER [PD_GSL_VECTOR]
			l_wi: REAL_64
			l_tmp: REAL_64
			l_row: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___0: PD_GSL_VECTOR_VIEW
			l_tmp___1: REAL_64
			l_wi___0: REAL_64
			l_tmp___2: REAL_64
			l_yi: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_alpha0: REAL_64
			l_tmp___5: REAL_64
			l_column: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___6: PD_GSL_VECTOR_VIEW
			l_alpha: REAL_64
			l_tmp___7: REAL_64
			l_row_i: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___8: PD_GSL_VECTOR_VIEW
			l_d_i: REAL_64
			l_tmp___9: REAL_64
			l_row_j: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___10: PD_GSL_VECTOR_VIEW
			l_d_j: REAL_64
			l_tmp___11: REAL_64
			l_s2: CE_ARRAY [REAL_64]
			l_r2___0: REAL_64
			l_yi___0: REAL_64
			l_tmp___12: REAL_64
			l_wi___1: REAL_64
			l_tmp___13: REAL_64
			l_row___0: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___14: PD_GSL_VECTOR_CONST_VIEW
			l_y_est: CE_ARRAY [REAL_64]
			l_ri: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_row.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_column.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_row_i.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_row_j.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s2.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_row___0.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_y_est.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if size1 = a_y.item.size then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of observations in y does not match rows of matrix X"), ce_string ("multilinear.c"), 249, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 3 then -- else:
						if size2 = a_c___1.item.size then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of parameters c does not match columns of matrix X"), ce_string ("multilinear.c"), 254, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 6 then -- else:
						if a_w.item.size = a_y.item.size then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of weights does not match number of observations"), ce_string ("multilinear.c"), 259, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 9 then -- else:
						if a_cov.item.size1 = a_cov.item.size2 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("covariance matrix is not square"), ce_string ("multilinear.c"), 263, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 12 then -- else:
						if a_c___1.item.size = a_cov.item.size1 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of parameters does not match size of covariance matrix"), ce_string ("multilinear.c"), 269, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 15 then -- else:
						if size1 = a_work.item.n then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						l_pos := -1 + 19 -- goto _L
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 18 then -- else:
						if size2 = a_work.item.p then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("size of workspace does not match size of observation matrix"), ce_string ("multilinear.c"), 275, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else:
						l_n := size1
						l_p := size2
						l_A___0 := a_work.item.A
						l_Q___1 := a_work.item.Q
						l_QSI := a_work.item.QSI
						l_S := a_work.item.S
						l_t := a_work.item.t
						l_xt := a_work.item.xt
						l_D := a_work.item.D
						l_A___0.item.gsl_matrix_memcpy ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_n)
						loop
							l_tmp := a_w.item.gsl_vector_get (l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_wi := l_tmp
							if l_wi < (0).to_double then
								l_wi := (0).to_double
							end
							l_tmp___0 := (l_A___0.item.gsl_matrix_row (l_i)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_row.item := l_tmp___0.twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___1 := sqrt (l_wi)
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_row.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_scale (l_tmp___1).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						if a_balance.to_boolean then
							l_A___0.item.gsl_linalg_balance_columns (l_D).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						else
							l_D.item.gsl_vector_set_all (1.0)
							update_class_fields -- Current might have been changed; synchronize
						end
						l_A___0.item.gsl_linalg_SV_decomp_mod (l_QSI, l_Q___1, l_S, l_xt).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_n)
						loop
							l_tmp___2 := a_w.item.gsl_vector_get (l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_wi___0 := l_tmp___2
							l_tmp___3 := a_y.item.gsl_vector_get (l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_yi := l_tmp___3
							if l_wi___0 < (0).to_double then
								l_wi___0 := (0).to_double
							end
							l_tmp___4 := sqrt (l_wi___0)
							update_class_fields -- Current might have been changed; synchronize
							l_t.item.gsl_vector_set (l_i, l_tmp___4 * l_yi)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_A___0.item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasTrans, 1.0, l_t, 0.0, l_xt).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_QSI.item.gsl_matrix_memcpy (l_Q___1).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___5 := l_S.item.gsl_vector_get ((0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_alpha0 := l_tmp___5
						l_p_eff := (0).to_natural_32
						l_j := (0).to_natural_32
						from
						until
							not (l_j < l_p)
						loop
							l_tmp___6 := (l_QSI.item.gsl_matrix_column (l_j)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_column.item := l_tmp___6.twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___7 := l_S.item.gsl_vector_get (l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_alpha := l_tmp___7
							if l_alpha <= a_tol * l_alpha0 then
								l_alpha := 0.0
							else
								l_alpha := 1.0 / l_alpha
								l_p_eff := l_p_eff + 1
							end
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_column.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_scale (l_alpha).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_j := l_j + 1
						end
						a_rank.item := l_p_eff
						update_class_fields -- Current might have been changed; synchronize
						a_c___1.item.gsl_vector_set_zero
						update_class_fields -- Current might have been changed; synchronize
						l_QSI.item.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, l_xt, 0.0, a_c___1).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						gsl_vector_div (a_c___1, l_D).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_p))
						loop
							l_tmp___8 := (l_QSI.item.gsl_matrix_row (l_i)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_row_i.item := l_tmp___8.twin
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___9 := l_D.item.gsl_vector_get (l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_d_i := l_tmp___9
							l_j := l_i
							from
							until
								not (l_j < l_p)
							loop
								l_tmp___10 := (l_QSI.item.gsl_matrix_row (l_j)).twin
								update_class_fields -- Current might have been changed; synchronize
								l_row_j.item := l_tmp___10.twin
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___11 := l_D.item.gsl_vector_get (l_j)
								update_class_fields -- Current might have been changed; synchronize
								l_d_j := l_tmp___11;
								(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_row_i.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_row_j.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_s2).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								a_cov.item.gsl_matrix_set (l_i, l_j, l_s2.item / (l_d_i * l_d_j))
								update_class_fields -- Current might have been changed; synchronize
								a_cov.item.gsl_matrix_set (l_j, l_i, l_s2.item / (l_d_i * l_d_j))
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						l_r2___0 := (0).to_double
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_n)
						loop
							l_tmp___12 := a_y.item.gsl_vector_get (l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_yi___0 := l_tmp___12
							l_tmp___13 := a_w.item.gsl_vector_get (l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_wi___1 := l_tmp___13
							l_tmp___14 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_row (l_i)).twin
							update_class_fields -- Current might have been changed; synchronize
							l_row___0.item := l_tmp___14.twin
							update_class_fields; -- Current might have been changed; synchronize
							(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_row___0.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot (a_c___1, l_y_est).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_ri := l_yi___0 - l_y_est.item
							l_r2___0 := l_r2___0 + ((l_wi___1 * l_ri) * l_ri)
							l_i := l_i + 1
						end
						a_chisq.item := l_r2___0
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
						l_return := True
				when 22 then -- if done | if done | if done | if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 23 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_multifit_wlinear (a_w, a_y, a_c___1: CE_POINTER [PD_GSL_VECTOR]; a_cov: CE_POINTER [PD_GSL_MATRIX]; a_chisq: CE_POINTER [REAL_64]; a_work: CE_POINTER [PD_GSL_MULTIFIT_LINEAR_WORKSPACE]): INTEGER_32
		require
			not a_y.is_void
		local
			l_rank: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			create {CE_NATURAL_32_ARRAY} l_rank.make (<<1>>)

			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.multifit_wlinear_svd (a_w, a_y, 2.2204460492503131e-16, 1, l_rank, a_c___1, a_cov, a_chisq, a_work)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_multifit_wlinear_svd (a_w, a_y: CE_POINTER [PD_GSL_VECTOR]; a_tol: REAL_64; a_rank: CE_POINTER [NATURAL_32]; a_c___1: CE_POINTER [PD_GSL_VECTOR]; a_cov: CE_POINTER [PD_GSL_MATRIX]; a_chisq: CE_POINTER [REAL_64]; a_work: CE_POINTER [PD_GSL_MULTIFIT_LINEAR_WORKSPACE]): INTEGER_32
		require
			not a_y.is_void
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.multifit_wlinear_svd (a_w, a_y, a_tol, 1, a_rank, a_c___1, a_cov, a_chisq, a_work)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_multifit_wlinear_usvd (a_w, a_y: CE_POINTER [PD_GSL_VECTOR]; a_tol: REAL_64; a_rank: CE_POINTER [NATURAL_32]; a_c___1: CE_POINTER [PD_GSL_VECTOR]; a_cov: CE_POINTER [PD_GSL_MATRIX]; a_chisq: CE_POINTER [REAL_64]; a_work: CE_POINTER [PD_GSL_MULTIFIT_LINEAR_WORKSPACE]): INTEGER_32
		require
			not a_y.is_void
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.multifit_wlinear_svd (a_w, a_y, a_tol, 0, a_rank, a_c___1, a_cov, a_chisq, a_work)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_multifit_linear_residuals (a_y, a_c___1, a_r___3: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_y.is_void
		local
			l_i: NATURAL_32
			l_yi: REAL_64
			l_tmp: REAL_64
			l_row: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_tmp___0: PD_GSL_VECTOR_CONST_VIEW
			l_y_est: CE_ARRAY [REAL_64]
			l_ri: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_row.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_y_est.make (<<1>>)

			if size1 /= a_y.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of observations in y does not match rows of matrix X"), ce_string ("multilinear.c"), 530, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif size2 /= a_c___1.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of parameters c does not match columns of matrix X"), ce_string ("multilinear.c"), 535, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_y.item.size /= a_r___3.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of observations in y does not match number of residuals"), ce_string ("multilinear.c"), 540, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_y.item.size)
				loop
					l_tmp := a_y.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_yi := l_tmp
					l_tmp___0 := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_const_row (l_i)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_row.item := l_tmp___0.twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_row.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot (a_c___1, l_y_est).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_ri := l_yi - l_y_est.item
					a_r___3.item.gsl_vector_set (l_i, l_ri)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_multifit_covar (a_epsrel: REAL_64; a_covar: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_tolr: REAL_64
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_kmax___0: NATURAL_32
			l_r___3: CE_POINTER [PD_GSL_MATRIX]
			l_tau: CE_POINTER [PD_GSL_VECTOR]
			l_norm___0: CE_POINTER [PD_GSL_VECTOR]
			l_perm: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]
			l_m___4: NATURAL_32
			l_n: NATURAL_32
			l_signum: CE_ARRAY [INTEGER_32]
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_rkk: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_t: REAL_64
			l_tmp___3: REAL_64
			l_rik: REAL_64
			l_tmp___4: REAL_64
			l_rij: REAL_64
			l_tmp___5: REAL_64
			l_rjk: REAL_64
			l_tmp___6: REAL_64
			l_rij___0: REAL_64
			l_tmp___7: REAL_64
			l_rik___0: REAL_64
			l_tmp___8: REAL_64
			l_t___0: REAL_64
			l_tmp___9: REAL_64
			l_rik___1: REAL_64
			l_tmp___10: REAL_64
			l_pj: NATURAL_32
			l_tmp___11: NATURAL_32
			l_pi: NATURAL_32
			l_tmp___12: NATURAL_32
			l_rij___1: REAL_64
			l_rjj: REAL_64
			l_tmp___13: REAL_64
			l_rji: REAL_64
			l_tmp___14: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_32_ARRAY} l_signum.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_kmax___0 := (0).to_natural_32
					l_m___4 := size1
					l_n := size2
					if l_m___4 < l_n then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Jacobian be rectangular M x N with M >= N"), ce_string ("covar.c"), 51, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						if a_covar.item.size1 = a_covar.item.size2 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if a_covar.item.size1 = l_n then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("covariance matrix must be square and match second dimension of jacobian"), ce_string ("covar.c"), 56, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						l_r___3 := (create {P_INIT_SOURCE_STATIC}).gsl_matrix_alloc (l_m___4, l_n)
						update_class_fields -- Current might have been changed; synchronize
						l_tau := (create {P_INIT_SOURCE_STATIC}).gsl_vector_alloc (l_n)
						update_class_fields -- Current might have been changed; synchronize
						l_perm := (create {P_INIT_STATIC}).gsl_permutation_alloc (l_n)
						update_class_fields -- Current might have been changed; synchronize
						l_norm___0 := (create {P_INIT_SOURCE_STATIC}).gsl_vector_alloc (l_n)
						update_class_fields -- Current might have been changed; synchronize
						l_signum.item := 0
						update_class_fields -- Current might have been changed; synchronize
						l_r___3.item.gsl_matrix_memcpy ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_r___3.item.gsl_linalg_QRPT_decomp (l_tau, l_perm, l_signum, l_norm___0).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_tmp := l_r___3.item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := fabs (l_tmp)
						update_class_fields -- Current might have been changed; synchronize
						l_tolr := a_epsrel * l_tmp___0
						l_k := (0).to_natural_32
						from
						until
							not (not (l_return or l_break) and then (l_k < l_n))
						loop
							l_break := False
							l_tmp___1 := l_r___3.item.gsl_matrix_get (l_k, l_k)
							update_class_fields -- Current might have been changed; synchronize
							l_rkk := l_tmp___1
							l_tmp___2 := fabs (l_rkk)
							update_class_fields -- Current might have been changed; synchronize
							if l_tmp___2 <= l_tolr then
								l_break := True
							end
							if not (l_return or l_break) then
								l_r___3.item.gsl_matrix_set (l_k, l_k, 1.0 / l_rkk)
								update_class_fields -- Current might have been changed; synchronize
								l_j := (0).to_natural_32
								from
								until
									not (not (l_return) and then (l_j < l_k))
								loop
									l_tmp___3 := l_r___3.item.gsl_matrix_get (l_j, l_k)
									update_class_fields -- Current might have been changed; synchronize
									l_t := l_tmp___3 / l_rkk
									l_r___3.item.gsl_matrix_set (l_j, l_k, 0.0)
									update_class_fields -- Current might have been changed; synchronize
									l_i := (0).to_natural_32
									from
									until
										not (l_i <= l_j)
									loop
										l_tmp___4 := l_r___3.item.gsl_matrix_get (l_i, l_k)
										update_class_fields -- Current might have been changed; synchronize
										l_rik := l_tmp___4
										l_tmp___5 := l_r___3.item.gsl_matrix_get (l_i, l_j)
										update_class_fields -- Current might have been changed; synchronize
										l_rij := l_tmp___5
										l_r___3.item.gsl_matrix_set (l_i, l_k, l_rik - l_t * l_rij)
										update_class_fields -- Current might have been changed; synchronize
										l_i := l_i + 1
									end
									l_j := l_j + 1
								end
								l_kmax___0 := l_k
								l_k := l_k + 1
							end
						end
						l_break := False
						l_k := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_k <= l_kmax___0))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (not (l_return) and then (l_j < l_k))
							loop
								l_tmp___6 := l_r___3.item.gsl_matrix_get (l_j, l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_rjk := l_tmp___6
								l_i := (0).to_natural_32
								from
								until
									not (l_i <= l_j)
								loop
									l_tmp___7 := l_r___3.item.gsl_matrix_get (l_i, l_j)
									update_class_fields -- Current might have been changed; synchronize
									l_rij___0 := l_tmp___7
									l_tmp___8 := l_r___3.item.gsl_matrix_get (l_i, l_k)
									update_class_fields -- Current might have been changed; synchronize
									l_rik___0 := l_tmp___8
									l_r___3.item.gsl_matrix_set (l_i, l_j, l_rij___0 + l_rjk * l_rik___0)
									update_class_fields -- Current might have been changed; synchronize
									l_i := l_i + 1
								end
								l_j := l_j + 1
							end
							l_tmp___9 := l_r___3.item.gsl_matrix_get (l_k, l_k)
							update_class_fields -- Current might have been changed; synchronize
							l_t___0 := l_tmp___9
							l_i := (0).to_natural_32
							from
							until
								not (l_i <= l_k)
							loop
								l_tmp___10 := l_r___3.item.gsl_matrix_get (l_i, l_k)
								update_class_fields -- Current might have been changed; synchronize
								l_rik___1 := l_tmp___10
								l_r___3.item.gsl_matrix_set (l_i, l_k, l_t___0 * l_rik___1)
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
							l_k := l_k + 1
						end
						l_j := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_j < l_n))
						loop
							l_tmp___11 := l_perm.item.gsl_permutation_get (l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_pj := l_tmp___11
							l_i := (0).to_natural_32
							from
							until
								not (l_i <= l_j)
							loop
								l_tmp___12 := l_perm.item.gsl_permutation_get (l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_pi := l_tmp___12
								if l_j > l_kmax___0 then
									l_r___3.item.gsl_matrix_set (l_i, l_j, 0.0)
									update_class_fields -- Current might have been changed; synchronize
									l_rij___1 := 0.0
								else
									l_rij___1 := l_r___3.item.gsl_matrix_get (l_i, l_j)
									update_class_fields -- Current might have been changed; synchronize
								end
								if l_pi > l_pj then
									l_r___3.item.gsl_matrix_set (l_pi, l_pj, l_rij___1)
									update_class_fields -- Current might have been changed; synchronize
								elseif l_pi < l_pj then
									l_r___3.item.gsl_matrix_set (l_pj, l_pi, l_rij___1)
									update_class_fields -- Current might have been changed; synchronize
								end
								l_i := l_i + 1
							end
							l_tmp___13 := l_r___3.item.gsl_matrix_get (l_j, l_j)
							update_class_fields -- Current might have been changed; synchronize
							l_rjj := l_tmp___13
							a_covar.item.gsl_matrix_set (l_pj, l_pj, l_rjj)
							update_class_fields -- Current might have been changed; synchronize
							l_j := l_j + 1
						end
						l_j := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_j < l_n))
						loop
							l_i := (0).to_natural_32
							from
							until
								not (l_i < l_j)
							loop
								l_tmp___14 := l_r___3.item.gsl_matrix_get (l_j, l_i)
								update_class_fields -- Current might have been changed; synchronize
								l_rji := l_tmp___14
								a_covar.item.gsl_matrix_set (l_j, l_i, l_rji)
								update_class_fields -- Current might have been changed; synchronize
								a_covar.item.gsl_matrix_set (l_i, l_j, l_rji)
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
							l_j := l_j + 1
						end
						(create {P_INIT_SOURCE_STATIC}).gsl_matrix_free (l_r___3)
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_INIT_STATIC}).gsl_permutation_free (l_perm)
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_INIT_SOURCE_STATIC}).gsl_vector_free (l_tau)
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_INIT_SOURCE_STATIC}).gsl_vector_free (l_norm___0)
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	newton_direction (a_qtf, a_p: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_status: INTEGER_32
			l_pi: REAL_64
			l_tmp: REAL_64
		do
			l_N := size2
			l_status := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_R_solve (a_qtf, a_p)
			update_class_fields -- Current might have been changed; synchronize
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_tmp := a_p.item.gsl_vector_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_pi := l_tmp
				a_p.item.gsl_vector_set (l_i, -l_pi)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := l_status
		end

	gradient_direction (a_qtf, a_diag, a_g: CE_POINTER [PD_GSL_VECTOR])
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_sum: REAL_64
			l_dj: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_M))
			loop
				l_sum := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := a_qtf.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_sum := l_sum + (l_tmp * l_tmp___0)
					l_i := l_i + 1
				end
				l_dj := a_diag.item.gsl_vector_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				a_g.item.gsl_vector_set (l_j, -l_sum / l_dj)
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_j + 1
			end
		end

	compute_Rg (a_gradient, a_Rg: CE_POINTER [PD_GSL_VECTOR])
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_sum: REAL_64
			l_gj: REAL_64
			l_tmp: REAL_64
			l_rij: REAL_64
			l_tmp___0: REAL_64
			l_return: BOOLEAN
		do
			l_N := size2
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_sum := (0).to_double
				l_j := l_i
				from
				until
					not (l_j < l_N)
				loop
					l_tmp := a_gradient.item.gsl_vector_get (l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_gj := l_tmp
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_rij := l_tmp___0
					l_sum := l_sum + (l_rij * l_gj)
					l_j := l_j + 1
				end
				a_Rg.item.gsl_vector_set (l_i, l_sum)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	dogleg (a_qtf, a_diag: CE_POINTER [PD_GSL_VECTOR]; a_delta___0: REAL_64; a_newton, a_gradient, a_p: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_newton.is_void
			not a_p.is_void
		local
			l_qnorm: REAL_64
			l_gnorm: REAL_64
			l_sgnorm: REAL_64
			l_bnorm: REAL_64
			l_temp: REAL_64
			l_alpha: REAL_64
			l_beta: REAL_64
			l_alpha___0: REAL_64
			l_beta___0: REAL_64
			l_bg: REAL_64
			l_bq: REAL_64
			l_dq: REAL_64
			l_dq2: REAL_64
			l_sd: REAL_64
			l_sd2: REAL_64
			l_t1: REAL_64
			l_u: REAL_64
			l_t2: REAL_64
			l_tmp: REAL_64
			l_alpha___1: REAL_64
			l_beta___1: REAL_64
			l_return: BOOLEAN
		do
			((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.newton_direction (a_qtf, a_newton).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			l_qnorm := (create {P_DOGLEG_STATIC}).scaled_enorm___0 (a_diag, a_newton)
			update_class_fields -- Current might have been changed; synchronize
			if l_qnorm <= a_delta___0 then
				a_p.item.gsl_vector_memcpy (a_newton).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gradient_direction (a_qtf, a_diag, a_gradient)
				update_class_fields -- Current might have been changed; synchronize
				l_gnorm := a_gradient.item.enorm___1
				update_class_fields -- Current might have been changed; synchronize
				if l_gnorm = (0).to_double then
					l_alpha := a_delta___0 / l_qnorm
					l_beta := (0).to_double
					scaled_addition (l_alpha, a_newton, l_beta, a_gradient, a_p)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				minimum_step (l_gnorm, a_diag, a_gradient)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.compute_Rg (a_gradient, a_p)
				update_class_fields -- Current might have been changed; synchronize
				l_temp := a_p.item.enorm___1
				update_class_fields -- Current might have been changed; synchronize
				l_sgnorm := (l_gnorm / l_temp) / l_temp
				if l_sgnorm > a_delta___0 then
					l_alpha___0 := (0).to_double
					l_beta___0 := a_delta___0
					scaled_addition (l_alpha___0, a_newton, l_beta___0, a_gradient, a_p)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_bnorm := a_qtf.item.enorm___1
				update_class_fields -- Current might have been changed; synchronize
				l_bg := l_bnorm / l_gnorm
				l_bq := l_bnorm / l_qnorm
				l_dq := a_delta___0 / l_qnorm
				l_dq2 := l_dq * l_dq
				l_sd := l_sgnorm / a_delta___0
				l_sd2 := l_sd * l_sd
				l_t1 := (l_bg * l_bq) * l_sd
				l_u := l_t1 - l_dq
				l_tmp := sqrt (l_u * l_u + ((1).to_double - l_dq2) * ((1).to_double - l_sd2))
				update_class_fields -- Current might have been changed; synchronize
				l_t2 := (l_t1 - l_dq * l_sd2) + l_tmp
				l_alpha___1 := (l_dq * ((1).to_double - l_sd2)) / l_t2
				l_beta___1 := ((1).to_double - l_alpha___1) * l_sgnorm
				scaled_addition (l_alpha___1, a_newton, l_beta___1, a_gradient, a_p)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	newton_direction___0 (a_qtf, a_p: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_status: INTEGER_32
			l_pi: REAL_64
			l_tmp: REAL_64
		do
			l_N := size2
			l_status := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_R_solve (a_qtf, a_p)
			update_class_fields -- Current might have been changed; synchronize
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_tmp := a_p.item.gsl_vector_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_pi := l_tmp
				a_p.item.gsl_vector_set (l_i, -l_pi)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := l_status
		end

	gradient_direction___0 (a_qtf, a_diag, a_g: CE_POINTER [PD_GSL_VECTOR])
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_sum: REAL_64
			l_dj: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_M))
			loop
				l_sum := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___0 := a_qtf.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_sum := l_sum + (l_tmp * l_tmp___0)
					l_i := l_i + 1
				end
				l_dj := a_diag.item.gsl_vector_get (l_j)
				update_class_fields -- Current might have been changed; synchronize
				a_g.item.gsl_vector_set (l_j, -l_sum / l_dj)
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_j + 1
			end
		end

	compute_Rg___0 (a_gradient, a_Rg: CE_POINTER [PD_GSL_VECTOR])
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_sum: REAL_64
			l_gj: REAL_64
			l_tmp: REAL_64
			l_rij: REAL_64
			l_tmp___0: REAL_64
			l_return: BOOLEAN
		do
			l_N := size2
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_sum := (0).to_double
				l_j := l_i
				from
				until
					not (l_j < l_N)
				loop
					l_tmp := a_gradient.item.gsl_vector_get (l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_gj := l_tmp
					l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get (l_i, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_rij := l_tmp___0
					l_sum := l_sum + (l_rij * l_gj)
					l_j := l_j + 1
				end
				a_Rg.item.gsl_vector_set (l_i, l_sum)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	dogleg___0 (a_qtf, a_diag: CE_POINTER [PD_GSL_VECTOR]; a_delta___0: REAL_64; a_newton, a_gradient, a_p: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_newton.is_void
			not a_p.is_void
		local
			l_qnorm: REAL_64
			l_gnorm: REAL_64
			l_sgnorm: REAL_64
			l_bnorm: REAL_64
			l_temp: REAL_64
			l_alpha: REAL_64
			l_beta: REAL_64
			l_alpha___0: REAL_64
			l_beta___0: REAL_64
			l_bg: REAL_64
			l_bq: REAL_64
			l_dq: REAL_64
			l_dq2: REAL_64
			l_sd: REAL_64
			l_sd2: REAL_64
			l_t1: REAL_64
			l_u: REAL_64
			l_t2: REAL_64
			l_tmp: REAL_64
			l_alpha___1: REAL_64
			l_beta___1: REAL_64
			l_return: BOOLEAN
		do
			((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.newton_direction___0 (a_qtf, a_newton).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			l_qnorm := (create {P_DOGLEG_STATIC}).scaled_enorm___1 (a_diag, a_newton)
			update_class_fields -- Current might have been changed; synchronize
			if l_qnorm <= a_delta___0 then
				a_p.item.gsl_vector_memcpy (a_newton).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gradient_direction___0 (a_qtf, a_diag, a_gradient)
				update_class_fields -- Current might have been changed; synchronize
				l_gnorm := a_gradient.item.enorm___2
				update_class_fields -- Current might have been changed; synchronize
				if l_gnorm = (0).to_double then
					l_alpha := a_delta___0 / l_qnorm
					l_beta := (0).to_double
					scaled_addition___0 (l_alpha, a_newton, l_beta, a_gradient, a_p)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				minimum_step___0 (l_gnorm, a_diag, a_gradient)
				update_class_fields; -- Current might have been changed; synchronize
				((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.compute_Rg___0 (a_gradient, a_p)
				update_class_fields -- Current might have been changed; synchronize
				l_temp := a_p.item.enorm___2
				update_class_fields -- Current might have been changed; synchronize
				l_sgnorm := (l_gnorm / l_temp) / l_temp
				if l_sgnorm > a_delta___0 then
					l_alpha___0 := (0).to_double
					l_beta___0 := a_delta___0
					scaled_addition___0 (l_alpha___0, a_newton, l_beta___0, a_gradient, a_p)
					update_class_fields -- Current might have been changed; synchronize
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_bnorm := a_qtf.item.enorm___2
				update_class_fields -- Current might have been changed; synchronize
				l_bg := l_bnorm / l_gnorm
				l_bq := l_bnorm / l_qnorm
				l_dq := a_delta___0 / l_qnorm
				l_dq2 := l_dq * l_dq
				l_sd := l_sgnorm / a_delta___0
				l_sd2 := l_sd * l_sd
				l_t1 := (l_bg * l_bq) * l_sd
				l_u := l_t1 - l_dq
				l_tmp := sqrt (l_u * l_u + ((1).to_double - l_dq2) * ((1).to_double - l_sd2))
				update_class_fields -- Current might have been changed; synchronize
				l_t2 := (l_t1 - l_dq * l_sd2) + l_tmp
				l_alpha___1 := (l_dq * ((1).to_double - l_sd2)) / l_t2
				l_beta___1 := ((1).to_double - l_alpha___1) * l_sgnorm
				scaled_addition___0 (l_alpha___1, a_newton, l_beta___1, a_gradient, a_p)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gen_schur_standardize2 (a_B: CE_POINTER [PD_GSL_MATRIX]; a_alpha1, a_alpha2: CE_POINTER [PD_GSL_COMPLEX]; a_beta1, a_beta2: CE_POINTER [REAL_64]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE]): INTEGER_32
		require
			not a_B.is_void
			not a_w.is_void
		local
			l_datB: CE_ARRAY [REAL_64]
			l_datV: CE_ARRAY [REAL_64]
			l_datS: CE_ARRAY [REAL_64]
			l_work: CE_ARRAY [REAL_64]
			l_uv: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp: PD_GSL_MATRIX_VIEW
			l_vv: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_tmp___0: PD_GSL_MATRIX_VIEW
			l_sv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: PD_GSL_VECTOR_VIEW
			l_wv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___2: PD_GSL_VECTOR_VIEW
			l_B11: REAL_64
			l_B22: REAL_64
			l_top: NATURAL_32
			l_det: REAL_64
			l_cr: REAL_64
			l_sr: REAL_64
			l_cl: REAL_64
			l_sl: REAL_64
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_yv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_s: INTEGER_32
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___10: REAL_64
			l_i: NATURAL_32
			l_tmp___11: REAL_64
			l_tmp___12: REAL_64
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_tmp___15: REAL_64
			l_tmp___16: REAL_64
		do
			create {CE_REAL_64_ARRAY} l_datB.make (<<4>>)
			create {CE_REAL_64_ARRAY} l_datV.make (<<4>>)
			create {CE_REAL_64_ARRAY} l_datS.make (<<2>>)
			create {CE_REAL_64_ARRAY} l_work.make (<<2>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_uv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_MATRIX_VIEW]} l_vv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_sv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_wv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_xv.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_yv.make (<<1>>)

			l_tmp := ((create {P_VIEW_SOURCE_STATIC}).gsl_matrix_view_array (l_datB, (2).to_natural_32, (2).to_natural_32)).twin
			update_class_fields -- Current might have been changed; synchronize
			l_uv.item := l_tmp.twin
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___0 := ((create {P_VIEW_SOURCE_STATIC}).gsl_matrix_view_array (l_datV, (2).to_natural_32, (2).to_natural_32)).twin
			update_class_fields -- Current might have been changed; synchronize
			l_vv.item := l_tmp___0.twin
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___1 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_datS, (2).to_natural_32)).twin
			update_class_fields -- Current might have been changed; synchronize
			l_sv.item := l_tmp___1.twin
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___2 := ((create {P_VIEW_SOURCE_STATIC}).gsl_vector_view_array (l_work, (2).to_natural_32)).twin
			update_class_fields -- Current might have been changed; synchronize
			l_wv.item := l_tmp___2.twin
			update_class_fields -- Current might have been changed; synchronize
			if (a_w.item.needtop).to_boolean then
				l_top := (a_w.item.H).item.gen_get_submatrix ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer)))
				update_class_fields -- Current might have been changed; synchronize
			end
			(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_uv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_memcpy (a_B).do_nothing
			update_class_fields; -- Current might have been changed; synchronize
			(create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_uv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_linalg_SV_decomp (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix")), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_sv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___3 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___4 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___5 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___6 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_det := l_tmp___3 * l_tmp___4 - l_tmp___5 * l_tmp___6
			if l_det < 0.0 then
				l_datS[1] := -l_datS[1]
				update_class_fields -- Current might have been changed; synchronize
			end
			l_cr := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_sr := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_vv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___7 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_uv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___8 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_uv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___9 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_uv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___10 := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_uv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_det := l_tmp___7 * l_tmp___8 - l_tmp___9 * l_tmp___10
			if l_det < 0.0 then
				l_datS[1] := -l_datS[1]
				update_class_fields -- Current might have been changed; synchronize
			end
			l_cl := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_uv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((0).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_sl := (create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (l_uv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))).item.gsl_matrix_get ((1).to_natural_32, (0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_B11 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_sv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			l_B22 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_sv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_get ((1).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			if l_B11 < 0.0 then
				l_B11 := -l_B11
				l_B22 := -l_B22
				l_cr := -l_cr
				l_sr := -l_sr
			end
			if (a_w.item.compute_s).to_boolean then
				l_xv.item := ((a_w.item.H).item.gsl_matrix_subrow (l_top, l_top, a_w.item.size - l_top)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.H).item.gsl_matrix_subrow (l_top + {NATURAL_32} 1, l_top, a_w.item.size - l_top)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cl, l_sl).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_xv.item := ((a_w.item.H).item.gsl_matrix_subcolumn (l_top, (0).to_natural_32, l_top + {NATURAL_32} 2)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.H).item.gsl_matrix_subcolumn (l_top + {NATURAL_32} 1, (0).to_natural_32, l_top + {NATURAL_32} 2)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cr, l_sr).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			else
				l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row ((0).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_row ((1).to_natural_32)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cl, l_sl).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_xv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column ((0).to_natural_32)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := (((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_column ((1).to_natural_32)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cr, l_sr).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			if (a_w.item.compute_t).to_boolean then
				if l_top /= a_w.item.size - {NATURAL_32} 2 then
					l_xv.item := ((a_w.item.R).item.gsl_matrix_subrow (l_top, l_top + {NATURAL_32} 2, (a_w.item.size - l_top) - {NATURAL_32} 2)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.R).item.gsl_matrix_subrow (l_top + {NATURAL_32} 1, l_top + {NATURAL_32} 2, (a_w.item.size - l_top) - {NATURAL_32} 2)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cl, l_sl).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
				if l_top /= {NATURAL_32} 0 then
					l_xv.item := ((a_w.item.R).item.gsl_matrix_subcolumn (l_top, (0).to_natural_32, l_top)).twin
					update_class_fields -- Current might have been changed; synchronize
					l_yv.item := ((a_w.item.R).item.gsl_matrix_subcolumn (l_top + {NATURAL_32} 1, (0).to_natural_32, l_top)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cr, l_sr).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
			end
			if (not (a_w.item.Q).is_void) then
				l_xv.item := ((a_w.item.Q).item.gsl_matrix_column (l_top)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.Q).item.gsl_matrix_column (l_top + {NATURAL_32} 1)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cl, l_sl).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			if (not (a_w.item.Z).is_void) then
				l_xv.item := ((a_w.item.Z).item.gsl_matrix_column (l_top)).twin
				update_class_fields -- Current might have been changed; synchronize
				l_yv.item := ((a_w.item.Z).item.gsl_matrix_column (l_top + {NATURAL_32} 1)).twin
				update_class_fields; -- Current might have been changed; synchronize
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_drot (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_yv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_cr, l_sr).do_nothing
				update_class_fields -- Current might have been changed; synchronize
			end
			a_B.item.gsl_matrix_set ((0).to_natural_32, (0).to_natural_32, l_B11)
			update_class_fields -- Current might have been changed; synchronize
			a_B.item.gsl_matrix_set ((0).to_natural_32, (1).to_natural_32, 0.0)
			update_class_fields -- Current might have been changed; synchronize
			a_B.item.gsl_matrix_set ((1).to_natural_32, (0).to_natural_32, 0.0)
			update_class_fields -- Current might have been changed; synchronize
			a_B.item.gsl_matrix_set ((1).to_natural_32, (1).to_natural_32, l_B22)
			update_class_fields -- Current might have been changed; synchronize
			if l_B22 < 0.0 then
				if (a_w.item.compute_s).to_boolean then
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_top + {NATURAL_32} 2)
					loop
						l_tmp___11 := (a_w.item.H).item.gsl_matrix_get (l_i, l_top + {NATURAL_32} 1)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.H).item.gsl_matrix_set (l_i, l_top + {NATURAL_32} 1, -l_tmp___11)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				else
					l_tmp___12 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((0).to_natural_32, (1).to_natural_32, -l_tmp___12)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___13 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
					update_class_fields; -- Current might have been changed; synchronize
					((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_set ((1).to_natural_32, (1).to_natural_32, -l_tmp___13)
					update_class_fields -- Current might have been changed; synchronize
				end
				if (a_w.item.compute_t).to_boolean then
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_top + {NATURAL_32} 2)
					loop
						l_tmp___14 := (a_w.item.R).item.gsl_matrix_get (l_i, l_top + {NATURAL_32} 1)
						update_class_fields; -- Current might have been changed; synchronize
						(a_w.item.R).item.gsl_matrix_set (l_i, l_top + {NATURAL_32} 1, -l_tmp___14)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				else
					l_tmp___15 := a_B.item.gsl_matrix_get ((0).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					a_B.item.gsl_matrix_set ((0).to_natural_32, (1).to_natural_32, -l_tmp___15)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___16 := a_B.item.gsl_matrix_get ((1).to_natural_32, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					a_B.item.gsl_matrix_set ((1).to_natural_32, (1).to_natural_32, -l_tmp___16)
					update_class_fields -- Current might have been changed; synchronize
				end
				if (not (a_w.item.Z).is_void) then
					l_xv.item := ((a_w.item.Z).item.gsl_matrix_column (l_top + {NATURAL_32} 1)).twin
					update_class_fields; -- Current might have been changed; synchronize
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_vector_scale (-1.0).do_nothing
					update_class_fields -- Current might have been changed; synchronize
				end
			end
			l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gen_compute_eigenvals (a_B, a_alpha1, a_alpha2, a_beta1, a_beta2)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_s
		end

	sup_norm: REAL_64
		local
			l_min: CE_ARRAY [REAL_64]
			l_max: CE_ARRAY [REAL_64]
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
		do
			create {CE_REAL_64_ARRAY} l_min.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_max.make (<<1>>);

			((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_matrix_minmax (l_min, l_max)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp := fabs (l_max.item)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___0 := fabs (l_min.item)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp___1 := (create {P_GSL_GSL_MINMAX_STATIC}).GSL_MAX_DBL (l_tmp___0, l_tmp)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp___1
		end

	obtain_suggestion (a_mode: NATURAL_32): PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION
		local
			l_mode_prec: NATURAL_32
			l_tmp: NATURAL_32
			l_norm_A: REAL_64
			l_tmp___0: REAL_64
			l_extra: REAL_64
			l_tmp___1: REAL_64
			l_extra_i: INTEGER_32
			l_tmp___2: REAL_64
			l_s: PD_MOLER_VANLOAN_OPTIMAL_SUGGESTION
		do
			l_tmp := (create {P_GSL_GSL_MODE_STATIC}).GSL_MODE_PREC (a_mode)
			update_class_fields -- Current might have been changed; synchronize
			l_mode_prec := l_tmp
			l_tmp___0 := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.sup_norm
			update_class_fields -- Current might have been changed; synchronize
			l_norm_A := l_tmp___0
			if l_norm_A < 0.01 then
				Result := ((mvl_tab.item[l_mode_prec.to_integer_32])[0]).twin
			elseif l_norm_A < 0.1 then
				Result := ((mvl_tab.item[l_mode_prec.to_integer_32])[1]).twin
			elseif l_norm_A < 1.0 then
				Result := ((mvl_tab.item[l_mode_prec.to_integer_32])[2]).twin
			elseif l_norm_A < 10.0 then
				Result := ((mvl_tab.item[l_mode_prec.to_integer_32])[3]).twin
			elseif l_norm_A < 100.0 then
				Result := ((mvl_tab.item[l_mode_prec.to_integer_32])[4]).twin
			elseif l_norm_A < 1000.0 then
				Result := ((mvl_tab.item[l_mode_prec.to_integer_32])[5]).twin
			else
				l_tmp___1 := log ((1.01 * l_norm_A) / 1000.0)
				update_class_fields -- Current might have been changed; synchronize
				l_extra := l_tmp___1 / 0.69314718055994530942
				l_tmp___2 := ceil (l_extra)
				update_class_fields -- Current might have been changed; synchronize
				l_extra_i := (l_tmp___2.truncated_to_integer_64.to_natural_32).to_integer_32
				l_s := ((mvl_tab.item[a_mode.to_integer_32])[5]).twin
				l_s.j := l_s.j + l_extra_i
				update_class_fields -- Current might have been changed; synchronize
				Result := l_s.twin
			end
		end

	gsl_linalg_hessenberg_unpack (a_tau: CE_POINTER [PD_GSL_VECTOR]; a_U: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_U.is_void
		local
			l_s: INTEGER_32
		do
			a_U.item.gsl_matrix_set_identity
			update_class_fields -- Current might have been changed; synchronize
			l_s := ((create {CE_CLASS_POINTER [PD_GSL_MATRIX]}.make_shared (c_pointer))).item.gsl_linalg_hessenberg_unpack_accum (a_tau, a_U)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_s
		end

feature {PD_GSL_MATRIX} -- private routines

	gsl_linalg_householder_hm (a_tau: REAL_64; a_v: CE_POINTER [PD_GSL_VECTOR]; a_A___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wj: REAL_64
			l_tmp___0: REAL_64
			l_A0j: REAL_64
			l_Aij: REAL_64
			l_vi: REAL_64
			l_return: BOOLEAN
		do
			if a_tau = 0.0 then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < a_A___0.item.size2))
				loop
					l_wj := (a_A___0.item.gsl_matrix_get ((0).to_natural_32, l_j))
					l_i := (1).to_natural_32
					from
					until
						not (l_i < a_A___0.item.size1)
					loop
						l_tmp___0 := a_A___0.item.gsl_matrix_get (l_i, l_j)
						l_wj := l_wj + (l_tmp___0 * (a_v.item.gsl_vector_get (l_i)))
						l_i := l_i + 1
					end
					l_A0j := (a_A___0.item.gsl_matrix_get ((0).to_natural_32, l_j))
					a_A___0.item.gsl_matrix_set ((0).to_natural_32, l_j, l_A0j - a_tau * l_wj)
					l_i := (1).to_natural_32
					from
					until
						not (l_i < a_A___0.item.size1)
					loop
						l_Aij := (a_A___0.item.gsl_matrix_get (l_i, l_j))
						l_vi := (a_v.item.gsl_vector_get (l_i))
						a_A___0.item.gsl_matrix_set (l_i, l_j, l_Aij - (a_tau * l_vi) * l_wj)
						l_i := l_i + 1
					end
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_linalg_householder_transform (a_v: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		require
			not a_v.is_void
		local
			l_n: NATURAL_32
			l_alpha: REAL_64
			l_beta: REAL_64
			l_tau: REAL_64
			l_x: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_xnorm: REAL_64
			l_tmp___1: REAL_64
			l_s: REAL_64
			l_return: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_x.make (<<1>>)

			l_n := a_v.item.size
			if l_n = {NATURAL_32} 1 then
				Result := (0.0)
			else
				l_x.item := ((a_v.item.gsl_vector_subvector ((1).to_natural_32, l_n - {NATURAL_32} 1)).twin).twin
				l_xnorm := ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2)
				if l_xnorm = (0).to_double then
					Result := (0.0)
					l_return := True
				end
				if not l_return then
					l_alpha := a_v.item.gsl_vector_get ((0).to_natural_32)
					if l_alpha >= 0.0 then
						l_tmp___1 := 1.0
					else
						l_tmp___1 := -1.0
					end
					l_beta := -l_tmp___1 * (hypot (l_alpha, l_xnorm))
					l_tau := (l_beta - l_alpha) / l_beta
					l_s := l_alpha - l_beta
					if (fabs (l_s)) > 2.2250738585072014e-308 then
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (1.0 / l_s)
						a_v.item.gsl_vector_set ((0).to_natural_32, l_beta)
					else
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (2.2204460492503131e-16 / l_s);
						(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (1.0 / 2.2204460492503131e-16)
						a_v.item.gsl_vector_set ((0).to_natural_32, l_beta)
					end
					Result := l_tau
				end
			end
		end

	gsl_linalg_householder_mh (a_tau: REAL_64; a_v: CE_POINTER [PD_GSL_VECTOR]; a_A___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wi: REAL_64
			l_tmp___0: REAL_64
			l_Ai0: REAL_64
			l_vj: REAL_64
			l_Aij: REAL_64
			l_return: BOOLEAN
		do
			if a_tau = (0).to_double then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_A___0.item.size1))
				loop
					l_wi := (a_A___0.item.gsl_matrix_get (l_i, (0).to_natural_32))
					l_j := (1).to_natural_32
					from
					until
						not (l_j < a_A___0.item.size2)
					loop
						l_tmp___0 := a_A___0.item.gsl_matrix_get (l_i, l_j)
						l_wi := l_wi + (l_tmp___0 * (a_v.item.gsl_vector_get (l_j)))
						l_j := l_j + 1
					end
					l_Ai0 := (a_A___0.item.gsl_matrix_get (l_i, (0).to_natural_32))
					a_A___0.item.gsl_matrix_set (l_i, (0).to_natural_32, l_Ai0 - a_tau * l_wi)
					l_j := (1).to_natural_32
					from
					until
						not (l_j < a_A___0.item.size2)
					loop
						l_vj := (a_v.item.gsl_vector_get (l_j))
						l_Aij := (a_A___0.item.gsl_matrix_get (l_i, l_j))
						a_A___0.item.gsl_matrix_set (l_i, l_j, l_Aij - (a_tau * l_wi) * l_vj)
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_permute_vector_inverse (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_p.is_void
			not a_v.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if a_v.item.size /= a_p.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector and permutation must be the same length"), ce_string ("permute_source.c"), 157, 19)
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				gsl_permute_inverse (a_p.item.data, a_v.item.data, a_v.item.stride, a_v.item.size).do_nothing
				Result := 0
			end
		end

	apply_givens_vec (a_v: CE_POINTER [PD_GSL_VECTOR]; a_i, a_j: NATURAL_32; a_c___1, a_s: REAL_64)
		require
			not a_v.is_void
		local
			l_vi: REAL_64
			l_vj: REAL_64
		do
			l_vi := (a_v.item.gsl_vector_get (a_i))
			l_vj := (a_v.item.gsl_vector_get (a_j))
			a_v.item.gsl_vector_set (a_i, a_c___1 * l_vi - a_s * l_vj)
			a_v.item.gsl_vector_set (a_j, a_s * l_vi + a_c___1 * l_vj)
		end

	gsl_linalg_householder_hv (a_tau: REAL_64; a_v, a_w: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_v.is_void
		local
			l_N: NATURAL_32
			l_d0: REAL_64
			l_d1: CE_ARRAY [REAL_64]
			l_d: REAL_64
			l_v1: CE_ARRAY [PD_GSL_VECTOR_CONST_VIEW]
			l_w1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_w0: REAL_64
			l_return: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_d1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_CONST_VIEW]} l_v1.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_w1.make (<<1>>)

			l_N := a_v.item.size
			if a_tau = (0).to_double then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_d0 := (a_w.item.gsl_vector_get ((0).to_natural_32))
				l_v1.item := ((gsl_vector_const_subvector (a_v, (1).to_natural_32, l_N - {NATURAL_32} 1)).twin).twin
				l_w1.item := ((a_w.item.gsl_vector_subvector ((1).to_natural_32, l_N - {NATURAL_32} 1)).twin).twin;
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))).item.gsl_blas_ddot ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_w1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_d1).do_nothing
				l_d := l_d0 + l_d1.item
				l_w0 := (a_w.item.gsl_vector_get ((0).to_natural_32))
				a_w.item.gsl_vector_set ((0).to_natural_32, l_w0 - a_tau * l_d);
				(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.item.c_pointer + (create {PD_GSL_VECTOR_CONST_VIEW}.make).offset_of ("vector"))).item.gsl_blas_daxpy (-a_tau * l_d, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_w1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
				Result := 0
			end
		end

	gsl_vector_const_subvector (a_v: CE_POINTER [PD_GSL_VECTOR]; a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_CONST_VIEW
			l_s: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.vector.size := (0).to_natural_32
			l_view.vector.stride := (0).to_natural_32
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			l_view.vector.owner := 0
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("subvector_source.c"), 28, 4)
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) >= a_v.item.size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("view would extend past end of vector"), ce_string ("subvector_source.c"), 34, 4)
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size := (0).to_natural_32
				l_s.stride := (0).to_natural_32
				l_s.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				l_s.owner := 0
				l_s.data := (a_v.item.data + (a_v.item.stride * a_offset).to_integer_32)
				l_s.size := a_n
				l_s.stride := a_v.item.stride
				l_s.block := a_v.item.block
				l_s.owner := 0
				l_view.vector := l_s.twin
				Result := l_view.twin
			end
		end

	gsl_blas_dsyr2 (a_Uplo: NATURAL_32; a_alpha: REAL_64; a_X, a_Y: CE_POINTER [PD_GSL_VECTOR]; a_A___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		require
			not a_A___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_A___0.item.size1
					l_N := a_A___0.item.size2
					if l_M = l_N then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square"), ce_string ("blas.c"), 1_285, 20)
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 3 then -- else:
						if a_X.item.size = l_N then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 6 then -- else:
						if a_Y.item.size = l_N then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_289, 19)
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						cblas_dsyr2 ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, l_N.to_integer_32, a_alpha, a_X.item.data, (a_X.item.stride).to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32, a_A___0.item.data, (a_A___0.item.tda).to_integer_32)
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	apply_givens_qr (a_M, a_N: NATURAL_32; a_Q___1, a_R: CE_POINTER [PD_GSL_MATRIX]; a_i, a_j: NATURAL_32; a_c___1, a_s: REAL_64)
		local
			l_k: NATURAL_32
			l_qki: REAL_64
			l_qkj: REAL_64
			l_rik: REAL_64
			l_rjk: REAL_64
		do
			l_k := (0).to_natural_32
			from
			until
				not (l_k < a_M)
			loop
				l_qki := (a_Q___1.item.gsl_matrix_get (l_k, a_i))
				l_qkj := (a_Q___1.item.gsl_matrix_get (l_k, a_j))
				a_Q___1.item.gsl_matrix_set (l_k, a_i, l_qki * a_c___1 - l_qkj * a_s)
				a_Q___1.item.gsl_matrix_set (l_k, a_j, l_qki * a_s + l_qkj * a_c___1)
				l_k := l_k + 1
			end
			if a_i < a_j then
				l_k := a_i
			else
				l_k := a_j
			end
			from
			until
				not (l_k < a_N)
			loop
				l_rik := (a_R.item.gsl_matrix_get (a_i, l_k))
				l_rjk := (a_R.item.gsl_matrix_get (a_j, l_k))
				a_R.item.gsl_matrix_set (a_i, l_k, a_c___1 * l_rik - a_s * l_rjk)
				a_R.item.gsl_matrix_set (a_j, l_k, a_s * l_rik + a_c___1 * l_rjk)
				l_k := l_k + 1
			end
		end

	apply_givens_lq (a_M, a_N: NATURAL_32; a_Q___1, a_L: CE_POINTER [PD_GSL_MATRIX]; a_i, a_j: NATURAL_32; a_c___1, a_s: REAL_64)
		local
			l_k: NATURAL_32
			l_qik: REAL_64
			l_qjk: REAL_64
			l_lki: REAL_64
			l_lkj: REAL_64
		do
			l_k := (0).to_natural_32
			from
			until
				not (l_k < a_M)
			loop
				l_qik := (a_Q___1.item.gsl_matrix_get (a_i, l_k))
				l_qjk := (a_Q___1.item.gsl_matrix_get (a_j, l_k))
				a_Q___1.item.gsl_matrix_set (a_i, l_k, l_qik * a_c___1 - l_qjk * a_s)
				a_Q___1.item.gsl_matrix_set (a_j, l_k, l_qik * a_s + l_qjk * a_c___1)
				l_k := l_k + 1
			end
			if a_i < a_j then
				l_k := a_i
			else
				l_k := a_j
			end
			from
			until
				not (l_k < a_N)
			loop
				l_lki := (a_L.item.gsl_matrix_get (l_k, a_i))
				l_lkj := (a_L.item.gsl_matrix_get (l_k, a_j))
				a_L.item.gsl_matrix_set (l_k, a_i, a_c___1 * l_lki - a_s * l_lkj)
				a_L.item.gsl_matrix_set (l_k, a_j, a_s * l_lki + a_c___1 * l_lkj)
				l_k := l_k + 1
			end
		end

	gsl_vector_div (a_a___5, a_b___0: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_a___5.is_void
			not a_b___0.is_void
		local
			l_N: NATURAL_32
			l_stride_a: NATURAL_32
			l_stride_b: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_N := a_a___5.item.size
			if a_b___0.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vectors must have same length"), ce_string ("oper_source.c"), 102, 19)
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_stride_a := a_a___5.item.stride
				l_stride_b := a_b___0.item.stride
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					a_a___5.item.data[(l_i * l_stride_a).to_integer_32] := a_a___5.item.data[(l_i * l_stride_a).to_integer_32] / (a_b___0.item.data[(l_i * l_stride_b).to_integer_32])
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_dasum (a_X: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		require
			not a_X.is_void
		do
			Result := (cblas_dasum ((a_X.item.size).to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32))
		end

	bspline_pppack_bsplvd (a_t: CE_POINTER [PD_GSL_VECTOR]; a_k: NATURAL_32; a_x: REAL_64; a_left: NATURAL_32; a_deltal, a_deltar: CE_POINTER [PD_GSL_VECTOR]; a_a___5, a_dbiatx: CE_POINTER [PD_GSL_MATRIX]; a_nderiv: NATURAL_32)
		require
			not a_dbiatx.is_void
		local
			l_i: INTEGER_32
			l_ideriv: INTEGER_32
			l_il: INTEGER_32
			l_j: INTEGER_32
			l_jlow: INTEGER_32
			l_jp1mid: INTEGER_32
			l_kmm: INTEGER_32
			l_ldummy: INTEGER_32
			l_m___4: INTEGER_32
			l_mhigh: INTEGER_32
			l_factor: REAL_64
			l_fkmm: REAL_64
			l_sum: REAL_64
			l_bsplvb_j: CE_ARRAY [NATURAL_32]
			l_dbcol: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_tmp___1: REAL_64
			l_tmp___3: REAL_64
			l_tmp___5: REAL_64
			l_return: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_bsplvb_j.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_dbcol.make (<<1>>)

			l_dbcol.item := ((a_dbiatx.item.gsl_matrix_column ((0).to_natural_32)).twin).twin
			l_mhigh := GSL_MIN_INT (a_nderiv.to_integer_32, (a_k - {NATURAL_32} 1).to_integer_32);
			(create {P_BSPLINE_STATIC}).bspline_pppack_bsplvb (a_t, a_k - l_mhigh.to_natural_32, (1).to_natural_32, a_x, a_left, l_bsplvb_j, a_deltal, a_deltar, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_dbcol.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
			if l_mhigh > 0 then
				l_ideriv := l_mhigh
				l_m___4 := 1
				from
				until
					not (not (l_return) and then (l_m___4 <= l_mhigh))
				loop
					l_j := l_ideriv
					l_jp1mid := 0
					from
					until
						not (l_j < a_k.to_integer_32)
					loop
						a_dbiatx.item.gsl_matrix_set (l_j.to_natural_32, l_ideriv.to_natural_32, (a_dbiatx.item.gsl_matrix_get (l_jp1mid.to_natural_32, (0).to_natural_32)))
						l_j := l_j + 1
						l_jp1mid := l_jp1mid + 1
					end
					l_ideriv := l_ideriv - 1;
					(create {P_BSPLINE_STATIC}).bspline_pppack_bsplvb (a_t, a_k - l_ideriv.to_natural_32, (2).to_natural_32, a_x, a_left, l_bsplvb_j, a_deltal, a_deltar, create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_dbcol.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
					l_m___4 := l_m___4 + 1
				end
				l_jlow := 0
				l_i := 0
				from
				until
					not (not (l_return) and then (l_i < a_k.to_integer_32))
				loop
					l_j := l_jlow
					from
					until
						not (l_j < a_k.to_integer_32)
					loop
						a_a___5.item.gsl_matrix_set (l_j.to_natural_32, l_i.to_natural_32, 0.0)
						l_j := l_j + 1
					end
					l_jlow := l_i
					a_a___5.item.gsl_matrix_set (l_i.to_natural_32, l_i.to_natural_32, 1.0)
					l_i := l_i + 1
				end
				l_m___4 := 1
				from
				until
					not (not (l_return) and then (l_m___4 <= l_mhigh))
				loop
					l_kmm := (a_k - l_m___4.to_natural_32).to_integer_32
					l_fkmm := (l_kmm.to_real).to_double
					l_il := a_left.to_integer_32
					l_i := (a_k - {NATURAL_32} 1).to_integer_32
					l_ldummy := 0
					from
					until
						not (not (l_return) and then (l_ldummy < l_kmm))
					loop
						l_tmp___1 := a_t.item.gsl_vector_get ((l_il + l_kmm).to_natural_32)
						l_factor := l_fkmm / (l_tmp___1 - (a_t.item.gsl_vector_get (l_il.to_natural_32)))
						l_j := 0
						from
						until
							not (l_j <= l_i)
						loop
							l_tmp___3 := a_a___5.item.gsl_matrix_get (l_i.to_natural_32, l_j.to_natural_32)
							a_a___5.item.gsl_matrix_set (l_i.to_natural_32, l_j.to_natural_32, l_factor * (l_tmp___3 - (a_a___5.item.gsl_matrix_get ((l_i - 1).to_natural_32, l_j.to_natural_32))))
							l_j := l_j + 1
						end
						l_il := l_il - 1
						l_i := l_i - 1
						l_ldummy := l_ldummy + 1
					end
					l_i := 0
					from
					until
						not (not (l_return) and then (l_i < a_k.to_integer_32))
					loop
						l_sum := (0).to_double
						l_jlow := GSL_MAX_INT (l_i, l_m___4)
						l_j := l_jlow
						from
						until
							not (l_j < a_k.to_integer_32)
						loop
							l_tmp___5 := a_a___5.item.gsl_matrix_get (l_j.to_natural_32, l_i.to_natural_32)
							l_sum := l_sum + (l_tmp___5 * (a_dbiatx.item.gsl_matrix_get (l_j.to_natural_32, l_m___4.to_natural_32)))
							l_j := l_j + 1
						end
						a_dbiatx.item.gsl_matrix_set (l_i.to_natural_32, l_m___4.to_natural_32, l_sum)
						l_i := l_i + 1
					end
					l_m___4 := l_m___4 + 1
				end
			end
		end

	GSL_MIN_INT (a_a___5, a_b___0: INTEGER_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			if a_a___5 < a_b___0 then
				l_tmp := a_a___5
			else
				l_tmp := a_b___0
			end
			Result := l_tmp
		end

	GSL_MAX_INT (a_a___5, a_b___0: INTEGER_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			if a_a___5 > a_b___0 then
				l_tmp := a_a___5
			else
				l_tmp := a_b___0
			end
			Result := l_tmp
		end

	nonsymmv_normalize_eigenvectors (a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX])
		require
			not a_evec.is_void
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_ei: PD_GSL_COMPLEX
			l_vi: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_re: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_im: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_scale: REAL_64
			l_tmp___0: REAL_64
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]} l_vi.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_re.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_im.make (<<1>>)

			l_N := a_evec.item.size1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_ei := (a_eval.item.gsl_vector_complex_get (l_i)).twin
				l_vi.item := (a_evec.item.gsl_matrix_complex_column (l_i)).twin
				l_re.item := (gsl_vector_complex_real (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
				if l_ei.dat[1] = 0.0 then
					l_scale := 1.0 / ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2);
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale)
				elseif l_ei.dat[1] > 0.0 then
					l_im.item := (gsl_vector_complex_imag (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					l_tmp___0 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
					l_scale := 1.0 / ((create {P_HYPOT_STATIC}).gsl_hypot (((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2), l_tmp___0));
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_blas_zdscal (l_scale)
					l_vi.item := (a_evec.item.gsl_matrix_complex_column (l_i + {NATURAL_32} 1)).twin;
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_blas_zdscal (l_scale)
				end
				l_i := l_i + 1
			end
		end

	chop_small_elements___0 (a_N: NATURAL_32; a_d, a_sd: CE_POINTER [REAL_64])
		require
			not a_d.is_void
		local
			l_d_i: REAL_64
			l_i: NATURAL_32
			l_sd_i: REAL_64
			l_d_ip1: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
		do
			l_d_i := a_d[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_N - {NATURAL_32} 1)
			loop
				l_sd_i := a_sd[l_i.to_integer_32]
				l_d_ip1 := a_d[(l_i + {NATURAL_32} 1).to_integer_32]
				l_tmp := fabs (l_sd_i)
				l_tmp___0 := fabs (l_d_i)
				if l_tmp < 2.2204460492503131e-16 * (l_tmp___0 + (fabs (l_d_ip1))) then
					a_sd[l_i.to_integer_32] := 0.0
				end
				l_d_i := l_d_ip1
				l_i := l_i + 1
			end
		end

	qrstep___0 (a_n: NATURAL_32; a_d, a_sd, a_gc, a_gs: CE_POINTER [REAL_64])
		require
			not a_d.is_void
			not a_sd.is_void
		local
			l_x: REAL_64
			l_z: REAL_64
			l_ak: REAL_64
			l_bk: REAL_64
			l_zk: REAL_64
			l_ap: REAL_64
			l_bp: REAL_64
			l_aq: REAL_64
			l_bq: REAL_64
			l_k: NATURAL_32
			l_mu: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_ap1: REAL_64
			l_bp1: REAL_64
			l_aq1: REAL_64
			l_c___2: CE_ARRAY [REAL_64]
			l_s___0: CE_ARRAY [REAL_64]
			l_bk1: REAL_64
			l_ap1___0: REAL_64
			l_bp1___0: REAL_64
			l_zp1: REAL_64
			l_aq1___0: REAL_64
			l_bq1: REAL_64
			l_return: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_c___2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s___0.make (<<1>>)

			l_mu := ((create {P_QRSTEP_STATIC}).trailing_eigenvalue (a_n, a_d, a_sd))
			l_tmp___0 := fabs (l_mu)
			l_tmp___1 := fabs (a_d[0])
			if 2.2204460492503131e-16 * l_tmp___0 > l_tmp___1 + (fabs (a_sd[0])) then
				l_mu := (0).to_double
			end
			l_x := a_d[0] - l_mu
			l_z := a_sd[0]
			l_ak := (0).to_double
			l_bk := (0).to_double
			l_zk := (0).to_double
			l_ap := a_d[0]
			l_bp := a_sd[0]
			l_aq := a_d[1]
			if a_n = {NATURAL_32} 2 then
				(create {P_QRSTEP_STATIC}).create_givens (l_x, l_z, l_c___1, l_s)
				if not a_gc.is_void then
					a_gc[0] := l_c___1.item
				end
				if not a_gs.is_void then
					a_gs[0] := l_s.item
				end
				l_ap1 := l_c___1.item * (l_c___1.item * l_ap - l_s.item * l_bp) + l_s.item * (l_s.item * l_aq - l_c___1.item * l_bp)
				l_bp1 := l_c___1.item * (l_s.item * l_ap + l_c___1.item * l_bp) - l_s.item * (l_s.item * l_bp + l_c___1.item * l_aq)
				l_aq1 := l_s.item * (l_s.item * l_ap + l_c___1.item * l_bp) + l_c___1.item * (l_s.item * l_bp + l_c___1.item * l_aq)
				l_ak := l_ap1
				l_bk := l_bp1
				l_ap := l_aq1
				a_d[0] := l_ak
				a_sd[0] := l_bk
				a_d[1] := l_ap
				l_return := True
			end
			if not l_return then
				l_bq := a_sd[1]
				l_k := (0).to_natural_32
				from
				until
					not (l_k < a_n - {NATURAL_32} 1)
				loop
					(create {P_QRSTEP_STATIC}).create_givens (l_x, l_z, l_c___2, l_s___0)
					if not a_gc.is_void then
						a_gc[l_k.to_integer_32] := l_c___2.item
					end
					if not a_gs.is_void then
						a_gs[l_k.to_integer_32] := l_s___0.item
					end
					l_bk1 := l_c___2.item * l_bk - l_s___0.item * l_zk
					l_ap1___0 := l_c___2.item * (l_c___2.item * l_ap - l_s___0.item * l_bp) + l_s___0.item * (l_s___0.item * l_aq - l_c___2.item * l_bp)
					l_bp1___0 := l_c___2.item * (l_s___0.item * l_ap + l_c___2.item * l_bp) - l_s___0.item * (l_s___0.item * l_bp + l_c___2.item * l_aq)
					l_zp1 := -l_s___0.item * l_bq
					l_aq1___0 := l_s___0.item * (l_s___0.item * l_ap + l_c___2.item * l_bp) + l_c___2.item * (l_s___0.item * l_bp + l_c___2.item * l_aq)
					l_bq1 := l_c___2.item * l_bq
					l_ak := l_ap1___0
					l_bk := l_bp1___0
					l_zk := l_zp1
					l_ap := l_aq1___0
					l_bp := l_bq1
					if l_k < a_n - {NATURAL_32} 2 then
						l_aq := a_d[(l_k + {NATURAL_32} 2).to_integer_32]
					end
					if l_k < a_n - {NATURAL_32} 3 then
						l_bq := a_sd[(l_k + {NATURAL_32} 2).to_integer_32]
					end
					a_d[l_k.to_integer_32] := l_ak
					if l_k > {NATURAL_32} 0 then
						a_sd[(l_k - {NATURAL_32} 1).to_integer_32] := l_bk1
					end
					if l_k < a_n - {NATURAL_32} 2 then
						a_sd[(l_k + {NATURAL_32} 1).to_integer_32] := l_bp
					end
					l_x := l_bk
					l_z := l_zk
					l_k := l_k + 1
				end
				a_d[l_k.to_integer_32] := l_ap
				a_sd[(l_k - {NATURAL_32} 1).to_integer_32] := l_bk
			end
		end

	genv_normalize_eigenvectors (a_alpha: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX])
		require
			not a_evec.is_void
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_ai: PD_GSL_COMPLEX
			l_vi: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_re: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_im: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_scale: REAL_64
			l_tmp___0: REAL_64
		do
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]} l_vi.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_re.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_VECTOR_VIEW]} l_im.make (<<1>>)

			l_N := a_evec.item.size1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_ai := (a_alpha.item.gsl_vector_complex_get (l_i)).twin
				l_vi.item := (a_evec.item.gsl_matrix_complex_column (l_i)).twin
				l_re.item := (gsl_vector_complex_real (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
				if l_ai.dat[1] = 0.0 then
					l_scale := 1.0 / ((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2);
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dscal (l_scale)
				elseif l_ai.dat[1] > 0.0 then
					l_im.item := (gsl_vector_complex_imag (create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					l_tmp___0 := (create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_im.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2
					l_scale := 1.0 / ((create {P_HYPOT_STATIC}).gsl_hypot (((create {CE_CLASS_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).item.gsl_blas_dnrm2), l_tmp___0));
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_blas_zdscal (l_scale)
					l_vi.item := (a_evec.item.gsl_matrix_complex_column (l_i + {NATURAL_32} 1)).twin;
					(create {CE_CLASS_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))).item.gsl_blas_zdscal (l_scale)
				end
				l_i := l_i + 1
			end
		end

	gsl_vector_complex_real (a_v: CE_POINTER [PD_GSL_VECTOR_COMPLEX]): PD_GSL_VECTOR_VIEW
		require
			not a_v.is_void
		local
			l_s: PD_GSL_VECTOR
			l_view: PD_GSL_VECTOR_VIEW
		do
			create l_s.make
			create l_view.make

			l_s.size := (0).to_natural_32
			l_s.stride := (0).to_natural_32
			l_s.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			l_s.owner := 0
			l_s.data := a_v.item.data
			l_s.size := a_v.item.size
			l_s.stride := {NATURAL_32} 2 * a_v.item.stride
			l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			l_s.owner := 0
			l_view.vector.size := (0).to_natural_32
			l_view.vector.stride := (0).to_natural_32
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			l_view.vector.owner := 0
			l_view.vector := l_s.twin
			Result := l_view.twin
		end

	gsl_vector_complex_imag (a_v: CE_POINTER [PD_GSL_VECTOR_COMPLEX]): PD_GSL_VECTOR_VIEW
		require
			not a_v.is_void
		local
			l_s: PD_GSL_VECTOR
			l_view: PD_GSL_VECTOR_VIEW
		do
			create l_s.make
			create l_view.make

			l_s.size := (0).to_natural_32
			l_s.stride := (0).to_natural_32
			l_s.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			l_s.owner := 0
			l_s.data := a_v.item.data + 1
			l_s.size := a_v.item.size
			l_s.stride := {NATURAL_32} 2 * a_v.item.stride
			l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			l_s.owner := 0
			l_view.vector.size := (0).to_natural_32
			l_view.vector.stride := (0).to_natural_32
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			l_view.vector.owner := 0
			l_view.vector := l_s.twin
			Result := l_view.twin
		end

	gen_store_eigval1 (a_H: CE_POINTER [PD_GSL_MATRIX]; a_a___5, a_b___0: REAL_64; a_alpha: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_beta: CE_POINTER [PD_GSL_VECTOR]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE])
		require
			not a_alpha.is_void
			not a_beta.is_void
			not a_w.is_void
		local
			l_top: NATURAL_32
			l_z: PD_GSL_COMPLEX
			l_break: BOOLEAN
		do
			create l_z.make

			l_top := ((a_w.item.H).item.gen_get_submatrix (a_H))
			from
			until
				(l_break)
			loop
				l_break := False
				l_z.dat[0] := a_a___5
				l_z.dat[1] := 0.0
				l_break := True
			end
			l_break := False
			a_alpha.item.gsl_vector_complex_set (l_top, l_z.twin)
			a_beta.item.gsl_vector_set (l_top, a_b___0)
			a_w.item.n_evals := (a_w.item.n_evals) + 1
			a_w.item.n_iter := (0).to_natural_32
			a_w.item.eshift := 0.0
		end

	gen_store_eigval2 (a_H: CE_POINTER [PD_GSL_MATRIX]; a_alpha1: CE_POINTER [PD_GSL_COMPLEX]; a_beta1: REAL_64; a_alpha2: CE_POINTER [PD_GSL_COMPLEX]; a_beta2: REAL_64; a_alpha: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_beta: CE_POINTER [PD_GSL_VECTOR]; a_w: CE_POINTER [PD_GSL_EIGEN_GEN_WORKSPACE])
		require
			not a_alpha1.is_void
			not a_alpha2.is_void
			not a_alpha.is_void
			not a_beta.is_void
			not a_w.is_void
		local
			l_top: NATURAL_32
		do
			l_top := ((a_w.item.H).item.gen_get_submatrix (a_H))
			a_alpha.item.gsl_vector_complex_set (l_top, (a_alpha1.item).twin)
			a_beta.item.gsl_vector_set (l_top, a_beta1)
			a_alpha.item.gsl_vector_complex_set (l_top + {NATURAL_32} 1, (a_alpha2.item).twin)
			a_beta.item.gsl_vector_set (l_top + {NATURAL_32} 1, a_beta2)
			a_w.item.n_evals := a_w.item.n_evals + ({NATURAL_32} 2)
			a_w.item.n_iter := (0).to_natural_32
			a_w.item.eshift := 0.0
		end

	chop_small_elements___2 (a_N: NATURAL_32; a_d, a_sd: CE_POINTER [REAL_64])
		require
			not a_d.is_void
		local
			l_d_i: REAL_64
			l_i: NATURAL_32
			l_sd_i: REAL_64
			l_d_ip1: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
		do
			l_d_i := a_d[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_N - {NATURAL_32} 1)
			loop
				l_sd_i := a_sd[l_i.to_integer_32]
				l_d_ip1 := a_d[(l_i + {NATURAL_32} 1).to_integer_32]
				l_tmp := fabs (l_sd_i)
				l_tmp___0 := fabs (l_d_i)
				if l_tmp < 2.2204460492503131e-16 * (l_tmp___0 + (fabs (l_d_ip1))) then
					a_sd[l_i.to_integer_32] := 0.0
				end
				l_d_i := l_d_ip1
				l_i := l_i + 1
			end
		end

	qrstep___2 (a_n: NATURAL_32; a_d, a_sd, a_gc, a_gs: CE_POINTER [REAL_64])
		require
			not a_d.is_void
			not a_sd.is_void
		local
			l_x: REAL_64
			l_z: REAL_64
			l_ak: REAL_64
			l_bk: REAL_64
			l_zk: REAL_64
			l_ap: REAL_64
			l_bp: REAL_64
			l_aq: REAL_64
			l_bq: REAL_64
			l_k: NATURAL_32
			l_mu: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_ap1: REAL_64
			l_bp1: REAL_64
			l_aq1: REAL_64
			l_c___2: CE_ARRAY [REAL_64]
			l_s___0: CE_ARRAY [REAL_64]
			l_bk1: REAL_64
			l_ap1___0: REAL_64
			l_bp1___0: REAL_64
			l_zp1: REAL_64
			l_aq1___0: REAL_64
			l_bq1: REAL_64
			l_return: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_c___2.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s___0.make (<<1>>)

			l_mu := ((create {P_QRSTEP_STATIC}).trailing_eigenvalue (a_n, a_d, a_sd))
			l_tmp___0 := fabs (l_mu)
			l_tmp___1 := fabs (a_d[0])
			if 2.2204460492503131e-16 * l_tmp___0 > l_tmp___1 + (fabs (a_sd[0])) then
				l_mu := (0).to_double
			end
			l_x := a_d[0] - l_mu
			l_z := a_sd[0]
			l_ak := (0).to_double
			l_bk := (0).to_double
			l_zk := (0).to_double
			l_ap := a_d[0]
			l_bp := a_sd[0]
			l_aq := a_d[1]
			if a_n = {NATURAL_32} 2 then
				(create {P_QRSTEP_STATIC}).create_givens (l_x, l_z, l_c___1, l_s)
				if not a_gc.is_void then
					a_gc[0] := l_c___1.item
				end
				if not a_gs.is_void then
					a_gs[0] := l_s.item
				end
				l_ap1 := l_c___1.item * (l_c___1.item * l_ap - l_s.item * l_bp) + l_s.item * (l_s.item * l_aq - l_c___1.item * l_bp)
				l_bp1 := l_c___1.item * (l_s.item * l_ap + l_c___1.item * l_bp) - l_s.item * (l_s.item * l_bp + l_c___1.item * l_aq)
				l_aq1 := l_s.item * (l_s.item * l_ap + l_c___1.item * l_bp) + l_c___1.item * (l_s.item * l_bp + l_c___1.item * l_aq)
				l_ak := l_ap1
				l_bk := l_bp1
				l_ap := l_aq1
				a_d[0] := l_ak
				a_sd[0] := l_bk
				a_d[1] := l_ap
				l_return := True
			end
			if not l_return then
				l_bq := a_sd[1]
				l_k := (0).to_natural_32
				from
				until
					not (l_k < a_n - {NATURAL_32} 1)
				loop
					(create {P_QRSTEP_STATIC}).create_givens (l_x, l_z, l_c___2, l_s___0)
					if not a_gc.is_void then
						a_gc[l_k.to_integer_32] := l_c___2.item
					end
					if not a_gs.is_void then
						a_gs[l_k.to_integer_32] := l_s___0.item
					end
					l_bk1 := l_c___2.item * l_bk - l_s___0.item * l_zk
					l_ap1___0 := l_c___2.item * (l_c___2.item * l_ap - l_s___0.item * l_bp) + l_s___0.item * (l_s___0.item * l_aq - l_c___2.item * l_bp)
					l_bp1___0 := l_c___2.item * (l_s___0.item * l_ap + l_c___2.item * l_bp) - l_s___0.item * (l_s___0.item * l_bp + l_c___2.item * l_aq)
					l_zp1 := -l_s___0.item * l_bq
					l_aq1___0 := l_s___0.item * (l_s___0.item * l_ap + l_c___2.item * l_bp) + l_c___2.item * (l_s___0.item * l_bp + l_c___2.item * l_aq)
					l_bq1 := l_c___2.item * l_bq
					l_ak := l_ap1___0
					l_bk := l_bp1___0
					l_zk := l_zp1
					l_ap := l_aq1___0
					l_bp := l_bq1
					if l_k < a_n - {NATURAL_32} 2 then
						l_aq := a_d[(l_k + {NATURAL_32} 2).to_integer_32]
					end
					if l_k < a_n - {NATURAL_32} 3 then
						l_bq := a_sd[(l_k + {NATURAL_32} 2).to_integer_32]
					end
					a_d[l_k.to_integer_32] := l_ak
					if l_k > {NATURAL_32} 0 then
						a_sd[(l_k - {NATURAL_32} 1).to_integer_32] := l_bk1
					end
					if l_k < a_n - {NATURAL_32} 2 then
						a_sd[(l_k + {NATURAL_32} 1).to_integer_32] := l_bp
					end
					l_x := l_bk
					l_z := l_zk
					l_k := l_k + 1
				end
				a_d[l_k.to_integer_32] := l_ap
				a_sd[(l_k - {NATURAL_32} 1).to_integer_32] := l_bk
			end
		end

	quiet_sqrt (a_x: REAL_64): REAL_64
		local
			l_tmp___1: REAL_64
		do
			if a_x >= (0).to_double then
				l_tmp___1 := (sqrt (a_x))
			else
				l_tmp___1 := (builtin_nanf (ce_string (""))).to_double
			end
			Result := l_tmp___1
		end

	chop_small_elements___3 (a_d, a_f: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_d.is_void
		local
			l_N: NATURAL_32
			l_d_i: REAL_64
			l_i: NATURAL_32
			l_f_i: REAL_64
			l_d_ip1: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
		do
			l_N := a_d.item.size
			l_d_i := (a_d.item.gsl_vector_get ((0).to_natural_32))
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N - {NATURAL_32} 1)
			loop
				l_f_i := (a_f.item.gsl_vector_get (l_i))
				l_d_ip1 := (a_d.item.gsl_vector_get (l_i + {NATURAL_32} 1))
				l_tmp___2 := fabs (l_f_i)
				l_tmp___3 := fabs (l_d_i)
				if l_tmp___2 < 2.2204460492503131e-16 * (l_tmp___3 + (fabs (l_d_ip1))) then
					a_f.item.gsl_vector_set (l_i, 0.0)
				end
				l_d_i := l_d_ip1
				l_i := l_i + 1
			end
		end

	qrstep___3 (a_d, a_f: CE_POINTER [PD_GSL_VECTOR]; a_U, a_V: CE_POINTER [PD_GSL_MATRIX])
		require
			not a_d.is_void
			not a_U.is_void
			not a_V.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_n2: NATURAL_32
			l_y: REAL_64
			l_z: REAL_64
			l_ak: REAL_64
			l_bk: REAL_64
			l_zk: REAL_64
			l_ap: REAL_64
			l_bp: REAL_64
			l_aq: REAL_64
			l_bq: REAL_64
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_d_i: REAL_64
			l_d_nm1: REAL_64
			l_d0: REAL_64
			l_f0: REAL_64
			l_d1: REAL_64
			l_f1: REAL_64
			l_mu: REAL_64
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_Vip: REAL_64
			l_Viq: REAL_64
			l_bk1: REAL_64
			l_ap1: REAL_64
			l_bp1: REAL_64
			l_zp1: REAL_64
			l_aq1: REAL_64
			l_Uip: REAL_64
			l_Uiq: REAL_64
			l_ak1: REAL_64
			l_bk1___0: REAL_64
			l_zk1: REAL_64
			l_ap1___0: REAL_64
			l_bp1___0: REAL_64
			l_return: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)

			l_M := a_U.item.size1
			l_N := a_V.item.size1
			l_n2 := a_d.item.size
			if l_n2 = {NATURAL_32} 1 then
				l_return := True
			end
			if not l_return and then (l_n2 = {NATURAL_32} 2) then
				a_d.item.svd2 (a_f, a_U, a_V)
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_n2 - {NATURAL_32} 1))
				loop
					l_d_i := (a_d.item.gsl_vector_get (l_i))
					if l_d_i = 0.0 then
						chase_out_intermediate_zero (a_d, a_f, a_U, l_i)
						l_return := True
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				l_d_nm1 := (a_d.item.gsl_vector_get (l_n2 - {NATURAL_32} 1))
				if l_d_nm1 = 0.0 then
					chase_out_trailing_zero (a_d, a_f, a_V)
					l_return := True
				end
			end
			if not l_return then
				l_d0 := (a_d.item.gsl_vector_get ((0).to_natural_32))
				l_f0 := (a_f.item.gsl_vector_get ((0).to_natural_32))
				l_d1 := (a_d.item.gsl_vector_get ((1).to_natural_32))
				l_f1 := (a_f.item.gsl_vector_get ((1).to_natural_32))
				l_mu := (trailing_eigenvalue___3 (a_d, a_f))
				l_y := l_d0 * l_d0 - l_mu
				l_z := l_d0 * l_f0
				l_ak := (0).to_double
				l_bk := (0).to_double
				l_ap := l_d0
				l_bp := l_f0
				l_aq := l_d1
				l_bq := l_f1
				l_k := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < l_n2 - {NATURAL_32} 1))
				loop
					(create {P_QRSTEP_STATIC}).create_givens (l_y, l_z, l_c___1, l_s)
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_N)
					loop
						l_Vip := (a_V.item.gsl_matrix_get (l_i, l_k))
						l_Viq := (a_V.item.gsl_matrix_get (l_i, l_k + {NATURAL_32} 1))
						a_V.item.gsl_matrix_set (l_i, l_k, l_c___1.item * l_Vip - l_s.item * l_Viq)
						a_V.item.gsl_matrix_set (l_i, l_k + {NATURAL_32} 1, l_s.item * l_Vip + l_c___1.item * l_Viq)
						l_i := l_i + 1
					end
					l_bk1 := l_c___1.item * l_bk - l_s.item * l_z
					l_ap1 := l_c___1.item * l_ap - l_s.item * l_bp
					l_bp1 := l_s.item * l_ap + l_c___1.item * l_bp
					l_zp1 := -l_s.item * l_aq
					l_aq1 := l_c___1.item * l_aq
					if l_k > {NATURAL_32} 0 then
						a_f.item.gsl_vector_set (l_k - {NATURAL_32} 1, l_bk1)
					end
					l_ak := l_ap1
					l_bk := l_bp1
					l_zk := l_zp1
					l_ap := l_aq1
					if l_k < l_n2 - {NATURAL_32} 2 then
						l_bp := a_f.item.gsl_vector_get (l_k + {NATURAL_32} 1)
					else
						l_bp := 0.0
					end
					l_y := l_ak
					l_z := l_zk;
					(create {P_QRSTEP_STATIC}).create_givens (l_y, l_z, l_c___1, l_s)
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_M)
					loop
						l_Uip := (a_U.item.gsl_matrix_get (l_i, l_k))
						l_Uiq := (a_U.item.gsl_matrix_get (l_i, l_k + {NATURAL_32} 1))
						a_U.item.gsl_matrix_set (l_i, l_k, l_c___1.item * l_Uip - l_s.item * l_Uiq)
						a_U.item.gsl_matrix_set (l_i, l_k + {NATURAL_32} 1, l_s.item * l_Uip + l_c___1.item * l_Uiq)
						l_i := l_i + 1
					end
					l_ak1 := l_c___1.item * l_ak - l_s.item * l_zk
					l_bk1___0 := l_c___1.item * l_bk - l_s.item * l_ap
					l_zk1 := -l_s.item * l_bp
					l_ap1___0 := l_s.item * l_bk + l_c___1.item * l_ap
					l_bp1___0 := l_c___1.item * l_bp
					a_d.item.gsl_vector_set (l_k, l_ak1)
					l_ak := l_ak1
					l_bk := l_bk1___0
					l_zk := l_zk1
					l_ap := l_ap1___0
					l_bp := l_bp1___0
					if l_k < l_n2 - {NATURAL_32} 2 then
						l_aq := a_d.item.gsl_vector_get (l_k + {NATURAL_32} 2)
					else
						l_aq := 0.0
					end
					l_y := l_bk
					l_z := l_zk
					l_k := l_k + 1
				end
				a_f.item.gsl_vector_set (l_n2 - {NATURAL_32} 2, l_bk)
				a_d.item.gsl_vector_set (l_n2 - {NATURAL_32} 1, l_ap)
			end
		end

	trailing_eigenvalue___3 (a_d, a_f: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		require
			not a_d.is_void
			not a_f.is_void
		local
			l_n: NATURAL_32
			l_da: REAL_64
			l_db: REAL_64
			l_fa: REAL_64
			l_tmp___2: REAL_64
			l_fb: REAL_64
			l_mu: REAL_64
			l_ta: REAL_64
			l_tb: REAL_64
			l_tab: REAL_64
			l_dt: REAL_64
			l_S: REAL_64
			l_da2: REAL_64
			l_db2: REAL_64
			l_fa2: REAL_64
			l_fb2: REAL_64
			l_P___2: REAL_64
			l_D: REAL_64
			l_r1___1: REAL_64
		do
			l_n := a_d.item.size
			l_da := (a_d.item.gsl_vector_get (l_n - {NATURAL_32} 2))
			l_db := (a_d.item.gsl_vector_get (l_n - {NATURAL_32} 1))
			if l_n > {NATURAL_32} 2 then
				l_tmp___2 := (a_f.item.gsl_vector_get (l_n - {NATURAL_32} 3))
			else
				l_tmp___2 := 0.0
			end
			l_fa := l_tmp___2
			l_fb := (a_f.item.gsl_vector_get (l_n - {NATURAL_32} 2))
			l_ta := l_da * l_da + l_fa * l_fa
			l_tb := l_db * l_db + l_fb * l_fb
			l_tab := l_da * l_fb
			l_dt := (l_ta - l_tb) / 2.0
			l_S := l_ta + l_tb
			l_da2 := l_da * l_da
			l_db2 := l_db * l_db
			l_fa2 := l_fa * l_fa
			l_fb2 := l_fb * l_fb
			l_P___2 := (l_da2 * l_db2 + l_fa2 * l_db2) + l_fa2 * l_fb2
			l_D := (hypot (l_dt, l_tab))
			l_r1___1 := l_S / (2).to_double + l_D
			if l_dt >= (0).to_double then
				if l_r1___1 > (0).to_double then
					l_mu := l_P___2 / l_r1___1
				else
					l_mu := 0.0
				end
			else
				l_mu := l_r1___1
			end
			Result := l_mu
		end

	chase_out_intermediate_zero (a_d, a_f: CE_POINTER [PD_GSL_VECTOR]; a_U: CE_POINTER [PD_GSL_MATRIX]; a_k0: NATURAL_32)
		require
			not a_d.is_void
			not a_f.is_void
			not a_U.is_void
		local
			l_M: NATURAL_32
			l_n: NATURAL_32
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_x: REAL_64
			l_y: REAL_64
			l_k: NATURAL_32
			l_i: NATURAL_32
			l_Uip: REAL_64
			l_Uiq: REAL_64
			l_z: REAL_64
			l_return: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)

			l_M := a_U.item.size1
			l_n := a_d.item.size
			l_x := a_f.item.gsl_vector_get (a_k0)
			l_y := a_d.item.gsl_vector_get (a_k0 + {NATURAL_32} 1)
			l_k := a_k0
			from
			until
				not (not (l_return) and then (l_k < l_n - {NATURAL_32} 1))
			loop
				(create {P_QRSTEP_STATIC}).create_givens (l_y, -l_x, l_c___1, l_s)
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_M)
				loop
					l_Uip := (a_U.item.gsl_matrix_get (l_i, a_k0))
					l_Uiq := (a_U.item.gsl_matrix_get (l_i, l_k + {NATURAL_32} 1))
					a_U.item.gsl_matrix_set (l_i, a_k0, l_c___1.item * l_Uip - l_s.item * l_Uiq)
					a_U.item.gsl_matrix_set (l_i, l_k + {NATURAL_32} 1, l_s.item * l_Uip + l_c___1.item * l_Uiq)
					l_i := l_i + 1
				end
				a_d.item.gsl_vector_set (l_k + {NATURAL_32} 1, l_s.item * l_x + l_c___1.item * l_y)
				if l_k = a_k0 then
					a_f.item.gsl_vector_set (l_k, l_c___1.item * l_x - l_s.item * l_y)
				end
				if l_k < l_n - {NATURAL_32} 2 then
					l_z := (a_f.item.gsl_vector_get (l_k + {NATURAL_32} 1))
					a_f.item.gsl_vector_set (l_k + {NATURAL_32} 1, l_c___1.item * l_z)
					l_x := -l_s.item * l_z
					l_y := a_d.item.gsl_vector_get (l_k + {NATURAL_32} 2)
				end
				l_k := l_k + 1
			end
		end

	chase_out_trailing_zero (a_d, a_f: CE_POINTER [PD_GSL_VECTOR]; a_V: CE_POINTER [PD_GSL_MATRIX])
		require
			not a_d.is_void
			not a_f.is_void
			not a_V.is_void
		local
			l_N: NATURAL_32
			l_n2: NATURAL_32
			l_c___1: CE_ARRAY [REAL_64]
			l_s: CE_ARRAY [REAL_64]
			l_x: REAL_64
			l_y: REAL_64
			l_k: NATURAL_32
			l_i: NATURAL_32
			l_Vip: REAL_64
			l_Viq: REAL_64
			l_z: REAL_64
			l_tmp___2: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_c___1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_s.make (<<1>>)

			l_N := a_V.item.size1
			l_n2 := a_d.item.size
			l_x := a_d.item.gsl_vector_get (l_n2 - {NATURAL_32} 2)
			l_y := a_f.item.gsl_vector_get (l_n2 - {NATURAL_32} 2)
			l_k := l_n2 - {NATURAL_32} 1
			from
			until
				(l_return or l_break)
			loop
				l_break := False
				l_tmp___2 := l_k
				l_k := l_k - 1
				if not (l_tmp___2 > {NATURAL_32} 0) then
					l_break := True
				end
				if not (l_return or l_break) then
					(create {P_QRSTEP_STATIC}).create_givens (l_x, l_y, l_c___1, l_s)
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_N)
					loop
						l_Vip := (a_V.item.gsl_matrix_get (l_i, l_k))
						l_Viq := (a_V.item.gsl_matrix_get (l_i, l_n2 - {NATURAL_32} 1))
						a_V.item.gsl_matrix_set (l_i, l_k, l_c___1.item * l_Vip - l_s.item * l_Viq)
						a_V.item.gsl_matrix_set (l_i, l_n2 - {NATURAL_32} 1, l_s.item * l_Vip + l_c___1.item * l_Viq)
						l_i := l_i + 1
					end
					a_d.item.gsl_vector_set (l_k, l_c___1.item * l_x - l_s.item * l_y)
					if l_k = l_n2 - {NATURAL_32} 2 then
						a_f.item.gsl_vector_set (l_k, l_s.item * l_x + l_c___1.item * l_y)
					end
					if l_k > {NATURAL_32} 0 then
						l_z := (a_f.item.gsl_vector_get (l_k - {NATURAL_32} 1))
						a_f.item.gsl_vector_set (l_k - {NATURAL_32} 1, l_c___1.item * l_z)
						l_x := a_d.item.gsl_vector_get (l_k - {NATURAL_32} 1)
						l_y := l_s.item * l_z
					end
				end
			end
			l_break := False
		end

	minimum_step (a_gnorm: REAL_64; a_diag, a_g: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_g.is_void
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_gi: REAL_64
			l_di: REAL_64
		do
			l_N := a_g.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_gi := (a_g.item.gsl_vector_get (l_i))
				l_di := (a_diag.item.gsl_vector_get (l_i))
				a_g.item.gsl_vector_set (l_i, (l_gi / a_gnorm) / l_di)
				l_i := l_i + 1
			end
		end

	scaled_addition (a_alpha: REAL_64; a_newton: CE_POINTER [PD_GSL_VECTOR]; a_beta: REAL_64; a_gradient, a_p: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_p.is_void
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_ni: REAL_64
			l_gi: REAL_64
		do
			l_N := a_p.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_ni := (a_newton.item.gsl_vector_get (l_i))
				l_gi := (a_gradient.item.gsl_vector_get (l_i))
				a_p.item.gsl_vector_set (l_i, a_alpha * l_ni + a_beta * l_gi)
				l_i := l_i + 1
			end
		end

	minimum_step___0 (a_gnorm: REAL_64; a_diag, a_g: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_g.is_void
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_gi: REAL_64
			l_di: REAL_64
		do
			l_N := a_g.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_gi := (a_g.item.gsl_vector_get (l_i))
				l_di := (a_diag.item.gsl_vector_get (l_i))
				a_g.item.gsl_vector_set (l_i, (l_gi / a_gnorm) / l_di)
				l_i := l_i + 1
			end
		end

	scaled_addition___0 (a_alpha: REAL_64; a_newton: CE_POINTER [PD_GSL_VECTOR]; a_beta: REAL_64; a_gradient, a_p: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_p.is_void
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_ni: REAL_64
			l_gi: REAL_64
		do
			l_N := a_p.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_ni := (a_newton.item.gsl_vector_get (l_i))
				l_gi := (a_gradient.item.gsl_vector_get (l_i))
				a_p.item.gsl_vector_set (l_i, a_alpha * l_ni + a_beta * l_gi)
				l_i := l_i + 1
			end
		end

	gsl_permute_inverse (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_pk: NATURAL_32
			l_a___5: NATURAL_32
			l_t: CE_ARRAY [REAL_64]
			l_r1___1: REAL_64
			l_return, l_continue: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_t.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < a_n))
			loop
				l_continue := False
				l_k := a_p[l_i.to_integer_32]
				from
				until
					not (l_k > l_i)
				loop
					l_k := a_p[l_k.to_integer_32]
				end
				if l_k < l_i then
					l_continue := True
				end
				if not (l_return or l_continue) then
					l_pk := a_p[l_k.to_integer_32]
					if l_pk = l_i then
						l_continue := True
					end
				end
				if not (l_return or l_continue) then
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						l_t[l_a___5.to_integer_32] := a_data[(l_k * a_stride + l_a___5).to_integer_32]
						l_a___5 := l_a___5 + 1
					end
					from
					until
						not (not (l_return) and then (l_pk /= l_i))
					loop
						l_a___5 := {NATURAL_32} 0
						from
						until
							not (l_a___5 < {NATURAL_32} 1)
						loop
							l_r1___1 := a_data[(l_pk * a_stride + l_a___5).to_integer_32]
							a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
							l_t[l_a___5.to_integer_32] := l_r1___1
							l_a___5 := l_a___5 + 1
						end
						l_k := l_pk
						l_pk := a_p[l_k.to_integer_32]
					end
					l_a___5 := {NATURAL_32} 0
					from
					until
						not (l_a___5 < {NATURAL_32} 1)
					loop
						a_data[(l_pk * a_stride + l_a___5).to_integer_32] := l_t[l_a___5.to_integer_32]
						l_a___5 := l_a___5 + 1
					end
				end
				if (not l_return) then
					l_i := l_i + 1
				end
			end
			l_continue := False
			Result := 0
		end

	gsl_hypot3 (a_x, a_y, a_z: REAL_64): REAL_64
		local
			l_xabs: REAL_64
			l_yabs: REAL_64
			l_zabs: REAL_64
			l_w: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_r___3: REAL_64
		do
			l_xabs := (fabs (a_x))
			l_yabs := (fabs (a_y))
			l_zabs := (fabs (a_z))
			if l_yabs > l_zabs then
				l_tmp___5 := l_yabs
			else
				l_tmp___5 := l_zabs
			end
			if l_xabs > l_tmp___5 then
				l_tmp___4 := l_xabs
			else
				if l_yabs > l_zabs then
					l_tmp___3 := l_yabs
				else
					l_tmp___3 := l_zabs
				end
				l_tmp___4 := l_tmp___3
			end
			l_w := l_tmp___4
			if l_w = 0.0 then
				Result := (0.0)
			else
				l_r___3 := l_w * (sqrt (((l_xabs / l_w) * (l_xabs / l_w) + (l_yabs / l_w) * (l_yabs / l_w)) + (l_zabs / l_w) * (l_zabs / l_w)))
				Result := l_r___3
			end
		end

	gsl_vector_view_array_with_stride (a_base: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): PD_GSL_VECTOR_VIEW
		local
			l_view: PD_GSL_VECTOR_VIEW
			l_v: PD_GSL_VECTOR
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			l_view.vector.stride := (0).to_natural_32
			l_view.vector.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
			l_view.vector.owner := 0
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("view_source.c"), 55, 4)
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_stride = {NATURAL_32} 0) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("stride must be positive integer"), ce_string ("view_source.c"), 61, 4)
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				l_v.stride := (0).to_natural_32
				l_v.data := create {CE_REAL_64_POINTER}.make_shared (default_pointer)
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				l_v.owner := 0
				l_v.data := a_base
				l_v.size := a_n
				l_v.stride := a_stride
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				l_v.owner := 0
				l_view.vector := l_v.twin
				Result := l_view.twin
			end
		end

	cblas_dasum (a_N: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32): REAL_64
		local
			l_r___3: REAL_64
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_return: BOOLEAN
		do
			l_r___3 := 0.0
			l_ix := 0
			if a_incX <= 0 then
				Result := (0).to_double
				l_return := True
			end
			if not l_return then
				l_i := 0
				from
				until
					not (l_i < a_N)
				loop
					l_r___3 := l_r___3 + (fabs (a_X[l_ix]))
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
				Result := l_r___3
			end
		end

	cblas_dgemm (a_Order, a_TransA, a_TransB: NATURAL_32; a_M, a_N, a_K: INTEGER_32; a_alpha: REAL_64; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_64]; a_ldb: INTEGER_32; a_beta: REAL_64; a_C: CE_POINTER [REAL_64]; a_ldc: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_n1: INTEGER_32
			l_n2: INTEGER_32
			l_ldf: INTEGER_32
			l_ldg: INTEGER_32
			l_TransF: INTEGER_32
			l_TransG: INTEGER_32
			l_F___0: CE_POINTER [REAL_64]
			l_G: CE_POINTER [REAL_64]
			l_temp: REAL_64
			l_temp___0: REAL_64
			l_temp___1: REAL_64
			l_temp___2: REAL_64
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha = 0.0 then
						if a_beta = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_Order = {NATURAL_32} 101 then
							l_n1 := a_M
							l_n2 := a_N
							l_F___0 := a_A___0
							l_ldf := a_lda
							if a_TransA = {NATURAL_32} 113 then
								l_TransF := 112
							else
								l_TransF := a_TransA.to_integer_32
							end
							l_G := a_B
							l_ldg := a_ldb
							if a_TransB = {NATURAL_32} 113 then
								l_TransG := 112
							else
								l_TransG := a_TransB.to_integer_32
							end
						else
							l_n1 := a_N
							l_n2 := a_M
							l_F___0 := a_B
							l_ldf := a_ldb
							if a_TransB = {NATURAL_32} 113 then
								l_TransF := 112
							else
								l_TransF := a_TransB.to_integer_32
							end
							l_G := a_A___0
							l_ldg := a_lda
							if a_TransA = {NATURAL_32} 113 then
								l_TransG := 112
							else
								l_TransG := a_TransA.to_integer_32
							end
						end
						if a_beta = 0.0 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (l_j < l_n2)
								loop
									a_C[(a_ldc * l_i + l_j)] := 0.0
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						elseif a_beta /= 1.0 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (l_j < l_n2)
								loop
									a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						end
						if a_alpha = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 11 -- skip if block
					end
				when 1 then -- if:
						if l_TransF /= 111 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_TransG = 111 then
							l_k := 0
							from
							until
								not (not (l_return) and then (l_k < a_K))
							loop
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_temp := a_alpha * l_F___0[(l_ldf * l_i + l_k)]
									if l_temp /= 0.0 then
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] + (l_temp * l_G[(l_ldg * l_k + l_j)])
											l_j := l_j + 1
										end
									end
									l_i := l_i + 1
								end
								l_k := l_k + 1
							end
						else
							l_pos := -1 + 4 -- goto _L___1
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 4 then -- else: | _L___1:
						if l_TransF /= 111 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_TransG = 112 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j < l_n2))
								loop
									l_temp___0 := 0.0
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp___0 := l_temp___0 + (l_F___0[(l_ldf * l_i + l_k)] * l_G[(l_ldg * l_j + l_k)])
										l_k := l_k + 1
									end
									a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] + (a_alpha * l_temp___0)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else: | _L___0:
						if l_TransF /= 112 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_TransG = 111 then
							l_k := 0
							from
							until
								not (not (l_return) and then (l_k < a_K))
							loop
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_temp___1 := a_alpha * l_F___0[(l_ldf * l_k + l_i)]
									if l_temp___1 /= 0.0 then
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] + (l_temp___1 * l_G[(l_ldg * l_k + l_j)])
											l_j := l_j + 1
										end
									end
									l_i := l_i + 1
								end
								l_k := l_k + 1
							end
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_TransF = 112 then
							if l_TransG = 112 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp___2 := 0.0
										l_k := 0
										from
										until
											not (l_k < a_K)
										loop
											l_temp___2 := l_temp___2 + (l_F___0[(l_ldf * l_k + l_i)] * l_G[(l_ldg * l_j + l_k)])
											l_k := l_k + 1
										end
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] + (a_alpha * l_temp___2)
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_gemm_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_gemm_r.h"), ce_string ("unrecognized operation")])
						end
				when 11 then -- if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dgemv (a_order, a_TransA: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_64; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_beta: REAL_64; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_lenX: INTEGER_32
			l_lenY: INTEGER_32
			l_Trans: INTEGER_32
			l_tmp: NATURAL_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
			l_iy___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_iy___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_temp: REAL_64
			l_ix: INTEGER_32
			l_tmp___3: INTEGER_32
			l_ix___0: INTEGER_32
			l_tmp___4: INTEGER_32
			l_temp___0: REAL_64
			l_iy___2: INTEGER_32
			l_tmp___5: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_TransA /= {NATURAL_32} 113 then
						l_tmp := a_TransA
					else
						l_tmp := {NATURAL_32} 112
					end
					l_Trans := l_tmp.to_integer_32
					if a_M = 0 then
						l_return := True
					elseif a_N = 0 then
						l_return := True
					end
					if not l_return and then (a_alpha = 0.0) then
						if a_beta = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if l_Trans = 111 then
							l_lenX := a_N
							l_lenY := a_M
						else
							l_lenX := a_M
							l_lenY := a_N
						end
						if a_beta = 0.0 then
							if a_incY > 0 then
								l_tmp___0 := 0
							else
								l_tmp___0 := (l_lenY - 1) * -a_incY
							end
							l_iy := l_tmp___0
							l_i := 0
							from
							until
								not (l_i < l_lenY)
							loop
								a_Y[l_iy] := 0.0
								l_iy := l_iy + a_incY
								l_i := l_i + 1
							end
						elseif a_beta /= 1.0 then
							if a_incY > 0 then
								l_tmp___1 := 0
							else
								l_tmp___1 := (l_lenY - 1) * -a_incY
							end
							l_iy___0 := l_tmp___1
							l_i := 0
							from
							until
								not (l_i < l_lenY)
							loop
								a_Y[l_iy___0] := a_Y[l_iy___0] * a_beta
								l_iy___0 := l_iy___0 + a_incY
								l_i := l_i + 1
							end
						end
						if a_alpha = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 20 -- skip if block
					end
				when 1 then -- if:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_Trans = 111 then
							l_pos := -1 + 6 -- goto _L___1
						else
							l_pos := -1 + 4 -- goto _L___3
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 4 then -- else: | _L___3:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 10 -- skip if block
						end
				when 5 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if: | _L___1:
						if a_incY > 0 then
							l_tmp___2 := 0
						else
							l_tmp___2 := (l_lenY - 1) * -a_incY
						end
						l_iy___1 := l_tmp___2
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < l_lenY))
						loop
							l_temp := 0.0
							if a_incX > 0 then
								l_tmp___3 := 0
							else
								l_tmp___3 := (l_lenX - 1) * -a_incX
							end
							l_ix := l_tmp___3
							l_j := 0
							from
							until
								not (l_j < l_lenX)
							loop
								l_temp := l_temp + (a_X[l_ix] * a_A___0[(a_lda * l_i + l_j)])
								l_ix := l_ix + a_incX
								l_j := l_j + 1
							end
							a_Y[l_iy___1] := a_Y[l_iy___1] + (a_alpha * l_temp)
							l_iy___1 := l_iy___1 + a_incY
							l_i := l_i + 1
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else:
						l_pos := -1 + 10 -- goto _L___2
				when 9 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 10 then -- else: | _L___2:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						if l_Trans = 112 then
							l_pos := -1 + 15 -- goto _L
						else
							l_pos := -1 + 13 -- goto _L___0
						end
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 13 then -- else: | _L___0:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 19 -- skip if block
						end
				when 14 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 17 -- skip if block
						end
				when 15 then -- if: | _L:
						if a_incX > 0 then
							l_tmp___4 := 0
						else
							l_tmp___4 := (l_lenX - 1) * -a_incX
						end
						l_ix___0 := l_tmp___4
						l_j := 0
						from
						until
							not (not (l_return) and then (l_j < l_lenX))
						loop
							l_temp___0 := a_alpha * a_X[l_ix___0]
							if l_temp___0 /= 0.0 then
								if a_incY > 0 then
									l_tmp___5 := 0
								else
									l_tmp___5 := (l_lenY - 1) * -a_incY
								end
								l_iy___2 := l_tmp___5
								l_i := 0
								from
								until
									not (l_i < l_lenY)
								loop
									a_Y[l_iy___2] := a_Y[l_iy___2] + (l_temp___0 * a_A___0[(a_lda * l_j + l_i)])
									l_iy___2 := l_iy___2 + a_incY
									l_i := l_i + 1
								end
							end
							l_ix___0 := l_ix___0 + a_incX
							l_j := l_j + 1
						end
				when 16 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 17 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_gemv_r.h"), ce_string ("unrecognized operation")])
				when 18 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 19 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_gemv_r.h"), ce_string ("unrecognized operation")])
				when 20 then -- if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dsymm (a_Order, a_Side, a_Uplo: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_64; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_64]; a_ldb: INTEGER_32; a_beta: REAL_64; a_C: CE_POINTER [REAL_64]; a_ldc: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_n1: INTEGER_32
			l_n2: INTEGER_32
			l_uplo: INTEGER_32
			l_side: INTEGER_32
			l_temp1: REAL_64
			l_temp2: REAL_64
			l_Aik: REAL_64
			l_temp1___0: REAL_64
			l_temp2___0: REAL_64
			l_Aik___0: REAL_64
			l_temp1___1: REAL_64
			l_temp2___1: REAL_64
			l_Ajk: REAL_64
			l_temp1___2: REAL_64
			l_temp2___2: REAL_64
			l_Ajk___0: REAL_64
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha = 0.0 then
						if a_beta = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_Order = {NATURAL_32} 101 then
							l_n1 := a_M
							l_n2 := a_N
							l_uplo := a_Uplo.to_integer_32
							l_side := a_Side.to_integer_32
						else
							l_n1 := a_N
							l_n2 := a_M
							if a_Uplo = {NATURAL_32} 121 then
								l_uplo := 122
							else
								l_uplo := 121
							end
							if a_Side = {NATURAL_32} 141 then
								l_side := 142
							else
								l_side := 141
							end
						end
						if a_beta = 0.0 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (l_j < l_n2)
								loop
									a_C[(a_ldc * l_i + l_j)] := 0.0
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						elseif a_beta /= 1.0 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (l_j < l_n2)
								loop
									a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						end
						if a_alpha = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 11 -- skip if block
					end
				when 1 then -- if:
						if l_side /= 141 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_uplo = 121 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j < l_n2))
								loop
									l_temp1 := a_alpha * a_B[(a_ldb * l_i + l_j)]
									l_temp2 := 0.0
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (l_temp1 * a_A___0[(l_i * a_lda + l_i)])
									l_k := l_i + 1
									from
									until
										not (l_k < l_n1)
									loop
										l_Aik := a_A___0[(l_i * a_lda + l_k)]
										a_C[(l_k * a_ldc + l_j)] := a_C[(l_k * a_ldc + l_j)] + (l_Aik * l_temp1)
										l_temp2 := l_temp2 + (l_Aik * a_B[(a_ldb * l_k + l_j)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp2)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 4 -- goto _L___1
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 4 then -- else: | _L___1:
						if l_side /= 141 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_uplo = 122 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j < l_n2))
								loop
									l_temp1___0 := a_alpha * a_B[(a_ldb * l_i + l_j)]
									l_temp2___0 := 0.0
									l_k := 0
									from
									until
										not (l_k < l_i)
									loop
										l_Aik___0 := a_A___0[(l_i * a_lda + l_k)]
										a_C[(l_k * a_ldc + l_j)] := a_C[(l_k * a_ldc + l_j)] + (l_Aik___0 * l_temp1___0)
										l_temp2___0 := l_temp2___0 + (l_Aik___0 * a_B[(a_ldb * l_k + l_j)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (l_temp1___0 * a_A___0[(l_i * a_lda + l_i)] + a_alpha * l_temp2___0)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else: | _L___0:
						if l_side /= 142 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_uplo = 121 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j < l_n2))
								loop
									l_temp1___1 := a_alpha * a_B[(a_ldb * l_i + l_j)]
									l_temp2___1 := 0.0
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (l_temp1___1 * a_A___0[(l_j * a_lda + l_j)])
									l_k := l_j + 1
									from
									until
										not (l_k < l_n2)
									loop
										l_Ajk := a_A___0[(l_j * a_lda + l_k)]
										a_C[(l_i * a_ldc + l_k)] := a_C[(l_i * a_ldc + l_k)] + (l_temp1___1 * l_Ajk)
										l_temp2___1 := l_temp2___1 + (a_B[(a_ldb * l_i + l_k)] * l_Ajk)
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp2___1)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_side = 142 then
							if l_uplo = 122 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp1___2 := a_alpha * a_B[(a_ldb * l_i + l_j)]
										l_temp2___2 := 0.0
										l_k := 0
										from
										until
											not (l_k < l_j)
										loop
											l_Ajk___0 := a_A___0[(l_j * a_lda + l_k)]
											a_C[(l_i * a_ldc + l_k)] := a_C[(l_i * a_ldc + l_k)] + (l_temp1___2 * l_Ajk___0)
											l_temp2___2 := l_temp2___2 + (a_B[(a_ldb * l_i + l_k)] * l_Ajk___0)
											l_k := l_k + 1
										end
										a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (l_temp1___2 * a_A___0[(l_j * a_lda + l_j)] + a_alpha * l_temp2___2)
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_symm_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_symm_r.h"), ce_string ("unrecognized operation")])
						end
				when 11 then -- if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dsymv (a_order, a_Uplo: NATURAL_32; a_N: INTEGER_32; a_alpha: REAL_64; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_beta: REAL_64; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_iy: INTEGER_32
			l_tmp: INTEGER_32
			l_iy___0: INTEGER_32
			l_tmp___0: INTEGER_32
			l_ix: INTEGER_32
			l_tmp___1: INTEGER_32
			l_iy___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_temp1: REAL_64
			l_temp2: REAL_64
			l_j_min: INTEGER_32
			l_j_max: INTEGER_32
			l_jx: INTEGER_32
			l_tmp___3: INTEGER_32
			l_jy: INTEGER_32
			l_tmp___4: INTEGER_32
			l_ix___0: INTEGER_32
			l_tmp___5: INTEGER_32
			l_iy___2: INTEGER_32
			l_tmp___6: INTEGER_32
			l_temp1___0: REAL_64
			l_temp2___0: REAL_64
			l_j_min___0: INTEGER_32
			l_j_max___0: INTEGER_32
			l_jx___0: INTEGER_32
			l_tmp___7: INTEGER_32
			l_jy___0: INTEGER_32
			l_tmp___8: INTEGER_32
			l_tmp___9: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha = 0.0 then
						if a_beta = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_beta = 0.0 then
							if a_incY > 0 then
								l_tmp := 0
							else
								l_tmp := (a_N - 1) * -a_incY
							end
							l_iy := l_tmp
							l_i := 0
							from
							until
								not (l_i < a_N)
							loop
								a_Y[l_iy] := 0.0
								l_iy := l_iy + a_incY
								l_i := l_i + 1
							end
						elseif a_beta /= 1.0 then
							if a_incY > 0 then
								l_tmp___0 := 0
							else
								l_tmp___0 := (a_N - 1) * -a_incY
							end
							l_iy___0 := l_tmp___0
							l_i := 0
							from
							until
								not (l_i < a_N)
							loop
								a_Y[l_iy___0] := a_Y[l_iy___0] * a_beta
								l_iy___0 := l_iy___0 + a_incY
								l_i := l_i + 1
							end
						end
						if a_alpha = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 20 -- skip if block
					end
				when 1 then -- if:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if a_Uplo = {NATURAL_32} 121 then
							l_pos := -1 + 6 -- goto _L___1
						else
							l_pos := -1 + 4 -- goto _L___3
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 4 then -- else: | _L___3:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 10 -- skip if block
						end
				when 5 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if: | _L___1:
						if a_incX > 0 then
							l_tmp___1 := 0
						else
							l_tmp___1 := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp___1
						if a_incY > 0 then
							l_tmp___2 := 0
						else
							l_tmp___2 := (a_N - 1) * -a_incY
						end
						l_iy___1 := l_tmp___2
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_temp1 := a_alpha * a_X[l_ix]
							l_temp2 := 0.0
							l_j_min := l_i + 1
							l_j_max := a_N
							if a_incX > 0 then
								l_tmp___3 := 0
							else
								l_tmp___3 := (a_N - 1) * -a_incX
							end
							l_jx := l_tmp___3 + l_j_min * a_incX
							if a_incY > 0 then
								l_tmp___4 := 0
							else
								l_tmp___4 := (a_N - 1) * -a_incY
							end
							l_jy := l_tmp___4 + l_j_min * a_incY
							a_Y[l_iy___1] := a_Y[l_iy___1] + (l_temp1 * a_A___0[(a_lda * l_i + l_i)])
							l_j := l_j_min
							from
							until
								not (l_j < l_j_max)
							loop
								a_Y[l_jy] := a_Y[l_jy] + (l_temp1 * a_A___0[(a_lda * l_i + l_j)])
								l_temp2 := l_temp2 + (a_X[l_jx] * a_A___0[(a_lda * l_i + l_j)])
								l_jx := l_jx + a_incX
								l_jy := l_jy + a_incY
								l_j := l_j + 1
							end
							a_Y[l_iy___1] := a_Y[l_iy___1] + (a_alpha * l_temp2)
							l_ix := l_ix + a_incX
							l_iy___1 := l_iy___1 + a_incY
							l_i := l_i + 1
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else:
						l_pos := -1 + 10 -- goto _L___2
				when 9 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 10 then -- else: | _L___2:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						if a_Uplo = {NATURAL_32} 122 then
							l_pos := -1 + 15 -- goto _L
						else
							l_pos := -1 + 13 -- goto _L___0
						end
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 13 then -- else: | _L___0:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 19 -- skip if block
						end
				when 14 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 17 -- skip if block
						end
				when 15 then -- if: | _L:
						if a_incX > 0 then
							l_tmp___5 := 0
						else
							l_tmp___5 := (a_N - 1) * -a_incX
						end
						l_ix___0 := l_tmp___5 + (a_N - 1) * a_incX
						if a_incY > 0 then
							l_tmp___6 := 0
						else
							l_tmp___6 := (a_N - 1) * -a_incY
						end
						l_iy___2 := l_tmp___6 + (a_N - 1) * a_incY
						l_i := a_N
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___9 := l_i
								l_i := l_i - 1
								if not l_tmp___9.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_temp1___0 := a_alpha * a_X[l_ix___0]
								l_temp2___0 := 0.0
								l_j_min___0 := 0
								l_j_max___0 := l_i
								if a_incX > 0 then
									l_tmp___7 := 0
								else
									l_tmp___7 := (a_N - 1) * -a_incX
								end
								l_jx___0 := l_tmp___7 + l_j_min___0 * a_incX
								if a_incY > 0 then
									l_tmp___8 := 0
								else
									l_tmp___8 := (a_N - 1) * -a_incY
								end
								l_jy___0 := l_tmp___8 + l_j_min___0 * a_incY
								a_Y[l_iy___2] := a_Y[l_iy___2] + (l_temp1___0 * a_A___0[(a_lda * l_i + l_i)])
								l_j := l_j_min___0
								from
								until
									not (l_j < l_j_max___0)
								loop
									a_Y[l_jy___0] := a_Y[l_jy___0] + (l_temp1___0 * a_A___0[(a_lda * l_i + l_j)])
									l_temp2___0 := l_temp2___0 + (a_X[l_jx___0] * a_A___0[(a_lda * l_i + l_j)])
									l_jx___0 := l_jx___0 + a_incX
									l_jy___0 := l_jy___0 + a_incY
									l_j := l_j + 1
								end
								a_Y[l_iy___2] := a_Y[l_iy___2] + (a_alpha * l_temp2___0)
								l_ix___0 := l_ix___0 - a_incX
								l_iy___2 := l_iy___2 - a_incY
							end
						end
						l_break := False
				when 16 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 17 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_symv.h"), ce_string ("unrecognized operation")])
				when 18 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 19 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_symv.h"), ce_string ("unrecognized operation")])
				when 20 then -- if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dsyr2 (a_order, a_Uplo: NATURAL_32; a_N: INTEGER_32; a_alpha: REAL_64; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_64]; a_incY: INTEGER_32; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp1: REAL_64
			l_tmp2: REAL_64
			l_jx: INTEGER_32
			l_jy: INTEGER_32
			l_ix___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_iy___0: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp1___0: REAL_64
			l_tmp2___0: REAL_64
			l_jx___0: INTEGER_32
			l_tmp___3: INTEGER_32
			l_jy___0: INTEGER_32
			l_tmp___4: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_N = 0 then
						l_return := True
					end
					if not l_return and then (a_alpha = 0.0) then
						l_return := True
					end
					if l_return then
						l_pos := -1 + 20 -- skip if block
					end
				when 1 then -- if:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if a_Uplo = {NATURAL_32} 121 then
							l_pos := -1 + 6 -- goto _L___1
						else
							l_pos := -1 + 4 -- goto _L___3
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 4 then -- else: | _L___3:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 10 -- skip if block
						end
				when 5 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if: | _L___1:
						if a_incX > 0 then
							l_tmp := 0
						else
							l_tmp := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp
						if a_incY > 0 then
							l_tmp___0 := 0
						else
							l_tmp___0 := (a_N - 1) * -a_incY
						end
						l_iy := l_tmp___0
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_tmp1 := a_alpha * a_X[l_ix]
							l_tmp2 := a_alpha * a_Y[l_iy]
							l_jx := l_ix
							l_jy := l_iy
							l_j := l_i
							from
							until
								not (l_j < a_N)
							loop
								a_A___0[(a_lda * l_i + l_j)] := a_A___0[(a_lda * l_i + l_j)] + (l_tmp1 * a_Y[l_jy] + l_tmp2 * a_X[l_jx])
								l_jx := l_jx + a_incX
								l_jy := l_jy + a_incY
								l_j := l_j + 1
							end
							l_ix := l_ix + a_incX
							l_iy := l_iy + a_incY
							l_i := l_i + 1
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else:
						l_pos := -1 + 10 -- goto _L___2
				when 9 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 10 then -- else: | _L___2:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						if a_Uplo = {NATURAL_32} 122 then
							l_pos := -1 + 15 -- goto _L
						else
							l_pos := -1 + 13 -- goto _L___0
						end
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 13 then -- else: | _L___0:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 19 -- skip if block
						end
				when 14 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 17 -- skip if block
						end
				when 15 then -- if: | _L:
						if a_incX > 0 then
							l_tmp___1 := 0
						else
							l_tmp___1 := (a_N - 1) * -a_incX
						end
						l_ix___0 := l_tmp___1
						if a_incY > 0 then
							l_tmp___2 := 0
						else
							l_tmp___2 := (a_N - 1) * -a_incY
						end
						l_iy___0 := l_tmp___2
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_tmp1___0 := a_alpha * a_X[l_ix___0]
							l_tmp2___0 := a_alpha * a_Y[l_iy___0]
							if a_incX > 0 then
								l_tmp___3 := 0
							else
								l_tmp___3 := (a_N - 1) * -a_incX
							end
							l_jx___0 := l_tmp___3
							if a_incY > 0 then
								l_tmp___4 := 0
							else
								l_tmp___4 := (a_N - 1) * -a_incY
							end
							l_jy___0 := l_tmp___4
							l_j := 0
							from
							until
								not (l_j <= l_i)
							loop
								a_A___0[(a_lda * l_i + l_j)] := a_A___0[(a_lda * l_i + l_j)] + (l_tmp1___0 * a_Y[l_jy___0] + l_tmp2___0 * a_X[l_jx___0])
								l_jx___0 := l_jx___0 + a_incX
								l_jy___0 := l_jy___0 + a_incY
								l_j := l_j + 1
							end
							l_ix___0 := l_ix___0 + a_incX
							l_iy___0 := l_iy___0 + a_incY
							l_i := l_i + 1
						end
				when 16 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 17 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syr2.h"), ce_string ("unrecognized operation")])
				when 18 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 19 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syr2.h"), ce_string ("unrecognized operation")])
				when 20 then -- if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dsyr2k (a_Order, a_Uplo, a_Trans: NATURAL_32; a_N, a_K: INTEGER_32; a_alpha: REAL_64; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_64]; a_ldb: INTEGER_32; a_beta: REAL_64; a_C: CE_POINTER [REAL_64]; a_ldc: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_uplo: INTEGER_32
			l_trans: INTEGER_32
			l_temp: REAL_64
			l_temp1: REAL_64
			l_temp2: REAL_64
			l_temp___0: REAL_64
			l_temp1___0: REAL_64
			l_temp2___0: REAL_64
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha = 0.0 then
						if a_beta = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_Order = {NATURAL_32} 101 then
							l_uplo := a_Uplo.to_integer_32
							if a_Trans = {NATURAL_32} 113 then
								l_trans := 112
							else
								l_trans := a_Trans.to_integer_32
							end
						else
							if a_Uplo = {NATURAL_32} 121 then
								l_uplo := 122
							else
								l_uplo := 121
							end
							if a_Trans = {NATURAL_32} 112 then
								l_trans := 111
							elseif a_Trans = {NATURAL_32} 113 then
								l_trans := 111
							else
								l_trans := 112
							end
						end
						if a_beta = 0.0 then
							if l_uplo = 121 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(a_ldc * l_i + l_j)] := 0.0
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (l_j <= l_i)
									loop
										a_C[(a_ldc * l_i + l_j)] := 0.0
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							end
						elseif a_beta /= 1.0 then
							if l_uplo = 121 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (l_j <= l_i)
									loop
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							end
						end
						if a_alpha = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 11 -- skip if block
					end
				when 1 then -- if:
						if l_uplo /= 121 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_trans = 111 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := l_i
								from
								until
									not (not (l_return) and then (l_j < a_N))
								loop
									l_temp := 0.0
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp := l_temp + (a_A___0[(l_i * a_lda + l_k)] * a_B[(l_j * a_ldb + l_k)] + a_B[(l_i * a_ldb + l_k)] * a_A___0[(l_j * a_lda + l_k)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 4 -- goto _L___1
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 4 then -- else: | _L___1:
						if l_uplo /= 121 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_trans = 112 then
							l_k := 0
							from
							until
								not (not (l_return) and then (l_k < a_K))
							loop
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_temp1 := a_alpha * a_A___0[(l_k * a_lda + l_i)]
									l_temp2 := a_alpha * a_B[(l_k * a_ldb + l_i)]
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(l_i * a_lda + l_j)] := a_C[(l_i * a_lda + l_j)] + (l_temp1 * a_B[(l_k * a_ldb + l_j)] + l_temp2 * a_A___0[(l_k * a_lda + l_j)])
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
								l_k := l_k + 1
							end
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else: | _L___0:
						if l_uplo /= 122 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_trans = 111 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j <= l_i))
								loop
									l_temp___0 := 0.0
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp___0 := l_temp___0 + (a_A___0[(l_i * a_lda + l_k)] * a_B[(l_j * a_ldb + l_k)] + a_B[(l_i * a_ldb + l_k)] * a_A___0[(l_j * a_lda + l_k)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp___0)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_uplo = 122 then
							if l_trans = 112 then
								l_k := 0
								from
								until
									not (not (l_return) and then (l_k < a_K))
								loop
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < a_N))
									loop
										l_temp1___0 := a_alpha * a_A___0[(l_k * a_lda + l_i)]
										l_temp2___0 := a_alpha * a_B[(l_k * a_ldb + l_i)]
										l_j := 0
										from
										until
											not (l_j <= l_i)
										loop
											a_C[(l_i * a_lda + l_j)] := a_C[(l_i * a_lda + l_j)] + (l_temp1___0 * a_B[(l_k * a_ldb + l_j)] + l_temp2___0 * a_A___0[(l_k * a_lda + l_j)])
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
									l_k := l_k + 1
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syr2k_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syr2k_r.h"), ce_string ("unrecognized operation")])
						end
				when 11 then -- if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dsyrk (a_Order, a_Uplo, a_Trans: NATURAL_32; a_N, a_K: INTEGER_32; a_alpha: REAL_64; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_beta: REAL_64; a_C: CE_POINTER [REAL_64]; a_ldc: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_uplo: INTEGER_32
			l_trans: INTEGER_32
			l_temp: REAL_64
			l_temp___0: REAL_64
			l_temp___1: REAL_64
			l_temp___2: REAL_64
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha = 0.0 then
						if a_beta = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_Order = {NATURAL_32} 101 then
							l_uplo := a_Uplo.to_integer_32
							if a_Trans = {NATURAL_32} 113 then
								l_trans := 112
							else
								l_trans := a_Trans.to_integer_32
							end
						else
							if a_Uplo = {NATURAL_32} 121 then
								l_uplo := 122
							else
								l_uplo := 121
							end
							if a_Trans = {NATURAL_32} 112 then
								l_trans := 111
							elseif a_Trans = {NATURAL_32} 113 then
								l_trans := 111
							else
								l_trans := 112
							end
						end
						if a_beta = 0.0 then
							if l_uplo = 121 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(a_ldc * l_i + l_j)] := 0.0
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (l_j <= l_i)
									loop
										a_C[(a_ldc * l_i + l_j)] := 0.0
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							end
						elseif a_beta /= 1.0 then
							if l_uplo = 121 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (l_j <= l_i)
									loop
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							end
						end
						if a_alpha = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 11 -- skip if block
					end
				when 1 then -- if:
						if l_uplo /= 121 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_trans = 111 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := l_i
								from
								until
									not (not (l_return) and then (l_j < a_N))
								loop
									l_temp := 0.0
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp := l_temp + (a_A___0[(l_i * a_lda + l_k)] * a_A___0[(l_j * a_lda + l_k)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 4 -- goto _L___1
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 4 then -- else: | _L___1:
						if l_uplo /= 121 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_trans = 112 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := l_i
								from
								until
									not (not (l_return) and then (l_j < a_N))
								loop
									l_temp___0 := 0.0
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp___0 := l_temp___0 + (a_A___0[(l_k * a_lda + l_i)] * a_A___0[(l_k * a_lda + l_j)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp___0)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else: | _L___0:
						if l_uplo /= 122 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_trans = 111 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j <= l_i))
								loop
									l_temp___1 := 0.0
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp___1 := l_temp___1 + (a_A___0[(l_i * a_lda + l_k)] * a_A___0[(l_j * a_lda + l_k)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp___1)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_uplo = 122 then
							if l_trans = 112 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j <= l_i))
									loop
										l_temp___2 := 0.0
										l_k := 0
										from
										until
											not (l_k < a_K)
										loop
											l_temp___2 := l_temp___2 + (a_A___0[(l_k * a_lda + l_i)] * a_A___0[(l_k * a_lda + l_j)])
											l_k := l_k + 1
										end
										a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp___2)
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syrk_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syrk_r.h"), ce_string ("unrecognized operation")])
						end
				when 11 then -- if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dtrmm (a_Order, a_Side, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_64; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_64]; a_ldb: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_n1: INTEGER_32
			l_n2: INTEGER_32
			l_nonunit: INTEGER_32
			l_side: INTEGER_32
			l_uplo: INTEGER_32
			l_trans: INTEGER_32
			l_temp: REAL_64
			l_temp___0: REAL_64
			l_tmp: INTEGER_32
			l_temp___1: REAL_64
			l_tmp___0: INTEGER_32
			l_temp___2: REAL_64
			l_temp___3: REAL_64
			l_tmp___1: INTEGER_32
			l_temp___4: REAL_64
			l_temp___5: REAL_64
			l_temp___6: REAL_64
			l_tmp___2: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nonunit := (a_Diag = {NATURAL_32} 131).to_integer
					if a_Order = {NATURAL_32} 101 then
						l_n1 := a_M
						l_n2 := a_N
						l_side := a_Side.to_integer_32
						l_uplo := a_Uplo.to_integer_32
						if a_TransA = {NATURAL_32} 113 then
							l_trans := 112
						else
							l_trans := a_TransA.to_integer_32
						end
					else
						l_n1 := a_N
						l_n2 := a_M
						if a_Side = {NATURAL_32} 141 then
							l_side := 142
						else
							l_side := 141
						end
						if a_Uplo = {NATURAL_32} 121 then
							l_uplo := 122
						else
							l_uplo := 121
						end
						if a_TransA = {NATURAL_32} 113 then
							l_trans := 112
						else
							l_trans := a_TransA.to_integer_32
						end
					end
					if l_side /= 141 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_uplo = 121 then
							if l_trans = 111 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp := 0.0
										if l_nonunit.to_boolean then
											l_temp := (a_A___0[(l_i * a_lda + l_i)] * a_B[(l_i * a_ldb + l_j)])
										else
											l_temp := a_B[(l_i * a_ldb + l_j)]
										end
										l_k := l_i + 1
										from
										until
											not (l_k < l_n1)
										loop
											l_temp := l_temp + (a_A___0[(a_lda * l_i + l_k)] * a_B[(l_k * a_ldb + l_j)])
											l_k := l_k + 1
										end
										a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 3 -- goto _L___12
							end
						else
							l_pos := -1 + 3 -- goto _L___12
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 3 then -- else: | _L___12:
						if l_side /= 141 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if l_uplo = 121 then
							if l_trans = 112 then
								l_i := l_n1
								from
								until
									(l_return or l_break)
								loop
									l_break := False
									if l_i > 0 then
										l_tmp := l_i
										l_i := l_i - 1
										if not l_tmp.to_boolean then
											l_break := True
										end
									else
										l_break := True
									end
									if not (l_return or l_break) then
										l_j := 0
										from
										until
											not (not (l_return) and then (l_j < l_n2))
										loop
											l_temp___0 := 0.0
											l_k := 0
											from
											until
												not (l_k < l_i)
											loop
												l_temp___0 := l_temp___0 + (a_A___0[(a_lda * l_k + l_i)] * a_B[(l_k * a_ldb + l_j)])
												l_k := l_k + 1
											end
											if l_nonunit.to_boolean then
												l_temp___0 := l_temp___0 + (a_A___0[(l_i * a_lda + l_i)] * a_B[(l_i * a_ldb + l_j)])
											else
												l_temp___0 := l_temp___0 + (a_B[(l_i * a_ldb + l_j)])
											end
											a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___0
											l_j := l_j + 1
										end
									end
								end
								l_break := False
							else
								l_pos := -1 + 6 -- goto _L___10
							end
						else
							l_pos := -1 + 6 -- goto _L___10
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 6 then -- else: | _L___10:
						if l_side /= 141 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						if l_uplo = 122 then
							if l_trans = 111 then
								l_i := l_n1
								from
								until
									(l_return or l_break)
								loop
									l_break := False
									if l_i > 0 then
										l_tmp___0 := l_i
										l_i := l_i - 1
										if not l_tmp___0.to_boolean then
											l_break := True
										end
									else
										l_break := True
									end
									if not (l_return or l_break) then
										l_j := 0
										from
										until
											not (not (l_return) and then (l_j < l_n2))
										loop
											l_temp___1 := 0.0
											l_k := 0
											from
											until
												not (l_k < l_i)
											loop
												l_temp___1 := l_temp___1 + (a_A___0[(a_lda * l_i + l_k)] * a_B[(l_k * a_ldb + l_j)])
												l_k := l_k + 1
											end
											if l_nonunit.to_boolean then
												l_temp___1 := l_temp___1 + (a_A___0[(l_i * a_lda + l_i)] * a_B[(l_i * a_ldb + l_j)])
											else
												l_temp___1 := l_temp___1 + (a_B[(l_i * a_ldb + l_j)])
											end
											a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___1
											l_j := l_j + 1
										end
									end
								end
								l_break := False
							else
								l_pos := -1 + 9 -- goto _L___8
							end
						else
							l_pos := -1 + 9 -- goto _L___8
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 9 then -- else: | _L___8:
						if l_side /= 141 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						if l_uplo = 122 then
							if l_trans = 112 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp___2 := 0.0
										if l_nonunit.to_boolean then
											l_temp___2 := (a_A___0[(l_i * a_lda + l_i)] * a_B[(l_i * a_ldb + l_j)])
										else
											l_temp___2 := a_B[(l_i * a_ldb + l_j)]
										end
										l_k := l_i + 1
										from
										until
											not (l_k < l_n1)
										loop
											l_temp___2 := l_temp___2 + (a_A___0[(a_lda * l_k + l_i)] * a_B[(l_k * a_ldb + l_j)])
											l_k := l_k + 1
										end
										a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___2
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 12 -- goto _L___6
							end
						else
							l_pos := -1 + 12 -- goto _L___6
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 12 then -- else: | _L___6:
						if l_side /= 142 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if l_uplo = 121 then
							if l_trans = 111 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := l_n2
									from
									until
										(l_return or l_break)
									loop
										l_break := False
										if l_j > 0 then
											l_tmp___1 := l_j
											l_j := l_j - 1
											if not l_tmp___1.to_boolean then
												l_break := True
											end
										else
											l_break := True
										end
										if not (l_return or l_break) then
											l_temp___3 := 0.0
											l_k := 0
											from
											until
												not (l_k < l_j)
											loop
												l_temp___3 := l_temp___3 + (a_A___0[(a_lda * l_k + l_j)] * a_B[(l_i * a_ldb + l_k)])
												l_k := l_k + 1
											end
											if l_nonunit.to_boolean then
												l_temp___3 := l_temp___3 + (a_A___0[(l_j * a_lda + l_j)] * a_B[(l_i * a_ldb + l_j)])
											else
												l_temp___3 := l_temp___3 + (a_B[(l_i * a_ldb + l_j)])
											end
											a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___3
										end
									end
									l_break := False
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 15 -- goto _L___4
							end
						else
							l_pos := -1 + 15 -- goto _L___4
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 15 then -- else: | _L___4:
						if l_side /= 142 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						if l_uplo = 121 then
							if l_trans = 112 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp___4 := 0.0
										if l_nonunit.to_boolean then
											l_temp___4 := (a_A___0[(l_j * a_lda + l_j)] * a_B[(l_i * a_ldb + l_j)])
										else
											l_temp___4 := a_B[(l_i * a_ldb + l_j)]
										end
										l_k := l_j + 1
										from
										until
											not (l_k < l_n2)
										loop
											l_temp___4 := l_temp___4 + (a_A___0[(a_lda * l_j + l_k)] * a_B[(l_i * a_ldb + l_k)])
											l_k := l_k + 1
										end
										a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___4
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 18 -- goto _L___2
							end
						else
							l_pos := -1 + 18 -- goto _L___2
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 18 then -- else: | _L___2:
						if l_side /= 142 then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if:
						if l_uplo = 122 then
							if l_trans = 111 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp___5 := 0.0
										if l_nonunit.to_boolean then
											l_temp___5 := (a_A___0[(l_j * a_lda + l_j)] * a_B[(l_i * a_ldb + l_j)])
										else
											l_temp___5 := a_B[(l_i * a_ldb + l_j)]
										end
										l_k := l_j + 1
										from
										until
											not (l_k < l_n2)
										loop
											l_temp___5 := l_temp___5 + (a_A___0[(a_lda * l_k + l_j)] * a_B[(l_i * a_ldb + l_k)])
											l_k := l_k + 1
										end
										a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___5
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 21 -- goto _L___0
							end
						else
							l_pos := -1 + 21 -- goto _L___0
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else: | _L___0:
						if l_side = 142 then
							if l_uplo = 122 then
								if l_trans = 112 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := l_n2
										from
										until
											(l_return or l_break)
										loop
											l_break := False
											if l_j > 0 then
												l_tmp___2 := l_j
												l_j := l_j - 1
												if not l_tmp___2.to_boolean then
													l_break := True
												end
											else
												l_break := True
											end
											if not (l_return or l_break) then
												l_temp___6 := 0.0
												l_k := 0
												from
												until
													not (l_k < l_j)
												loop
													l_temp___6 := l_temp___6 + (a_A___0[(a_lda * l_j + l_k)] * a_B[(l_i * a_ldb + l_k)])
													l_k := l_k + 1
												end
												if l_nonunit.to_boolean then
													l_temp___6 := l_temp___6 + (a_A___0[(l_j * a_lda + l_j)] * a_B[(l_i * a_ldb + l_j)])
												else
													l_temp___6 := l_temp___6 + (a_B[(l_i * a_ldb + l_j)])
												end
												a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___6
											end
										end
										l_break := False
										l_i := l_i + 1
									end
								else
									(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmm_r.h"), ce_string ("unrecognized operation")])
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmm_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmm_r.h"), ce_string ("unrecognized operation")])
						end
				when 22 then -- if done | if done | if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dtrmv (a_order, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_N: INTEGER_32; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_nonunit: INTEGER_32
			l_Trans: INTEGER_32
			l_tmp: NATURAL_32
			l_ix: INTEGER_32
			l_tmp___0: INTEGER_32
			l_temp: REAL_64
			l_j_min: INTEGER_32
			l_j_max: INTEGER_32
			l_jx: INTEGER_32
			l_tmp___1: INTEGER_32
			l_ix___0: INTEGER_32
			l_tmp___2: INTEGER_32
			l_temp___0: REAL_64
			l_j_min___0: INTEGER_32
			l_j_max___0: INTEGER_32
			l_jx___0: INTEGER_32
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_ix___1: INTEGER_32
			l_tmp___5: INTEGER_32
			l_temp___1: REAL_64
			l_j_min___1: INTEGER_32
			l_j_max___1: INTEGER_32
			l_jx___1: INTEGER_32
			l_tmp___6: INTEGER_32
			l_tmp___7: INTEGER_32
			l_ix___2: INTEGER_32
			l_tmp___8: INTEGER_32
			l_temp___2: REAL_64
			l_j_min___2: INTEGER_32
			l_j_max___2: INTEGER_32
			l_jx___2: INTEGER_32
			l_tmp___9: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nonunit := (a_Diag = {NATURAL_32} 131).to_integer
					if a_TransA /= {NATURAL_32} 113 then
						l_tmp := a_TransA
					else
						l_tmp := {NATURAL_32} 112
					end
					l_Trans := l_tmp.to_integer_32
					if a_order /= {NATURAL_32} 101 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_Trans = 111 then
							if a_Uplo = {NATURAL_32} 121 then
								l_pos := -1 + 6 -- goto _L___12
							else
								l_pos := -1 + 3 -- goto _L___16
							end
						else
							l_pos := -1 + 3 -- goto _L___16
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 49
				when 3 then -- else: | _L___16:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 12 -- skip if block
						end
				when 4 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 10 -- skip if block
						end
				when 5 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if: | _L___12:
						if a_incX > 0 then
							l_tmp___0 := 0
						else
							l_tmp___0 := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp___0
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_temp := 0.0
							l_j_min := l_i + 1
							l_j_max := a_N
							if a_incX > 0 then
								l_tmp___1 := 0
							else
								l_tmp___1 := (a_N - 1) * -a_incX
							end
							l_jx := l_tmp___1 + l_j_min * a_incX
							l_j := l_j_min
							from
							until
								not (l_j < l_j_max)
							loop
								l_temp := l_temp + (a_X[l_jx] * a_A___0[(a_lda * l_i + l_j)])
								l_jx := l_jx + a_incX
								l_j := l_j + 1
							end
							if l_nonunit.to_boolean then
								a_X[l_ix] := l_temp + a_X[l_ix] * a_A___0[(a_lda * l_i + l_i)]
							else
								a_X[l_ix] := a_X[l_ix] + l_temp
							end
							l_ix := l_ix + a_incX
							l_i := l_i + 1
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else:
						l_pos := -1 + 12 -- goto _L___14
				when 9 then -- if done | if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else:
						l_pos := -1 + 12 -- goto _L___14
				when 11 then -- if done | if block done, skip else block:
						l_pos := -1 + 49
				when 12 then -- else: | _L___14:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if l_Trans = 111 then
							if a_Uplo = {NATURAL_32} 122 then
								l_pos := -1 + 18 -- goto _L___7
							else
								l_pos := -1 + 15 -- goto _L___11
							end
						else
							l_pos := -1 + 15 -- goto _L___11
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 49
				when 15 then -- else: | _L___11:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 24 -- skip if block
						end
				when 16 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 22 -- skip if block
						end
				when 17 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 20 -- skip if block
						end
				when 18 then -- if: | _L___7:
						if a_incX > 0 then
							l_tmp___2 := 0
						else
							l_tmp___2 := (a_N - 1) * -a_incX
						end
						l_ix___0 := l_tmp___2 + (a_N - 1) * a_incX
						l_i := a_N
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___4 := l_i
								l_i := l_i - 1
								if not l_tmp___4.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_temp___0 := 0.0
								l_j_min___0 := 0
								l_j_max___0 := l_i
								if a_incX > 0 then
									l_tmp___3 := 0
								else
									l_tmp___3 := (a_N - 1) * -a_incX
								end
								l_jx___0 := l_tmp___3 + l_j_min___0 * a_incX
								l_j := l_j_min___0
								from
								until
									not (l_j < l_j_max___0)
								loop
									l_temp___0 := l_temp___0 + (a_X[l_jx___0] * a_A___0[(a_lda * l_i + l_j)])
									l_jx___0 := l_jx___0 + a_incX
									l_j := l_j + 1
								end
								if l_nonunit.to_boolean then
									a_X[l_ix___0] := l_temp___0 + a_X[l_ix___0] * a_A___0[(a_lda * l_i + l_i)]
								else
									a_X[l_ix___0] := a_X[l_ix___0] + l_temp___0
								end
								l_ix___0 := l_ix___0 - a_incX
							end
						end
						l_break := False
				when 19 then -- if block done, skip else block:
						l_pos := -1 + 21
				when 20 then -- else:
						l_pos := -1 + 24 -- goto _L___9
				when 21 then -- if done | if block done, skip else block:
						l_pos := -1 + 23
				when 22 then -- else:
						l_pos := -1 + 24 -- goto _L___9
				when 23 then -- if done | if block done, skip else block:
						l_pos := -1 + 49
				when 24 then -- else: | _L___9:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 27 -- skip if block
						end
				when 25 then -- if:
						if l_Trans = 112 then
							if a_Uplo = {NATURAL_32} 121 then
								l_pos := -1 + 30 -- goto _L___2
							else
								l_pos := -1 + 27 -- goto _L___6
							end
						else
							l_pos := -1 + 27 -- goto _L___6
						end
				when 26 then -- if block done, skip else block:
						l_pos := -1 + 49
				when 27 then -- else: | _L___6:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 36 -- skip if block
						end
				when 28 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 34 -- skip if block
						end
				when 29 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 32 -- skip if block
						end
				when 30 then -- if: | _L___2:
						if a_incX > 0 then
							l_tmp___5 := 0
						else
							l_tmp___5 := (a_N - 1) * -a_incX
						end
						l_ix___1 := l_tmp___5 + (a_N - 1) * a_incX
						l_i := a_N
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___7 := l_i
								l_i := l_i - 1
								if not l_tmp___7.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_temp___1 := 0.0
								l_j_min___1 := 0
								l_j_max___1 := l_i
								if a_incX > 0 then
									l_tmp___6 := 0
								else
									l_tmp___6 := (a_N - 1) * -a_incX
								end
								l_jx___1 := l_tmp___6 + l_j_min___1 * a_incX
								l_j := l_j_min___1
								from
								until
									not (l_j < l_j_max___1)
								loop
									l_temp___1 := l_temp___1 + (a_X[l_jx___1] * a_A___0[(a_lda * l_j + l_i)])
									l_jx___1 := l_jx___1 + a_incX
									l_j := l_j + 1
								end
								if l_nonunit.to_boolean then
									a_X[l_ix___1] := l_temp___1 + a_X[l_ix___1] * a_A___0[(a_lda * l_i + l_i)]
								else
									a_X[l_ix___1] := a_X[l_ix___1] + l_temp___1
								end
								l_ix___1 := l_ix___1 - a_incX
							end
						end
						l_break := False
				when 31 then -- if block done, skip else block:
						l_pos := -1 + 33
				when 32 then -- else:
						l_pos := -1 + 36 -- goto _L___4
				when 33 then -- if done | if block done, skip else block:
						l_pos := -1 + 35
				when 34 then -- else:
						l_pos := -1 + 36 -- goto _L___4
				when 35 then -- if done | if block done, skip else block:
						l_pos := -1 + 49
				when 36 then -- else: | _L___4:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 39 -- skip if block
						end
				when 37 then -- if:
						if l_Trans = 112 then
							if a_Uplo = {NATURAL_32} 122 then
								l_pos := -1 + 42 -- goto _L
							else
								l_pos := -1 + 39 -- goto _L___1
							end
						else
							l_pos := -1 + 39 -- goto _L___1
						end
				when 38 then -- if block done, skip else block:
						l_pos := -1 + 49
				when 39 then -- else: | _L___1:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 48 -- skip if block
						end
				when 40 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 46 -- skip if block
						end
				when 41 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 44 -- skip if block
						end
				when 42 then -- if: | _L:
						if a_incX > 0 then
							l_tmp___8 := 0
						else
							l_tmp___8 := (a_N - 1) * -a_incX
						end
						l_ix___2 := l_tmp___8
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_temp___2 := 0.0
							l_j_min___2 := l_i + 1
							l_j_max___2 := a_N
							if a_incX > 0 then
								l_tmp___9 := 0
							else
								l_tmp___9 := (a_N - 1) * -a_incX
							end
							l_jx___2 := l_tmp___9 + (l_i + 1) * a_incX
							l_j := l_j_min___2
							from
							until
								not (l_j < l_j_max___2)
							loop
								l_temp___2 := l_temp___2 + (a_X[l_jx___2] * a_A___0[(a_lda * l_j + l_i)])
								l_jx___2 := l_jx___2 + a_incX
								l_j := l_j + 1
							end
							if l_nonunit.to_boolean then
								a_X[l_ix___2] := l_temp___2 + a_X[l_ix___2] * a_A___0[(a_lda * l_i + l_i)]
							else
								a_X[l_ix___2] := a_X[l_ix___2] + l_temp___2
							end
							l_ix___2 := l_ix___2 + a_incX
							l_i := l_i + 1
						end
				when 43 then -- if block done, skip else block:
						l_pos := -1 + 45
				when 44 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmv_r.h"), ce_string ("unrecognized operation")])
				when 45 then -- if done | if block done, skip else block:
						l_pos := -1 + 47
				when 46 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmv_r.h"), ce_string ("unrecognized operation")])
				when 47 then -- if done | if block done, skip else block:
						l_pos := -1 + 49
				when 48 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmv_r.h"), ce_string ("unrecognized operation")])
				when 49 then -- if done | if done | if done | if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dtrsm (a_Order, a_Side, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_64; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_64]; a_ldb: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_n1: INTEGER_32
			l_n2: INTEGER_32
			l_nonunit: INTEGER_32
			l_side: INTEGER_32
			l_uplo: INTEGER_32
			l_trans: INTEGER_32
			l_Aii: REAL_64
			l_Aki: REAL_64
			l_tmp: INTEGER_32
			l_Aii___0: REAL_64
			l_Aik: REAL_64
			l_Aii___1: REAL_64
			l_Aki___0: REAL_64
			l_Aii___2: REAL_64
			l_Aik___0: REAL_64
			l_tmp___0: INTEGER_32
			l_Ajj: REAL_64
			l_Bij: REAL_64
			l_Ajj___0: REAL_64
			l_Bij___0: REAL_64
			l_tmp___1: INTEGER_32
			l_Ajj___1: REAL_64
			l_Bij___1: REAL_64
			l_tmp___2: INTEGER_32
			l_Ajj___2: REAL_64
			l_Bij___2: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nonunit := (a_Diag = {NATURAL_32} 131).to_integer
					if a_Order = {NATURAL_32} 101 then
						l_n1 := a_M
						l_n2 := a_N
						l_side := a_Side.to_integer_32
						l_uplo := a_Uplo.to_integer_32
						if a_TransA = {NATURAL_32} 113 then
							l_trans := 112
						else
							l_trans := a_TransA.to_integer_32
						end
					else
						l_n1 := a_N
						l_n2 := a_M
						if a_Side = {NATURAL_32} 141 then
							l_side := 142
						else
							l_side := 141
						end
						if a_Uplo = {NATURAL_32} 121 then
							l_uplo := 122
						else
							l_uplo := 121
						end
						if a_TransA = {NATURAL_32} 113 then
							l_trans := 112
						else
							l_trans := a_TransA.to_integer_32
						end
					end
					if l_side /= 141 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_uplo = 121 then
							if l_trans = 111 then
								if a_alpha /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := l_n1
								from
								until
									(l_return or l_break)
								loop
									l_break := False
									if l_i > 0 then
										l_tmp := l_i
										l_i := l_i - 1
										if not l_tmp.to_boolean then
											l_break := True
										end
									else
										l_break := True
									end
									if not (l_return or l_break) then
										if l_nonunit.to_boolean then
											l_Aii := a_A___0[(a_lda * l_i + l_i)]
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Aii
												l_j := l_j + 1
											end
										end
										l_k := 0
										from
										until
											not (not (l_return) and then (l_k < l_i))
										loop
											l_Aki := a_A___0[(l_k * a_lda + l_i)]
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_k + l_j)] := a_B[(a_ldb * l_k + l_j)] - (l_Aki * a_B[(a_ldb * l_i + l_j)])
												l_j := l_j + 1
											end
											l_k := l_k + 1
										end
									end
								end
								l_break := False
							else
								l_pos := -1 + 3 -- goto _L___12
							end
						else
							l_pos := -1 + 3 -- goto _L___12
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 3 then -- else: | _L___12:
						if l_side /= 141 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if l_uplo = 121 then
							if l_trans = 112 then
								if a_alpha /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									if l_nonunit.to_boolean then
										l_Aii___0 := a_A___0[(a_lda * l_i + l_i)]
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Aii___0
											l_j := l_j + 1
										end
									end
									l_k := l_i + 1
									from
									until
										not (not (l_return) and then (l_k < l_n1))
									loop
										l_Aik := a_A___0[(l_i * a_lda + l_k)]
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_k + l_j)] := a_B[(a_ldb * l_k + l_j)] - (l_Aik * a_B[(a_ldb * l_i + l_j)])
											l_j := l_j + 1
										end
										l_k := l_k + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 6 -- goto _L___10
							end
						else
							l_pos := -1 + 6 -- goto _L___10
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 6 then -- else: | _L___10:
						if l_side /= 141 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						if l_uplo = 122 then
							if l_trans = 111 then
								if a_alpha /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									if l_nonunit.to_boolean then
										l_Aii___1 := a_A___0[(a_lda * l_i + l_i)]
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Aii___1
											l_j := l_j + 1
										end
									end
									l_k := l_i + 1
									from
									until
										not (not (l_return) and then (l_k < l_n1))
									loop
										l_Aki___0 := a_A___0[(l_k * a_lda + l_i)]
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_k + l_j)] := a_B[(a_ldb * l_k + l_j)] - (l_Aki___0 * a_B[(a_ldb * l_i + l_j)])
											l_j := l_j + 1
										end
										l_k := l_k + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 9 -- goto _L___8
							end
						else
							l_pos := -1 + 9 -- goto _L___8
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 9 then -- else: | _L___8:
						if l_side /= 141 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						if l_uplo = 122 then
							if l_trans = 112 then
								if a_alpha /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := l_n1
								from
								until
									(l_return or l_break)
								loop
									l_break := False
									if l_i > 0 then
										l_tmp___0 := l_i
										l_i := l_i - 1
										if not l_tmp___0.to_boolean then
											l_break := True
										end
									else
										l_break := True
									end
									if not (l_return or l_break) then
										if l_nonunit.to_boolean then
											l_Aii___2 := a_A___0[(a_lda * l_i + l_i)]
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Aii___2
												l_j := l_j + 1
											end
										end
										l_k := 0
										from
										until
											not (not (l_return) and then (l_k < l_i))
										loop
											l_Aik___0 := a_A___0[(l_i * a_lda + l_k)]
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_k + l_j)] := a_B[(a_ldb * l_k + l_j)] - (l_Aik___0 * a_B[(a_ldb * l_i + l_j)])
												l_j := l_j + 1
											end
											l_k := l_k + 1
										end
									end
								end
								l_break := False
							else
								l_pos := -1 + 12 -- goto _L___6
							end
						else
							l_pos := -1 + 12 -- goto _L___6
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 12 then -- else: | _L___6:
						if l_side /= 142 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if l_uplo = 121 then
							if l_trans = 111 then
								if a_alpha /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										if l_nonunit.to_boolean then
											l_Ajj := a_A___0[(a_lda * l_j + l_j)]
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Ajj
										end
										l_Bij := a_B[(a_ldb * l_i + l_j)]
										l_k := l_j + 1
										from
										until
											not (l_k < l_n2)
										loop
											a_B[(a_ldb * l_i + l_k)] := a_B[(a_ldb * l_i + l_k)] - (a_A___0[(l_j * a_lda + l_k)] * l_Bij)
											l_k := l_k + 1
										end
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 15 -- goto _L___4
							end
						else
							l_pos := -1 + 15 -- goto _L___4
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 15 then -- else: | _L___4:
						if l_side /= 142 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						if l_uplo = 121 then
							if l_trans = 112 then
								if a_alpha /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := l_n2
									from
									until
										(l_return or l_break)
									loop
										l_break := False
										if l_j > 0 then
											l_tmp___1 := l_j
											l_j := l_j - 1
											if not l_tmp___1.to_boolean then
												l_break := True
											end
										else
											l_break := True
										end
										if not (l_return or l_break) then
											if l_nonunit.to_boolean then
												l_Ajj___0 := a_A___0[(a_lda * l_j + l_j)]
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Ajj___0
											end
											l_Bij___0 := a_B[(a_ldb * l_i + l_j)]
											l_k := 0
											from
											until
												not (l_k < l_j)
											loop
												a_B[(a_ldb * l_i + l_k)] := a_B[(a_ldb * l_i + l_k)] - (a_A___0[(l_k * a_lda + l_j)] * l_Bij___0)
												l_k := l_k + 1
											end
										end
									end
									l_break := False
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 18 -- goto _L___2
							end
						else
							l_pos := -1 + 18 -- goto _L___2
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 18 then -- else: | _L___2:
						if l_side /= 142 then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if:
						if l_uplo = 122 then
							if l_trans = 111 then
								if a_alpha /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := l_n2
									from
									until
										(l_return or l_break)
									loop
										l_break := False
										if l_j > 0 then
											l_tmp___2 := l_j
											l_j := l_j - 1
											if not l_tmp___2.to_boolean then
												l_break := True
											end
										else
											l_break := True
										end
										if not (l_return or l_break) then
											if l_nonunit.to_boolean then
												l_Ajj___1 := a_A___0[(a_lda * l_j + l_j)]
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Ajj___1
											end
											l_Bij___1 := a_B[(a_ldb * l_i + l_j)]
											l_k := 0
											from
											until
												not (l_k < l_j)
											loop
												a_B[(a_ldb * l_i + l_k)] := a_B[(a_ldb * l_i + l_k)] - (a_A___0[(l_j * a_lda + l_k)] * l_Bij___1)
												l_k := l_k + 1
											end
										end
									end
									l_break := False
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 21 -- goto _L___0
							end
						else
							l_pos := -1 + 21 -- goto _L___0
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else: | _L___0:
						if l_side = 142 then
							if l_uplo = 122 then
								if l_trans = 112 then
									if a_alpha /= 1.0 then
										l_i := 0
										from
										until
											not (not (l_return) and then (l_i < l_n1))
										loop
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
												l_j := l_j + 1
											end
											l_i := l_i + 1
										end
									end
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (not (l_return) and then (l_j < l_n2))
										loop
											if l_nonunit.to_boolean then
												l_Ajj___2 := a_A___0[(a_lda * l_j + l_j)]
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Ajj___2
											end
											l_Bij___2 := a_B[(a_ldb * l_i + l_j)]
											l_k := l_j + 1
											from
											until
												not (l_k < l_n2)
											loop
												a_B[(a_ldb * l_i + l_k)] := a_B[(a_ldb * l_i + l_k)] - (a_A___0[(l_k * a_lda + l_j)] * l_Bij___2)
												l_k := l_k + 1
											end
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								else
									(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsm_r.h"), ce_string ("unrecognized operation")])
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsm_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsm_r.h"), ce_string ("unrecognized operation")])
						end
				when 22 then -- if done | if done | if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_dtrsv (a_order, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_N: INTEGER_32; a_A___0: CE_POINTER [REAL_64]; a_lda: INTEGER_32; a_X: CE_POINTER [REAL_64]; a_incX: INTEGER_32)
		local
			l_nonunit: INTEGER_32
			l_ix: INTEGER_32
			l_jx: INTEGER_32
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_Trans: INTEGER_32
			l_tmp: NATURAL_32
			l_tmp___0: INTEGER_32
			l_tmp___1: REAL_64
			l_Aij: REAL_64
			l_tmp___2: INTEGER_32
			l_tmp___3: REAL_64
			l_Aij___0: REAL_64
			l_tmp___4: REAL_64
			l_Aji: REAL_64
			l_tmp___5: INTEGER_32
			l_tmp___6: REAL_64
			l_Aji___0: REAL_64
			l_tmp___7: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nonunit := (a_Diag = {NATURAL_32} 131).to_integer
					if a_TransA /= {NATURAL_32} 113 then
						l_tmp := a_TransA
					else
						l_tmp := {NATURAL_32} 112
					end
					l_Trans := l_tmp.to_integer_32
					if a_N = 0 then
						l_return := True
					end
					if l_return then
						l_pos := -1 + 50 -- skip if block
					end
				when 1 then -- if:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_Trans = 111 then
							if a_Uplo = {NATURAL_32} 121 then
								l_pos := -1 + 7 -- goto _L___12
							else
								l_pos := -1 + 4 -- goto _L___16
							end
						else
							l_pos := -1 + 4 -- goto _L___16
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 50
				when 4 then -- else: | _L___16:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 13 -- skip if block
						end
				when 5 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 11 -- skip if block
						end
				when 6 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L___12:
						if a_incX > 0 then
							l_tmp___0 := 0
						else
							l_tmp___0 := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp___0 + a_incX * (a_N - 1)
						if l_nonunit.to_boolean then
							a_X[l_ix] := a_X[l_ix] / (a_A___0[(a_lda * (a_N - 1) + (a_N - 1))])
						end
						l_ix := l_ix - a_incX
						l_i := a_N - 1
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___2 := l_i
								l_i := l_i - 1
								if not l_tmp___2.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_tmp___1 := a_X[l_ix]
								l_jx := l_ix + a_incX
								l_j := l_i + 1
								from
								until
									not (l_j < a_N)
								loop
									l_Aij := a_A___0[(a_lda * l_i + l_j)]
									l_tmp___1 := l_tmp___1 - (l_Aij * a_X[l_jx])
									l_jx := l_jx + a_incX
									l_j := l_j + 1
								end
								if l_nonunit.to_boolean then
									a_X[l_ix] := l_tmp___1 / a_A___0[(a_lda * l_i + l_i)]
								else
									a_X[l_ix] := l_tmp___1
								end
								l_ix := l_ix - a_incX
							end
						end
						l_break := False
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						l_pos := -1 + 13 -- goto _L___14
				when 10 then -- if done | if block done, skip else block:
						l_pos := -1 + 12
				when 11 then -- else:
						l_pos := -1 + 13 -- goto _L___14
				when 12 then -- if done | if block done, skip else block:
						l_pos := -1 + 50
				when 13 then -- else: | _L___14:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						if l_Trans = 111 then
							if a_Uplo = {NATURAL_32} 122 then
								l_pos := -1 + 19 -- goto _L___7
							else
								l_pos := -1 + 16 -- goto _L___11
							end
						else
							l_pos := -1 + 16 -- goto _L___11
						end
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 50
				when 16 then -- else: | _L___11:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 25 -- skip if block
						end
				when 17 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 23 -- skip if block
						end
				when 18 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if: | _L___7:
						if a_incX > 0 then
							l_ix := 0
						else
							l_ix := (a_N - 1) * -a_incX
						end
						if l_nonunit.to_boolean then
							a_X[l_ix] := a_X[l_ix] / (a_A___0[0])
						end
						l_ix := l_ix + a_incX
						l_i := 1
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_tmp___3 := a_X[l_ix]
							if a_incX > 0 then
								l_jx := 0
							else
								l_jx := (a_N - 1) * -a_incX
							end
							l_j := 0
							from
							until
								not (l_j < l_i)
							loop
								l_Aij___0 := a_A___0[(a_lda * l_i + l_j)]
								l_tmp___3 := l_tmp___3 - (l_Aij___0 * a_X[l_jx])
								l_jx := l_jx + a_incX
								l_j := l_j + 1
							end
							if l_nonunit.to_boolean then
								a_X[l_ix] := l_tmp___3 / a_A___0[(a_lda * l_i + l_i)]
							else
								a_X[l_ix] := l_tmp___3
							end
							l_ix := l_ix + a_incX
							l_i := l_i + 1
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else:
						l_pos := -1 + 25 -- goto _L___9
				when 22 then -- if done | if block done, skip else block:
						l_pos := -1 + 24
				when 23 then -- else:
						l_pos := -1 + 25 -- goto _L___9
				when 24 then -- if done | if block done, skip else block:
						l_pos := -1 + 50
				when 25 then -- else: | _L___9:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 28 -- skip if block
						end
				when 26 then -- if:
						if l_Trans = 112 then
							if a_Uplo = {NATURAL_32} 121 then
								l_pos := -1 + 31 -- goto _L___2
							else
								l_pos := -1 + 28 -- goto _L___6
							end
						else
							l_pos := -1 + 28 -- goto _L___6
						end
				when 27 then -- if block done, skip else block:
						l_pos := -1 + 50
				when 28 then -- else: | _L___6:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 37 -- skip if block
						end
				when 29 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 35 -- skip if block
						end
				when 30 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 33 -- skip if block
						end
				when 31 then -- if: | _L___2:
						if a_incX > 0 then
							l_ix := 0
						else
							l_ix := (a_N - 1) * -a_incX
						end
						if l_nonunit.to_boolean then
							a_X[l_ix] := a_X[l_ix] / (a_A___0[0])
						end
						l_ix := l_ix + a_incX
						l_i := 1
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_tmp___4 := a_X[l_ix]
							if a_incX > 0 then
								l_jx := 0
							else
								l_jx := (a_N - 1) * -a_incX
							end
							l_j := 0
							from
							until
								not (l_j < l_i)
							loop
								l_Aji := a_A___0[(a_lda * l_j + l_i)]
								l_tmp___4 := l_tmp___4 - (l_Aji * a_X[l_jx])
								l_jx := l_jx + a_incX
								l_j := l_j + 1
							end
							if l_nonunit.to_boolean then
								a_X[l_ix] := l_tmp___4 / a_A___0[(a_lda * l_i + l_i)]
							else
								a_X[l_ix] := l_tmp___4
							end
							l_ix := l_ix + a_incX
							l_i := l_i + 1
						end
				when 32 then -- if block done, skip else block:
						l_pos := -1 + 34
				when 33 then -- else:
						l_pos := -1 + 37 -- goto _L___4
				when 34 then -- if done | if block done, skip else block:
						l_pos := -1 + 36
				when 35 then -- else:
						l_pos := -1 + 37 -- goto _L___4
				when 36 then -- if done | if block done, skip else block:
						l_pos := -1 + 50
				when 37 then -- else: | _L___4:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 40 -- skip if block
						end
				when 38 then -- if:
						if l_Trans = 112 then
							if a_Uplo = {NATURAL_32} 122 then
								l_pos := -1 + 43 -- goto _L
							else
								l_pos := -1 + 40 -- goto _L___1
							end
						else
							l_pos := -1 + 40 -- goto _L___1
						end
				when 39 then -- if block done, skip else block:
						l_pos := -1 + 50
				when 40 then -- else: | _L___1:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 49 -- skip if block
						end
				when 41 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 47 -- skip if block
						end
				when 42 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 45 -- skip if block
						end
				when 43 then -- if: | _L:
						if a_incX > 0 then
							l_tmp___5 := 0
						else
							l_tmp___5 := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp___5 + (a_N - 1) * a_incX
						if l_nonunit.to_boolean then
							a_X[l_ix] := a_X[l_ix] / (a_A___0[(a_lda * (a_N - 1) + (a_N - 1))])
						end
						l_ix := l_ix - a_incX
						l_i := a_N - 1
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___7 := l_i
								l_i := l_i - 1
								if not l_tmp___7.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_tmp___6 := a_X[l_ix]
								l_jx := l_ix + a_incX
								l_j := l_i + 1
								from
								until
									not (l_j < a_N)
								loop
									l_Aji___0 := a_A___0[(a_lda * l_j + l_i)]
									l_tmp___6 := l_tmp___6 - (l_Aji___0 * a_X[l_jx])
									l_jx := l_jx + a_incX
									l_j := l_j + 1
								end
								if l_nonunit.to_boolean then
									a_X[l_ix] := l_tmp___6 / a_A___0[(a_lda * l_i + l_i)]
								else
									a_X[l_ix] := l_tmp___6
								end
								l_ix := l_ix - a_incX
							end
						end
						l_break := False
				when 44 then -- if block done, skip else block:
						l_pos := -1 + 46
				when 45 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsv_r.h"), ce_string ("unrecognized operation")])
				when 46 then -- if done | if block done, skip else block:
						l_pos := -1 + 48
				when 47 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsv_r.h"), ce_string ("unrecognized operation")])
				when 48 then -- if done | if block done, skip else block:
						l_pos := -1 + 50
				when 49 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsv_r.h"), ce_string ("unrecognized operation")])
				when 50 then -- if done | if done | if done | if done | if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			size1 := a_other.size1
			size2 := a_other.size2
			tda := a_other.tda
			data := a_other.data
			block := a_other.block
			owner := a_other.owner
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				size1 = a_other.size1 and
				size2 = a_other.size2 and
				tda = a_other.tda and
				data.is_equal (a_other.data) and
				block.is_equal (a_other.block) and
				owner = a_other.owner
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("size1"),
					create {CE_STRUCTURE_FIELD}.make ("size2"),
					create {CE_STRUCTURE_FIELD}.make ("tda"),
					create {CE_STRUCTURE_FIELD}.make ("data"),
					create {CE_STRUCTURE_FIELD}.make ("block"),
					create {CE_STRUCTURE_FIELD}.make ("owner")
				>>)
			init_structure_info
		end

end
